package de.bsvrz.buv.plugin.bmvew;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.tabellen.ElementFilter;
import de.bsvrz.buv.rw.basislib.tabellen.ElementRegEx;
import de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nach Auswahl des Menpunkts "Betriebsmeldungen..." in der Rahmenapplikation
 * muss sich der Dialog zur Anzeige der Betriebsmeldungen ffnen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und
 * Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-8</li>
 * </ul>
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/02/04 05:50:35 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class MeldungsanzeigeSicht extends ViewPart implements
		IBetriebsmeldungListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Die ID der View
   */
  public static final String ID = "de.bsvrz.buv.plugin.bmvew.MeldungsanzeigeSicht"; 

  /**
   * Meldungen Speicher
   */
  private MeldungenSpeicher _meldungenSpeicher = MeldungenSpeicher.getInstanz(); // @jve:decl-index=0:

  /**
   * Filter Speicher
   */
  private FilterSpeicher _filterSpeicher = FilterSpeicher.getInstanz();

  /**
   * Gibt an, ob der Filter fr den Betriebsmeldungsdialog aktiv (<code>true</code>) oder inaktiv (<code>false</code>)ist.
   */
  private boolean _filterAktiv = false;

  /**
   * TableViewer
   */
  private static TableViewer _tvBetriebsMeldungen = null;

  /**
   * Gibt die Sortierreihenfolge der Spalten an.
   */
  private static int[] _tvSpaltenSortierung = null;

  private Composite top = null;

  private Composite compositeTop = null;

  private Composite composite = null;

  private Composite composite3 = null;

  private Button checkBox = null;

  private Combo combo = null;

  private Button buttonVerwalten = null;

  private Button buttonOk = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private Composite parent = null;

  private DaVKommunikation _daVKommunikation = DaVKommunikation.getInstanz();

  /**
   * DropAdapter fr den TableViewer
   */
  private MeldungsanzeigeFallAdapter _meldungsAnzeigeFallAdapter;

  /**
   * DragListener fr den TableViewer
   */
  private ZugZuhoererMeldung _zugZuhoererMeldung;

  @Override
  public void createPartControl(Composite parent)
  {
    
    // this.parent = parent;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    top = new Composite(parent, SWT.NONE);
    createCompositeTop();
    top.setLayout(gridLayout);
    createCompositeTable();
    createCompositeBottom();
    

    _daVKommunikation.hinzufuegenBetriebsmeldungsListener(this);

    fuelleDialogBetriebsmeldungsListe();
    if (_tvSpaltenSortierung != null)
      spaltenSortierungLaden();
    
    try
    {
      XmlFilterVerwaltung xmvLesen = new XmlFilterVerwaltung();
      xmvLesen.einlesenXmlAusDatei("c:/filter.xml");
      LinkedList<ElementFilter> efs = xmvLesen.getElementFilter();

      if (efs != null){
      
        for (ElementFilter ef : efs)
        {
          Filter filter = new Filter();
         
          filter.setName(ef.getBezeichnung());
          
          LinkedList<ElementRegEx> regs = ef.getRegex();
          
          for (ElementRegEx e : regs)
          {
            if (e.getBezeichnung().equals(Feldbezeichner.ApplikationsID))
              filter.getApplikationsID().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.LaufendeNummer))
              filter.getLaufendeNummer().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.ApplikationsTyp))
              filter.getApplikationsTyp().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.ApplikationsKennung))
              filter.getApplikationsKennung().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.ID))
              filter.getID().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.MeldungsTyp))
              filter.getMeldungsTyp().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.MeldungsTypZusatz))
              filter.getMeldungsTypZusatz().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.MeldungsKlasse))
              filter.getMeldungsKlasse().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.Referenz))
              filter.getReferenz().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.Status))
              filter.getStatus().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.UrlasserBenutzerReferenz))
              filter.getUrlasserBenutzerReferenz().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.UrlasserUrsache))
              filter.getUrlasserUrsache().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.UrlasserVeranlasser))
              filter.getUrlasserVeranlasser().setRegulaererAusdruck(e.getRegEx());
            else if (e.getBezeichnung().equals(Feldbezeichner.MeldungsText))
              filter.getMeldungsText().setRegulaererAusdruck(e.getRegEx());
            else{
              _debug.fine("Fehler: " + e.getBezeichnung());
            }
          }
          
          _filterSpeicher.filterHinzufuegen(filter);
                  
        }
      
      }

    }
    catch (IllegalArgumentException e)
    {
      _debug.fine("Datei existiert nicht");
      
    }
    

  }

  @Override
  public void setFocus()
  {
  }

  /**
   * Generiert die Elemente des Dialogs {@link MeldungsanzeigeDialog}.
   * 
   */
  private void generiereGui()
  {
  }

  /**
   * ffnet die Verwaltung der Filter, dort lassen sich Filter anlegen, bearbeiten, kopieren und lschen.
   * 
   */
  private void aktionFilterVerwalten()
  {
    FilterVerwaltungsdialog filterVerwaltungsdialog = new FilterVerwaltungsdialog(PlatformUI.getWorkbench()
        .getDisplay().getActiveShell());
    
    // _filterSpeicher = filterVerwaltungsdialog.oeffne(_filterSpeicher);
    FilterSpeicher fsNeu = filterVerwaltungsdialog.oeffne(_filterSpeicher);
       
    XmlFilterVerwaltung xmvSchreiben = new XmlFilterVerwaltung();
    java.util.List<Filter> filterListe = fsNeu.getFilterListe();
    int cnt = 0;
    for (Filter filter : filterListe)
    {
      ElementFilter ef = new ElementFilter(cnt++, filter.getName());
      ef.setBezeichnung(filter.getName());
        
      FilterElement fe = filter.getApplikationsID();
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));       
      
      fe = filter.getLaufendeNummer();
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));
           
      fe = filter.getApplikationsTyp();
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));

      fe = filter.getApplikationsKennung();
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));

      fe = filter.getID();          
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));

      fe = filter.getMeldungsTyp();          
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));

      fe = filter.getMeldungsTypZusatz();          
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));
      
      fe = filter.getMeldungsKlasse();         
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));
            
      fe = filter.getReferenz();
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));

      fe = filter.getStatus();
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));

      fe = filter.getUrlasserBenutzerReferenz();          
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));

      fe = filter.getUrlasserUrsache();          
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));
      
      fe = filter.getUrlasserVeranlasser();          
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));
      
      fe = filter.getMeldungsText();
      ef.addRegEx(new ElementRegEx(ef.bestimmeNaechsteFreieIdRegEx(), fe.getFilterElementTitel() , fe.getRegulaererAusdruck()));
       
      xmvSchreiben.addElementFilter(ef);
            
    }
    
    String xmlString = xmvSchreiben.ausgabeXmlFormatiert();
    xmvSchreiben.schreibeString("C:/filter.xml", xmlString, true);
    
  }

  /**
   * ffnet die Hilfe zum Dialog {@link MeldungsanzeigeDialog}.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Aktivator.PLUGIN_ID + ".betriebsmeldungsverwaltung";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * Schliet den Dialog {@link MeldungsanzeigeDialog}.
   * 
   * Betriebsmeldungen werden auch nach dem Schlieen des Dialogs gespeichert, so dass beim nchsten ffnen des Fensters
   * alle bisher empfangenen Betriebsmeldungen (bis zur maximalen Anzahl) dargestellt werden.
   * 
   */
  private void aktionSchliessen()
  {
    _debug.fine("Schliessen");
    spaltenSortierungSpeichern();
    getViewSite().getPage().hideView(MeldungsanzeigeSicht.this);
  }

  /**
   * Fr den Button ok ist keinerlei Funktionalitt vorgegeben.
   * 
   */
  private void aktionOk()
  {
    _debug.fine("Ok");
    fuelleDialogBetriebsmeldungsListe();

  }

  /**
   * Fllt die Liste der Betriebsmeldungen des Dialogs.
   */
  private void fuelleDialogBetriebsmeldungsListe()
  {

    if (_filterAktiv)
    {

      List<Meldung> tmpListe = new ArrayList<Meldung>();

      List<Meldung> meldungListe = _meldungenSpeicher.getBetriebsmeldungen();
      for (Meldung meldung : meldungListe)
      {

        List<Filter> filterListe = _filterSpeicher.getFilterListe();

        String filterName = combo.getText();

        Filter filter = null;
        for (int i = 0; i < filterListe.size(); i++)
        {
          filter = filterListe.get(i);

          if (filter.getName().equals(filterName))
            break;

        }

        if (filter == null)
        {
          _debug.fine("Filter ist null!");
        }

        List<FilterElement> filterElementListe = filter.getFilterElemente();

        Boolean[] found = new Boolean[filterElementListe.size()];

        for (int i = 0; i < filterElementListe.size(); i++)
        {

          FilterElement filterElement = filterElementListe.get(i);

          String regEx = filterElement.getRegulaererAusdruck();

          String input = null;

          if (filterElement.getFilterElementTitel().equals(Feldbezeichner.ApplikationsID))
          {
            input = meldung.getApplikationsID();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.LaufendeNummer))
          {
            input = meldung.getLaufendeNummer();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.ApplikationsTyp))
          {
            input = meldung.getApplikationsTyp();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.ApplikationsKennung))
          {
            input = meldung.getApplikationsKennung();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.ID))
          {
            input = meldung.getID();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.MeldungsTyp))
          {
            input = meldung.getMeldungsTyp();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.MeldungsTypZusatz))
          {
            input = meldung.getMeldungsTypZusatz();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.MeldungsKlasse))
          {
            input = meldung.getMeldungsKlasse();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.Referenz))
          {
            input = meldung.getReferenz();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.Status))
          {
            input = meldung.getStatus();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.UrlasserBenutzerReferenz))
          {
            input = meldung.getUrlasserBenutzerReferenz();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.UrlasserUrsache))
          {
            input = meldung.getUrlasserUrsache();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.UrlasserVeranlasser))
          {
            input = meldung.getUrlasserVeranlasser();
          }
          else if (filterElement.getFilterElementTitel().equals(Feldbezeichner.MeldungsText))
          {
            input = meldung.getMeldungsText();
          }
          else
          {
            _debug.fine("Fehler Feldbezeichner: " + filterElement.getFilterElementTitel());
          }

          found[i] = true;

          Pattern p = Pattern.compile(regEx);

          if (input.length() > 0)
          {

            Matcher matcher = p.matcher(input);
            found[i] = matcher.find();

          }
          else
            found[i] = true;

        }

        boolean ok = true;
        for (Boolean b : found)
        {
          if (!b)
            ok = false;
        }

        if (ok)
          tmpListe.add(meldung);

      }

      _tvBetriebsMeldungen.setInput(tmpListe);

    }
    else
    {

      List<Meldung> list = _meldungenSpeicher.getBetriebsmeldungen();
      _tvBetriebsMeldungen.setInput(list);

    }

  }

  /**
   * Speichert die Spaltenreihenfolge ab.
   * 
   */
  private void spaltenSortierungSpeichern()
  {

    String ordnung = "(";
    // Auslesen der Spaltenordnung
    _tvSpaltenSortierung = _tvBetriebsMeldungen.getTable().getColumnOrder();
    for (int i = 0; i < _tvSpaltenSortierung.length; i++)
    {
      ordnung = ordnung + "[" + _tvSpaltenSortierung[i] + "] ";
    }
    ordnung = ordnung + ")";
    _debug.fine("Spaltensortierung: " + ordnung);

    _tvSpaltenSortierung = _tvBetriebsMeldungen.getTable().getColumnOrder();

  }

  /**
   * Ld die Sortierung der Spalten.
   * 
   */
  private void spaltenSortierungLaden()
  {
    // Setzen der Spaltenordnung
    _tvBetriebsMeldungen.getTable().setColumnOrder(_tvSpaltenSortierung);
  }

  /**
   * Kopiert die selektierten Zeilen der Betriebsmeldungstabelle in die Zwischenablage, die Werte in den Spalten werden
   * durch Tabulatoren getrennt.
   * 
   */
  private void kopiereSelektierteZeilen()
  {
    /*
     * Stichwort: TextTransfer.getInstance() Beispiel:
     * http://www.java2s.com/Code/Java/SWT-JFace-Eclipse/SWTClipboardExample.htm
     */
    // TextTransfer
    // copyText = new Text(composite, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL
    // | SWT.H_SCROLL);
    // copyText.setText("some\nplain\ntext");
    // GridData data = new GridData(GridData.FILL_HORIZONTAL);
    // data.heightHint = data.widthHint = SIZE;
    // copyText.setLayoutData(data);
    // Button b = new Button(copyParent, SWT.PUSH);
    // b.setText("Copy");
    // b.addSelectionListener(new SelectionAdapter() {
    // public void widgetSelected(SelectionEvent e) {
    // String data = copyText.getText();
    // if (data.length() > 0) {
    // status.setText("");
    // clipboard.setContents(new Object[] { data },
    // new Transfer[] { TextTransfer.getInstance() });
    // } else {
    // status.setText("nothing to copy");
    // }
    // }
    // });
    //
    // l = new Label(pasteParent, SWT.NONE);
    // l.setText("TextTransfer:"); //$NON-NLS-1$
    // pasteText = new Text(pasteParent, SWT.READ_ONLY | SWT.MULTI
    // | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
    // data = new GridData(GridData.FILL_HORIZONTAL);
    // data.heightHint = data.widthHint = SIZE;
    // pasteText.setLayoutData(data);
    // b = new Button(pasteParent, SWT.PUSH);
    // b.setText("Paste");
    // b.addSelectionListener(new SelectionAdapter() {
    // public void widgetSelected(SelectionEvent e) {
    // String data = (String) clipboard.getContents(TextTransfer
    // .getInstance());
    // if (data != null && data.length() > 0) {
    // status.setText("");
    // pasteText.setText("begin paste>" + data + "<end paste");
    // } else {
    // status.setText("nothing to paste");
    // }
    // }
    // });

  }

  /**
   * This method initializes compositeTop
   * 
   */
  private void createCompositeTop()
  {
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 3;
    GridData gridData = new GridData();
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    gridData.horizontalAlignment = GridData.FILL;
    compositeTop = new Composite(top, SWT.NONE);
    compositeTop.setLayoutData(gridData);
    compositeTop.setLayout(gridLayout1);
    checkBox = new Button(compositeTop, SWT.CHECK);
    checkBox.setText("Filter aktiv");
    checkBox.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        super.widgetSelected(e);
        _filterAktiv = checkBox.getSelection();
        if (_filterAktiv)
          _debug.fine("checked: " + combo.getText());
        else
          _debug.fine("unchecked: " + combo.getText());
      }

    });

    combo = new Combo(compositeTop, SWT.READ_ONLY);
    combo.setLayoutData(gridData11);
    for (Filter f : _filterSpeicher.getFilterListe())
    {
      combo.add(f.getName());
    }
    combo.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        
        combo.removeAll();

        List<Filter> liste = _filterSpeicher.getFilterListe();

        String[] strings = new String[liste.size()];
        for (int i = 0; i < strings.length; i++)
        {
          strings[i] = liste.get(i).getName();

        }

        Arrays.sort(strings);

        for (int i = 0; i < strings.length; i++)
        {
          combo.add(strings[i]);
        }

      }

    });
    combo.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        super.widgetSelected(e);
        
      }

    });

    buttonVerwalten = new Button(compositeTop, SWT.NONE);
    buttonVerwalten.setText("Filter Verwalten");
    buttonVerwalten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionFilterVerwalten();
			}			
		});

  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTable()
  {
    GridData gridData1 = new GridData();
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalAlignment = GridData.FILL;
    composite = new Composite(top, SWT.NONE);
    composite.setLayout(new FillLayout());
    composite.setLayoutData(gridData1);

    erzeugeTabelle();

  }

  /**
   * This method initializes composite3
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout1 = new RowLayout();
    rowLayout1.pack = false;
    rowLayout1.justify = false;
    rowLayout1.fill = false;
    rowLayout1.wrap = false;
    GridData gridData2 = new GridData();
    gridData2.grabExcessHorizontalSpace = false;
    gridData2.verticalAlignment = GridData.CENTER;
    gridData2.horizontalAlignment = GridData.END;
    composite3 = new Composite(top, SWT.NONE);
    composite3.setLayoutData(gridData2);
    composite3.setLayout(rowLayout1);
    buttonOk = new Button(composite3, SWT.NONE);
    buttonOk.setText("OK");
    buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}			
		});

    buttonSchliessen = new Button(composite3, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});

    buttonHilfe = new Button(composite3, SWT.NONE);
    buttonHilfe.setText("Hilfe");    
    buttonHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}			
		});
  }

  private void erzeugeTabelle()
  {

    _tvBetriebsMeldungen = new TableViewer(composite, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

    final TableColumn tc0 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc0.setText("Objekt:");
    tc0.setWidth(100);
    tc0.setMoveable(true);
    tc0.addListener(SWT.Selection, new Listener() {
      private int colPos = 0;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc1 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc1.setText("Zeitstempel:");
    tc1.setWidth(100);
    tc1.setMoveable(true);
    tc1.addListener(SWT.Selection, new Listener() {
      private int colPos = 1;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new TimeComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc2 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc2.setText("ApplikationsID:");
    tc2.setWidth(100);
    tc2.setMoveable(true);
    tc2.addListener(SWT.Selection, new Listener() {
      private int colPos = 2;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new NumberComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc3 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc3.setText("LaufendeNummer:");
    tc3.setWidth(100);
    tc3.setMoveable(true);
    tc3.addListener(SWT.Selection, new Listener() {
      private int colPos = 3;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new NumberComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc4 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc4.setText("ApplikationsTyp:");
    tc4.setWidth(100);
    tc4.setMoveable(true);
    tc4.addListener(SWT.Selection, new Listener() {
      private int colPos = 4;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc5 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc5.setText("ApplikationsKennung:");
    tc5.setWidth(100);
    tc5.setMoveable(true);
    tc5.addListener(SWT.Selection, new Listener() {
      private int colPos = 5;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc6 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc6.setText("ID:");
    tc6.setWidth(100);
    tc6.setMoveable(true);
    tc6.addListener(SWT.Selection, new Listener() {
      private int colPos = 6;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc7 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc7.setText("MeldungsTyp:");
    tc7.setWidth(100);
    tc7.setMoveable(true);
    tc7.addListener(SWT.Selection, new Listener() {
      private int colPos = 7;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc8 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc8.setText("MeldungsTypZusatz:");
    tc8.setWidth(100);
    tc8.setMoveable(true);
    tc8.addListener(SWT.Selection, new Listener() {
      private int colPos = 8;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc9 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc9.setText("MeldungsKlasse:");
    tc9.setWidth(100);
    tc9.setMoveable(true);
    tc9.addListener(SWT.Selection, new Listener() {
      private int colPos = 9;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc10 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc10.setText("Referenz[]:");
    tc10.setWidth(100);
    tc10.setMoveable(true);
    tc10.addListener(SWT.Selection, new Listener() {
      private int colPos = 10;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc11 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc11.setText("Status:");
    tc11.setWidth(100);
    tc11.setMoveable(true);
    tc11.addListener(SWT.Selection, new Listener() {
      private int colPos = 11;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc12 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc12.setText("BenutzerReferenz:");
    tc12.setWidth(100);
    tc12.setMoveable(true);
    tc12.addListener(SWT.Selection, new Listener() {
      private int colPos = 12;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc13 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc13.setText("Ursache:");
    tc13.setWidth(100);
    tc13.setMoveable(true);
    tc13.addListener(SWT.Selection, new Listener() {
      private int colPos = 13;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc14 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc14.setText("Veranlasser:");
    tc14.setWidth(100);
    tc14.setMoveable(true);
    tc14.addListener(SWT.Selection, new Listener() {
      private int colPos = 14;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc15 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT);
    tc15.setText("MeldungsText:");
    tc15.setWidth(100);
    tc15.setMoveable(true);
    tc15.addListener(SWT.Selection, new Listener() {
      private int colPos = 15;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }
          // _debug.fine();

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    _tvBetriebsMeldungen.getTable().setHeaderVisible(true);
    _tvBetriebsMeldungen.getTable().setLinesVisible(true);

    _tvBetriebsMeldungen.setContentProvider(new TabelleInhaltsVersorger());
    _tvBetriebsMeldungen.setLabelProvider(new TabelleDarstellungsVersorger());

    /** ************************************************************************************** */   
    int ops = DND.DROP_COPY |DND.DROP_MOVE;
    _meldungsAnzeigeFallAdapter = new MeldungsanzeigeFallAdapter(_tvBetriebsMeldungen, _meldungenSpeicher
        .getBetriebsmeldungen());
    Transfer[] dropTransfers = new Transfer[] { TextTransfer.getInstance() };
    _tvBetriebsMeldungen.addDropSupport(ops, dropTransfers, _meldungsAnzeigeFallAdapter);

    _zugZuhoererMeldung = new ZugZuhoererMeldung(_tvBetriebsMeldungen, _meldungenSpeicher.getBetriebsmeldungen());
    Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
    _tvBetriebsMeldungen.addDragSupport(ops, dragTransfers, _zugZuhoererMeldung);
    /** ************************************************************************************** */

    _tvBetriebsMeldungen.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof Meldung)
        {
          Meldung meldung = (Meldung)firstElement;
          _debug.fine(meldung.getPid() + " : " + meldung.getZeitStempel() + " : " + meldung.getApplikationsID());

        }
      }
    });
    
    getSite().setSelectionProvider(_tvBetriebsMeldungen);

  }

  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((Meldung)object).getPid();
          break;
        case 1:
          returnString = ((Meldung)object).getZeitStempel();
          break;
        case 2:
          returnString = ((Meldung)object).getApplikationsID();
          break;
        case 3:
          returnString = ((Meldung)object).getLaufendeNummer();
          break;
        case 4:
          returnString = ((Meldung)object).getApplikationsTyp();
          break;
        case 5:
          returnString = ((Meldung)object).getApplikationsKennung();
          break;
        case 6:
          returnString = ((Meldung)object).getID();
          break;
        case 7:
          returnString = ((Meldung)object).getMeldungsTyp();
          break;
        case 8:
          returnString = ((Meldung)object).getMeldungsTypZusatz();
          break;
        case 9:
          returnString = ((Meldung)object).getMeldungsKlasse();
          break;
        case 10:
          returnString = ((Meldung)object).getReferenz();
          break;
        case 11:
          returnString = ((Meldung)object).getStatus();
          break;
        case 12:
          returnString = ((Meldung)object).getUrlasserBenutzerReferenz();
          break;
        case 13:
          returnString = ((Meldung)object).getUrlasserUrsache();
          break;
        case 14:
          returnString = ((Meldung)object).getUrlasserVeranlasser();
          break;
        case 15:
          returnString = ((Meldung)object).getMeldungsText();
          break;
        default:
          returnString = "";

      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  @Override
  public void betriebsmeldungEingang(BetriebsmeldungEreignis e)
  {
    fuelleDialogBetriebsmeldungsListe();
  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen MyComperatorArray umkopiert und darauf hin
   * wird dieser mittels der Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die ursprngliche
   * Position in der Tabelle gemerkt hat, kann man nun ganz einfach die entsprechenden Werte im Original Array tauschen
   * und schon ist man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems, int colPos, BaseComapator[][] comparables,
      boolean sortFlag)
  {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
    String[] textBuffer = null;
    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);
    }

  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */

  class BaseComapator implements Comparable
  {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos)
    {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o)
    {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der einzelnen Tabellenzeilen vornimmt.
   */

  class StringComparator extends BaseComapator implements Comparable
  {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer :
     *          Der Text der Zeile
     * @param rowNum :
     *          Die Zeilennr der Tabellenzeile
     * @param sortFlag :
     *          Aufsteigend false, Absteigend true
     * @param colPos :
     *          Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
    }

    public final int compareTo(Object arg0)
    {
      int ret = _textBuffer[_colPos].compareTo(((StringComparator)arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * NumberComparator kmmert sich um das Vergleichen von Numerischen Werten.
   * 
   */

  class NumberComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public NumberComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        _longValue = Long.parseLong(textBuffer[colPos]);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((NumberComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * TimeComparator kmmert sich um das Vergleichen von Datumsangaben.
   * 
   */

  class TimeComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public TimeComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
        Date d = sdf.parse(textBuffer[colPos]);

        _longValue = d.getTime();

        // _debug.fine(textBuffer[colPos] + " : " + _longValue);

      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((TimeComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  @Override
  public void dispose()
  {
    _daVKommunikation.entferneBetriebsmeldungsListener(this);
    super.dispose();
  }

} // @jve:decl-index=0:visual-constraint="10,10,847,295"
