package de.bsvrz.buv.plugin.engstvw;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.basislib.ort.FilterStrassenSegmentOffset;
import de.bsvrz.buv.rw.basislib.ort.OrtsangabeEnumeratorAuswahl;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IBenutzerZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import de.bsvrz.sys.funclib.vewdynobj.VerwaltungDynObj;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der Betriebsmeldungsverwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface, MutableSetChangeListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Das Verbindungsobjekt
   */
  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;
  
  /**
   * SynchronisationsObjekt fr den Singleton
   */
  private static Object _syncObject = new Object();

  /**
   * Liste der {@link IBenutzerZuhoerer}, die ber einen neue angemeldeten Benuter informiert werden mchten.
   */
  private LinkedList<IEngstellenListener> _engstellenenListener = new LinkedList<IEngstellenListener>();

  private List<Engstelle> _eintraege = new LinkedList<Engstelle>();

  private ConfigurationObject _verkehrsModellObjekt = null;
  
  /**
   * Instanz der Kommandozeilenparameter
   */
  private Kommandozeilenparameter _kommandoZeilenParameter = Kommandozeilenparameter.getInstanz();


  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }
//    if (_instanz == null) { 
//      synchronized(_syncObject) { 
//          if (_instanz == null) { 
//              _instanz = new DaVKommunikation();
//          }
//      }
//    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {

    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };

    if (_connection != null)
    {      
      KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-verkehrsModellObjekt");
      _verkehrsModellObjekt = (ConfigurationObject)_connection.getDataModel().getObject(kp.getWert());

      if (_verkehrsModellObjekt != null)
      {

        anmeldeMengenEngstellen();

      }
      else{
        _debug.error("VerkehrsmodellObjekt ist null");
      }

    }
    else
      _debug.error("Plugin Engstelleneingabe : Connection ist null!");

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Fgt einen Listener zur Liste der Betriebsmeldungslistener hinzu.
   * 
   * @param listener
   */
  public void hinzufuegenEngstellenListener(IEngstellenListener listener)
  {
    if (!_engstellenenListener.contains(listener))
    {
      _engstellenenListener.add(listener);
    }
    else
    {
      _debug
          .error("Der EngstellenZuhrer ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt.");
    }
  }

  /**
   * Entfernt einen {@link IBenutzerZuhoerer} von der Liste der Zuhrer.
   * 
   * @param benutzerZuhoerer
   *          Zu entfernender {@link IBenutzerZuhoerer}.
   */
  public void entferneEngstellenListener(IEngstellenListener listener)
  {
    if (_engstellenenListener.contains(listener))
    {
      _engstellenenListener.remove(listener);
    }
    else
    {
      _debug.error("Der EngstellenZuhrer ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }

  /**
   * Benachrichtigt alle registrierten {@link IBenutzerZuhoerer} darber, dass ein neuer Benutzer sich angemeldet hat.
   * 
   */
  public void benachrichtigeEngstellenListener(List<Engstelle> eintraege)
  {
    EngstellenEreignis ereignis = new EngstellenEreignis(this, eintraege);

    Iterator<IEngstellenListener> it = _engstellenenListener.iterator();
    while (it.hasNext())
    {
      IEngstellenListener listener = it.next();
      listener.engstellenEingang(ereignis);
    }
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
  {
    if (objId == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objId);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeMengenEngstellen()
  {

    MutableSet ms1 = _verkehrsModellObjekt.getMutableSet("Baustellen");

    if (ms1 != null)
    {
      ms1.addChangeListener(this);
      _debug.fine("Listener Menge Baustellen angemeldet");
      
    }
    else{
      _debug.error("Menge Baustellen ist null!");
      ClientApplication app = _connection.getLocalApplicationObject();

      SystemObject user = (SystemObject)_connection.getLocalUser();
      MessageCauser causer = new MessageCauser(user, "Fehler: Anmelden auf Menge", app.getName());
      MessageSender instance = MessageSender.getInstance();
      instance.setApplicationLabel(app.getName());
      
      Long id = app.getId();        
      
      instance.sendMessage(id.toString(), 
          MessageType.SYSTEM_DOMAIN, 
          "", 
          MessageGrade.ERROR,
          (SystemObject)_connection.getLocalConfigurationAuthority(), 
          MessageState.NEW_MESSAGE, 
          causer,
          "Menge Baustellen ist null!");         
    }

    MutableSet ms2 = _verkehrsModellObjekt.getMutableSet("Unflle");

    if (ms2 != null)
    {
      ms2.addChangeListener(this);
      _debug.fine("Listener Menge Unflle angemeldet");

    }
    else{
      _debug.error("Menge Unflle ist null!");
      ClientApplication app = _connection.getLocalApplicationObject();

      SystemObject user = (SystemObject)_connection.getLocalUser();
      MessageCauser causer = new MessageCauser(user, "Fehler: Anmelden auf Menge", app.getName());
      MessageSender instance = MessageSender.getInstance();
      instance.setApplicationLabel(app.getName());

      Long id = app.getId();        
      
      instance.sendMessage(id.toString(), 
          MessageType.SYSTEM_DOMAIN, 
          "", 
          MessageGrade.ERROR,
          (SystemObject)_connection.getLocalConfigurationAuthority(), 
          MessageState.NEW_MESSAGE, 
          causer,
          "Menge Unflle ist null!");         
    }

  }

  /**
   * Verarbeitet smtliche Updates
   */
  public void update(ResultData[] results)
  {
    List<Engstelle> listLoesch = new ArrayList<Engstelle>();
    List<Engstelle> listHinzu = new ArrayList<Engstelle>();
    
    for (int i = 0; i < results.length; i++)
    {

      final ResultData result = results[i];
      final Data data = result.getData();
      final SystemObject object = result.getObject();
      final AttributeGroup atg = result.getDataDescription().getAttributeGroup();

      Engstelle engstelle = null;
      if (data != null)
      {
        List<Engstelle> delList = new ArrayList<Engstelle>();
        
        String vorhanden = null;
        for (Engstelle e : _eintraege)
        {
          if (e.getPid() != null){
          
            if (e.getPid().equals(object.getPid()))
            {              
              engstelle = e;
              vorhanden = e.getPid();
            }
          
          }else{
            _debug.fine("del: " + e.getPid());
            delList.add(e);
          }
            
        }
        
              
        _eintraege.removeAll(delList);

        if (vorhanden == null)
        {

          engstelle = new Engstelle();
        }

        if (atg.getPid().equals("atg.unfallEigenschaften"))
        {
          engstelle.setPid(object.getPid());
          engstelle.setName(object.getName());

          engstelle.setGewaehlterEngstellenTyp(Engstelle.EnumEngstellenTypen.Unfall);

          Long rk = data.getItem("RestKapazitt").asScaledValue().longValue();
          
          if (rk != null)
            engstelle.setRestkapazitaet(rk);
          else
            engstelle.setRestkapazitaet(0L);
            

        }
        else if (atg.getPid().equals("atg.baustellenEigenschaften"))
        {
          engstelle.setPid(object.getPid());
          engstelle.setName(object.getName());

          engstelle.setGewaehlterEngstellenTyp(Engstelle.EnumEngstellenTypen.Baustelle);

          if (data.getItem("Status").asTextValue().getText().equals(Engstelle.EnumStatusBaustelle.entworfen.toString()))
            engstelle.setGewaehlterStatusBaustelle(Engstelle.EnumStatusBaustelle.entworfen);
          else if (data.getItem("Status").asTextValue().getText().equals(
              Engstelle.EnumStatusBaustelle.geplant.toString()))
            engstelle.setGewaehlterStatusBaustelle(Engstelle.EnumStatusBaustelle.geplant);
          else if (data.getItem("Status").asTextValue().getText().equals(
              Engstelle.EnumStatusBaustelle.gltig.toString()))
            engstelle.setGewaehlterStatusBaustelle(Engstelle.EnumStatusBaustelle.gltig);
          else if (data.getItem("Status").asTextValue().getText().equals(
              Engstelle.EnumStatusBaustelle.storniert.toString()))
            engstelle.setGewaehlterStatusBaustelle(Engstelle.EnumStatusBaustelle.storniert);

          engstelle.setRestkapazitaet(data.getItem("RestKapazitt").asUnscaledValue().longValue());

        }
        else if (atg.getPid().equals("atg.situationsEigenschaften"))
        {
          long start = data.getItem("StartZeit").asTimeValue().getMillis();
          engstelle.setZeitlicheGueltigkeitVon(start);
          long ende = start + data.getItem("Dauer").asTimeValue().getMillis();
          engstelle.setZeitlicheGueltigkeitBis(ende);

          long startOffset = -1;

          if (data.getItem("StartOffset").asScaledValue().isNumber())
          {
            startOffset = data.getItem("StartOffset").asScaledValue().longValue();
          }

          long endOffset = -1;

          if (data.getItem("EndOffset").asScaledValue().isNumber())
          {
            endOffset = data.getItem("EndOffset").asScaledValue().longValue();
          }
          
          Array array = data.getItem("StraenSegment").asArray();
          
          if (array.getLength() > 0)
          {
            SystemObject soStart = array.getItem(0).asReferenceValue().getSystemObject();
  
            FilterStrassenSegmentOffset fso1 = new FilterStrassenSegmentOffset((ClientDavConnection)_connection);
            fso1.setPidSelektiertesStrassenSegment(soStart.getPid());
            fso1.setKm(startOffset);
            AttributeGroup atgConf1 = _connection.getDataModel().getAttributeGroup("atg.straenSegment");
            String pidStrasse1 = soStart.getConfigurationData(atgConf1).getItem("gehrtZuStrae").asReferenceValue()
                .getSystemObjectPid();
            fso1.setPidSelektierteStrasse(pidStrasse1);
  
            engstelle.setVonDatenStrassenSegmentOffset(fso1);
            
            SystemObject soEnde = array.getItem(array.getLength() - 1).asReferenceValue().getSystemObject();
  
            FilterStrassenSegmentOffset fso2 = new FilterStrassenSegmentOffset((ClientDavConnection)_connection);
            fso2 = new FilterStrassenSegmentOffset((ClientDavConnection)_connection);
            fso2.setPidSelektiertesStrassenSegment(soEnde.getPid());
            fso2.setKm(endOffset);
            AttributeGroup atgConf2 = _connection.getDataModel().getAttributeGroup("atg.straenSegment");
            String pidStrasse2 = soEnde.getConfigurationData(atgConf2).getItem("gehrtZuStrae").asReferenceValue()
                .getSystemObjectPid();
            fso2.setPidSelektierteStrasse(pidStrasse2);
  
            if (soStart.getPid().equals(soEnde.getPid()) && startOffset == endOffset){
              _debug.fine("Referenz und  Offset gleich!");
            }else
                engstelle.setBisDatenStrassenSegmentOffset(fso2);
            
          }
                    
          engstelle.setLageVonEngstellenAnfangOrtsangabeTyp(OrtsangabeEnumeratorAuswahl.strassensegment);
          engstelle.setLageBisEngstellenEndeOrtsangabeTyp(OrtsangabeEnumeratorAuswahl.strassensegment);
        }

        if (vorhanden == null)
        {
          _eintraege.add(engstelle);
        }
        else
        {
          for (Engstelle e : _eintraege)
          {
            if (e.getPid().equals(engstelle.getPid()))
            {
              if (atg.getPid().equals("atg.situationsEigenschaften"))
              {
                listLoesch.add(e);
                listHinzu.add(engstelle);
              }

            }
          }
        }
      }
    }

    _eintraege.removeAll(listLoesch);
    _eintraege.addAll(listHinzu);

    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        benachrichtigeEngstellenListener(_eintraege);
      }
    });

  }

  public List<Engstelle> getEngstellen()
  {

    return _eintraege;

  }

  @Override
  public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects)
  {
    _debug.fine("update Menge: " + set.getName());

    // if (set.getName().equals("Ereignisse"))
    {

      List<Engstelle> listAdd = new ArrayList<Engstelle>();

      if (addedObjects != null)
      {
        for (int i = 0; i < addedObjects.length; i++)
        {

          SystemObject so = addedObjects[i];

          if (!so.isValid())
          {
            continue;
          }
          
          _debug.fine("added - " + addedObjects[i]);

          DataDescription dd1 = null;

          if (so.isOfType("typ.baustelle"))
          {

            AttributeGroup atg1 = _connection.getDataModel().getAttributeGroup("atg.baustellenEigenschaften");
            Aspect asp1 = _connection.getDataModel().getAspect("asp.parameterSoll");
            dd1 = new DataDescription(atg1, asp1);

          }
          else if (so.isOfType("typ.unfall"))
          {

            AttributeGroup atg1 = _connection.getDataModel().getAttributeGroup("atg.unfallEigenschaften");
            Aspect asp1 = _connection.getDataModel().getAspect("asp.parameterSoll");
            dd1 = new DataDescription(atg1, asp1);

          }

          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.situationsEigenschaften");
          Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
          DataDescription dd = new DataDescription(atg, asp);

          _connection.subscribeReceiver(this, so, dd1, ReceiveOptions.normal(), ReceiverRole.receiver());
          _connection.subscribeReceiver(this, so, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

        }

        _eintraege.addAll(listAdd);

      }

      List<Engstelle> listRem = new ArrayList<Engstelle>();

      if (removedObjects != null)
      {
        for (int i = 0; i < removedObjects.length; i++)
        {
          SystemObject so = removedObjects[i];

          _debug.fine("removed - " + removedObjects[i]);

          for (Engstelle e : _eintraege)
          {
            if (e.getPid().equals(so.getPid()))
            {

              listRem.add(e);
              break;
            }
          }

        }

      }

      _eintraege.removeAll(listRem);

      Display.getDefault().asyncExec(new Runnable() {
        public void run()
        {
          benachrichtigeEngstellenListener(_eintraege);
        }
      });

    }

  }

  public void erzeugeEngstelle(Engstelle eintrag)
  {

    DataModel dataModel = _connection.getDataModel();

    ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
    // ConfigurationAuthority configObj = _connection.getLocalConfigurationAuthority();

    if (_verkehrsModellObjekt != null)
    {

      if (eintrag.getGewaehlterEngstellenTyp().equals(Engstelle.EnumEngstellenTypen.Baustelle))
      {

        DynamicObjectType dynObjTyp = (DynamicObjectType)dataModel.getType("typ.baustelle");
        AttributeGroup attributgruppe = dataModel.getAttributeGroup("atg.baustellenEigenschaften");
        Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
        VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection, dataModel, configArea, dynObjTyp,
            _verkehrsModellObjekt, attributgruppe, aspekt);

        //if (dataModel.getObject(eintrag.getPid()) == null)
        {

          vewEigenschaften.erzeuge(eintrag.getPid(), eintrag.getName(), "Baustellen", null);

        }

      }
      else
      {

        DynamicObjectType dynObjTyp = (DynamicObjectType)dataModel.getType("typ.unfall");
        AttributeGroup attributgruppe = dataModel.getAttributeGroup("atg.unfallEigenschaften");
        Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
        VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection, dataModel, configArea, dynObjTyp,
            _verkehrsModellObjekt, attributgruppe, aspekt);

        //if (dataModel.getObject(eintrag.getPid()) == null)
        {

          vewEigenschaften.erzeuge(eintrag.getPid(), eintrag.getName(), "Unflle", null);

        }
      }

    }
    else
    {
      _debug.error("VerkehrModellObjekt ungueltig oder nicht vorhanden!");
    }

  }

  public void loescheEngstelle(String pid)
  {

    DataModel dataModel = _connection.getDataModel();

    SystemObject o = dataModel.getObject(pid);
    AttributeGroup atg = null;
    DynamicObjectType dynObjType = null;

    if (o.isOfType("typ.baustelle"))
    {
      dynObjType = (DynamicObjectType)dataModel.getType("typ.baustelle");
      atg = dataModel.getAttributeGroup("atg.baustellenEigenschaften");
    }
    else if (o.isOfType("typ.unfall"))
    {
      dynObjType = (DynamicObjectType)dataModel.getType("typ.unfall");
      atg = dataModel.getAttributeGroup("atg.unfallEigenschaften");
    }

    Aspect asp = dataModel.getAspect("asp.parameterSoll");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, dataModel.getObject(pid), dd);

    ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
    // ConfigurationAuthority confObject = _connection.getLocalConfigurationAuthority();
    ConfigurationObject konfigurationsObjekt = _verkehrsModellObjekt;

    asp = dataModel.getAspect("asp.eigenschaften");
    VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection, dataModel, configArea, dynObjType,
        konfigurationsObjekt, atg, asp);
    Data data = _connection.createData(atg);
    Data[] datas = new Data[1];
    datas[0] = data;
    vewEigenschaften.setDynamicObject((DynamicObject)dataModel.getObject(pid));

    if (o.isOfType("typ.baustelle"))
    {
      vewEigenschaften.loesche(pid, "Baustellen");
    }
    else if (o.isOfType("typ.unfall"))
    {
      vewEigenschaften.loesche(pid, "Unflle");
    }

  }

  public void anmeldenNeu()
  {

    // ConfigurationObject konfigurationsObjekt = (ConfigurationObject)_connection.getLocalConfigurationAuthority();
    if (_verkehrsModellObjekt != null)
    {

      ObjectSet objekte1 = _verkehrsModellObjekt.getObjectSet("Baustellen");

      List<SystemObject> list1 = objekte1.getElements();

      List<SystemObject> listTmp1 = new ArrayList<SystemObject>();
      for (SystemObject systemObject : list1)
      {
        if (systemObject.isValid())
        {
          listTmp1.add(systemObject);
        }else
          _debug.fine("ungueltig: " + systemObject.getPid());
      }
      // list.removeAll(listTmp);

      ObjectSet objekte2 = _verkehrsModellObjekt.getObjectSet("Unflle");

      List<SystemObject> list2 = objekte2.getElements();

      List<SystemObject> listTmp2 = new ArrayList<SystemObject>();
      for (SystemObject systemObject : list2)
      {
        if (systemObject.isValid())
        {
          listTmp2.add(systemObject);
        }else
          _debug.fine("ungueltig: " + systemObject.getPid());
      }
      // list.removeAll(listTmp);

      anmeldeLeseDaV(listTmp1, "atg.baustellenEigenschaften", "asp.parameterSoll", ReceiverRole.receiver());
      anmeldeLeseDaV(listTmp1, "atg.situationsEigenschaften", "asp.parameterSoll", ReceiverRole.receiver());
      anmeldeLeseDaV(listTmp2, "atg.unfallEigenschaften", "asp.parameterSoll", ReceiverRole.receiver());
      anmeldeLeseDaV(listTmp2, "atg.situationsEigenschaften", "asp.parameterSoll", ReceiverRole.receiver());

    }
    else
    {

      _debug.error("Kein Verkehrsmodellnetz angegeben oder ungueltig");
    }

  }

  public ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }
  
  public boolean isObjektVorhanden(String pid){
    
    SystemObject object = _connection.getDataModel().getObject(pid);
    
    if (object != null)
      return true;
    
    return false;
  }

}
