package de.bsvrz.buv.plugin.ereigniskal;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoerer;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * In diesem Dialog sind die aktuell definierten Eintrge darzustellen. Fr jeden Eintrag ist die Eigenschaft Name
 * darzustellen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-9</li>
 * </ul>
 * 
 * @version $Revision: 1.4 $ / $Date: 2010/02/04 05:57:26 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class VerwaltungsSicht extends ViewPart implements IEreignisKalenderListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Verbindung zum DaV
   */
  private DaVKommunikation _daVKommunikation = null; // @jve:decl-index=0:

  /**
   * TableViewer
   */
  private static TableViewer _tvSystemKalender = null;

  /**
   * Gibt die Sortierreihenfolge der Spalten an.
   */
  private static int[] _tvSpaltenSortierung = null;

  /**
   * Gibt die Sortierreihenfolge der Spalten an.
   */
  private Eintrag _eintrag = null;

  /**
   * Speichert die SystemkalenderDatenstze fr die tabellarische Anzeige.
   */
  private java.util.List<Eintrag> _ereignisKalenderDatensaetze = null; // @jve:decl-index=0:

  private Composite composite = null; // @jve:decl-index=0:visual-constraint="144,147"

  private Composite compositeTop = null;

  private Composite compositeBottom = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private Composite compositeRight = null;

  private Composite compositeTable = null;

  private Button buttonLoeschen = null;

  private Button buttonKopieren = null;

  private Button buttonBearbeiten = null;

  private Button buttonNeu = null;

  /**
   * Der Listener fr die Berechtigungen
   */
  private IOberflaechenBerechtigungZuhoerer _berechtigungsZuhoerer = null;

  /**
   * Liste der GUI-Elemente fr das Berechtigungskonzept
   */
  private Map<String, Object> _oberflaechenElemente = new HashMap<String, Object>();
  
  /**
   * Instanz der Kommandozeilenparameter
   */
  private Kommandozeilenparameter _kommandoZeilenParameter = Kommandozeilenparameter.getInstanz();
  

  @Override
  public void createPartControl(Composite parent)
  {
    composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    createCompositeTop();
    createCompositeTable();
    createCompositeRight();
    createCompositeBot();

    _daVKommunikation = DaVKommunikation.getInstanz();

    _daVKommunikation.hinzufuegenEreignisKalenderListener(this);

    KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
    if (kp.getWert().equals("true")){
      erzeugeOberflaechenBerechtigung();
    }
    
    _daVKommunikation.anmeldenNeu();

    _ereignisKalenderDatensaetze = _daVKommunikation.getEreignisKalenderEintraege();

    fuelleEreignisKalenderEintraege();

  }

  @Override
  public void setFocus()
  {
  }

  /**
   * Definiert einen neuen Eintrag in der Liste von Filtern fr Betriebsmeldungen.
   * 
   */
  private void aktionNeu()
  {
    EreignisKalenderDefinitionsdialog ereignisDefinitionsdialog = new EreignisKalenderDefinitionsdialog(PlatformUI
        .getWorkbench().getDisplay().getActiveShell());
    _eintrag = ereignisDefinitionsdialog.oeffne();

  }

  /**
   * Bearbeitet einen Eintrag in der Liste von Filtern fr Betriebsmeldungen. Es ffnet sich der Definitionsdialg ({@link EreignisKalenderDefinitionsdialog})
   * zum Bearbeiten des Eintrags.
   * 
   */
  private void aktionBearbeiten()
  {

    if (_eintrag != null){
    
    EreignisKalenderDefinitionsdialog ereignisDefinitionsdialog = new EreignisKalenderDefinitionsdialog(PlatformUI
        .getWorkbench().getDisplay().getActiveShell());
    _eintrag = ereignisDefinitionsdialog.oeffne(_eintrag, false);
    
    }
    else
    {

      MessageBox mb1 = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb1.setText("Fehler");
      mb1.setMessage("Es muss ein Eintrag selektiert werden!");
      mb1.open();

    }

  }

  /**
   * Kopiert einen Eintrag in der Liste von Filtern fr Betriebsmeldungen. Es ffnet sich der Definitionsdialg ({@link EreignisKalenderDefinitionsdialog})
   * zum Bearbeiten des Eintrags.
   * 
   */
  private void aktionKopieren()
  {
    if (_eintrag != null)
    {
    EreignisKalenderDefinitionsdialog ereignisDefinitionsdialog = new EreignisKalenderDefinitionsdialog(PlatformUI
        .getWorkbench().getDisplay().getActiveShell());

    Eintrag clone = _eintrag.clone();

    // clone.setName("Kopie von " + clone.getName());
    _eintrag = ereignisDefinitionsdialog.oeffne(clone, true);
    }
    else
    {

      MessageBox mb1 = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb1.setText("Fehler");
      mb1.setMessage("Es muss ein Eintrag selektiert werden!");
      mb1.open();

    }

  }

  /**
   * Lscht einen selektierten Eintrag nach vorhergehender "Sind sie sicher?"-Abfrage.
   * 
   */
  private void aktionLoeschen()
  {
    if (_eintrag != null)
    {

      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL);
      mb.setText("EreignisKalender Eintrag lschen");
      mb.setMessage("Wollen Sie den EreignisKalenderEintrag: " + _eintrag.getName() + " wirklich lschen?");

      if (mb.open() == SWT.OK)
      {
        _daVKommunikation.loescheEreignisKalenderEintrag(_eintrag.getPid());

        _eintrag = null;
      }

    }
    else
    {

      MessageBox mb1 = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb1.setText("Fehler");
      mb1.setMessage("Es muss ein Eintrag selektiert werden!");
      mb1.open();

    }

  }

  /**
   * Schliet den Verwaltungsdialog {@link VerwaltungsSicht}.
   * 
   */
  private void aktionSchliessen()
  {
    getViewSite().getPage().hideView(VerwaltungsSicht.this);
  }

  /**
   * Zeigt die Hilfe zum Verwaltungsdialog {@link VerwaltungsSicht} an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Aktivator.PLUGIN_ID + ".ereigniskalender";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTop()
  {
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.grabExcessVerticalSpace = true;
    gridData5.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(composite, SWT.NONE);
    compositeTop.setLayoutData(gridData3);
    compositeTop.setLayout(gridLayout);

  }

  private void createCompositeTable()
  {
    GridData gridData1 = new GridData();
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalAlignment = GridData.FILL;
    compositeTable = new Composite(compositeTop, SWT.NONE);
    compositeTable.setLayout(new FillLayout());
    compositeTable.setLayoutData(gridData1);

    erzeugeTabelle();

  }

  /**
   * This method initializes compositeBot
   * 
   */
  private void createCompositeBot()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    rowLayout.fill = false;
    rowLayout.wrap = false;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.END;
    gridData4.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(composite, SWT.NONE);
    compositeBottom.setLayoutData(gridData4);
    compositeBottom.setLayout(rowLayout);
    buttonSchliessen = new Button(compositeBottom, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});

    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}			
		});
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeRight()
  {
    RowLayout rowLayout2 = new RowLayout();
    rowLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout2.wrap = false;
    rowLayout2.pack = false;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.CENTER;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    compositeRight = new Composite(compositeTop, SWT.NONE);
    compositeRight.setLayoutData(gridData6);
    compositeRight.setLayout(rowLayout2);
    buttonNeu = new Button(compositeRight, SWT.NONE);
    buttonNeu.setText("Neu");
    buttonNeu.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionNeu();
			}			
		});

    buttonBearbeiten = new Button(compositeRight, SWT.NONE);
    buttonBearbeiten.setText("Bearbeiten...");   
    buttonBearbeiten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionBearbeiten();
			}			
		});

    buttonKopieren = new Button(compositeRight, SWT.NONE);
    buttonKopieren.setText("Kopieren...");
    buttonKopieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionKopieren();
			}			
		});

    buttonLoeschen = new Button(compositeRight, SWT.NONE);
    buttonLoeschen.setText("Lschen");
    buttonLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionLoeschen();
			}			
		});
  }

  private void erzeugeTabelle()
  {

    _tvSystemKalender = new TableViewer(compositeTable, SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);

    final TableColumn tc0 = new TableColumn(_tvSystemKalender.getTable(), SWT.LEFT | SWT.Move);
    tc0.setText("Name");
    tc0.setWidth(150);
    tc0.setMoveable(true);
    tc0.addListener(SWT.Selection, new Listener() {
      private int colPos = 0;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvSystemKalender.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc1 = new TableColumn(_tvSystemKalender.getTable(), SWT.LEFT | SWT.Move);
    tc1.setText("Zeitspezifikation");
    tc1.setWidth(200);
    tc1.setMoveable(true);
    tc1.addListener(SWT.Selection, new Listener() {
      private int colPos = 1;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvSystemKalender.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    _tvSystemKalender.getTable().setHeaderVisible(true);
    _tvSystemKalender.getTable().setLinesVisible(true);

    _tvSystemKalender.setContentProvider(new TabelleInhaltsVersorger());
    _tvSystemKalender.setLabelProvider(new TabelleDarstellungsVersorger());

    _tvSystemKalender.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof Eintrag)
        {
          _eintrag = (Eintrag)firstElement;

          KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
          if (kp.getWert().equals("true")){
          
            ArrayList<SystemObject> liste = new ArrayList<SystemObject>();

            liste.add(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()));

            pruefeBerechtigung(liste);
          }

        }
        else
          _eintrag = null;

      }

    });

  }

  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((Eintrag)object).getName();
          break;
        case 1:

          Eintrag eintrag = (Eintrag)object;

          if (eintrag.getArtDerZeitspezifikation().equals(Eintrag._artDerZeitspezifikation.Zeitbereichsangabe))
          {

            SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm");
            Date date = new Date();
            date.setTime(eintrag.getAnfangsZeitpunkt());
            String anfang = sdf.format(date);
            date.setTime(eintrag.getEndZeitpunkt());
            String ende = sdf.format(date);

            returnString = anfang + " " + ende;

          }
          else if (eintrag.getArtDerZeitspezifikation().equals(Eintrag._artDerZeitspezifikation.Systemkalendereintrag))
          {

            if (eintrag.getSystemKalenderEintrag() == null)
            {
              returnString = "<keine Zeitspezifikation vorhanden>";
            }
            else
            {
              returnString = eintrag.getSystemKalenderEintrag().getName();
            }
          }
          else
          {
            returnString = "";
          }

          break;

        default:
          returnString = "";

      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen MyComperatorArray umkopiert und darauf hin
   * wird dieser mittels der Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die ursprngliche
   * Position in der Tabelle gemerkt hat, kann man nun ganz einfach die entsprechenden Werte im Original Array tauschen
   * und schon ist man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems, int colPos, BaseComapator[][] comparables,
      boolean sortFlag)
  {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
    String[] textBuffer = null;
    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);
    }

  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */
  class BaseComapator implements Comparable
  {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos)
    {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o)
    {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der einzelnen Tabellenzeilen vornimmt...
   */

  class StringComparator extends BaseComapator implements Comparable
  {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer :
     *          Der Text der Zeile
     * @param rowNum :
     *          Die Zeilennr der Tabellenzeile
     * @param sortFlag :
     *          Aufsteigend false, Absteigend true
     * @param colPos :
     *          Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
    }

    public final int compareTo(Object arg0)
    {
      int ret = _textBuffer[_colPos].compareTo(((StringComparator)arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * NumberComparator kmmert sich um das Vergleichen von Numerischen Werten...
   * 
   */

  class NumberComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public NumberComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        _longValue = Long.parseLong(textBuffer[colPos]);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((NumberComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * TimeComparator kmmert sich um das Vergleichen von Datumsangaben...
   * 
   */

  class TimeComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public TimeComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
        Date d = sdf.parse(textBuffer[colPos]);

        _longValue = d.getTime();

        // _debug.fine(textBuffer[colPos] + " : " + _longValue);

      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((TimeComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  @Override
  public void ereignisKalenderEingang(EreignisKalenderEreignis e)
  {
    // _systemkalenderDatensaetze = e.getSystemKalenderEintraege();
    fuelleEreignisKalenderEintraege();

  }

  /**
   * Fllt die Liste der Engstellen des Dialogs.
   */
  private void fuelleEreignisKalenderEintraege()
  {

    // java.util.List<Eintrag> eintraege = _eintraegeSpeicher.getEintraege();
    _tvSystemKalender.setInput(_ereignisKalenderDatensaetze);

  }

  private void erzeugeOberflaechenBerechtigung()
  {
    
    _berechtigungsZuhoerer = new IOberflaechenBerechtigungZuhoerer() {
      @Override
      public void freigabe(OberflaechenBerechtigungEreignis e)
      {
        _debug.fine("Info Freigabe erhalten: " + e.hatFreigabe());

        List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
        List<SystemObject> liste2 = e.getSystemObjekteSperrung();
        for (SystemObject so : liste1)
        {
          _debug.fine("Objekte Freigabe: " + so);
        }
        for (SystemObject so : liste2)
        {
          _debug.fine("Objekte Sperrung: " + so);
        }

        List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
        List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
        for (OberflaechenFunktion of : liste3)
        {
          _debug.fine("Funktion Freigabe: " + of.getFunktionsName());
          if (_oberflaechenElemente.containsKey(of.getFunktionsName()))
          {
            Object object = _oberflaechenElemente.get(of.getFunktionsName());
            if (object instanceof Button)
            {
              Button button = (Button)object;
              button.setEnabled(true);
              
            }
            
          }

        }
        for (OberflaechenFunktion of : liste4)
        {
          _debug.fine("Funktion Sperrung: " + of.getFunktionsName());
        }

      }

      @Override
      public void sperrung(OberflaechenBerechtigungEreignis e)
      {
        _debug.fine("Info Sperrung erhalten: " + e.hatFreigabe());

        List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
        List<SystemObject> liste2 = e.getSystemObjekteSperrung();
        for (SystemObject so : liste1)
        {
          _debug.fine("Objekte Freigabe: " + so);
        }
        for (SystemObject so : liste2)
        {
          _debug.fine("Objekte Sperrung: " + so);
        }

        List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
        List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
        for (OberflaechenFunktion of : liste3)
        {
          _debug.fine("Funktion Freigabe: " + of.getFunktionsName());
        }
        for (OberflaechenFunktion of : liste4)
        {
          _debug.fine("Funktion Sperrung: " + of.getFunktionsName());
          if (_oberflaechenElemente.containsKey(of.getFunktionsName()))
          {
            Object object = _oberflaechenElemente.get(of.getFunktionsName());
            if (object instanceof Button)
            {
              Button button = (Button)object;
              button.setEnabled(false);
              
            }
           
          }

        }

      }
    };

    de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz().hinzufuegenOberflaechenBenutzerZuhoerer(
        _berechtigungsZuhoerer);
    

    OberflaechenFunktion funktion1 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht.schaltflaeche.neu", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht", "anlegen");
    OberflaechenFunktion funktion2 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht.schaltflaeche.bearbeiten", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht", "bearbeiten");
    OberflaechenFunktion funktion3 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht.schaltflaeche.kopieren", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht", "kopieren");
    OberflaechenFunktion funktion4 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht.schaltflaeche.loeschen", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.ereigniskal.VerwaltungsSicht", "loeschen");

    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion1);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion2);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion3);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion4);

    _oberflaechenElemente.put(funktion1.getFunktionsName(), buttonNeu);
    _oberflaechenElemente.put(funktion2.getFunktionsName(), buttonBearbeiten);
    _oberflaechenElemente.put(funktion3.getFunktionsName(), buttonKopieren);
    _oberflaechenElemente.put(funktion4.getFunktionsName(), buttonLoeschen);

  }

  @Override
  public void dispose()
  {
    _daVKommunikation.entferneEreignisKalenderListener(this);
    KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
    if (kp.getWert().equals("true")){
      de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
          _berechtigungsZuhoerer);
    }
    super.dispose();
  }
  
  /**
   * Prft alle SystemObjekte der Liste ob eine Freigabe oder Sperrung vorliegt 
   * @param liste
   */
  private void pruefeBerechtigung(List<SystemObject> liste){
    for (Map.Entry<String, Object> me : _oberflaechenElemente.entrySet())
    {
      OberflaechenBerechtigung.getInstanz().pruefeBerechtigungElement(
          liste, me.getKey());
      
    }
  }

} // @jve:decl-index=0:visual-constraint="10,10,306,259"
