/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw;

import org.eclipse.core.resources.ISaveContext;
import org.eclipse.core.resources.ISaveParticipant;
import org.eclipse.core.runtime.CoreException;

/**
 * Diese Klasse demonstriert, wie die Implementierung eines
 * {@link ISaveParticipant} aussehen kann. <br>
 * <br>
 * Wird ein {@link ISaveParticipant} beim <code>ResourcesPlugin</code>
 * registriert (blicher weise erfolgt dies im Aktivator des Plug-Ins), wird
 * dieser bei Beendigung des Rahmenwerkes ausgefhrt.<br>
 * Es bietet sich an, hier smtliche Aktivitten zum Speichern des Zustandes des
 * Plug-Ins unterzubringen.
 * 
 * @author HofmannJ
 * 
 */
public class Saver implements ISaveParticipant, Runnable
{
	/**
	 * Timeout vor dem Beenden
	 */
	private static Integer _timeOut = null;

	/**
	 * Thread-Flag das anzeigt, das der Thread zum "Sichern" aktiv bzw. nicht
	 * aktiv ist.
	 */
	private boolean _threadLaufe = false;

	public Saver()
	{
	}

	@Override
	public void doneSaving(ISaveContext context)
	{
		while (_threadLaufe)
		{
			/*
			 * Warten, bis der Hintergrundthread zum "Sichern" mit seiner Aufgabe
			 * fertig ist.
			 */
		}
	}

	@Override
	public void prepareToSave(ISaveContext context) throws CoreException
	{
	}

	@Override
	public void rollback(ISaveContext context)
	{		
	}

	@Override
	public void saving(ISaveContext context) throws CoreException
	{
		/*
		 * Hier sollten nun alle Aktivitten durchgefhrt werden, die den Status
		 * dieses Plug-Ins sichern.
		 */
		startThread();
	}

	public void startThread()
	{
		if (!_threadLaufe)
		{
			_threadLaufe = true;
			Thread th = new Thread(this);
			th.setName("ISaveParticipant Warte-Test Thread");
			th.start();
		}
	}

	public void stopThread()
	{
		if (_threadLaufe)
		{
			_threadLaufe = false;
		}
	}

	@Override
	public void run()
	{
		while (_threadLaufe)
		{
			int zeitSpanne = 1;
			if (_timeOut != null)
			{
				zeitSpanne = _timeOut.intValue();
			}

			try
			{
				/*
				 * Auf diese Weise kann das Rahmenwerk beim Beenden aufgehalten werden.
				 */
				// Thread.sleep(10000);
				Thread.sleep(zeitSpanne);
				stopThread();
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}
	}

	public static void setTimeOut(int timeout)
	{
		if (timeout >= 0)
		{
			_timeOut = new Integer(timeout);
		}
	}

}
