/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.aktionen;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.legende.ILegendenAbschnitt;
import de.bsvrz.buv.rw.basislib.legende.LegendeDarzustellendeZeit;
import de.bsvrz.buv.rw.basislib.legende.LegendeSicht;

/**
 * Diese Klasse liefert ein Beispiel, wie man {@link ILegendenAbschnitt}e
 * erzeugt und in der {@link LegendeSicht} anzeigt.
 * 
 * @author HofmannJ
 * 
 */
public class AktionLegendeBeispiel implements IWorkbenchWindowActionDelegate
{
	/**
	 * ID der Sicht der Legende.
	 */
	private static final String _idSichtLegende = "de.bsvrz.buv.rw.basislib.sichtLegende";

	/**
	 * 1.ter Legendenabschnitt
	 */
	private LegendenAbschnittGenerator _lg1 = null;

	/**
	 * 2.ter Legendenabschnitt
	 */
	private LegendenAbschnittGenerator _lg2 = null;

	@Override
	public void dispose()
	{
	}

	@Override
	public void init(IWorkbenchWindow window)
	{
		/*
		 * Erzeugen der Legendenabschnitte
		 */

		_lg1 = new LegendenAbschnittGenerator("1",
				"Der erste Abschnitt der Legende.");
		_lg2 = new LegendenAbschnittGenerator("2",
				"Ein zweiter Abschnitt fr die Legende.");
	}

	@Override
	public void run(IAction action)
	{
		LegendeDarzustellendeZeit.getInstanz().setZeitpunktJetzt();
		try
		{
			/*
			 * Erzeugen und ffnen der View
			 */
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
					.showView(_idSichtLegende);

			/*
			 * Beschaffen eines Zugriffs auf die View
			 */
			LegendeSicht ls = (LegendeSicht) PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage().findView(_idSichtLegende);

			/*
			 * Hinzufgen der beiden Legendenabschnitte.
			 */
			ls.hinzufuegenLegendenAbschnitt(_lg1);
			ls.hinzufuegenLegendenAbschnitt(_lg2);

		} catch (PartInitException e)
		{
			e.printStackTrace();
		}

	}

	@Override
	public void selectionChanged(IAction action, ISelection selection)
	{		
	}

	/**
	 * Diese Klasse liefert die Implementierung des Interface
	 * {@link ILegendenAbschnitt}. Der Konstruktor bernimmt ID und Titel des
	 * Legendenabschnittes, in der Methode
	 * {@link ILegendenAbschnitt#erzeugeAbschnittsInhalt(Composite)} wird der
	 * Inhalt des Legendenabschnittes mit einem Button erzeugt.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class LegendenAbschnittGenerator implements ILegendenAbschnitt
	{
		/**
		 * Speichert die ID dieses Legendenabschnittes.
		 */
		private String _id = null;

		/**
		 * Speichert den Titel dieses Legendenabschnittes
		 */
		private String _titel = null;

		/**
		 * Konstruktor, bernimmt ID und Titel des {@link ILegendenAbschnitt}es.
		 * 
		 * @param id
		 *          ID
		 * @param titel
		 *          Titel
		 */
		public LegendenAbschnittGenerator(String id, String titel)
		{			
			this._id = id;
			this._titel = titel;			
		}

		@Override
		public void erzeugeAbschnittsInhalt(Composite flaeche)
		{
			GridData gd = new GridData();
			gd.grabExcessHorizontalSpace = true;
			gd.grabExcessVerticalSpace = true;
			gd.horizontalAlignment = GridData.FILL;
			gd.verticalAlignment = GridData.FILL;

			flaeche.setLayoutData(gd);

			flaeche.setLayout(new GridLayout());

			
			/*
			 * Einen Button hinzufgen und mit einer Aktion ausstatten.
			 */
			Button bu = new Button(flaeche, SWT.PUSH);
			bu.setText("Ein Button in einem Legendenabschnitt.");
			bu.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
							.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
					mb.setText(_titel + ":");
					mb
							.setMessage("Der Button aus der Legende kann eine Aktion durchfhren!");
					mb.open();
				}			
			});
		}

		@Override
		public String getId()
		{
			return _id;
		}

		@Override
		public String getTitel()
		{
			return _titel;
		}
	}

}
