/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.screenshot;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.pruefrw.screenshot.DialogAuswahlScreenShotArt.BildschirmSicherungsBereich;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Erstellt einen Screenshot des aktuellen Fensters oder des gesamten
 * Bildschirmbereiches.
 * 
 * @author HofmannJ
 * 
 */
public class BildschirmInhaltFangen
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger(); 
  
	/**
	 * Konstruktor
	 */
	public BildschirmInhaltFangen()
	{
		DialogAuswahlScreenShotArt dia = new DialogAuswahlScreenShotArt(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());

		BildschirmSicherungsBereich bereichsWahl = dia.open();

		if (bereichsWahl != null)
		{
			switch (bereichsWahl)
			{
			case fenster:
				_debug.fine("BEREICH");
				aktionFangeInhaltAktuellesFenster();
				break;
			case bildschirm:
				_debug.fine("FENSTER");
				aktionFangeInhaltGesamterBildschirm();
				break;
			default:
				_debug.fine("Nix!");
				break;
			}
		}
	}

	/**
	 * Erzeugt den Screenshot des aktuellen Fensters.
	 */
	public void aktionFangeInhaltAktuellesFenster()
	{
		Rectangle bereich = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getShell().getBounds();
		aktionBereichFangen(bereich);

	}

	/**
	 * Erzeugt einen Screenshot des gesamten Bildschirmbereiches.
	 */
	public void aktionFangeInhaltGesamterBildschirm()
	{
		Rectangle bereich = PlatformUI.getWorkbench().getDisplay().getBounds();
		aktionBereichFangen(bereich);
	}

	/**
	 * Liefert den Pfad, unter dem die Datei gespeichert werden soll.
	 * 
	 * @return Pfad zur Datei
	 */
	private String speichernUnterDialog()
	{
		FileDialog dialog = new FileDialog(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.SAVE);

		dialog.setFilterNames(new String[] { "Portable Network Graphics" });
		dialog.setFilterExtensions(new String[] { "*.png" });
		String pfad = dialog.open();
		return pfad;
	}

	/**
	 * "Fngt" den durch das {@link Rectangle} angegebenen Bereichs des
	 * Bildschirms und speichert ihn in einer PNG-Datei ab.
	 * 
	 * @param bereich
	 *          Zu "fangender" Bereich.
	 */
	private void aktionBereichFangen(Rectangle bereich)
	{
		int x = bereich.x;
		int y = bereich.y;
		int breite = bereich.width;
		int hoehe = bereich.height;

			_debug.fine("PosX: " + x + " PosY: " + y + " Breite: " + breite
					+ " Hhe: " + hoehe);

		GC gc = new GC(PlatformUI.getWorkbench().getDisplay());
		Image image = new Image(PlatformUI.getWorkbench().getDisplay(),
				new Rectangle(x, y, breite, hoehe));
		gc.copyArea(image, x, y);

		ImageLoader loader = new ImageLoader();
		loader.data = new ImageData[] { image.getImageData() };

		String pfad = speichernUnterDialog();

		if (pfad != null)
		{
			// loader.save("C:\\shot.png", SWT.IMAGE_PNG);
			loader.save(pfad, SWT.IMAGE_PNG);
		}
	}

	/**
	 * Liefert ein Vorschaufenster, in dem ein {@link Image} angezeigt werden
	 * kann.
	 * 
	 * @param img
	 *          Anzuzeigendes {@link Image}
	 */
	public void vorschauScreenShot(Image img)
	{
		final Image image = img;

		Shell popup = new Shell(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.SHELL_TRIM);
		popup.setLayout(new FillLayout());
		popup.setText("Image");
		popup.setBounds(50, 50, image.getBounds().width, image.getBounds().height);
		popup.addListener(SWT.Close, new Listener()
		{
			public void handleEvent(Event e)
			{
				image.dispose();
			}
		});

		ScrolledComposite sc = new ScrolledComposite(popup, SWT.V_SCROLL
				| SWT.H_SCROLL);
		Canvas canvas = new Canvas(sc, SWT.NONE);
		sc.setContent(canvas);
		canvas.setBounds(PlatformUI.getWorkbench().getDisplay().getBounds());
		canvas.addPaintListener(new PaintListener()
		{
			public void paintControl(PaintEvent e)
			{
				e.gc.drawImage(image, 0, 0);
			}
		});
		popup.open();
	}

}
