/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.screenshot;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

/**
 * Mit Hilfe dieses Dialogs kann eine Auswahl getroffen werden, welcher
 * Bildschirmbereich via Screenshot gesichert werden soll.
 * 
 * @author HofmannJ
 * 
 */
public class DialogAuswahlScreenShotArt extends Dialog
{
	private Shell _shell = null;
	private Group _gruppeOben = null;
	private Composite _compositeUnten = null;
	private Composite _compositeUntenRechts = null;
	private Button _radioButtonFenster = null;
	private Button _radioButtonGesamterBildschirm = null;
	private Button _buttonOk = null;
	private Button _buttonAbbrechen = null;

	/**
	 * Bereich des Bildschirms, der in eine Datei gespeichert werden soll.
	 */
	private BildschirmSicherungsBereich _sicherungsBereich = null;

	public static void main(String[] args)
	{
		final Display display = new Display();
		final Shell parent = new Shell(display, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);

		DialogAuswahlScreenShotArt dlg = new DialogAuswahlScreenShotArt(parent);
		dlg.open();
	}

	public DialogAuswahlScreenShotArt(Shell parent)
	{
		super(parent, 0);
	}

	public BildschirmSicherungsBereich open()
	{
		Shell parent = getParent();
		_shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		_shell.setText("Bildschirm Bereich whlen");
		_shell.setSize(new Point(300, 145));
		_shell.setLayout(new GridLayout());

		erzeugeGruppeOben();
		erzeugeCompositeUnten();

		_shell.open();
		Display display = parent.getDisplay();
		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _sicherungsBereich;

	}

	/**
	 * Initialisierung der oberen Gruppe Bereichswahl. Beinhaltet zwei
	 * Radiobuttons zur Auswahl des aktuellen Fensters bzw. des gesamten
	 * Bildschirmbereiches.
	 * 
	 */
	private void erzeugeGruppeOben()
	{
		GridData gridDataGruppeOben = new GridData();
		gridDataGruppeOben.horizontalAlignment = GridData.FILL;
		gridDataGruppeOben.grabExcessHorizontalSpace = true;
		gridDataGruppeOben.grabExcessVerticalSpace = true;
		gridDataGruppeOben.verticalAlignment = GridData.FILL;
		_gruppeOben = new Group(_shell, SWT.NONE);
		_gruppeOben.setLayout(new GridLayout());
		_gruppeOben.setLayoutData(gridDataGruppeOben);
		_gruppeOben.setText("Bereichswahl:");

		_radioButtonFenster = new Button(_gruppeOben, SWT.RADIO);
		_radioButtonFenster.setText("Fenster");
		_radioButtonFenster.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				_sicherungsBereich = BildschirmSicherungsBereich.fenster;
			}
		});

		_radioButtonGesamterBildschirm = new Button(_gruppeOben, SWT.RADIO);
		_radioButtonGesamterBildschirm.setText("Gesamter Bildschirmbereich");
		_radioButtonGesamterBildschirm.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				_sicherungsBereich = BildschirmSicherungsBereich.bildschirm;
			}
		});
	}

	/**
	 * Initialisierung der unteren Composites
	 * 
	 */
	private void erzeugeCompositeUnten()
	{
		GridData gridDataCompositeUnten = new GridData();
		gridDataCompositeUnten.grabExcessHorizontalSpace = true;
		gridDataCompositeUnten.verticalAlignment = GridData.CENTER;
		gridDataCompositeUnten.horizontalAlignment = GridData.FILL;
		_compositeUnten = new Composite(_shell, SWT.NONE);
		_compositeUnten.setLayout(new GridLayout());
		erzeugeCompositeUntenRechts();
		_compositeUnten.setLayoutData(gridDataCompositeUnten);
	}

	/**
	 * Initialisierung des unteren, rechten Composites. Beinhaltet die
	 * {@link Button} bernehmen und Abbrechen.
	 * 
	 */
	private void erzeugeCompositeUntenRechts()
	{
		GridData gridDataCompositeUntenRechts = new GridData();
		gridDataCompositeUntenRechts.horizontalAlignment = GridData.END;
		gridDataCompositeUntenRechts.grabExcessHorizontalSpace = true;
		gridDataCompositeUntenRechts.verticalAlignment = GridData.CENTER;
		RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.pack = false;
		rowLayout.fill = false;
		_compositeUntenRechts = new Composite(_compositeUnten, SWT.NONE);
		_compositeUntenRechts.setLayout(rowLayout);
		_compositeUntenRechts.setLayoutData(gridDataCompositeUntenRechts);
		_buttonOk = new Button(_compositeUntenRechts, SWT.NONE);
		_buttonOk.setText("bernehmen");
		_buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_shell.dispose();
			}			
		});

		_buttonAbbrechen = new Button(_compositeUntenRechts, SWT.NONE);
		_buttonAbbrechen.setText("Abbrechen");
		_buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_sicherungsBereich = null;
				_shell.dispose();
			}			
		});
	}

	/**
	 * Enumerator zur Auswahl, welcher der Bildschirmbereiche via Screenshot
	 * gesichert werden soll.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum BildschirmSicherungsBereich
	{
		/**
		 * Sichern des aktuellen Fensters
		 */
		fenster,

		/**
		 * Sichern des gesamten Bildschirmbereiches
		 */
		bildschirm
	}

}
