/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.symbolleiste;

import java.io.File;

import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;

import de.bsvrz.buv.rw.basislib.menu.generatoren.SymbolleisteGenerator;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;

/**
 * Mit Hilfe dieser View kann der Symbolleisteneditor gestartet werden. Es kann
 * dann eine XML-Datei mit der Struktur der Symbolleiste geladen werden. Beim
 * Speichern wird die Darstellung der Symbolleiste aktualisiert.
 * 
 * @author HofmannJ
 * 
 */
public class ViewSymbolleisteBeispiel extends ViewPart
{

	public static final String ID = "de.bsvrz.buv.plugin.beispiele.symbolleiste.ViewSymbolleiseBeispiel";
	private Composite top = null;
	private Group gruppeUebersicht = null;
	private Button buttonSymbolleistenStrukturLaden = null;

	@Override
	public void createPartControl(Composite parent)
	{
		top = new Composite(parent, SWT.NONE);
		top.setLayout(new GridLayout());
		createGruppeUebersicht();
	}

	@Override
	public void setFocus()
	{
	}

	private void aktionSymbolleistenStrukturAusDateiLaden()
	{
		FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(),
				SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.OPEN);
		dialog.setFilterNames(new String[] { "XML-Dateien" });
		dialog.setFilterExtensions(new String[] { "*.xml" });
		String dateiPfad = dialog.open();

		if (dateiPfad != null)
		{
			MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();

			File file = new File(dateiPfad);
			if (file.exists())
			{
				msld.einlesenXmlAusDatei(dateiPfad);
				ICoolBarManager coolBarManager = ElementRegistrierung.getInstanz()
						.getCoolBarManager();
				SymbolleisteGenerator generator = new SymbolleisteGenerator(
						coolBarManager, msld);
				generator.generiereSymbolLeiste();
			}
		}

	}

	/**
	 * This method initializes gruppeUebersicht
	 * 
	 */
	private void createGruppeUebersicht()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		gruppeUebersicht = new Group(top, SWT.NONE);
		gruppeUebersicht.setLayout(new GridLayout());
		gruppeUebersicht.setLayoutData(gridData);
		gruppeUebersicht.setText("bersicht");
		buttonSymbolleistenStrukturLaden = new Button(gruppeUebersicht, SWT.NONE);
		buttonSymbolleistenStrukturLaden
				.setText("Symbolleistenstruktur aus Datei laden");
		buttonSymbolleistenStrukturLaden.setToolTipText("ffnet die Symbolleisteneditorperspektive.");
		buttonSymbolleistenStrukturLaden.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSymbolleistenStrukturAusDateiLaden();
			}			
		});
	}

}
