/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.zustaende;

import java.util.ArrayList;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.zustaende.Zustaende;
import de.bsvrz.sys.funclib.debug.Debug;


/**
 * In diesem Dialog sind die aktuell definierten Eintrge darzustellen. Fr
 * jeden Eintrag ist die Eigenschaft Name darzustellen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und
 * Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-9</li>
 * </ul>
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/14 14:11:39 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class ZustandVerwaltungSicht extends ViewPart
{
  
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  
  public static enum Zustand {
    Zustand1("Ok"), Zustand2("Teilstrung"), Zustand3("Gestrt"), Zustand4("nicht definiert");
    @SuppressWarnings("unused")
		private String wert;

    private Zustand(String s)
    {
      wert = s;
    }
  };
 
  
  
	/**
	 * Ist <code>true</code>, wenn sich etwas an den Filtern gendert hat,
	 * sonst <code>false</code>;
	 */
	@SuppressWarnings("unused")
	private boolean _geandert = false;

	/**
	 * FilterSpeicher.
	 */
	private Zustaende _zustaende = Zustaende.getInstanz();
	
  private Composite composite = null;  //  @jve:decl-index=0:visual-constraint="144,147"

  private Composite compositeTop = null;

  private Composite compositeBottom = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private List list = null;

  private Composite compositeRight = null;

  private Button buttonLoeschen = null;

  private Button buttonBearbeiten = null;

  private Button buttonNeu = null;
  
  private Group _gruppeBeschreibung = null;

  private Text _textAreaBeschreibung = null;

  
  @Override
  public void createPartControl(Composite parent)
  {
    composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    createCompositeTop();
    createGruppeBeschreibung();
    createCompositeBot();

    
  }

  @Override
  public void setFocus()
  {
  }
  

	/**
	 * Definiert einen neuen Eintrag in der Liste von Filtern fr
	 * Betriebsmeldungen.
	 * 
	 */
	private void aktionNeu()
	{
    ZustandDefinitionsdialog zustandDefinitionsdialog = new ZustandDefinitionsdialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
    @SuppressWarnings("unused")
		String s = zustandDefinitionsdialog.oeffne();
      
    java.util.List<String> l = new ArrayList<String>();
    for (int i = 0; i < list.getItemCount(); i++)
    {      
      l.add(list.getItem(i));     
    }

    
    for (Map.Entry<String , Object> me : _zustaende.getZustaende().entrySet())        
    {
      if (!l.contains(me.getKey()))      
        list.add(me.getKey());
    }
    _textAreaBeschreibung.setText("Aktion Neu wurde ausgefhrt");
       
	}

	/**
	 * Bearbeitet einen Eintrag in der Liste von Filtern fr Betriebsmeldungen. Es
	 * ffnet sich der Definitionsdialg ({@link ZustandDefinitionsdialog}) zum
	 * Bearbeiten des Eintrags.
	 * 
	 */
	private void aktionBearbeiten()
	{
	  
 
    ZustandDefinitionsdialog zustandDefinitionsdialog = new ZustandDefinitionsdialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
    
    String name = list.getSelection()[0];
    
    if (name != null){
      @SuppressWarnings("unused")
			String s = zustandDefinitionsdialog.oeffne(name);
    }
    else
      System.out.println("Keine Selektion");
 
    _textAreaBeschreibung.setText("Aktion Bearbeiten wurde ausgefhrt");
	}

	/**
	 * Lscht einen selektierten Eintrag nach vorhergehender "Sind sie
	 * sicher?"-Abfrage.
	 * 
	 */
	private void aktionLoeschen()
	{
	  
    String string = list.getSelection()[0];
    
    java.util.List<String> l = new ArrayList<String>();
    for (int i = 0; i < list.getItemCount(); i++)
    {      
      l.add(list.getItem(i));     
    }
    
    if (l.contains(string))
      list.remove(string);
   
    
    if (_zustaende.getZustaende().containsKey(string))      
      _zustaende.entferneZustand(string);
      
    
    
    for (Map.Entry<String , Object> me : _zustaende.getZustaende().entrySet())        
    {
      if (!l.contains(me.getKey()))      
        list.add(me.getKey());
    }
        
    _textAreaBeschreibung.setText("Aktion Lschen wurde ausgefhrt");
   
	}

	/**
	 * Schliet den Verwaltungsdialog {@link ZustandVerwaltungSicht}.
	 * 
	 */
	private void aktionSchliessen()
	{
	  getViewSite().getPage().hideView(ZustandVerwaltungSicht.this);
	}

	/**
	 * Zeigt die Hilfe zum Verwaltungsdialog {@link ZustandVerwaltungSicht} an.
	 * 
	 */
	private void aktionHilfe()
	{

	}


    /**
     * This method initializes composite	
     *
     */
    private void createCompositeTop()
    {
      GridData gridData5 = new GridData();
      gridData5.horizontalAlignment = GridData.FILL;
      gridData5.grabExcessHorizontalSpace = true;
      gridData5.grabExcessVerticalSpace = true;
      gridData5.verticalAlignment = GridData.FILL;
      GridLayout gridLayout = new GridLayout();
      gridLayout.numColumns = 2;
      GridData gridData3 = new GridData();
      gridData3.horizontalAlignment = GridData.FILL;
      gridData3.grabExcessHorizontalSpace = true;
      gridData3.grabExcessVerticalSpace = true;
      gridData3.verticalAlignment = GridData.FILL;
      compositeTop = new Composite(composite, SWT.NONE);
      compositeTop.setLayoutData(gridData3);
      compositeTop.setLayout(gridLayout);
      list = new List(compositeTop, SWT.H_SCROLL | SWT.V_SCROLL);
      list.setLayoutData(gridData5);
      
      for (Map.Entry<String , Object> me : _zustaende.getZustaende().entrySet())        
      {
        list.add(me.getKey());
      }
      
      
      list.addSelectionListener(new SelectionAdapter(){
        @Override
        public void widgetDefaultSelected(SelectionEvent e)
        {
          super.widgetDefaultSelected(e);
          ZustandDefinitionsdialog filterDefinitionsdialog = new ZustandDefinitionsdialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
         
          String name = null;
          for (Map.Entry<String , Object> me : _zustaende.getZustaende().entrySet())        
          {
            if (me.getKey().equals(list.getSelection()[0])){
              name = me.getKey();
              break;
            }
          }
          
          if (name != null)
            
            filterDefinitionsdialog.oeffne(name);
          
        }
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          super.widgetSelected(e);
          _debug.fine("List: " + list.getSelection()[0]);
        }
        
      });
      
      createCompositeRight();
      
    }

    /**
     * This method initializes compositeBot	
     *
     */
    private void createCompositeBot()
    {
      RowLayout rowLayout = new RowLayout();
      rowLayout.pack = false;
      rowLayout.fill = false;
      rowLayout.wrap = false;
      GridData gridData4 = new GridData();
      gridData4.horizontalAlignment = GridData.END;
      gridData4.verticalAlignment = GridData.CENTER;
      compositeBottom = new Composite(composite, SWT.NONE);
      compositeBottom.setLayoutData(gridData4);
      compositeBottom.setLayout(rowLayout);
      buttonSchliessen = new Button(compositeBottom, SWT.NONE);
      buttonSchliessen.setText("Schlieen");
      buttonSchliessen.addSelectionListener(new SelectionAdapter()
  		{
  			@Override
  			public void widgetSelected(SelectionEvent e)
  			{
  				aktionSchliessen();
  			}			
  		});
      
      
      buttonHilfe = new Button(compositeBottom, SWT.NONE);
      buttonHilfe.setText("Hilfe");
      buttonHilfe.addSelectionListener(new SelectionAdapter()
  		{
  			@Override
  			public void widgetSelected(SelectionEvent e)
  			{
  				aktionHilfe();
  			}			
  		});
    }

    /**
     * This method initializes composite	
     *
     */
    private void createCompositeRight()
    {
      RowLayout rowLayout2 = new RowLayout();
      rowLayout2.type = org.eclipse.swt.SWT.VERTICAL;
      rowLayout2.wrap = false;
      rowLayout2.pack = false;
      GridData gridData6 = new GridData();
      gridData6.horizontalAlignment = GridData.CENTER;
      gridData6.grabExcessVerticalSpace = true;
      gridData6.verticalAlignment = GridData.FILL;
      compositeRight = new Composite(compositeTop, SWT.NONE);
      compositeRight.setLayoutData(gridData6);
      compositeRight.setLayout(rowLayout2);
      buttonNeu = new Button(compositeRight, SWT.NONE);
      buttonNeu.setText("Neu");
      buttonNeu.addSelectionListener(new SelectionAdapter()
  		{
  			@Override
  			public void widgetSelected(SelectionEvent e)
  			{
  				aktionNeu();          
  			}			
  		});
         
      buttonBearbeiten = new Button(compositeRight, SWT.NONE);
      buttonBearbeiten.setText("Bearbeiten...");
      buttonBearbeiten.addSelectionListener(new SelectionAdapter()
  		{
  			@Override
  			public void widgetSelected(SelectionEvent e)
  			{
  				aktionBearbeiten();
  			}			
  		});
      
      
      buttonLoeschen = new Button(compositeRight, SWT.NONE);
      buttonLoeschen.setText("Lschen");     
      buttonLoeschen.addSelectionListener(new SelectionAdapter()
  		{
  			@Override
  			public void widgetSelected(SelectionEvent e)
  			{
  				aktionLoeschen();
  			}			
  		});
    }
    
    /**
     * This method initializes gruppeBeschreibung
     * 
     */
    private void createGruppeBeschreibung()
    {
      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;
      _gruppeBeschreibung = new Group(composite, SWT.NONE);
      _gruppeBeschreibung.setLayout(new FillLayout());
      _gruppeBeschreibung.setLayoutData(gridData2);
      _gruppeBeschreibung.setText("Beschreibung");
      _textAreaBeschreibung = new Text(_gruppeBeschreibung, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
      _textAreaBeschreibung.setEditable(false);
    }

  

}  //  @jve:decl-index=0:visual-constraint="10,10,306,259"
