package de.bsvrz.buv.plugin.sim;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Kapselt die Daten, die hinter einer Simulation stehen.
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/02/02 15:36:55 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class Simulation implements Cloneable
{
  /**
   * {@link SimulationssteuerungDaten} Instanz zu dieser Klasse.
   */
  private SimulationssteuerungDaten _simulationssteuerungDaten = null;

  /**
   * Pid der Simulation
   */
  private String _pid = null;

  /**
   * Name der Simulation
   */
  private String _name = null;

  /**
   * Name der Simulation
   */
  private String _variante = null;

  /**
   * Enumerator fr die verschiedenen Simulationsarten
   * 
   * @author HofmannJ
   */
  public enum SimulationsArt {
    /**
     * Online Simulation
     */
    Online,

    /**
     * Offline Simulation
     */
    Offline
  }

  /**
   * Legt die gewhlte Simulationsart fest.
   */
  private SimulationsArt _gewaehlteSimulationsArt = null;

  /**
   * Simulationsstrecke dieser Simulation
   */
  private SystemObject _simulationsStrecke = null;

  /**
   * Simulationsendzeitpunkt in Millisekunden seit dem 1.1.1970
   */
  private long _simulationsende = 0;

  /**
   * Simulationsstartzeitpunkt in Millisekunden seit dem 1.1.1970
   */
  private long _simulationsstart = 0;

  /**
   * Konstruktor, setzt den Namen der Simulation.
   * 
   * @param name
   *          Name der Simulation.
   */
  public Simulation(String name)
  {
    this._name = name;
    _simulationssteuerungDaten = new SimulationssteuerungDaten(this);
  }

  /**
   * Liefert die Instanz der Klasse {@link SimulationssteuerungDaten}, zugehrig zu dieser Simulatio.
   * 
   * @return Zugehrige Instanz der Klasse {@link SimulationssteuerungDaten}.
   */
  public SimulationssteuerungDaten getSimulationssteuerungDaten()
  {
    return this._simulationssteuerungDaten;
  }

  /**
   * Liefert die Pid der Simulation.
   * 
   * @return Pid der Simulation.
   */
  public String getPid()
  {
    return _pid;
  }

  /**
   * Setzt die Pid der Simulation.
   * 
   * @param pid
   *          Pid der Simulation.
   */
  public void setPid(String pid)
  {
    this._pid = pid;
  }

  /**
   * Liefert den Namen der Simulation.
   * 
   * @return Name der Simulation.
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Setzt den Namen der Simulation.
   * 
   * @param name
   *          Name der Simulation.
   */
  public void setName(String name)
  {
    this._name = name;
  }

  /**
   * Liefert die Simulationsart als Enum <code>SimulationsArt</code>.
   * 
   * @return Simulationsart
   */
  public SimulationsArt getGewaehlteSimulationsArt()
  {
    return _gewaehlteSimulationsArt;
  }

  /**
   * Setzt die Simulationsart als Enum <code>SimulationsArt</code>.
   * 
   * @param simulationsArt
   *          Simulationsart.
   */
  public void setGewaehlteSimulationsArt(SimulationsArt simulationsArt)
  {
    _gewaehlteSimulationsArt = simulationsArt;
  }

  /**
   * Liefert die Simulationsstrecke.
   * 
   * @return Simulationsstrecke
   */
  public SystemObject getSimulationsStrecke()
  {
    return _simulationsStrecke;
  }

  /**
   * Setzt die Simulationsstrecke.
   * 
   * @param simulationsStrecke
   *          Simulationsstrecke
   */
  public void setSimulationsStrecke(SystemObject simulationsStrecke)
  {
    this._simulationsStrecke = simulationsStrecke;
  }

  /**
   * Liefert den Endzeitpunkt der Simulation in Millisekunden seit 1970, wenn es sich um eine OfflineSimulation handelt,
   * sonst -1.
   * 
   * @return Endzeitpunkt der Simulation in Millisekunden seit 1970.
   */
  public long getSimulationsende()
  {
    if (_gewaehlteSimulationsArt.equals(SimulationsArt.Offline))
    {
      return _simulationsende;
    }
    else
    {
      return -1;
    }
  }

  /**
   * Setzt den Endzeitpunkt der Simulation in Millisekunden seit 1970, wenn es sich um eine OfflineSimulation handelt.
   * 
   * @param _simulationsende
   *          Endzeitpunkt der Simulation in Millisekunden seit 1970.
   */
  public void setSimulationsende(long _simulationsende)
  {
    if (_gewaehlteSimulationsArt.equals(SimulationsArt.Offline))
    {
      this._simulationsende = _simulationsende;
    }
  }

  /**
   * Liefert den Startzeitpunkt der Simulation in Millisekunden seit 1970, wenn es sich um eine OfflineSimulation
   * handelt, sonst -1.
   * 
   * @return Startzeitpunkt der Simulation in Millisekunden seit 1970.
   */
  public long getSimulationsstart()
  {
    if (_gewaehlteSimulationsArt.equals(SimulationsArt.Offline))
    {
      return _simulationsstart;
    }
    else
    {
      return -1;
    }
  }

  /**
   * Setzt den Startzeitpunkt der Simulation in Millisekunden seit 1970, wenn es sich um eine OfflineSimulation handelt.
   * 
   * @param _simulationsstart
   *          Startzeitpunkt der Simulation in Millisekunden seit 1970.
   */
  public void setSimulationsstart(long _simulationsstart)
  {
    this._simulationsstart = _simulationsstart;
  }

  /**
   * Liefert einen Clone dieser Klasse.
   * 
   * @return Clone dieser Klasse.
   */
  public Simulation clone()
  {
    Simulation simulationDatensatz = new Simulation("Kopie von " + getName());
    simulationDatensatz.setPid(getPid());
    simulationDatensatz.setGewaehlteSimulationsArt(getGewaehlteSimulationsArt());
    simulationDatensatz.setSimulationsStrecke(getSimulationsStrecke());
    simulationDatensatz.setVariante(getVariante());
    if (getGewaehlteSimulationsArt().equals(SimulationsArt.Offline))
    {
      simulationDatensatz.setSimulationsstart(getSimulationsstart());
      simulationDatensatz.setSimulationsende(getSimulationsende());
    }

    return simulationDatensatz;
  }

  public String getVariante()
  {
    return _variante;
  }

  public void setVariante(String variante)
  {
    _variante = variante;
  }

}
