package de.bsvrz.buv.plugin.sim;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.sim.SimulationssteuerungDaten.SimulationsZustand;
import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoerer;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * In dieser Sicht sind die aktuell Simulationen darzustellen. Fr jeden Eintrag ist die Eigenschaft Name darzustellen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-9</li>
 * </ul>
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:03:57 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class VerwaltungsSicht extends ViewPart implements ISimulationListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Verbindung zum DaV
   */
  private DaVKommunikation _daVKommunikation = null; // @jve:decl-index=0:

  /**
   * TableViewer
   */
  private static TableViewer _tvSimualtion = null;

  /**
   * Gibt die Sortierreihenfolge der Spalten an.
   */
  private static int[] _tvSpaltenSortierung = null;

  /**
   * Speichert die SystemkalenderDatenstze fr die tabellarische Anzeige.
   */
  private java.util.List<Simulation> _simulationDatensaetze = null; // @jve:decl-index=0:

  private Composite composite = null; // @jve:decl-index=0:visual-constraint="144,147"

  private Composite compositeTop = null;

  private Composite compositeBottom = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private Composite compositeRight = null; // @jve:decl-index=0:visual-constraint="693,0"

  private Composite compositeRight1 = null;

  private Composite compositeTable = null;

  private Button buttonNeu = null;

  private Button buttonKopieren = null;

  private Button buttonLoeschen = null;

  private Composite compositeRight2 = null;

  private Button buttonStart = null;

  private Button buttonStop = null;

  private Simulation _eintrag = null;

  /**
   * Der Listener fr die Berechtigungen
   */
  private IOberflaechenBerechtigungZuhoerer _berechtigungsZuhoerer = null;

  /**
   * Die GUI-Elemente fr das Berechtigungskonzept
   */
  private Map<String, Object> _oberflaechenElemente = new HashMap<String, Object>();
  
  /**
   * Instanz der Kommandozeilenparameter
   */
  private Kommandozeilenparameter _kommandoZeilenParameter = Kommandozeilenparameter.getInstanz();

  private Map<String, SimulationSteuerungsdialog> _mapDialoge = new HashMap<String, SimulationSteuerungsdialog>();

  @Override
  public void createPartControl(Composite parent)
  {
    composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    createCompositeTop();
    createCompositeTable();
    createCompositeRight();
    createCompositeBot();

    buttonKopieren.setEnabled(false);
    buttonLoeschen.setEnabled(false);
    buttonStart.setEnabled(false);
    buttonStop.setEnabled(false);

    _daVKommunikation = DaVKommunikation.getInstanz();

    _daVKommunikation.hinzufuegenSimulationListener(this);

    KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
    if (kp.getWert().equals("true")){
      erzeugeOberflaechenBerechtigung();
    }

    _daVKommunikation.anmelden();

    fuelleEreignisKalenderEintraege();

  }

  @Override
  public void setFocus()
  {
  }

  /**
   * Definiert einen neue Simulation in der Liste von Simulationen.
   * 
   */
  private void aktionNeu()
  {
    SimulationDefinitionsdialog simualtionDefinitionsdialog = new SimulationDefinitionsdialog(PlatformUI.getWorkbench()
        .getDisplay().getActiveShell());
    _eintrag = simualtionDefinitionsdialog.oeffne();
    
  }

  /**
   * Kopiert eine Simulation in der Liste von Simulationen. Es ffnet sich der Definitionsdialg ({@link SimulationDefinitionsdialog})
   * zum Bearbeiten der Simulation.
   * 
   */
  private void aktionKopieren()
  {

    SimulationDefinitionsdialog simualtionDefinitionsdialog = new SimulationDefinitionsdialog(PlatformUI.getWorkbench()
        .getDisplay().getActiveShell());

    Simulation clone = _eintrag.clone();

    _eintrag = simualtionDefinitionsdialog.oeffne(clone);

  }

  /**
   * Lscht einen selektierten Eintrag nach vorhergehender "Sind sie sicher?"-Abfrage.
   * 
   */
  private void aktionLoeschen()
  {

    if (_eintrag != null)
    {

      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL);
      mb.setText("Simulation lschen");
      mb.setMessage("Wollen Sie die Simulation: " + _eintrag.getName() + " wirklich lschen?");

      if (mb.open() == SWT.OK)
      {

        if (!_eintrag.getGewaehlteSimulationsArt().equals(SimulationssteuerungDaten.SimulationsZustand.gelscht))
        {
                    
          DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

          AttributeGroup atg = null;

          if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
          {
            atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOnline");
          }
          else if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
          {
            atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOffline");
          }

          Aspect asp = dataModel.getAspect("asp.vorgabe");
          Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

          data.getItem("SimulationsZustand").asTextValue().setText("gelscht");
          if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
          {
            data.getItem("Simulationsgeschwindigkeit").asTextValue().setText("1");
          }

          Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
          sender.sende(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()), atg, asp,
              data, SenderRole.sender());

        }

      }

    }
    else
    {

      MessageBox mb1 = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb1.setText("Fehler");
      mb1.setMessage("Es muss ein Eintrag selektiert werden!");
      mb1.open();

    }
  }

  /**
   * Startet eine Simulation
   */
  private void aktionStart()
  {
    SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();

    if (_eintrag != null)
    {

      DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

      AttributeGroup atg = null;

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOnline");
      }
      else if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOffline");

      }

      Aspect asp = dataModel.getAspect("asp.vorgabe");
      Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

      // SystemObject so = (SystemObject)_daVKommunikation.holeDaVVerbindung().getLocalApplicationObject();

      data.getItem("SimulationsZustand").asTextValue().setText("Vorstart");
      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        data.getItem("Simulationsgeschwindigkeit").asTextValue().setText("1");
      }

      if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.neu)
          || zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Stop))
      {
        Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
        sender.sende(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()), atg, asp, data,
            SenderRole.sender());
      }

      
      if (!_mapDialoge.containsKey(_eintrag.getPid())){
      
        SimulationSteuerungsdialog simulationSteuerungsdialog = new SimulationSteuerungsdialog(PlatformUI.getWorkbench()
            .getDisplay().getActiveShell());
        
        _mapDialoge.put(_eintrag.getPid(), simulationSteuerungsdialog);
        
        Simulation eintrag = simulationSteuerungsdialog.oeffne(_eintrag);
        
        if (eintrag != null){
          _mapDialoge.remove(eintrag.getPid());
        }
        
      }

    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL);
      mb.setText("Fehler Selektion");
      mb.setMessage("Es muss ein Eintrag selektiert werden!");
      mb.open();
    }

  }

  /**
   * Stopt eine Simulation
   */
  private void aktionStop()
  {
    SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();

    if (_eintrag != null)
    {

      DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

      AttributeGroup atg = null;

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOnline");
      }
      else if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOffline");

      }

      Aspect asp = dataModel.getAspect("asp.vorgabe");
      Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

      SystemObject so = (SystemObject)_daVKommunikation.holeDaVVerbindung().getLocalApplicationObject();

      data.getItem("SimulationsZustand").asTextValue().setText("Stop");
      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        data.getItem("Simulationsgeschwindigkeit").asTextValue().setText("1");
      }

      Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
      sender.sende(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()), atg, asp, data,
          SenderRole.sender());

    }

  }

  /**
   * Schliet den Verwaltungsdialog {@link VerwaltungsSicht}.
   * 
   */
  private void aktionSchliessen()
  {
    getViewSite().getPage().hideView(VerwaltungsSicht.this);
  }

  /**
   * Zeigt die Hilfe zum Verwaltungsdialog {@link VerwaltungsSicht} an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Aktivator.PLUGIN_ID + ".simulation";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTop()
  {
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.grabExcessVerticalSpace = true;
    gridData5.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    gridLayout.makeColumnsEqualWidth = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(composite, SWT.NONE);
    compositeTop.setLayoutData(gridData3);
    compositeTop.setLayout(gridLayout);

  }

  private void createCompositeTable()
  {
    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.VERTICAL;
    GridData gridData1 = new GridData();
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalAlignment = GridData.FILL;
    compositeTable = new Composite(compositeTop, SWT.NONE);
    compositeTable.setLayoutData(gridData1);
    compositeTable.setLayout(fillLayout);

    erzeugeTabelle();

  }

  /**
   * This method initializes compositeBot
   * 
   */
  private void createCompositeBot()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    rowLayout.fill = false;
    rowLayout.wrap = false;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.END;
    gridData4.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(composite, SWT.NONE);
    compositeBottom.setLayoutData(gridData4);
    compositeBottom.setLayout(rowLayout);
    buttonSchliessen = new Button(compositeBottom, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});

    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}			
		});
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeRight()
  {
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.makeColumnsEqualWidth = false;
    gridLayout1.numColumns = 1;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.CENTER;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = false;
    gridData.verticalAlignment = GridData.FILL;
    compositeRight = new Composite(compositeTop, SWT.NONE);
    compositeRight.setSize(new Point(82, 151));
    compositeRight.setLayoutData(gridData);
    createCompositeRight1();
    compositeRight.setLayout(gridLayout1);
    createCompositeRight2();
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeRight1()
  {
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.verticalAlignment = GridData.CENTER;
    GridData gridData10 = new GridData();
    gridData10.horizontalAlignment = GridData.FILL;
    gridData10.grabExcessHorizontalSpace = true;
    gridData10.verticalAlignment = GridData.CENTER;
    GridData gridData9 = new GridData();
    gridData9.horizontalAlignment = GridData.FILL;
    gridData9.grabExcessHorizontalSpace = true;
    gridData9.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalAlignment = GridData.CENTER;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.FILL;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.grabExcessHorizontalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    compositeRight1 = new Composite(compositeRight, SWT.NONE);
    compositeRight1.setLayoutData(gridData6);
    compositeRight1.setLayout(new GridLayout());
    buttonNeu = new Button(compositeRight1, SWT.NONE);
    buttonNeu.setText("Neu");
    buttonNeu.setLayoutData(gridData9);
    buttonNeu.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionNeu();
			}			
		});

    buttonKopieren = new Button(compositeRight1, SWT.NONE);
    buttonKopieren.setText("Kopieren...");
    buttonKopieren.setLayoutData(gridData10);
    buttonKopieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionKopieren();
			}			
		});

    buttonLoeschen = new Button(compositeRight1, SWT.NONE);
    buttonLoeschen.setText("Lschen");
    buttonLoeschen.setLayoutData(gridData12);
    buttonLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionLoeschen();
			}			
		});
  }

  private void erzeugeTabelle()
  {

    _tvSimualtion = new TableViewer(compositeTable, SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);

    final TableColumn tc0 = new TableColumn(_tvSimualtion.getTable(), SWT.LEFT | SWT.Move);
    tc0.setText("Name");
    tc0.setWidth(150);
    tc0.setMoveable(true);
    tc0.addListener(SWT.Selection, new Listener() {
      private int colPos = 0;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvSimualtion.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });
    final TableColumn tc1 = new TableColumn(_tvSimualtion.getTable(), SWT.LEFT | SWT.Move);
    tc1.setText("Typ");
    tc1.setWidth(100);
    tc1.setMoveable(true);
    tc1.addListener(SWT.Selection, new Listener() {
      private int colPos = 1;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvSimualtion.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc2 = new TableColumn(_tvSimualtion.getTable(), SWT.LEFT | SWT.Move);
    tc2.setText("Zustand");
    tc2.setWidth(100);
    tc2.setMoveable(true);
    tc2.addListener(SWT.Selection, new Listener() {
      private int colPos = 2;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvSimualtion.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    final TableColumn tc3 = new TableColumn(_tvSimualtion.getTable(), SWT.LEFT | SWT.Move);
    tc3.setText("Strecke");
    tc3.setWidth(150);
    tc3.setMoveable(true);
    tc3.addListener(SWT.Selection, new Listener() {
      private int colPos = 3;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvSimualtion.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });
    final TableColumn tc4 = new TableColumn(_tvSimualtion.getTable(), SWT.LEFT | SWT.Move);
    tc4.setText("Variante");
    tc4.setWidth(50);
    tc4.setMoveable(true);
    tc4.addListener(SWT.Selection, new Listener() {
      private int colPos = 4;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvSimualtion.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new NumberComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    _tvSimualtion.getTable().setHeaderVisible(true);
    _tvSimualtion.getTable().setLinesVisible(true);

    _tvSimualtion.setContentProvider(new TabelleInhaltsVersorger());
    _tvSimualtion.setLabelProvider(new TabelleDarstellungsVersorger());

    _tvSimualtion.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {

        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof Simulation)
        {
          _eintrag = (Simulation)firstElement;

          KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
          if (kp.getWert().equals("true")){
          
            ArrayList<SystemObject> liste = new ArrayList<SystemObject>();
  
            liste.add(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()));
  
            pruefeBerechtigung(liste);
          }

          buttonKopieren.setEnabled(true);

          SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();

          if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.neu))
          {
            buttonLoeschen.setEnabled(false);
            buttonStart.setText("Start");
            buttonStart.setEnabled(true);
            buttonStop.setEnabled(false);

          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Vorstart))
          {
            buttonLoeschen.setEnabled(false);
            buttonStart.setText("Start");
            buttonStart.setEnabled(true);
            buttonStop.setEnabled(false);

          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Start))
          {
            buttonLoeschen.setEnabled(false);
            buttonStart.setText("Simulationssteuerung...");
            buttonStart.setEnabled(true);
            buttonStop.setEnabled(true);

          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Pause))
          {
            buttonLoeschen.setEnabled(false);
            buttonStart.setText("Start");
            buttonStart.setEnabled(true);
            buttonStop.setEnabled(true);

          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Stop))
          {
            buttonLoeschen.setEnabled(true);
            buttonStart.setText("Start");
            buttonStart.setEnabled(true);
            buttonStop.setEnabled(false);

          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.gelscht))
          {
            buttonKopieren.setEnabled(false);
            buttonLoeschen.setEnabled(false);
            buttonStart.setText("Start");
            buttonStart.setEnabled(false);
            buttonStop.setEnabled(false);
          }

          compositeTop.layout();
          
          if (_mapDialoge.containsKey(_eintrag.getPid())){
          
            SimulationSteuerungsdialog dialog = _mapDialoge.get(_eintrag.getPid());
            dialog.getShell().setFocus();
          }
            

        }
        else
        {
          _eintrag = null;
          buttonKopieren.setEnabled(false);
          buttonLoeschen.setEnabled(false);
          buttonStart.setEnabled(false);
          buttonStop.setEnabled(false);
        }

      }

    });

  }

  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      Simulation simulation = (Simulation)object;

      switch (columnIndex)
      {
        case 0:
          if (simulation.getName() != null)
            returnString = simulation.getName();
          else
            returnString = "";
          break;
        case 1:
          if (simulation.getGewaehlteSimulationsArt() != null)
            returnString = simulation.getGewaehlteSimulationsArt().toString();
          else
            returnString = "";
          break;
        case 2:
          if (simulation.getSimulationssteuerungDaten().getAktuellerZustand() != null)
            returnString = simulation.getSimulationssteuerungDaten().getAktuellerZustand().toString();
          else
            returnString = "";
          break;
        case 3:
          if (simulation.getSimulationsStrecke() != null)
            returnString = simulation.getSimulationsStrecke().getName();
          else
            returnString = "";
          break;
        case 4:
          if (simulation.getVariante() != null)
            returnString = simulation.getVariante();
          else
            returnString = "";
          break;

        default:
          returnString = "";

      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen MyComperatorArray umkopiert und darauf hin
   * wird dieser mittels der Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die ursprngliche
   * Position in der Tabelle gemerkt hat, kann man nun ganz einfach die entsprechenden Werte im Original Array tauschen
   * und schon ist man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems, int colPos, BaseComapator[][] comparables,
      boolean sortFlag)
  {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
    String[] textBuffer = null;
    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);
    }

  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */

  class BaseComapator implements Comparable
  {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos)
    {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o)
    {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der einzelnen Tabellenzeilen vornimmt...
   */

  class StringComparator extends BaseComapator implements Comparable
  {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer :
     *          Der Text der Zeile
     * @param rowNum :
     *          Die Zeilennr der Tabellenzeile
     * @param sortFlag :
     *          Aufsteigend false, Absteigend true
     * @param colPos :
     *          Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
    }

    public final int compareTo(Object arg0)
    {
      int ret = _textBuffer[_colPos].compareTo(((StringComparator)arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * NumberComparator kmmert sich um das Vergleichen von Numerischen Werten...
   * 
   */

  class NumberComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public NumberComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        _longValue = Long.parseLong(textBuffer[colPos]);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((NumberComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * TimeComparator kmmert sich um das Vergleichen von Datumsangaben...
   * 
   */

  class TimeComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public TimeComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
        Date d = sdf.parse(textBuffer[colPos]);

        _longValue = d.getTime();

        // _debug.fine(textBuffer[colPos] + " : " + _longValue);

      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((TimeComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  @Override
  public void simulationEingang(SimulationEreignis e)
  {
    _simulationDatensaetze = e.getSimulationen();

    for (Simulation s : _simulationDatensaetze)
    {
      if (_eintrag != null && s.getPid().equals(_eintrag.getPid()))
      {

        SimulationsZustand zustand = s.getSimulationssteuerungDaten().getAktuellerZustand();

        if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.gelscht))
        {

          DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

          AttributeGroup atg = dataModel.getAttributeGroup("atg.simulationVerwaltungAnfrage");
          Aspect asp = dataModel.getAspect("asp.anfrage");
          Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

          SystemObject so = (SystemObject)_daVKommunikation.holeDaVVerbindung().getLocalApplicationObject();
          data.getItem("AbsenderId").asReferenceValue().setSystemObject(so);
          data.getItem("AbsenderZeichen").asTextValue().setText(PlatformUI.PLUGIN_ID);

          Array array = data.getItem("LoescheSimulationAnfrage").asArray();
          array.setLength(1);
          SystemObject so1 = dataModel.getObject(_eintrag.getPid());
          array.getItem(0).getItem("ReferenzZuLoeschendeSimulation").asReferenceValue().setSystemObject(so1);
          Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
          sender.sende(_daVKommunikation.holeDaVVerbindung().getLocalConfigurationAuthority(), atg, asp, data,
              SenderRole.sender());

        }

        break;
      }
    }

    fuelleEreignisKalenderEintraege();

  }

  /**
   * Fllt die Liste der Betriebsmeldungen des Dialogs.
   */
  private void fuelleEreignisKalenderEintraege()
  {

    _tvSimualtion.setInput(_simulationDatensaetze);

  }

  /**
   * This method initializes compositeRight2
   * 
   */
  private void createCompositeRight2()
  {
    GridData gridData8 = new GridData();
    gridData8.horizontalAlignment = GridData.FILL;
    gridData8.grabExcessHorizontalSpace = true;
    gridData8.verticalAlignment = GridData.CENTER;
    GridData gridData7 = new GridData();
    gridData7.horizontalAlignment = GridData.FILL;
    gridData7.grabExcessHorizontalSpace = true;
    gridData7.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.makeColumnsEqualWidth = false;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    compositeRight2 = new Composite(compositeRight, SWT.NONE);
    compositeRight2.setLayoutData(gridData2);
    compositeRight2.setLayout(gridLayout2);
    buttonStart = new Button(compositeRight2, SWT.NONE);
    buttonStart.setText("Start");
    buttonStart.setLayoutData(gridData7);
    buttonStart.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionStart();
			}			
		});
    
    buttonStop = new Button(compositeRight2, SWT.NONE);
    buttonStop.setText("Stop");
    buttonStop.setLayoutData(gridData8);
    buttonStop.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionStop();
			}			
		});

  }

  private void erzeugeOberflaechenBerechtigung()
  {
    _berechtigungsZuhoerer = new IOberflaechenBerechtigungZuhoerer() {
      @Override
      public void freigabe(OberflaechenBerechtigungEreignis e)
      {
        _debug.fine("Info Freigabe erhalten: " + e.hatFreigabe());

        List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
        List<SystemObject> liste2 = e.getSystemObjekteSperrung();
        for (SystemObject so : liste1)
        {
          _debug.fine("Objekte Freigabe: " + so);
        }
        for (SystemObject so : liste2)
        {
          _debug.fine("Objekte Sperrung: " + so);
        }

        List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
        List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
        for (OberflaechenFunktion of : liste3)
        {
          _debug.fine("Funktion Freigabe: " + of.getFunktionsName());
          if (_oberflaechenElemente.containsKey(of.getFunktionsName()))
          {
            Object object = _oberflaechenElemente.get(of.getFunktionsName());
            if (object instanceof Button)
            {
              Button button = (Button)object;
              button.setEnabled(true);
              
            }
          }

        }
        for (OberflaechenFunktion of : liste4)
        {
          _debug.fine("Funktion Sperrung: " + of.getFunktionsName());
        }

      }

      @Override
      public void sperrung(OberflaechenBerechtigungEreignis e)
      {
        _debug.fine("Info Sperrung erhalten: " + e.hatFreigabe());

        List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
        List<SystemObject> liste2 = e.getSystemObjekteSperrung();
        for (SystemObject so : liste1)
        {
          _debug.fine("Objekte Freigabe: " + so);
        }
        for (SystemObject so : liste2)
        {
          _debug.fine("Objekte Sperrung: " + so);
        }

        List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
        List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
        for (OberflaechenFunktion of : liste3)
        {
          _debug.fine("Funktion Freigabe: " + of.getFunktionsName());
        }
        for (OberflaechenFunktion of : liste4)
        {
          _debug.fine("Funktion Sperrung: " + of.getFunktionsName());
          if (_oberflaechenElemente.containsKey(of.getFunktionsName()))
          {
            Object object = _oberflaechenElemente.get(of.getFunktionsName());
            if (object instanceof Button)
            {
              Button button = (Button)object;
              button.setEnabled(false);              
            }
          }

        }

      }
    };

    de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz().hinzufuegenOberflaechenBenutzerZuhoerer(
        _berechtigungsZuhoerer);

    OberflaechenFunktion funktion1 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht.schaltflaeche.neu", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht", "anlegen");
    OberflaechenFunktion funktion2 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht.schaltflaeche.kopieren", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht", "kopieren");
    OberflaechenFunktion funktion3 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht.schaltflaeche.loeschen", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht", "loeschen");
    OberflaechenFunktion funktion4 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht.schaltflaeche.start", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht", "simulationstart");
    OberflaechenFunktion funktion5 = new OberflaechenFunktion(
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht.schaltflaeche.stop", _berechtigungsZuhoerer,
        "de.bsvrz.buv.plugin.sim.VerwaltungsSicht", "simulationstop");

    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion1);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion2);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion3);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion4);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion5);

    _oberflaechenElemente.put(funktion1.getFunktionsName(), buttonNeu);
    _oberflaechenElemente.put(funktion2.getFunktionsName(), buttonKopieren);
    _oberflaechenElemente.put(funktion3.getFunktionsName(), buttonLoeschen);
    _oberflaechenElemente.put(funktion4.getFunktionsName(), buttonStart);
    _oberflaechenElemente.put(funktion5.getFunktionsName(), buttonStop);

  }

  @Override
  public void dispose()
  {
    _daVKommunikation.entferneSimulationListener(this);
    KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
    if (kp.getWert().equals("true")){
      de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
          _berechtigungsZuhoerer);
    }
    super.dispose();
  }
  
  /**
   * Prft alle SystemObjekte der Liste ob eine Freigabe oder Sperrung vorliegt 
   * @param liste
   */
  private void pruefeBerechtigung(List<SystemObject> liste){
    for (Map.Entry<String, Object> me : _oberflaechenElemente.entrySet())
    {
      OberflaechenBerechtigung.getInstanz().pruefeBerechtigungElement(
          liste, me.getKey());
      
    }
  }

} // @jve:decl-index=0:visual-constraint="10,10,306,259"
