/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import static org.junit.Assert.assertEquals;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Random;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * @author HofmannJ
 * 
 */
public class KalenderZeitFilterDatenStrukturTest
{

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStruktur#KalenderZeitFilterDatenStruktur(java.lang.String)}.
	 */
	@Test
	public void testKalenderZeitFilterDatenStrukturString()
	{
		int[] alleMonate = new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };

		/*
		 * TestString 1:
		 */
		boolean testString1Korrekt = true;
		KalenderZeitFilterDatenStruktur struktur1 = new KalenderZeitFilterDatenStruktur(
				"* 2,4,5 Di,Do,Sa 00:00:0.0-00:00:12.0,00:13:0.0-00:44:0.0");

		// Monate
		int[] selektierteMonate = struktur1.getSelektierteMonate();
		if (!istArrayGleich(alleMonate, selektierteMonate))
		{
			testString1Korrekt = false;
		}

		// Tage
		if (!istArrayGleich(new int[] { 2 - 1, 4 - 1, 5 - 1 }, struktur1
				.getSelektierteTage()))
		{
			testString1Korrekt = false;
		}

		// Wochentage
		if (!istArrayGleich(new int[] { 1, 3, 5 }, struktur1
				.getSelektierteWochentage()))
		{
			testString1Korrekt = false;
		}

		// Zeitgruppen
		List<KalenderZeitFilterDatenStrukturZeitGruppe> listeZeitGruppen = struktur1
				.getListeZeitGruppen();
		if (listeZeitGruppen.size() == 2)
		{
			// 1. Zeitgruppe
			KalenderZeitFilterDatenStrukturZeitGruppe gruppe0 = listeZeitGruppen
					.get(0);
			if (!((gruppe0.getStartzeitpunktAlsString().equals("00:00:0.0")) && (gruppe0
					.getEndzeitpunktAlsString().equals("00:00:12.0"))))
			{
				testString1Korrekt = false;
			}

			// 2. Zeitgruppe
			KalenderZeitFilterDatenStrukturZeitGruppe gruppe1 = listeZeitGruppen
					.get(1);
			if (!((gruppe1.getStartzeitpunktAlsString().equals("00:13:0.0")) && (gruppe1
					.getEndzeitpunktAlsString().equals("00:44:0.0"))))
			{
				testString1Korrekt = false;
			}

		}

		/*
		 * TestString2 - Zahlen bei den Monaten, * bei Wochentagen und Stunden, *
		 * bei Zeitgruppen
		 */
		boolean testString2Korrekt = true;
		KalenderZeitFilterDatenStruktur struktur2 = new KalenderZeitFilterDatenStruktur(
				"1,4,12 * * *");

		// Monate TestString2
		int[] selektierteMonate2 = struktur2.getSelektierteMonate();
		if (!istArrayGleich(new int[] { 0, 3, 11 }, selektierteMonate2))
		{
			testString2Korrekt = false;
		}

		// Tage TestString2
		int[] erwartungSelektierteTage2 = new int[31];
		for (int i = 0; i < erwartungSelektierteTage2.length; i++)
		{
			erwartungSelektierteTage2[i] = i;
		}

		int[] selektierteTage2Ist = struktur2.getSelektierteTage();
		if (!istArrayGleich(erwartungSelektierteTage2, selektierteTage2Ist))
		{
			testString2Korrekt = false;
		}

		// Wochentage TestString2
		int[] erwartungWochentage2 = new int[7];
		for (int i = 0; i < erwartungWochentage2.length; i++)
		{
			erwartungWochentage2[i] = i;
		}

		int[] selektierteWochentage = struktur2.getSelektierteWochentage();
		if (!istArrayGleich(erwartungWochentage2, selektierteWochentage))
		{
			testString2Korrekt = false;
		}

		// Zeitgruppen TestString2
		List<KalenderZeitFilterDatenStrukturZeitGruppe> listeZeitGruppen2 = struktur2
				.getListeZeitGruppen();
		if (listeZeitGruppen2.size() == 1)
		{
			KalenderZeitFilterDatenStrukturZeitGruppe zeitGruppe2 = listeZeitGruppen2
					.get(0);

			if (!((zeitGruppe2.getStartzeitpunktAlsString().equals("00:00:0.0")) && (zeitGruppe2
					.getEndzeitpunktAlsString().equals("23:59:59.999"))))
			{
				testString2Korrekt = false;
			}

		} else
		{
			testString2Korrekt = false;
		}

		assertEquals(true, testString1Korrekt & testString2Korrekt);
	}

	/**
	 * Hilfsmethode, prft ob die Inhalte zweier int-Arrays gleich sind.
	 * 
	 * @param erwartungsWert
	 *          Erwartungswert, der eintreten muss.
	 * @param istWert
	 *          Ist-Wert, der gegen den Erwartungswert geprft wird.
	 * @return Liefert <code>true</code>, wenn die Arrays vom Inhalt her gleich
	 *         sind, sonst <code>false</code>.
	 */
	private boolean istArrayGleich(int[] erwartungsWert, int[] istWert)
	{
		boolean re = true;

		if (erwartungsWert.length == istWert.length)
		{
			for (int i = 0; i < erwartungsWert.length; i++)
			{
				int wertA = erwartungsWert[i];
				int wertB = istWert[i];

				if (wertA != wertB)
				{
					re = false;
					break;
				}
			}
		} else
		{
			re = false;
		}

		return re;
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStruktur#erzeugeFilterString()}.
	 */
	@Test
	public void testErzeugeFilterString()
	{
		/*
		 * Struktur 1
		 */
		KalenderZeitFilterDatenStruktur struktur1 = new KalenderZeitFilterDatenStruktur();

		// Struktur 1 - Alle Monate selektieren
		for (int i = 0; i <= 11; i++)
		{
			struktur1.selektiereMonat(i);
		}

		// Struktur 1 - Alle Tage selektieren
		for (int i = 0; i <= 30; i++)
		{
			struktur1.selektiereTag(i);
		}

		// Struktur 1 - Alle Wochentage selektieren
		for (int i = 0; i <= 6; i++)
		{
			struktur1.selektiereWochentag(i);
		}

		boolean testStruktur1A = true;
		boolean testStruktur1B = true;

		// System.out.println("Struktur1: \"" + struktur1.erzeugeFilterString() +
		// "\"");
		if (!struktur1.erzeugeFilterString().equals("* * * *"))
		{
			testStruktur1A = false;
		}

		try
		{
			KalenderZeitFilterDatenStrukturZeitGruppe zg = new KalenderZeitFilterDatenStrukturZeitGruppe(
					"0:0:0.0", "23:59:59.999");
			struktur1.fuegeZeitGruppeHinzu(zg);

		} catch (ParseException e)
		{
			e.printStackTrace();
		}

		// System.out.println("Struktur1: \"" + struktur1.erzeugeFilterString() +
		// "\"");
		if (!struktur1.erzeugeFilterString().equals("* * * 00:00:0.0-23:59:59.999"))
		{
			testStruktur1B = false;
		}

		/*
		 * Monate
		 */
		boolean testMonate = true;
		KalenderZeitFilterDatenStruktur strukturMonate = new KalenderZeitFilterDatenStruktur();

		if (!strukturMonate.erzeugeFilterString().equals("* * * *"))
		{
			testMonate = false;
		}

		String mon = "";
		for (int i = 0; i <= 11; i++)
		{
			/*
			 * Prfstring Aufbauen
			 */
			if (i > 0)
			{
				mon = mon + ",";
			}

			mon = mon + (i + 1);

			String pruef = mon + " * * *";

			/*
			 * Monate selektieren
			 */
			strukturMonate.selektiereMonat(i);

			/*
			 * Prfung durchfhren
			 */
			if (i < 11)
			{
				if (!pruef.equals(strukturMonate.erzeugeFilterString()))
				{
					testMonate = false;
				}
			}
			if (i == 11)
			{
				if (!"* * * *".equals(strukturMonate.erzeugeFilterString()))
				{
					testMonate = false;
				}
			}
		}

		/*
		 * Tage
		 */
		boolean testTage = true;

		KalenderZeitFilterDatenStruktur strukturTage = new KalenderZeitFilterDatenStruktur();

		if (!strukturTage.erzeugeFilterString().equals("* * * *"))
		{
			testTage = false;
		}

		String tagVorne = "* ";
		String tagMitte = "";
		String tagHinten = " * *";

		for (int i = 0; i <= 30; i++)
		{
			String tagPruef = "";

			/*
			 * Prfstring Aufbauen
			 */
			if (i > 0)
			{
				tagMitte = tagMitte + ",";
			}
			tagMitte = tagMitte + (i + 1);

			tagPruef = tagVorne + tagMitte + tagHinten;
			// System.out.println("Prfstring Tage: " + tagPruef);

			/*
			 * Tage selektieren
			 */
			strukturTage.selektiereTag(i);

			/*
			 * Prfung durchfhren
			 */
			if (i < 30)
			{
				if (!tagPruef.equals(strukturTage.erzeugeFilterString()))
				{
					testTage = false;
				}
			} else if (i == 30)
			{
				if (!"* * * *".equals(strukturTage.erzeugeFilterString()))
				{
					testTage = false;
				}
			}
		}

		/*
		 * Wochentage
		 */
		boolean testWochentage = true;
		KalenderZeitFilterDatenStruktur strukturWochentage = new KalenderZeitFilterDatenStruktur();

		String wochenTagVorne = "* * ";
		String wochenTagHinten = " *";

		for (int i = 0; i <= 6; i++)
		{

			String wochenTagMitte = erzeugeWochentagString(i);
			String wochenTagPruef = wochenTagVorne + wochenTagMitte + wochenTagHinten;

			strukturWochentage.selektiereWochentag(i);

			if (i < 6)
			{
				if (!wochenTagPruef.equals(strukturWochentage.erzeugeFilterString()))
				{
					testWochentage = false;
				}
			} else if (i == 6)
			{
				if (!"* * * *".equals(strukturWochentage.erzeugeFilterString()))
				{
					testWochentage = false;
				}
			}

			// System.out.println("Prfstring Wochentage: " + wochenTagPruef);
		}

		/*
		 * Zeitdauer
		 */
		boolean testDauer = true;
		DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		GregorianCalendar gcStart = new GregorianCalendar(1970, 0, 1, 0, 0, 0);
		gcStart.set(Calendar.MILLISECOND, 0);
		long start = gcStart.getTimeInMillis();

		GregorianCalendar gcEnde = new GregorianCalendar(1970, 0, 1, 23, 59, 59);
		gcEnde.set(Calendar.MILLISECOND, 999);
		long ende = gcEnde.getTimeInMillis();

		// System.out.println("Start: " + start + " Ende: " + ende);

		Random r = new Random();
		for (long von = start; von <= ende; von += 12000000)
		{
			/*
			 * Millisekunden beim Von-Wert zufllig setzen
			 */
			int numRanI = 1 + Math.abs(r.nextInt()) % 999;
			if (von + numRanI <= ende)
			{
				von = von + numRanI;
			}

			if (testDauer)
			{
				for (long bis = start + 1; bis <= ende; bis += 6000)
				{
					/*
					 * Millisekunden beim Bis-Wert zufllig setzen 
					 */
					int numRanJ = 1 + Math.abs(r.nextInt()) % 999;

					if (bis + numRanJ <= ende)
					{
						bis = bis + numRanJ;
					}

					String pruefString = "* * * " + df.format(new Date(von)) + "-"
							+ df.format(new Date(bis));

					KalenderZeitFilterDatenStruktur strukturDauer = new KalenderZeitFilterDatenStruktur();
					strukturDauer.fuegeZeitGruppeHinzu(von, bis);

					// System.out.println("Prf: " + pruefString);
					// System.out.println("Ist: " + strukturDauer.erzeugeFilterString());

					if (!pruefString.equals(strukturDauer.erzeugeFilterString()))
					{
						System.out.println("Dauer Soll-Zustand: " + pruefString);
						System.out.println("Dauer Ist-Zustand : "
								+ strukturDauer.erzeugeFilterString());

						testDauer = false;
						break;
					}
				}
			} else
			{
				break;
			}
		}

		if (!testStruktur1A || !testStruktur1B || !testMonate || !testTage
				|| !testWochentage || !testDauer)
		{
			String msg = "";
			msg = msg + "===testErzeugeFilterString()===\n";			
			msg = msg + "testStruktur1A: " + testStruktur1A + "\n";
			msg = msg + "testStruktur1B: " + testStruktur1B + "\n";
			msg = msg + "testMonate:     " + testMonate + "\n";
			msg = msg + "testTage:       " + testTage + "\n";
			msg = msg + "testWochentage: " + testWochentage + "\n";
			msg = msg + "testDauer:      " + testDauer + "\n";
			msg = msg + "===============================\n";
			System.err.println(msg);
		}

		assertEquals(true, testStruktur1A && testStruktur1B && testMonate
				&& testTage && testWochentage && testDauer);
	}

	/**
	 * Erzeugt einen Wochentags String von Montag, dem 0-ten Tag bis Tag i.
	 * 
	 * @param bis
	 *          Tag, bist zu dem der Wochentagsstring erzeugt werden soll. Bsp: 3
	 *          entspricht <code>Mo,Di,Mi,Do</code>
	 * @return Liefert den Wochentagsstring oder <code>null</code>, wenn kein
	 *         Wochentagsstring erzeugt werden konnte.
	 */
	private String erzeugeWochentagString(int bis)
	{
		String[] wochentage = new String[] { "Mo", "Di", "Mi", "Do", "Fr", "Sa",
				"So" };

		String re = null;

		if (bis <= 6)
		{
			re = "";

			for (int i = 0; i <= bis; i++)
			{
				re = re + wochentage[i];

				if (i < bis)
				{
					re = re + ",";
				}
			}
		}

		return re;
	}

}
