/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.Iterator;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class EinstellungenSpeicherTest
{
	private EinstellungenSpeicher _esp = null;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	@Before
	public void setUp() throws Exception
	{
		_esp = new EinstellungenSpeicher();

		_esp.hinzufuegenParameter("par.1", "Erster Testparameter");
		_esp.hinzufuegenParameter("par.2", "Zweiter Testparameter");
		_esp.hinzufuegenParameter("par.3", "Dritter Testparameter");
	}

	private void init()
	{
		_esp = new EinstellungenSpeicher();

		_esp.hinzufuegenParameter("par.1", "Erster Testparameter");
		_esp.hinzufuegenParameter("par.2", "Zweiter Testparameter");
		_esp.hinzufuegenParameter("par.3", "Dritter Testparameter");
	}

	@After
	public void tearDown() throws Exception
	{
	}

	@Test
	public void testSchreibeString()
	{
		/*
		 * Pfad zur Datei
		 */
		String dateiPfad = "C:\\test.txt";

		/*
		 * Es wird am Ende ein Zeilenumbruch eingefgt. Dem wird hier Rechnung
		 * getragen.
		 */
		String schreiben = "Dieser String wird in eine Datei geschrieben und muss wieder ausgelesen werden.\n";

		/*
		 * Zu schreibende Datei
		 */
		_esp.schreibeString(dateiPfad, schreiben, true);

		/*
		 * Einlesen der Datei
		 */
		String einlesenXmlAusDatei = _esp.einlesenXmlAusDatei(dateiPfad);

		init();

		assertEquals(schreiben, einlesenXmlAusDatei);
	}

	@Test
	public void testHinzufuegenParameter()
	{
		String parameterId = "par.4";
		String wert = "Hinzugefgter Parameter 4";

		_esp.hinzufuegenParameter(parameterId, wert);

		String parameter = _esp.getParameter(parameterId);
		if (parameter != null)
		{
			assertEquals(wert, parameter);
		} else
		{
			fail("Es wurde kein Parameter geliefert!");
		}

	}

	@Test
	public void testEditiereParameter()
	{
		String parameterId = "par.2";
		String neuerWert = "Editierter Testparameter 2";

		/*
		 * Parameter Editieren
		 */
		_esp.editiereParameter(parameterId, neuerWert);

		/*
		 * Editierten Parameter herauslsen
		 */
		String parameter = _esp.getParameter(parameterId);

		/*
		 * Prfen ob der herausgelste Parameter mit dem zusetzenden bereinstimmt.
		 */
		assertEquals(neuerWert, parameter);
	}

	@Test
	public void testUeberschreibeParameter()
	{
		String parameterId = "par.3";
		String neuerWert = "berschriebener Testparameter 3";

		_esp.ueberschreibeParameter(parameterId, neuerWert);
		String parameter = _esp.getParameter(parameterId);

		assertEquals(neuerWert, parameter);
	}

	@Test
	public void testLoescheParameter()
	{
		String parameterId = "par.1";

		/*
		 * Lschen des Parameters
		 */
		_esp.loescheParameter(parameterId);

		/*
		 * Prfung: Wenn es in der "Liste" der Parameter keinen Parameter mit der ID
		 * "par.1" gibt, so ist der Test bestanden.
		 */
		Boolean geloescht = new Boolean(true);
		Iterator<Entry<String, String>> it = _esp.getAlleParameter().entrySet()
				.iterator();
		while (it.hasNext())
		{
			Entry<String, String> entry = it.next();
			if (entry.equals(parameterId))
			{
				geloescht = false;
				break;
			}
		}

		assertEquals(new Boolean(true), geloescht);
	}

}
