/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.login;

import static org.junit.Assert.assertEquals;

import java.util.Date;
import java.util.LinkedList;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Mit Hilfe dieses Tests wird die eigentliche Funktionalitt des
 * {@link UrlasserDialog}es, welche sich in der Klasse {@link UrlasserObjekt}
 * befindet, getestet.<br>
 * <br>
 * <b>Vorsicht:</b> Die Daten in typ.autarkeOrganisationsEinheit /
 * atg.globaleEinstellungenOberflcheNetzweit werden verndert! Nur auf einem
 * Test-Datenverteiler anwenden!!! <br>
 * <br>
 * <b>Wichtig:</b> Dieser Test kann nur als PDE JUnit-Test erfolgen!
 * 
 * @author HofmannJ
 * 
 */
public class UrlasserTest implements ClientReceiverInterface
{
	/**
	 * Benutzer fr die Anmeldung am DaV.
	 */
	private static final String _benutzer = "Tester";

	/**
	 * Passwort fr die Anmeldung am DaV.
	 */
	private static final String _passwort = "geheim";

	/**
	 * Ursache fr die Anmeldung am DaV.
	 */
	private static final String _ursache = "Eine Ursache fr den JUnit-Test";

	/**
	 * Veranlasser fr die Anmeldung am DaV.
	 */
	private static final String _veranlasser = "Ein Veranlasser fr den JUnit-Test";

	/**
	 * Adresse des Datenverteilers
	 */
	private static final String _adresseDaV = "127.0.0.1";

	/**
	 * Port des DaV
	 */
	private static final int _portDaV = 8083;

	/**
	 * Speichert die Verbindung zum Datenverteiler
	 */
	private ClientDavConnection _verbindung = null;

	/**
	 * Datenmodell des DaV
	 */
	private DataModel _dataModell = null;

	/**
	 * SystemObjekt
	 */
	private SystemObject _systemObject = null;

	/**
	 * Bezeichnung der Attributgruppe
	 */
	private static final String _atgBezeichnung = "atg.globaleEinstellungenOberflcheNetzweit";

	/**
	 * Bezeichnung des Aspektes soll (zum Empfangen)
	 */
	private static final String _aspSoll = "asp.parameterSoll";

	/**
	 * Bezeichnung des Aspektes vorgabe (zum Senden)
	 */
	private static final String _aspVorgabe = "asp.parameterVorgabe";

	/**
	 * Attributgruppe
	 */
	private AttributeGroup _atg = null;

	/**
	 * Aspekt
	 */
	private Aspect _asp = null;

	/**
	 * Datenbeschreibung
	 */
	private DataDescription _dataDescription = null;

	/**
	 * Bezeichnung des Items BenutzereinstellungenGlobal
	 */
	private static final String _itemBenutzereinstellungenGlobal = "BenutzereinstellungenGlobal";

	/**
	 * Bezeichnung des Items Voreinstellungen
	 */
	private static final String _itemVoreinstellungen = "Voreinstellungen";

	/**
	 * Bezeichnung des Items Urlasser
	 */
	private static final String _itemUrlasser = "Urlasser";

	/**
	 * Bezeichnung des Items BenutzerReferenz
	 */
	private static final String _itemBenutzerReferenz = "BenutzerReferenz";

	/**
	 * Ursache
	 */
	private static final String _itemUrsache = "Ursache";

	/**
	 * Bezeichnung des Items Veranlasser
	 */
	private static final String _itemVeranlasser = "Veranlasser";

	/**
	 * Erwartungswert fr die Benutzerreferenz
	 */
	private String _erwartungBenutzerReferenz = "benutzer.tester";

	/**
	 * Erwartungswert fr die Ursache
	 */
	private String _erwartungUrsache = "Eine Ursache fr den JUnit-Test";

	/**
	 * Erwartungswert fr den Veranlasser
	 */
	private String _erwartungVeranlasser = "Ein Veranlasser fr den JUnit-Test";

	/**
	 * Empfangene BenutzerReferenz
	 */
	private String _aktuellBenutzerReferenz = null;

	/**
	 * Empfangene Ursache
	 */
	private String _aktuellUrsache = null;

	/**
	 * Empfangener Veranlasser
	 */
	private String _aktuellVeranlasser = null;

	/**
	 * Das UrlasserObjekt
	 */
	private UrlasserObjekt _urlasserObjekt = null;

	/**
	 * XML-Struktur zum Senden
	 */
	private String _xmlStrukturSenden = null;

	/**
	 * Empfangene XML-Struktur
	 */
	private String _xmlStrukturEmpfangen = null;

	/**
	 * Flag, zeigt an ob das Senden der Daten erfolgreich abgeschlossen werden
	 * konnte
	 */
	private boolean _sendenErfolgreich = true;

	/**
	 * Flag, zeigt an ob die gesendeten Daten erfolgreich empfangen werden
	 * konnten.
	 */
	private boolean _empfangenErfolgreich = true;

	/**
	 * Flag, zeigt an ob die Struktur die gesendet und die empfangen wurde gleich
	 * ist.
	 */
	private boolean _gesendetEmpfangGleich = true;

	/**
	 * Flag, zeigt an ob nach dem Senden der Daten ber das {@link UrlasserObjekt}
	 * die Urlasser-Information der Empfangenen Daten dem Erwartungswert
	 * entspricht.
	 */
	private boolean _urlasserInfoEntsprichtErwartungswert = true;

	/**
	 * Objekt zum Synchronisieren, dass der Thread die Empfangenen Daten
	 * verarbeitet hat (auch wenn hier keine groartige Verarbeitung stattfindet).
	 * Mit notify wird eine Methode zur Rckgabe der gelesenen Daten
	 * benachrichtigt.
	 */
	private Boolean _endeVerarbeitungEmpfang = new Boolean(true);

	/**
	 * Ergebnis
	 */
	private String _ergebnisEmpfang = "";

	/**
	 * Objekt zum Synchronisieren des Threads auf den Empfang von Daten. Empfngt
	 * die update-Methode etwas, wird Empfang notified.
	 */
	private Boolean _empfang = new Boolean(true);

	/**
	 * Empfangsthread Flag
	 */
	private boolean _laufeEmpfangsThread = false;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
	}

	@Test
	public void testUrlasserObjekt()
	{
		/*
		 * Verbindung zum DaV aufbauen
		 */
		DaVVerbindung.getInstanz().setDaVBenutzer(_benutzer);
		DaVVerbindung.getInstanz().setDaVPasswort(_passwort);
		DaVVerbindung.getInstanz().setDaVIp(_adresseDaV);
		DaVVerbindung.getInstanz().setDaVPort(_portDaV);

		DaVVerbindung.getInstanz().baueVerbindungMitDaVAuf();

		_verbindung = (ClientDavConnection) DaVVerbindung.getInstanz()
				.getDaVVerbindung();

		_dataModell = _verbindung.getDataModel();

		/*
		 * Testdaten holen
		 */
		// String vorgefertigteXmlStruktur = null;
		// // GlobaleEinstellungenNetzweit global = new
		// GlobaleEinstellungenNetzweit(
		// // _verbindung);
		// // vorgefertigteXmlStruktur = global.empfangeXmlStruktur();
		//
		// InputStream is = this.getClass().getResourceAsStream("struktur.xml");
		// BufferedInputStream bis = new BufferedInputStream(is, 4096);
		// StringBuffer sb = new StringBuffer();
		// byte[] buf = new byte[4096];
		// try
		// {
		// for (int len; (len = bis.read(buf)) != -1;)
		// {
		// sb.append(new String(buf, 0, len));
		// }
		// } catch (IOException e)
		// {
		// e.printStackTrace();
		// }
		//
		// vorgefertigteXmlStruktur = sb.toString();
		String vorgefertigteXmlStruktur = new StrukturEinlesen().einlesen();

		/*
		 * Einstellungenspeicher erzeugen und initialisieren
		 */
		EinstellungenSpeicher espGlobal = new EinstellungenSpeicher();
		espGlobal.einlesenXmlAusString(vorgefertigteXmlStruktur);

		/*
		 * UrlasserObjekt erzeugen
		 */
		UrlasserInfo urlasserInfo = new UrlasserInfo(_benutzer, _passwort,
				_ursache, _veranlasser);
		_urlasserObjekt = new UrlasserObjekt(urlasserInfo);

		/*
		 * Senden der XML-Struktur mittels Mock-Objekt
		 */
		_xmlStrukturSenden = espGlobal.ausgabeXmlFormatiert(false);

		MockObjSenden mockSenden = new MockObjSenden();
		mockSenden.setXmlStruktur(_xmlStrukturSenden);
		mockSenden.startSenden();
		synchronized (mockSenden)
		{
			try
			{
				mockSenden.wait(10000);
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}

		/*
		 * An dieser Stelle wird gewartet, bis der DaV die Daten gespeichert hat.
		 * Ansonsten wrde der Test mit einer Fehlermeldung abbrechen.
		 */
		Warten warten = new Warten();
		warten.warte(5000);

		/*
		 * Empfangen der gesendeten Struktur
		 */
		_xmlStrukturEmpfangen = empfangeXmlStruktur();

		if ((_xmlStrukturEmpfangen != null) && (_xmlStrukturEmpfangen.length() > 0))
		{
			_empfangenErfolgreich = true;

			EinstellungenSpeicher espEmpfangen = new EinstellungenSpeicher();
			espEmpfangen.einlesenXmlAusString(_xmlStrukturEmpfangen);

			/**
			 * Ist die empfangene XML-Struktur mit der ursprnglich gesendeten
			 * identisch?
			 */
			if (_xmlStrukturSenden.equals(espEmpfangen.ausgabeXmlFormatiert(false)))
			{
				_gesendetEmpfangGleich = true;
			}
			else
			{
				_gesendetEmpfangGleich = false;
			}

			/**
			 * Sind die empfangenen Urlasserinformationen mit den gesendeten
			 * identisch?
			 */
			if ((_erwartungBenutzerReferenz.equals(_aktuellBenutzerReferenz))
					&& (_erwartungUrsache.equals(_aktuellUrsache))
					&& (_erwartungVeranlasser.equals(_aktuellVeranlasser)))
			{
				_urlasserInfoEntsprichtErwartungswert = true;
			}
			else
			{
				_urlasserInfoEntsprichtErwartungswert = false;
			}

			System.out.println("Erwarteter Benutzer:    \""
					+ _erwartungBenutzerReferenz + "\", Ist:\""
					+ _aktuellBenutzerReferenz + "\"");
			System.out.println("Erwartete Ursache:      \"" + _erwartungUrsache
					+ "\",         Ist:\"" + _aktuellUrsache + "\"");
			System.out.println("Erwarteter Veranlasser: \"" + _erwartungVeranlasser
					+ "\",     Ist:\"" + _aktuellVeranlasser + "\"");
		}

		/*
		 * Trennen der Verbindung zum DaV
		 */
		DaVVerbindung.getInstanz().trenneAlleVerbindungen();

		String staErg = "";
		staErg = staErg + "============Status:============\n";
		staErg = staErg + "Klasse:               " + this.getClass().getName()
				+ "\n";
		staErg = staErg + "SendenErfolgreich:    " + _sendenErfolgreich + "\n";
		staErg = staErg + "EmpfangenErfolgreich: " + _empfangenErfolgreich + "\n";
		staErg = staErg + "Gleich:               " + _gesendetEmpfangGleich + "\n";
		staErg = staErg + "Urlasser OK:          "
				+ _urlasserInfoEntsprichtErwartungswert + "\n";
		staErg = staErg + "===============================\n";
		System.out.println(staErg);

		assertEquals(true, _sendenErfolgreich && _empfangenErfolgreich
				&& _gesendetEmpfangGleich && _urlasserInfoEntsprichtErwartungswert);

		new Warten().warte(5000);
	}

	/**
	 * Sendet die XML-Struktur.
	 * 
	 * @param xmlStruktur
	 *          Zu sendende XML-Struktur innerhalb eines Strings.
	 */
	private boolean sendeXmlStruktur(String xmlStruktur)
	{
		// System.out.println("SENDE:\n" + xmlStruktur);

		boolean re = false;

		if (_verbindung != null)
		{
			String[] split = xmlStruktur.split("\n");
			re = sendeZeilen(split);
		}

		return re;
	}

	/**
	 * Sendet die Werte aus einem String-Array.
	 * 
	 * @param wert
	 *          String-Array mit den Werten als String.
	 */
	private boolean sendeZeilen(String[] wert)
	{
		_atg = _dataModell.getAttributeGroup(_atgBezeichnung);
		Data data = _verbindung.createData(_atg);
		Data item = data.getItem(_itemBenutzereinstellungenGlobal);
		TextArray textArray = item.getTextArray(_itemVoreinstellungen);
		textArray.setLength(wert.length);

		for (int i = 0; i < wert.length; i++)
		{
			textArray.getTextValue(i).setText(wert[i]);
		}

		_atg = _dataModell.getAttributeGroup(_atgBezeichnung);
		_asp = _dataModell.getAspect(_aspVorgabe);
		_dataDescription = new DataDescription(_atg, _asp);

		/*
		 * Aus der alten DataDescription eine neue unter dem Parameter
		 * asp.parameterVorgabe erzeugen.
		 */
		DataDescription dataDescriptionNeu = new DataDescription(_dataDescription
				.getAttributeGroup(), _verbindung.getDataModel().getAspect(_aspVorgabe));

		/*
		 * SystemObjekt bestimmen
		 */
		String objekt = bestimmePidOberflaechenObjekt();

		if (objekt != null)
		{
			_systemObject = _verbindung.getDataModel().getObject(objekt);

			ResultData datenTelegramm = new ResultData(_systemObject,
					dataDescriptionNeu, new Date().getTime(), data);

			LinkedList<ResultData> liste = new LinkedList<ResultData>();
			liste.add(datenTelegramm);

			_urlasserObjekt.baueDaVVerbindungAuf();
			LinkedList<ResultData> sendenFehlgeschlagen = _urlasserObjekt
					.sendeDaten(liste);

			/*
			 * Ist das Senden fehlgeschlagen?!?
			 */
			if (sendenFehlgeschlagen.size() > 0)
			{
				_sendenErfolgreich = false;
			}
		}

		return _sendenErfolgreich;
	}

	/**
	 * Bestimmt die PID des Oberflchenobjektes an Hand des
	 * Kommandozeilenparameters <code>-oberflaechenObjekt</code>. Wird dieser
	 * nicht gefunden wird der AOE verwendet.
	 * 
	 * @return Pid des Oberflchenobjektes oder <code>null</code>, wenn diese
	 *         Pid nicht bestimmt werden konnte.
	 */
	private String bestimmePidOberflaechenObjekt()
	{
		String[] kommandozeilenArgumente = Platform.getCommandLineArgs();

		String pidSysObj = null;

		/*
		 * Suchen des bentigten Parameters -oberflaechenObjekt
		 */
		for (int i = 0; i < kommandozeilenArgumente.length; i++)
		{
			if (kommandozeilenArgumente[i].contains("-oberflaechenObjekt"))
			{
				String[] split = kommandozeilenArgumente[i].split("=");
				if (split.length == 2)
				{
					pidSysObj = split[1];
				}

				/*
				 * Prfen ob ein Objekt zu dieser PID existiert. Ist dies nicht der
				 * Fall, wird pidSysObj auf Null gesetzt und der AOE wird verwendet.
				 */
				_systemObject = _dataModell.getObject(pidSysObj);
				if (_systemObject == null)
				{
					String meldung = "Der Parameter -oberflaechenObjekt ist fehlerhaft. "
							+ "Es existiert kein OberflchenObjekt mit der PID "
							+ pidSysObj
							+ ". Bestimme und verwende die PID der AutarkenOrganisationsEinheit "
							+ "zur Bestimmung der globalen, netzweiten Einstellungen (Klasse:"
							+ this.getClass().getName() + ".";

					Activator.getDefault().getLog().log(
							new Status(Status.WARNING, Activator.PLUGIN_ID, Status.OK,
									meldung, null));

					pidSysObj = null;
				}

				break;
			}
		}

		/*
		 * Wenn dieser nicht gefunden wurde, AOE verwenden!
		 */
		if (pidSysObj == null)
		{
			pidSysObj = _verbindung.getLocalConfigurationAuthority().getPid();
		}

		return pidSysObj;
	}

	/**
	 * MOCK-Objekt zum Senden von Daten in einem eigenen Thread und zur
	 * Synchronisation ber {@link MockObjSenden}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class MockObjSenden implements Runnable
	{
		/**
		 * XML-Struktur, die gesendet werden soll
		 */
		private String _xmlStruktur = null;

		public MockObjSenden()
		{
		}

		public void setXmlStruktur(String xmlStruktur)
		{
			this._xmlStruktur = xmlStruktur;
		}

		public void startSenden()
		{
			Thread th = new Thread(this);
			th.setName(this.getClass().getName());
			th.start();
		}

		@Override
		public void run()
		{
			boolean status = sendeXmlStruktur(_xmlStruktur);
			_sendenErfolgreich = status;

			synchronized (this)
			{
				notifyAll();
			}

		}
	}

	/**
	 * Ruft aus dem DaV die XML-Strukturen der allgemeinen, netzweiten
	 * Einstellungen ab.
	 */
	public String empfangeXmlStruktur()
	{
		Empfaenger e = new Empfaenger();

		synchronized (_endeVerarbeitungEmpfang)
		{
			try
			{

				e.startThread();
				anmeldenEmpfang();

				synchronized (_endeVerarbeitungEmpfang)
				{
					_endeVerarbeitungEmpfang.wait();
				}

				/*
				 * Abmeldung erfolgt durch die update-Methode
				 */

			}
			catch (InterruptedException e1)
			{
				e1.printStackTrace();
			}
		}

		return _ergebnisEmpfang;
	}

	private void anmeldenEmpfang()
	{
		String pidSysObj = bestimmePidOberflaechenObjekt();

		if (pidSysObj != null)
		{
			_systemObject = _dataModell.getObject(pidSysObj);

			_atg = _dataModell.getAttributeGroup(_atgBezeichnung);
			_asp = _dataModell.getAspect(_aspSoll);
			_dataDescription = new DataDescription(_atg, _asp);
			ReceiveOptions receiveOptions = ReceiveOptions.normal();
			ReceiverRole receiverRole = ReceiverRole.receiver();

			_verbindung.subscribeReceiver(this, _systemObject, _dataDescription,
					receiveOptions, receiverRole);
		}
	}

	/**
	 * bernimmt die Abmeldung vom Empfang
	 */
	private void abmeldenEmpfang()
	{
		_verbindung.unsubscribeReceiver(this, _systemObject, _dataDescription);
	}

	@Override
	public void update(ResultData[] results)
	{
		for (ResultData resultData : results)
		{
			if (resultData.hasData())
			{

				if (resultData.getDataDescription().getAttributeGroup().getPid()
						.equalsIgnoreCase(_atgBezeichnung))
				{
					StringBuffer buf = new StringBuffer();

					Data data = resultData.getData();
					Data item = data.getItem(_itemBenutzereinstellungenGlobal);

					TextArray textArray = item.getTextArray(_itemVoreinstellungen);

					for (int i = 0; i < textArray.getLength(); i++)
					{
						String wert = textArray.getTextValue(i).getText();
						buf.append(wert);
					}

					_ergebnisEmpfang = buf.toString();

					/*
					 * Urlasser auswerten
					 */
					_aktuellBenutzerReferenz = data.getItem(_itemUrlasser)
							.getReferenceValue(_itemBenutzerReferenz).getSystemObjectPid();
					_aktuellUrsache = data.getItem(_itemUrlasser).getTextValue(
							_itemUrsache).getText();
					_aktuellVeranlasser = data.getItem(_itemUrlasser).getTextValue(
							_itemVeranlasser).getText();

					System.out.println("==>PID Urlasser: " + _aktuellBenutzerReferenz
							+ " Ursache: " + _aktuellUrsache + " Veranlasser: "
							+ _aktuellVeranlasser);

					synchronized (_empfang)
					{
						_empfang.notify();
					}
				}

			}
		}

		abmeldenEmpfang();
	}

	/**
	 * Empfngerklasse, sorgt fr den Empfang der Daten in einem eigenstndigen
	 * Thread.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Empfaenger implements Runnable
	{
		public Empfaenger()
		{
		}

		/**
		 * Startet den Thread
		 */
		public void startThread()
		{
			if (!_laufeEmpfangsThread)
			{
				_laufeEmpfangsThread = true;
				Thread empfangsThread = new Thread(this);
				empfangsThread.setName("Empfangs Thread");
				empfangsThread.start();
			}
		}

		/**
		 * Stoppt den Thread
		 */
		public void stopThread()
		{
			if (_laufeEmpfangsThread)
			{
				_laufeEmpfangsThread = false;
			}
		}

		@Override
		public void run()
		{

			while (_laufeEmpfangsThread)
			{
				synchronized (_empfang)
				{
					try
					{
						_empfang.wait();
						synchronized (_endeVerarbeitungEmpfang)
						{
							_endeVerarbeitungEmpfang.notify();
						}
						stopThread();
					}
					catch (InterruptedException e)
					{
						e.printStackTrace();
					}
				}

			}

		}

	}

	/**
	 * Thread-Klasse, liefert einenb Befehl <code>warten</code>, mit dessen
	 * Hilfe die Ausfhrung des JUnit-Testes pausiert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Warten implements Runnable
	{
		private boolean _laufeThread = false;
		private Long _warteZeit = new Long(1000);

		public void warte(long warteZeit)
		{
			synchronized (_warteZeit)
			{
				_warteZeit = warteZeit;
			}

			startThread();
			synchronized (this)
			{
				try
				{
					wait();
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

		private void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Warten...");
				_laufeThread = true;
				th.start();
			}
		}

		@Override
		public void run()
		{
			if (_laufeThread)
			{
				try
				{
					Thread.sleep(_warteZeit);
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
				finally
				{
					synchronized (this)
					{
						notifyAll();
					}
				}
			}

			_laufeThread = false;
		}

	}

}
