/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import static org.junit.Assert.assertEquals;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.EnumAsbVerkehrsRichtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenKnoten;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenSegment;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.AsbKnotenKombiniert;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.BereichsInfoAsbKnotenKombiniert;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Test der Ortsangabe
 * 
 * @author HofmannJ
 * 
 */
public class OrtsangabeTest
{
	/**
	 * Ausgaben auf die Shell
	 */
	private static final boolean _shellAusgabe = false;

	/**
	 * DaV-Benutzer
	 */
	private static final String _benutzer = "Tester";

	/**
	 * Benutzer Passwort
	 */
	private static final String _passwort = "geheim";

	/**
	 * IP-Adresse des DaV
	 */
	private static final String _ipAdr = "127.0.0.1";
	// private static final String _ipAdr = "192.0.1.59";

	/**
	 * Portnummer des DaV
	 */
	private static final int _port = 8083;

	/**
	 * Klasse zur Verwaltung der Verbindung mit dem DaV
	 */
	private static DaVVerbindung _daVVerbindung = null;

	/**
	 * Verbindung mit dem DaV
	 */
	private static ClientDavInterface _connection = null;

	/**
	 * Instanz der Klasse Warten fr Warte-Aktionen
	 */
	@SuppressWarnings("unused")
	private Warten _warten = new Warten();

	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		_daVVerbindung = DaVVerbindung.getInstanz();

		if (_daVVerbindung.getDaVVerbindung() == null)
		{
			_daVVerbindung.setDaVBenutzer(_benutzer);
			_daVVerbindung.setDaVPasswort(_passwort);
			_daVVerbindung.setDaVIp(_ipAdr);
			_daVVerbindung.setDaVPort(_port);
			boolean baueVerbindungMitDaVAuf = _daVVerbindung
					.baueVerbindungMitDaVAuf();
			if (baueVerbindungMitDaVAuf)
			{
				_connection = _daVVerbindung.getDaVVerbindung();

				if (_shellAusgabe)
				{
					System.out.println("Verbindung aufgebaut.");
				}
			}
		} else
		{
			System.out.println("Benutze existierende Verbindung.");
			_connection = _daVVerbindung.getDaVVerbindung();
		}

		System.out.println("===Start Test Ortsangaben===");
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
		_daVVerbindung.trenneAlleVerbindungen();
		if (_shellAusgabe)
		{
			System.out.println("Verbindung getrennt!");
		}

		System.out.println("=== Ende Test Ortsangaben===");
	}

	@Before
	public void setUp() throws Exception
	{
	}

	@After
	public void tearDown() throws Exception
	{
	}

	@Test
	public void testStrassenSegmentOffset()
	{
		System.out.println("...Teste StrassenSegmentOffsets");
		/*
		 * A5 Heidelberg -> Darmstadt, PID=strasse.A.5.tmclc.07129
		 * ass.n.11611.n.11610.0 Max. Km.: 2,867
		 */

		// String pidStrasse = "strasse.A.5.tmclc.07129";
		// String pidStrSeg = "ass.n.11611.n.11610.0";

		String pidStrasse = "strasse.A.5.tmclc.07127";
		String pidStrSeg = "ass.n.11591.n.11590.0";

		/*
		 * Filter erzeugen
		 */
		FilterStrassenSegmentOffset filter = new FilterStrassenSegmentOffset(
				(ClientDavConnection) _connection);

		/*
		 * Strae bestimmen
		 */
		Strasse prStrasse = null;

		LinkedList<Strasse> strassen = filter.getStrassen();
		Iterator<Strasse> it = strassen.iterator();
		while (it.hasNext())
		{
			Strasse str = it.next();
			if (str.getPid().equals(pidStrasse))
			{
				prStrasse = str;
				if (_shellAusgabe)
				{
					System.out.println("Strae gefunden: " + prStrasse);
				}
			}
		}

		/*
		 * Segment bestimmen
		 */
		StrassenSegment prStrassenSegment = null;

		if (prStrasse != null)
		{
			filter.setPidSelektierteStrasse(prStrasse.getPid());
			LinkedList<StrassenSegment> strassenSegmenteGefiltert = filter
					.getStrassenSegmenteGefiltert(prStrasse.getPid());
			Iterator<StrassenSegment> itSS = strassenSegmenteGefiltert.iterator();
			while (itSS.hasNext())
			{
				StrassenSegment strSeg = itSS.next();

				if (strSeg.getPid().equals(pidStrSeg))
				{
					prStrassenSegment = strSeg;
					filter.setPidSelektiertesStrassenSegment(prStrassenSegment.getPid());
					if (_shellAusgabe)
					{
						System.out.println("Straensegment gefunden: " + prStrassenSegment);
					}
				}
			}
		}

		boolean unterGrenze = false;
		boolean anGrenze = false;
		boolean ueberGrenze = false;

		/*
		 * Prfen
		 */
		if ((prStrasse != null) && (prStrassenSegment != null))
		{
			unterGrenze = filter.pruefeKilometerSegment(2.0f, prStrassenSegment
					.getPid());
			anGrenze = filter.pruefeKilometerSegment(14.3f, prStrassenSegment
					.getPid());
			ueberGrenze = filter.pruefeKilometerSegment(14.4f, prStrassenSegment
					.getPid());
		}

		if (_shellAusgabe)
		{
			String msg = "";
			msg = msg + "===Test StrassenSegment Offset===\n";
			msg = msg + "unterGrenze: " + unterGrenze + "\n";
			msg = msg + "anGrenze:    " + anGrenze + "\n";
			msg = msg + "ueberGrenze: " + ueberGrenze + "\n";
			msg = msg + "=================================\n";
			System.out.println(msg);
		}

		assertEquals(true, unterGrenze && anGrenze && (!ueberGrenze));
	}

	@Test
	public void testBetriebskilometer()
	{
		System.out.println("...Teste Betriebskilometer");
		// Strae: A5 von Basel nach Karlsruhe (strasse.A.5.tmclc.07127)
		// -ass.n.11591.n.11590.0*
		// ---632000 BL 0
		// ---633000 BL KeineAngabe,0
		// ---222 BL - Nicht vorhanden
		// ass.n.11592.n.11591.0
		// ass.n.11593.n.11592.0

		/*
		 * A5 Basel -> Karlsruhe
		 */

		String pidStrasse = "strasse.A.5.tmclc.07127";
		long wert1 = 632000l;
		long wert2 = 633000;
		long wert3 = 222;

		FilterBetriebskilometer filter = new FilterBetriebskilometer(
				(ClientDavConnection) _connection);

		/*
		 * Strae bestimmen
		 */
		Strasse prStrasse = null;
		LinkedList<Strasse> strassen = filter.getStrassen();
		Iterator<Strasse> it = strassen.iterator();
		while (it.hasNext())
		{
			Strasse strasse = it.next();
			if (pidStrasse.equals(strasse.getPid()))
			{
				prStrasse = strasse;
				filter.setSelektierteStrasse(prStrasse.getPid());
				if (_shellAusgabe)
				{
					System.out.println("Strae gefunden: " + prStrasse);
				}
			}
		}

		/*
		 * Prfungen
		 */
		boolean pruefung1 = false;
		boolean pruefung2 = false;
		boolean pruefung3 = false;

		if (prStrasse != null)
		{
			/*
			 * Wert 1
			 */
			filter.setBetriebskilometer(wert1);
			LinkedList<String> vorhandeneBlockNummernWert1 = filter
					.getVorhandeneBlockNummern();
			if (vorhandeneBlockNummernWert1.size() >= 1)
			{
				// if (vorhandeneBlockNummernWert1.get(0).equals("KeineAngabe"))
				// {
				pruefung1 = true;
				// }
			}

			/*
			 * Wert 2
			 */
			filter.setBetriebskilometer(wert2);
			LinkedList<String> vorhandeneBlockNummernWert2 = filter
					.getVorhandeneBlockNummern();

			if (vorhandeneBlockNummernWert2.size() >= 1)
			{
				// if (vorhandeneBlockNummernWert2.get(0).equals("KeineAngabe"))
				// {
				pruefung2 = true;
				// }
			}

			/*
			 * Wert 3
			 */
			filter.setBetriebskilometer(wert3);
			LinkedList<String> vorhandeneBlockNummernWert3 = filter
					.getVorhandeneBlockNummern();
			if (vorhandeneBlockNummernWert3.size() == 0)
			{
				pruefung3 = true;
			}
		}

		if (_shellAusgabe)
		{
			String out = "\n";
			out = out + "Prfung1: " + pruefung1 + "\n";
			out = out + "Prfung2: " + pruefung2 + "\n";
			out = out + "Prfung3: " + pruefung3 + "\n";
			System.out.println("BKTEST\n====>" + out + "<====");
		}

		assertEquals(true, pruefung1 && pruefung2 && pruefung3);
	}

	@Test
	public void testKnotenOffset()
	{
		System.out.println("...Teste KnotenOffsets");
		/*
		 * A5 von Karlsruhe nach Heidelberg (strasse.A.5.tmclc.07128))
		 */
		String pidStrasse = "strasse.A.5.tmclc.07128";

		FilterKnoten filter = new FilterKnoten((ClientDavConnection) _connection);

		/*
		 * Strae bestimmen
		 */
		Strasse prStrasse = null;
		LinkedList<Strasse> strassen = filter.getStrassen();
		Iterator<Strasse> it = strassen.iterator();
		while (it.hasNext())
		{
			Strasse strasse = it.next();
			if (pidStrasse.equals(strasse.getPid()))
			{
				prStrasse = strasse;
				filter.setPidStrasse(prStrasse.getPid());
				if (_shellAusgabe)
				{
					System.out.println("Strae gefunden: " + prStrasse);
				}
			}
		}

		// Autobahnanschlussstelle Ettlingen [NK 7016034]
		// (strassenKnoten.11592.52955.53077)
		// Autobahndreieck Karlsruhe [NK 7016035] (strassenKnoten.11593.12513)
		// Autobahnanschlussstelle Karlsruhe-Mitte [NK 7016053]
		// (strassenKnoten.11594.30884)
		// Autobahnanschlussstelle Karlsruhe-Durlach [NK 6916012]
		// (strassenKnoten.11595.20287)
		// Autobahnanschlussstelle Bruchsal [NK 6817071]
		// (strassenKnoten.11597.23086)
		// Raststtte Bruchsal [NK 6817047] (strassenKnoten.11598)
		// Autobahnanschlussstelle Kronau [NK 6717009] (strassenKnoten.11599)
		// Autobahnkreuz Walldorf [NK 6717023] (strassenKnoten.11600.11882)
		// Autobahnanschlussstelle Walldorf/Wiesloch [NK 6617008]
		// (strassenKnoten.11601.23150)
		// Autobahnanschlussstelle Karlsruhe-Nord [NK 6817071]
		// (strassenKnoten.41323)
		String[] pidsSoll = new String[] { "strassenKnoten.11592.52955.53077",
				"strassenKnoten.11593.12513", "strassenKnoten.11594.30884",
				"strassenKnoten.11595.20287", "strassenKnoten.11597.23086",
				"strassenKnoten.11598", "strassenKnoten.11599",
				"strassenKnoten.11600.11882", "strassenKnoten.11601.23150",
				"strassenKnoten.41323" };

		String[] pidsIst = null;

		/*
		 * Knoten bestimmen
		 */
		if (prStrasse != null)
		{
			List<StrassenKnoten> strKnoten = filter.getKnoten(prStrasse.getPid());
			pidsIst = new String[strKnoten.size()];
			Iterator<StrassenKnoten> itKnoten = strKnoten.iterator();
			int i = 0;
			while (itKnoten.hasNext())
			{
				StrassenKnoten knoten = itKnoten.next();
				pidsIst[i] = knoten.getPid();
				i++;
			}
		}

		boolean felderSindGleich = false;
		if (pidsIst != null)
		{
			felderSindGleich = istStringArrayGleich(pidsSoll, pidsIst);
		}

		assertEquals(true, felderSindGleich);
	}

	@Test
	public void testAsbOrtsangabe()
	{
		System.out.println("...Teste ASB Orstsanabe");
		
		if (_shellAusgabe)
		{
			System.out.println("\n======TEST ASB======");
		}

		AsbKnotenKombiniert prVonKnotenKombiniert = null;
		AsbKnotenKombiniert prBisKnotenKombiniert = null;

		/*
		 * A5 von Basel nach Karlsruhe (strasse.A.5.tmclc.07127)
		 */
		String pidStrasse = "strasse.A.5.tmclc.07127";
		String vonAsbKnotenText = "7016018";
		String bisAsbKnotenText = "7016034";

		/*
		 * Filter
		 */
		FilterAsbOrtsangabe filter = new FilterAsbOrtsangabe(
				(ClientDavConnection) _connection);

		/*
		 * Strae bestimmen
		 */
		Strasse prStrasse = null;
		LinkedList<Strasse> strassen = filter.getStrassen();
		Iterator<Strasse> it = strassen.iterator();
		while (it.hasNext())
		{
			Strasse strasse = it.next();
			if (pidStrasse.equals(strasse.getPid()))
			{
				prStrasse = strasse;
				filter.setSelektierteStrasse(prStrasse.getPid());
				if (_shellAusgabe)
				{
					System.out.println("Strae gefunden: " + prStrasse);
				}
			}
		}

		/*
		 * Von Knoten
		 */

		if (prStrasse != null)
		{

			LinkedList<AsbKnotenKombiniert> vonAsbKnoten = filter.getAsbOrtsangabe()
					.getVonAuswahlfeldKombiniert();

			Iterator<AsbKnotenKombiniert> itAsbKnoten = vonAsbKnoten.iterator();
			while (itAsbKnoten.hasNext())
			{
				AsbKnotenKombiniert asbKnotenVon = itAsbKnoten.next();
				prVonKnotenKombiniert = asbKnotenVon;

				if (_shellAusgabe)
				{
					System.out.println("ASB-Knoten: " + asbKnotenVon);
				}

				if (asbKnotenVon.getKnoten().equals(vonAsbKnotenText))
				{

					/*
					 * Nach Knoten
					 */
					if (prVonKnotenKombiniert != null)
					{
						LinkedList<AsbKnotenKombiniert> erreichbareAsbKnoten = filter
								.getAsbOrtsangabe().bestimmeErreichbareAsbKnotenKombiniert(
										prVonKnotenKombiniert).getListeKombinierteKnoten();
						Iterator<AsbKnotenKombiniert> itErAsbKnoten = erreichbareAsbKnoten
								.iterator();
						while (itErAsbKnoten.hasNext())
						{
							prBisKnotenKombiniert = itErAsbKnoten.next();

							if (_shellAusgabe)
							{
								System.out.println("\tErreichbare Knoten: "
										+ prBisKnotenKombiniert);
							}

							if (prBisKnotenKombiniert.getKnoten().equals(bisAsbKnotenText))
							{
								break;
							}

						}
					}

					break;
				}
			}

		}

		boolean knotenGefunden = false;
		boolean pruefung1 = false;
		boolean pruefung2 = false;
		boolean pruefung3 = false;
		boolean pruefung4 = false;
		boolean pruefung5 = false;

		if ((prVonKnotenKombiniert != null) && (prBisKnotenKombiniert != null))
		{
			knotenGefunden = true;

			BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert1 = filter
					.getAsbOrtsangabe().pruefeAsbStationierungKombiniert(
							prVonKnotenKombiniert, prBisKnotenKombiniert, 0);
			pruefung1 = bereichsInfoStationierungKombiniert1.istInBereich();

			BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert2 = filter
					.getAsbOrtsangabe().pruefeAsbStationierungKombiniert(
							prVonKnotenKombiniert, prBisKnotenKombiniert, 1200);
			pruefung2 = bereichsInfoStationierungKombiniert2.istInBereich();

			BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert3 = filter
					.getAsbOrtsangabe().pruefeAsbStationierungKombiniert(
							prVonKnotenKombiniert, prBisKnotenKombiniert, 1652);
			boolean[] istInBereichGefiltert = bereichsInfoStationierungKombiniert3
					.istInBereichGefiltert(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG);
			if (istInBereichGefiltert.length == 2)
			{
				if (istInBereichGefiltert[0] || istInBereichGefiltert[1])
				{
					pruefung3 = true;
				}
			}

			BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert4 = filter
					.getAsbOrtsangabe().pruefeAsbStationierungKombiniert(
							prVonKnotenKombiniert, prBisKnotenKombiniert, 3401);
			boolean[] istInBereichGefiltert4 = bereichsInfoStationierungKombiniert4
					.istInBereichGefiltert(EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG);
			if (istInBereichGefiltert4.length == 2)
			{
				if (!istInBereichGefiltert4[0] || !istInBereichGefiltert4[1])
				{
					pruefung4 = true;
				}
			}

			BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert5 = filter
					.getAsbOrtsangabe().pruefeAsbStationierungKombiniert(
							prVonKnotenKombiniert, prBisKnotenKombiniert, 3402);
			pruefung5 = !bereichsInfoStationierungKombiniert5.istInBereich();
		}

		if (_shellAusgabe)
		{
			System.out.println("Knoten gefunden: " + knotenGefunden);
			System.out.println("Prfung 1      : " + pruefung1);
			System.out.println("Prfung 2      : " + pruefung2);
			System.out.println("Prfung 3      : " + pruefung3);
			System.out.println("Prfung 4      : " + pruefung4);
			System.out.println("Prfung 5      : " + pruefung5);
		}

		assertEquals(true, knotenGefunden && pruefung1 && pruefung2 && pruefung3
				&& pruefung4 && pruefung5);
	}

	/**
	 * Prft, ob der Inhalt von zwei {@link String}-Arrays gleich ist.
	 * 
	 * @param soll
	 *          Array, welches den Soll-Wert darstellt.
	 * @param ist
	 *          Array mit dem Ist-Wert, welches gegen das Soll-Array geprft
	 *          werden soll.
	 * @return Liefert <code>true</code>, wenn die beiden Array gleich sind, sonst
	 *         <code>false</code>.
	 */
	private boolean istStringArrayGleich(String[] soll, String[] ist)
	{
		boolean re = true;

		if (soll.length == ist.length)
		{
			for (int i = 0; i < soll.length; i++)
			{
				if (!soll[i].equals(ist[i]))
				{
					re = false;
					break;
				}
			}
		} else
		{
			re = false;
		}

		return re;
	}

	/**
	 * Thread-Klasse, liefert einen Befehl <code>warten</code>, mit dessen Hilfe
	 * die Ausfhrung des JUnit-Testes pausiert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Warten implements Runnable
	{
		private boolean _laufeThread = false;
		private Long _warteZeit = new Long(1000);

		public Warten()
		{
		}

		/**
		 * Veranlasst das Programm dazu, eine bestimmte Zeit lang zu warten.
		 * 
		 * @param warteZeit
		 *          Wartezeit in Millisekunden.
		 */
		@SuppressWarnings("unused")
		public void warte(long warteZeit)
		{
			synchronized (_warteZeit)
			{
				_warteZeit = warteZeit;
			}

			startThread();
			synchronized (this)
			{
				try
				{
					wait();
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

		private void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Warten...");
				_laufeThread = true;
				th.start();
			}
		}

		@Override
		public void run()
		{
			if (_laufeThread)
			{
				try
				{
					Thread.sleep(_warteZeit);
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				} finally
				{
					synchronized (this)
					{
						notifyAll();
					}
				}
			}

			_laufeThread = false;
		}

	}

}
