/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib;

import java.io.File;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen.DefinitionenEinstellungen;
import de.bsvrz.buv.rw.basislib.menu.generatoren.MenuGenerator;
import de.bsvrz.buv.rw.basislib.menu.generatoren.SymbolleisteGenerator;
import de.bsvrz.buv.rw.basislib.statusleiste.generator.StatusLeisteGenerator;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse nimmt die Initialisierung der Leisten (Men-, Symbol- und
 * Statusleiste) vor.
 * 
 * @author HofmannJ
 * 
 */
public class LeistenInitialisierung implements Runnable, IDaVVerbindungZuhoerer
{
	/**
	 * Eclipse RCP Logger
	 */
	private ILog _log = null;

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Thread-Flag
	 */
	private boolean _laufeThread = false;

	/**
	 * Maximale Zeit in Millisekunden die gewartet werden soll, bis kein neues
	 * Element fr Status/Symbol/Menleiste mehr hinzukommt.
	 */
	private long _maxZeit = 3000l;

	/**
	 * Nimmt die Anzahl der Elemente fr die Statusleiste auf.
	 */
	private int _anzStatusElemente = 0;

	/**
	 * Nimmt den letzten Zeitpunkt auf, an dem ein Element zur
	 * {@link StatusLeisteVerwaltung} hinzugefgt wurde.
	 */
	private long _zeitStatusElemente = 0l;

	/**
	 * Nimmt die Anzahl der Deskriptoren (fr konkrete Actions von Plug-Ins und
	 * noch durch kein Plug-In registrierten Actions auf.
	 */
	private int _anzDeskriptorenKombiniert = 0;

	/**
	 * Speichert den Zeitpunkt, zu dem ein kombinierter Deskriptor hinzugefgt
	 * wurde.
	 */
	private long _zeitDeskiptorenKombiniert = 0l;

	/**
	 * Speichert die Anzahl der globalen Aktionen, die zur
	 * {@link ElementRegistrierung} hinzugefgt wurden.
	 */
	private int _anzGlobalerAktionen = 0;

	/**
	 * Speichert den Zeitpunkt, zu dem die letzte globale Aktion hinzugefgt
	 * wurde.
	 */
	private long _zeitGlobalerAktionen = 0l;

	/**
	 * Verweis auf die Verwaltung der Statusleiste
	 */
	private StatusLeisteVerwaltung _statusLeisteVerwaltung = null;

	/**
	 * Verweis auf die Elementregistrierung
	 */
	private ElementRegistrierung _elementRegistrierung = null;

	/**
	 * Menleiste
	 */
	private MenuSymbolLeisteDaten _strukturMenuLeiste = null;

	/**
	 * Symbolleiste
	 */
	private MenuSymbolLeisteDaten _strukturSymbolLeiste = null;

	/**
	 * Zur berwachung des angemeldeten Benutzers
	 */
	private BenutzerAngemeldet _benAngemeldet = BenutzerAngemeldet.getInstanz();

	/**
	 * Speichert den Namen des "alten", angemeldeten Benutzers
	 */
	private String _alterAngemeldeterBenutzer = null;

	/**
	 * Zeigt an, ob alle Elemente fr die Men- & Symbolleiste nach dem alten
	 * Muster geladen wurden.
	 */
	private boolean _pruefungenOk = false;

	/**
	 * Flag. Zeigt an, ob die initiale Struktur von Men-, Symbol- und
	 * Statusleiste geladen wurde.
	 */
	private boolean _initialeStrukturGeladen = false;

	/**
	 * Gibt an, ob die Daten fr die Leisten schnell bezogen werden sollen. Bei
	 * <code>true</code> werden also Aktionen ber den Erweiterungspunkt
	 * <code>org.eclipse.ui.actionSets</code> eingebunden.<br>
	 * Bei <code>false</code> wird das alte, langsame, bisherige Men eingebunden.
	 */
	private static final boolean DatenSchnellHolen = true;

	/**
	 * Privater Konstruktor
	 */
	private LeistenInitialisierung()
	{
		_statusLeisteVerwaltung = StatusLeisteVerwaltung.getInstanz();
		_elementRegistrierung = ElementRegistrierung.getInstanz();

		/*
		 * Logger initalisieren
		 */
		_log = Activator.getDefault().getLog();

		// _log.log(new Status(Status.INFO, Activator.PLUGIN_ID,
		// "Leisteninitialisierung ist aktiv!"));

		aktionVorLadenMenuSymbolLeisteAusXmlDateien();

		if (DatenSchnellHolen)
		{
			DaVVerbindung.getInstanz().hinzufuegenZuhoerer(this);
		}

		startThread();
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link LeistenInitialisierung}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen
	{
		private static LeistenInitialisierung INSTANZ = new LeistenInitialisierung();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link LeistenInitialisierung}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link LeistenInitialisierung}
	 */
	public static LeistenInitialisierung getInstanz()
	{
		return Innen.INSTANZ;
	}

	/**
	 * Startet den Thread zur Leisteninitialisierung
	 */
	public void startThread()
	{
		if (!_laufeThread)
		{
			Thread th = new Thread(this);
			th.setName("Leisteninitiialisierung");
			_laufeThread = true;
			th.start();
		}
	}

	/**
	 * Stoppt den Thread zur Leisteninitialisierung
	 */
	public void stopThread()
	{
		if (_laufeThread)
		{
			_laufeThread = false;
		}
	}

	/**
	 * Sorgt dafr, dass der {@link MenuGenerator} fr die Menleiste
	 * initialisiert wird.<br>
	 * <b>Achtung:</b> Die Initialisierung erfolgt asynchron im SWT-Thread.
	 */
	private void darstellenMenuLeiste()
	{
		Display display = PlatformUI.getWorkbench().getDisplay();
		display.asyncExec(new Runnable()
		{

			@Override
			public void run()
			{
				MenuGenerator mg = new MenuGenerator(ElementRegistrierung.getInstanz()
						.getMenuManager(), _strukturMenuLeiste);
				mg.generiereMenu();
			}

		});
	}

	/**
	 * Sorgt dafr, dass der {@link SymbolleisteGenerator} fr die Symbolleiste
	 * initialisiert wird.<br>
	 * <b>Achtung:</b> Die Initialisierung erfolgt asynchron im SWT-Thread.
	 */
	private void darstellenSymbolLeiste()
	{
		Display display = PlatformUI.getWorkbench().getDisplay();
		display.asyncExec(new Runnable()
		{

			@Override
			public void run()
			{
				ICoolBarManager coolBarManager = ElementRegistrierung.getInstanz()
						.getCoolBarManager();
				SymbolleisteGenerator symbolleisteGenerator = new SymbolleisteGenerator(
						coolBarManager, _strukturSymbolLeiste);
				symbolleisteGenerator.generiereSymbolLeiste();
			}

		});

	}

	/**
	 * Sorgt dafr, dass der {@link StatusLeisteGenerator} fr die Darstellung der
	 * Statuleiste initialisiert wird.<br>
	 * <b>Achtung:</b> Die Initialisierung erfolgt asynchron im SWT-Thread.
	 */
	private void darstellenStatusLeiste()
	{
		Display display = PlatformUI.getWorkbench().getDisplay();
		display.asyncExec(new Runnable()
		{
			@Override
			public void run()
			{
				StatusLeisteGenerator.getInstanz().generiereStatusLeiste();
				_statusLeisteVerwaltung.aktualisieren();
			}
		});
	}

	/**
	 * Ldt die in den allgemeinen, netzweiten Einstellungen vorhandene
	 * Menstruktur. Glckt dies nicht, wird die Struktur der Menleiste nicht
	 * verndert.
	 */
	private void ladeMenuLeisteAllgemein()
	{
		boolean aufgebautMitNetzweitenEinstellungen = false;

		_debug.fine("==>LeistenInitialisierer: Versuche das Men mit den "
				+ "allgemeinen, netzweiten Einstellungen aufzubauen...");

		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungenSpeicherNetzweit = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungenSpeicherNetzweit != null)
		{
			String xmlStruktur = null;

			xmlStruktur = einstellungenSpeicherNetzweit
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

			if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
			{
				_strukturMenuLeiste = new MenuSymbolLeisteDaten();
				_strukturMenuLeiste.einlesenXmlAusString(xmlStruktur);

				darstellenMenuLeiste();
				_debug.fine("==>LeistenInitialisierer: Das Men mit den allgemeinen, "
						+ "Netzweiten Einstellungen wurde aufgebaut.");
				aufgebautMitNetzweitenEinstellungen = true;
			} else
			{
				String msg = "";
				msg = msg + "Das Men aus den allgemeinen, netzweiten Einstellungen\n";
				msg = msg
						+ "konnte aufgrund der fehlenden XML-Struktur der Menleiste\n";
				msg = msg + "nicht aufgebaut werden.\n";
				msg = msg
						+ "Mit Hilfe des Meneditors kann eine solche Struktur in den\n";
				msg = msg
						+ "allgemeinen, netzweiten Einstellungen hinterlegt werden.\n";
				msg = msg + this.getClass().getName() + "::ladeMenuLeisteAllgemein";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));

			}
		} else
		{
			String msg = "";
			msg = msg
					+ "Das Men aus den allgemeinen, netzweiten Einstellungen konnte\n";
			msg = msg + "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
			msg = msg
					+ "netzweiten, allgemeinen Einstellungen steht z.Zt. nicht zur\n";
			msg = msg + "Verfgung.\n";
			msg = msg
					+ "Stellen Sie sicher, dass der DaV erreichbar ist und korrekt\n";
			msg = msg + "arbeitet.\n";
			msg = msg + this.getClass().getName() + "::ladeMenuLeisteAllgemein";
			_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
		}

		if (!aufgebautMitNetzweitenEinstellungen)
		{
			EinstellungenSpeicher einstellungenSpeicherLokal = esw
					.getEinstellungen(EinstellungsArt.allgemeinLokal);

			if (einstellungenSpeicherLokal != null)
			{
				String xmlStruktur = null;

				xmlStruktur = einstellungenSpeicherLokal
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);
				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_strukturMenuLeiste = new MenuSymbolLeisteDaten();
					_strukturMenuLeiste.einlesenXmlAusString(xmlStruktur);

					darstellenMenuLeiste();
					_debug
							.fine("==>LeistenInitialisierer: Das Men mit den allgemeinen, "
									+ "lokalen Einstellungen wurde aufgebaut.");
				} else
				{
					String msg = "";
					msg = msg + "Das Men aus den allgemeinen, lokalen Einstellungen\n";
					msg = msg
							+ "konnte aufgrund der fehlenden XML-Struktur der Menleiste\n";
					msg = msg + "nicht aufgebaut werden.\n";
					msg = msg
							+ "Mit Hilfe des Meneditors kann eine solche Struktur in den\n";
					msg = msg + "allgemeinen, lokalen Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeMenuLeisteAllgemein";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}
			} else
			{
				String msg = "";
				msg = msg
						+ "Das Men aus den allgemeinen, lokalen Einstellungen konnte\n";
				msg = msg
						+ "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
				msg = msg
						+ "lokalen, allgemeinen Einstellungen steht z.Zt. nicht zur\n";
				msg = msg + "Verfgung.\n";
				msg = msg + "Stellen Sie sicher, dass der Pfad zum lokalen\n";
				msg = msg
						+ "Einstellungenspeicher existiert und alle Rechte zum Zugriff\n";
				msg = msg + "auf die dortigen Dateien verfgbar sind.";
				msg = msg + this.getClass().getName() + "::ladeMenuLeisteAllgemein";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));

			}
		}

	}

	/**
	 * Versucht das Einlesen der Menleistenstruktur fr den aktuell angemeldeten
	 * Benutzer. Glckt dies nicht wird versucht, die in den allgemeinen,
	 * netzweiten Einstellungen vorhandene Menleistenstruktur zu laden.
	 */
	private void ladeMenuLeisteBenutzer()
	{
		boolean aufgebaut = false;

		String name = BenutzerAngemeldet.getInstanz().getName();

		if (name != null)
		{
			_debug.info("==>LeistenInitialisierer: Versuche das Men fr Benutzer "
					+ name + " aufzubauen...");
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicherNetzweit = esw
					.getEinstellungen(EinstellungsArt.benutzerNetzweit);

			if (einstellungenSpeicherNetzweit != null)
			{
				String xmlStruktur = null;

				xmlStruktur = einstellungenSpeicherNetzweit
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_strukturMenuLeiste = new MenuSymbolLeisteDaten();
					_strukturMenuLeiste.einlesenXmlAusString(xmlStruktur);

					darstellenMenuLeiste();
					_debug.fine("==>LeistenInitialisierer: Das Men fr Benutzer " + name
							+ "wurde mit den netzweiten Einstellungen aufgebaut...");
					aufgebaut = true;
				} else
				{
					String msg = "";
					msg = msg
							+ "Das Men aus den benutzerspezifischen, netzweiten Einstellungen\n";
					msg = msg
							+ "konnte aufgrund der fehlenden XML-Struktur der Menleiste fr\n";
					msg = msg + "den Benutzer " + name + " nicht aufgebaut werden.\n";
					msg = msg
							+ "Mit Hilfe des Meneditors kann eine solche Struktur in den\n";
					msg = msg
							+ "benutzerspezifschen, netzweiten Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeMenuLeisteBenutzer";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}

			} else
			{
				String msg = "";
				msg = msg
						+ "Das Men aus den benutzerspezifischen, netzweiten Einstellungen konnte\n";
				msg = msg
						+ "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
				msg = msg
						+ "netzweiten, benutzerspezifischen Einstellungen fr den Benutzer "
						+ name + "\n";
				msg = msg + "steht z.Zt. nicht zur Verfgung.\n";
				msg = msg
						+ "Stellen Sie sicher, dass der DaV erreichbar ist und korrekt\n";
				msg = msg + "arbeitet.\n";
				msg = msg + this.getClass().getName() + "::ladeMenuLeisteBenutzer";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
			}

		}

		/*
		 * Lokale Einstellungen
		 */
		if (!aufgebaut)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicherLokal = esw
					.getEinstellungen(EinstellungsArt.benutzerLokal);

			if (einstellungenSpeicherLokal != null)
			{
				String xmlStruktur = einstellungenSpeicherLokal
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);
				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_strukturMenuLeiste = new MenuSymbolLeisteDaten();
					_strukturMenuLeiste.einlesenXmlAusString(xmlStruktur);

					darstellenMenuLeiste();
					_debug.fine("==>LeistenInitialisierer: Das Men fr Benutzer " + name
							+ "wurde mit den lokalen Einstellungenaufgebaut...");
					aufgebaut = true;
				} else
				{
					String msg = "";
					msg = msg
							+ "Das Men aus den benutzerspezifischen, lokalen Einstellungen\n";
					msg = msg
							+ "konnte aufgrund der fehlenden XML-Struktur der Menleiste fr\n";
					msg = msg + "den Benutzer " + name + " nicht aufgebaut werden.\n";
					msg = msg
							+ "Mit Hilfe des Meneditors kann eine solche Struktur in den\n";
					msg = msg
							+ "benutzerspezifschen, lokalen Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeMenuLeisteBenutzer";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}
			} else
			{
				String msg = "";
				msg = msg
						+ "Das Men aus den benutzerspezifischen, lokalen Einstellungen konnte\n";
				msg = msg
						+ "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
				msg = msg
						+ "lokalen, benutzerspezifischen Einstellungen fr den Benutzer "
						+ name + "\n";
				msg = msg + "steht z.Zt. nicht zur Verfgung.\n";
				msg = msg + "Stellen Sie sicher, dass der Pfad zum lokalen\n";
				msg = msg
						+ "Einstellungenspeicher existiert und alle Rechte zum Zugriff\n";
				msg = msg + "auf die dortigen Dateien verfgbar sind.\n";
				msg = msg + "Fr den Benutzer " + name
						+ " muss dort eine XML-Datei mit\n";
				msg = msg
						+ "dessen Namen existieren - ist dies nicht der Fall, kann mit\n";
				msg = msg
						+ "Hilfe des Meneditors eine solche Datei, durch Speichern in\n";
				msg = msg
						+ "lokalen Einstellungen fr den jeweiligen Benutzer, angelegt werden.";
				msg = msg + this.getClass().getName() + "::ladeMenuLeisteBenutzer";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
			}

			if (!aufgebaut)
			{
				_debug.fine("==>LeistenInitialisierer: Das Men fr Benutzer " + name
						+ "konnte nicht aufgebaut werden. Der Benutzer existiert nicht.");

				String msg = "";
				msg = msg
						+ "Leisteninitalisierer: Es konnte keine Menleiste fr den Benutzer "
						+ name + "\n";
				msg = msg
						+ "aufgebaut werden. Versuche die in den allgemeinen Einstellungen definierte\n";
				msg = msg + "Menleistenstruktur zu laden.\n";
				msg = msg + this.getClass().getName() + "::ladeMenuLeisteBenutzer";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));

				ladeMenuLeisteAllgemein();
			}
		}

	}

	/**
	 * Ldt die in den allgemeinen, netzweiten Einstellungen vorhandene
	 * Symbolleistenstruktur. Glckt dies nicht, wird die Struktur der
	 * Symbolleiste nicht verndert.
	 */
	private void ladeSymbolLeisteAllgemein()
	{
		boolean aufgebaut = false;

		_debug.fine("==>LeistenInitialisierer: Versuche die Symbolleiste mit den "
				+ "allgemeinen, netzweiten Einstellungen aufzubauen...");

		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungenSpeicherNetzweit = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);
		if (einstellungenSpeicherNetzweit != null)
		{
			String xmlStruktur = null;

			xmlStruktur = einstellungenSpeicherNetzweit
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

			if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
			{
				_strukturSymbolLeiste = new MenuSymbolLeisteDaten();
				_strukturSymbolLeiste.einlesenXmlAusString(xmlStruktur);

				darstellenSymbolLeiste();
				_debug
						.fine("==>LeistenInitialisierer: Die Symbolleiste mit den allgemeinen, "
								+ "netzweiten Einstellungen wurde aufgebaut.");

				aufgebaut = true;
			} else
			{
				String msg = "";
				msg = msg
						+ "Die Symbolleiste aus den allgemeinen, netzweiten Einstellungen\n";
				msg = msg
						+ "konnte aufgrund der fehlenden XML-Struktur der Symbolleiste\n";
				msg = msg + "nicht aufgebaut werden.\n";
				msg = msg
						+ "Mit Hilfe des Symbolleistenditors kann eine solche Struktur in den\n";
				msg = msg
						+ "allgemeinen, netzweiten Einstellungen hinterlegt werden.\n";
				msg = msg + "" + this.getClass().getName()
						+ "::ladeSymbolLeisteAllgemein";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
			}

		} else
		{
			_debug.fine("==>LeistenInitialisierer: Das Men mit den allgemeinen, "
					+ "Netzweiten Einstellungen konnte nicht aufgebaut werden.");

			String msg = "";
			msg = msg
					+ "Die Symbolleiste aus den allgemeinen, netzweiten Einstellungen konnte\n";
			msg = msg + "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
			msg = msg
					+ "netzweiten, allgemeinen Einstellungen steht z.Zt. nicht zur\n";
			msg = msg + "Verfgung.\n";
			msg = msg
					+ "Stellen Sie sicher, dass der DaV erreichbar ist und korrekt\n";
			msg = msg + "arbeitet.\n";
			msg = msg + this.getClass().getName() + "::ladeSymbolLeisteAllgemein";
			_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
		}

		/*
		 * Aus den lokalen Einstellungen aufbauen
		 */
		if (!aufgebaut)
		{
			EinstellungenSpeicher einstellungenSpeicherLokal = esw
					.getEinstellungen(EinstellungsArt.allgemeinLokal);
			if (einstellungenSpeicherLokal != null)
			{
				String xmlStruktur = einstellungenSpeicherLokal
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_strukturSymbolLeiste = new MenuSymbolLeisteDaten();
					_strukturSymbolLeiste.einlesenXmlAusString(xmlStruktur);

					darstellenSymbolLeiste();
					_debug
							.fine("==>LeistenInitialisierer: Die Symbolleiste mit den allgemeinen, "
									+ "lokalen Einstellungen wurde aufgebaut.");

					aufgebaut = true;
				} else
				{
					String msg = "";
					msg = msg
							+ "Die Symbolleiste aus den allgemeinen, lokalen Einstellungen konnte aufgrund der\n";
					msg = msg
							+ "fehlenden XML-Struktur der Menleiste nicht aufgebaut werden.\n";
					msg = msg
							+ "Mit Hilfe des Meneditors kann eine solche Struktur in den allgemeinen, lokalen\n";
					msg = msg + "Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeSymbolLeisteallgemein";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}
			} else
			{
				String msg = "";
				msg = msg
						+ "Die Symbolleiste aus den allgemeinen, lokalen Einstellungen konnte nicht \n";
				msg = msg
						+ "aufgebaut werden. Der Einstellungenspeicher mit den lokalen, allgemeinen \n";
				msg = msg + "Einstellungen steht z.Zt. nicht zur Verfgung.\n";
				msg = msg
						+ "Stellen Sie sicher, dass der Pfad zum lokalen Einstellungenspeicher existiert\n";
				msg = msg
						+ "und alle Rechte zum Zugriff auf die dortigen Dateien verfgbar sind. Mit Hilfe\n";
				msg = msg
						+ "des Symbolleistenediors kann eine Symbolleistenstruktur in den allgemeinen,\n";
				msg = msg + "lokalen Einstellungen hinterlegt werden.\n";
				msg = msg + this.getClass().getName() + "::ladeSymbolLeisteallgemein";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));

			}
		}

	}

	/**
	 * Versucht das Einlesen der Symbolleistenstruktur fr den aktuell
	 * angemeldeten Benutzer. Glckt dies nicht wird versucht, die in den
	 * allgemeinen, netzweiten Einstellungen vorhandene Symbolleistenstruktur zu
	 * laden.
	 */
	private void ladeSymbolLeisteBenutzer()
	{
		boolean aufgebaut = false;

		String name = BenutzerAngemeldet.getInstanz().getName();
		if (name != null)
		{
			_debug.fine("==>LeistenInitialisierer: Versuche das Men fr Benutzer "
					+ name + "aufzubauen...");
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicherBenutzerNetzweit = esw
					.getEinstellungen(EinstellungsArt.benutzerNetzweit);
			if (einstellungenSpeicherBenutzerNetzweit != null)
			{
				String xmlStruktur = null;

				xmlStruktur = einstellungenSpeicherBenutzerNetzweit
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_strukturSymbolLeiste = new MenuSymbolLeisteDaten();
					_strukturSymbolLeiste.einlesenXmlAusString(xmlStruktur);

					darstellenSymbolLeiste();

					_debug
							.fine("==>LeistenInitialisierer: Die Symbolleiste fr Benutzer "
									+ name
									+ " wurde aus den netzweiten Einstellungen aufgebaut...");

					aufgebaut = true;

				} else
				{
					String msg = "";
					msg = msg
							+ "Die Symbolleiste aus den benutzerspezifischen, netzweiten Einstellungen\n";
					msg = msg
							+ "konnte aufgrund der fehlenden XML-Struktur der Symbolleiste fr\n";
					msg = msg + "den Benutzer " + name + " nicht aufgebaut werden.\n";
					msg = msg
							+ "Mit Hilfe des Meneditors kann eine solche Struktur in den\n";
					msg = msg
							+ "benutzerspezifschen, netzweiten Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeMenuLeisteBenutzer";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}
			} else
			{
				String msg = "";
				msg = msg
						+ "Die Symbolleiste aus den benutzerspezifischen, netzweiten Einstellungen konnte\n";
				msg = msg
						+ "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
				msg = msg
						+ "netzweiten, benutzerspezifischen Einstellungen fr den Benutzer "
						+ name + "\n";
				msg = msg + "steht z.Zt. nicht zur Verfgung.\n";
				msg = msg
						+ "Stellen Sie sicher, dass der DaV erreichbar ist und korrekt\n";
				msg = msg + "arbeitet.\n";
				msg = msg + this.getClass().getName() + "::ladeSymbolLeisteBenutzer";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
			}

			/*
			 * Aufbauen aus den lokalen Einstellungen
			 */
			if (!aufgebaut)
			{
				EinstellungenSpeicher einstellungenSpeicherBenutzerLokal = esw
						.getEinstellungen(EinstellungsArt.benutzerLokal);
				if (einstellungenSpeicherBenutzerLokal != null)
				{
					String xmlStruktur = einstellungenSpeicherBenutzerLokal
							.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

					if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
					{
						_strukturSymbolLeiste = new MenuSymbolLeisteDaten();
						_strukturSymbolLeiste.einlesenXmlAusString(xmlStruktur);

						darstellenSymbolLeiste();

						_debug
								.fine("==>LeistenInitialisierer: Die Symbolleiste fr Benutzer "
										+ name
										+ " wurde aus den lokalen Einstellungen aufgebaut...");

						aufgebaut = true;

					} else
					{
						String msg = "";
						msg = msg
								+ "Die Symbolleiste aus den benutzerspezifschen, lokalen Einstellungen fr den\n";
						msg = msg + "Benutzer " + name
								+ " konnte aufgrund der fehlenden XML-Struktur der \n";
						msg = msg + "Menleiste nicht aufgebaut werden.\n";
						msg = msg
								+ "Mit Hilfe des Meneditors kann eine solche Struktur in den benutzerspezifischen,\n";
						msg = msg + "lokalen Einstellungen hinterlegt werden.\n";
						msg = msg + this.getClass().getName()
								+ "::ladeSymbolLeisteallgemein";
						_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
					}
				} else
				{
					String msg = "";
					msg = msg
							+ "Die Symbolleiste aus den benutzerspezifischen, lokalen Einstellungen konnte\n";
					msg = msg
							+ "fr den Benutzer <angemeldeter Benutzer> nicht aufgebaut werden. Der\n";
					msg = msg
							+ "Einstellungenspeicher mit den lokalen, allgemeinen Einstellungen steht z.Zt. \n";
					msg = msg + "nicht zur Verfgung.\n";
					msg = msg
							+ "Stellen Sie sicher, dass der Pfad zum lokalen Einstellungenspeicher existiert\n";
					msg = msg
							+ "und alle Rechte zum Zugriff auf die dortigen Dateien verfgbar sind. Mit Hilfe\n";
					msg = msg
							+ "des Symbolleistenediors kann eine Symbolleistenstruktur in den\n";
					msg = msg
							+ "benutzerpsezifischen, lokalen Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeSymbolLeisteallgemein";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}
			}

			if (!aufgebaut)
			{
				String msg = "";
				msg = msg
						+ "Leisteninitalisierer: Es konnte keine Symbolleiste fr den Benutzer "
						+ name + "\n";
				msg = msg
						+ "aufgebaut werden. Versuche die in den allgemeinen Einstellungen definierte\n";
				msg = msg + "Symbolleistenstruktur zu laden.\n";
				msg = msg + this.getClass().getName() + "::ladeSymbolLeisteBenutzer";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));

				ladeSymbolLeisteAllgemein();
			}

		} else
		{
			_debug.fine("==>LeistenInitialisierer: Die Symbolleiste fr Benutzer "
					+ name
					+ "konnte nicht aufgebaut werden. Der Benutzer existiert nicht.");
		}
	}

	/**
	 * Ldt die in den allgemeinen, netzweiten Einstellungen vorhandene
	 * Statusleistenstruktur. Glckt dies nicht, wird die Struktur der
	 * Statusleiste nicht verndert.
	 */
	private void ladeStatusZeileAllgemein()
	{
		boolean aufgebaut = false;

		_debug.fine("==>LeistenInitialisierer: Versuche die Statusleiste mit den "
				+ "netzweiten, allgemeinen Einstellungen aufzubauen...");

		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungenSpeicherNetzweit = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungenSpeicherNetzweit != null)
		{
			String xmlStruktur = einstellungenSpeicherNetzweit
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

			if (xmlStruktur != null)
			{
				_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
				// StatusLeisteGenerator.getInstanz().generiereStatusLeiste();
				// _statusLeisteVerwaltung.aktualisieren();
				darstellenStatusLeiste();
				aufgebaut = true;
			} else
			{
				String msg = "";
				msg = msg
						+ "Die Statusleiste aus den allgemeinen, netzweiten Einstellungen\n";
				msg = msg
						+ "konnte aufgrund der fehlenden XML-Struktur der Statusleiste\n";
				msg = msg + "nicht aufgebaut werden.\n";
				msg = msg
						+ "Mit Hilfe des Statusleisteneditors kann eine solche Struktur in den\n";
				msg = msg
						+ "allgemeinen, netzweiten Einstellungen hinterlegt werden.\n";
				msg = msg + this.getClass().getName() + "::ladeStatusZeileAllgemein";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
			}
		} else
		{
			String msg = "";
			msg = msg
					+ "Die Statusleiste aus den allgemeinen, netzweiten Einstellungen konnte\n";
			msg = msg + "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
			msg = msg
					+ "netzweiten, allgemeinen Einstellungen steht z.Zt. nicht zur\n";
			msg = msg + "Verfgung.\n";
			msg = msg
					+ "Stellen Sie sicher, dass der DaV erreichbar ist und korrekt\n";
			msg = msg + "arbeitet.\n";
			msg = msg + this.getClass().getName() + "::ladeStatusZeileAllgemein";
			_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
		}

		/*
		 * Lokale Einstellungen
		 */
		if (!aufgebaut)
		{
			EinstellungenSpeicher einstellungenSpeicherLokal = esw
					.getEinstellungen(EinstellungsArt.allgemeinLokal);
			if (einstellungenSpeicherLokal != null)
			{
				String xmlStruktur = einstellungenSpeicherLokal
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
					darstellenStatusLeiste();
					aufgebaut = true;
				} else
				{
					String msg = "";
					msg = msg
							+ "Die Statusleiste aus den allgemeinen, lokalen Einstellungen konnte aufgrund der\n";
					msg = msg
							+ "fehlenden XML-Struktur der Statusleiste nicht aufgebaut werden.\n";
					msg = msg
							+ "Mit Hilfe des Statusleisteneditors kann eine solche Struktur in den allgemeinen,\n";
					msg = msg + "lokalen Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeSymbolLeisteallgemein";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}
			} else
			{
				String msg = "";
				msg = msg
						+ "Die Statusleiste aus den allgemeinen, lokalen Einstellungen konnte nicht \n";
				msg = msg
						+ "aufgebaut werden. Der Einstellungenspeicher mit den lokalen, allgemeinen \n";
				msg = msg + "Einstellungen steht z.Zt. nicht zur Verfgung.\n";
				msg = msg
						+ "Stellen Sie sicher, dass der Pfad zum lokalen Einstellungenspeicher existiert \n";
				msg = msg
						+ "und alle Rechte zum Zugriff auf die dortigen Dateien verfgbar sind. Mit Hilfe\n";
				msg = msg
						+ "des Statusleistenediors kann eine Statusleistenstruktur in den allgemeinen,\n";
				msg = msg + "lokalen Einstellungen hinterlegt werden.\n";
				msg = msg + this.getClass().getName() + "::ladeSymbolLeisteallgemein";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
			}
		}

		if (!aufgebaut)
		{
			_debug
					.fine("==>LeistenInitialisierer: Die Statusleiste konnte nicht aufgebaut werden. "
							+ "Es sind keine XML-Strukturdaten verfgbar.");
		}

	}

	/**
	 * Versucht das Einlesen der Statusleistenstruktur fr den aktuell
	 * angemeldeten Benutzer. Glckt dies nicht wird versucht, die in den
	 * allgemeinen, netzweiten Einstellungen vorhandene Statusleistenstruktur zu
	 * laden.
	 */
	private void ladeStatusZeileBenutzer()
	{
		boolean aufgebaut = false;

		String name = BenutzerAngemeldet.getInstanz().getName();

		_debug
				.fine("==>LeistenInitialisierer: Versuche die Statusleiste fr Benutzer "
						+ name + "aufzubauen...");

		if (name != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicherNetzweit = esw
					.getEinstellungen(EinstellungsArt.benutzerNetzweit);

			if (einstellungenSpeicherNetzweit != null)
			{
				String xmlStruktur = einstellungenSpeicherNetzweit
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

				if (xmlStruktur != null)
				{
					_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
					// StatusLeisteGenerator.getInstanz().generiereStatusLeiste();
					// _statusLeisteVerwaltung.aktualisieren();
					darstellenStatusLeiste();
					aufgebaut = true;
				} else
				{
					String msg = "";
					msg = msg
							+ "Die Statusleiste aus den benutzerspezifischen, netzweiten Einstellungen\n";
					msg = msg
							+ "konnte aufgrund der fehlenden XML-Struktur der Statusleiste fr\n";
					msg = msg + "den Benutzer " + name + " nicht aufgebaut werden.\n";
					msg = msg
							+ "Mit Hilfe des Statusleisteneditors kann eine solche Struktur in den\n";
					msg = msg
							+ "benutzerspezifschen, netzweiten Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeStatusZeileBenutzer";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}
			} else
			{
				String msg = "";
				msg = msg
						+ "Die Statusleiste aus den benutzerspezifischen, netzweiten Einstellungen konnte\n";
				msg = msg
						+ "nicht aufgebaut werden. Der Einstellungenspeicher mit den\n";
				msg = msg
						+ "netzweiten, benutzerspezifischen Einstellungen fr den Benutzer "
						+ name + "\n";
				msg = msg + "steht z.Zt. nicht zur Verfgung.\n";
				msg = msg
						+ "Stellen Sie sicher, dass der DaV erreichbar ist und korrekt\n";
				msg = msg + "arbeitet.\n";
				msg = msg + this.getClass().getName() + "::ladeStatusZeileBenutzer";
				_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
			}

			/*
			 * Aufbauen aus den Lokalen Einstellungen
			 */
			if (!aufgebaut)
			{
				EinstellungenSpeicher einstellungenSpeicherBenutzerLokal = esw
						.getEinstellungen(EinstellungsArt.benutzerLokal);
				if (einstellungenSpeicherBenutzerLokal != null)
				{
					String xmlStruktur = einstellungenSpeicherBenutzerLokal
							.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

					if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
					{
						_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
						darstellenStatusLeiste();
						aufgebaut = true;
					} else
					{
						String msg = "";
						msg = msg
								+ "Die Statusleiste aus den benutzerspezifschen, lokalen Einstellungen fr den\n";
						msg = msg
								+ "Benutzer <angemeldeter Benutzer> konnte aufgrund der fehlenden XML-Struktur der\n";
						msg = msg + "Statusleiste nicht aufgebaut werden.\n";
						msg = msg
								+ "Mit Hilfe des Statusleisteneditors kann eine solche Struktur in den\n";
						msg = msg
								+ "benutzerspezifischen, lokalen Einstellungen hinterlegt werden.\n";
						msg = msg + this.getClass().getName()
								+ "::ladeSymbolLeisteallgemein";
						_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
					}
				} else
				{
					String msg = "";
					msg = msg
							+ "Die Statusleiste aus den benutzerspezifischen, lokalen Einstellungen konnte\n";
					msg = msg
							+ "fr den Benutzer <angemeldeter Benutzer> nicht aufgebaut werden. Der\n";
					msg = msg
							+ "Einstellungenspeicher mit den lokalen, allgemeinen Einstellungen steht z.Zt.\n";
					msg = msg + "nicht zur Verfgung.\n";
					msg = msg
							+ "Stellen Sie sicher, dass der Pfad zum lokalen Einstellungenspeicher existiert\n";
					msg = msg
							+ "und alle Rechte zum Zugriff auf die dortigen Dateien verfgbar sind. Mit Hilfe\n";
					msg = msg
							+ "des Statusleistenediors kann eine Statusleistenstruktur in den\n";
					msg = msg
							+ "benutzerpsezifischen, lokalen Einstellungen hinterlegt werden.\n";
					msg = msg + this.getClass().getName() + "::ladeSymbolLeisteallgemein";
					_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
				}

			}

		}

		if (!aufgebaut)
		{
			String msg = "";
			msg = msg
					+ "Leisteninitalisierer: Es konnte keine Statusleiste fr den Benutzer "
					+ name + "\n";
			msg = msg
					+ "aufgebaut werden. Versuche die in den allgemeinen Einstellungen definierte\n";
			msg = msg + "Statusleistenstruktur zu laden.\n";
			msg = msg + this.getClass().getName() + "::ladeStatusZeileBenutzer";
			_log.log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));

			ladeStatusZeileAllgemein();
		} else
		{
			_debug.fine("==>LeistenInitialisierer: Die Statusleiste fr Benutzer "
					+ name
					+ "konnte nicht aufgebaut werden. Der Benutzer existiert nicht.");
		}
	}

	/**
	 * Ldt die Strukturen der Leisten.
	 */
	private void ladeLeistenStrukturen()
	{
		System.out.println("Lade Leisten Strukturen");
		ladeMenuLeisteBenutzer();
		ladeSymbolLeisteBenutzer();
		ladeStatusZeileBenutzer();

		_initialeStrukturGeladen = true;
	}

	/**
	 * Diese Methode wertet die Kommandozeilenparameter
	 * <code>-menueXmlDatei</code> sowie <code>-symbolXmlDatei</code> aus welche
	 * einen Pfad beinhalten und auf die Struktur von einer Men- bzw.
	 * Symbolleiste verweisen. Diese werden geladen, bevor alle Plug-Ins ihre
	 * Elemente fr Men- und Symbolleisten bekannt gegeben haben, somit stehen
	 * nur die Basis-Actions des Rahmenwerkes zur Verfgung. Die ber die
	 * Parameter angegebenen Pfade mssen absolut sein!<br>
	 * <br>
	 * Stehen die angegebenen Dateien nicht zur Verfgung, wird dies im Error-Log
	 * vermerkt. <br>
	 * <br>
	 * Haben alle Plug-Ins ihre Elemente bekannt gegeben, wird die im
	 * {@link EinstellungenSpeicher} hinterlegte Struktur fr Men- und
	 * Symbolleiste verwendet.
	 */
	private void aktionVorLadenMenuSymbolLeisteAusXmlDateien()
	{
		Kommandozeilenparameter cmdPar = Kommandozeilenparameter.getInstanz();
		KmdParameter parMenueXmlDatei = cmdPar
				.getKommandoZeilenParameter("-menueXmlDatei");
		KmdParameter parSymbolXmlDatei = cmdPar
				.getKommandoZeilenParameter("-symbolXmlDatei");

		/*
		 * Parameter -menueXmlDatei auswerten
		 */
		if (parMenueXmlDatei != null)
		{
			String pfadMenueXmlDatei = parMenueXmlDatei.getWert();
			try
			{
				ladeMenuStrukturAusXmlDatei(pfadMenueXmlDatei);
			} catch (IllegalArgumentException iae)
			{
				String meldung = "Es kann keine Menleiste aus dem angegebenen Parameter "
						+ "-menueXmlDatei ("
						+ parMenueXmlDatei.toString()
						+ ") aufgebaut werden. Die Datei exitiert nicht.";

				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID, meldung, iae));
			}
		}

		/*
		 * Parameter -symbolXmlDatei auswerten
		 */
		if (parSymbolXmlDatei != null)
		{
			String pfadSymbolXmlDatei = parSymbolXmlDatei.getWert();
			try
			{
				ladeSymbolleistenStrukturAusXmlDatei(pfadSymbolXmlDatei);
			} catch (IllegalArgumentException iae)
			{
				String meldung = "Es kann keine Symbolleiste aus dem angegebenen Parameter "
						+ "-symbolXmlDatei ("
						+ parSymbolXmlDatei.toString()
						+ ") aufgebaut werden. Die Datei existiert nicht.";

				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID, meldung, iae));
			}
		}
	}

	/**
	 * Ldt die XML-Struktur der Menleiste aus einer Datei. Diese Methode wird
	 * direkt nach dem Start des Rahmenwerkes, bevor alle Plug-Ins ihre
	 * Menelemente bekannt gegeben haben, geladen. Es stehen somit nur die
	 * Standard Eclipse-Actions zur Verfgung.
	 * 
	 * @param pfadXmlDatei
	 *          Pfad zur XML-Datei, aus deren Struktur das Men aufgebaut werden
	 *          soll.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn die angegebene Datei nicht existiert.
	 */
	private void ladeMenuStrukturAusXmlDatei(String pfadXmlDatei)
			throws IllegalArgumentException
	{
		File file = new File(pfadXmlDatei);
		if ((file.exists()) && (file.isFile()))
		{
			_strukturMenuLeiste = new MenuSymbolLeisteDaten();
			_strukturMenuLeiste.einlesenXmlAusDatei(pfadXmlDatei);
			darstellenMenuLeiste();
		} else
		{
			throw new IllegalArgumentException("Die Datei " + pfadXmlDatei
					+ " existiert nicht! Kann keine Menstruktur darstellen.");
		}
	}

	/**
	 * Ldt die XML-Struktur der Symbolleiste aus einer Datei. Diese Methode wird
	 * direkt nach dem Start des Rahmenwerkes, bevor alle Plug-Ins ihre
	 * Symbolelemente bekannt gegeben haben, geladen. Es stehen somit nur die
	 * Standard Eclipse-Actions zur Verfgung.
	 * 
	 * @param pfadXmlDatei
	 *          Pfad zur XML-Datei, aus deren Struktur die Symbolleiste aufgebaut
	 *          werden soll.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn die angegebene Datei nicht existiert.
	 */
	private void ladeSymbolleistenStrukturAusXmlDatei(String pfadXmlDatei)
			throws IllegalArgumentException
	{
		File file = new File(pfadXmlDatei);
		if ((file.exists()) && (file.isFile()))
		{
			_strukturSymbolLeiste = new MenuSymbolLeisteDaten();
			_strukturSymbolLeiste.einlesenXmlAusDatei(pfadXmlDatei);
			darstellenSymbolLeiste();
		} else
		{
			throw new IllegalArgumentException("Die Datei " + pfadXmlDatei
					+ " existiert nicht! Kann keine Menstruktur darstellen.");
		}
	}

	/**
	 * Prft, ob in dem durch _maxZeit definierten vergangenen Zeitraum keine neue
	 * Statusleistenelemente zur {@link StatusLeisteVerwaltung} hinzukamen.
	 * 
	 * @return Liefert <code>true</code>, wenn keine Statusleistenelemente mehr
	 *         hinzukamen.
	 */
	private boolean pruefeStatusLeiste()
	{
		boolean re = false;

		int size = _statusLeisteVerwaltung.getVerfuegbareStatusLeistenElemente()
				.size();

		if (size != _anzStatusElemente)
		{
			_anzStatusElemente = size;
			_zeitStatusElemente = new Date().getTime();
		}

		long zeitPktJetzt = new Date().getTime();
		if ((zeitPktJetzt - _zeitStatusElemente) > _maxZeit)
		{
			re = true;
		}

		return re;
	}

	/**
	 * Prft, ob in dem durch _maxZeit definierten vergangenen Zeitraum keine neue
	 * Elemente zur {@link ElementRegistrierung} hinzukamen.
	 * 
	 * @return Liefert <code>false</code>, wenn keine neuen Elemente im
	 *         vergangenen Zeitraum hinzu kamen, sonst <code>true</code>.
	 */
	private boolean pruefeElemente()
	{
		boolean re = false;

		int anzKomb = 0;
		int anzGlob = 0;

		List<String> pluginIDs = _elementRegistrierung.getPluginIDs();
		Iterator<String> it = pluginIDs.iterator();
		while (it.hasNext())
		{
			String plId = it.next();
			anzKomb = anzKomb
					+ _elementRegistrierung.getPluginDeskriptorenKombiniert(plId).size();
			anzGlob = anzGlob
					+ _elementRegistrierung.getPluginAktionDeskriptorenGlobal(plId)
							.size();
		}

		long zeitPktJetzt = new Date().getTime();

		if (anzKomb != _anzDeskriptorenKombiniert)
		{
			_anzDeskriptorenKombiniert = anzKomb;
			_zeitDeskiptorenKombiniert = zeitPktJetzt;
		}

		if (anzGlob != _anzGlobalerAktionen)
		{
			_anzGlobalerAktionen = anzGlob;
			_zeitGlobalerAktionen = zeitPktJetzt;
		}

		boolean komb = false;
		boolean glob = false;

		if ((zeitPktJetzt - _zeitDeskiptorenKombiniert) > _maxZeit)
		{
			komb = true;
		}
		if ((zeitPktJetzt - _zeitGlobalerAktionen) > _maxZeit)
		{
			glob = true;
		}

		if (komb && glob)
		{
			re = true;
		}

		return re;
	}

	@Override
	public void run()
	{
		_debug.fine("Thread zur Initialisierung der Leisten wurde gestartet.");

		if (DatenSchnellHolen)
		{
			quickRun();
		} else
		{
			while (_laufeThread)
			{
				try
				{
					Thread.sleep(1000);
					// System.out.println("Nach Schlaffen Thread 1");

					if (!_pruefungenOk)
					{
						if (pruefeStatusLeiste() && pruefeElemente())
						{
							_pruefungenOk = true;
						}
					}

					if (_pruefungenOk)
					{
						String aktName = _benAngemeldet.getName();

						/*
						 * Mit Hilfe des angemeldeten Benutzers dessen Men- und
						 * Symbolleistenstruktur laden.
						 */
						if (aktName != null)
						{

							if ((_alterAngemeldeterBenutzer != null)
									&& (!_alterAngemeldeterBenutzer.equals(aktName)))
							{
								System.out.println("NEU=>" + aktName);
								_alterAngemeldeterBenutzer = aktName;
								Thread.sleep(2000);
								ladeLeistenStrukturen();
							} else if (_alterAngemeldeterBenutzer == null)
							{
								_alterAngemeldeterBenutzer = aktName;
								Thread.sleep(2000);
								ladeLeistenStrukturen();
							}
						}
						/*
						 * Ansonsten allgemeine Sturktur laden
						 */
						else
						{
							if (!_initialeStrukturGeladen)
							{
								ladeMenuLeisteAllgemein();
								ladeSymbolLeisteAllgemein();
								ladeStatusZeileAllgemein();
								_initialeStrukturGeladen = true;
							}
						}

						// _laufeThread = false;
					}

				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

		_debug.fine("Thread zur Initialisierung der Leisten wurde gestoppt.");

	}

	/**
	 * Kurz laufende Variante der RUN-Methode. Sollte verwendet werden, wenn auf
	 * Mens basierend auf dem Erweiterungspunkt org.eclipse.ui.actionSets
	 * gearbeitet wird.
	 */
	private void quickRun()
	{
		_debug.fine("Thread zur Initialisierung der Leisten wurde gestartet.");

		while (_laufeThread)
		{

			String aktName = _benAngemeldet.getName();

			/*
			 * Mit Hilfe des angemeldeten Benutzers dessen Men- und
			 * Symbolleistenstruktur laden.
			 */
			if (aktName != null)
			{

				if ((_alterAngemeldeterBenutzer != null)
						&& (!_alterAngemeldeterBenutzer.equals(aktName)))
				{
					_alterAngemeldeterBenutzer = aktName;
					ladeLeistenStrukturen();
				} else if (_alterAngemeldeterBenutzer == null)
				{
					_alterAngemeldeterBenutzer = aktName;
					ladeLeistenStrukturen();
				}
			}
			/*
			 * Ansonsten allgemeine Sturktur laden
			 */
			else
			{
				if (!_initialeStrukturGeladen)
				{
					ladeMenuLeisteAllgemein();
					ladeSymbolLeisteAllgemein();
					ladeStatusZeileAllgemein();
					_initialeStrukturGeladen = true;
				}
			}

			_laufeThread = false;
		}

		_debug.fine("Thread zur Initialisierung der Leisten wurde gestoppt.");

	}

	@Override
	public void verbindungAufgebaut(DaVVerbindungEreignis e)
	{
			ClientDavInterface con = e.getDaVVerbindung();
			if (con != null)
			{
				ladeLeistenStrukturen();
				// new WartenLaden(2000l);
			}
	}

	@Override
	public void verbindungAbgebaut(DaVVerbindungEreignis e)
	{
		//Hier gibt es momentan nichts zu tun.
	}

	/**
	 * Klasse zum verzgerten Auslsen der Ladens der Leistenstrukturen. Arbeitet
	 * als Thread und nutzt {@link Display#asyncExec(Runnable)} um die Oberflche
	 * zu aktualisieren.
	 * 
	 * @author HofmannJ
	 * 
	 */
	@SuppressWarnings("unused")
	private class WartenLaden implements Runnable
	{
		private long _time = 1000l;

		public WartenLaden(long time)
		{
			this._time = time;
			Thread th = new Thread(this);
			th.setName("WartenLaden");
			th.start();
		}

		@Override
		public void run()
		{
			try
			{
				Thread.sleep(_time);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			} finally
			{
				Runnable run = new Runnable()
				{
					@Override
					public void run()
					{
						ladeLeistenStrukturen();
					}
				};
				Display display = PlatformUI.getWorkbench().getDisplay();
				display.asyncExec(run);
			}
		}

	}

}
