/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.dialogs.PreferencesUtil;

import de.bsvrz.buv.rw.basislib.listendarstellung.TestKTableAenderungsDarstellung;
import de.bsvrz.buv.rw.basislib.menu.editoren.EditorMenuSymbolLeiste;
import de.bsvrz.buv.rw.basislib.menu.editoren.EditorMenuSymbolLeisterInput;
import de.bsvrz.buv.rw.basislib.menu.editoren.EditorOeffnenAktionsSpeicher;
import de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.EnumLeistenArt;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.Seiteneinstellungen;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckVorschauAction;
import de.bsvrz.buv.rw.rw.FensterSpeicher;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten.enumLeistenArt;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;
import de.bsvrz.sys.funclib.debug.Debug;

public class LeistenStarter implements IStartup
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private static final SimpleDateFormat SDF = new SimpleDateFormat("HH:mm:ss");

	@Override
	public void earlyStartup()
	{
		LeistenInitialisierung.getInstanz();

		registriereActions();
		registriereSymbolleistenElemente();
	}

	/**
	 * Registriert die Actions dieses Plug-Ins bei der
	 * {@link ElementRegistrierung}.
	 */
	private void registriereActions()
	{
		/*
		 * Action des Symbolleisteneditors mit Laden-Dialog
		 */
		Action actionSymbolLeistenEditor = new Action()
		{
			public void run()
			{
				/*
				 * Perspective ffnen
				 */
				String perspectiveID = "de.bsvrz.buv.rw.basislib.perspectiveMenuEditor";
				try
				{
					PlatformUI.getWorkbench().showPerspective(perspectiveID,
							PlatformUI.getWorkbench().getActiveWorkbenchWindow());

					MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();
					msld.setzeLeistenArt(enumLeistenArt.symbol);
					EditorMenuSymbolLeisterInput ei = new EditorMenuSymbolLeisterInput(
							msld);
					IWorkbenchPage page = PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getActivePage();
					try
					{
						page.openEditor(ei, "de.bsvrz.buv.rw.basislib.editorMenu");

						IEditorPart editor = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow().getActivePage().findEditor(ei);
						if (editor instanceof EditorMenuSymbolLeiste)
						{
							EditorMenuSymbolLeiste editorMenuSymbolLeiste = (EditorMenuSymbolLeiste) editor;
							editorMenuSymbolLeiste
									.setLadenSpeichernLeistenArt(EnumLeistenArt.symbolLeiste);
							editorMenuSymbolLeiste.aktionLadenGlobal();
						}

					} catch (PartInitException pie)
					{
						_debug.fine("Der Symbolleisteneditor konnte nicht "
								+ "geffnet werden\n\t=> " + pie.toString());
					}

				} catch (WorkbenchException wbe)
				{
					_debug.fine("Die Symbolleisteneditorperspektive "
							+ "konnte nicht geffnet werden\n\t=> " + wbe.toString());
				}
			}
		};

		actionSymbolLeistenEditor.setText("Symbolleisteneditor");
		actionSymbolLeistenEditor
				.setToolTipText("ffnet den Editor zum Bearbeiten der Strukturen "
						+ "von Symbolleisten");
		actionSymbolLeistenEditor.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/menubar.gif"));
		actionSymbolLeistenEditor
				.setId("de.bsvrz.buv.rw.basislib.symbolLeistenEditor");
		EditorOeffnenAktionsSpeicher.getInstanz()
				.setActionSymbolleisteEditorOeffnenDialog(actionSymbolLeistenEditor);
		DeskriptorAktion da = new DeskriptorAktion(Activator.PLUGIN_ID,
				actionSymbolLeistenEditor, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(da);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "des Men-/Symbolleisteneditors konnte nicht "
			// + "gestartet werden\n\t=> " + iae.toString());
		}

		/*
		 * Action des Symbolleisteneditors mit LEERER Darstellung
		 */
		Action actionSymbolLeistenEditorLeer = new Action()
		{
			public void run()
			{
				/*
				 * Perspective ffnen
				 */
				String perspectiveID = "de.bsvrz.buv.rw.basislib.perspectiveMenuEditor";
				try
				{
					PlatformUI.getWorkbench().showPerspective(perspectiveID,
							PlatformUI.getWorkbench().getActiveWorkbenchWindow());

					MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();
					msld.setzeLeistenArt(enumLeistenArt.symbol);
					EditorMenuSymbolLeisterInput ei = new EditorMenuSymbolLeisterInput(
							msld);
					IWorkbenchPage page = PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getActivePage();
					try
					{
						page.openEditor(ei, "de.bsvrz.buv.rw.basislib.editorMenu");

						IEditorPart editor = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow().getActivePage().findEditor(ei);
						if (editor instanceof EditorMenuSymbolLeiste)
						{
							EditorMenuSymbolLeiste editorMenuSymbolLeiste = (EditorMenuSymbolLeiste) editor;
							editorMenuSymbolLeiste
									.setLadenSpeichernLeistenArt(EnumLeistenArt.symbolLeiste);
						}

					} catch (PartInitException pie)
					{
						_debug.finest("Der Symbolleisteneditor konnte nicht "
								+ "geffnet werden\n\t=> " + pie.toString());
					}

				} catch (WorkbenchException wbe)
				{
					_debug.finest("Die Symbolleisteneditorperspektive "
							+ "konnte nicht geffnet werden\n\t=> " + wbe.toString());
				}
			}
		};

		actionSymbolLeistenEditorLeer.setText("Symbolleisteneditor (leer)");
		actionSymbolLeistenEditorLeer
				.setToolTipText("ffnet den Editor zum Bearbeiten der Strukturen "
						+ "von Symbolleisten mit leerer Darstellung");
		actionSymbolLeistenEditorLeer.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/menubar.gif"));
		actionSymbolLeistenEditorLeer
				.setId("de.bsvrz.buv.rw.basislib.symbolLeistenEditorLeer");
		EditorOeffnenAktionsSpeicher.getInstanz().setActionSymbolleisteEditorLeer(
				actionSymbolLeistenEditorLeer);
		DeskriptorAktion daSymbolLeistenEditorLeer = new DeskriptorAktion(
				Activator.PLUGIN_ID, actionSymbolLeistenEditorLeer, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					daSymbolLeistenEditorLeer);
		} catch (IllegalArgumentException iae)
		{
			// _debug
			// .warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "des Men-/Symbolleisteneditors mit leerer Darstellung konnte nicht "
			// + "gestartet werden\n\t=> " + iae.toString());
		}

		/*
		 * Action des Meneditor ffnen
		 */
		Action actionMenuLeistenEditor = new Action()
		{
			public void run()
			{
				/*
				 * Perspective ffnen
				 */
				String perspectiveID = "de.bsvrz.buv.rw.basislib.perspectiveMenuEditor";
				try
				{
					PlatformUI.getWorkbench().showPerspective(perspectiveID,
							PlatformUI.getWorkbench().getActiveWorkbenchWindow());

					MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();
					msld.setzeLeistenArt(enumLeistenArt.menue);
					EditorMenuSymbolLeisterInput ei = new EditorMenuSymbolLeisterInput(
							msld);
					IWorkbenchPage page = PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getActivePage();
					try
					{
						page.openEditor(ei, "de.bsvrz.buv.rw.basislib.editorMenu");

						IEditorPart editor = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow().getActivePage().findEditor(ei);
						if (editor instanceof EditorMenuSymbolLeiste)
						{
							EditorMenuSymbolLeiste editorMenuSymbolLeiste = (EditorMenuSymbolLeiste) editor;
							editorMenuSymbolLeiste
									.setLadenSpeichernLeistenArt(EnumLeistenArt.menuLeiste);
							editorMenuSymbolLeiste.aktionLadenGlobal();
						}

					} catch (PartInitException pie)
					{
						_debug.finest("Der Menleisteneditor konnte nicht "
								+ "geffnet werden\n\t=> " + pie.toString());
					}

				} catch (WorkbenchException wbe)
				{
					_debug.finest("Die Men- und Symbolleisteneditorperspektive "
							+ "konnte nicht geffnet werden\n\t=> " + wbe.toString());
				}
			}
		};

		actionMenuLeistenEditor.setText("Meneditor");
		actionMenuLeistenEditor
				.setToolTipText("ffnet den Editor zum Bearbeiten der Strukturen "
						+ "von Menleisten");
		actionMenuLeistenEditor.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/menu.gif"));
		actionMenuLeistenEditor.setId("de.bsvrz.buv.rw.basislib.menuLeistenEditor");
		EditorOeffnenAktionsSpeicher.getInstanz().setActionMenuEditorOeffnenDialog(
				actionMenuLeistenEditor);
		DeskriptorAktion deskriptorAktionMenuLeistenEditor = new DeskriptorAktion(
				Activator.PLUGIN_ID, actionMenuLeistenEditor, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					deskriptorAktionMenuLeistenEditor);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "des Menleisteneditors konnte nicht "
			// + "registriert werden\n\t=> " + iae.toString());
		}

		/*
		 * Action des Meneditor mit leerer Darstellung ffnen
		 */
		Action actionMenuLeistenEditorLeer = new Action()
		{
			public void run()
			{
				/*
				 * Perspective ffnen
				 */
				String perspectiveID = "de.bsvrz.buv.rw.basislib.perspectiveMenuEditor";
				try
				{
					PlatformUI.getWorkbench().showPerspective(perspectiveID,
							PlatformUI.getWorkbench().getActiveWorkbenchWindow());

					MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();
					msld.setzeLeistenArt(enumLeistenArt.menue);
					EditorMenuSymbolLeisterInput ei = new EditorMenuSymbolLeisterInput(
							msld);
					IWorkbenchPage page = PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getActivePage();
					try
					{
						page.openEditor(ei, "de.bsvrz.buv.rw.basislib.editorMenu");

						IEditorPart editor = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow().getActivePage().findEditor(ei);
						if (editor instanceof EditorMenuSymbolLeiste)
						{
							EditorMenuSymbolLeiste editorMenuSymbolLeiste = (EditorMenuSymbolLeiste) editor;
							editorMenuSymbolLeiste
									.setLadenSpeichernLeistenArt(EnumLeistenArt.menuLeiste);
						}

					} catch (PartInitException pie)
					{
						_debug.finest("Der Menleisteneditor konnte nicht "
								+ "geffnet werden\n\t=> " + pie.toString());
					}

				} catch (WorkbenchException wbe)
				{
					_debug.finest("Die Men- und Symbolleisteneditorperspektive "
							+ "konnte nicht geffnet werden\n\t=> " + wbe.toString());
				}
			}
		};

		actionMenuLeistenEditorLeer.setText("Meneditor (leer)");
		actionMenuLeistenEditorLeer
				.setToolTipText("ffnet den Editor zum Bearbeiten der Strukturen "
						+ "von Menleisten mit leerer Darstellung.");
		actionMenuLeistenEditorLeer.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/menu.gif"));
		actionMenuLeistenEditorLeer
				.setId("de.bsvrz.buv.rw.basislib.menuLeistenEditorLeer");
		EditorOeffnenAktionsSpeicher.getInstanz().setActionMenuEditorLeer(
				actionMenuLeistenEditorLeer);
		DeskriptorAktion deskriptorAktionMenuLeistenEditorLeer = new DeskriptorAktion(
				Activator.PLUGIN_ID, actionMenuLeistenEditorLeer, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					deskriptorAktionMenuLeistenEditorLeer);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "des Menleisteneditors mit leerer Darstellung konnte nicht "
			// + "registriert werden\n\t=> " + iae.toString());
		}

		/*
		 * Action zum ffnen der Seiteneinstellungen
		 */
		Action actionSeitenEinstellungen = new Action()
		{
			public void run()
			{
				PreferenceDialog dialog = PreferencesUtil.createPreferenceDialogOn(
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
						Seiteneinstellungen.ID, null, null);
				dialog.open();
			}
		};

		actionSeitenEinstellungen
				.setId("de.bsvrz.buv.rw.basislib.actionSeitenEinstellungen");
		actionSeitenEinstellungen.setText("Seiteneinstellungen");
		actionSeitenEinstellungen.setToolTipText("Seiteneinstellungen vornehmen");
		actionSeitenEinstellungen.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/menu.gif"));

		DeskriptorAktion deskriptorAktionSeitenEinstellungen = new DeskriptorAktion(
				Activator.PLUGIN_ID, actionSeitenEinstellungen, false);

		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					deskriptorAktionSeitenEinstellungen);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "des Dialogs zum Vornehmen der Seiteneinstellungen konnte nicht "
			// + "registriert werden\n\t=> " + iae.toString());
		}

		DruckVorschauAction druckVorschauAktuell = new DruckVorschauAction(
				"de.bsvrz.buv.rw.basislib.actionDruckVorschauAllgemein");
		DeskriptorAktion deskriptorAktionDruckVorschauAktion = new DeskriptorAktion(
				Activator.PLUGIN_ID, druckVorschauAktuell, false);

		IWorkbenchWindow window = FensterSpeicher.getInstanz().getWindow();
		window.getPartService().addPartListener(druckVorschauAktuell);

		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					deskriptorAktionDruckVorschauAktion);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "des Druckvorschaudialoges" + "registriert werden\n\t=> "
			// + iae.toString());
		}

		/*
		 * Test der Listendarstellung
		 */
		Action actionTestKTable = new Action()
		{
			public void run()
			{
				TestKTableAenderungsDarstellung te = new TestKTableAenderungsDarstellung();
				te.open();
			}
		};
		actionTestKTable.setId("de.bsvrz.buv.rw.basislib.actionListenDarstellung");
		actionTestKTable.setText("Listendarstellung");
		actionTestKTable.setToolTipText("Listendarstellung ffnen");
		DeskriptorAktion deskriptorAktionListenDarstellung = new DeskriptorAktion(
				Activator.PLUGIN_ID, actionTestKTable, false);

		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					deskriptorAktionListenDarstellung);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "der Listendarstellung konnte nicht registriert werden\n\t=> "
			// + iae.toString());
		}

		/*
		 * Aktion zum ffnen der Error Log View
		 */
		Action actionErrorLogViewOeffnen = new Action()
		{
			public void run()
			{
				try
				{
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
							.showView("org.eclipse.pde.runtime.LogView");
				} catch (PartInitException e)
				{
					e.printStackTrace();
				}
			}
		};
		// actionErrorLogViewOeffnen.setImageDescriptor(Activator
		// .getImageDescriptor("icons/menubar/speichern_16x16.png"));
		actionErrorLogViewOeffnen.setId("actionErrorLogViewOeffnen");
		actionErrorLogViewOeffnen.setText("Error Log");
		actionErrorLogViewOeffnen.setToolTipText("ffnet die Error Log View");
		actionErrorLogViewOeffnen.setImageDescriptor(Activator
				.getImageDescriptor("icons/actions/error_stack.gif"));

		DeskriptorAktion daErrorLogViewOeffnen = new DeskriptorAktion(
				Activator.PLUGIN_ID, actionErrorLogViewOeffnen, false);

		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					daErrorLogViewOeffnen);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "der Error Log View konnte nicht registriert werden\n\t=> "
			// + iae.toString());
		}

		/*
		 * Aktion zum Dateiffnen
		 */
		Action actionDateiOeffnen = new Action()
		{
			public void run()
			{
			}
		};
		actionDateiOeffnen.getId();

		/*
		 * Aktion zum ffnen des Statusleisteneditors
		 */
		Action actionStatusLeistenEditor = new Action()
		{
			public void run()
			{
				/*
				 * Perspective ffnen
				 */
				String perspectiveID = "de.bsvrz.buv.rw.basislib.perspectiveStatusLeisteEditor";
				try
				{
					PlatformUI.getWorkbench().showPerspective(perspectiveID,
							PlatformUI.getWorkbench().getActiveWorkbenchWindow());

					// MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();
					// msld.setzeLeistenArt(enumLeistenArt.menue);
					// EditorMenuSymbolLeisterInput ei = new EditorMenuSymbolLeisterInput(
					// msld);
					// IWorkbenchPage page = PlatformUI.getWorkbench()
					// .getActiveWorkbenchWindow().getActivePage();
					// try
					// {
					// page.openEditor(ei, "de.bsvrz.buv.rw.basislib.editorMenu");
					//
					// IEditorPart editor = PlatformUI.getWorkbench()
					// .getActiveWorkbenchWindow().getActivePage().findEditor(ei);
					// if (editor instanceof EditorMenuSymbolLeiste)
					// {
					// EditorMenuSymbolLeiste editorMenuSymbolLeiste =
					// (EditorMenuSymbolLeiste) editor;
					// editorMenuSymbolLeiste
					// .setLadenSpeichernLeistenArt(EnumLeistenArt.menuLeiste);
					// }
					//
					// } catch (PartInitException pie)
					// {
					// _debug.finest("Der Menleisteneditor konnte nicht "
					// + "geffnet werden\n\t=> " + pie.toString());
					// }

				} catch (WorkbenchException wbe)
				{
					_debug.finest("Die Men- und Symbolleisteneditorperspektive "
							+ "konnte nicht geffnet werden\n\t=> " + wbe.toString());
				}
			}
		};

		actionStatusLeistenEditor.setText("Statusleisteneditor");
		actionStatusLeistenEditor
				.setToolTipText("ffnet den Editor zum Bearbeiten der Struktur der Statusleiste");
		actionStatusLeistenEditor.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/menu.gif"));
		actionStatusLeistenEditor
				.setId("de.bsvrz.buv.rw.basislib.menuStatusLeistenEditor");
		DeskriptorAktion deskriptorAktionStatusLeistenEditor = new DeskriptorAktion(
				Activator.PLUGIN_ID, actionStatusLeistenEditor, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					deskriptorAktionStatusLeistenEditor);
		} catch (IllegalArgumentException iae)
		{
			// _debug.warning(getClass().getName()
			// + "::registriereActions Die Action zum Starten "
			// + "des Menleisteneditors mit leerer Darstellung konnte nicht "
			// + "registriert werden\n\t=> " + iae.toString());
		}

	}

	/**
	 * Fhrt die Registrierung von Symbolleistenelementen durch
	 */
	private void registriereSymbolleistenElemente()
	{
		/*
		 * String id3 = "sl.zeitelement"; SlZeitItem szi = new SlZeitItem(id);
		 * StatusLeisteElement sle3 = new StatusLeisteElement(id3, "Zeitelement#" +
		 * id3, szi); StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(sle3);
		 */
		String idZeitItem = "sl.zeitelement";
		SymbolleisteZeitItem szi = new SymbolleisteZeitItem(idZeitItem);
		StatusLeisteElement sleZeit = new StatusLeisteElement(idZeitItem,
				"Zeitelement", szi);

		try
		{
			StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(sleZeit);
		} catch (IllegalArgumentException e)
		{
		}
	}

	/**
	 * Diese Klasse liefert ein {@link ContributionItem}, welches eine Uhr
	 * beinhaltet und in der Statusleiste positioniert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SymbolleisteZeitItem extends ContributionItem
	{
		/**
		 * Textfeld zur Darstellung der Zeit.
		 */
		private Text _textZeitfeld = null;

		/**
		 * Timer zum setzen der aktuellen Zeit.
		 */
		private Timer _timer;

		/**
		 * Konstruktor.
		 * 
		 * @param id
		 *          ID des {@link ContributionItem}.
		 */
		public SymbolleisteZeitItem(String id)
		{
			super(id);
		}

		public void fill(Composite parent)
		{
			this._textZeitfeld = new Text(parent, SWT.BORDER);
			_textZeitfeld.setEditable(false);

			startTimer();
		}

		/**
		 * Startet den Timer, der die Uhrzeit aktualisiert.
		 */
		private void startTimer()
		{
			if (_timer == null)
			{
				TimerTask task = new TimerTask()
				{
					@Override
					public void run()
					{
						Runnable run = new Runnable()
						{

							@Override
							public void run()
							{
								if ((_textZeitfeld != null) && (!_textZeitfeld.isDisposed()))
								{
									_textZeitfeld.setText(SDF.format(new Date()));
								}
							}
						};

						PlatformUI.getWorkbench().getDisplay().asyncExec(run);
					}
				};

				this._timer = new Timer("Timer Uhrzeit Statusleiste @ " + new Date());
				_timer.schedule(task, 0l, 250l);
			}
		}

		/**
		 * Stoppt den Timer, der die Uhrzeit aktualisiert.
		 */
		private void stopTimer()
		{
			if (_timer != null)
			{
				_timer.cancel();
				this._timer = null;
			}
		}

		public void dispose()
		{
			stopTimer();
			super.dispose();
		}
	}

}
