/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.basislib.berechtigung.SuchBaum.BaumKnoten;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IBenutzerZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.IDaVPreLader;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * bernimmt die Kommunikation mit dem DaV und enthlt Funktionen zum Abfragen
 * der im DaV hinterlegten Berechtigungen.
 * 
 * Diese Klasse implementiert das Singleton-Entwurfsmuster.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author:
 *          HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface
{
	/**
	 * Liefert die einzigste Instanz der Klasse {@link DaVKommunikation}.
	 */
	private static DaVKommunikation _instanz = null;

	/**
	 * Verbindung zum DaV.
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Liste der {@link IOberflaechenBerechtigungZuhoerer}, die ber die
	 * Berechtigung informiert werden mchten.
	 */
	private LinkedList<IOberflaechenBerechtigungZuhoerer> _oberflaechenBenutzerZuhoerer = new LinkedList<IOberflaechenBerechtigungZuhoerer>();

	/**
	 * Der Suchbaum
	 */
	private SuchBaum _baum = new SuchBaum();

	/**
	 * Maximale Anzahl der Schleifendurchlaeufe
	 */
	private static final int MAX_CNT = 100;

	// /**
	// * Instanz der Klasse der SystemObjekte
	// */
	// private SystemObjekteSystem _systemObjekteSystem = null;

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Instanz der Kommandozeilenparameter
	 */
	private Kommandozeilenparameter _kommandoZeilenParameter = Kommandozeilenparameter
			.getInstanz();

	/**
	 * Prelader fr Informationen vom DaV
	 */
	private DaVPreLader _preLader = null;

	/**
	 * Listener fr den Prelader. Ist u.a. fr das Ein- und Ausblenden des
	 * Hinzufgenknopfes zustndig.
	 */
	private IDaVPreLader _preLaderListener = null;

	private int _cntBenutzer;

	private Map<String, Data> _benutzerList;

	private IDaVVerbindungZuhoerer _daVVerbindungZuhoerer;

	private boolean _isPreladerFertig = false;

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link DaVKommunikation}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link DaVKommunikation}.
	 */
	public static DaVKommunikation getInstanz()
	{
		if (_instanz == null)
		{
			_instanz = new DaVKommunikation();
		}
		return _instanz;
	}

	/**
	 * Privater Konstruktor des Singletons.
	 * 
	 */
	private DaVKommunikation()
	{

		this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		this._benutzerList = new HashMap<String, Data>();

		_daVVerbindungZuhoerer = new IDaVVerbindungZuhoerer()
		{
			public void verbindungAbgebaut(DaVVerbindungEreignis e)
			{
				_connection = null;
			}

			public void verbindungAufgebaut(DaVVerbindungEreignis e)
			{
				_connection = e.getDaVVerbindung();
			}

		};
		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(_daVVerbindungZuhoerer);

		_preLader = DaVPreLader.getInstanz();

		_preLaderListener = new IDaVPreLader()
		{

			@Override
			public void systemObjekteAktualisierung(GtmDatenBasis datenBasis)
			{
				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						berechneBerechtigungFrAlle();
					}
				});
			}

			@Override
			public void systemObjekteInitial(GtmDatenBasis datenBasis)
			{
				_isPreladerFertig = true;
				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						for (Object o : _oberflaechenBenutzerZuhoerer)
						{
							if (o instanceof IOberflaechenBerechtigungZuhoererErweitert)
							{
								IOberflaechenBerechtigungZuhoererErweitert zuhoererErweitert = (IOberflaechenBerechtigungZuhoererErweitert) o;
								zuhoererErweitert.oberflaechenBerechtigungBereit();

							}
						}
					}
				});
			}

		};
		_preLader.hinzufuegenListenerPreLader(_preLaderListener);

		KmdParameter kp1 = _kommandoZeilenParameter
				.getKommandoZeilenParameter("-berechtigungsKonzept");
		String wert1 = kp1.getWert();

		if (kp1 == null || wert1.equals("true"))
			_preLader.getGtmDatenBasis();

		neuAnmeldung();

		// System.out.println("B.Anmeldung!");

		benutzerAnmeldung();

	}

	/**
	 * Diese Methode prft, ob auf einem Systemobjekt ein bestimmter Benutzer das
	 * Recht hat, eine Funktion durchzufhren.
	 * 
	 * @param systemObject
	 *          Systemobjekt, fr das die Berechtigung geprft werden soll.
	 *          <b>Worauf?</b>
	 * @param benutzer
	 *          Benutzer, fr den die Berechtigung auf diesem Systemobjekt geprft
	 *          werden soll. <b>Wer?</b>
	 * @param funktion
	 *          Funktion, fr welche auf dem Systemobjekt fr den gegebenen
	 *          Benutzer die Berechtigung geprft werden soll. <b>Was?</b>
	 * @return Liefert <code>true</code>, wenn der Benutzer die Berechtigung hat,
	 *         sonst false.
	 */
	@SuppressWarnings("unchecked")
	public boolean hatBerechtigung(SystemObject systemObject, String benutzer,
			OberflaechenFunktion funktion)
	{

		KmdParameter kp1 = _kommandoZeilenParameter
				.getKommandoZeilenParameter("-berechtigungsKonzept");
		String wert1 = kp1.getWert();

		if (kp1 == null || wert1.equals("false"))
		{
			return true;
		} else if (!_isPreladerFertig)
		{
			return false;
		}

		Map<Data, List<Data>> map = _baum.getPreorder();

		Set<Data> set = map.keySet();
		Object[] array = set.toArray();

		Data[] datas = new Data[array.length];

		for (Object o : array)
		{
			Data d = (Data) o;

			Integer index = new Integer(d.getName());

			try
			{
				datas[index] = d;
			} catch (ArrayIndexOutOfBoundsException e)
			{
				_debug
						.error("Fehler Berechtigungskonzept: Parameter atg.oberflchenBerechtigung hat falschen Index: "
								+ index);
				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						setzeBerechtigungFrAlle(false);
					}
				});
				return false;
				// continue;
			}

		}

		// System.out.println("Start");
		//
		// for (int i = 0; i < datas.length; i++)
		// {
		// System.out.println(datas[i].getName());
		// }
		//
		// System.out.println("Ende");

		List<Object[]> mapNew = new ArrayList<Object[]>();

		for (int i = 0; i < datas.length; i++)
		{
			Data key = datas[i];

			List<Data> list = map.get(key);

			if (list == null)
				continue;

			// for (Map.Entry<Data, List<Data>> me : map.entrySet())
			// {
			// Data key = me.getKey();
			//
			// List<Data> list = me.getValue();

			List<Data> listNeu = new ArrayList<Data>();

			int cnt = 0;

			if (list.size() == 1)
			{

				// System.out.println("hier1");

				for (Data data : list)
				{

					long ebene = data.getItem("Ebene").asScaledValue().longValue();

					if (cnt > 0 && ebene == 1)
					{
						listNeu.add(data);
					} else
					{
						_debug.fine("Fehler: Auf Ebene 0 muss Ebene 1 folgen!");
					}

					cnt++;
				}

				list.removeAll(listNeu);

				Object[] objekte1 = new Object[2];
				objekte1[0] = key;
				objekte1[1] = list;

				mapNew.add(objekte1);

				for (Data data2 : listNeu)
				{
					List<Data> tmp = new ArrayList<Data>();

					tmp.add(data2);

					Object[] o2 = new Object[2];
					o2[0] = key;
					o2[1] = tmp;

					mapNew.add(o2);
				}
			} else
			{
				// System.out.println("hier2");

				Object[] o1 = new Object[2];
				o1[0] = key;
				o1[1] = list;

				mapNew.add(o1);
			}

		}

		boolean freigabe = true;

		List<String> klBerechtigung = new ArrayList<String>();
		List<String> soBerechtigung = new ArrayList<String>();
		List<String> ofBerechtigung = new ArrayList<String>();

		// System.out.println("Size: " + mapNew.size());

		List<String> warningList = new ArrayList<String>();

		for (Object[] objects : mapNew)
		{

			// List<String> klBerechtigung = new ArrayList<String>();
			// List<String> soBerechtigung = new ArrayList<String>();
			// List<String> ofBerechtigung = new ArrayList<String>();

			Data data1 = (Data) objects[0];
			List<Data> list = (ArrayList<Data>) objects[1];

			// System.out.println("Data : " + data1);

			String b1 = data1.getItem("Berechtigung").asTextValue().getText();

			Array a1 = data1.getItem("Berechtigungsklasse").asArray();
			for (int i = 0; i < a1.getLength(); i++)
			{
				String klasse = a1.getItem(i).asTextValue().getText();
				if (klasse.equals("*"))
				{

					SystemObjectType sot = _connection.getDataModel().getType(
							"typ.benutzer");
					List<SystemObject> sots = sot.getElements();
					for (SystemObject so : sots)
					{
						if (list.size() > 0)
							klBerechtigung.add(so.getPid());
						else if (b1.equals("Freigabe"))
							klBerechtigung.add(so.getPid());
					}
					break;
				} else
				{

					if (list.size() > 0)
					{
						List<String> l = pruefeObjektAufBerechtigungsklasse(klasse);

						if (l != null)
						{

							for (String s : l)
							{
								klBerechtigung.add(s);
							}

						} else
						{
							String s = "Berechtigungskonzept: Benutzer oder Berechtigungsklasse nicht vorhanden: "
									+ klasse;
							if (!warningList.contains(s))
								warningList.add(s);
							// setzeBerechtigungFrAlle(false);
						}

					} else if (b1.equals("Freigabe"))
					{
						List<String> l = pruefeObjektAufBerechtigungsklasse(klasse);
						// System.out.println(klasse);

						if (l != null)
						{

							for (String s : l)
							{
								// System.out.println(s);
								klBerechtigung.add(s);
							}
						} else
						{
							String s = "Berechtigungskonzept: Benutzer oder Berechtigungsklasse nicht vorhanden: "
									+ klasse;
							if (!warningList.contains(s))
								warningList.add(s);
							// setzeBerechtigungFrAlle(false);
						}
					}
				}
			}

			Array a2 = data1.getItem("ObjektMenge").asArray();
			for (int i = 0; i < a2.getLength(); i++)
			{
				String objekt = a2.getItem(i).asTextValue().getText();
				if (objekt.equals("*"))
				{

					if (list.size() > 0)
					{
						// List<String> sotsPid =
						// _systemObjekteSystem.getSystemObjektePid();
						// soBerechtigung.addAll(sotsPid);
						LinkedList<SystemObject> sos = _preLader.getSystemObjekte();
						for (SystemObject so : sos)
						{
							soBerechtigung.add(so.getPid());
						}
					} else if (b1.equals("Freigabe"))
					{
						// List<String> sotsPid =
						// _systemObjekteSystem.getSystemObjektePid();
						// soBerechtigung.addAll(sotsPid);
						LinkedList<SystemObject> sos = _preLader.getSystemObjekte();
						for (SystemObject so : sos)
						{
							soBerechtigung.add(so.getPid());
						}
					}
					break;
				} else
				{
					// soBerechtigung.add(objekt);
					List<String> l1 = pruefeObjektAufKonfigurationsBereich(objekt);
					// soBerechtigung.addAll(l1);
					if (l1 != null)
					{
						for (String s : l1)
						{
							if (list.size() > 0)
								soBerechtigung.add(s);
							else if (b1.equals("Freigabe"))
								soBerechtigung.add(s);
						}
					} else
					{
						String s = "Berechtigungskonzept: Objekt oder Konfigurationsbereich nicht vorhanden: "
								+ objekt;
						if (!warningList.contains(s))
							warningList.add(s);
						// setzeBerechtigungFrAlle(false);
					}

				}
			}

			Array a3 = data1.getItem("Funktion").asArray();
			for (int i = 0; i < a3.getLength(); i++)
			{

				String funk = a3.getItem(i).asTextValue().getText();

				if (funk.equals("*"))
				{
					List<OberflaechenFunktion> funktionen = OberflaechenBerechtigung
							.getInstanz().getFunktionen();
					for (OberflaechenFunktion of : funktionen)
					{
						if (list.size() > 0)
							ofBerechtigung.add(of.getFunktionsName());
						else if (b1.equals("Freigabe"))
							ofBerechtigung.add(of.getFunktionsName());
					}
					break;
				} else
				{
					List<OberflaechenFunktion> funktionen = OberflaechenBerechtigung
							.getInstanz().getFunktionen();

					boolean gefunden = false;
					for (OberflaechenFunktion of : funktionen)
					{
						if (of.getFunktionsName().equals(funk))
							gefunden = true;
					}

					if (gefunden)
					{
						if (list.size() > 0)
							ofBerechtigung.add(funk);
						else if (b1.equals("Freigabe"))
							ofBerechtigung.add(funk);
					} else
					{
						String s = "Berechtigungskonzept: Funktion nicht vorhanden: "
								+ funk;
						if (!warningList.contains(s))
							warningList.add(s);
						// setzeBerechtigungFrAlle(false);
					}

				}
			}

			// for (String s : ofBerechtigung)
			// {
			// System.out.println(funktion.getFunktionsName() + ": " + s);
			// }

			// List<Data> list = (ArrayList<Data>)objects[1];

			// System.out.println("List: " + list.size());

			for (Data data2 : list)
			{

				List<String> klAusnahmeTmp = new ArrayList<String>();
				List<String> soAusnahmeTmp = new ArrayList<String>();
				List<String> ofAusnahmeTmp = new ArrayList<String>();

				String b2 = data2.getItem("Berechtigung").asTextValue().getText();
				Array a4 = data2.getItem("Berechtigungsklasse").asArray();
				for (int i = 0; i < a4.getLength(); i++)
				{
					String klasse = a4.getItem(i).asTextValue().getText();
					if (klasse.equals("*"))
					{
						SystemObjectType sot = _connection.getDataModel().getType(
								"typ.benutzer");
						List<SystemObject> sots = sot.getElements();
						for (SystemObject so : sots)
						{
							klAusnahmeTmp.add(so.getPid());
						}
						break;
					} else
					{
						List<String> l = pruefeObjektAufBerechtigungsklasse(klasse);

						if (l != null)
						{
							for (String s : l)
							{
								// System.out.println(s);
								klAusnahmeTmp.add(s);
							}
						} else
						{
							String s = "Berechtigungskonzept: Benutzer oder Berechtigungsklasse nicht vorhanden: "
									+ klasse;
							if (!warningList.contains(s))
								warningList.add(s);
						}

					}
				}

				Array a5 = data2.getItem("ObjektMenge").asArray();
				for (int i = 0; i < a5.getLength(); i++)
				{
					String objekt = a5.getItem(i).asTextValue().getText();
					if (objekt.equals("*"))
					{

						// List<String> sotsPid =
						// _systemObjekteSystem.getSystemObjektePid();
						// soAusnahmeTmp.addAll(sotsPid);
						LinkedList<SystemObject> sos = _preLader.getSystemObjekte();
						for (SystemObject so : sos)
						{
							soAusnahmeTmp.add(so.getPid());
						}
						break;
					} else
					{
						// soAusnahmeTmp.add(objekt);
						List<String> l2 = pruefeObjektAufKonfigurationsBereich(objekt);
						// soAusnahmeTmp.addAll(l2);

						if (l2 != null)
						{
							for (String s : l2)
							{
								soAusnahmeTmp.add(s);
							}
						} else
						{
							String s = "Berechtigungskonzept: Objekt oder Konfigurationsbereich nicht vorhanden: "
									+ objekt;
							if (!warningList.contains(s))
								warningList.add(s);

						}

					}
				}

				Array a6 = data2.getItem("Funktion").asArray();
				for (int i = 0; i < a6.getLength(); i++)
				{
					String funk = a6.getItem(i).asTextValue().getText();

					if (funk.equals("*"))
					{
						List<OberflaechenFunktion> funktionen = OberflaechenBerechtigung
								.getInstanz().getFunktionen();

						for (OberflaechenFunktion of : funktionen)
						{
							ofAusnahmeTmp.add(of.getFunktionsName());
						}
						break;
					} else
					{
						List<OberflaechenFunktion> funktionen = OberflaechenBerechtigung
								.getInstanz().getFunktionen();

						boolean gefunden = false;
						for (OberflaechenFunktion of : funktionen)
						{
							if (of.getFunktionsName().equals(funk))
								gefunden = true;
						}

						if (gefunden)
						{
							ofAusnahmeTmp.add(funk);
						} else
						{
							String s = "Berechtigungskonzept: Funktion nicht vorhanden: "
									+ funk;
							if (!warningList.contains(s))
								warningList.add(s);

						}
					}
				}

				if (klAusnahmeTmp.contains(benutzer)
						&& soAusnahmeTmp.contains(systemObject.getPid())
						&& ofAusnahmeTmp.contains(funktion.getFunktionsName()))
				{

					if (b1.equals("Freigabe") && b2.equals("Sperrung"))
					{
						klBerechtigung.removeAll(klAusnahmeTmp);
						// soBerechtigung.removeAll(soAusnahmeTmp);
						for (String s : soAusnahmeTmp)
						{
							soBerechtigung.remove(s);
						}

						ofBerechtigung.removeAll(ofAusnahmeTmp);
					} else if (b1.equals("Sperrung") && b2.equals("Freigabe"))
					{
						klBerechtigung.removeAll(klAusnahmeTmp);
						// soBerechtigung.removeAll(soAusnahmeTmp);
						for (String s : soAusnahmeTmp)
						{
							soBerechtigung.remove(s);
						}

						ofBerechtigung.removeAll(ofAusnahmeTmp);
					} else if (b1.equals("Freigabe") && b2.equals("Freigabe"))
					{
						klBerechtigung.addAll(klAusnahmeTmp);
						// soBerechtigung.addAll(soAusnahmeTmp);
						for (String s : soAusnahmeTmp)
						{
							soBerechtigung.add(s);
						}
						ofBerechtigung.addAll(ofAusnahmeTmp);
					} else if (b1.equals("Sperrung") && b2.equals("Sperrung"))
					{
						klBerechtigung.addAll(klAusnahmeTmp);
						// soBerechtigung.addAll(soAusnahmeTmp);
						for (String s : soAusnahmeTmp)
						{
							soBerechtigung.add(s);
						}
						ofBerechtigung.addAll(ofAusnahmeTmp);
					} else
					{
						_debug.fine("Fehler: Konnte Attribut Berechtigung nicht auswerten");
					}
				}

			}

			// for (String s : ofBerechtigung)
			// {
			// System.out.println(funktion.getFunktionsName() + ": " + s);
			// }

			// }

			boolean f1 = klBerechtigung.contains(benutzer);
			boolean f2 = soBerechtigung.contains(systemObject.getPid());
			boolean f3 = ofBerechtigung.contains(funktion.getFunktionsName());

			// System.out.println("Einzelne Freigaben -> Kl: So: Of: " + f1 + " " + f2
			// + " " + f3);

			if (list.size() > 0)
			{

				if (b1.equals("Freigabe"))
				{
					if (f1 && f2 && f3)
						freigabe = true;
					else
						freigabe = false;
				} else
				{
					if (f1 && f2 && f3)
						freigabe = false;
					else
						freigabe = true;
				}

			} else
			{
				if (f1 && f2 && f3)
					freigabe = true;
				else
					freigabe = false;
			}

		}

		for (String s : warningList)
		{
			_debug.warning(s);
		}

		// for (String s : ofBerechtigung)
		// {
		// System.out.println(funktion.getFunktionsName() + ": " + s);
		// }

		return freigabe;
	}

	/**
	 * Fgt einen Listener zur Liste der Betriebsmeldungslistener hinzu.
	 * 
	 * @param listener
	 */
	public void hinzufuegenOberflaechenBenutzerZuhoerer(
			IOberflaechenBerechtigungZuhoerer listener)
	{
		if (!_oberflaechenBenutzerZuhoerer.contains(listener))
		{
			_oberflaechenBenutzerZuhoerer.add(listener);
		}

	}

	/**
	 * Entfernt einen {@link IBenutzerZuhoerer} von der Liste der Zuhrer.
	 * 
	 * @param benutzerZuhoerer
	 *          Zu entfernender {@link IBenutzerZuhoerer}.
	 */
	public void entferneOberflaechenBenutzerZuhoerer(
			IOberflaechenBerechtigungZuhoerer listener)
	{
		if (_oberflaechenBenutzerZuhoerer.contains(listener))
		{
			_oberflaechenBenutzerZuhoerer.remove(listener);
		}

	}

	/**
	 * Benachrichtigt alle registrierten {@link IBenutzerZuhoerer} darber, dass
	 * ein neuer Benutzer sich angemeldet hat.
	 * 
	 */
	public void benachrichtigeOberflaechenBenutzerZuhoerer(
			OberflaechenBerechtigungEreignis ereignis)
	{

		Iterator<IOberflaechenBerechtigungZuhoerer> it = _oberflaechenBenutzerZuhoerer
				.iterator();
		while (it.hasNext())
		{
			IOberflaechenBerechtigungZuhoerer listener = it.next();

			if (ereignis.hatFreigabe())
				listener.freigabe(ereignis);
			else
				listener.sperrung(ereignis);
		}
	}

	@Override
	public void update(ResultData[] results)
	{
		for (int i = 0; i < results.length; i++)
		{
			ResultData result = results[i];

			if (result.getDataDescription().getAttributeGroup().getPid().equals(
					"atg.oberflchenBerechtigung"))
			{

				if (result != null)
				{
					_baum = new SuchBaum();

					if (result.getDataState().equals(DataState.NO_SOURCE))
					{
						_debug
								.error("Fehler Berechtigungskonzept: Parameter atg.oberflchenBerechtigung kein Quelle");
						Display.getDefault().asyncExec(new Runnable()
						{
							public void run()
							{
								setzeBerechtigungFrAlle(false);
							}
						});
						return;
					}

					Data data = result.getData();

					if (data != null)
					{

						Array array = result.getData().getItem("OberflchenBerechtigung")
								.asArray();

						if (array.getLength() == 0)
						{
							_debug
									.error("Fehler Berechtigungskonzept: Parameter atg.oberflchenBerechtigung ist nicht definiert");
							Display.getDefault().asyncExec(new Runnable()
							{
								public void run()
								{
									setzeBerechtigungFrAlle(false);
								}
							});
							return;
						}

						BaumKnoten knoten = null;
						List<String> besuchtList = new ArrayList<String>();

						for (int j = 0; j < array.getLength(); j++)
						{
							Data d = array.getItem(j);

							// String ausnahme =
							// d.getItem("AusnahmeDefinition").asTextValue().getText();
							// Integer loc = new Integer(ausnahme)

							besuchtList.add(d.getName());

						}

						int index = 0;
						int cnt = 0;

						while (besuchtList.size() > 0)
						{
							if (cnt > MAX_CNT)
							{

								break;
							}

							try
							{
								Data d = array.getItem(index);
								String ebene = d.getItem("Ebene").asTextValue().getText();
								Integer e = new Integer(ebene);
								if (e > 0)
									knoten = _baum.einfuegen(knoten, d);
								else
									knoten = _baum.einfuegen(null, d);
								String ausnahme = d.getItem("AusnahmeDefinition").asTextValue()
										.getText();
								index = new Integer(ausnahme);
								if (!besuchtList.remove(d.getName()))
								{
									_debug.fine("Fehler, konnte nicht Lschen: " + d);
									break;
								}
								cnt++;
							} catch (ArrayIndexOutOfBoundsException e)
							{
								_debug
										.error("Fehler Berechtigungskonzept: Parameter atg.oberflchenBerechtigung hat falschen Index: "
												+ index);
								Display.getDefault().asyncExec(new Runnable()
								{
									public void run()
									{
										setzeBerechtigungFrAlle(false);
									}
								});
								return;

							}
						}
					} else
					{
						_debug
								.error("Fehler Berechtigungskonzept: Parameter atg.oberflchenBerechtigung hat keine Daten");
						Display.getDefault().asyncExec(new Runnable()
						{
							public void run()
							{
								setzeBerechtigungFrAlle(true);
							}
						});
						return;
					}

				} else
				{
					_debug.fine("result ist null");
				}

				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						berechneBerechtigungFrAlle();
					}
				});

			} else if (result.getDataDescription().getAttributeGroup().getPid()
					.equals("atg.benutzerParameter"))
			{
				if (result != null)
				{
					String pid = result.getObject().getPid();
					Data data = result.getData();

					// synchronized (this)
					{

						// for (Map.Entry<String, Data> me : _benutzerList.entrySet())
						// {
						// String key = me.getKey();
						// Data value = me.getValue();
						//            
						// System.out.println(key + " : " +
						// data.getItem("berechtigungsklasse").asReferenceValue().getSystemObjectPid());
						//            
						// }

						if (!_benutzerList.containsKey(pid))
						{

							synchronized (this)
							{
								_cntBenutzer++;
								_benutzerList.put(pid, data);
								this.notify();
							}
						} else
						{
							synchronized (this)
							{
								_benutzerList.remove(pid);
								_benutzerList.put(pid, data);

								// for (Map.Entry<String, Data> me : _benutzerList.entrySet())
								// {
								// String key = me.getKey();
								// Data value = me.getValue();
								// System.out.println(key + " : "
								// +
								// value.getItem("berechtigungsklasse").asReferenceValue().getSystemObjectPid());
								// }

							}

							berechneBerechtigungFrAlle();
						}
						// System.out.println("erhalten: " + pid + " : " + data);
					}
				}
			}
		}
	}

	/**
	 * Holt das Verbindungungsobjekt
	 * 
	 * @return
	 */
	public ClientDavInterface getConnection()
	{
		return _connection;
	}

	/**
	 * Holt den Suchbaum
	 * 
	 * @return
	 */
	public SuchBaum getBaum()
	{
		return _baum;

	}

	/**
	 * Prueft, ob die Pid eines SystemObjekts ein Menge ist. Ist dies der Fall
	 * wird eine Liste mit den Pids aller SystemObjektes der Menge zurck. Wenn
	 * die bergegebene Pid keine Menge ist, beinhaltet die Liste nur die Pid des
	 * bergebenen SystemObjekts.
	 * 
	 * @param objekt
	 *          die Pid eines SystemObjekts
	 * @return Liste der Pids von SystemObjekten
	 */

	@SuppressWarnings("unused")
	private List<String> pruefeObjektAufMenge(String objekt)
	{

		List<String> list = new ArrayList<String>();

		SystemObject so = null;

		// Es wird ueberprueft ob der String nur aus Ziffern besteht,
		// dann ist es wahrscheinlich eine ID.
		if (objekt.matches("[0-9]*"))
		{

			Long l = new Long(objekt);
			so = _connection.getDataModel().getObject(l);

		} else
			so = _connection.getDataModel().getObject(objekt);

		if (so == null)
		{
			_debug.fine("Konnte Objekt nicht in der Konfiguration finden: " + objekt);
			return null;
		}

		// Ist das Objekt eine Menge, dann Elemnte der Menge holen
		if (so.isOfType("typ.menge"))
		{

			List<SystemObject> elements = ((ObjectSet) so).getElements();

			List<String> tmp = new ArrayList<String>();

			for (SystemObject object : elements)
			{
				tmp.add(object.getPid());
			}

			list.addAll(tmp);

		} else
		{
			list.add(so.getPid());
		}

		return list;
	}

	/**
	 * Prueft, ob die Pid eines SystemObjektes ein KonfigurationsBereich ist. Ist
	 * dies der Fall wird eine Liste mit den Pids aller SystemObjekte des
	 * Konfigurationsbereichs zurckgegeben. Wenn die bergegebene Pid kein
	 * KonfiguartionsBereich ist, beinhaltet die Liste nur die Pid des bergebenen
	 * SystemObjekts.
	 * 
	 * 
	 * @param objekt
	 *          die Pid eines SystemObjekts
	 * @return Liste der Pids von SystemObjekten
	 */
	private List<String> pruefeObjektAufKonfigurationsBereich(String objekt)
	{

		List<String> list = new ArrayList<String>();

		SystemObject so = null;

		so = _connection.getDataModel().getObject(objekt);

		if (so == null)
		{
			_debug.fine("Konnte Objekt nicht in der Konfiguration finden: " + objekt);
			return null;
		}

		// Ist das Objekt eine Menge, dann Elemnte der Menge holen
		if (so.isOfType("typ.konfigurationsBereich"))
		{

			Collection<SystemObject> elements = ((ConfigurationArea) so)
					.getCurrentObjects();

			List<String> tmp = new ArrayList<String>();

			for (SystemObject object : elements)
			{
				tmp.add(object.getPid());
			}

			list.addAll(tmp);

		} else
		{
			list.add(so.getPid());
		}

		return list;
	}

	/**
	 * Prueft, ob die Pid eines SystemObjektes eine Berechtigungsklasse ist. Ist
	 * dies der Fall wird eine Liste mit den Pids aller Benutzer der
	 * Berechtigungsklasse zurckgegeben. Wenn die bergegebene Pid kein
	 * Berechtigungsklasse ist, beinhaltet die Liste nur die Pid des bergebenen
	 * Benutzers.
	 * 
	 * 
	 * @param objekt
	 *          die Pid eines SystemObjektes
	 * @return Liste der Pids von Benutzern
	 */
	private List<String> pruefeObjektAufBerechtigungsklasse(String objekt)
	{

		List<String> list = new ArrayList<String>();

		SystemObject so = null;

		if (_connection != null)
		{

			so = _connection.getDataModel().getObject(objekt);

			if (so == null)
			{
				_debug.fine("Konnte Objekt nicht in der Konfiguration finden: "
						+ objekt);
				return null;
			}

			// Ist das Objekt eine Berechtigungsklasse, dann Elemnte der Menge holen
			if (so.isOfType("typ.berechtigungsklasse"))
			{

				for (Map.Entry<String, Data> me : _benutzerList.entrySet())
				{
					String pid = me.getKey();
					Data data = me.getValue();

					String s = data.getItem("berechtigungsklasse").asReferenceValue()
							.getSystemObjectPid();

					if (s.equals(objekt))
					{
						// System.out.println("klasse: " + pid);
						list.add(pid);
					}

				}

			} else
			{
				list.add(so.getPid());
				// System.out.println("benutzer: " + so.getPid());
			}
		}

		return list;
	}

	public void neuAnmeldung()
	{
		if (_connection != null)
		{
			Kommandozeilenparameter kzp = Kommandozeilenparameter.getInstanz();

			String s = kzp.getKommandoZeilenParameter("-berechtigungsKonzept")
					.getWert();

			if (s.equals("true"))
			{

				AttributeGroup atg = _connection.getDataModel().getAttributeGroup(
						"atg.oberflchenBerechtigung");
				Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
				DataDescription dd = new DataDescription(atg, asp);

				KmdParameter param = kzp
						.getKommandoZeilenParameter("-oberflaechenObjekt");

				SystemObject object = null;
				String objekt = null;

				if (param != null)
				{

					objekt = param.getWert();

					if (objekt != null && objekt.length() > 0)
					{
						// Ist die Pid auch guletig?
						object = _connection.getDataModel().getObject(objekt);
						if (object == null)
							objekt = _connection.getLocalConfigurationAuthority().getPid();
					} else
					{
						objekt = _connection.getLocalConfigurationAuthority().getPid();
					}
				} else
					objekt = _connection.getLocalConfigurationAuthority().getPid();

				object = _connection.getDataModel().getObject(objekt);

				_connection.subscribeReceiver(this, object, dd,
						ReceiveOptions.normal(), ReceiverRole.receiver());

				// _systemObjekteSystem = SystemObjekteSystem.getInstanz();

			} else
			{
				_debug.info("Berechtigungskonzept ist nicht aktiv!");
			}

		}
	}

	public void benutzerAnmeldung()
	{
		if (_connection != null)
		{
			Kommandozeilenparameter kzp = Kommandozeilenparameter.getInstanz();

			String s = kzp.getKommandoZeilenParameter("-berechtigungsKonzept")
					.getWert();

			if (s.equals("true"))
			{

				AttributeGroup atg = _connection.getDataModel().getAttributeGroup(
						"atg.benutzerParameter");
				Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
				DataDescription dd = new DataDescription(atg, asp);

				SystemObjectType sot = _connection.getDataModel().getType(
						"typ.benutzer");
				List<SystemObject> sos = sot.getElements();

				_connection.subscribeReceiver(this, sos, dd, ReceiveOptions.normal(),
						ReceiverRole.receiver());

				_cntBenutzer = 0;

				synchronized (this)
				{

					if (_cntBenutzer < sos.size())
					{
						try
						{
							this.wait(10000);
						} catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}
				}
			} else
			{
				_debug.info("Berechtigungskonzept ist nicht aktiv!");
			}

		}
	}

	@SuppressWarnings("unchecked")
	private void setzeBerechtigungFrAlle(boolean berechtigung)
	{

		List list = new ArrayList();

		List<OberflaechenFunktion> fs = OberflaechenBerechtigung.getInstanz()
				.getFunktionen();

		final OberflaechenBerechtigungEreignis oe1 = new OberflaechenBerechtigungEreignis(
				this, berechtigung, list, list, list, fs);
		// Display.getDefault().asyncExec(new Runnable() {
		// public void run()
		// {
		benachrichtigeOberflaechenBenutzerZuhoerer(oe1);
		// }
		// });
	}

	private void berechneBerechtigungFrAlle()
	{

		final OberflaechenBerechtigung instanz = OberflaechenBerechtigung
				.getInstanz();

		List<OberflaechenFunktion> funktionen = instanz.getFunktionen();

		for (OberflaechenFunktion of : funktionen)
		{
			final OberflaechenFunktion f = of;
			// Display.getDefault().asyncExec(new Runnable() {
			// public void run()
			// {
			// for (OberflaechenFunktion of : funktionen)
			{
				// System.out.println("size: " + _preLader.getSystemObjekte().size());
				instanz.pruefeBerechtigungElement(instanz.getSystemObjekte(), f
						.getFunktionsName());
				// instanz.pruefeBerechtigungElement(_preLader.getSystemObjekte(),
				// f.getFunktionsName());
			}

			// }
			// });

		}
	}

}
