/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen;

import de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen.EinstellungenBenutzerSpezifischNetzweit;
import de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen.GlobaleEinstellungenNetzweit;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der Systemkalenderverwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation
{
  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;

  /**
   * Zugriff auf die {@link GlobaleEinstellungenNetzweit}
   */
  private GlobaleEinstellungenNetzweit _gen = null;

  /**
   * Zugriff auf die netzweiten, benutzerspezifischen Einstellungen {@link EinstellungenBenutzerSpezifischNetzweit}.
   */
  private EinstellungenBenutzerSpezifischNetzweit _ebs = null;
  
	/**
	 * Wenn <code>true</code>, erfolgt das senden ohne den
	 * {@link UrlasserDialog}.
	 */
	private boolean _sendenOhneUrlasserDlg = false;

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {
    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        _connection = e.getDaVVerbindung();
      }

    };

    DaVVerbindung.getInstanz().hinzufuegenZuhoerer(zuhoerer);
  }

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  protected static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }

    return _instanz;
  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Liefert die XML-Struktur der Voreinstellungen fr einen spezifischen Benutzer aus dem DaV
   * 
   * @param benutzername
   *          Benutzername
   * @return String mit der XML-Struktur der Voreinstellungen fr den Benutzer.
   */
  public String getVoreinstellungenBenutzerNetzweit(String benutzername)
  {
    String re = null;

    if (_connection != null)
    {
      if (_ebs == null)
      {
        _ebs = new EinstellungenBenutzerSpezifischNetzweit((ClientDavConnection)_connection);
      }
      else
      {
        _ebs.setDaVVerbindung((ClientDavConnection)_connection);
      }

      re = _ebs.getEintrag(benutzername);
    }

    return re;
  }

  /**
   * Setzt die XML-Struktur der Voreinstellungen fr einen spezifischen Benutzer im DaV.
   * 
   * @param einstellungen
   *          String mit der XML-Struktur der Voreinstellungen fr den Benutzer
   * @param benutzername
   *          Name des Benutzers.
   * @return Liefert <code>true</code>, wenn der Eintrag fr den Benutzer geschrieben werden konnte, sonst
   *         <code>false</code>.
   */
  public boolean setVoreinstellungenBenutzerNetzweit(String einstellungen, String benutzername)
  {
    boolean re = false;

    if (_connection != null)
    {
      if (_ebs == null)
      {
        _ebs = new EinstellungenBenutzerSpezifischNetzweit((ClientDavConnection)_connection);
      }
      else
      {
        _ebs.setDaVVerbindung((ClientDavConnection)_connection);
      }
      
      /*
       * Senden ohne UrlasserDialog?
       */
      _ebs.setSendenOhneUrlasserDialog(_sendenOhneUrlasserDlg);

      re = _ebs.editEintrag(benutzername, einstellungen);
    }

    return re;
  }

  /**
   * Liefert die XML-Struktur der allgemeinen, netzweiten Einstellungen aus dem DaV.
   * 
   * @return String mit der XML-Struktur der allgemeinen, netzweiten Voreinstellungen oder <code>null</code>, wenn
   *         keine Verbindung zum DaV existiert.
   */
  public String getEinstellungenAllgemeinNetzweit()
  {
  	
  	
    if (_connection != null)
    {
      if (_gen == null)
      {
        _gen = new GlobaleEinstellungenNetzweit((ClientDavConnection)_connection);
      }
      else
      {
        _gen.setDaVVerbindung((ClientDavConnection)_connection);
      }

      String xmlStruktur = _gen.empfangeXmlStruktur();

      return xmlStruktur;
    }
    else
    {
      return null;
    }
  }

  /**
   * Setzt die XML-Struktur der allgemeinen, netzweiten Einstellungen im DaV.
   * 
   * @param einstellungen
   *          String mit der XML-Struktur der allgemeinen, netzweiten Voreinstellungen, der gesetzt werden soll.
   * @return Liefert <code>true</code>, wenn die Einstellungen gespeichert werden konnten, sonst <code>false</code>
   *         (dann war i.d.R. keine Verbindung zum DaV mglich).
   */
  public boolean setVoreinstellungenAllgemeinNetzweit(String einstellungen)
  {
    boolean re = false;

    if (_connection != null)
    {
      if (_gen == null)
      {
        _gen = new GlobaleEinstellungenNetzweit((ClientDavConnection)_connection);
      }
      else
      {
        _gen.setDaVVerbindung((ClientDavConnection)_connection);
      }

      /*
       * Senden ohne UrlasserDialog?
       */
      _gen.setSendenOhneUrlasserDialog(_sendenOhneUrlasserDlg);
      
      _gen.sendeXmlStruktur(einstellungen);
      re = true;
    }

    return re;
  }

  /**
   * Prft, ob eine DaV-Verbindung existiert.<br>
   * Existiert diese nicht, z.B. weil der {@link IDaVVerbindungZuhoerer} zu
   * langsam benachrichtigt wurde, wird bei der DaVVerbindung direkt nach
   * der offenen Verbindung angefragt. Hierdurch ist gewhrleistet, dass
   * der Einstellungenspeicher schnell mit Daten aus dem DaV versorgt werden
   * kann.
   * 
   * @return Liefert <code>true</code>, wenn eine DaV-Verbindung existiert, sonst <code>false</code>.
   */
	public boolean istVerbunden()
	{
		boolean re = false;
		if (_connection != null)
		{
			re = true;
		} else
		{
			if (_connection == null)
			{
				/*
				 * Direkte Prfung bei der DaVVerbindung
				 */
				ClientDavInterface conNeu = DaVVerbindung.getInstanz()
						.getDaVVerbindung();
				if (conNeu != null)
				{
					_connection = conNeu;
					re = true;
				}
			}
		}
		return re;
	}
  
	/**
	 * Legt fest, ob mit oder ohne {@link UrlasserDialog} gesendet werden soll.
	 * 
	 * @param sendenOhneUrlasserDialog
	 *          Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *          gesendet, sonst nicht.
	 */
	public void setSendenOhneUrlasserDialog(boolean sendenOhneUrlasserDialog)
	{
		this._sendenOhneUrlasserDlg = sendenOhneUrlasserDialog;
	}

	/**
	 * Prft, ob mit oder ohne {@link UrlasserDialog} gesendet wird.
	 * 
	 * @return Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *         gesendet, sonst nicht.
	 */
	public boolean isSendenOhneUrlasserDialog()
	{
		return _sendenOhneUrlasserDlg;
	}

}
