/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;

public class TestBenutzerSpezifisch
{
  /**
   * Verbindung zum DaV
   */
  private ClientDavConnection _verbindung = null;

  /**
   * IP
   */
  private static final String _ip = "localhost";

  /**
   * Port
   */
  private static final int _port = 8083;

  /**
   * Login
   */
  private static final String _benutzer = "Tester";

  /**
   * Passwort
   */
  private static final String _passwort = "geheim";

  /**
   * @param args
   */
  public static void main(String[] args)
  {
    @SuppressWarnings("unused")
    TestBenutzerSpezifisch tbs = new TestBenutzerSpezifisch();
  }

  public TestBenutzerSpezifisch()
  {
    baueDaVVerbindungAuf();
    EinstellungenBenutzerSpezifischNetzweit ebs = new EinstellungenBenutzerSpezifischNetzweit(_verbindung);

    /*
     * Test Empfangen
     */
    // String eintrag = ebs.getEintrag("Tester");
    // System.out.println("Eintrag Tester:\n" + eintrag);
    //		
    // String eintragTestDatenverteilerBenutzer = ebs.getEintrag("TestDatenverteilerBenutzer");
    // System.out.println("Eintrag TestDatenverteilerBenutzer:\n" + eintragTestDatenverteilerBenutzer);
    /*
     * Test Senden
     */
    EinstellungenSpeicher sp = new EinstellungenSpeicher();
    // sp.hinzufuegenParameter("Benutzername", "TestDatenverteilerBenutzer");
    sp.hinzufuegenParameter("Benutzername", "Tester");

    String xmlStruktur = sp.ausgabeXmlFormatiert(true);
    System.out.println("XML: " + xmlStruktur);

    // ebs.editEintrag("Tester", xmlStruktur);
    // ebs.editEintrag("TestDatenverteilerBenutzer", xmlStruktur);
    ebs.editEintrag("Tester", xmlStruktur);

    schliesseDaVVerbindung();
  }

  /**
   * Baut die Verbindung zum DaV auf.
   */
  private void baueDaVVerbindungAuf()
  {
    try
    {
      _verbindung = new ClientDavConnection();

      // IP setzen
      _verbindung.getClientDavParameters().setDavCommunicationAddress(_ip);

      // Port setzen
      _verbindung.getClientDavParameters().setDavCommunicationSubAddress(_port);

      // Pfad zum Zwischenspeichern setzen
      _verbindung.getClientDavParameters().setConfigurationPath("C:\\temp");

      try
      {
        _verbindung.connect();
        System.out.println("Verbindungsaufbau: Verbindung erfolgreich aufgebaut.");

        try
        {
          _verbindung.login(_benutzer, _passwort);
          System.out.println("Verbindungsaufbau: Benutzer erfolgreich eingeloggt.");

        }
        catch (InconsistentLoginException e)
        {
          System.err.println("Verbindungsaufbau Fehler: Konnte Benutzer nicht einloggen.");
          _verbindung = null;
        }

      }
      catch (CommunicationError e)
      {
        System.err.println("Verbindungsaufbau Fehler: " + e.toString());
        _verbindung = null;
      }
      catch (ConnectionException e)
      {
        System.err.println("Verbindungsaufbau Fehler: " + e.toString());
        _verbindung = null;
      }

    }
    catch (MissingParameterException e)
    {
      System.err.println("Verbindungsaufbau Fehler: " + e.toString());
      _verbindung = null;
    }
  }

  /**
   * Schliet die Verbindung zum DaV
   */
  private void schliesseDaVVerbindung()
  {
    if (_verbindung != null)
    {
      _verbindung.disconnect(false, "Kein Fehler.");
      System.out.println("\nVerbindungsabbau: Verbindung wurde abgebaut.");
    }
  }

}
