/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.kalender.DatumZeit.Eingabetyp;

/**
 * Composite zur Darstellung von Zeitbereichen mit Plausibilittsprfung. Ist
 * Endzeitpunkt < Startzeitpunkt, wird eine entsprechende Meldung dargestellt.
 * 
 * @author HofmannJ
 * 
 */
public class CompositeZeitBereich extends Composite
{

	private Label _labelVon = null;
	// private Text _textVon = null;
	private DatumZeit _von = null;
	private Label _labelBis = null;
	// private Text _textBis = null;
	private DatumZeit _bis = null;
	private Label _labelInfo = null;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *          Eltern {@link Composite}.
	 * @param style
	 *          SWT-Stil.
	 */
	public CompositeZeitBereich(Composite parent, int style)
	{
		super(parent, style);
		initialize();
	}

	/**
	 * Fhrt die Initialisierugn des GUI dieses Composites durch.
	 */
	private void initialize()
	{
		GridData gridDataBis = new GridData();
		gridDataBis.horizontalAlignment = GridData.BEGINNING;
		gridDataBis.grabExcessHorizontalSpace = false;
		gridDataBis.verticalAlignment = GridData.CENTER;
		GridData gridDataVon = new GridData();
		gridDataVon.horizontalAlignment = GridData.BEGINNING;
		gridDataVon.grabExcessHorizontalSpace = false;
		gridDataVon.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		_labelVon = new Label(this, SWT.NONE);
		_labelVon.setText("Von:");
		// _textVon = new Text(this, SWT.BORDER);
		// _textVon.setLayoutData(gridDataVon);
		_von = new DatumZeit(this, SWT.NONE, Eingabetyp.datumuhrEinfachMS, false,
				false);
		_von.setLayoutData(gridDataVon);

		_labelBis = new Label(this, SWT.NONE);
		_labelBis.setText("Bis:");
		// _textBis = new Text(this, SWT.BORDER);
		// _textBis.setLayoutData(gridDataBis);

		_bis = new DatumZeit(this, SWT.NONE, Eingabetyp.datumuhrEinfachMS, false,
				false);
		_bis.setLayoutData(gridDataBis);

		_labelInfo = new Label(this, SWT.NONE);
		_labelInfo.setForeground(new Color(PlatformUI.getWorkbench().getDisplay(),
				255, 0, 0));
		_labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
		_labelInfo.setVisible(false);

		_von.hinzufuegenSelektionsZuhoerer(new SelectionListener()
		{

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				gemeinsam(e);
			}

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				gemeinsam(e);
			}

			private void gemeinsam(SelectionEvent e)
			{
				if (e.data instanceof Date)
				{
					Date startDatum = (Date) e.data;
					Date endDatum = _bis.getDatum();
					labelSetzen(startDatum, endDatum);

				}
			}

		});

		_bis.hinzufuegenSelektionsZuhoerer(new SelectionListener()
		{

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				gemeinsam(e);
			}

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				gemeinsam(e);
			}

			private void gemeinsam(SelectionEvent e)
			{
				if (e.data instanceof Date)
				{
					Date startDatum = _von.getDatum();
					Date endDatum = (Date) e.data;
					labelSetzen(startDatum, endDatum);

				}
			}

		});

		this.setLayout(gridLayout);
	}

	/**
	 * Setzt den Startzeitpunkt.
	 * 
	 * @param start
	 *          Startzeitpunkt.
	 */
	public void setzeStartZeitPunkt(Date start)
	{
		_von.setDatum(start);
		labelSetzen(_von.getDatum(), _bis.getDatum());
	}

	/**
	 * Liefert den Startzeitpunkt.
	 * 
	 * @return Startzeitpunkt.
	 */
	public Date getStartZeitPunkt()
	{
		return _von.getDatum();
	}

	/**
	 * Setzt den Endzeitpunkt.
	 * 
	 * @param ende
	 *          Endzeitpunkt.
	 */
	public void setzeEndZeitPunkt(Date ende)
	{
		_bis.setDatum(ende);
		labelSetzen(_von.getDatum(), _bis.getDatum());
	}

	/**
	 * Liefert den Endzeitpunkt.
	 * 
	 * @return Endzeitpunkt.
	 */
	public Date getEndZeitPunkt()
	{
		return _bis.getDatum();
	}

	/**
	 * Zeigt oder versteckt das Label zur Darstellung, dass die
	 * Plausibilittsprfung gescheitert ist.
	 * 
	 * @param start
	 *          Startzeitpunkt.
	 * @param ende
	 *          Endzeitpunkt.
	 */
	private void labelSetzen(Date start, Date ende)
	{
		if (pruefePlausibilitaet(start, ende))
		{
			_labelInfo.setVisible(false);
		} else
		{
			_labelInfo.setVisible(true);
		}
	}

	/**
	 * Prft die Plausibilitt von Start- und Endzeitpunkt.
	 * 
	 * @param start
	 *          Startzeitpunkt.
	 * @param ende
	 *          Endzeitpunkt.
	 * @return Liefert <code>true</code> wenn plausiebel, sonst <code>false</code>
	 *         .
	 */
	private boolean pruefePlausibilitaet(Date start, Date ende)
	{
		boolean re = false;

		if ((start != null) && (ende != null))
		{
			if (!(start.compareTo(ende) > 0))
			{
				re = true;
			}
		}

		return re;
	}

}
