/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Widget;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert das eigentliche Composite, das hinter den Zeitdialog steht. Bei den
 * Stunden, Minuten & Sekunden Elementen dieses Dialogs handelt es sich
 * letztendlich um Labels, die mit einem MouseListener austestattet wurden.
 * 
 * @author HofmannJ
 * 
 */
public class ZeitDauerWaehler extends Composite implements MouseListener,
		FocusListener
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	// Farbdefinitionen
	private Color _aktiveAuswahlHintergrund;

	private Color _inaktiveAuswahlHintergrund;

	private Color _aktiveAuswahlVordergrund;

	private Color _inaktiveAuswahlVordergrund;

	private Color _textVordergrundFarbe;

	private Color _textHintergrundFarbe;

	// Styledefinition
	@SuppressWarnings("unused")
	private int _stil;

	// Composites
	private Composite _compositeJahreInfo;

	private Composite _compositeJahre;

	private Composite _compositeMonateInfo;

	private Composite _compositeMonate;

	private Composite _compositeTageInfo;

	private Composite _compositeTage;

	private Composite _compositeStundenInfo;

	private Composite _compositeStunden;

	private Composite _compositeMinutenInfo;

	private Composite _compositeMinuten;

	private Composite _compositeSekundenInfo;

	private Composite _compositeSekunden;

	// Elemente
	private JahreKontrolle[] _jahre;

	private MonateKontrolle[] _monate;

	private TageKontrolle[] _tage;

	private StundenKontrolle[] _stunden;

	private MinutenKontrolle[] _minuten;

	private SekundenKontrolle[] _sekunden;

	// Alte Selektionen
	private int _altesSelektiertesJahr = -1;

	private int _neuesSelektiertesJahr = -1;

	private int _alterSelektierterMonat = -1;

	private int _neuerSelektierterMonat = -1;

	private int _alterSelektierterTag = -1;

	private int _neuerSelektierterTag = -1;

	private int _alteSelektierteStunde = -1;

	private int _neueSelektierteStunde = -1;

	private int _alteSelektierteMinute = -1;

	private int _neueSelektierteMinute = -1;

	private int _alteSelektierteSekunde = -1;

	private int _neueSelektierteSekunde = -1;

	/**
	 * Beinhaltet die Datenstruktur der Darstellung
	 */
	private CompositeZeitDauer _zeitDauerComposite;

	/**
	 * Granularitt
	 */
	private ZeitDauerGranularitaet _granularitaet = ZeitDauerGranularitaet.jahr;

	/**
	 * Speichert die Zeitspanne in Millisekunden
	 */
	// private Long _zeitDauerInMillisekunden;
	public ZeitDauerWaehler(Composite parent, int style, CompositeZeitDauer dauer)
	{
		super(parent, style);

		this._stil = style;
		this._zeitDauerComposite = dauer;
		this._granularitaet = dauer.getGranularitaet();

		erzeugeDarstellung(getParent());

		// INIT-TRICK
		// setzeZeit(0, 0, 0, 0, 0, 0);

		int jahre = _zeitDauerComposite.getJahre();
		int monate = _zeitDauerComposite.getMonate();
		int tage = _zeitDauerComposite.getTage();
		int stunden = _zeitDauerComposite.getStunden();
		int minuten = _zeitDauerComposite.getMinuten();
		int sekunden = _zeitDauerComposite.getSekunden();

		System.out.println("J: " + jahre + " M: " + monate + " T: " + tage + " h: "
				+ stunden + " m: " + minuten + " s: " + sekunden);

		setzeZeit(jahre, monate, tage, stunden, minuten, sekunden);
		jahrZuruecksetzen(0);
		monatZuruecksetzen(0);
		tagZuruecksetzen(0);
		stundeZuruecksetzen(0);
		minuteZuruecksetzen(0);
		sekundeZuruecksetzen(0);

		initKalender();

	}

	private void erzeugeDarstellung(Composite parent)
	{

		setBackground(getDisplay().getSystemColor(SWT.COLOR_WHITE));

		_aktiveAuswahlHintergrund = getDisplay().getSystemColor(
				SWT.COLOR_LIST_SELECTION);
		_inaktiveAuswahlHintergrund = getDisplay().getSystemColor(SWT.COLOR_GRAY);
		_aktiveAuswahlVordergrund = getDisplay().getSystemColor(
				SWT.COLOR_LIST_SELECTION_TEXT);
		_inaktiveAuswahlVordergrund = getForeground();

		_textVordergrundFarbe = getDisplay().getSystemColor(
				SWT.COLOR_LIST_FOREGROUND);
		_textHintergrundFarbe = getDisplay().getSystemColor(
				SWT.COLOR_LIST_BACKGROUND);

		// 3 Zeilen fr die Darstellung der Composites von Stunden, Minuten und
		// Sekunden
		GridLayout gridLayout = new GridLayout();
		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.numColumns = 1;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		setLayout(gridLayout);

		// ////
		// Composites definieren

		// composite JahreInfo
		_compositeJahreInfo = new Composite(this, SWT.NONE);
		final Label SeparatorJahreInfo = new Label(_compositeJahreInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		SeparatorJahreInfo.setLayoutData(gridData);
		final Label labelJahre = new Label(_compositeJahreInfo, SWT.NONE);
		labelJahre.setText("Jahre");
		_compositeJahreInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// composite Jahre
		_compositeJahre = new Composite(this, SWT.NONE);
		_compositeJahre.setBackground(getDisplay().getSystemColor(SWT.COLOR_WHITE));

		// compositeMonateInfo
		_compositeMonateInfo = new Composite(this, SWT.NONE);
		final Label SeparatorMonateInfo = new Label(_compositeMonateInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		SeparatorMonateInfo.setLayoutData(gridData);
		final Label labelMonate = new Label(_compositeMonateInfo, SWT.NONE);
		labelMonate.setText("Monate");
		_compositeMonateInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeMonate
		_compositeMonate = new Composite(this, SWT.NONE);
		_compositeMonate
				.setBackground(getDisplay().getSystemColor(SWT.COLOR_WHITE));

		// compositeTageInfo
		_compositeTageInfo = new Composite(this, SWT.NONE);
		final Label SeparatorTageInfo = new Label(_compositeTageInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		SeparatorTageInfo.setLayoutData(gridData);
		final Label labelTage = new Label(_compositeTageInfo, SWT.NONE);
		labelTage.setText("Tage");
		_compositeTageInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeTage
		_compositeTage = new Composite(this, SWT.NONE);
		_compositeTage.setBackground(getDisplay().getSystemColor(SWT.COLOR_WHITE));

		// compositeStundenInfo
		_compositeStundenInfo = new Composite(this, SWT.NONE);
		final Label SeparatorStundenInfo = new Label(_compositeStundenInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		SeparatorStundenInfo.setLayoutData(gridData);
		final Label labelStunden = new Label(_compositeStundenInfo, SWT.NONE);
		labelStunden.setText("Stunden");
		_compositeStundenInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeStunden
		_compositeStunden = new Composite(this, SWT.NONE);
		_compositeStunden.setBackground(getDisplay()
				.getSystemColor(SWT.COLOR_WHITE));

		// compositeMinutenInfo
		_compositeMinutenInfo = new Composite(this, SWT.NONE);
		final Label SeparatorMinutenInfo = new Label(_compositeMinutenInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		SeparatorMinutenInfo.setLayoutData(gridData);
		final Label labelMinuten = new Label(_compositeMinutenInfo, SWT.NONE);
		labelMinuten.setText("Minuten");
		_compositeMinutenInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeMinuten
		_compositeMinuten = new Composite(this, SWT.NONE);
		_compositeMinuten.setBackground(getDisplay()
				.getSystemColor(SWT.COLOR_WHITE));

		// compositeSekundeniNFO
		_compositeSekundenInfo = new Composite(this, SWT.NONE);
		final Label SeparatorSekundenInfo = new Label(_compositeSekundenInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		SeparatorSekundenInfo.setLayoutData(gridData);
		final Label labelSekunden = new Label(_compositeSekundenInfo, SWT.NONE);
		labelSekunden.setText("Sekunden");
		_compositeSekundenInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeSekunden
		_compositeSekunden = new Composite(this, SWT.NONE);
		_compositeSekunden.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// 1 Spalte fr JahrInfo
		{
			final GridData gridDataJahreInfo = new GridData(GridData.FILL_HORIZONTAL);
			_compositeJahreInfo.setLayoutData(gridDataJahreInfo);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 1;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeJahreInfo.setLayout(gridLayoutTopHeader);
		}

		// 10 Spalten fr die Jahre
		{
			final GridData gridDataJahre = new GridData(GridData.FILL_HORIZONTAL);
			_compositeJahre.setLayoutData(gridDataJahre);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 10;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeJahre.setLayout(gridLayoutTopHeader);
		}

		// 1 Spalte fr MonateInfo
		{
			final GridData gridDataMonateInfo = new GridData(GridData.FILL_HORIZONTAL);
			_compositeMonateInfo.setLayoutData(gridDataMonateInfo);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 1;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeMonateInfo.setLayout(gridLayoutTopHeader);
		}

		// 12 Spalten fr die Monate
		{
			final GridData gridDataMonate = new GridData(GridData.FILL_HORIZONTAL);
			_compositeMonate.setLayoutData(gridDataMonate);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 10;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeMonate.setLayout(gridLayoutTopHeader);
		}

		// 1 Spalte fr TageInfo
		{
			final GridData gridDataTageInfo = new GridData(GridData.FILL_HORIZONTAL);
			_compositeTageInfo.setLayoutData(gridDataTageInfo);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 1;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeTageInfo.setLayout(gridLayoutTopHeader);
		}

		// 10 Spalten fr die Tage
		{
			final GridData gridDataTage = new GridData(GridData.FILL_HORIZONTAL);
			_compositeTage.setLayoutData(gridDataTage);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 10;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeTage.setLayout(gridLayoutTopHeader);
		}

		// 1 Spalte fr StundenInfo
		{
			final GridData gridDataStundenInfo = new GridData(
					GridData.FILL_HORIZONTAL);
			_compositeStundenInfo.setLayoutData(gridDataStundenInfo);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 1;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeStundenInfo.setLayout(gridLayoutTopHeader);
		}

		// 10 Spalten fr die Stunden
		{
			final GridData gridDataStunden = new GridData(GridData.FILL_HORIZONTAL);
			_compositeStunden.setLayoutData(gridDataStunden);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 10;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeStunden.setLayout(gridLayoutTopHeader);
		}

		// 2 Spalten fr die Minuteninfo
		{
			final GridData gridDataMinutenInfo = new GridData(
					GridData.FILL_HORIZONTAL);
			_compositeMinutenInfo.setLayoutData(gridDataMinutenInfo);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 1;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeMinutenInfo.setLayout(gridLayoutTopHeader);
		}

		// 10 Spalten fr die Minuten zu 6 Zeilen
		{
			final GridData gridDataMinuten = new GridData(GridData.FILL_HORIZONTAL);
			_compositeMinuten.setLayoutData(gridDataMinuten);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 10;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeMinuten.setLayout(gridLayoutTopHeader);
		}

		// 2 Spalten fr die SekundenInfo
		{
			final GridData gridDataSekundenInfo = new GridData(
					GridData.FILL_HORIZONTAL);
			_compositeSekundenInfo.setLayoutData(gridDataSekundenInfo);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 1;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeSekundenInfo.setLayout(gridLayoutTopHeader);
		}

		// 10 Spalten fr die Sekunden zu 6 Zeilen
		{
			final GridData gridDataSekunden = new GridData(GridData.FILL_HORIZONTAL);
			_compositeSekunden.setLayoutData(gridDataSekunden);
			final GridLayout gridLayoutTopHeader = new GridLayout();
			gridLayoutTopHeader.numColumns = 10;
			gridLayoutTopHeader.marginWidth = 0;
			gridLayoutTopHeader.marginHeight = 0;
			_compositeSekunden.setLayout(gridLayoutTopHeader);
		}

		// Jahrekontrolle anlegen
		_jahre = new JahreKontrolle[20];
		for (int i = 0; i < _jahre.length; i++)
		{
			JahreKontrolle jahr = new JahreKontrolle(_compositeJahre);
			_jahre[i] = jahr;
			jahr.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			jahr.addMouseListener(this);
		}

		// Monatekontrolle anlegen
		_monate = new MonateKontrolle[13];
		for (int i = 0; i < _monate.length; i++)
		{
			MonateKontrolle monat = new MonateKontrolle(_compositeMonate);
			_monate[i] = monat;
			monat.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			monat.addMouseListener(this);
		}

		// Tagekontrolle anlegen
		_tage = new TageKontrolle[32];
		for (int i = 0; i < _tage.length; i++)
		{
			TageKontrolle tag = new TageKontrolle(_compositeTage);
			_tage[i] = tag;
			tag.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			tag.addMouseListener(this);
		}

		// Stundenkontrolle anlegen
		_stunden = new StundenKontrolle[24];
		for (int i = 0; i < _stunden.length; i++)
		{
			StundenKontrolle stunde = new StundenKontrolle(_compositeStunden);
			_stunden[i] = stunde;
			stunde.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			stunde.addMouseListener(this);
		}

		// Minutenkontrolle anlegen
		_minuten = new MinutenKontrolle[60];
		for (int i = 0; i < _minuten.length; i++)
		{
			MinutenKontrolle minute = new MinutenKontrolle(_compositeMinuten);
			_minuten[i] = minute;
			minute.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			minute.addMouseListener(this);
		}

		// Sekundenkontrolle anlegen
		_sekunden = new SekundenKontrolle[60];
		for (int i = 0; i < _sekunden.length; i++)
		{
			SekundenKontrolle sekunde = new SekundenKontrolle(_compositeSekunden);
			_sekunden[i] = sekunde;
			sekunde.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			sekunde.addMouseListener(this);
		}

		setFont(parent.getFont());
		init();

		addMouseListener(this);
		addFocusListener(this);

	}

	/**
	 * Initialisiert die verwendeten Kalender
	 * 
	 * @param dauer
	 *          Gibt die Datenstrukturklasse an.
	 * @param gran
	 *          Gibt die Granularitt der Anzeige an.
	 * 
	 */
	private void initKalender()
	{
		verwalteJahre(_zeitDauerComposite.getJahre());
		waehleJahrLaufzeit(_zeitDauerComposite.getJahre());
		verwalteMonate(_zeitDauerComposite.getMonate());
		waehleMonatLaufzeit(_zeitDauerComposite.getMonate());
		verwalteTage(_zeitDauerComposite.getTage());
		waehleTagLaufzeit(_zeitDauerComposite.getTage());
		verwalteStunden(_zeitDauerComposite.getStunden());
		waehleStundeLaufzeit(_zeitDauerComposite.getStunden());
		verwalteMinuten(_zeitDauerComposite.getMinuten());
		waehleMinuteLaufzeit(_zeitDauerComposite.getMinuten());
		verwalteSekunden(_zeitDauerComposite.getSekunden());
		waehleSekundeLaufzeit(_zeitDauerComposite.getSekunden());
	}

	protected void passeDarstellungAnGranularitaetAn()
	{
		switch (_granularitaet)
		{
		case jahr:
			break;
		case sekunde:
			_compositeMinutenInfo.setVisible(false);
			_compositeMinuten.setVisible(false);
		case minute:
			_compositeStundenInfo.setVisible(false);
			_compositeStunden.setVisible(false);
		case stunde:
			_compositeTageInfo.setVisible(false);
			_compositeTage.setVisible(false);
		case tag:
			_compositeMonateInfo.setVisible(false);
			_compositeMonate.setVisible(false);
		case monat:
			_compositeJahreInfo.setVisible(false);
			_compositeJahre.setVisible(false);
			break;
		}

	}

	private void init()
	{
		zeichneJahre();
		zeichneMonate();
		zeichneTage();
		zeichneStunden();
		zeichneMinuten();
		zeichneSekunden();
	}

	/**
	 * Zeichnet die Jahre aus dem Array <code>jahre</code>
	 * 
	 */
	private void zeichneJahre()
	{
		for (int i = 0; i < _jahre.length; i++)
		{
			JahreKontrolle jahrKontrolle = _jahre[i];
			jahrKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Monate aus dem Array <code>monate</code>
	 * 
	 */
	private void zeichneMonate()
	{
		for (int i = 0; i < _monate.length; i++)
		{
			MonateKontrolle monatKontrolle = _monate[i];
			monatKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Tage aus dem Array <code>tage</code>
	 * 
	 */
	private void zeichneTage()
	{
		for (int i = 0; i < _tage.length; i++)
		{
			TageKontrolle tagKontrolle = _tage[i];
			tagKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Stunden aus dem Array <code>stunden</code>
	 * 
	 */
	private void zeichneStunden()
	{
		for (int i = 0; i < _stunden.length; i++)
		{
			StundenKontrolle stuKontrolle = _stunden[i];
			stuKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Minuten aus dem Array <code>minuten</code>
	 * 
	 */
	private void zeichneMinuten()
	{
		for (int i = 0; i < _minuten.length; i++)
		{
			MinutenKontrolle minKontrolle = _minuten[i];
			minKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Sekunden aus dem Array <code>sekunden</code>
	 * 
	 */
	private void zeichneSekunden()
	{
		for (int i = 0; i < _sekunden.length; i++)
		{
			SekundenKontrolle sekKontrolle = _sekunden[i];
			sekKontrolle.setText(String.valueOf(i));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
	 */
	public void setFont(Font font)
	{
		super.setFont(font);

		// Jahre
		for (JahreKontrolle jahreKontrolle : _jahre)
		{
			jahreKontrolle.setFont(font);
		}

		// Monate
		for (MonateKontrolle monateKontrolle : _monate)
		{
			monateKontrolle.setFont(font);
		}

		// Tage
		for (TageKontrolle tageKontrolle : _tage)
		{
			tageKontrolle.setFont(font);
		}

		// Stunden
		for (int i = 0; i < _stunden.length; i++)
		{
			_stunden[i].setFont(font);
		}

		// Minuten
		for (MinutenKontrolle minutenKontrolle : _minuten)
		{
			minutenKontrolle.setFont(font);
		}

		// Sekunden
		for (SekundenKontrolle sekundenKontrolle : _sekunden)
		{
			sekundenKontrolle.setFont(font);
		}

	}

	/**
	 * mouseDown
	 */
	public void mouseDown(MouseEvent event)
	{
		if (event.button == 1)
		{
			setFocus();

			// JahreKontrolle
			if (event.widget instanceof JahreKontrolle)
			{
				int index = findeJahr(event.widget);
				_debug.fine("Jahr " + index + " wurde geklickt!");

				verwalteJahre(index);

				// waehleJahr(index);
				waehleJahrLaufzeit(index);
			}

			// MonateKontrolle
			if (event.widget instanceof MonateKontrolle)
			{
				int index = findeMonat(event.widget);
				_debug.fine("Monat " + index + " wurde geklickt!");

				verwalteMonate(index);

				// waehleMonat(index);
				waehleMonatLaufzeit(index);
			}

			// TageKontrolle
			if (event.widget instanceof TageKontrolle)
			{
				int index = findeTag(event.widget);
				_debug.fine("Tag " + index + " wurde geklickt!");

				verwalteTage(index);

				// waehleTag(index);
				waehleTagLaufzeit(index);
			}

			// StundenKontrolle
			if (event.widget instanceof StundenKontrolle)
			{
				int index = findeStunde(event.widget);
				_debug.fine("Stunde " + index + " wurde geklickt!");

				verwalteStunden(index);

				// waehleStunde(index);
				waehleStundeLaufzeit(index);
			}

			// MinutenKontrolle
			if (event.widget instanceof MinutenKontrolle)
			{
				int index = findeMinute(event.widget);
				_debug.fine("Minute " + index + " wurde geklickt!");

				verwalteMinuten(index);

				// waehleMinute(index);
				waehleMinuteLaufzeit(index);

			}

			// Sekundenkontrolle
			if (event.widget instanceof SekundenKontrolle)
			{
				int index = findeSekunde(event.widget);
				_debug.fine("Sekunde " + index + " wurde geklickt!");

				verwalteSekunden(index);

				// waehleSekunde(index);
				waehleSekundeLaufzeit(index);
			}

		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDoubleClick(MouseEvent event)
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseUp(MouseEvent event)
	{
	}

	public void focusGained(FocusEvent event)
	{
		// if (event.widget instanceof StundenKontrolle){
		int index = findeStunde(event.widget);
		_stunden[index].setBackground(getSelectionBackgroundColor());
		_stunden[index].setForeground(getSelectionForegroundColor());

		// }
		_debug.fine("Focus gained");
	}

	public void focusLost(FocusEvent event)
	{
		// if (event.widget instanceof StundenKontrolle){
		int index = findeStunde(event.widget);
		_stunden[index].setBackground(getSelectionBackgroundColor());
		_stunden[index].setForeground(getSelectionForegroundColor());

		// }
		_debug.fine("FocusLost");
	}

	/**
	 * Setzt die Zeit im Kalender. Dabei wird <code>calendar</code> und die
	 * grafische Darstellung aktuallisiert.
	 * 
	 * @param Stunde
	 *          Zu setzende Stunde
	 * @param Minute
	 *          Zu setzende Minute
	 * @param Sekunde
	 *          Zu setzemde Sekunde
	 */
	public void setzeZeit(int Jahre, int Monate, int Tage, int Stunde,
			int Minute, int Sekunde)
	{
		verwalteJahre(Jahre);
		verwalteMonate(Monate);
		verwalteTage(Tage);
		verwalteStunden(Stunde);
		verwalteMinuten(Minute);
		verwalteSekunden(Sekunde);

		waehleJahr(Jahre);
		waehleMonat(Monate);
		waehleTag(Tage);
		waehleStunde(Stunde);
		waehleMinute(Minute);
		waehleSekunde(Sekunde);

	}

	/**
	 * Setzt in den internen Variablen <code>altesSelektiertesjtunde</code> und
	 * <code>neueSelektiertesJahr</code> die entsprechenden Werte
	 * 
	 * @param index
	 *          Index des neuen Jahres
	 */
	private void verwalteJahre(int index)
	{
		if ((_altesSelektiertesJahr == -1) & (_neuesSelektiertesJahr == -1))
		{
			_neuesSelektiertesJahr = index;
		} else
		{
			_altesSelektiertesJahr = _neuesSelektiertesJahr;
			_neuesSelektiertesJahr = index;
			jahrZuruecksetzen(_altesSelektiertesJahr);
		}

		_debug.fine("Altes Jahr: " + _altesSelektiertesJahr + " Neues Jahr: "
				+ _neuesSelektiertesJahr);

		if (_zeitDauerComposite != null)
		{
			_zeitDauerComposite.setJahre(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alterSelektierterMonat</code> und
	 * <code>neuerSelektierterMonat</code> die entsprechenden Werte
	 * 
	 * @param index
	 *          Index des neuen Monats
	 */
	private void verwalteMonate(int index)
	{
		if ((_alterSelektierterMonat == -1) & (_neuerSelektierterMonat == -1))
		{
			_neuerSelektierterMonat = index;
		} else
		{
			_alterSelektierterMonat = _neuerSelektierterMonat;
			_neuerSelektierterMonat = index;
			monatZuruecksetzen(_alterSelektierterMonat);
		}

		_debug.fine("Alter Monat: " + _alterSelektierterMonat + " Neuer Monat: "
				+ _neuerSelektierterMonat);

		if (_zeitDauerComposite != null)
		{
			_zeitDauerComposite.setMonate(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alterSelektierterTag</code> und
	 * <code>neuerSelektierterTag</code> die entsprechenden Werte
	 * 
	 * @param index
	 *          Index des neuen Tags
	 */
	private void verwalteTage(int index)
	{
		if ((_alterSelektierterTag == -1) & (_neuerSelektierterTag == -1))
		{
			_neuerSelektierterTag = index;
		} else
		{
			_alterSelektierterTag = _neuerSelektierterTag;
			_neuerSelektierterTag = index;
			tagZuruecksetzen(_alterSelektierterTag);
		}

		_debug.fine("Alter Tag: " + _alterSelektierterTag + " Neuer Tag: "
				+ _neuerSelektierterTag);

		if (_zeitDauerComposite != null)
		{
			_zeitDauerComposite.setTage(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alteSelektierteStunde</code> und
	 * <code>neueSelektierteStunde</code> die entsprechenden Werte
	 * 
	 * @param index
	 *          Index der neuen Stunde
	 */
	private void verwalteStunden(int index)
	{
		if ((_alteSelektierteStunde == -1) & (_neueSelektierteStunde == -1))
		{
			_neueSelektierteStunde = index;
		} else
		{
			_alteSelektierteStunde = _neueSelektierteStunde;
			_neueSelektierteStunde = index;
			stundeZuruecksetzen(_alteSelektierteStunde);
		}

		_debug.fine("Alte Stunde: " + _alteSelektierteStunde + " Neue Stunde: "
				+ _neueSelektierteStunde);

		if (_zeitDauerComposite != null)
		{
			_zeitDauerComposite.setStunden(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alteSelektierteMinute</code> und
	 * <code>neueSelektierteMinute</code> die entsprechenden Werte
	 * 
	 * @param index
	 *          Index der neuen Minute
	 */
	private void verwalteMinuten(int index)
	{
		if ((_alteSelektierteMinute == -1) & (_neueSelektierteMinute == -1))
		{
			_neueSelektierteMinute = index;
		} else
		{
			_alteSelektierteMinute = _neueSelektierteMinute;
			_neueSelektierteMinute = index;
			minuteZuruecksetzen(_alteSelektierteMinute);
		}

		_debug.fine("AlteMinute: " + _alteSelektierteMinute + " Neue Minute: "
				+ _neueSelektierteMinute);

		if (_zeitDauerComposite != null)
		{
			_zeitDauerComposite.setMinuten(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alteSelektierteSekunde</code> und
	 * <code>neueSelektierteSekunde</code> die entsprechenden Werte
	 * 
	 * @param index
	 *          Index der neuen Sekunde
	 */
	private void verwalteSekunden(int index)
	{
		if ((_alteSelektierteSekunde == -1) & (_neueSelektierteSekunde == -1))
		{
			_neueSelektierteSekunde = index;
		} else
		{
			_alteSelektierteSekunde = _neueSelektierteSekunde;
			_neueSelektierteSekunde = index;
			sekundeZuruecksetzen(_alteSelektierteSekunde);
		}

		_debug.fine("Alte Sekunde: " + _alteSelektierteSekunde + " Neue Sekunde: "
				+ _neueSelektierteSekunde);

		if (_zeitDauerComposite != null)
		{
			_zeitDauerComposite.setSekunden(index);
		}
	}

	/**
	 * Findet die Position eines Controls im <code>jahre</code> Array.
	 * 
	 * @param jahrKontrolle
	 *          Das Widget, dessen zugehriges Jahrelement gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeJahr(Widget jahrKontrolle)
	{
		for (int i = 0; i < _jahre.length; i++)
		{
			if (_jahre[i] == jahrKontrolle)
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>monate</code> Array.
	 * 
	 * @param monatKontrolle
	 *          Das Widget, dessen zugehriges Monatelement gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeMonat(Widget monatKontrolle)
	{
		for (int i = 0; i < _monate.length; i++)
		{
			if (_monate[i] == monatKontrolle)
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>tage</code> Array.
	 * 
	 * @param tagKontrolle
	 *          Das Widget, dessen zugehriges Tagelement gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeTag(Widget tagKontrolle)
	{
		for (int i = 0; i < _tage.length; i++)
		{
			if (_tage[i] == tagKontrolle)
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>stunden</code> Array.
	 * 
	 * @param stuKontrolle
	 *          Das Widget, dessen zugehriges Stundenelement gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeStunde(Widget stuKontrolle)
	{
		for (int i = 0; i < _stunden.length; i++)
		{
			if (_stunden[i] == stuKontrolle)
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>minuten</code> Array.
	 * 
	 * @param minKontrolle
	 *          Das Widget, dessen zugehriges Minutenelement gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeMinute(Widget minKontrolle)
	{
		for (int i = 0; i < _minuten.length; i++)
		{
			if (_minuten[i] == minKontrolle)
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>sekunden</code> Array.
	 * 
	 * @param sekKontrolle
	 *          Das Widget, dessen zugehriges Sekundenelement gefunden werden
	 *          soll
	 * @return Index des gefundenen Elements
	 */
	private int findeSekunde(Widget sekKontrolle)
	{
		for (int i = 0; i < _sekunden.length; i++)
		{
			if (_sekunden[i] == sekKontrolle)
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Whlt das Jahr aus und markiert es
	 * 
	 * @param iJahr
	 *          Index des Jahres
	 */
	private void waehleJahr(int iJahr)
	{
		JahreKontrolle gewaehltesJahr = _jahre[iJahr];

		gewaehltesJahr.setBackground(getSelectionBackgroundColor());
		gewaehltesJahr.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt den Monat aus und markiert ihn
	 * 
	 * @param iMonat
	 *          Index des Monats
	 */
	private void waehleMonat(int iMonat)
	{
		MonateKontrolle gewaehlterMonat = _monate[iMonat];

		gewaehlterMonat.setBackground(getSelectionBackgroundColor());
		gewaehlterMonat.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt den Tag aus und markiert ihn
	 * 
	 * @param iTag
	 *          Index des Tags
	 */
	private void waehleTag(int iTag)
	{
		TageKontrolle gewaehlterTag = _tage[iTag];

		gewaehlterTag.setBackground(getSelectionBackgroundColor());
		gewaehlterTag.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt die Stunde aus und markiert sie
	 * 
	 * @param iStunde
	 *          Index der Stunde
	 */
	private void waehleStunde(int iStunde)
	{
		StundenKontrolle gewaehlteStunde = _stunden[iStunde];

		gewaehlteStunde.setBackground(getSelectionBackgroundColor());
		gewaehlteStunde.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt die Minute aus und markiert sie
	 * 
	 * @param iMinute
	 *          Index der Minute
	 */
	private void waehleMinute(int iMinute)
	{
		MinutenKontrolle gewaehlteMinute = _minuten[iMinute];
		gewaehlteMinute.setBackground(getSelectionBackgroundColor());
		gewaehlteMinute.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt die Sekunde aus und markiert sie
	 * 
	 * @param iSekunde
	 *          Index der Sekunde
	 */
	private void waehleSekunde(int iSekunde)
	{
		SekundenKontrolle gewaehlteSekunde = _sekunden[iSekunde];
		gewaehlteSekunde.setBackground(getSelectionBackgroundColor());
		gewaehlteSekunde.setForeground(getSelectionForegroundColor());

	}

	private void waehleJahrLaufzeit(int iJahr)
	{
		JahreKontrolle gewaehltesJahr = _jahre[iJahr];

		gewaehltesJahr.setBackground(getSelectionBackgroundColor());
		gewaehltesJahr.setForeground(getSelectionForegroundColor());

	}

	private void waehleMonatLaufzeit(int iMonat)
	{
		MonateKontrolle gewaehlterMonat = _monate[iMonat];

		gewaehlterMonat.setBackground(getSelectionBackgroundColor());
		gewaehlterMonat.setForeground(getSelectionForegroundColor());

	}

	private void waehleTagLaufzeit(int iTag)
	{
		TageKontrolle gewaehlterTag = _tage[iTag];

		gewaehlterTag.setBackground(getSelectionBackgroundColor());
		gewaehlterTag.setForeground(getSelectionForegroundColor());

	}

	private void waehleStundeLaufzeit(int iStunde)
	{
		StundenKontrolle gewaehlteStunde = _stunden[iStunde];

		gewaehlteStunde.setBackground(getSelectionBackgroundColor());
		gewaehlteStunde.setForeground(getSelectionForegroundColor());

	}

	private void waehleMinuteLaufzeit(int iMinute)
	{
		MinutenKontrolle gewaehlteMinute = _minuten[iMinute];
		gewaehlteMinute.setBackground(getSelectionBackgroundColor());
		gewaehlteMinute.setForeground(getSelectionForegroundColor());

	}

	private void waehleSekundeLaufzeit(int iSekunde)
	{
		SekundenKontrolle gewaehlteSekunde = _sekunden[iSekunde];
		gewaehlteSekunde.setBackground(getSelectionBackgroundColor());
		gewaehlteSekunde.setForeground(getSelectionForegroundColor());

	}

	private Color getSelectionBackgroundColor()
	{
		return isFocusControl() ? _aktiveAuswahlHintergrund
				: _inaktiveAuswahlHintergrund;
	}

	private Color getSelectionForegroundColor()
	{
		return isFocusControl() ? _aktiveAuswahlVordergrund
				: _inaktiveAuswahlVordergrund;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#isFocusControl()
	 */
	public boolean isFocusControl()
	{
		for (Control control = getDisplay().getFocusControl(); control != null; control = control
				.getParent())
		{
			if (control == this)
			{
				return true;
			}
		}

		return false;
	}

	/**
	 * Setzt ein demarkiertes Jahr in den Ursprungszustand zurck
	 * 
	 * @param index
	 *          Index des demarkierten Jahres
	 */
	private void jahrZuruecksetzen(int index)
	{
		JahreKontrolle altesGewaehltesJahr = _jahre[index];
		altesGewaehltesJahr.setBackground(_textHintergrundFarbe);
		altesGewaehltesJahr.setForeground(_textVordergrundFarbe);
	}

	/**
	 * Setzt einen demarkierten Monat in seinen Ursprungszustand zurck
	 * 
	 * @param index
	 *          Index des demarkierten Monats
	 */
	private void monatZuruecksetzen(int index)
	{
		MonateKontrolle alterGewaehlterMonat = _monate[index];
		alterGewaehlterMonat.setBackground(_textHintergrundFarbe);
		alterGewaehlterMonat.setForeground(_textVordergrundFarbe);
	}

	/**
	 * Setzt einen demarkierten Tag in seinen Ursprungszustand zurck
	 * 
	 * @param index
	 *          Index des demarkierten Tags
	 */
	private void tagZuruecksetzen(int index)
	{
		TageKontrolle alterGewaehlterTag = _tage[index];
		alterGewaehlterTag.setBackground(_textHintergrundFarbe);
		alterGewaehlterTag.setForeground(_textVordergrundFarbe);
	}

	/**
	 * Setzt eine demarkierte Stunde in ihren Ursprungszustand zurck
	 * 
	 * @param index
	 *          Index der demarkierten Stunde
	 */
	private void stundeZuruecksetzen(int index)
	{
		StundenKontrolle alteGewaehlteStunde = _stunden[index];
		alteGewaehlteStunde.setBackground(_textHintergrundFarbe);
		alteGewaehlteStunde.setForeground(_textVordergrundFarbe);
	}

	/**
	 * Setzt eine demarkierte Minute in ihren Ursprungszustand zurck
	 * 
	 * @param index
	 *          Index der demarkierten Minute
	 */
	private void minuteZuruecksetzen(int index)
	{
		MinutenKontrolle alteGewaehlteMinute = _minuten[index];
		alteGewaehlteMinute.setBackground(_textHintergrundFarbe);
		alteGewaehlteMinute.setForeground(_textVordergrundFarbe);
	}

	/**
	 * Setzt eine demarkierte Sekunde in ihren Ursprungszustand zurck *
	 * 
	 * @param index
	 *          Index der demarkierten Sekunde
	 */
	private void sekundeZuruecksetzen(int index)
	{
		SekundenKontrolle alteGewaehlteSekunde = _sekunden[index];
		alteGewaehlteSekunde.setBackground(_textHintergrundFarbe);
		alteGewaehlteSekunde.setForeground(_textVordergrundFarbe);
	}

	/**
	 * Jahrekontrolle
	 * 
	 * @author HofmannJ
	 * 
	 */
	static private class JahreKontrolle extends Composite implements Listener
	{
		private Composite fueller;

		private Label label;

		/**
		 * Stundenkontrolle
		 * 
		 * @param parent
		 *          Composite, in dem das Element gezeichnet werden soll
		 */
		public JahreKontrolle(Composite parent)
		{
			super(parent, SWT.NO_FOCUS);
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 1;
				gridLayout.marginHeight = 1;
				setLayout(gridLayout);
			}

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 2;
				gridLayout.marginHeight = 0;
				fueller.setLayout(gridLayout);
			}
			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(String text)
		{
			label.setText(text);
		}

		@SuppressWarnings("unused")
		public String getText()
		{
			return label.getText();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
		 */
		public void setFont(Font font)
		{
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
		 */
		public void setBackground(Color color)
		{
			fueller.setBackground(color);
			label.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setForeground(org.eclipse.swt.graphics.Color)
		 */
		public void setForeground(Color color)
		{
			label.setForeground(color);
		}

		public void setRahmenFarbe(Color color)
		{
			super.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event event)
		{
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Monatekontrolle
	 * 
	 * @author HofmannJ
	 * 
	 */
	static private class MonateKontrolle extends Composite implements Listener
	{
		/**
		 * Der Debugger
		 */
		private Debug _debug = Debug.getLogger();

		private Composite fueller;

//		private boolean debug = false;

		private Label label;

		/**
		 * Stundenkontrolle
		 * 
		 * @param parent
		 *          Composite, in dem das Element gezeichnet werden soll
		 */
		public MonateKontrolle(Composite parent)
		{
			super(parent, SWT.NO_FOCUS);
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 1;
				gridLayout.marginHeight = 1;
				setLayout(gridLayout);
			}

			_debug.fine("Generiere Element!");

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 2;
				gridLayout.marginHeight = 0;
				fueller.setLayout(gridLayout);
			}
			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(String text)
		{
			label.setText(text);
		}

		@SuppressWarnings("unused")
		public String getText()
		{
			return label.getText();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
		 */
		public void setFont(Font font)
		{
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
		 */
		public void setBackground(Color color)
		{
			fueller.setBackground(color);
			label.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setForeground(org.eclipse.swt.graphics.Color)
		 */
		public void setForeground(Color color)
		{
			label.setForeground(color);
		}

		public void setRahmenFarbe(Color color)
		{
			super.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event event)
		{
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Tagekontrolle
	 * 
	 * @author HofmannJ
	 * 
	 */
	static private class TageKontrolle extends Composite implements Listener
	{
		private Composite fueller;

		private Label label;

		/**
		 * Stundenkontrolle
		 * 
		 * @param parent
		 *          Composite, in dem das Element gezeichnet werden soll
		 */
		public TageKontrolle(Composite parent)
		{
			super(parent, SWT.NO_FOCUS);
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 1;
				gridLayout.marginHeight = 1;
				setLayout(gridLayout);
			}

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 2;
				gridLayout.marginHeight = 0;
				fueller.setLayout(gridLayout);
			}
			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(String text)
		{
			label.setText(text);
		}

		@SuppressWarnings("unused")
		public String getText()
		{
			return label.getText();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
		 */
		public void setFont(Font font)
		{
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
		 */
		public void setBackground(Color color)
		{
			fueller.setBackground(color);
			label.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setForeground(org.eclipse.swt.graphics.Color)
		 */
		public void setForeground(Color color)
		{
			label.setForeground(color);
		}

		public void setRahmenFarbe(Color color)
		{
			super.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event event)
		{
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Stundenkontrolle
	 * 
	 * @author HofmannJ
	 * 
	 */
	static private class StundenKontrolle extends Composite implements Listener
	{
		private Composite fueller;

		private Label label;

		/**
		 * Stundenkontrolle
		 * 
		 * @param parent
		 *          Composite, in dem das Element gezeichnet werden soll
		 */
		public StundenKontrolle(Composite parent)
		{
			super(parent, SWT.NO_FOCUS);
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 1;
				gridLayout.marginHeight = 1;
				setLayout(gridLayout);
			}

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 2;
				gridLayout.marginHeight = 0;
				fueller.setLayout(gridLayout);
			}
			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(String text)
		{
			label.setText(text);
		}

		@SuppressWarnings("unused")
		public String getText()
		{
			return label.getText();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
		 */
		public void setFont(Font font)
		{
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
		 */
		public void setBackground(Color color)
		{
			fueller.setBackground(color);
			label.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setForeground(org.eclipse.swt.graphics.Color)
		 */
		public void setForeground(Color color)
		{
			label.setForeground(color);
		}

		public void setRahmenFarbe(Color color)
		{
			super.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event event)
		{
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Zustndig fr die Minuten
	 * 
	 * @author HofmannJ
	 * 
	 */
	static private class MinutenKontrolle extends Composite implements Listener
	{
		private Composite fueller;

		private Label label;

		/**
		 * Stundenkontrolle
		 * 
		 * @param parent
		 *          Composite, in dem das Element gezeichnet werden soll
		 */
		public MinutenKontrolle(Composite parent)
		{
			super(parent, SWT.NO_FOCUS);
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 1;
				gridLayout.marginHeight = 1;
				setLayout(gridLayout);
			}

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 2;
				gridLayout.marginHeight = 0;
				fueller.setLayout(gridLayout);
			}
			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(String text)
		{
			label.setText(text);
		}

		@SuppressWarnings("unused")
		public String getText()
		{
			return label.getText();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
		 */
		public void setFont(Font font)
		{
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
		 */
		public void setBackground(Color color)
		{
			fueller.setBackground(color);
			label.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setForeground(org.eclipse.swt.graphics.Color)
		 */
		public void setForeground(Color color)
		{
			label.setForeground(color);
		}

		public void setRahmenFarbe(Color color)
		{
			super.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event event)
		{
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Zustndig fr die Sekunden
	 * 
	 * @author HofmannJ
	 * 
	 */
	static private class SekundenKontrolle extends Composite implements Listener
	{
		private Composite fueller;

		private Label label;

		/**
		 * Stundenkontrolle
		 * 
		 * @param parent
		 *          Composite, in dem das Element gezeichnet werden soll
		 */
		public SekundenKontrolle(Composite parent)
		{
			super(parent, SWT.NO_FOCUS);
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 1;
				gridLayout.marginHeight = 1;
				setLayout(gridLayout);
			}

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));
			{
				final GridLayout gridLayout = new GridLayout();
				gridLayout.marginWidth = 2;
				gridLayout.marginHeight = 0;
				fueller.setLayout(gridLayout);
			}
			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(String text)
		{
			label.setText(text);
		}

		@SuppressWarnings("unused")
		public String getText()
		{
			return label.getText();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
		 */
		public void setFont(Font font)
		{
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
		 */
		public void setBackground(Color color)
		{
			fueller.setBackground(color);
			label.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#setForeground(org.eclipse.swt.graphics.Color)
		 */
		public void setForeground(Color color)
		{
			label.setForeground(color);
		}

		public void setRahmenFarbe(Color color)
		{
			super.setBackground(color);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event event)
		{
			notifyListeners(event.type, event);
		}
	}

	static private class ElementLabel extends Label
	{
		public ElementLabel(Composite parent, int style)
		{
			super(parent, style);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Control#computeSize(int, int, boolean)
		 */
		public Point computeSize(int wHint, int hHint, boolean changed)
		{
			if (wHint == SWT.DEFAULT)
			{
				GC gc = new GC(this);
				wHint = gc.textExtent("22").x; //$NON-NLS-1$
				gc.dispose();
			}

			return super.computeSize(wHint, hHint, changed);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Widget#checkSubclass()
		 */
		protected void checkSubclass()
		{
		}
	}

	// public Long getZeitDauerInMillisekunden()
	// {
	// return _zeitDauerInMillisekunden;
	// }

}
