/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Kapselt die Bereiche von Ereigniskalendereintrgen.
 * 
 * @author HofmannJ
 * 
 */
public class ErkaBereich implements IEintragBereich
{
	/**
	 * Speichert den Startzeitpunkt
	 */
	private Long _von = null;

	/**
	 * Speichert den Endzeitpunkt
	 */
	private Long _bis = null;

	/**
	 * Formatierer fr das Datum
	 */
	private SimpleDateFormat _format = new SimpleDateFormat(
			"dd.MM.yyyy HH:mm:ss,SSS");

	public ErkaBereich(Long von, Long bis)
	{
		this._von = von;
		this._bis = bis;
	}

	@Override
	public Long getBis()
	{
		return _bis;
	}

	@Override
	public String getBisLesbar()
	{
		return _format.format(new Date(_bis));
	}

	@Override
	public Long getVon()
	{
		return _von;
	}

	@Override
	public String getVonLesbar()
	{
		return _format.format(new Date(_von));
	}

	@Override
	public boolean istAtomar()
	{
		return _von.equals(_bis);
	}

}
