/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;

/**
 * Farbik-Klasse, produziert Buttons zum Aufruf vom Dialog zum Auswhlen von
 * Ereignis- und Systemkalendereintrgen.
 * 
 * @author HofmannJ
 * 
 */
public class SchaltflaechenFabrikEreignisSystemKalenderDialog
{

	/**
	 * Diese Methode erzeugt einen {@link Button}, mit dessen Hilfe sich der
	 * Dialog zur Auswahl von Ereignis- und Systemkalendereintrgen aufrufen lsst
	 * und sich Ereignisse bernehmen lassen.
	 * 
	 * @param elternComposite
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @param felderFuellen
	 *          Instanz des Interface {@link IFelderFuellen}, bestimmt welche
	 *          Felder mit dem Ergebnissen der Auswahl aus dem Dialog gefllt
	 *          werden.
	 * @param text
	 *          Text des Buttons. Wenn dieser gesetzt ist, wird der Button mit
	 *          Text dargestellt. Wenn <code>null</code>, dann wird auf dem
	 *          Button ein Bild dargestellt.
	 * @param bild
	 *          Bild welches auf dem Button dargestellt werden soll, wenn der Text
	 *          auf <code>null</code> gesetzt wurde. Wenn hier <code>null</code>
	 *          angegeben wird, wird ein Default-Bild dargestellt.
	 * @return
	 */
	public static Button erzeugeButton(Composite elternComposite,
			IFelderFuellen felderFuellen, String text, Image bild)
	{
		Button re = null;

		if (felderFuellen != null)
		{
			re = new Button(elternComposite, SWT.PUSH);
			if (text != null)
			{
				re.setText(text);
			} else if ((text == null) && (bild != null))
			{
				re.setImage(bild);
			} else if ((text == null) && (bild == null))
			{
				re.setImage(Activator.getImageDescriptor(
						"icons/sonstiges/zeit_datum_16x16.png").createImage());
			}

			re
					.setToolTipText("ffenet den Dialog zur bernahme von Ereignis- und Systemkalendereintrgen");

			re
			.addSelectionListener(new SchaltflaechenFabrikEreignisSystemKalenderDialog().new MausSelectionAdapter(
					felderFuellen));			
		}

		return re;
	}


	
	/**
	 * Hilfsklasse, implementiert einen Listener fr den Button, welcher den
	 * Dialog zur bernahme von System- und Ereigniskalendereintrgen ffnet.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class MausSelectionAdapter extends SelectionAdapter
	{
		/**
		 * Interface, enthlt alle notwendigen Informationen zu den Feldern, die
		 * gefllt werden sollen.
		 */
		private IFelderFuellen _felderFuellen = null;

		public MausSelectionAdapter(IFelderFuellen felderFuellen)
		{
			this._felderFuellen = felderFuellen;
		}

		/**
		 * ffnet den Dialog zur bernahme von Ereignis- und
		 * Systemkalendereintrgen. Wurde das Interface {@link IFelderFuellen}
		 * gesetzt, wird dessen Methode
		 * {@link IFelderFuellen#fuellen(IEintragBereich)} mit den von dem Dialog
		 * zurckgelieferten Daten als Parameter aufgerufen.
		 */
		private void aktionDialogOeffnen()
		{
			DialogEreignisSystemkalenderUebernahme dialog = new DialogEreignisSystemkalenderUebernahme(
					PlatformUI.getWorkbench().getDisplay().getActiveShell());

			LinkedList<IEintragBereich> bereich = dialog.oeffnen();			

			if (_felderFuellen != null)
			{
				if (bereich != null)
				{
					if (bereich.size() > 0)
					{				
						_felderFuellen.fuellen(bereich);
					}
				}				
			}
			
			

		}
		
		@Override
		public void widgetSelected(SelectionEvent e)
		{
			aktionDialogOeffnen();
		}	
				

	
	}

}
