/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Kapselt die Bereiche von Systemkalendereintrgen.
 * 
 * @author HofmannJ
 * 
 */
public class SkeBereich implements IEintragBereich
{
	/**
	 * Von-Zeitpunkt dieses Bereichs
	 */
	private Long _von = null;

	/**
	 * Bis-Zeitpunkt dieses Bereichs
	 */
	private Long _bis = null;
	
	/**
	 * Formatierer fr das Datum
	 */
	private SimpleDateFormat format = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

	/**
	 * Konstruktor. bernimmt von und bis Zeitpunkt dieses Bereiches
	 * 
	 * @param von
	 *          Von-Zeitpunkt des Bereiches
	 * @param bis
	 *          Bis-Zeitpunkt des Bereiches
	 */
	public SkeBereich(Long von, Long bis)
	{
		this._von = von;
		this._bis = bis;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragBereich#getVon()
	 */
	public Long getVon()
	{
		return _von;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragBereich#getBis()
	 */
	public Long getBis()
	{
		return _bis;
	}
	
	
	
	/* (non-Javadoc)
	 * @see de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragBereich#getVonLesbar()
	 */
	public String getVonLesbar()
	{
		String re = "";
		if(_von != null)
		{
			re = format.format(new Date(_von));
		}
		
		return re;
	}
	
	
	/* (non-Javadoc)
	 * @see de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragBereich#getBisLesbar()
	 */
	public String getBisLesbar()
	{
		String re = "";
		if(_bis != null)
		{
			re = format.format(new Date(_bis));
		}
		
		return re;
	}
	
	
	

	/* (non-Javadoc)
	 * @see de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragBereich#istAtomar()
	 */
	public boolean istAtomar()
	{
		return _von.equals(_bis);
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragBereich#toString()
	 */
	public String toString()
	{
		String re = "";

		
		if (istAtomar())
		{
			re = re + format.format(new Date(_von)) + " (atomar)";
		} else
		{
			re = re + format.format(new Date(_von)) + " bis "
					+ format.format(new Date(_bis));
		}

		return re;
	}
}