/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.ListeZustandsWechsel;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderArbeiter;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

public class SysKalAnfrageArbeiter //extends SystemkalenderArbeiter
{
  SystemkalenderArbeiter _syskalArbeiter;

	public SystemkalenderArbeiter getSyskalArbeiter()
  {
    return _syskalArbeiter;
  }

  public void setSyskalArbeiter(SystemkalenderArbeiter arbeiter)
  {
    _syskalArbeiter = arbeiter;
  }

  public SysKalAnfrageArbeiter(ClientDavInterface connection, String kalender)
	{
		_syskalArbeiter = SystemkalenderArbeiter.getInstance(connection, kalender);
	}

	@SuppressWarnings("static-access")
	public LinkedList<SkeKapsel> berechneGueltigVonBisS(Long von, Long bis)
	{
		LinkedList<SkeKapsel> listeKapseln = new LinkedList<SkeKapsel>();

		// Sortierte Map mit den zu sendenden Daten wird erstellt
		SortedMap<String, String[]> sendMap = new TreeMap<String, String[]>();

		int len = 0;

		// Eintragen der Ereignisse und Berechnen der Anzahl
		for (Map.Entry<String, SystemkalenderEintrag> me : _syskalArbeiter.getSkeList().entrySet())
		{

			SystemkalenderEintrag ske = me.getValue();

			SkeKapsel kapsel = new SkeKapsel(ske);
			listeKapseln.add(kapsel);

			// if (olzw.getListeZustandsWechsel() != null)
			{

				SortedMap<Long, Boolean> sm = ske.berecheneZustandsWechselVonBis(von,
						bis);

				Long wertA = null;
				Long wertB = null;
				Iterator<Entry<Long, Boolean>> itEn = sm.entrySet().iterator();
				while (itEn.hasNext())
				{
					Entry<Long, Boolean> en = itEn.next();

					if (wertA == null)
					{
						wertA = en.getKey();
					} else if ((wertA != null) && (wertB == null))
					{
						wertB = en.getKey();
					}

					if ((wertA != null) && (wertB != null))
					{
						/*
						 * Befindet sich das Ergebnis auch im geforderten Zeitbereich?
						 */
						if (wertA >= von && wertB <= bis)
						{
							kapsel.addBereich(wertA, wertB);
						}
						wertA = null;
						wertB = null;
					}
				}

				// Es wurde kein Ereigniswechsel im definierten Zeitraum festgestellt
				if (sm == null)
				{
					break;
				}

				for (Map.Entry<Long, Boolean> meAend : sm.entrySet())
				{

					Long time = meAend.getKey();

					Boolean gueltig = meAend.getValue();

					String[] sf = new String[2];

					sf[0] = me.getKey();

					if (gueltig)
						sf[1] = "gltig";
					else
						sf[1] = "nicht gltig";

					// Es wird ein eindeutiger Schluessel erzeugt
					String uniqueKey = time + "_" + me.getKey();

					if (time >= von && time <= bis)
					{
						sendMap.put(uniqueKey, sf);
						len++;
					}

				}

			}

		}

		for (Map.Entry<String, String[]> entry : sendMap.entrySet())
		{

			String tmp = entry.getKey();

			// Zeitinfo wird aus dem Schluessel extrahiert
			String millis = tmp.substring(0, tmp.indexOf("_"));

			Long l = Long.decode(millis);

			Date d = new Date();

			d.setTime(l);
		}

		/*
		 * Prfen, ob sich in der Kapsel-Liste Eintrge ohne Bereich befinden. Diese
		 * Eintrge werden gelscht.
		 */
		LinkedList<SkeKapsel> loeschListe = new LinkedList<SkeKapsel>();
		Iterator<SkeKapsel> itLeereBereiche = listeKapseln.iterator();
		while (itLeereBereiche.hasNext())
		{
			SkeKapsel skeKapsel = itLeereBereiche.next();
			if (skeKapsel.getBereicheAnzahl() == 0)
			{
				loeschListe.add(skeKapsel);
			}
		}
		Iterator<SkeKapsel> itLoeschen = loeschListe.iterator();
		while (itLoeschen.hasNext())
		{
			IEintragKapsel skeKapselLoeschen = itLoeschen.next();
			if (listeKapseln.contains(skeKapselLoeschen))
			{
				listeKapseln.remove(skeKapselLoeschen);
			}
		}

		return listeKapseln;
	}

	@SuppressWarnings("static-access")
	public IEintragKapsel berechneGueltigVonBisS(String pid, Long von, Long bis)
	{
		pid = pid.toLowerCase();

		// Sortierte Map mit den zu sendenden Daten wird erstellt
		SortedMap<String, String[]> sendMap = new TreeMap<String, String[]>();

		/**
		 * Zurckzugebende Kapsel, beinhaltet die Daten des Systemkalendereintrags
		 */
		IEintragKapsel skeKapsel = null;

		int len = 0;

		if (_syskalArbeiter.getSkeList().containsKey(pid))
		{

			Calendar cal1 = new GregorianCalendar();
			Calendar cal2 = new GregorianCalendar();
			cal1.setTimeInMillis(von);
			cal2.setTimeInMillis(bis);

			SystemkalenderEintrag ske = _syskalArbeiter.getSkeList().get(pid);

			/*
			 * Kapsel Instanziieren
			 */
			skeKapsel = new SkeKapsel(ske);

			@SuppressWarnings("unused")
			ListeZustandsWechsel olzw = ske.getObjektListeZustandsWechsel();

			// Besitzt das Ereignis auch eine Aenderungs-Liste?
			// if (olzw.getListeZustandsWechsel() != null)
			{

				SortedMap<Long, Boolean> sm = ske.berecheneZustandsWechselVonBis(von,
						bis);

				// Es wurde kein Ereigniswechsel im definierten Zeitraum festgestellt
				if (sm == null)
				{
				}

				if (sm != null)
				{
					Long wertA = null;
					Long wertB = null;
					for (Map.Entry<Long, Boolean> meAend : sm.entrySet())
					{
						if (wertA == null)
						{
							wertA = meAend.getKey();
						} else if ((wertA != null) && (wertB == null))
						{
							wertB = meAend.getKey();
						}

						if ((wertA != null) && (wertB != null))
						{
							/*
							 * Befindet sich das Ergebnis auch im geforderten Zeitbereich?
							 */
							if (wertA >= von && wertB <= bis)
							{
								skeKapsel.addBereich(wertA, wertB);
							}
							wertA = null;
							wertB = null;
						}

						Long time = meAend.getKey();

						Boolean gueltig = meAend.getValue();

						String[] sf = new String[2];

						// sf[0] = ske.getPid();
						sf[0] = pid;

						if (gueltig)
							sf[1] = "gltig";
						else
							sf[1] = "noch gltig";

						/** *************************************************************** */
						// Calendar cal = new GregorianCalendar();
						// cal.setTimeInMillis(time);
						// if ((cal.get(Calendar.MILLISECOND) == 999) && !meAend.getValue())
						// {
						// time += 1;
						// }
						/** *************************************************************** */

						// Es wird ein eindeutiger Schluessel erzeugt
						String uniqueKey = time + "_" + ske.getPid();

						if (time >= von && time <= bis)
						{
							sendMap.put(uniqueKey, sf);
							len++;
						}

					}

				}

			}
			// else
			// {
			// Ereignis hat keine Aenderungs-Liste
			// }

		} else
		{
			// Nicht in SKE-Liste
		}

		for (Map.Entry<String, String[]> entry : sendMap.entrySet())
		{

			String tmp = entry.getKey();

			// Zeitinfo wird aus dem Schluessel extrahiert
			String millis = tmp.substring(0, tmp.indexOf("_"));

			Long l = Long.decode(millis);

			Date d = new Date();

			d.setTime(l);

//			SimpleDateFormat format = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
//			String datum = format.format(d);
//			System.out.println("anfrage intervall: " + datum + " : "
//					+ entry.getValue()[0] + " : " + entry.getValue()[1]);
		}

		return skeKapsel;
	}

}
