/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Test Empfnger fr dynamische Daten fr die Vorarbeit zur Listendarstellung. Arbeitet mit dynamischen Daten und nicht
 * mit KonfigurationsDaten.
 * 
 * @author HofmannJ
 * 
 */
public class DaVComListendarstellungstest
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /*
   * Zugangsdaten zum DaV
   */
  // private String _DaVIp = "192.0.1.117";
  private String _DaVIp = "127.0.0.1";

  private int _DaVPort = 8083;

  private String _DaVBenutzer = "Tester";

  private String _DaVPasswort = "geheim";

  /**
   * Verbindung zum DaV.
   */
  private ClientDavInterface _connection = null;

  /**
   * Ist <code>true</code>, wenn eine Verbindung zum DaV besteht, sonst <code>false</code>.
   */
  private boolean _verbunden = false;
  
  public DaVComListendarstellungstest()
  {
    _debug.fine(getClass().getName() + " => BEREIT.");
  }

  /**
   * Verbindungsaufbau zum DaV.
   * 
   * @return Liefert <code>true</code>, wenn die Verbindung zum DaV aufgebaut werden konnte, sonst <code>false</code>.
   */
  public boolean baueVerbindungMitDaVAuf()
  {
    _connection = verbindeMitDav();
    _verbunden = (_connection != null);

    if (_verbunden)
    {
      _debug.fine(getClass().getName() + " => Verbindung zum DaV wurde erfolgreich aufgebaut.");
    }
    else
    {
      _debug.fine(getClass().getName() + " => Es konnte keine Verbindung zum DaV aufgebaut werden.");
    }

    return _verbunden;
  }

  /**
   * Baut die Verbindung zum DaV auf.
   * 
   * @return Verbindung zum DaV.
   */
  private ClientDavInterface verbindeMitDav()
  {
    try
    {
      _connection = new ClientDavConnection();
    }
    catch (MissingParameterException e)
    {
      e.printStackTrace();
    }

    // IP setzen
    _connection.getClientDavParameters().setDavCommunicationAddress(_DaVIp);

    // Port setzen
    _connection.getClientDavParameters().setDavCommunicationSubAddress(_DaVPort);

    // Verbinden
    try
    {
      _connection.connect();
    }
    catch (CommunicationError e)
    {
      _debug.fine("Kommunikationsfehler: " + e.toString());

    }
    catch (ConnectionException e)
    {
      _debug.fine("Verbindungsausnahmefehler: " + e.toString());

    }

    // Benutzer und Passwort setzen
    try
    {
      _connection.login(_DaVBenutzer, _DaVPasswort);
    }
    catch (InconsistentLoginException e)
    {
      _debug.fine("Falsche Zugangsdaten zum DaV: " + e.toString());

    }
    catch (CommunicationError e)
    {
      System.out.println("Kommunikationsfehler bei bergabe der Zugangsdaten: " + e.toString());

    }

    return _connection;
  }

  /**
   * Trennt die Verbindung zum DaV.
   * 
   */
  public void trenneDaVVerbindung()
  {
    if (_verbunden)
    {
      _connection.disconnect(false, null);
      _verbunden = false;
      _connection = null;
      _debug.fine(getClass().getName() + " => Verbindung zum DaV wurde getrennt.");

    }
    else
    {
      _debug.fine(getClass().getName()
          + " => Es konnte keine Verbindung getrennt werden, da keine Verbindung zum DaV existierte.");
    }
  }

  /**
   * Liefert die Verbindung zum DaV, wenn diese Aufgebaut wurde, sonst <code>false</code>.
   * 
   * @return Verbindung zum DaV oder null.
   */
  public ClientDavInterface getVerbindung()
  {
    if (_verbunden)
    {
      return this._connection;
    }
    else
    {
      return null;
    }
  }

  /**
   * Liefert <code>true</code>, wenn eine Verbindung zum DaV besteht, sonst <code>false</code>.
   * 
   * @return <code>true</code>, wenn eine Verbindung zum DaV besteht, sonst <code>false</code>.
   */
  public boolean istVerbunden()
  {
    return _verbunden;
  }

}
