/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.kupzog.ktable.KTable;

/**
 * Klasse zum Speichern und Lesen der benutzerspezifischen, netzweiten
 * Einstellungen.
 * 
 * @author HofmannJ
 * 
 */
public class DavEmpfangenSendenAenderungsDarstellung implements
		ClientSenderInterface, ClientReceiverInterface
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _verbindung = null;

	/**
	 * IP
	 */
	private static final String _ip = "localhost";

	/**
	 * Port
	 */
	private static final int _port = 8083;

	/**
	 * Login
	 */
	private static final String _benutzer = "Tester";

	/**
	 * Passwort
	 */
	private static final String _passwort = "geheim";

	/**
	 * Display
	 */
	private Display _display = null;

	/**
	 * {@link TabellenModell}, welches die darzustellenden Daten aufnimmt.
	 */
	private TabellenModell _tabellenModell = null;

	/**
	 * {@link KTable}, welche das {@link TabellenModell} anzeigt. Hier muss die
	 * redraw-Methode aufgerufen werden, wenn sich etwas am {@link TabellenModell}
	 * gendert hat.
	 */
	private KTable _kTable = null;

	/**
	 * Datenmodell des DaV
	 */
	private DataModel _dataModell = null;

	/**
	 * SystemObjekt
	 */
	private SystemObject _systemObject = null;

	/**
	 * Bezeichnung der Attributgruppe
	 */
	private String _atgPid = "atg.benutzerEinstellungenOberflcheNetzweit";

	/**
	 * Item Benutzereinstellungen
	 */
	private final String _itemBenutzereinstellungen = "Benutzereinstellungen";

	/**
	 * Item Voreinstellungen
	 */
	private final String _itemVoreinstellungen = "Voreinstellungen";

	/**
	 * Item Benutzer
	 */
	private final String _itemBenutzer = "benutzer";

	/**
	 * Bezeichnung des Aspektes soll (zum Empfangen)
	 */
	private String _aspSollPid = "asp.parameterSoll";

	/**
	 * Bezeichnung des Aspektes vorgabe (zum Senden)
	 */
	private String _aspVorgabePid = "asp.parameterVorgabe";

	/**
	 * Typ Autarke Organisationseinheit
	 */
	private String _typAOE = "typ.autarkeOrganisationsEinheit";

	/**
	 * Attributgruppe
	 */
	private AttributeGroup _atg = null;

	/**
	 * Aspekt
	 */
	private Aspect _asp = null;

	/**
	 * Datenbeschreibung
	 */
	private DataDescription _dataDescription = null;

	/**
	 * Empfangsthread Flag
	 */
	private boolean _laufeEmpfangsThread = false;

	/**
	 * Kopie der atg.benutzerEinstellungenOberflcheNetzweit zwecks nderung eines
	 * Eintrages.
	 */
	private Data _kopiertData = null;

	/**
	 * Objekt zum Synchronisieren des Threads auf den Empfang von Daten. Empfngt
	 * die update-Methode etwas, wird Empfang notified.
	 */
	private Boolean _empfang = new Boolean(true);

	/**
	 * Objekt zum Synchronisieren, dass der Thread die Empfangenen Daten
	 * verarbeitet hat (auch wenn hier keine groartige Verarbeitung stattfindet).
	 * Mit notify wird eine Methode zur Rckgabe der gelesenen Daten
	 * benachrichtigt.
	 */
	private Boolean _endeVerarbeitungEmpfang = new Boolean(true);

	/**
	 * Synchronisation der Sende Steuerung
	 */
	private Boolean _sendeSteuerung = new Boolean(true);

	private Boolean _warten = new Boolean(true);

	private DataTableObjectErweitert _dto = null;

	/**
	 * Max. Gre der Queue
	 */
	private static final int MAXQUEUE = 10;

	/**
	 * Queue
	 */
	private final Queue<Data> produkte = new LinkedList<Data>();

	/**
	 * Lock der Queue
	 */
	private final Lock lock = new ReentrantLock();

	/**
	 * Bedingung fr Queue nicht voll
	 */
	private final Condition nichtVoll = lock.newCondition();

	/**
	 * Bedingung fr Queue nicht leer
	 */
	private final Condition nichtLeer = lock.newCondition();

	// public EinstellungenBenutzerSpezifischNetzweit()
	// {
	// baueDaVVerbindungAuf();
	//
	// _dataModell = _verbindung.getDataModel();
	//
	// // testObjekteBestimmen();
	// String pid = bestimmePidEinesBenutzers("Tester");
	// System.out.println("PID: " + pid);
	// // senden();
	//
	// // Data empfangeDaten = empfangeDatenBenutzereinstellungenNetzweit();
	// // System.out.println("EMPFANGEN: " + empfangeDaten.toString());
	// // sendenBearbeitetTest();
	//
	// SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
	// String d = sdf.format(new Date());
	//
	// editEintrag("TestDatenverteilerBenutzer", "XML\nIST\nToll\n!\n" + d);
	//
	// Warten w = new Warten();
	// synchronized (_warten)
	// {
	// try
	// {
	// _warten.wait();
	// } catch (InterruptedException e)
	// {
	// e.printStackTrace();
	// }
	// }
	//
	// String eintrag = getEintrag("TestDatenverteilerBenutzer");
	// System.out.println("Eintrag:\n" + eintrag);
	//
	// // editEintrag("Tester", "Noch\nEin\nPaar\nZeilen\nMehr!!!");
	// // String eintragTester = getEintrag("Tester");
	// // System.out.println("Eintrag Tester: " + eintragTester);
	//
	// schliesseDaVVerbindung();
	// }

	/**
	 * Konstruktor, bernimmt die Verbindung zum DaV.
	 * 
	 * @param verbindung
	 *          DaV-Verbindung
	 */
	public DavEmpfangenSendenAenderungsDarstellung(ClientDavConnection verbindung)
	{
		this._verbindung = verbindung;
		this._dataModell = _verbindung.getDataModel();
	}

	/**
	 * Setzt das Display. Der Thread der DAV-Kommunikation bentigt den Zugriff
	 * hierauf, um asynchron die Darstellung der Tabelle anpassen zu knnen.
	 * 
	 * @param display
	 *          Display.
	 */
	public void setDisplay(Display display)
	{
		this._display = display;
	}

	/**
	 * Setzt die zu aktualisierende {@link KTable}.
	 * 
	 * @param kTable
	 */
	public void setKTable(KTable kTable)
	{
		this._kTable = kTable;
	}

	/**
	 * Setzt das zu aktualisierende {@link TabellenModell}.
	 * 
	 * @param tabellenModell
	 */
	public void setTabellenModell(TabellenModell tabellenModell)
	{
		this._tabellenModell = tabellenModell;
	}

	/**
	 * Anmelden zum Senden von Daten
	 */
	private void anmeldenSenden()
	{
		_debug.fine("Anmelden Senden...");
		_dataModell = _verbindung.getDataModel();

		List<SystemObject> objekte = bestimmeObjekte("typ.autarkeOrganisationsEinheit");
		if (objekte.size() == 1)
		{
			_systemObject = objekte.get(0);
			_atg = _dataModell.getAttributeGroup(_atgPid);

			_asp = _dataModell.getAspect(_aspVorgabePid);
			_dataDescription = new DataDescription(_atg, _asp);

			SenderRole senderRole = SenderRole.sender();

			try
			{
				_verbindung.subscribeSender(this, _systemObject, _dataDescription,
						senderRole);
			} catch (OneSubscriptionPerSendData e)
			{
				e.printStackTrace();
			}

		}

	}

	protected void senden()
	{
		anmeldenSenden();
		_atg = _dataModell.getAttributeGroup(_atgPid);
		Data data = _verbindung.createData(_atg);
		Array array = data.getArray("Benutzereinstellungen");
		array.setLength(2);

		Data item = null;

		for (int i = 0; i < 2; i++)
		{
			item = array.getItem(i);
			// TextArray textArray = item.getTextArray(_itemVoreinstellungen);

			Array arrayText = item.getArray(_itemVoreinstellungen);

			ReferenceValue referenceValue = item.getReferenceValue(_itemBenutzer);

			// textArray.setLength(2);
			arrayText.setLength(2);

			// textArray.getTextValue(0).setText("Hallo Welt! <= Zeile 1");
			// textArray.getTextValue(1).setText("Hallo Welt! <= Zeile 2");

//			arrayText.getTextValue(0).setText("Hallo Welt! <= Zeile 1");
//			arrayText.getTextValue(1).setText("Hallo Welt! <= Zeile 2");

			referenceValue.setSystemObject(bestimmeSOEinesBenutzers("Tester"));
		}

		ResultData datenTelegramm = new ResultData(_systemObject, _dataDescription,
				new Date().getTime(), data);

		try
		{
			_verbindung.sendData(datenTelegramm);
		} catch (DataNotSubscribedException e)
		{
			System.err.println(getClass().getName() + "::");
			e.printStackTrace();
		} catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}

		abmeldenSenden();
	}

	/**
	 * Senden von Testdaten
	 */
	protected void sendenBearbeitetTest()
	{

		/*
		 * Daten Holen
		 */
		Data data = empfangeDatenBenutzereinstellungenNetzweit();

		/*
		 * Benutzereintrag suchen und Werte ndern (TEST)
		 */
		Data benEintrag = sucheBenutzerspezifischenEintrag("Tester", data);
		setBenutzersepzifischeEinstellungen(benEintrag,
				"123\nTest\nUndNochMal\n111\nfjfjfj\nelffl\n12i123i");

		/*
		 * Suchen nach TestDatenverteilerBenutzer
		 */
		Data test = sucheBenutzerspezifischenEintrag("TestDatenverteilerBenutzer",
				data);
		if (test == null)
		{
			System.err.println("Benutzer TestDatenverteilerBenutzer nicht gefunden!");
		} else
		{
			System.err.println("Benutzer TestDatenverteilerBenutzer gefunden.");
		}

		/*
		 * Anlegen des Benutzers
		 */
		anlegenBenutzerspezifischenEintrag(data, "TestDatenverteilerBenutzer");

		// System.out.println("DATEN: " + data.toString());
		//
		// Array array = data.getArray("Benutzereinstellungen");
		//
		// System.out.println("Lnge: " + array.getLength());
		//
		// System.out.println("ARRAY: " + array.toString());
		//
		// array.setLength(6);

		// System.out.println("ARRAY: " + array.toString());

		anmeldenSenden();
		ResultData datenTelegramm = new ResultData(_systemObject, _dataDescription,
				new Date().getTime(), data);

		try
		{
			_verbindung.sendData(datenTelegramm);
		} catch (DataNotSubscribedException e)
		{
			e.printStackTrace();
		} catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}
		abmeldenSenden();
	}

	/**
	 * Berarbeitet einen benutzerspezifischen Eintrag.
	 * 
	 * @param benutzerName
	 *          Benutzer, fr den ein Eintrag angepasst werden soll.
	 * @param xmlStruktur
	 *          String der XML-Struktur, die gespeichert werden soll.
	 * @return Liefert <code>true</code>, wenn erfolgreich gespeichert werden
	 *         konnte.
	 */
	public boolean editEintrag(String benutzerName, String xmlStruktur)
	{
		boolean erfolgreich = false;

		/*
		 * Daten Holen
		 */
		Data data = empfangeDatenBenutzereinstellungenNetzweit();

		/*
		 * Benutzerdaten suchen
		 */
		Data benutzerDaten = sucheBenutzerspezifischenEintrag(benutzerName, data);

		/*
		 * Existiert Benutzer bzw. dessen Daten?
		 */
		if (benutzerDaten != null)
		{
			setBenutzersepzifischeEinstellungen(benutzerDaten, xmlStruktur);
			erfolgreich = true;
		} else
		{
			if (anlegenBenutzerspezifischenEintrag(data, benutzerName))
			{
				Data neuangelegterBenutzereintrag = sucheBenutzerspezifischenEintrag(
						benutzerName, data);
				setBenutzersepzifischeEinstellungen(neuangelegterBenutzereintrag,
						xmlStruktur);
				erfolgreich = true;
			}
		}

		if (erfolgreich)
		{
			anmeldenSenden();
			ResultData datenTelegramm = new ResultData(_systemObject,
					_dataDescription, new Date().getTime(), data);

			try
			{
				synchronized (_sendeSteuerung)
				{
					try
					{

						_debug.fine("=====>_sendenEnde.wait");
						_sendeSteuerung.wait();
						_debug.fine("=====>_sendenEnde notified");
						_verbindung.sendData(datenTelegramm);
					} catch (InterruptedException e)
					{
						System.err.println(getClass().getName()
								+ "::editEintrag Wurde unterbrochen\n\t=> " + e.toString());
						e.printStackTrace();
					}
				}
			} catch (DataNotSubscribedException e)
			{
				e.printStackTrace();
			} catch (SendSubscriptionNotConfirmed e)
			{
				e.printStackTrace();
			}
			abmeldenSenden();
		}

		return erfolgreich;
	}

	public String getEintrag(String benutzerName)
	{
		String re = null;

		/*
		 * Daten Holen
		 */
		Data data = empfangeDatenBenutzereinstellungenNetzweit();

		Data eintrag = sucheBenutzerspezifischenEintrag(benutzerName, data);
		if (eintrag != null)
		{
			// TextArray textArray =
			// eintrag.getTextArray(_itemVoreinstellungen);
			Array textArray = eintrag.getArray(_itemVoreinstellungen);
			if (textArray.getLength() > 0)
			{
				StringBuffer buf = new StringBuffer();

				for (int i = 0; i < textArray.getLength(); i++)
				{
					buf.append(textArray.getTextValue(i).getText());
					buf.append("\n");
				}

				re = buf.toString();
			}
		}
		return re;
	}

	/**
	 * Fgt einen neuen Eintrag fr einen Benutzer hinzu, so dass
	 * Benutzerspezifische Einstellungen fr den Benutzer gespeichert werden
	 * knnen.
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit
	 * 
	 * @param data
	 *          Daten, an die der Benutzer angehngt werden soll (in Attributliste
	 *          Benutzereinstellungen)
	 * @param benutzerName
	 *          Benutzername
	 * @return Liefert <code>true</code>, wenn der Benutzereintrag den
	 *         bergebenen Daten angefgt werden kann, sonnst <code>false</code>.
	 */
	private boolean anlegenBenutzerspezifischenEintrag(Data data,
			String benutzerName)
	{
		boolean erfolgreich = false;

		SystemObject soBenutzer = bestimmeSOEinesBenutzers(benutzerName);
		if (soBenutzer != null)
		{
			/*
			 * Lnge des Arrays um 1 erhhen
			 */
			data.getArray(_itemBenutzereinstellungen).setLength(
					data.getArray(_itemBenutzereinstellungen).getLength() + 1);

			int pos = data.getArray(_itemBenutzereinstellungen).getLength() - 1;

			Data benutzerItem = data.getArray(_itemBenutzereinstellungen)
					.getItem(pos);

			/*
			 * TextArray mit einer leeren XML-Struktur initialisieren
			 */
			@SuppressWarnings("unused")
			TextArray textArrayVoreinstellungen = benutzerItem
					.getTextArray(_itemVoreinstellungen);

			// EinstellungenSpeicher es = new EinstellungenSpeicher();
			// String[] split = es.ausgabeXmlFormatiert().split("\n");
			// textArrayVoreinstellungen.setLength(split.length);
			// for (int i = 0; i < split.length; i++)
			// {
			// textArrayVoreinstellungen.getTextValue(i).setText(split[i]);
			// }

			/*
			 * Benutzerreferenz setzen
			 */
			benutzerItem.getReferenceValue(_itemBenutzer).setSystemObject(soBenutzer);

			erfolgreich = true;

		}

		return erfolgreich;
	}

	/**
	 * Liefert das ITEM der Benutzereinstellungen zum angegebenen Benutzer
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit
	 * 
	 * @param benutzer
	 *          Name des Benutzers
	 * @param data
	 *          Daten, aus denen der Benutzer herausgeholt werden soll.
	 * @return Liefert das "ITEM" als {@link Data}, welches zu dem jeweiligen
	 *         Benutzer gehrt oder <code>null</code>, falls dieser Benutzer
	 *         nicht existiert.
	 */
	private Data sucheBenutzerspezifischenEintrag(String benutzer, Data data)
	{
		Data re = null;

		SystemObject soBenutzer = bestimmeSOEinesBenutzers(benutzer);
		if (soBenutzer != null)
		{
			Array array = data.getArray(_itemBenutzereinstellungen);
			int laenge = array.getLength();
			for (int i = 0; i < laenge; i++)
			{
				Data item = array.getItem(i);

				ReferenceValue referenceValue = item.getReferenceValue(_itemBenutzer);
				// System.out.println("REF: " + referenceValue.toString());

				if (referenceValue.getSystemObject() != null)
				{
					if (referenceValue.getSystemObject().equals(soBenutzer))
					{
						re = item;
						break;
					}
				} else
				{
					// System.out.println("UNDEF");
				}
			}
		}

		return re;
	}

	/**
	 * Setzt die Benutzersperzifischen Einstellungen in das Item der
	 * Benutzereinstellungen, die von der Funktion
	 * {@link EinstellungenBenutzerSpezifisch#sucheBenutzerspezifischenEintrag(String, Data)}
	 * geliefert wurde.
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit -> Benutzereinstellungen
	 * 
	 * @param data
	 *          "Item" (aus der Attributliste Benutzereinstellungen), in dem die
	 *          Benutzerspezifischen Einstellungen gesetzt werden sollen.
	 * @param xmlStruktur
	 *          Zu setzende XML-Struktur.
	 */
	private void setBenutzersepzifischeEinstellungen(Data data, String xmlStruktur)
	{
		// TextArray textArray = data.getTextArray(_itemVoreinstellungen);
		// _debug.fine("Data: " + data.toString());
		Array textArray = data.getArray(_itemVoreinstellungen);
		String[] split = xmlStruktur.split("\\n");
		textArray.setLength(split.length);

		for (int i = 0; i < split.length; i++)
		{
			textArray.getTextValue(i).setText(split[i]);
		}
	}

	/**
	 * Abmeldung
	 */
	private void abmeldenSenden()
	{
		_debug.fine("Abmelden Senden..");
		_verbindung.unsubscribeSender(this, _systemObject, _dataDescription);

	}

	/**
	 * Bestimmt die PID des zu einem Benutzer zugehrigen Objektes vom Typ
	 * <code>typ.benutzer</code>.
	 * 
	 * @param benutzer
	 *          Name des Benutzer-Objektes
	 * @return {@link SystemObject} des Benutzers oder <code>null</code>, wenn
	 *         kein entsprechender Benutzer existiert.
	 */
	private SystemObject bestimmeSOEinesBenutzers(String benutzer)
	{
		SystemObject reSo = null;

		List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		Iterator<SystemObject> it = bestimmeObjekte.iterator();
		while (it.hasNext())
		{
			SystemObject so = it.next();
			if (so.getName().equals(benutzer))
			{
				reSo = so;
				// System.out.println("Benutzer gefunden: " + so.getName() + "("
				// + so.getPid() + ")");
				break;
			}
		}

		return reSo;
	}

	/**
	 * Bestimmt eine Liste von zu Benutzern (typ.benutzer) zugehrigen
	 * {@link SystemObject}s.
	 * 
	 * @return Liste von zu Benutzern (typ.benutzer) zugehrigen
	 *         {@link SystemObject}s
	 */
	public List<SystemObject> bestimmeAlleBenutzerSO()
	{
		List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		return bestimmeObjekte;
	}

	/**
	 * Bestimmt die PID des zu einem Benutzer zugehrigen Objektes vom Typ
	 * <code>typ.benutzer</code>.
	 * 
	 * @param benutzer
	 *          Name des Benutzer-Objektes
	 * @return PID des Benutzerobjektes
	 */
	@SuppressWarnings("unused")
	private String bestimmePidEinesBenutzers(String benutzer)
	{
		String rePid = null;

		List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		Iterator<SystemObject> it = bestimmeObjekte.iterator();
		while (it.hasNext())
		{
			SystemObject so = it.next();
			if (so.getName().equals(benutzer))
			{
				rePid = so.getPid();
				// System.out.println("Benutzer gefunden: " + so.getName() + "("
				// + so.getPid() + ")");
				break;
			}
		}

		return rePid;
	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de)
	 * 
	 * @param objPid
	 *          Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	public List<SystemObject> bestimmeObjekte(String objPid)
	{
		// System.out.println("bestimmeObjekte --> " + objPid);

		List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = _verbindung.getDataModel().getType(objPid);

		if (typeSysObj == null)
			return objekte;

		if (typeSysObj.isConfigurating())
		{
			typeSysObj = (ConfigurationObjectType) typeSysObj;
			// System.out.println("isConfigurating");
		} else
		{
			typeSysObj = (DynamicObjectType) typeSysObj;
			// System.out.println("Dynamic");
		}

		// Elemente dieses Systemobjekts einlesen

		List<SystemObject> typobjekte = typeSysObj.getElements();

		Iterator<SystemObject> iterator_Typen = typobjekte.iterator();

		while (iterator_Typen.hasNext())
		{
			SystemObject sysObj = (SystemObject) iterator_Typen.next();

			objekte.add(sysObj);

			// System.out.println(sysObj.getPid());
		}

		return objekte;
	}

	@Override
	public void dataRequest(SystemObject object, DataDescription dataDescription,
			byte state)
	{
		if (state == 0)
		{
			synchronized (_sendeSteuerung)
			{
				_sendeSteuerung.notify();
			}
			_debug.fine("=====>_sendenEnde.notify durch Methode dataRequest");
		}
	}

	@Override
	public boolean isRequestSupported(SystemObject object,
			DataDescription dataDescription)
	{
		return true;
	}

	@SuppressWarnings("unused")
	private class Sender implements Runnable
	{
		/**
		 * Thread-Flag
		 */
		private boolean _laufeThread = false;

		public Sender()
		{
		}

		public void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Sender");
				_laufeThread = true;
				th.start();
			}
		}

		public void stopThread()
		{
			if (_laufeThread)
			{
				_laufeThread = false;
			}
		}

		@Override
		public void run()
		{
			while (_laufeThread)
			{

			}
		}

	}

	/*
	 * Empfang
	 */
	/**
	 * Ruft die netzweiten, benutzerspezifischen Einstellungen vom DaV ab und
	 * liefert diese als modifiableCopy eines {@link Data}-Objektes zurck.
	 */
	private Data empfangeDatenBenutzereinstellungenNetzweit()
	{
		@SuppressWarnings("unused")
		Empfaenger e = new Empfaenger();

		synchronized (_endeVerarbeitungEmpfang)
		{
			try
			{

				// e.startThread();
				anmeldenEmpfang();

				synchronized (_endeVerarbeitungEmpfang)
				{
					anmeldenEmpfang();
					_endeVerarbeitungEmpfang.wait();

					_debug
							.fine(getClass().getName()
									+ "::empfangeXmlStruktur Wurde ber den Empfang einer XML-Struktur benachrichtigt.");

				}

				/*
				 * Abmeldung erfolgt durch die update-Methode
				 */

			} catch (InterruptedException e1)
			{
				e1.printStackTrace();
			}
		}

		return _kopiertData;
	}

	/**
	 * Ruft die netzweiten, benutzerspezifischen Einstellungen vom DaV ab und
	 * liefert diese als modifiableCopy eines {@link Data}-Objektes zurck.
	 */
	@SuppressWarnings("unused")
	private DataTableObjectErweitert empfangeDatenBenutzereinstellungenNetzweitAlsDTO()
	{
		Empfaenger e = new Empfaenger();

		synchronized (_endeVerarbeitungEmpfang)
		{
			try
			{

				// e.startThread();
				anmeldenEmpfang();

				synchronized (_endeVerarbeitungEmpfang)
				{
					anmeldenEmpfang();
					_endeVerarbeitungEmpfang.wait();

					_debug
							.fine(getClass().getName()
									+ "::empfangeXmlStruktur Wurde ber den Empfang einer XML-Struktur benachrichtigt.");

				}

				/*
				 * Abmeldung erfolgt durch die update-Methode
				 */

			} catch (InterruptedException e1)
			{
				e1.printStackTrace();
			}
		}

		return _dto;
	}

	/**
	 * Anmelden zum Empfang der netzweiten, benutzerspezifischen Einstellungen.
	 */
	private void anmeldenEmpfang()
	{
		List<SystemObject> objekte = bestimmeObjekte(_typAOE);
		if (objekte.size() == 1)
		{
			_systemObject = objekte.get(0);
			_atg = _dataModell.getAttributeGroup(_atgPid);
			_asp = _dataModell.getAspect(_aspSollPid);
			_dataDescription = new DataDescription(_atg, _asp);
			ReceiveOptions receiveOptions = ReceiveOptions.normal();
			ReceiverRole receiverRole = ReceiverRole.receiver();

			_verbindung.subscribeReceiver(this, _systemObject, _dataDescription,
					receiveOptions, receiverRole);
		}
	}

	/**
	 * Anmelden zum Empfang der netzweiten, benutzerspezifischen Einstellungen.
	 * bernimmt verschiedene Parameter.
	 * 
	 * @param pidSystemObject
	 *          Pid des {@link SystemObject}s, auf welches die Anmeldung erfolgen
	 *          soll.
	 * @param pidAtg
	 *          Pid der Attributgruppe, unter der die Anmeldung erfolgen soll.
	 * @param pidAsp
	 *          Pid des Aspekts, unter dem die Anmeldung erfolgen soll.
	 */
	public void anmeldenEmpfang(String pidSystemObject, String pidAtg,
			String pidAsp)
	{
		AttributeGroup atg = _dataModell.getAttributeGroup(pidAtg);
		Aspect asp = _dataModell.getAspect(pidAsp);
		DataDescription dataDescription = new DataDescription(atg, asp);
		ReceiveOptions receiveOptions = ReceiveOptions.normal();
		ReceiverRole receiverRole = ReceiverRole.receiver();

		_verbindung.subscribeReceiver(this, _dataModell.getObject(pidSystemObject),
				dataDescription, receiveOptions, receiverRole);

	}

	/**
	 * Abmelden vom Empfang der netzweiten, benutzerspezifischen Einstellungen.
	 */
	public void abmeldenEmpfang()
	{
		_verbindung.unsubscribeReceiver(this, _systemObject, _dataDescription);
		_debug.fine("Abgemeldet Empfang");
	}

	/**
	 * Meldet vom Empfang ab.
	 * 
	 * @param pidSystemObject
	 *          Pid des {@link SystemObject}, von dem der Empfang abgemeldet
	 *          werden soll.
	 * @param pidAtg
	 *          Pid der Attributgruppe, von welcher der Empfang abgemeldet werden
	 *          soll.
	 * @param pidAsp
	 *          Pid des Aspekts, unter dem die Empfangsabmeldung erfolgen soll.
	 */
	public void abmeldenEmpfang(String pidSystemObject, String pidAtg,
			String pidAsp)
	{
		SystemObject systemObject = _dataModell.getObject(pidSystemObject);
		AttributeGroup atg = _dataModell.getAttributeGroup(pidAtg);
		Aspect asp = _dataModell.getAspect(pidAsp);
		DataDescription dataDescription = new DataDescription(atg, asp);
		_verbindung.unsubscribeReceiver(this, systemObject, dataDescription);
		_debug.fine("Abgemeldet Empfang");
	}

	@Override
	public void update(ResultData[] results)
	{
//		System.out.println("=> " + results.length + " Elmente empfangen");

		for (ResultData resultData : results)
		{
//			System.out.println("ResultData " + resultData);
				
			DataTableObjectErweitert dto = new DataTableObjectErweitert(resultData);
//			System.out.println("=> DataTableObject erzeugt");

			/*
			 * TabellenModell aktualisieren
			 */
			if (_tabellenModell != null)
			{
				_tabellenModell.addDTO(dto);
				_tabellenModell.initialisiereDaten();

				/*
				 * Darstellung der Tabelle aktualisieren
				 */
				if ((_kTable != null) && (_display != null))
				{
					_display.asyncExec(new Runnable(){

						@Override
						public void run()
						{

							_kTable.redraw();
						}
						
					});

				}
				else
				{
					System.out.println("Keine aktualisierung mglich!");
				}
			}
		}
	}

	/**
	 * Empfnger Hilfsklasse
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Empfaenger implements Runnable
	{
		public Empfaenger()
		{
		}

		/**
		 * Startet den Thread
		 */
		@SuppressWarnings("unused")
		public void startThread()
		{
			if (!_laufeEmpfangsThread)
			{
				_laufeEmpfangsThread = true;
				Thread empfangsThread = new Thread(this);
				empfangsThread.setName("Empfangs Thread");
				empfangsThread.start();
			}
		}

		/**
		 * Stoppt den Thread
		 */
		public void stopThread()
		{
			if (_laufeEmpfangsThread)
			{
				_laufeEmpfangsThread = false;
			}
		}

		@Override
		public void run()
		{

			while (_laufeEmpfangsThread)
			{
				synchronized (_empfang)
				{
					try
					{
						_empfang.wait();
						synchronized (_endeVerarbeitungEmpfang)
						{
							_endeVerarbeitungEmpfang.notify();
						}
						stopThread();
					} catch (InterruptedException e)
					{
						e.printStackTrace();
					}
				}

			}

		}

	}

	/**
	 * Einfgen von {@link Data}s in die Queue
	 * 
	 * @param data
	 *          Einzufgendes {@link Data}
	 * @throws InterruptedException
	 *           Wird geworfen, wenn eine Unterbrechung eintritt.
	 */
	public void rein(Data data) throws InterruptedException
	{
		lock.lock();
		try
		{
			while (produkte.size() == MAXQUEUE)
				nichtVoll.await();

			produkte.add(data);

			nichtLeer.signalAll();
		} finally
		{
			lock.unlock();
		}
	}

	/**
	 * Rausholen von {@link Data}s aus der Queue
	 * 
	 * @return Jeweils das erste {@link Data} in der Queue
	 * @throws InterruptedException
	 *           Wird geworfen, wenn eine Unterbrechung eintritt.
	 */
	public Data raus() throws InterruptedException
	{
		lock.lock();
		try
		{
			while (produkte.size() == 0)
				nichtLeer.await();

			Data elem = produkte.poll();

			nichtVoll.signalAll();
			return elem;
		} finally
		{
			lock.unlock();
		}
	}

	@SuppressWarnings("unused")
	private class Warten implements Runnable
	{
		/**
		 * Thread-Flag
		 */
		private boolean _laufeThread = false;

		public Warten()
		{
			startThread();
		}

		public void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Wartethread");
				_laufeThread = true;
				th.start();
			}
		}

		public void stopThread()
		{
			if (_laufeThread)
			{
				_laufeThread = false;
			}
		}

		@Override
		public void run()
		{
			while (_laufeThread)
			{
				try
				{
					Thread.sleep(3000);
					synchronized (_warten)
					{
						_warten.notify();
					}
					stopThread();
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

	}

	/**
	 * Baut die Verbindung zum DaV auf.
	 */
	@SuppressWarnings("unused")
	private void baueDaVVerbindungAuf()
	{
		try
		{
			_verbindung = new ClientDavConnection();

			// IP setzen
			_verbindung.getClientDavParameters().setDavCommunicationAddress(_ip);

			// Port setzen
			_verbindung.getClientDavParameters().setDavCommunicationSubAddress(_port);

			// Pfad zum Zwischenspeichern setzen
			_verbindung.getClientDavParameters().setConfigurationPath("C:\\temp");

			try
			{
				_verbindung.connect();
				System.out
						.println("Verbindungsaufbau: Verbindung erfolgreich aufgebaut.");

				try
				{
					_verbindung.login(_benutzer, _passwort);
					System.out
							.println("Verbindungsaufbau: Benutzer erfolgreich eingeloggt.");

				} catch (InconsistentLoginException e)
				{
					System.err
							.println("Verbindungsaufbau Fehler: Konnte Benutzer nicht einloggen.");
					_verbindung = null;
				}

			} catch (CommunicationError e)
			{
				System.err.println("Verbindungsaufbau Fehler: " + e.toString());
				_verbindung = null;
			} catch (ConnectionException e)
			{
				System.err.println("Verbindungsaufbau Fehler: " + e.toString());
				_verbindung = null;
			}

		} catch (MissingParameterException e)
		{
			System.err.println("Verbindungsaufbau Fehler: " + e.toString());
			_verbindung = null;
		}
	}

	/**
	 * Schliet die Verbindung zum DaV
	 */
	@SuppressWarnings("unused")
	private void schliesseDaVVerbindung()
	{
		if (_verbindung != null)
		{
			_verbindung.disconnect(false, "Kein Fehler.");
			_debug.fine("\nVerbindungsabbau: Verbindung wurde abgebaut.");
		}
	}

	/**
	 * Setzt die DaV-Verbindung.
	 * 
	 * @param verbindung
	 *          DaV-Verbindung
	 */
	public void setDaVVerbindung(ClientDavConnection verbindung)
	{
		this._verbindung = verbindung;
		if (_verbindung != null)
		{
			_dataModell = _verbindung.getDataModel();
		}
	}

}
