/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.login;

/**
 * Diese Klasse liefert ein {@link UrlasserInfo}-Objekt, welches die Inhalte
 * des Urlasserdialoges, bestehend aus:
 * <ul>
 * <li>Benutzername,</li>
 * <li>Passwort,</li>
 * <li>Ursache und</li>
 * <li>Veranlasser</li>
 * </ul>
 * beschreibt.
 * 
 * @author HofmannJ
 * 
 */
public class UrlasserInfo
{
	/**
	 * Benutzer
	 */
	private String _benutzer = null;

	/**
	 * Passwort
	 */
	private String _passwort = null;

	/**
	 * Ursache
	 */
	private String _ursache = null;

	/**
	 * Veranlasser
	 */
	private String _veranlasser = null;

	/**
	 * Konstruktor. bernimmt Benutzer, Passwort, Ursache und den Veranlasser
	 * 
	 * @param benutzer
	 *          Benutzer
	 * @param passwort
	 *          Passwort
	 * @param ursache
	 *          Ursache
	 * @param veranlasser
	 *          Veranlasser
	 */
	public UrlasserInfo(String benutzer, String passwort, String ursache,
			String veranlasser)
	{
		this._benutzer = benutzer;
		this._passwort = passwort;
		this._veranlasser = veranlasser;
		this._ursache = ursache;
	}

	/**
	 * Liefert den Benutzer.
	 * 
	 * @return Benutzer. Kann <code>null</code> sein, wenn kein Benutzer gesetzt
	 *         wurde.
	 */
	public String getBenutzer()
	{
		return _benutzer;
	}

	/**
	 * Setzt den Benutzer.
	 * 
	 * @param benutzer
	 *          Benutzer
	 */
	public void setBenutzer(String benutzer)
	{
		this._benutzer = benutzer;
	}

	/**
	 * Liefert das Passwort.
	 * 
	 * @return Passwort. Kann <code>null</code> sein, wenn kein Passwort gesetzt
	 *         wurde.
	 */
	public String getPasswort()
	{
		return _passwort;
	}

	/**
	 * Setzt das Passwort.
	 * 
	 * @param passwort
	 *          Passwort
	 */
	public void setPasswort(String passwort)
	{
		this._passwort = passwort;
	}

	/**
	 * Liefert die Ursache.
	 * 
	 * @return Ursache. Kann <code>null</code> sein, wenn keine Ursache gesetzt
	 *         wurde.
	 */
	public String getUrsache()
	{
		return _ursache;
	}

	/**
	 * Setzt die Ursache.
	 * 
	 * @param ursache
	 *          Ursache
	 */
	public void setUrsache(String ursache)
	{
		this._ursache = ursache;
	}

	/**
	 * Liefert den Veranlasser.
	 * 
	 * @return Veranlasser. Kann <code>null</code> sein, wenn kein Veranlasser
	 *         gesetzt wurde.
	 */
	public String getVeranlasser()
	{
		return _veranlasser;
	}

	/**
	 * Setzt den Veranlasser.
	 * 
	 * @param veranlasser
	 *          Veranlasser
	 */
	public void setVeranlasser(String veranlasser)
	{
		this._veranlasser = veranlasser;
	}
}