/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Startup fr die Mehrfachzwischenablageninitialisierung.
 * 
 * @author HofmannJ
 * 
 */
public class StartMehrfachZwischenablage implements IStartup, Runnable
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Flag, gibt an ob der Thread noch laufen soll.
   */
  private boolean _laufe = true;

  @SuppressWarnings("unused")
  private MehrfachZwischenablageDaten _mehrfachZwischenablageDaten;

  public void earlyStartup()
  {
    _debug.fine(getClass().getName() + "::earlyStartup \n\t=> Early Startup MehrfachZwischenablage\n");

    starteThread();
    _debug.fine(getClass().getName() + "+=> StartUP beendet");
  }

  private void initMehrfachZwischenablage()
  {
    _mehrfachZwischenablageDaten = MehrfachZwischenablageDaten.getInstanz();
  }

  /**
   * Startet den Thread, der kurz nach dem Start der Eclipse RCP die {@link MehrfachZwischenablageDaten} initialisiert.
   * 
   */
  private void starteThread()
  {
    Thread th = new Thread(this);
    th.setName("MehrfachZwischenablage InitialisierungsThread");
    Display display = PlatformUI.getWorkbench().getDisplay();
    display.asyncExec(th);
    // th.start();
  }

  /**
   * Thread, initialisiert die {@link MehrfachZwischenablageDaten} und beendet sich danach.
   */
  public void run()
  {
    while (_laufe)
    {
      _debug.fine(getClass().getName()
          + "::run \n\t=> Thread zur Initialisierung der Mehrfachzwischenablage ist aktiv.\n");

      try
      {
        Thread.sleep(MehrfachZwischenablageParameter.threadInitialisierungsLaufzeit);

        initMehrfachZwischenablage();
        _laufe = false;
        _debug.fine(getClass().getName()
            + "::run \n\t=> Thread zur Initialisierung der Mehrfachzwischenablage beendet sich jetzt.\n");

      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }

    }

  }

}
