/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorStart;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorWurzel;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.EnumDeskriptorTyp;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * View zur Darstellung aller registrierten {@link DeskriptorAktion}s.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:19:32 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class SichtDeskriptorAktionen extends ViewPart
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * ID der View
   */
  public static final String ID = "de.bsvrz.buv.rw.basislib.viewMenuEditorXmlActions";

  /**
   * DescriptorElement
   */
  protected static final Object _DeskriptorElement = null;

  /**
   * DescriptorTyp
   */
  private EnumDeskriptorTyp _enumDeskriptorTyp = EnumDeskriptorTyp.deskriptorAktion;

  private Composite _top = null;

  @SuppressWarnings("unused")
  private Group _gruppeKontrolle = null;

  @SuppressWarnings("unused")
  private Group _gruppeTreeViewer = null;

  @SuppressWarnings("unused")
  private Button _buttonAnlegen = null;

  @SuppressWarnings("unused")
  private Button _buttonAktualisieren = null;

  @SuppressWarnings("unused")
  private Button _buttonXmlElementTest = null;

  @SuppressWarnings("unused")
  private Button _buttonKopieren = null;

  @SuppressWarnings("unused")
  private Button _buttonEinfuegenElement = null;

  @SuppressWarnings("unused")
  private Button _buttonEinfuegenAction = null;

  private CompositeDarstellungRegistrierterElemente compositeDarstellungRegistrierteElemente;

  @Override
  public void createPartControl(Composite parent)
  {
    _top = new Composite(parent, SWT.NONE);
    _top.setLayout(new GridLayout(1, false));

    erzeugeRegistrierteElementeComposite(_top);
    parent.setLayout(new FillLayout());
    erzeugeKontrolle();
    aktionAktualisieren();

    PlatformUI.getWorkbench().getHelpSystem().setHelp(_top, "de.bsvrz.buv.rw.basislib.menuleisteneditor");
  }

  /**
   * Erzeugt das Composite zur Darstellung der registrierten Elemente.
   * 
   * @param comp
   *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
   */
  private void erzeugeRegistrierteElementeComposite(Composite comp)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeDarstellungRegistrierteElemente = new CompositeDarstellungRegistrierterElemente(comp, SWT.NONE,
        _enumDeskriptorTyp);
    compositeDarstellungRegistrierteElemente.setLayoutData(gridData);
  }

  /**
   * Erzeugt Kontrollknpfe i.d. ToolBar der View
   * 
   */
  private void erzeugeKontrolle()
  {
    Action actionAktualisieren = new Action() {
      public void run()
      {
        aktionAktualisieren();
      }
    };
    actionAktualisieren.setText("Aktualisieren");
    actionAktualisieren.setToolTipText("Aktualisieren");
    actionAktualisieren.setImageDescriptor(Activator.getImageDescriptor("icons/allgemein/refresh.gif"));

    getViewSite().getActionBars().getToolBarManager().add(actionAktualisieren);

  }

  /**
   * Aktualisiert die Darstellung der TreeViewers.
   * 
   */
  private void aktionAktualisieren()
  {
    _debug.fine(getClass().getName() + "::aktionAktualisieren => \nAktualisiere den TreeViewer...\n");

    DeskriptorWurzel wurzel = new DeskriptorWurzel("Plug-Ins", _enumDeskriptorTyp);

    DeskriptorStart descriptorStart = new DeskriptorStart(wurzel);
    compositeDarstellungRegistrierteElemente.setzeInhalt(descriptorStart);
  }

  @Override
  public void setFocus()
  {
    if (_top != null)
      _top.setFocus();

    aktionAktualisieren();

  }

}
