/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.jdom.Element;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorStart;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorWurzel;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.EnumDeskriptorTyp;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * View zur Darstellung aller registrierten {@link DeskriptorElement}s.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/14 13:15:29 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class SichtDeskriptorElemente extends ViewPart
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * ID der View
   */
  public static final String ID = "de.bsvrz.buv.rw.basislib.viewMenuEditorXmlElemente";

  /**
   * DescriptorElement
   */
  protected static final Object _DeskriptorElement = null;

  /**
   * DescriptorTyp
   */
  private EnumDeskriptorTyp _enumDeskriptorTyp = EnumDeskriptorTyp.deskriptorElement;

  /**
   * Composite der Darstellung
   */
  private Composite _top = null;

  /**
   * Gruppe f.d. Darstellung der Kontrollelemente
   */
  private Group _gruppeKontrolle = null;

  /**
   * Gruppe zur Darstellung des TreeViewers
   */
  private Group _gruppeTreeViewer = null;

  /**
   * TreeViewer Composite
   */
  private CompositeDarstellungRegistrierterElemente _compositeTreeViewerRegistrierteElemente;

  private Button _buttonAnlegen = null;

  private Button _buttonAktualisieren = null;

  private Button _buttonXmlElementTest = null;

  private Button _buttonKopieren = null;

  private Button _buttonEinfuegenElement = null;

  private Button _buttonEinfuegenAction = null;

  @Override
  public void createPartControl(Composite parent)
  {
    // Composite fr die Basisdarstellung
    _top = new Composite(parent, SWT.NONE);

    // Sub-Container erzeugen
    erzeugeGruppeTreeViewer(_top);
    erzeugeRegistrierteElementeComposite(_gruppeTreeViewer);
    // erzeugeGruppeKontrolle();
    erzeugeKontrolle();

    // Layout erzeugen
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _top.setLayout(gridLayout);

    PlatformUI.getWorkbench().getHelpSystem().setHelp(_top, "de.bsvrz.buv.rw.basislib.menuleisteneditor");
  }

  /**
   * Erzeugt die Steuerung als ViewActions
   * 
   */
  private void erzeugeKontrolle()
  {
    IAction actionLadenAusDatei = new Action() {
      public void run()
      {
        _debug.fine("Laden");
        FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(), SWT.APPLICATION_MODAL
            | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.OPEN);
        dialog.setFilterNames(new String[] { "XML-Dateien" });
        dialog.setFilterExtensions(new String[] { "*.xml" });
        String dateiPfad = dialog.open();
        if (dateiPfad != null)
        {
          xmlStrukturEinlesenAusDatei(dateiPfad);
        }

      }
    };
    actionLadenAusDatei.setText("Tabelle aus XML-Datei laden");
    actionLadenAusDatei.setToolTipText("Tabelle aus XML-Datei laden");
    actionLadenAusDatei.setImageDescriptor(Activator.getImageDescriptor("icons/menubar/laden_16x16.png"));

    IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
    toolBarManager.add(actionLadenAusDatei);

  }

  /**
   * Erzeugt die Gruppe, in der der TreeViewer dargestellt werden soll.
   * 
   * @param comp
   *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
   */
  private void erzeugeGruppeTreeViewer(Composite comp)
  {
    _gruppeTreeViewer = new Group(comp, SWT.NONE);
    _gruppeTreeViewer.setText("Elemente aus Tabelle:");

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _gruppeTreeViewer.setLayout(new GridLayout());
    _gruppeTreeViewer.setLayoutData(gridData);

  }

  /**
   * Erzeugt das Composite zur Darstellung der registrierten Elemente.
   * 
   * @param comp
   *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
   */
  private void erzeugeRegistrierteElementeComposite(Composite comp)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeTreeViewerRegistrierteElemente = new CompositeDarstellungRegistrierterElemente(comp, SWT.NONE,
        _enumDeskriptorTyp);
    _compositeTreeViewerRegistrierteElemente.setLayoutData(gridData);
  }

  /**
   * Erzeugt die Kontroll-Gruppe
   * 
   */
  @SuppressWarnings("unused")
	private void erzeugeGruppeKontrolle()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    _gruppeKontrolle = new Group(_top, SWT.NONE);
    _gruppeKontrolle.setLayout(new RowLayout());
    _gruppeKontrolle.setLayoutData(gridData1);
    _gruppeKontrolle.setText("Kontrolle:");
    _buttonAnlegen = new Button(_gruppeKontrolle, SWT.NONE);
    _buttonAnlegen.setText("Anlegen");
    _buttonAnlegen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionKlassischeAktionenAnlegen();
				aktionAktualisieren();
			}			
		});
    
    
    
    _buttonAktualisieren = new Button(_gruppeKontrolle, SWT.NONE);
    _buttonAktualisieren.setText("Aktualisieren");
    _buttonAktualisieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAktualisieren();
			}			
		});
    

    _buttonXmlElementTest = new Button(_gruppeKontrolle, SWT.PUSH);
    _buttonXmlElementTest.setText("XML Element Test");
    _buttonXmlElementTest.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				 aktionXmlStrukturErzeugenUndEinlesen();
			}			
		});

    _buttonKopieren = new Button(_gruppeKontrolle, SWT.PUSH);
    _buttonKopieren.setText("Kopieren");    
    _buttonKopieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{

        /*
         * Clipboard cp = new Clipboard( PlatformUI.getWorkbench().getDisplay() ); Transfer[] dataTypes = new
         * Transfer[]{ KonfigurationsObjektIdTransfer.getInstanz(), KonfigurationsObjektPidTransfer.getInstanz(),
         * KonfigurationsObjektXmlTransfer.getInstanz(), TextTransfer.getInstance(), }; Object[] data = new Object[]{
         * Long.parseLong(sourceId.getText()), sourcePid.getText(), sourceXml.getText(), sourceXml.getText(), };
         * cp.setContents( data, dataTypes ); cp.dispose();
         * 
         */
        Clipboard cp = new Clipboard(PlatformUI.getWorkbench().getDisplay());
        // Transfer[] dataTypes = new
        // Transfer[]{TransferDescriptorElement.getInstanz(),
        // TransferDescriptorAction.getInstanz()};
        Transfer[] dataTypes = new Transfer[] { TransferDeskriptorElement.getInstanz() };
        // ISelection selection =
        // compositeTreeViewerRegistrierteElemente._treeViewer.getSelection();
        // _selectedObject = ((IStructuredSelection)
        // event.getSelection()).getFirstElement();
        Object firstElement = ((IStructuredSelection)_compositeTreeViewerRegistrierteElemente._treeViewer
            .getSelection()).getFirstElement();

        if (firstElement instanceof DeskriptorElement)
        {
          DeskriptorElement tempElement = (DeskriptorElement)firstElement;
          Object[] data = new Object[] { tempElement };
          cp.setContents(data, dataTypes);
          _debug.fine("DescriptorElement kopiert!");
        }

        // else if(firstElement instanceof DescriptorAction)
        // {
        // DescriptorAction tempAction = (DescriptorAction) firstElement;
        // Object[] data = new Object[]{tempAction};
        // cp.setContents(data, dataTypes);
        // _debug.fine("DescriptorAction kopiert!");
        // }
        cp.dispose();
			}			
		});

    _buttonEinfuegenElement = new Button(_gruppeKontrolle, SWT.PUSH);
    _buttonEinfuegenElement.setText("Einfgen DescriptorElement"); 
    _buttonEinfuegenElement.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{

        /*
         * Clipboard cp = new Clipboard( PlatformUI.getWorkbench().getDisplay() ); Long[] ergebnis = ( Long[]
         * )cp.getContents( KonfigurationsObjektIdTransfer.getInstanz() ); cp.dispose();
         * 
         */
        Clipboard cp = new Clipboard(PlatformUI.getWorkbench().getDisplay());

        Object contents = cp.getContents(TransferDeskriptorElement.getInstanz());

        DeskriptorElement[] tem = (DeskriptorElement[])contents;

        _debug.fine("Einfgen => AUSGABE: " + tem[0].getAktionID() + " " + tem[0].getPluginID() + " "
            + tem[0].getIconPfad());
        cp.clearContents();
        cp.dispose();
			}			
		});

    _buttonEinfuegenAction = new Button(_gruppeKontrolle, SWT.PUSH);
    _buttonEinfuegenAction.setText("Einfgen DescriptorAction");    
    _buttonEinfuegenAction.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
        /*
         * Clipboard cp = new Clipboard( PlatformUI.getWorkbench().getDisplay() ); Long[] ergebnis = ( Long[]
         * )cp.getContents( KonfigurationsObjektIdTransfer.getInstanz() ); cp.dispose();
         * 
         */
        Clipboard cp = new Clipboard(PlatformUI.getWorkbench().getDisplay());

        Object contents = cp.getContents(TransferDeskriptorAktion.getInstanz());

        DeskriptorElement[] tem = (DeskriptorElement[])contents;

        _debug.fine("Einfgen => AUSGABE: " + tem[0].getAktionID() + " " + tem[0].getPluginID() + " "
            + tem[0].getIconPfad());
        cp.clearContents();
        cp.dispose();
			}			
		});
  }

  /**
   * Legt fr den Test klassische Actions an und registriert diese ber ein {@link DeskriptorAktion} und
   * {@link DeskriptorElement}.
   * 
   */
  private void aktionKlassischeAktionenAnlegen()
  {
    _debug.fine("Erzeuge Testelemente...");
    ElementRegistrierung registrierung = ElementRegistrierung.getInstanz();

    /*
     * Testelemente erzeugen
     */
    DeskriptorElement element1 = new DeskriptorElement("Plugin 1", "1.1", "Ein Label 1.1", null, null, null, null,
        null, true);
    DeskriptorElement element2 = new DeskriptorElement("Plugin 2", "2.1", "Ein Label 1.1", null, null, null, null,
        null, false);

    IAction action = new Action() {
      public void run()
      {
        _debug.fine("Hallo Welt!");
      }
    };
    action.setId("Action.1");
    action.setText("Action 1 Text");

    DeskriptorAktion action1 = new DeskriptorAktion("Plugin 3", action, false);

    DeskriptorElement element3 = new DeskriptorElement("Plugin 3", "3.1", "Ein Label 1.1", null, null, null, null,
        null, true);

    // Globale Aktion
    IAction globaleAction = new Action() {
      public void run()
      {
        _debug.fine("Globale Aktion - Hallo Welt!");
      }
    };
    globaleAction.setId("Action.Global.1");
    globaleAction.setText("Action 1 Global");
    DeskriptorAktion descriptorActionGlobal = new DeskriptorAktion("Plugin GlobalTest", globaleAction, true);

    /*
     * Testelemente registrieren
     */
    try
    {
      registrierung.registriereDeskriptorElement(element1);
      registrierung.registriereDeskriptorElement(element2);
      registrierung.registriereDeskriptorAktion(action1);
      registrierung.registriereDeskriptorElement(element3);
      registrierung.registriereDeskriptorAktionGlobal(descriptorActionGlobal);
    }
    catch (IllegalArgumentException err)
    {
      err.printStackTrace();
    }

  }

  /**
   * Aktualisiert die Darstellung der TreeViewers.
   * 
   */
  private void aktionAktualisieren()
  {
    _debug.fine(getClass().getName() + "::aktionAktualisieren => \n Aktualisiere den TreeViewer...\n");

    DeskriptorWurzel wurzel = new DeskriptorWurzel("Plug-Ins", _enumDeskriptorTyp);
    // DescriptorWurzel wurzel = new DescriptorWurzel("Plug-Ins",
    // EnumDescriptorTyp.descriptorElement);
    DeskriptorStart descriptorStart = new DeskriptorStart(wurzel);
    _compositeTreeViewerRegistrierteElemente.setzeInhalt(descriptorStart);
  }

  /**
   * Erzeugt eine XML-Struktur zum Testen und liest diese ein.
   * 
   */
  private void aktionXmlStrukturErzeugenUndEinlesen()
  {
    /*
     * Erzeugt ein XML-Element zum Testen.
     */
    XmlDeskriptorElementImport elementImport = new XmlDeskriptorElementImport();
    String testString = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" + "<elemente>"
        + "<element pluginId=\"de.plugin\" actionId=\"de.plugin.action.1\" labelText=\"1. Action\"/>"
        + "<element pluginId=\"de.plugin\" actionId=\"de.plugin.action.2\" labelText=\"2. Action\"/>"
        + "<element pluginId=\"de.plugin\" actionId=\"de.plugin.action.3\" labelText=\"3. Action\"/>"
        + "<element pluginId=\"de.plugin.addon\" actionId=\"de.plugin.addon.action.1\" labelText=\"1. Action\"/>"
        + "<element pluginId=\"de.plugin.addon\" actionId=\"de.plugin.addon.action.2\" labelText=\"2. Action\"/>"
        + "<element pluginId=\"de.plugin.addon\" actionId=\"de.plugin.addon.action.3\" labelText=\"3. Action\"/>"
        + "</elemente>";
    elementImport.einlesenXml(testString);
    Element wurzel = elementImport.getWurzel();

    _debug.fine("Wurzel: " + wurzel.getName());
    elementImport.importieren();
    MessageBox box = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell());
    box.setText("Info");
    box.setMessage("XML Struktur wurde eingelesen.");
    box.open();
    aktionAktualisieren();
  }

  private void xmlStrukturEinlesenAusDatei(String dateiPfad)
  {
    String xmlDateiInhalt = "";
    FileReader fr;
    try
    {
      fr = new FileReader(dateiPfad);
      BufferedReader br = new BufferedReader(fr);
      String line;
      while ((line = br.readLine()) != null)
      {
        xmlDateiInhalt = xmlDateiInhalt + line;
      }
      xmlStrukturEinlesenAusString(xmlDateiInhalt);
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
    }
    catch (IOException ioe)
    {
      ioe.printStackTrace();
    }

  }

  /**
   * Liest eine XML-Struktur aus einem String ein.
   * 
   * @param xmlString
   *          String mit der XML-Struktur
   */
  private void xmlStrukturEinlesenAusString(String xmlString)
  {
    XmlDeskriptorElementImport elementImport = new XmlDeskriptorElementImport();
    elementImport.einlesenXml(xmlString);
    Element wurzel = elementImport.getWurzel();
    _debug.fine("Wurzel: " + wurzel.getName());
    elementImport.importieren();
    MessageBox box = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell());
    box.setText("Info");
    box.setMessage("XML Struktur wurde eingelesen.");
    box.open();
    aktionAktualisieren();

  }

  @Override
  public void setFocus()
  {
    if (_top != null)
      _top.setFocus();
  }

}
