/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.LinkedList;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert den Transfer fr {@link DeskriptorAktion}s.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 07:44:07 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class TransferDeskriptorAktion extends ByteArrayTransfer
{
	/**
	 * Der Debugger
	 */
	private static Debug _debug = Debug.getLogger();

	/**
	 * TypName
	 */
	private static final String _typName = "Descriptor-Action-Format";

	/**
	 * TypeId
	 */
	private static int _typeId = -1;

	/**
	 * Instanz
	 */
	private static TransferDeskriptorAktion _instanz = null;

	public static TransferDeskriptorAktion getInstanz()
	{
		if (_instanz == null)
		{
			_instanz = new TransferDeskriptorAktion();
			_typeId = registerType(_typName);
		}

		_debug.fine(_instanz.getClass().getName() + " => TypeID " + _typeId);

		return _instanz;
	}

	public void javaToNative(Object object, TransferData transferData)
	{
		if (object == null || !(object instanceof DeskriptorAktion[])
				&& !(object instanceof DeskriptorAktion))
		{
			_debug.fine(getClass().getName() + "Abbruch: "
					+ object.getClass().getName());

			return;
		}

		if (isSupportedType(transferData))
		{
			_debug.fine(getClass().getName()
					+ " => javaToNative untersttzer Transfertyp");

			DeskriptorAktion[] actions = null;

			// Instanz von DescriptorAction?
			if (object instanceof DeskriptorAktion)
			{
				// dann in ein DescriptorAction[] mit einem Element umwandeln
				DeskriptorAktion action = (DeskriptorAktion) object;
				actions = new DeskriptorAktion[] { action };
			} else if (object instanceof DeskriptorAktion[])
			{
				// Sonst nur nach DescriptorElement[]-casten
				actions = (DeskriptorAktion[]) object;
			}

			try
			{
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				DataOutputStream writeOut = new DataOutputStream(out);
				for (int i = 0; i < actions.length; i++)
				{

					// PluginID
					String pluginID = actions[i].getPluginID();
					if (pluginID == null)
					{
						pluginID = "";
					}
					byte[] buffer = pluginID.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// ActionId
					String actionID = actions[i].getAktionID();
					if (actionID == null)
					{
						actionID = "";
					}
					buffer = actionID.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// TestID
					String testID = "testId";
					buffer = testID.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// GlobaleAction, umgesetzt als ja oder nein (entstprechend
					// true||false)
					String globaleAction = "";
					if (actions[i].istGlobaleAktion())
					{
						globaleAction = "ja";
					} else
					{
						globaleAction = "nein";
					}
					buffer = globaleAction.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// LabelText
					String labelText = actions[i].getText();
					if (labelText == null)
					{
						labelText = "";
					}
					buffer = labelText.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// ToolTipText
					String toolTipText = actions[i].getToolTipText();
					if (toolTipText == null)
					{
						toolTipText = "";
					}
					_debug.fine(toolTipText);
					buffer = toolTipText.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// ActionDescription
					String actionDescription = actions[i].getAktionBeschreibung();
					if (actionDescription == null)
					{
						actionDescription = "";
					}
					buffer = actionDescription.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// IconPfad
					String iconPfad = actions[i].getIconPfad();
					if (iconPfad == null)
					{
						iconPfad = "";
					}
					buffer = iconPfad.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// HoverIconPfad
					String hoverIconPfad = actions[i].getHoverIconPfad();
					if (hoverIconPfad == null)
					{
						hoverIconPfad = "";
					}
					buffer = hoverIconPfad.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// DisabledIconPfad
					String disabledIconPfad = actions[i].getDeaktiviertesIconPfad();
					if (disabledIconPfad == null)
					{
						disabledIconPfad = "";
					}
					buffer = disabledIconPfad.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);
				}

				byte[] buffer = out.toByteArray();
				writeOut.close();

				super.javaToNative(buffer, transferData);

			} catch (IOException e)
			{
				e.printStackTrace();
			}
		} else
		{
			_debug.fine(getClass().getName()
					+ " => javaToNative Nicht untersttzer Transfertyp");

		}
	}

	public Object nativeToJava(TransferData transferData)
	{
		if (isSupportedType(transferData))
		{
			_debug.fine(getClass().getName()
					+ " => nativeToJava untersttzer Transfertyp");

			byte[] buffer = (byte[]) super.nativeToJava(transferData);
			if (buffer == null)
			{
				_debug.fine(getClass().getName() + " => nativeToJava: Buffer == NULL");

				return null;
			}

			LinkedList<DeskriptorAktion> lDeskriptorAktionen = new LinkedList<DeskriptorAktion>();

			String PluginId = null;
			String ActionId = null;
			@SuppressWarnings("unused")
			String TestId = null;
			boolean GlobaleAction = false;
			String LabelText = null;
			String ToolTipText = null;
			String ActionDescripton = null;
			String IconPfad = null;
			String HoverIconPfad = null;
			String DisabledIconPfad = null;

			try
			{
				ByteArrayInputStream in = new ByteArrayInputStream(buffer);
				DataInputStream readIn = new DataInputStream(in);
				while (readIn.available() > 20)
				{
					// PluginId
					int size = readIn.readInt();
					byte[] pluginid = new byte[size];
					readIn.read(pluginid);
					PluginId = new String(pluginid);

					// ActionId
					size = readIn.readInt();
					byte[] actionid = new byte[size];
					readIn.read(actionid);
					ActionId = new String(actionid);

					// TestID
					size = readIn.readInt();
					byte[] testid = new byte[size];
					readIn.read(testid);
					TestId = new String(testid);

					// Globale Action
					size = readIn.readInt();
					byte[] globaleaction = new byte[size];
					readIn.read(globaleaction);
					String tempGlobaleAction = new String(globaleaction);
					if (tempGlobaleAction.equals("ja"))
					{
						GlobaleAction = true;
					} else
					{
						GlobaleAction = false;
					}

					// LabelText
					size = readIn.readInt();
					byte[] labeltext = new byte[size];
					readIn.read(labeltext);
					LabelText = new String(labeltext);

					// ToolTipText
					size = readIn.readInt();
					byte[] tooltiptext = new byte[size];
					readIn.read(tooltiptext);
					ToolTipText = new String(tooltiptext);

					// ActionDescription
					size = readIn.readInt();
					byte[] actiondescription = new byte[size];
					readIn.read(actiondescription);
					ActionDescripton = new String(actiondescription);

					// IconPfad
					size = readIn.readInt();
					byte[] iconpfad = new byte[size];
					readIn.read(iconpfad);
					IconPfad = new String(iconpfad);

					// HoverIconPfad
					size = readIn.readInt();
					byte[] hovericonpfad = new byte[size];
					readIn.read(hovericonpfad);
					HoverIconPfad = new String(hovericonpfad);

					// DisabledIconPfad
					size = readIn.readInt();
					byte[] disablediconpfad = new byte[size];
					readIn.read(disablediconpfad);
					DisabledIconPfad = new String(disablediconpfad);

					DeskriptorAktion tempAction = new DeskriptorAktion(PluginId,
							ActionId, GlobaleAction, LabelText, ToolTipText,
							ActionDescripton, IconPfad, HoverIconPfad, DisabledIconPfad);

					DeskriptorAktion descriptorAction = null;
					if (!GlobaleAction)
					{
						descriptorAction = ElementRegistrierung.getInstanz()
								.getDeskriptorAktion(ActionId);
					} else
					{
						descriptorAction = ElementRegistrierung.getInstanz()
								.getDeskriptorAktionGlobal(ActionId);
					}

					if (descriptorAction != null)
					{
						lDeskriptorAktionen.add(descriptorAction);
					} else
					{
						lDeskriptorAktionen.add(tempAction);
					}

				}
				readIn.close();

			} catch (IOException e)
			{
				e.printStackTrace();
				return null;
			}

			DeskriptorAktion[] aDeskriptorAktionen = lDeskriptorAktionen
					.toArray(new DeskriptorAktion[lDeskriptorAktionen.size()]);

			return aDeskriptorAktionen;
		}
		return null;
	}

	@Override
	protected int[] getTypeIds()
	{
		return new int[] { _typeId };
	}

	@Override
	protected String[] getTypeNames()
	{
		return new String[] { _typName };
	}

}
