/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff;

import java.util.LinkedList;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.TransferData;

import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert den {@link DragSourceAdapter} fr die Elemente des TreeViewers im
 * {@link CompositeDarstellungRegistrierterElemente}.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 07:44:07 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class ZugZuhoererDeskriptor extends DragSourceAdapter
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Viewer, auf dem gearbeitet wird.
	 */
	private StructuredViewer _sicht;

	public ZugZuhoererDeskriptor(StructuredViewer viewer)
	{
		this._sicht = viewer;
	}

	public void dragFinished(DragSourceEvent event)
	{

		if (!event.doit)
			return;
		// if the gadget was moved, remove it from the source viewer
		// if (event.detail == DND.DROP_MOVE) {
		// IStructuredSelection selection =
		// (IStructuredSelection)_viewer.getSelection();
		// for (Iterator it = selection.iterator(); it.hasNext();) {
		// //Entfernen der Elemente
		// }
		// _viewer.refresh();
		// }
		if (event.detail == DND.DROP_MOVE)
		{
			_debug.fine(getClass().getName() + " => Lschen.");
			// IStructuredSelection selection
			// =(IStructuredSelection)_viewer.getSelection();
			// Iterator und aktualisieren der Elemente
			_sicht.refresh();
		}

		_debug.fine(getClass().getName() + " => dragFinished");

	}

	/**
	 * Liefert die per dragging selektierten Elemente des TreeViewers.
	 */
	public void dragSetData(DragSourceEvent event)
	{
		_debug.fine(getClass().getName() + "::dragSetData => begin.");

		IStructuredSelection selection = (IStructuredSelection) _sicht
				.getSelection();

		Object selectedObject = selection.getFirstElement();

		// DescriporElement?
		if (selectedObject instanceof DeskriptorElement)
		{
			_debug.fine(getClass().getName()
					+ "::dragSetData => DescriptorElement festgestellt.");

			if (TransferDeskriptorElement.getInstanz()
					.isSupportedType(event.dataType))
			{
				_debug.fine(getClass().getName()
						+ "::dragSetData => DescriptorElement dragSetData isSupportedType");

				LinkedList<DeskriptorElement> lDeskriptorElements = new LinkedList<DeskriptorElement>();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					Object[] aSelObj = ss.toArray();
					for (Object selObj : aSelObj)
					{
						if (selObj instanceof DeskriptorElement)
						{
							lDeskriptorElements.add((DeskriptorElement) selObj);
						}
					}
				}
				event.data = lDeskriptorElements
						.toArray(new DeskriptorElement[lDeskriptorElements.size()]);

			} else
			{
				System.out
						.println(getClass().getName()
								+ "::dragSetData => DescriptorElement dragSetData !isSupportedType");

			}

		}
		// DescriptorAction?
		if (selectedObject instanceof DeskriptorAktion)
		{
			_debug.fine(getClass().getName()
					+ "::dragSetData => DescriptorAction festgestellt.");

			if (TransferDeskriptorAktion.getInstanz().isSupportedType(event.dataType))
			{
				_debug.fine(getClass().getName()
						+ "::dragSetData => DescriptorAction dragSetData isSupportedType");

				LinkedList<DeskriptorAktion> lDeskriptorAktions = new LinkedList<DeskriptorAktion>();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					Object[] aSelObj = ss.toArray();
					for (Object selObj : aSelObj)
					{
						if (selObj instanceof DeskriptorAktion)
						{
							lDeskriptorAktions.add((DeskriptorAktion) selObj);
						}
					}
				}
				event.data = lDeskriptorAktions
						.toArray(new DeskriptorAktion[lDeskriptorAktions.size()]);

			} else
			{
				System.out.println(getClass().getName()
						+ "::dragSetData => DescriptorAction dragSetData !isSupportedType");

			}

		}

		_debug.fine(getClass().getName() + "::dragSetData => begin.");

	}

	/**
	 * Wird whrend des Starts der Drag-Vorgangs aufgerufen. Bestimmt den Datentyp
	 * und stellt sicher, dass keine leere Selektion "gedraged" werden kann.
	 */
	public void dragStart(DragSourceEvent event)
	{
		_debug.fine(getClass().getName() + "::dragStart => Start");

		@SuppressWarnings("unused")
		TransferData dataType = event.dataType;
		event.doit = !_sicht.getSelection().isEmpty();
	}

}
