/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.editoren;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.EditorPart;
import org.jdom.Attribute;
import org.jdom.Element;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.basislib.einstellungen.dialog.DialogBenutzerWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen.DefinitionenEinstellungen;
import de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff.TransferDeskriptorAktion;
import de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff.TransferDeskriptorElement;
import de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.EnumLeistenArt;
import de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.EnumSpeicherOrt;
import de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.EnumSubjekt;
import de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.LadeSpeicherDialogMenu;
import de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.SpeicherInfo;
import de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.LadeSpeicherDialogMenu.EnumDialogSpeichernLaden;
import de.bsvrz.buv.rw.basislib.menu.generatoren.MenuGenerator;
import de.bsvrz.buv.rw.basislib.menu.generatoren.SymbolleisteGenerator;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.IFeuereSchmutzigGeandert;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.IZuhoererUndoStackMenuSymbol;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.UndoStackMenuSymbol;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlBezeichnerMenuSymbol;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlElementMenu;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlElementSeparator;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlElementSubmenu;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten.enumLeistenArt;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Implementiert den Men-Editor innerhalb eines TreeViewers. Dies beinhaltet
 * die grafische Darstellung innerhalb eines Views sowie die Bereitstellung
 * eines Kontextmens mit den entsprechenden Editierfunktionen.
 * 
 * @version $Revision: 1.6 $ / $Date: 2010/06/02 09:43:21 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class EditorMenuSymbolLeiste extends EditorPart implements
		IFeuereSchmutzigGeandert
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * ID dieses Parts
	 */
	public static final String ID = "de.bsvrz.buv.rw.basislib.editorMenu";

	/**
	 * Top Composite
	 */
	private Composite _compositeOben = null;

	/**
	 * Composite zur Darstellung des TreeViewers
	 */
	private Composite _compositeBaum = null;

	/**
	 * Composite fr die Darstellung der Buttons
	 */
	private Composite _compositeButtons = null;

	/**
	 * Composite fr die Darstellung des InfoBereichs
	 */
	private Composite _compositeInfoBereich = null;

	/*
	 * Element-Definitionen
	 */

	/**
	 * Name des Elements item
	 */
	// private String _elementItem = "item";
	private String _elementItem = XmlBezeichnerMenuSymbol.elementItem;

	/**
	 * Name des Elements separator
	 */
	// private String _elementSeparator = "separator";
	private String _elementSeparator = XmlBezeichnerMenuSymbol.elementSeparator;

	// /**
	// * Name des Elements groupmarker
	// */
	// private String _elementGroupMarker = "groupmarker";

	/**
	 * Namen des Elements menu
	 */
	// private String _elementMenu = "menu";
	private String _elementMenu = XmlBezeichnerMenuSymbol.elementMenu;

	/**
	 * Namen des Elements submenu
	 */
	// private String _elementSubmenu = "submenu";
	private String _elementSubmenu = XmlBezeichnerMenuSymbol.elementSubMenu;

	private Button _buttonEinfuegen = null;

	private Text _textElementSpeicher = null;

	/**
	 * Gespeichertes Element, z.B. bei Kopieraktionen
	 */
	private Element _einfuegenGespeichertesElement = null;

	/**
	 * Element fr _compositeInfoBereich
	 */
	private Text _textElementInfo = null;

	/**
	 * TreeViewer, in dem die Darstellung der Menstruktur erfolgen soll
	 */
	private TreeViewer _treeViewer = null;

	/**
	 * Reprsentiert das Objekt hinter der Selektion im TreeViewer
	 */
	@SuppressWarnings("unused")
	private Object _selektiertesObject = null;

	/**
	 * Liefert die Datenstruktur + Speichern / Hinzufgen / Lschen / Laden
	 */
	private MenuSymbolLeisteDaten _msld;

	/*
	 * Globale Actions
	 */

	/**
	 * Action fr "Speichern unter"
	 */
	private Action _aktionSpeichernUnter;

	/**
	 * Action Speichern unter fr einen Bestimmten Benutzer auf dem DaV als
	 * Menleiste
	 */
	private Action _aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV = null;

	/**
	 * Action Speichern unter fr den angemeldeten Benutzer auf dem DaV als
	 * Menleiste
	 */
	private Action _aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV = null;

	/**
	 * Action Speichern unter fr einen Bestimmten Benutzer auf dem DaV als
	 * Menleiste
	 */
	private Action _aktionMenuleisteSpeichernUnterNetzweitDaV = null;

	/**
	 * Action Einlesen der Menstruktur fr einen bestimmten Benutzer aus dem DaV.
	 */
	private Action _aktionMenuleisteEinlesenBestimmterBenutzerDaV = null;

	/**
	 * Aktion Einlesen der Menstruktur fr den angemeldeten Benutzer aus dem DaV
	 */
	private Action _aktionMenuleisteEinlesenAngemeldeterBenutzerDav = null;

	/**
	 * Aktion Einlesen der Menstruktur aus den netzweiten Einstellungen
	 */
	private Action _aktionMenuleisteEinlesenNetzweitDaV = null;

	/**
	 * Aktiviert die aktuell im Editor dargestellte Menstruktur.
	 */
	private Action _aktionMenuAktuellAktivieren = null;

	/**
	 * Action Speichern unter fr einen Bestimmten Benutzer auf dem DaV als
	 * Symbolleiste
	 */
	private Action _aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV = null;

	/**
	 * Action Speichern unter fr den angemeldeten Benutzer auf dem DaV als
	 * Symbolleiste
	 */
	private Action _aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV = null;

	/**
	 * Action Speichern unter fr einen Bestimmten Benutzer auf dem DaV als
	 * Symbolleiste
	 */
	private Action _aktionSymbolleisteSpeichernUnterNetzweitDaV = null;

	/**
	 * Action Einlesen der Symbolleistenstruktur fr einen bestimmten Benutzer aus
	 * dem DaV.
	 */
	private Action _aktionSymbolleisteEinlesenBestimmterBenutzerDaV = null;

	/**
	 * Aktion Einlesen der Symbolleistenstruktur fr den angemeldeten Benutzer aus
	 * dem DaV
	 */
	private Action _aktionSymbolleisteEinlesenAngemeldeterBenutzerDav = null;

	/**
	 * Aktion Einlesen der Symbolleistenstruktur aus den netzweiten Einstellungen
	 */
	private Action _aktionSymbolleisteEinlesenNetzweitDaV = null;

	/**
	 * Aktiviert die aktuell im Editor dargestellte Symbolleistenstruktur.
	 */
	private Action _aktionSymbolleisteAktuellAktivieren = null;

	/**
	 * Aktion zum Umbenennen des aktuell selektierten Submens
	 */
	private Action _aktionSubmenuUmbenennen = null;

	/**
	 * Aktion fr die globale Aktion "Laden"
	 */
	private Action _aktionLadenGlobal = null;

	/**
	 * Aktion fr die globale Aktion "Speichern"
	 */
	private Action _aktionSpeichernGlobal = null;

	/**
	 * Action fr "Speichern"
	 */
	private Action _aktionSpeichern;

	/**
	 * Handler fr die globale Aktion <code>UNDO</code>.
	 */
	private Action _aktionHandlerUndo;

	/**
	 * Handler fr die globale Aktion <code>REDO</code>.
	 */
	private Action _aktionHandlerRedo;

	/**
	 * Action fr "Laden aus Datei"
	 */
	private Action _ladenAusBestimmterDatei;

	private BaumFallAdapterElementSichtMenuLeisteEditor _baumFallAdapterElementSichtMenuLeisteEditor;

	/**
	 * {@link ZugZuhoererXmlElement}, merkt sich das gezogene Element
	 */
	private ZugZuhoererXmlElement _zugZuhoererXmlElement;

	/**
	 * Flag das anzeigt, das bestimmte Dateien verwendet werden sollen.
	 */
	private boolean _festeDatei = false;

	/**
	 * XML-Datei, auf der der TreeViewer aufbaut.
	 */
	public String _xmlDateiPfad = "";

	/**
	 * Listener fr den aktuellen Zustand des UNDO-Stacks
	 */
	private IZuhoererUndoStackMenuSymbol _zuhoererUndoStack = null;

	/**
	 * Was soll im Laden Dialog ausgewhlt werden? Menleiste oder Symbolleiste
	 * wird hierdurch bestimmt.
	 */
	private de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.EnumLeistenArt ladenLeistenArt = EnumLeistenArt.menuLeiste;

	private ImageRegistry _ir = new ImageRegistry();

	@Override
	public void createPartControl(Composite parent)
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;

		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;

		_compositeOben = new Composite(parent, SWT.NONE);
		_compositeOben.setLayout(new GridLayout());

		_compositeBaum = new Composite(_compositeOben, SWT.NONE);
		_compositeBaum.setLayout(new FillLayout());
		_compositeButtons = new Composite(_compositeOben, SWT.BORDER);
		_compositeButtons.setLayout(new RowLayout());

		erzeugeEditor(_compositeBaum);
		// buildButtons(_compositeButtons);
		erzeugeInfoBereich();

		_compositeBaum.setLayoutData(gridData);
		_compositeButtons.setLayoutData(gridData2);

		IEditorInput input = getEditorInput();
		if (input instanceof EditorMenuSymbolLeisterInput)
		{
			EditorMenuSymbolLeisterInput ei = (EditorMenuSymbolLeisterInput) input;
			this._msld = ei.getMSLD();
			_msld.setFeuereSchmutzigGeaendert(this);
			if (_baumFallAdapterElementSichtMenuLeisteEditor != null)
			{
				_baumFallAdapterElementSichtMenuLeisteEditor
						.setMenuSymbolLeistedaten(_msld);
			}

			// treeViewerAktuallisieren();
			treeViewerNeuLaden();
		}

		/*
		 * Listener fr den UNDO-Stack erzeugen und registrieren
		 */
		erzeugeUndoStackListener();
		_msld.getUndoStack().addZuhoererUndoStack(_zuhoererUndoStack);

		PlatformUI.getWorkbench().getHelpSystem().setHelp(_compositeOben,
				"de.bsvrz.buv.rw.basislib.menuleisteneditor");

	}

	/**
	 * Baut die Darstellung des TreeViewers auf
	 * 
	 * @param fillComposite
	 *          Composite, in dem die Darstellung des TreeViewers erfolgen soll
	 */
	private void erzeugeEditor(Composite fillComposite)
	{
		_treeViewer = new TreeViewer(fillComposite, SWT.BORDER | SWT.H_SCROLL
				| SWT.V_SCROLL);
		_treeViewer.setContentProvider(new XmlBaumInhaltsVersorger());
		_treeViewer.setLabelProvider(new XmlBaumDarstellungsVersorger());
		_msld = new MenuSymbolLeisteDaten();

		// Basisdarstellung eines leeren Menbaumes.
		_treeViewer.setInput(_msld.getWurzel());
		_treeViewer.expandAll();

		// int ops = DND.DROP_COPY;
		int ops = DND.DROP_MOVE;
		Transfer[] dropTransfers = new Transfer[] {
				TransferDeskriptorElement.getInstanz(),
				TransferDeskriptorAktion.getInstanz(), TextTransfer.getInstance() };
		// _treeViewer.addDropSupport(ops, transfers, new
		// TreeDropAdapterDescriptorElement(_treeViewer));

		_baumFallAdapterElementSichtMenuLeisteEditor = new BaumFallAdapterElementSichtMenuLeisteEditor(
				_treeViewer, _msld);
		_treeViewer.addDropSupport(ops, dropTransfers,
				_baumFallAdapterElementSichtMenuLeisteEditor);

		Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
		_zugZuhoererXmlElement = new ZugZuhoererXmlElement(_treeViewer);
		_treeViewer.addDragSupport(ops, dragTransfers, _zugZuhoererXmlElement);

		// _menuBarData.einlesenXml(_xmlFile);
		// _menuBarData.ausgabe();
		// Element wurzel = _menuBarData.getWurzel();

		// _treeViewer.setInput(wurzel);
		// _treeViewer.expandAll();

		_debug.fine(getClass().getName() + "::buildViewer => Tree Aufgebaut!");

		_treeViewer.getTree().addListener(SWT.MouseDown, new Listener()
		{

			public void handleEvent(Event event)
			{

				IStructuredSelection is = (IStructuredSelection) _treeViewer
						.getSelection();
				Object firstElement = is.getFirstElement();
				if (firstElement instanceof Element)
				{
					Element tempElement = (Element) firstElement;
					_debug.fine("Angeklickt ==> " + elementNachString(tempElement));

					setTextElementInfo("Aktuelle Selektion: "
							+ elementNachString(tempElement));
				}

				// _buttonLoeschen.setEnabled(true);
				// _buttonAusschneiden.setEnabled(true);
			}

		});

		_treeViewer.getTree().addMouseListener(new MouseAdapter()
		{
			public void mouseDown(MouseEvent event)
			{
				Point point = new Point(event.x, event.y);
				TreeItem item = _treeViewer.getTree().getItem(point);
				if (item != null && (event.button == 2 || event.button == 3))
				{
					System.out
							.println("An Punkt "
									+ event.x
									+ ","
									+ event.y
									+ " wurde im TreeItem ein Objekt von folgender Klasse hinterlegt: "
									+ item.getData().getClass().getName());
				}
			}
		});

		// "POPUP"-Listener
		_treeViewer.addSelectionChangedListener(new ISelectionChangedListener()
		{

			public void selectionChanged(SelectionChangedEvent event)
			{
				_selektiertesObject = ((IStructuredSelection) event.getSelection())
						.getFirstElement();
				_debug.fine("Poplistener selectionChanged Event registriert.");

			}
		});

		generiereBasisActions();
		verankereKontextMenu();
		verteileInActionBars();

	}

	/*
	 * Fhrt die Initialisierung des <code>TableViewers</code> mit einer
	 * bestimmten XML-Datei durch. @param xmlPfad Pfad zur XML-Datei.
	 */

	/*
	 * Fhrt die Initialisierung des <code>TreeViewers</code> mit einer bestimmten
	 * XML-Datei durch.
	 * 
	 * @param xmlPfad Pfad zur XML-Datei.
	 */
	// public void externeInitialisierung(String xmlPfad)
	// {
	// // neues MenuBarData erzeugen
	// _menuBarData = new MenuBarDaten();
	//
	// // Pfad zur XML-Datei setzen
	// _xmlFile = xmlPfad;
	//
	// // einlesen der XML-Datei in MenuBarData
	// _menuBarData.einlesenXmlAusDatei(_xmlFile);
	//
	// // Wurzel der XML-Datei als Input des TreeViewers setzen
	// _treeViewer.setInput(_menuBarData.getWurzel());
	//
	// // Alle Knoten des TreeViewers ausklappen
	// _treeViewer.expandAll();
	//
	// }
	/**
	 * Fhrt die externe Initialisierung durch.
	 * 
	 * @param msld
	 *          {@link MenuSymbolLeisteDaten}.
	 */
	public void externeInitialisierung(MenuSymbolLeisteDaten msld)
	{
		/*
		 * Der Verweis der alten Datenstruktur auf die Instanz dieses Editors wird
		 * auf null gesetzt. Der GC kann dann aufrumen.
		 */
		_msld.setFeuereSchmutzigGeaendert(null);

		/*
		 * Neue Instanz von MenuBarDaten setzen.
		 */
		this._msld = msld;

		/*
		 * Dem BaumFallAdapter die neue Instanz der Klasse MenuSymbolLeisteDaten
		 * mitteilen.
		 */
		_baumFallAdapterElementSichtMenuLeisteEditor
				.setMenuSymbolLeistedaten(_msld);

		/*
		 * Der Datenstruktur mitteilen, dass diese Klasse ber das Dirty-Flag
		 * informiert werden muss.
		 */
		_msld.setFeuereSchmutzigGeaendert(this);

		/*
		 * Wurzel der XML-Datei als Input des TreeViewers setzen
		 */
		_treeViewer.setInput(_msld.getWurzel());

		/*
		 * Alle Knoten des TreeViewers ausklappen
		 */
		_treeViewer.expandAll();
	}

	/**
	 * Baut den Info-Bereich auf
	 * 
	 */
	private void erzeugeInfoBereich()
	{
		_compositeInfoBereich = new Composite(_compositeOben, SWT.BORDER);

		Label labelInfo = new Label(_compositeInfoBereich, SWT.NONE);
		labelInfo.setText("Info: ");

		_textElementInfo = new Text(_compositeInfoBereich, SWT.NONE);
		_textElementInfo.setEditable(false);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		_compositeInfoBereich.setLayout(new RowLayout());
		_compositeInfoBereich.setLayoutData(gridData);
	}

	/**
	 * Verankert das Kontextmen
	 * 
	 */
	private void verankereKontextMenu()
	{

		_debug.fine(getClass().getName()
				+ "::verankereContextMenu => Verankere Kontextmen");

		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{

				_debug.fine("menuaboutToShow");

				fuelleKontextMenu(manager);
			}
		});

		Menu menu = menuMgr.createContextMenu(_treeViewer.getControl());
		_treeViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, _treeViewer);
	}

	/**
	 * Fllt das Kontextmen
	 * 
	 * @param manager
	 */
	private void fuelleKontextMenu(IMenuManager manager)
	{
		_debug.fine("Flle Kontextmen!");

		/*
		 * Hier erfolgt die Intitialisierung der Actions, die bereitgestellt werden
		 * solle. Mittels ((IStructuredSelection)
		 * _treeViewer.getSelection()).getFirstElement() wird das aktuell
		 * selektierte Element im TreeViewer bestimmt.
		 */

		// Action neu aus Datei laden
		Action actionNeuLadenAusDatei = new Action()
		{
			public void run()
			{
				aktionMethodeNeuLadenAusBestimmterDatei();
			}
		};
		actionNeuLadenAusDatei.setText("Laden aus (XML-Datei)");
		actionNeuLadenAusDatei.setToolTipText("Laden einer XML-Datei");
		actionNeuLadenAusDatei
				.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));
		manager.add(actionNeuLadenAusDatei);

		// Action Speichern Unter hinzufgen
		manager.add(_aktionSpeichernUnter);

		// Action Speichern hinzufgen
		// manager.add(_aktionSpeichern);

		/*
		 * Menleistenspezifisches
		 */
		manager.add(new Separator());
		MenuManager mgrMenuleiste = new MenuManager("Menleiste");
		manager.add(mgrMenuleiste);

		if (_aktionSpeichernGlobal != null)
		{
			mgrMenuleiste.add(_aktionSpeichernGlobal);
		}

		if (_aktionLadenGlobal != null)
		{
			mgrMenuleiste.add(_aktionLadenGlobal);
		}

		if (_aktionMenuAktuellAktivieren != null)
		{
			mgrMenuleiste.add(new Separator());
			mgrMenuleiste.add(_aktionMenuAktuellAktivieren);
		}

		/*
		 * Symbolleisten spezifisches
		 */
		MenuManager mgrSymbolleiste = new MenuManager("Symbolleiste");
		manager.add(mgrSymbolleiste);

		manager.add(new Separator());

		if (_aktionSpeichernGlobal != null)
		{
			mgrSymbolleiste.add(_aktionSpeichernGlobal);
		}

		if (_aktionLadenGlobal != null)
		{
			mgrSymbolleiste.add(_aktionLadenGlobal);
		}

		if (_aktionSymbolleisteAktuellAktivieren != null)
		{
			mgrSymbolleiste.add(new Separator());
			mgrSymbolleiste.add(_aktionSymbolleisteAktuellAktivieren);
		}

		manager.add(new Separator());

		// Action ausschneiden
		Action actionAusschneiden = new Action()
		{
			public void run()
			{
				Object firstElement = ((IStructuredSelection) _treeViewer
						.getSelection()).getFirstElement();
				if (firstElement instanceof Element)
				{
					aktionMethodeAusschneiden();
				}

			}
		};
		actionAusschneiden.setText("Ausschneiden");
		actionAusschneiden.setToolTipText("Action 1 tooltip");
		actionAusschneiden.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_CUT));
		manager.add(actionAusschneiden);

		// Action einfgen
		Action actionEinfuegen = new Action()
		{
			public void run()
			{
				Object firstElement = ((IStructuredSelection) _treeViewer
						.getSelection()).getFirstElement();
				if (firstElement instanceof Element)
				{
					aktionMethodeEinfuegen();
				}
			}
		};
		if (_einfuegenGespeichertesElement != null)
		{
			actionEinfuegen.setText("Einfgen von "
					+ elementNachString(_einfuegenGespeichertesElement));
			actionEinfuegen.setToolTipText("Einfgen an einer bestimmten Stelle");
			actionEinfuegen.setImageDescriptor(PlatformUI.getWorkbench()
					.getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));

			manager.add(actionEinfuegen);
		}

		// Einfgen Submen
		Action actionSubMenuEinfuegen = new Action()
		{
			public void run()
			{
				Object firstElement = ((IStructuredSelection) _treeViewer
						.getSelection()).getFirstElement();
				if (firstElement instanceof Element)
				{
					aktionMethodeSubmenuEinfuegen();
				}
			}
		};
		actionSubMenuEinfuegen.setText("Neues Submen einfgen");
		actionSubMenuEinfuegen
				.setToolTipText("Einfgen eines neuen Submens an einer bestimmten Stelle");
		actionSubMenuEinfuegen
				.setImageDescriptor(getImageDescriptor("icons/menubar/menusubmenu.png"));
		manager.add(actionSubMenuEinfuegen);

		/*
		 * Submen umbenennen
		 */
		Element element = aktuellSelektiertesElement();
		if ((element != null)
				&& (element.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu)))
		{
			manager.add(_aktionSubmenuUmbenennen);
		}

		// Einfgen Separator
		Action actionSeparatoreinfuegen = new Action()
		{
			public void run()
			{
				aktionMethodeSeparatorEinfuegen();
			}
		};
		actionSeparatoreinfuegen.setText("Separator einfgen");
		actionSeparatoreinfuegen
				.setImageDescriptor(getImageDescriptor("icons/menubar/separator_16x16.png"));
		manager.add(actionSeparatoreinfuegen);

		/*
		 * Action Lschen. Gelscht werden darf alles, auer den Grundelementen von
		 * Men- und Symbolleisten.
		 */
		IStructuredSelection is = (IStructuredSelection) _treeViewer.getSelection();
		Object firstElement = is.getFirstElement();
		if (firstElement instanceof Element)
		{
			Element loeschElement = (Element) firstElement;
			if (MenuSymbolLeisteDaten.pruefeObElementLoeschenErlaubt(loeschElement))
			{
				Action actionLoeschen = new Action()
				{
					public void run()
					{
						Object firstElement = ((IStructuredSelection) _treeViewer
								.getSelection()).getFirstElement();
						if (firstElement instanceof Element)
						{
							aktionMethodeLoeschen();
						}
					}
				};
				actionLoeschen.setText("Lschen");
				actionLoeschen
						.setToolTipText("Lscht das Element und seine Kinder, falls vorhanden.");
				actionLoeschen.setImageDescriptor(PlatformUI.getWorkbench()
						.getSharedImages()
						.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
				manager.add(actionLoeschen);
			}
		}

		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/**
	 * Liefert einen {@link ImageDescriptor} aus der {@link ImageRegistry}. Ist
	 * dort der entsprechende {@link ImageDescriptor} noch nicht vorhanden wird
	 * aus dem bergebenen Pfad versucht, diesen aufzubauene.
	 * 
	 * @param imagePath
	 *          Pfad zum Image.
	 * @return {@link ImageDescriptor} oder <code>null</code>, wenn der Pfad nicht
	 *         existiert und die {@link ImageRegistry} den {@link ImageDescriptor}
	 *         nicht erzeugen konnte.
	 */
	private ImageDescriptor getImageDescriptor(String imagePath)
	{
		ImageDescriptor re = null;

		if ((imagePath != null) && (_ir != null) && (imagePath.length() > 0))
		{
			if (_ir.getDescriptor(imagePath) == null)
			{
				_ir.put(imagePath, Activator.getImageDescriptor(imagePath));
			}

			re = _ir.getDescriptor(imagePath);
		}

		return re;
	}

	protected void undo()
	{
		if (_msld != null)
		{
			MenuSymbolLeisteDaten pop = _msld.getUndoStack().pop(_msld);
			if (pop != null)
			{
				_msld = pop;
				// treeViewerAktuallisieren(); // TODO JH AUS
				treeViewerNeuLaden();
				_baumFallAdapterElementSichtMenuLeisteEditor
						.setMenuSymbolLeistedaten(_msld);
			}
		}
	}

	protected void redo()
	{
		if (_msld != null)
		{
			UndoStackMenuSymbol undoStack = _msld.getUndoStack();
			if (undoStack != null)
			{
				MenuSymbolLeisteDaten redo = undoStack.getRedo();
				if (redo != null)
				{
					undoStack.push(_msld);
					_msld = redo;
					// treeViewerAktuallisieren();//TODO JH AUS
					treeViewerNeuLaden();
					_baumFallAdapterElementSichtMenuLeisteEditor
							.setMenuSymbolLeistedaten(_msld);
				}

			}
		}
	}

	/**
	 * Erzeugt einen Listener vom Typ {@link IZuhoererUndoStackMenuSymbol} bei der
	 * Klasse {@link MenuSymbolLeisteDaten}, um ber deren aktuellen Zustand
	 * informiert zu sein.
	 */
	protected void erzeugeUndoStackListener()
	{
		if (_zuhoererUndoStack == null)
		{
			_zuhoererUndoStack = new IZuhoererUndoStackMenuSymbol()
			{

				@Override
				public void rueckgaengigmachenMoeglich(boolean moeglich)
				{
					if (_aktionHandlerUndo != null)
					{
						if (moeglich)
						{
							_aktionHandlerUndo.setEnabled(true);
						}
					} else
					{
						_aktionHandlerUndo.setEnabled(false);
					}
				}

				@Override
				public void wiederholenMoeglich(boolean moeglich)
				{
					if (_aktionHandlerRedo != null)
					{
						if (moeglich)
						{
							_aktionHandlerRedo.setEnabled(true);
						} else
						{
							_aktionHandlerRedo.setEnabled(false);
						}
					}
				}

			};
		}
	}

	/**
	 * Generiert die globalen Actions, die in verschiedenen Mens bzw.
	 * Symbolleisten bentigt werden.
	 * 
	 */
	private void generiereBasisActions()
	{
		/*
		 * Testweises Einfgen einer globalen Aktion zum Speichern
		 */
		IActionBars bars = getEditorSite().getActionBars();

		_aktionHandlerUndo = new Action()
		{
			public void run()
			{
				_debug.fine("=>UNDO");
				undo();
			}
		};
		bars.setGlobalActionHandler(ActionFactory.UNDO.getId(), _aktionHandlerUndo);

		_aktionHandlerRedo = new Action()
		{
			public void run()
			{
				_debug.fine("=>REDO");
				redo();
			}
		};
		bars.setGlobalActionHandler(ActionFactory.REDO.getId(), _aktionHandlerRedo);

		/*
		 * "COPY"-Action
		 */
		Action acDoit = new Action()
		{
			public void run()
			{
				aktionLadenGlobal();
				// aktionSpeichernGlobal();
			}
		};
		String id = ActionFactory.SAVE.getId();
		id = ActionFactory.COPY.getId();

		bars.setGlobalActionHandler(id, acDoit);

		/*
		 * Laden Global
		 */
		_aktionLadenGlobal = new Action()
		{
			public void run()
			{
				aktionLadenGlobal();
			}
		};
		_aktionLadenGlobal.setText("Laden (DaV/Lokale Einstellungen");
		_aktionLadenGlobal
				.setText("Ldt die Einstellungen aus dem DaV oder den lokalen Einstellungen");
		_aktionLadenGlobal.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Speichern Global
		 */
		_aktionSpeichernGlobal = new Action()
		{
			public void run()
			{
				aktionSpeichernGlobal();
			}
		};
		_aktionSpeichernGlobal.setText("Speichern (DaV/Lokale Einstellungen)");
		_aktionSpeichernGlobal
				.setToolTipText("Speichert die Einstellungen im DaV oder in den lokalen Einstellungen");
		_aktionSpeichernGlobal.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		// Action zum Laden aus einer bestimmten Datei
		_ladenAusBestimmterDatei = new Action()
		{
			public void run()
			{
				aktionMethodeNeuLadenAusBestimmterDatei();
			}
		};
		_ladenAusBestimmterDatei.setText("Laden aus Datei");
		// _ladenAusBestimmterDatei.setToolTipText("Aus einer auszuwhlenden Datei
		// die XML-Datenstruktur laden.");
		_ladenAusBestimmterDatei.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));

		// Speichern Unter generieren
		_aktionSpeichernUnter = new Action()
		{
			public void run()
			{
				aktionMethodeSpeichernInDatei();
			}
		};
		_aktionSpeichernUnter.setText("Speichern unter (XML-Datei)");
		_aktionSpeichernUnter
				.setToolTipText("Speichert die Einstellungen in einer XML-Datei");
		_aktionSpeichernUnter.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		// Speichern generieren
		_aktionSpeichern = new Action()
		{
			public void run()
			{
				aktionMethodeSpeichern();
			}
		};
		_aktionSpeichern.setText("Speichern");
		_aktionSpeichern
				.setToolTipText("Men in auszuwhlender XML-Datei speichern");
		_aktionSpeichern.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_16x16.png"));

		/*
		 * Aktion Speichern Unter DaV fr bestimmten Benutzer als Menleiste
		 */
		if (_aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV == null)
		{
			_aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Aktion Speichern Unter fr bestimmten Benutzer Dav als Menleiste");
					aktionMenuleisteSpeichernDaVFuerBestimmtenBenutzer();
				}
			};

			_aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV
					.setText("Speichern Unter bestimmten Benutzer DaV als Menleiste");
			_aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV
					.setToolTipText("Speichert die XML-Struktur der Menstruktur fr einen Benutzer im DaV.");
			_aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV
					.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));
		}

		/*
		 * Aktion Speichern Unter DaV fr den angemeldeten Benutzer als Menleiste
		 */
		if (_aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV == null)
		{
			_aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Aktion Speichern Unter fr angemeldeten Benutzer Dav als Menleiste");
					aktionMenuleisteSpeichernDaVFuerAngemeldetenBenutzer();
				}
			};

			_aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV
					.setText("Speichern Unter Angemeldetem Benutzer DaV als Menleiste");
			_aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV
					.setToolTipText("Speichert die XML-Struktur der Menstruktur fr einen Benutzer im DaV.");
			_aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV
					.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));
		}

		/*
		 * Aktion Speichern netzweite Einstellungen Menleiste im DaV
		 */
		if (_aktionMenuleisteSpeichernUnterNetzweitDaV == null)
		{
			_aktionMenuleisteSpeichernUnterNetzweitDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Aktion Speichern Unter Benutzer Dav als Symbolleiste");
					aktionMenuleisteSpeichernDaVAllgemeinNetzweit();
				}
			};
			_aktionMenuleisteSpeichernUnterNetzweitDaV
					.setText("Speichern Unter Netzweiten Einstellungen im DaV als Symbolleiste");
			_aktionMenuleisteSpeichernUnterNetzweitDaV
					.setToolTipText("Speichert die XML-Struktur der Menleiste fr alle Benutzer im DaV.");
			_aktionMenuleisteSpeichernUnterNetzweitDaV.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));
		}

		/*
		 * Einlesen der Menleiste fr einen bestimmten Benutzer
		 */
		if (_aktionMenuleisteEinlesenBestimmterBenutzerDaV == null)
		{
			_aktionMenuleisteEinlesenBestimmterBenutzerDaV = new Action()
			{
				public void run()
				{
					_debug
							.fine("Einlesen Xml-Menstruktur fr einen bestimmten Benutzer");

					aktionMenuleisteLadenDavFuerBestimmtenBenutzer();
				}
			};

			_aktionMenuleisteEinlesenBestimmterBenutzerDaV
					.setText("Einlesen der XML-Menstruktur Benutzer aus DaV");
			_aktionMenuleisteEinlesenBestimmterBenutzerDaV
					.setToolTipText("Liest fr einen bestimmten Benutzer die XML-Menstruktur aus dem DaV ein.");
			_aktionMenuleisteEinlesenBestimmterBenutzerDaV
					.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));
		}

		/*
		 * Einlesen fr den angemeldeten Benutzer
		 */
		if (_aktionMenuleisteEinlesenAngemeldeterBenutzerDav == null)
		{
			_aktionMenuleisteEinlesenAngemeldeterBenutzerDav = new Action()
			{
				public void run()
				{
					_debug
							.fine("Einlesen der Xml-Menstruktur fr den angemeldeten Benutzer");

					aktionMenuleisteLadenDavFuerAngemeldetenBenutzer();
				}
			};
			_aktionMenuleisteEinlesenAngemeldeterBenutzerDav
					.setText("Einlesen der XML-Menstruktur angemeldeter Benutzer aus DaV");
			_aktionMenuleisteEinlesenAngemeldeterBenutzerDav
					.setToolTipText("Liest fr den angemeldeten Benutzer die XML-Menstruktur aus dem DaV ein.");
			_aktionMenuleisteEinlesenAngemeldeterBenutzerDav
					.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));
		}

		/*
		 * Einlesen der netzweiten Einstellungen
		 */
		if (_aktionMenuleisteEinlesenNetzweitDaV == null)
		{
			_aktionMenuleisteEinlesenNetzweitDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Einlesen der Xml-Menstruktur aus den netzweiten Einstellungen");

					aktionMenuleisteLadenDaVAllgemeinNetzweit();
				}
			};
			_aktionMenuleisteEinlesenNetzweitDaV
					.setText("Einlesen der XML-Menstruktur aus den netzweiten Einstellungen.");
			_aktionMenuleisteEinlesenNetzweitDaV
					.setToolTipText("Liest aus den netzweiten Einstellungen die XML-Menstruktur aus dem DaV ein");
			_aktionMenuleisteEinlesenNetzweitDaV.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));
		}

		/*
		 * Aktivieren der Menstruktur
		 */
		if (_aktionMenuAktuellAktivieren == null)
		{
			_aktionMenuAktuellAktivieren = new Action()
			{
				public void run()
				{
					_debug.fine("Aktiviere aktuelle Menstruktur!");

					aktionMenuAktuellAktivieren();
				}
			};
			_aktionMenuAktuellAktivieren.setText("Aktiviere aktuelle Menstruktur");
			_aktionMenuAktuellAktivieren
					.setToolTipText("Aktiviert die aktuell dargestellte Menstruktur.");
			_aktionMenuAktuellAktivieren.setImageDescriptor(getImageDescriptor("icons/allgemein/refresh.gif"));
		}

		// //===================>>>>

		/*
		 * Aktion Speichern Unter DaV fr bestimmten Benutzer als Symbolleiste
		 */
		if (_aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV == null)
		{
			_aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Aktion Speichern Unter fr bestimmten Benutzer Dav als Symbolleiste");
					aktionSymbolleisteSpeichernDaVFuerBestimmtenBenutzer();
				}
			};

			_aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV
					.setText("Speichern Unter bestimmten Benutzer DaV als Symbolleiste");
			_aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV
					.setToolTipText("Speichert die XML-Struktur der Symbolleistenstruktur fr einen Benutzer im DaV.");
			_aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV
					.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));
		}

		/*
		 * Aktion Speichern Unter DaV fr den angemeldeten Benutzer als Symbolleiste
		 */
		if (_aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV == null)
		{
			_aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Aktion Speichern Unter fr angemeldeten Benutzer Dav als Symbolleiste");
					aktionSymbolleisteSpeichernDaVFuerAngemeldetenBenutzer();
				}
			};

			_aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV
					.setText("Speichern Unter Angemeldetem Benutzer DaV als Symbolleiste");
			_aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV
					.setToolTipText("Speichert die XML-Struktur der Symbolleistenstruktur fr einen Benutzer im DaV.");
			_aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV
					.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));
		}

		/*
		 * Aktion Speichern netzweite Einstellungen Symbolleiste im DaV
		 */
		if (_aktionSymbolleisteSpeichernUnterNetzweitDaV == null)
		{
			_aktionSymbolleisteSpeichernUnterNetzweitDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Aktion Speichern Unter Benutzer Dav als Symbolleiste");
					aktionSymbolleisteSpeichernDaVAllgemeinNetzweit();
				}
			};
			_aktionSymbolleisteSpeichernUnterNetzweitDaV
					.setText("Speichern Unter Netzweiten Einstellungen im DaV als Symbolleiste");
			_aktionSymbolleisteSpeichernUnterNetzweitDaV
					.setToolTipText("Speichert die XML-Struktur der Symbolleiste fr alle Benutzer im DaV.");
			_aktionSymbolleisteSpeichernUnterNetzweitDaV.setImageDescriptor(getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));
		}

		/*
		 * Einlesen der Symbolleiste fr einen bestimmten Benutzer
		 */
		if (_aktionSymbolleisteEinlesenBestimmterBenutzerDaV == null)
		{
			_aktionSymbolleisteEinlesenBestimmterBenutzerDaV = new Action()
			{
				public void run()
				{
					_debug
							.fine("Einlesen Xml-Symbolleistenstruktur fr einen bestimmten Benutzer");

					aktionSymbolleisteLadenDavFuerBestimmtenBenutzer();
				}
			};

			_aktionSymbolleisteEinlesenBestimmterBenutzerDaV
					.setText("Einlesen der XML-Symbolleistenstruktur Benutzer aus DaV");
			_aktionSymbolleisteEinlesenBestimmterBenutzerDaV
					.setToolTipText("Liest fr einen bestimmten Benutzer die XML-Symbolleistenstruktur aus dem DaV ein.");
			_aktionSymbolleisteEinlesenBestimmterBenutzerDaV
					.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));
		}

		/*
		 * Einlesen Xml-Symbolleistenstruktur fr den angemeldeten Benutzer
		 */
		if (_aktionSymbolleisteEinlesenAngemeldeterBenutzerDav == null)
		{
			_aktionSymbolleisteEinlesenAngemeldeterBenutzerDav = new Action()
			{
				public void run()
				{
					System.out
							.println("Einlesen der Xml-Symbolleistenstruktur fr den angemeldeten Benutzer");

					aktionSymbolleisteLadenDavFuerAngemeldetenBenutzer();
				}
			};
			_aktionSymbolleisteEinlesenAngemeldeterBenutzerDav
					.setText("Einlesen der XML-Symbolleistenstruktur angemeldeter Benutzer aus DaV");
			_aktionSymbolleisteEinlesenAngemeldeterBenutzerDav
					.setToolTipText("Liest fr den angemeldeten Benutzer die XML-Symbolleistenstruktur aus dem DaV ein.");
			_aktionSymbolleisteEinlesenAngemeldeterBenutzerDav
					.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));
		}

		/*
		 * Einlesen der netzweiten Einstellungen der XML-Symbolleistenstruktur
		 */
		if (_aktionSymbolleisteEinlesenNetzweitDaV == null)
		{
			_aktionSymbolleisteEinlesenNetzweitDaV = new Action()
			{
				public void run()
				{
					System.out
							.println("Einlesen der XML-Symbolleistenstruktur aus den netzweiten Einstellungen");

					aktionSymbolleisteLadenDaVAllgemeinNetzweit();
				}
			};
			_aktionSymbolleisteEinlesenNetzweitDaV
					.setText("Einlesen der XML-Symbolleistenstruktur aus den netzweiten Einstellungen.");
			_aktionSymbolleisteEinlesenNetzweitDaV
					.setToolTipText("Liest aus den netzweiten Einstellungen die XML-Symbolleistenstruktur aus dem DaV ein");
			_aktionSymbolleisteEinlesenNetzweitDaV.setImageDescriptor(getImageDescriptor("icons/menubar/laden_16x16.png"));
		}

		// //<<=====================

		/*
		 * Aktivieren der Symbolleistenstruktur
		 */
		if (_aktionSymbolleisteAktuellAktivieren == null)
		{
			_aktionSymbolleisteAktuellAktivieren = new Action()
			{
				public void run()
				{
					_debug.fine("Aktiviere aktuelle Symbolleistenstruktur!");

					aktionSymbolleisteAktuellAktivieren();
				}
			};
			_aktionSymbolleisteAktuellAktivieren
					.setText("Aktiviere aktuelle Symbolleistenstruktur");
			_aktionSymbolleisteAktuellAktivieren
					.setToolTipText("Aktiviert die aktuell dargestellte Symbolleistenstruktur.");
			_aktionSymbolleisteAktuellAktivieren.setImageDescriptor(getImageDescriptor("icons/allgemein/refresh.gif"));
		}

		/*
		 * Umbennenen des aktuell selektierten Submens
		 */
		if (_aktionSubmenuUmbenennen == null)
		{
			_aktionSubmenuUmbenennen = new Action()
			{
				public void run()
				{
					aktionSubmenuUmbenennen();
				}
			};
		}
		_aktionSubmenuUmbenennen.setText("Submen umbenennen");
		_aktionSubmenuUmbenennen
				.setToolTipText("Benennt das aktuell selektierte Submen um.");
	}

	/*
	 * Aktionen hinter globalen Aktionen
	 */

	/**
	 * Diese Methode fgt die Funktionen hinter der globalen Action Speichern aus.
	 */
	protected void aktionSpeichernGlobal()
	{
		_debug.fine("Speichern");
		LadeSpeicherDialogMenu dialog = new LadeSpeicherDialogMenu(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());
		SpeicherInfo speicherInfo = dialog.oeffne(
				EnumDialogSpeichernLaden.speichern, ladenLeistenArt);
		if (speicherInfo != null)
		{
			_debug.fine("Speicherort:=> " + speicherInfo.toString());

			EnumLeistenArt leistenArt = speicherInfo.getLeistenArt();
			EnumSpeicherOrt speicherOrt = speicherInfo.getSpeicherOrt();
			EnumSubjekt subjekt = speicherInfo.getSubjekt();

			switch (leistenArt)
			{
			case menuLeiste:
				switch (speicherOrt)
				{
				case benutzerNetzweit:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionMenuleisteSpeichernDaVFuerAngemeldetenBenutzer();
						break;
					case benutzerAuswahl:
						aktionMenuleisteSpeichernDaVFuerBestimmtenBenutzer();
						break;
					}
					break;
				case benutzerLokal:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionMenuLeisteSpeichernLokalFuerAngemeldetenBenzutzer();
						break;
					case benutzerAuswahl:
						aktionMenuleisteSpeichernLokalFuerBestimmtenBenutzer();
						break;
					}
					break;
				case allgemeinNetzweit:
					aktionMenuleisteSpeichernDaVAllgemeinNetzweit();
					break;
				case allgemeinLokal:
					aktionMenuLeisteSpeichernLokalAllgemein();
					break;
				}
				break;
			case symbolLeiste:
				switch (speicherOrt)
				{
				case benutzerNetzweit:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionSymbolleisteSpeichernDaVFuerAngemeldetenBenutzer();
						break;
					case benutzerAuswahl:
						aktionSymbolleisteSpeichernDaVFuerBestimmtenBenutzer();
						break;
					}
					break;
				case benutzerLokal:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionSymbolleisteSpeichernLokalFuerAngemeldetenBenutzer();
						break;
					case benutzerAuswahl:
						aktionSymbolleisteSpeichernLokalFuerBestimmtenBenutzer();
						break;
					}
					break;
				case allgemeinNetzweit:
					aktionSymbolleisteSpeichernDaVAllgemeinNetzweit();
					break;
				case allgemeinLokal:
					aktionSymbolleisteSpeichernLokalAllgemein();
					break;
				}
				break;
			}

		}
	}

	/**
	 * Setzt die Art der Leiste und initialisiert damit den Dialog
	 * {@link LadeSpeicherDialogMenu}
	 * 
	 * @param art
	 *          Art der Leiste
	 */
	public void setLadenSpeichernLeistenArt(
			de.bsvrz.buv.rw.basislib.menu.editoren.dialoge.EnumLeistenArt art)
	{
		this.ladenLeistenArt = art;
	}

	/**
	 * Diese Methode fgt die Funktion hinter der globalen Action Laden aus.
	 */
	public void aktionLadenGlobal()
	{
		MenuSymbolLeisteDaten tempMSLD = this._msld;

		LadeSpeicherDialogMenu dialog = new LadeSpeicherDialogMenu(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());
		SpeicherInfo speicherInfo = dialog.oeffne(EnumDialogSpeichernLaden.laden,
				ladenLeistenArt);

		if (speicherInfo != null)
		{
			_debug.fine("Speicherort:=> " + speicherInfo.toString());

			EnumLeistenArt leistenArt = speicherInfo.getLeistenArt();
			EnumSpeicherOrt speicherOrt = speicherInfo.getSpeicherOrt();
			EnumSubjekt subjekt = speicherInfo.getSubjekt();

			switch (leistenArt)
			{
			case menuLeiste:
				switch (speicherOrt)
				{
				case benutzerNetzweit:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionMenuleisteLadenDavFuerAngemeldetenBenutzer();
						break;
					case benutzerAuswahl:
						aktionMenuleisteLadenDavFuerBestimmtenBenutzer();
						break;
					}
					break;
				case benutzerLokal:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionMenuLeisteLadenLokalFuerAngemeldetenBenutzer();
						break;
					case benutzerAuswahl:
						aktionMenuleisteLadenLokalFuerBestimmtenBenutzer();
						break;
					}
					break;
				case allgemeinNetzweit:
					aktionMenuleisteLadenDaVAllgemeinNetzweit();
					break;
				case allgemeinLokal:
					aktionMenuLeisteLadenLokalAllgemein();
					break;
				}
				break;
			case symbolLeiste:
				switch (speicherOrt)
				{
				case benutzerNetzweit:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionSymbolleisteLadenDavFuerAngemeldetenBenutzer();
						break;
					case benutzerAuswahl:
						aktionSymbolleisteLadenDavFuerBestimmtenBenutzer();
						break;
					}
					break;
				case benutzerLokal:
					switch (subjekt)
					{
					case benutzerAngemeldet:
						aktionSymbolleisteLadenLokalFuerAngemeldetenBenutzer();
						break;
					case benutzerAuswahl:
						aktionSymbolleisteLadenLokalFuerBestimmtenBenutzer();
						break;
					}
					break;
				case allgemeinNetzweit:
					aktionSymbolleisteLadenDaVAllgemeinNetzweit();
					break;
				case allgemeinLokal:
					aktionSymbolleisteLadenLokalAllgemein();
					break;
				}
				break;
			}

			if (!tempMSLD.equals(_msld))
			{
				tempMSLD.setFeuereSchmutzigGeaendert(null);
				_msld.setFeuereSchmutzigGeaendert(this);
				if (_baumFallAdapterElementSichtMenuLeisteEditor != null)
				{
					_baumFallAdapterElementSichtMenuLeisteEditor
							.setMenuSymbolLeistedaten(_msld);
				}

				editorTitelAktualisieren();
			}

		} else
		{
			// Speicherinfo == null
		}
	}

	/**
	 * Erzeugt eine neue Instanz der Klasse MSLD und referenziert diese in der
	 * Variablen _msld. Dabei wird der Listener vom Typ
	 * {@link IZuhoererUndoStackMenuSymbol} von der alten referenzierten
	 * {@link MenuSymbolLeisteDaten}-Instanz deregistriert und in der neuen
	 * angemeldet.
	 * 
	 * @param xmlStruktur
	 *          XML-Struktur, aus der eine neue {@link MenuSymbolLeisteDaten}
	 *          -Instanz generiert werden soll.
	 * @param leistenArt
	 *          Art der Leiste
	 */
	protected void generiereMSLD(String xmlStruktur, enumLeistenArt leistenArt)
	{
		if (_zuhoererUndoStack != null)
		{
			_msld.getUndoStack().removeZuhoererUndoStack(_zuhoererUndoStack);
		}

		_msld = new MenuSymbolLeisteDaten();
		_msld.einlesenXmlAusString(xmlStruktur);
		_msld.setzeLeistenArt(leistenArt);

		if (_zuhoererUndoStack == null)
		{
			erzeugeUndoStackListener();
		}

		if (_zuhoererUndoStack != null)
		{
			_msld.getUndoStack().addZuhoererUndoStack(_zuhoererUndoStack);
		}

	}

	/*
	 * Menleistenstruktur netzweit via DaV
	 */
	/**
	 * Ldt die Menleistenstruktur fr einen bestimmten Benutzer. Es erscheint
	 * ein Dialog zur Benutzerauswahl.
	 */
	private void aktionMenuleisteLadenDavFuerBestimmtenBenutzer()
	{
		DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());
		String benutzer = bw.oeffne();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenzutzerBestimmtNetzweit(benutzer);
			if (einstellungenSpeicher != null)
			{
				String parameter = null;

				parameter = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

				if ((parameter != null) && (parameter.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();
					// _msld.einlesenXmlAusString(parameter);
					// _msld.setzeLeistenArt(enumLeistenArt.menue);

					generiereMSLD(parameter, enumLeistenArt.menue);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} else
				{
					MessageBox msb = new MessageBox(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					msb.setText("Achtung!");
					msb
							.setMessage("Es existiert kein Parameter xmlStrukturMenuleiste im\n"
									+ "EinstellungenSpeicher fr den Benutzer " + benutzer + "!");

					msb.open();
				}
			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Achtung!");
				msb
						.setMessage("Es existiert kein EinstellungenSpeicher fr\nden Benutzer "
								+ benutzer + "!");
				msb.open();
			}
		}
	}

	/**
	 * Speichert die Symbolleiste fr einen auszuwhlenden Benutzer
	 */
	private void aktionMenuleisteSpeichernDaVFuerBestimmtenBenutzer()
	{

		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());
		String benutzer = bw.oeffne();
		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			// esw.setSendenOhneUrlasserDialog(true);
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenzutzerBestimmtNetzweit(benutzer);
			if (einstellungenSpeicher != null)
			{

				boolean gespeichert = false;

				if (einstellungenSpeicher.editiereParameter(
						DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur))
				{
					_debug.fine("EDITIERT");
				} else
				{
					einstellungenSpeicher
							.hinzufuegenParameter(
									DefinitionenEinstellungen.paramXmlStrukturMenuleiste,
									xmlStruktur);
					_debug.fine("HINZUGEFGT");
				}

				gespeichert = esw.sichereEinstellungenSpeicherBenutzerBestimmtNetzweit(
						benutzer, einstellungenSpeicher);

				if (gespeichert)
				{
					_debug.fine("GESICHERT");
				} else
				{
					// NICHT GESICHERT
				}
			} else
			{
				// ESP nicht da!
			}

		}
	}

	/**
	 * Ldt die Struktur der Menleiste aus den allgemeinen, netzweiten
	 * Einstellungen.
	 */
	private void aktionMenuleisteLadenDaVAllgemeinNetzweit()
	{
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungen != null)
		{
			String xmlStruktur = null;

			xmlStruktur = einstellungen
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

			if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
			{
				_msld = new MenuSymbolLeisteDaten();

				// _msld.einlesenXmlAusString(xmlStruktur);
				// _msld.setzeLeistenArt(enumLeistenArt.menue);

				generiereMSLD(xmlStruktur, enumLeistenArt.menue);

				_treeViewer.setInput(_msld.getWurzel());
				_treeViewer.expandAll();
			}
		}
	}

	/**
	 * Speichert die Struktur der Menleiste in den allgemeinen, netzweiten
	 * Einstellungen.
	 */
	private void aktionMenuleisteSpeichernDaVAllgemeinNetzweit()
	{

		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		// esw.setSendenOhneUrlasserDialog(true);

		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungen.editiereParameter(
				DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur))
		{
		} else
		{
			einstellungen.hinzufuegenParameter(
					DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur);
		}

		esw.sichereSpeicherAllgemeinNetzweit();
	}

	/**
	 * Ldt die Struktur der Menleiste aus den netzweiten Einstellungen fr
	 * diesen Benutzer
	 */
	private void aktionMenuleisteLadenDavFuerAngemeldetenBenutzer()
	{
		String name = BenutzerAngemeldet.getInstanz().getName();
		if (name != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungen(EinstellungsArt.benutzerNetzweit);
			if (einstellungenSpeicher != null)
			{
				String xmlStruktur = null;

				xmlStruktur = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();

					// _msld.einlesenXmlAusString(xmlStruktur);
					// _msld.setzeLeistenArt(enumLeistenArt.menue);

					generiereMSLD(xmlStruktur, enumLeistenArt.menue);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				}

			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Kein EinstellungenSpeicher vorhanden!");
				msb.setMessage("Fr den angemeldeten Benutzer " + name
						+ " existiert kein EinstellungenSpeicher!");
				msb.open();
			}
		} else
		{
			MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
			msb.setText("Kein Benutzer angemeldet!");
			msb.setMessage("Es ist zur Zeit kein Benutzer angemeldet!");
			msb.open();
		}

	}

	/**
	 * Speichert die Struktur der Menleiste in den netzweiten,
	 * benuterspezifischen Einstellungen.
	 */
	private void aktionMenuleisteSpeichernDaVFuerAngemeldetenBenutzer()
	{
		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungenSpeicher = esw
				.getEinstellungen(EinstellungsArt.benutzerNetzweit);
		if (einstellungenSpeicher != null)
		{
			if (einstellungenSpeicher.editiereParameter(
					DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur))
			{
				_debug.fine("EDITIERT");
			} else
			{
				einstellungenSpeicher.hinzufuegenParameter(
						DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur);
				_debug.fine("Hinzugefgt");
			}

			esw.sichereSpeicher();
			_debug.fine("SICHERE");

		} else
		{
			// ESP nicht da!
		}
	}

	/**
	 * Aktiviert die aktuell im Editor dargestellte Menstruktur.
	 */
	private void aktionMenuAktuellAktivieren()
	{
		if (_msld != null)
		{
			MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();
			msld.einlesenXmlAusString(_msld.ausgabeXmlFormatiert());

			IMenuManager menuManager = ElementRegistrierung.getInstanz()
					.getMenuManager();
			if (menuManager == null)
			{
				System.out.println("MenuManager == NULL");
			}

			MenuGenerator mg = new MenuGenerator(menuManager, msld);
			mg.generiereMenu();
		}
	}

	/*
	 * Menleistenstruktur lokal.
	 */
	/**
	 * Ldt die lokale Menleistenstruktur fr einen bestimmten Benutzer. Es
	 * erscheint ein Dialog zur Benutzerauswahl.
	 */
	private void aktionMenuleisteLadenLokalFuerBestimmtenBenutzer()
	{
		String benutzer = null;

		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			benutzer = bw.oeffne();
		} else
		{
			InputDialog inputDialog = new InputDialog(PlatformUI.getWorkbench()
					.getDisplay().getActiveShell(), "Benutzernamen eingeben",
					"Es existiert keine Verbindung zum DaV, bitte geben Sie\n"
							+ "den Namen des Benutzers manuell ein:",
					"Hier den Benutzernamen eingeben", null);
			if (inputDialog.open() == Dialog.OK)
			{
				if (inputDialog.getValue() != null)
				{
					benutzer = inputDialog.getValue();
				}
			}
		}

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				String parameter = null;

				parameter = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

				if ((parameter != null) && (parameter.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();

					// _msld.einlesenXmlAusString(parameter);
					// _msld.setzeLeistenArt(enumLeistenArt.menue);

					generiereMSLD(parameter, enumLeistenArt.menue);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} else
				{
					MessageBox msb = new MessageBox(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					msb.setText("Achtung!");
					msb
							.setMessage("Es existiert kein Parameter xmlStrukturMenuleiste im\n"
									+ "EinstellungenSpeicher fr den Benutzer " + benutzer + "!");

					msb.open();
				}
			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Achtung!");
				msb
						.setMessage("Es existiert kein EinstellungenSpeicher fr\nden Benutzer "
								+ benutzer + "!");
				msb.open();
			}
		}
	}

	/**
	 * Speichert die aktuelle Menleiste <i>lokal</i> fr einen bestimmten
	 * Benutzer.
	 * 
	 * @return Liefert <code>true</code>, wenn die Menleiste erfolgreich
	 *         gespeichert werden konnte, sonst <code>false</code>.
	 */
	private boolean aktionMenuleisteSpeichernLokalFuerBestimmtenBenutzer()
	{
		boolean erfolgreich = false;
		String xmlStruktur = _msld.ausgabeXmlFormatiert();

		String benutzer = null;
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			benutzer = bw.oeffne();
		} else
		{
			// String benutzer = "BLuBb";
			InputDialog inputDialog = new InputDialog(
					PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					"Benutzernamen eingeben",
					"Es existiert keine Verbindung zum DaV, bitte geben Sie den Namen des Benutzers manuell ein:",
					"Hier den Benutzernamen eingeben", null);
			if (inputDialog.open() == Dialog.OK)
			{
				if (inputDialog.getValue() != null)
				{
					benutzer = inputDialog.getValue();
				}
			}
		}

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				if (einstellungenSpeicher.editiereParameter(
						DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur))
				{
					_debug.fine("EDITIERT");
				} else
				{
					einstellungenSpeicher
							.hinzufuegenParameter(
									DefinitionenEinstellungen.paramXmlStrukturMenuleiste,
									xmlStruktur);
					_debug.fine("HINZUGEFGT");
				}

				erfolgreich = esw.sichereEinstellungenSpeicherBenutzerBestimmtLokal(
						benutzer, einstellungenSpeicher);
			}
		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR | SWT.APPLICATION_MODAL);
			mb.setText("Kein Benutzer angegeben");
			mb
					.setMessage("Sie haben keinen Benutzer angegeben, die Struktur der Menleiste kann nicht gespeichert werden");
			mb.open();
		}

		return erfolgreich;
	}

	/**
	 * Ldt fr den aktuell angemeldeten Benutzer die Symbolleistenstruktur aus
	 * den lokalen, benutzerspezifischen Einstellungen.
	 */
	private void aktionMenuLeisteLadenLokalFuerAngemeldetenBenutzer()
	{
		String benutzer = BenutzerAngemeldet.getInstanz().getName();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				String parameter = null;

				parameter = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

				if ((parameter != null) && (parameter.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();

					// _msld.einlesenXmlAusString(parameter);
					// _msld.setzeLeistenArt(enumLeistenArt.menue);

					generiereMSLD(parameter, enumLeistenArt.menue);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} else
				{
					MessageBox msb = new MessageBox(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					msb.setText("Achtung!");
					msb
							.setMessage("Es existiert kein Parameter xmlStrukturMenuleiste im\n"
									+ "EinstellungenSpeicher fr den Benutzer " + benutzer + "!");

					msb.open();
				}
			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Achtung!");
				msb
						.setMessage("Es existiert kein EinstellungenSpeicher fr\nden Benutzer "
								+ benutzer + "!");
				msb.open();
			}
		} else
		{
			MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
			msb.setText("Kein Benutzer angemeldet");
			msb
					.setMessage("Es ist kein Benutzer angemeldet.\n"
							+ "Die lokalen Einstellungen fr die Menleiste knnen nicht geladen werden!");
			msb.open();
		}
	}

	/**
	 * Speichert fr den aktuell angemeldeten Benutzer die Struktur der
	 * Symbolleiste in den lokalen, benutzerspezifischen Einstellungen.
	 * 
	 * @return Liefert <code>true</code>, wenn erfolgreich gespeichert werden
	 *         konnte, sonst <code>false</code>.
	 */
	private boolean aktionMenuLeisteSpeichernLokalFuerAngemeldetenBenzutzer()
	{
		boolean erfolgreich = false;
		String xmlStruktur = _msld.ausgabeXmlFormatiert();

		String benutzer = BenutzerAngemeldet.getInstanz().getName();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				if (einstellungenSpeicher.editiereParameter(
						DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur))
				{
					_debug.fine("EDITIERT");
				} else
				{
					einstellungenSpeicher
							.hinzufuegenParameter(
									DefinitionenEinstellungen.paramXmlStrukturMenuleiste,
									xmlStruktur);
					_debug.fine("HINZUGEFGT");
				}

				erfolgreich = esw.sichereEinstellungenSpeicherBenutzerBestimmtLokal(
						benutzer, einstellungenSpeicher);
			}
		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR | SWT.APPLICATION_MODAL);
			mb.setText("Kein Benutzer angemeldet");
			mb
					.setMessage("Es ist kein Benutzer angemeldet. Die Struktur der Menleiste\n"
							+ "kann nicht in den lokalen Einstellungen fr einen Benutzer\n"
							+ "gespeichert werden.");
			mb.open();
		}

		return erfolgreich;
	}

	/**
	 * Ldt die Struktur der Menleiste aus den lokalen, allgemeinen Einstellungen
	 */
	private void aktionMenuLeisteLadenLokalAllgemein()
	{
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinLokal);

		if (einstellungen != null)
		{
			String xmlStruktur = null;

			xmlStruktur = einstellungen
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturMenuleiste);

			if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
			{
				_msld = new MenuSymbolLeisteDaten();

				// _msld.einlesenXmlAusString(xmlStruktur);
				// _msld.setzeLeistenArt(enumLeistenArt.menue);

				generiereMSLD(xmlStruktur, enumLeistenArt.menue);

				_treeViewer.setInput(_msld.getWurzel());
				_treeViewer.expandAll();

			}
		}
	}

	/**
	 * Speichert die Struktur der Menleiste in den lokalen, allgemeinen
	 * Einstellungen.
	 * 
	 * @return Liefert <code>true</code>, wenn die Menleistenstruktur erfolgreich
	 *         gesichert werden konnte, sonst <code>false</code>.
	 */
	private boolean aktionMenuLeisteSpeichernLokalAllgemein()
	{
		boolean erfolgreich = false;

		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinLokal);

		if (einstellungen.editiereParameter(
				DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur))
		{
		} else
		{
			einstellungen.hinzufuegenParameter(
					DefinitionenEinstellungen.paramXmlStrukturMenuleiste, xmlStruktur);
		}

		erfolgreich = esw.sichereSpeicherAllgemeinLokal();

		return erfolgreich;
	}

	/*
	 * Symbolleistenstruktur Global via DaV
	 */

	/**
	 * Ldt die Symbolleistenstruktur fr einen bestimmten Benutzer aus den
	 * netzweiten, benutzerspezifischen Einstellungen. Es erscheint ein Dialog zur
	 * Benutzerauswahl.
	 */
	private void aktionSymbolleisteLadenDavFuerBestimmtenBenutzer()
	{
		DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());
		String benutzer = bw.oeffne();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenzutzerBestimmtNetzweit(benutzer);
			if (einstellungenSpeicher != null)
			{
				String parameter = null;

				parameter = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

				if ((parameter != null) && (parameter.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();

					// _msld.einlesenXmlAusString(parameter);
					// _msld.setzeLeistenArt(enumLeistenArt.symbol);

					generiereMSLD(parameter, enumLeistenArt.symbol);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} else
				{
					MessageBox msb = new MessageBox(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					msb.setText("Achtung!");
					msb
							.setMessage("Es existiert kein Parameter xmlStrukturSymbolleiste im\n"
									+ "EinstellungenSpeicher fr den Benutzer " + benutzer + "!");

					msb.open();
				}
			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Achtung!");
				msb
						.setMessage("Es existiert kein EinstellungenSpeicher fr\nden Benutzer "
								+ benutzer + "!");
				msb.open();
			}
		}
	}

	/**
	 * Speichert die Symbolleistenstruktur fr einen auszuwhlenden Benutzer in
	 * den netzweiten, benutzerspezifischen Einstellungen. Es erscheint ein
	 * Benutzerauswahldialog.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Struktur fr einen
	 *         Benutzer erfolgreich gespeichert werden konnte, sonst
	 *         <code>false</code>.
	 */
	private boolean aktionSymbolleisteSpeichernDaVFuerBestimmtenBenutzer()
	{
		boolean erfolgreich = false;

		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());
		String benutzer = bw.oeffne();
		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenzutzerBestimmtNetzweit(benutzer);
			if (einstellungenSpeicher != null)
			{

				if (einstellungenSpeicher
						.editiereParameter(
								DefinitionenEinstellungen.paramXmlStrukturSymbolleiste,
								xmlStruktur))
				{
				} else
				{
					einstellungenSpeicher.hinzufuegenParameter(
							DefinitionenEinstellungen.paramXmlStrukturSymbolleiste,
							xmlStruktur);
				}

				erfolgreich = esw.sichereEinstellungenSpeicherBenutzerBestimmtNetzweit(
						benutzer, einstellungenSpeicher);

				if (erfolgreich)
				{
					_debug.fine("GESICHERT");
				} else
				{
					// NICHT GESICHERT
				}
			} else
			{
				// ESP nicht da!
			}

		}

		return erfolgreich;
	}

	/**
	 * Ldt die Struktur der Symbolleiste aus den netzweiten, allgemeinen
	 * Einstellungen.
	 */
	private void aktionSymbolleisteLadenDaVAllgemeinNetzweit()
	{
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungen != null)
		{
			String xmlStruktur = null;

			xmlStruktur = einstellungen
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

			if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
			{
				_msld = new MenuSymbolLeisteDaten();

				// _msld.einlesenXmlAusString(xmlStruktur);
				// _msld.setzeLeistenArt(enumLeistenArt.symbol);

				generiereMSLD(xmlStruktur, enumLeistenArt.symbol);

				_treeViewer.setInput(_msld.getWurzel());
				_treeViewer.expandAll();
			}
		}
	}

	/**
	 * Speichert die Symbolleistenstruktur in den netzweiten, allgemeinen
	 * Einstellungen
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Symbolleistenstruktur
	 *         erfolgreich in den netzweiten, allgemeinen Einstellungen
	 *         gespeichert werden konnte, sonst <code>false</code>.
	 */
	private boolean aktionSymbolleisteSpeichernDaVAllgemeinNetzweit()
	{
		boolean erfolgreich = false;

		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungen.editiereParameter(
				DefinitionenEinstellungen.paramXmlStrukturSymbolleiste, xmlStruktur))
		{
		} else
		{
			einstellungen.hinzufuegenParameter(
					DefinitionenEinstellungen.paramXmlStrukturSymbolleiste, xmlStruktur);
		}

		erfolgreich = esw.sichereSpeicherAllgemeinNetzweit();

		return erfolgreich;
	}

	/**
	 * Ldt die Symbolleistenstruktur fr den aktuell angemeldeten Benutzer.
	 */
	private void aktionSymbolleisteLadenDavFuerAngemeldetenBenutzer()
	{
		String name = BenutzerAngemeldet.getInstanz().getName();
		if (name != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungen(EinstellungsArt.benutzerNetzweit);
			if (einstellungenSpeicher != null)
			{
				String xmlStruktur = null;

				xmlStruktur = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

				if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();

					// _msld.einlesenXmlAusString(xmlStruktur);
					// _msld.setzeLeistenArt(enumLeistenArt.symbol);

					generiereMSLD(xmlStruktur, enumLeistenArt.symbol);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				}

			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Kein EinstellungenSpeicher vorhanden!");
				msb.setMessage("Fr den angemeldeten Benutzer " + name
						+ " existiert kein EinstellungenSpeicher!");
				msb.open();
			}
		} else
		{
			MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
			msb.setText("Kein Benutzer angemeldet!");
			msb.setMessage("Es ist zur Zeit kein Benutzer angemeldet!");
			msb.open();
		}

	}

	/**
	 * Speichert die Symbolleistenstruktur fr den aktuell angemeldeten Benutzer.
	 */
	private void aktionSymbolleisteSpeichernDaVFuerAngemeldetenBenutzer()
	{
		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungenSpeicher = esw
				.getEinstellungen(EinstellungsArt.benutzerNetzweit);
		if (einstellungenSpeicher != null)
		{
			if (einstellungenSpeicher.editiereParameter(
					DefinitionenEinstellungen.paramXmlStrukturSymbolleiste, xmlStruktur))
			{
			} else
			{
				einstellungenSpeicher
						.hinzufuegenParameter(
								DefinitionenEinstellungen.paramXmlStrukturSymbolleiste,
								xmlStruktur);
			}

			esw.sichereSpeicher();
		} else
		{

		}
	}

	/**
	 * Aktiviert die aktuell im Editor dargestellte Symbolleistenstruktur.
	 */
	private void aktionSymbolleisteAktuellAktivieren()
	{
		if (_msld != null)
		{
			MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();
			msld.einlesenXmlAusString(_msld.ausgabeXmlFormatiert());

			ICoolBarManager coolBarManager = ElementRegistrierung.getInstanz()
					.getCoolBarManager();
			SymbolleisteGenerator symbolleisteGenerator = new SymbolleisteGenerator(
					coolBarManager, msld);
			symbolleisteGenerator.generiereSymbolLeiste();
		}
	}

	/*
	 * Symbolleistenstruktur im lokalen EinstellungenSpeicher
	 */

	/**
	 * Ldt die Struktur der Symbolleiste fr einen bestimmten Benutzer aus den
	 * lokalen Einstellungen. Der Benutzer wird vorher via Dialog abgefragt.
	 */
	private void aktionSymbolleisteLadenLokalFuerBestimmtenBenutzer()
	{
		String benutzer = null;

		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			benutzer = bw.oeffne();
		} else
		{
			InputDialog inputDialog = new InputDialog(PlatformUI.getWorkbench()
					.getDisplay().getActiveShell(), "Benutzernamen eingeben",
					"Es existiert keine Verbindung zum DaV, bitte\n"
							+ "geben Sie den Namen des Benutzers manuell ein:",
					"Hier den Benutzernamen eingeben", null);
			if (inputDialog.open() == Dialog.OK)
			{
				if (inputDialog.getValue() != null)
				{
					benutzer = inputDialog.getValue();
				}
			}
		}

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				String parameter = null;

				parameter = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

				if ((parameter != null) && (parameter.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();

					// _msld.einlesenXmlAusString(parameter);
					// _msld.setzeLeistenArt(enumLeistenArt.symbol);

					generiereMSLD(parameter, enumLeistenArt.symbol);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} else
				{
					MessageBox msb = new MessageBox(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					msb.setText("Achtung!");
					msb
							.setMessage("Es existiert kein Parameter xmlStrukturSymbolleiste im\n"
									+ "EinstellungenSpeicher fr den Benutzer " + benutzer + "!");

					msb.open();
				}
			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Achtung!");
				msb
						.setMessage("Es existiert kein EinstellungenSpeicher fr\nden Benutzer "
								+ benutzer + "!");
				msb.open();
			}
		}
	}

	/**
	 * Speichert die Struktur der Symbolleiste in den lokalen Einstellungen fr
	 * einen bestimmten Benutzer. Der Benutzer wird vorhergehend abgefragt.
	 * 
	 * @return Liefert <code>true</code>, wenn die Struktur der Symbolleiste
	 *         erfolgreich gesichert werden konnte, sonnst <code>false</code>.
	 */
	private boolean aktionSymbolleisteSpeichernLokalFuerBestimmtenBenutzer()
	{
		boolean erfolgreich = false;
		String xmlStruktur = _msld.ausgabeXmlFormatiert();

		String benutzer = null;
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			benutzer = bw.oeffne();
		} else
		{
			InputDialog inputDialog = new InputDialog(
					PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					"Benutzernamen eingeben",
					"Es existiert keine Verbindung zum DaV, bitte geben Sie den Namen des Benutzers manuell ein:",
					"Hier den Benutzernamen eingeben", null);
			if (inputDialog.open() == Dialog.OK)
			{
				if (inputDialog.getValue() != null)
				{
					benutzer = inputDialog.getValue();
				}
			}
		}

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				if (einstellungenSpeicher
						.editiereParameter(
								DefinitionenEinstellungen.paramXmlStrukturSymbolleiste,
								xmlStruktur))
				{
				} else
				{
					einstellungenSpeicher.hinzufuegenParameter(
							DefinitionenEinstellungen.paramXmlStrukturSymbolleiste,
							xmlStruktur);
				}

				erfolgreich = esw.sichereEinstellungenSpeicherBenutzerBestimmtLokal(
						benutzer, einstellungenSpeicher);
			}
		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR | SWT.APPLICATION_MODAL);
			mb.setText("Kein Benutzer angegeben");
			mb
					.setMessage("Sie haben keinen Benutzer angegeben, die Struktur der Symbolleiste kann nicht gespeichert werden");
			mb.open();
		}

		return erfolgreich;
	}

	/**
	 * Ldt die Struktur der Symbolleiste aus den lokalen, allgemeinen
	 * Einstellungen.
	 */
	private void aktionSymbolleisteLadenLokalAllgemein()
	{
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinLokal);

		if (einstellungen != null)
		{
			String xmlStruktur = null;

			xmlStruktur = einstellungen
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

			if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
			{
				_msld = new MenuSymbolLeisteDaten();

				// _msld.einlesenXmlAusString(xmlStruktur);
				// _msld.setzeLeistenArt(enumLeistenArt.symbol);

				generiereMSLD(xmlStruktur, enumLeistenArt.symbol);

				_treeViewer.setInput(_msld.getWurzel());
				_treeViewer.expandAll();

			}
		}
	}

	/**
	 * Speichert die Struktur der Symbolleiste in den lokalen, allgemeinen
	 * Einstellungen.
	 * 
	 * @return Liefert <code>true</code>, wenn die Symbolleistenstruktur
	 *         erfolgreich gespeichert werden konnte, sonst <code>false</code>.
	 */
	private boolean aktionSymbolleisteSpeichernLokalAllgemein()
	{
		boolean erfolgreich = false;

		String xmlStruktur = _msld.ausgabeXmlFormatiert();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinLokal);

		if (einstellungen.editiereParameter(
				DefinitionenEinstellungen.paramXmlStrukturSymbolleiste, xmlStruktur))
		{
		} else
		{
			einstellungen.hinzufuegenParameter(
					DefinitionenEinstellungen.paramXmlStrukturSymbolleiste, xmlStruktur);
		}

		erfolgreich = esw.sichereSpeicherAllgemeinLokal();

		return erfolgreich;
	}

	/**
	 * Ldt die Struktur der Symbolleiste aus den lokalen, benutzerspezifischen
	 * Einstellungen des angemeldeten Benutzers.
	 */
	private void aktionSymbolleisteLadenLokalFuerAngemeldetenBenutzer()
	{
		String benutzer = BenutzerAngemeldet.getInstanz().getName();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				String parameter = null;

				parameter = einstellungenSpeicher
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturSymbolleiste);

				if ((parameter != null) && (parameter.length() > 0))
				{
					_msld = new MenuSymbolLeisteDaten();

					// _msld.einlesenXmlAusString(parameter);
					// _msld.setzeLeistenArt(enumLeistenArt.symbol);

					generiereMSLD(parameter, enumLeistenArt.symbol);

					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} else
				{
					MessageBox msb = new MessageBox(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					msb.setText("Achtung!");
					msb
							.setMessage("Es existiert kein Parameter xmlStrukturSymbolleiste im\n"
									+ "EinstellungenSpeicher fr den Benutzer " + benutzer + "!");

					msb.open();
				}
			} else
			{
				MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
				msb.setText("Achtung!");
				msb
						.setMessage("Es existiert kein EinstellungenSpeicher fr\nden Benutzer "
								+ benutzer + "!");
				msb.open();
			}
		} else
		{
			MessageBox msb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
			msb.setText("Kein Benutzer angemeldet");
			msb
					.setMessage("Es ist kein Benutzer angemeldet.\n"
							+ "Die lokalen Einstellungen fr die Symbolleiste knnen nicht geladen werden!");
			msb.open();
		}
	}

	/**
	 * Speichert die Struktur der Symbolleiste in den lokalen,
	 * benutzerspezifischen Einstellungen fr den angemeldeten Benutzer.
	 * 
	 * @return Liefert <code>true</code>, wenn die Einstellungen erfolgreich
	 *         gesichert werden konnten, sonnst <code>false</code>.
	 */
	private boolean aktionSymbolleisteSpeichernLokalFuerAngemeldetenBenutzer()
	{
		boolean erfolgreich = false;
		String xmlStruktur = _msld.ausgabeXmlFormatiert();

		String benutzer = BenutzerAngemeldet.getInstanz().getName();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungenSpeicher = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
			if (einstellungenSpeicher != null)
			{
				if (einstellungenSpeicher
						.editiereParameter(
								DefinitionenEinstellungen.paramXmlStrukturSymbolleiste,
								xmlStruktur))
				{
				} else
				{
					einstellungenSpeicher.hinzufuegenParameter(
							DefinitionenEinstellungen.paramXmlStrukturSymbolleiste,
							xmlStruktur);
				}

				erfolgreich = esw.sichereEinstellungenSpeicherBenutzerBestimmtLokal(
						benutzer, einstellungenSpeicher);
			}
		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_ERROR | SWT.APPLICATION_MODAL);
			mb.setText("Kein Benutzer angemeldet");
			mb
					.setMessage("Es ist kein Benutzer angemeldet. Die Struktur der Symbolleiste\n"
							+ "kann nicht in den lokalen Einstellungen fr einen Benutzer\n"
							+ "gespeichert werden.");
			mb.open();
		}

		return erfolgreich;
	}

	@Override
	public void setFocus()
	{
		if (_compositeOben != null)
			_compositeOben.setFocus();
	}

	/**
	 * Initiiert das Fllen der von der View bereitgestellten ActionBars.
	 * 
	 */
	private void verteileInActionBars()
	{
		// IActionBars bars = getEditorSite().getActionBars();
		// fuelleLokalesPullDownMenu(bars.getMenuManager());
		// fuelleLokaleToolBar(bars.getToolBarManager());
	}

	/**
	 * Fllt das Men der View
	 * 
	 * @param manager
	 *          MenManager des Mens der View
	 */
	@SuppressWarnings("unused")
	private void fuelleLokalesPullDownMenu(IMenuManager manager)
	{
		manager.add(_ladenAusBestimmterDatei);
		manager.add(_aktionSpeichernUnter);

		// if (KomandoZeilenParameter.getInstanz().istAdministrator())
		if (true)
		{
			/*
			 * Menleistenspezifisches
			 */
			manager.add(new Separator());

			if (_aktionMenuleisteEinlesenAngemeldeterBenutzerDav != null)
			{
				manager.add(_aktionMenuleisteEinlesenAngemeldeterBenutzerDav);
			}

			if (_aktionMenuleisteEinlesenBestimmterBenutzerDaV != null)
			{
				manager.add(_aktionMenuleisteEinlesenBestimmterBenutzerDaV);
			}

			if (_aktionMenuleisteEinlesenNetzweitDaV != null)
			{
				manager.add(_aktionMenuleisteEinlesenNetzweitDaV);
			}

			if (_aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV != null)
			{
				manager.add(_aktionMenuleisteSpeichernUnterAngemeldetemBenutzerDaV);
			}

			if (_aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV != null)
			{
				manager.add(_aktionMenuleisteSpeichernUnterBestimmtenBenutzerDaV);
			}

			if (_aktionMenuleisteSpeichernUnterNetzweitDaV != null)
			{
				manager.add(_aktionMenuleisteSpeichernUnterNetzweitDaV);
			}

			if (_aktionMenuAktuellAktivieren != null)
			{
				manager.add(new Separator());
				manager.add(_aktionMenuAktuellAktivieren);
			}

			/*
			 * Symbolleisten spezifisches
			 */
			manager.add(new Separator());

			if (_aktionSymbolleisteEinlesenBestimmterBenutzerDaV != null)
			{
				manager.add(_aktionSymbolleisteEinlesenBestimmterBenutzerDaV);
			}

			if (_aktionSymbolleisteEinlesenAngemeldeterBenutzerDav != null)
			{
				manager.add(_aktionSymbolleisteEinlesenAngemeldeterBenutzerDav);
			}

			if (_aktionSymbolleisteEinlesenNetzweitDaV != null)
			{
				manager.add(_aktionSymbolleisteEinlesenNetzweitDaV);
			}

			if (_aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV != null)
			{
				manager.add(_aktionSymbolleisteSpeichernUnterBestimmtenBenutzerDaV);
			}

			if (_aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV != null)
			{
				manager.add(_aktionSymbolleisteSpeichernUnterAngemeldetemBenutzerDaV);
			}

			if (_aktionSymbolleisteSpeichernUnterNetzweitDaV != null)
			{
				manager.add(_aktionSymbolleisteSpeichernUnterNetzweitDaV);
			}

			if (_aktionSymbolleisteAktuellAktivieren != null)
			{
				manager.add(new Separator());
				manager.add(_aktionSymbolleisteAktuellAktivieren);
			}

		}
	}

	/**
	 * Fllt die ToolBar der View
	 * 
	 * @param manager
	 *          ToolBarManager der ToolBar der View
	 */
	@SuppressWarnings("unused")
	private void fuelleLokaleToolBar(IToolBarManager manager)
	{
		manager.add(_ladenAusBestimmterDatei);
		manager.add(_aktionSpeichernUnter);
	}

	/*
	 * 
	 * Listener-Methoden
	 */

	/**
	 * Enthlt die Aktionen, die vom Listener des Buttons _buttonAusschneiden
	 * durchgefhrt werden
	 */
	private void aktionMethodeAusschneiden()
	{
		_debug.fine("Ausschneiden");

		IStructuredSelection is = (IStructuredSelection) _treeViewer.getSelection();
		Object firstElement = is.getFirstElement();
		if (firstElement instanceof Element)
		{
			Element tempElement = (Element) firstElement;
			_einfuegenGespeichertesElement = tempElement;
			setTextElementSpeicher(elementNachString(tempElement));

			// Einfgen eingrauen
			_buttonEinfuegen.setEnabled(true);
		}
	}

	/**
	 * Enthlt die Aktionen, die vom Listener des Buttons _buttonEinfuegen
	 * durchgefhrt werden
	 * 
	 */
	private void aktionMethodeEinfuegen()
	{
		_debug.fine("Einfgen!");

		// Einfgeoperation

		// Bestimme aktuell selektiertes Element
		Element zielElement = aktuellSelektiertesElement();

		// ////
		// Typ des Quell-Elementes bestimmen

		// item einzufgen?
		if (_einfuegenGespeichertesElement.getName().equalsIgnoreCase(_elementItem))
		{
			_debug.fine("ZIEL:::::" + zielElement.getName());
			// Ist Ziel Men? Dann ans Ende hngen
			if (zielElement.getName().equalsIgnoreCase(_elementMenu))
			{
				// _menuBarData.verschiebeItem(
				// getElementId(_einfuegenGespeichertesElement),
				// getElementId(zielElement));

				_msld.verschiebeItem(_einfuegenGespeichertesElement, zielElement);

				treeViewerAktuallisieren();
			}
			// Ist Ziel Item oder ein Submenu? Dann davor einfgen
			else if ((zielElement.getName().equalsIgnoreCase(_elementItem))
					|| (zielElement.getName().equalsIgnoreCase(_elementSubmenu))
					|| (zielElement.getName().equalsIgnoreCase(_elementItem))
					|| (zielElement.getName().equalsIgnoreCase(_elementSeparator)))
			{
				// Position des ZielElementes bestimmen
				// int zielPosition = bestimmePositionImElternElement(zielElement);
				int zielPosition = _msld
						.bestimmeElementPositionInElternElement(zielElement);

				// String idEinzufuegendesElement =
				// getElementId(_einfuegenGespeichertesElement);
				// String idZielMenu = getElementId(zielElement.getParentElement());
				//
				// _menuBarData.verschiebeItem(idEinzufuegendesElement, idZielMenu,
				// zielPosition);

				_msld.verschiebeItem(_einfuegenGespeichertesElement, zielElement
						.getParentElement(), zielPosition);

				treeViewerAktuallisieren();

			}
		}
		// Separator
		else if (_einfuegenGespeichertesElement.getName().equals(_elementSeparator))
		{
			if (zielElement.getName().equalsIgnoreCase(_elementMenu))
			{
				_msld.hinzufuegenSeparatorAnMenu(_einfuegenGespeichertesElement,
						zielElement);
			} else if (zielElement.getName().equals(_elementItem)
					|| zielElement.getName().equals(_elementSubmenu))
			{
				_msld.hinzufuegenSeparatorInMenu(_einfuegenGespeichertesElement,
						zielElement);
			}

			treeViewerAktuallisieren();
		}
		// men einzufgen?
		else if (_einfuegenGespeichertesElement.getName().equalsIgnoreCase(
				_elementMenu))
		{
			// if(pruefeIstZielElementNichtEigenesSubElement(_einfuegenGespeichertesElement,
			// zielElement))
			if (pruefeIstZielElementNichtEigenesSubElementRekursiv(
					_einfuegenGespeichertesElement, zielElement))
			{
				if (zielElement.getName().equalsIgnoreCase(_elementSubmenu))
				{
					// Prfen, ob Submen leer ist
					if (bestimmeAnzahlKinderImElement(zielElement) == 0)
					{
						String idEinzufuegendesElement = getElementId(_einfuegenGespeichertesElement);
						String idZielSubmenu = getElementId(zielElement);
						_debug.fine("ID QUELLE: " + idEinzufuegendesElement + " ID Ziel: "
								+ idZielSubmenu);

						// _menuBarData.verschiebeMenu(idEinzufuegendesElement,
						// idZielSubmenu);

						_msld.verschiebeMenu(_einfuegenGespeichertesElement, zielElement);

						treeViewerAktuallisieren();
					} else
					{
						// _debug.fine("Im Ziel-Submen ist bereits ein Men fest
						// verankert. Aktion abgebrochen!");
						zeigeInfoDialog("Achtung",
								"Im Ziel-Submen ist bereits ein Men fest verankert. Aktion abgebrochen!");
					}
				} else
				{
					// _debug.fine("Ein Men kann nur in ein Submen verschoben
					// werden. Aktion abgebrochen!");
					zeigeInfoDialog("Achtung",
							"Ein Men kann nur in ein Submen verschoben werden. Aktion abgebrochen!");
				}
			} else
			{
				// System.out.println("Ein Men kann nicht in ein eigenes Untermen
				// verschoben werden. Aktion abgebrochen!");
				zeigeInfoDialog(
						"Achtung",
						"Ein Men kann nicht in ein eigenes Untermen verschoben werden. Aktion abgebrochen!");
			}

		}
		// submen einfgen?
		else if (_einfuegenGespeichertesElement.getName().equalsIgnoreCase(
				_elementSubmenu))
		{
			if (pruefeIstZielElementNichtEigenesSubElementRekursiv(
					_einfuegenGespeichertesElement, zielElement))
			{
				// Ist Zielmen ein menu?
				if (zielElement.getName().equals(_elementMenu))
				{
					// String idEinzufuegendesElement =
					// getElementId(_einfuegenGespeichertesElement);
					// String idZielMenu = getElementId(zielElement);
					// _menuBarData.verschiebeSubMenu(idEinzufuegendesElement,
					// idZielMenu);

					_msld.verschiebeSubMenu(_einfuegenGespeichertesElement, zielElement);

					treeViewerAktuallisieren();
				} else
				{
					if (zielElement.getName().equals(_elementItem)
							|| zielElement.getName().equals(_elementSubmenu))
					{
						// String idEinzufuegendesElement =
						// getElementId(_einfuegenGespeichertesElement);
						// int zielPosition = bestimmePositionImElternElement(zielElement);
						int zielPosition = _msld
								.bestimmeElementPositionInElternElement(zielElement);

						// Element elternElementVomZielElement = zielElement
						// .getParentElement();
						// String idZielMenu = getElementId(elternElementVomZielElement);

						// _menuBarData.verschiebeSubMenu(idEinzufuegendesElement,
						// idZielMenu,
						// zielPosition);

						_msld.verschiebeSubMenu(_einfuegenGespeichertesElement, zielElement
								.getParentElement(), zielPosition);

						treeViewerAktuallisieren();
					}
				}
			} else
			{
				zeigeInfoDialog(
						"Achtung",
						"Ein Submen kann nicht in eines seiner Unter-Submens verschoben werden. Aktion abgebrochen!");
			}

		} else
		{
			// System.out.println("Noch nicht untersttzt!");
			zeigeInfoDialog("Achtung", "Dieses Element wird noch nicht untersttzt!");
		}

		// Am Ende:
		_einfuegenGespeichertesElement = null;
		setTextElementSpeicher("leer");
		_buttonEinfuegen.setEnabled(false);
	}

	/**
	 * Enthlt die Aktionen, die zum einfgen eines neuen Submens notwendig sind.
	 * 
	 */
	private void aktionMethodeSubmenuEinfuegen()
	{
		String subId = null;
		String subName = null;
		String menuId = null;

		Shell shell = Display.getCurrent().getActiveShell();

		subId = ""
				+ _msld.bestimmeFreieId(_msld.getWurzel(),
						XmlBezeichnerMenuSymbol.elementSubMenu,
						XmlBezeichnerMenuSymbol.attId);

		InputDialog inputDialog = new InputDialog(shell, "Eingabe Submen Name",
				"Bitte geben Sie den Namen des neuen Submens ein!", null, null);
		if (inputDialog.open() == Dialog.OK)
		{
			subName = inputDialog.getValue();
		}

		menuId = ""
				+ _msld.bestimmeFreieId(_msld.getWurzel(),
						XmlBezeichnerMenuSymbol.elementMenu, XmlBezeichnerMenuSymbol.attId);

		// System.out.println("*" + subId + "*" + subName + "*" + menuId + "*");

		if ((subId != null && subName != null && menuId != null) && subId != ""
				&& subName != "" && menuId != "")
		{
			XmlElementSubmenu sub = new XmlElementSubmenu(subId, subName);
			XmlElementMenu menu = new XmlElementMenu(menuId);

			Element elementSubmenu = sub.getElementSubmenu();
			Element elementMenu = menu.getElementMenu();
			elementSubmenu.addContent(elementMenu);

			_debug.fine("Submenu: " + elementSubmenu.getAttributeValue("id"));
			_debug.fine("KindelementId: "
					+ elementSubmenu.getChild("menu").getAttributeValue("id"));

			// submen einfgen?
			// Bestimme aktuell selektiertes Element
			Element zielElement = aktuellSelektiertesElement();
			Element einfuegeElement = elementSubmenu;

			if (einfuegeElement.getName().equalsIgnoreCase(_elementSubmenu))
			{
				// Ist Zielmen ein menu?
				if (zielElement.getName().equalsIgnoreCase(_elementMenu))
				{
					@SuppressWarnings("unused")
					String idEinzufuegendesElement = getElementId(einfuegeElement);

					_msld.setzeElementBei(zielElement, einfuegeElement, 0);

					System.out.println("XML => " + _msld.ausgabeXmlFormatiert());

					treeViewerAktuallisieren();
				} else
				{
					if (zielElement.getName().equalsIgnoreCase(_elementItem)
							|| zielElement.getName().endsWith(_elementSubmenu)
							|| zielElement.getName().equalsIgnoreCase(_elementSeparator))
					{
						Element elternElementVomZielElement = zielElement
								.getParentElement();

						int pos = _msld.bestimmeElementPositionInElternElement(zielElement);

						// System.out.println("ZielElement : " + zielElement);
						// System.out.println("ElternElement: " +
						// elternElementVomZielElement);
						// System.out.println("Pos in Eltern: " + pos);

						_msld.setzeElementBei(elternElementVomZielElement, einfuegeElement,
								pos);
						// elternElementVomZielElement.addContent(einfuegeElement);
						// elternElementVomZielElement.addContent(pos, einfuegeElement);

						System.out.println("XML => " + _msld.ausgabeXmlFormatiert());

						treeViewerAktuallisieren();

						// @SuppressWarnings("unused")
						// String idEinzufuegendesElement = getElementId(einfuegeElement);
						// int zielPosition = bestimmePositionImElternElement(zielElement);
						// Element elternElementVomZielElement = zielElement
						// .getParentElement();
						// String idZielMenu = getElementId(elternElementVomZielElement);
						//
						// // _menuBarData.verschiebeSubMenu(idEinzufuegendesElement,
						// // idZielMenu, zielPosition);
						// _msld.hinzufuegenSubMenu(sub, idZielMenu, zielPosition);
						//
						// treeViewerAktuallisieren();
					}
				}
			}
		} else
		{
			MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR);
			box.setText("Fehler");
			box.setMessage("Sie haben eine Eingabe vergessen!");
			box.open();
		}
	}

	/**
	 * Benennt das aktuell selektierte Submen um.
	 */
	private void aktionSubmenuUmbenennen()
	{
		Element element = aktuellSelektiertesElement();
		if (element.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
		{
			_debug.fine("Element: " + element.toString());
			Attribute name = element.getAttribute("name");
			if (name != null)
			{
				InputDialog eingabeDialog = new InputDialog(PlatformUI.getWorkbench()
						.getDisplay().getActiveShell(), "Submennamen bearbeiten:",
						"Bitte geben Sie einen neuen Namen fr das Submen \""
								+ name.getValue() + "\" ein.", name.getValue(), null);

				if (eingabeDialog.open() == Dialog.OK)
				{
					String ergebnis = eingabeDialog.getValue();
					if (ergebnis != null)
					{
						if (_msld != null)
						{
							_msld.pushAktuellerZustandUndoStack();
						}

						name.setValue(ergebnis);
						treeViewerAktuallisieren();
					}
				}
			}

		} else
		{
			// Keine Aktion
		}
	}

	/**
	 * Enthlt die Aktionen, die zum Einfgen eines neuen Separators notwendig
	 * sind. Kann aus dem Kontextmen heraus aufgerufen werden.
	 * 
	 */
	private void aktionMethodeSeparatorEinfuegen()
	{
		// Bestimme aktuell selektiertes Element
		Element zielElement = aktuellSelektiertesElement();

		XmlElementSeparator sep = new XmlElementSeparator("1", "Separator");

		if (zielElement.getName().equalsIgnoreCase(_elementMenu))
		{
			_msld.hinzufuegenSeparatorInMenuDep(sep, zielElement);

		} else if (zielElement.getName().equals(_elementItem)
				|| zielElement.getName().equals(_elementSubmenu))
		{
			_msld.hinzufuegenSeparatorAnPositionInMenuDep(sep, zielElement);
		}

		treeViewerAktuallisieren();

	}

	// /**
	// * Enthlt die Aktionen, die zum Einfgen eines neuen GroupMarkers notwendig
	// * sind. Kann aus dem Kontextmen heraus aufgerufen werden.
	// *
	// */
	// private void actionMethodeGroupMarkerEinfuegen()
	// {
	// //Implementieren und in einer Action im Kontextmen einhngen.
	// }

	/**
	 * Enthlt die Aktionen, die vom Listener des Buttons _buttonLoeschen
	 * durchgefhrt werden
	 * 
	 */
	private void aktionMethodeLoeschen()
	{
		IStructuredSelection is = (IStructuredSelection) _treeViewer.getSelection();
		Object firstElement = is.getFirstElement();
		if (firstElement instanceof Element)
		{
			Element loeschElement = (Element) firstElement;

			_msld.loescheElement(loeschElement);

			// String id = getElementId(loeschElement);
			//
			// // Item?
			// if (loeschElement.getName().equalsIgnoreCase(_elementItem))
			// {
			// _menuBarData.loescheItem(id);
			// }
			// // Menu?
			// else if (loeschElement.getName().equalsIgnoreCase(_elementMenu))
			// {
			// _menuBarData.loescheMenu(id);
			// }
			// // Submen
			// else if (loeschElement.getName().equalsIgnoreCase(_elementSubmenu))
			// {
			// _menuBarData.loescheSubMenu(id);
			// }
			// // Separator
			// else if (loeschElement.getName().equals(_elementSeparator))
			// {
			// _menuBarData.loescheSeparator(loeschElement);
			// }
			// // sonst...
			// else
			// {
			// System.out
			// .println(getClass().getName()
			// + "::actionMethodeLoeschen \n=> Element ist in der Datenstruktur nicht
			// bekannt und kann somit nicht gelscht werden!\n");
			// }
			treeViewerAktuallisieren();

			// _buttonLoeschen.setEnabled(false);
		}
	}

	/**
	 * Enthlt die Aktionen zum Neuladen aus einer bestimmten Datei.
	 * 
	 */
	private void aktionMethodeNeuLadenAusBestimmterDatei()
	{
		if (_festeDatei)
		{
			File inFile = new File("C:\\men.xml");
			if (inFile.exists())
			{
				try
				{
					_msld.einlesenXmlAusDatei("C:\\men.xml");
					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} catch (IllegalArgumentException iae)
				{
					iae.printStackTrace();
				}
			}
		} else
		{

			FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(),
					SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.OPEN);
			dialog.setFilterNames(new String[] { "XML-Dateien" });
			dialog.setFilterExtensions(new String[] { "*.xml" });
			String dateiPfad = dialog.open();

			_debug.fine(getClass().getName()
					+ "::actionMethodeNeuLadenAusBestimmterDatei => \nffnen von Datei: "
					+ dateiPfad + "\n");

			if (dateiPfad != null)
			{
				_xmlDateiPfad = dateiPfad;
				try
				{
					_msld.einlesenXmlAusDatei(dateiPfad);
					_treeViewer.setInput(_msld.getWurzel());
					_treeViewer.expandAll();
				} catch (IllegalArgumentException iae)
				{
					iae.printStackTrace();
				}
			}

		}

	}

	/**
	 * Enthlt die Aktionen, die vom Listener des Buttons _buttonSpeichernInDatei
	 * durchgefhrt werden
	 * 
	 */
	private void aktionMethodeSpeichernInDatei()
	{
		if (_festeDatei)
		{
			File outFile = new File("C:\\men.xml");
			if (outFile.exists())
			{
				String ausgabeString = _msld.ausgabeXmlFormatiert();
				if (ausgabeString != null)
				{
					_msld.schreibeString("C:\\men.xml", ausgabeString, true);
					// feureEreignis();
				} else
				{
					System.out
							.println(getClass().getName()
									+ "::actionMethodeSpeichernInDatei => \nEs wurde kein kompletter Dateipfad angegeben!\n");
				}
			}

		} else
		{
			FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(),
					SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.SAVE);
			// dialog.setFilterNames (new String [] {"XML-Dateien", "Alle Dateien
			// (*.*)"});
			// dialog.setFilterExtensions(new String[] {"*.xml", "*.*"});
			dialog.setFilterNames(new String[] { "XML-Dateien" });
			dialog.setFilterExtensions(new String[] { "*.xml" });
			// dialog.setFilterPath("c://");

			String dateiPfad = dialog.open();

			_debug.fine(getClass().getName()
					+ "::actionMethodeSpeichernInDatei => \nSpeichern unter: "
					+ dateiPfad + "\n");

			String ausgabeString = _msld.ausgabeXmlFormatiert();
			if ((dateiPfad != null) && (ausgabeString != null))
			{
				_xmlDateiPfad = dateiPfad;
				_msld.schreibeString(dateiPfad, ausgabeString, true);
				// feureEreignis();
			} else
			{
				System.out
						.println(getClass().getName()
								+ "::actionMethodeSpeichernInDatei => \nEs wurde kein kompletter Dateipfad angegeben!\n");
			}
		}

	}

	/**
	 * Speichert in die Datei, aus der der XML-Baum aufgebaut wurde.
	 * 
	 */
	private void aktionMethodeSpeichern()
	{
		String ausgabeString = _msld.ausgabeXmlFormatiert();
		if ((ausgabeString != null) && (_xmlDateiPfad != null))
		{
			_msld.schreibeString(_xmlDateiPfad, ausgabeString, true);
			// feureEreignis();
		} else
		{
			System.out
					.println(getClass().getName()
							+ "::actionMethodeSpeichern => \nEs wurde kein kompletter Dateipfad angegeben!\n");
		}
	}

	/*
	 * 
	 * Hilfs-Methoden
	 */

	/**
	 * Liefert eine String-Darstellung eines beliebigen Elements vom Typ
	 * org.jdom.Element. Kindelemente werden bei der Darstellung NICHT
	 * bercksichtigt
	 * 
	 * @param element
	 *          Element
	 * @return String-Darstellung des Elements
	 */
	private String elementNachString(Element element)
	{
		if (_msld != null)
		{
			return _msld.ElementNachString(element);
		} else
		{
			return ("Die Stringdarstellung des Elements kann nicht bestimmt werden!");
		}
	}

	/**
	 * Setzt den Text von _textElementSpeicher
	 * 
	 * @param text
	 *          Text, der gesetzt werden soll.
	 */
	private void setTextElementSpeicher(String text)
	{
		_textElementSpeicher.setText(text);
		_textElementSpeicher.pack();
	}

	/**
	 * Setzt den Text von _textElementInfo, sofern dieses != null
	 * 
	 * @param text
	 *          Text, der in das Textfeld gesetzt werden soll.
	 */
	private void setTextElementInfo(String text)
	{
		if (_textElementInfo != null)
		{
			_textElementInfo.setText(text);
			_textElementInfo.pack();
		}
	}

	/**
	 * Liefert das aktuell selektierte Element im _treeViewer zurck. Ist keines
	 * selektiert, wird null geliefert
	 * 
	 * @return Das aktuell selektierte Element im _treeViewer
	 */
	private Element aktuellSelektiertesElement()
	{
		IStructuredSelection is = (IStructuredSelection) _treeViewer.getSelection();
		Object firstElement = is.getFirstElement();
		if (firstElement instanceof Element)
		{
			Element tempElement = (Element) firstElement;
			return tempElement;
		} else
		{
			return null;
		}
	}

	/**
	 * Liefert von jedem org.jdom.Element den Wert des Attributes id.
	 * 
	 * @param element
	 *          Element, von der Wert der Attributs id bestimmt werden soll
	 * @return Wert des Attributs id
	 */
	private String getElementId(Element element)
	{
		String returnString = null;
		returnString = element.getAttributeValue("id");
		return returnString;
	}

	/**
	 * Bestimmt die Position des Kindelements in der Liste der Kinder des
	 * Elternelements <br>
	 * <b>Funkttion aus {@link MenuSymbolLeisteDaten} benutzen!</b>
	 * 
	 * @deprecated
	 * @param kindElement
	 * @return Position des Kindelements
	 */
	@SuppressWarnings("unused")
	private int bestimmePositionImElternElement(Element kindElement)
	{
		int returnWert = -1;
		Element parentElement = kindElement.getParentElement();
		@SuppressWarnings("unchecked")
		Iterator it = parentElement.getChildren().iterator();
		int i = -1;
		while (it.hasNext())
		{
			i++;
			Element vergleichsElement = (Element) it.next();

			if (vergleichsElement.equals(kindElement))
			{
				break;
			}
		}
		returnWert = i;

		return returnWert;
	}

	/**
	 * Bestimmt die Anzahl der Kindelemente von einem Element<br>
	 * <b> {@link MenuSymbolLeisteDaten#bestimmeAnzahlKinderImElement(Element)}
	 * benutzen!</b>
	 * 
	 * @deprecated
	 * @param element
	 *          Element, von dem die Anzahl der Kindelemente bestimmt werden soll
	 * @return Anzahl der Kindelemente des Elements
	 */
	private int bestimmeAnzahlKinderImElement(Element element)
	{
		int returnWert = 0;
		returnWert = element.getChildren().size();
		return returnWert;
	}

	/**
	 * Prft auf Quell-Element als Parent<br>
	 * <b> {@link MenuSymbolLeisteDaten#istXmlElementKindVon(Element, Element)}
	 * benutzen</b>
	 * 
	 * @param quelle
	 *          Quell-Element, dass nicht ein Parent vom Ziel Element sein darf
	 * @param ziel
	 *          Ziel-Element, bei dem das Quell-Element kein Parent sein darf
	 * @return true, falls Quell-Element kein Parent von Ziel-Element ist, sonst
	 *         false
	 */
	private boolean pruefeIstZielElementNichtEigenesSubElementRekursiv(
			Element quelle, Element ziel)
	{
		boolean returnWert = true;

		if (ziel.equals(quelle))
		{
			returnWert = false;
		} else
		{
			if (ziel.getParentElement() != null)
			{
				returnWert = pruefeIstZielElementNichtEigenesSubElementRekursiv(quelle,
						ziel.getParentElement());
			}
		}

		return returnWert;
	}

	/**
	 * Aktuallisiert den TreeViewer mit den Daten aus der hinterlegten
	 * {@link MenuSymbolLeisteDaten}-Struktur.
	 * 
	 */
	private void treeViewerAktuallisieren()
	{
		_treeViewer.refresh();
		editorTitelAktualisieren();
	}

	/**
	 * Initialisiert den {@link TreeViewer} mit den Daten einer (neu) hinterlegten
	 * {@link MenuSymbolLeisteDaten}-Struktur.
	 */
	private void treeViewerNeuLaden()
	{
		_treeViewer.setInput(_msld.getWurzel());
		_treeViewer.expandAll();
		editorTitelAktualisieren();
	}

	/**
	 * Aktualisiert den Titel des Editors an Hand der hinterlegten
	 * {@link MenuSymbolLeisteDaten}-Struktur.
	 */
	private void editorTitelAktualisieren()
	{
		/*
		 * PartName setzen
		 */
		switch (_msld.getLeistenArt())
		{
		case menue:
			this.setPartName("Menleisten Editor");
			firePropertyChange(PROP_TITLE);
			break;
		case symbol:
			this.setPartName("Symbolleisten Editor");
			firePropertyChange(PROP_TITLE);
			break;
		default:
			this.setPartName("Undefiniert");
			break;
		}
	}

	/**
	 * Zeigt einen Info-Dialog an. Titel + Text sind parametrierbar.
	 * 
	 * @param textTitel
	 *          Titel des Dialogs
	 * @param textInfo
	 *          Text des Dialogs
	 */
	private void zeigeInfoDialog(String textTitel, String textInfo)
	{
		MessageBox box = new MessageBox(Display.getCurrent().getActiveShell(),
				SWT.OK | SWT.ICON_WARNING);
		box.setText(textTitel);
		box.setMessage(textInfo);
		box.open();
	}

	/*
	 * Listener/Event-Verarbeitung
	 */

	// /**
	// * Fgt einen <code>IListenerXmlBaumChanged</code> Listener hinzu.
	// *
	// * @param l
	// * <code>IListenerXmlBaumChanged</code>, der hinzugefgt werden
	// * soll.
	// */
	// public void hinzufuegenZuhoerer(IZuhoererMenuLeisteAenderung l)
	// {
	// if (_debug)
	// {
	// System.out.println(getClass().getName()
	// + "::hinzufuegenListener => Event registriert!");
	// }
	//
	// _zuhoerer.add(l);
	// }
	//
	// /**
	// * Entfernt einen <code>IListenerXmlBaumChanged</code>.
	// *
	// * @param l
	// * <code>IListenerXmlBaumChanged</code>, der entfernt werden soll.
	// */
	// public void entferneZuhoerer(IZuhoererMenuLeisteAenderung l)
	// {
	// _zuhoerer.remove(l);
	// }
	// /**
	// * Abfeuern des Events
	// *
	// */
	// private void feureEreignis()
	// {
	// if (_debug)
	// {
	// System.out.println(this.getClass().getName()
	// + "::feureEvent => Event abgefeuert!");
	// System.out.println(this.getClass().getName()
	// + "::feureEvent => Anzahl Listener: " + _zuhoerer.size());
	// }
	//
	// if (_zuhoerer.size() > 0)
	// {
	// EreignisMenuLeisteGeandert e = new EreignisMenuLeisteGeandert(
	// "XmlBaumChanged");
	//
	// Iterator<IZuhoererMenuLeisteAenderung> it = _zuhoerer.iterator();
	// while (it.hasNext())
	// {
	// IZuhoererMenuLeisteAenderung changed = it.next();
	// changed.ausfuehren(e);
	// }
	// }
	// }
	@SuppressWarnings("unused")
	private enum Quelle
	{
		benutzerNetzweit, benutzerLokal, allgemeinNetzweit, allgemeinLokal
	}

	/**
	 * Contentprovider fr den Meneditor TreeViewer. Stellt die Inhalte fr den
	 * verwendeten <code>TreeViewer</code> bereit.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class XmlBaumInhaltsVersorger implements ITreeContentProvider
	{

		/**
		 * Liefert die Kindelemente des angegebenen Elternelements
		 * 
		 * @param parentElement
		 *          Elternelement, zu dem die Kindelemente bestimmt werden sollen
		 * @return Kindelemente des Elternelements
		 */
		public Object[] getChildren(Object parentElement)
		{

			Object[] objects = null;

			// Element?
			if (parentElement instanceof Element)
			{
				Element parent = (Element) parentElement;
				@SuppressWarnings("unchecked")
				List children = parent.getChildren();
				objects = new Object[children.size()];
				int i = -1;
				@SuppressWarnings("unchecked")
				Iterator it = children.iterator();
				while (it.hasNext())
				{
					i++;
					objects[i] = (Element) it.next();

					/*
					 * Wenn die Leistenart festgelegt ist, dann fr das erste Element die
					 * Ausgabe anpassen.
					 */
					if (_msld.getLeistenArt() != null)
					{
						if (i == 0)
						{
							if (objects[i] instanceof Element)
							{
								Element te = (Element) objects[i];
								switch (_msld.getLeistenArt())
								{
								case menue:
									te.setAttribute(new Attribute("art", "Menleiste"));
									break;
								case symbol:
									te.setAttribute(new Attribute("art", "Symbolleiste"));
									break;
								default:
									te.setAttribute(new Attribute("art", "Unbekannt"));
									break;
								}
							}
						}
					}
				}
			} else
			{
				// Keine Kindelemente, dann wird null zurckgegeben
			}

			return objects;
		}

		/**
		 * Bestimmt das Elternelement eines Elements
		 * 
		 * @param element
		 *          Element, zu dem das Elternelement bestimmt werden soll
		 * @return Elternelement des Elements oder <code>null</code>, falls kein
		 *         Elternelement bestimmt werden kann
		 */
		public Object getParent(Object element)
		{
			if (element instanceof Element)
			{
				Element tempElement = (Element) element;
				return tempElement.getParentElement();
			} else
			{
				return null;
			}

		}

		/**
		 * Prft, ob das bergebene Element ber Kindelemente verfgt
		 * 
		 * @param element
		 *          Element, zu dem die Prfung durchefhrt werden soll
		 * @return <code>true</code> falls das Element ber Kindelemente verfgt
		 *         sonst <code>false</code>
		 */
		public boolean hasChildren(Object element)
		{
			return getChildren(element).length > 0;
		}

		/**
		 * Liefert die Elemente eines Elements
		 * 
		 * @param inputElement
		 *          Element, zu dem die Elemente bestimmt werden sollen
		 * @return Elemente, die zu einem Element bestimmt wurden
		 */
		public Object[] getElements(Object inputElement)
		{
			return getChildren(inputElement);
		}

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * LabelProvider fr den <code>TreeViewer</code> der Baumdarstellung Stellt
	 * die Bezeichnungen sowie die Symbole der Elemente des verwendeten
	 * <code>TreeViewers</code> bereit.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class XmlBaumDarstellungsVersorger extends LabelProvider
	{
		// Bezeichnungen der Elemente aus der korrespondierenden XML-Datei
		private String _menuElement = "menu";

		private String _itemElement = "item";

		private String _itemElementAttributName = "name";

		private String _submenuElement = "submenu";

		private String _submentElementAttributName = "name";

		private HashMap<ImageDescriptor, Image> imageCache = new HashMap<ImageDescriptor, Image>();

		/**
		 * Liefert den Text zu einem Element
		 * 
		 * @param element
		 *          Element, zu dem der Text bestimmt werden soll
		 * @return Text des Elements
		 */
		public String getText(Object element)
		{
			if (element instanceof Element)
			{
				Element tempElement = (Element) element;

				// Men
				if (tempElement.getName().equalsIgnoreCase(_menuElement))
				{
					// String returnString = "MEN (ID: "
					// + tempElement.getAttributeValue(_menuElementAttributId) + ")";

					String returnString = "MEN";

					String art = tempElement.getAttributeValue("art");
					if (art != null)
					{
						returnString = art;
					}

					return returnString;
				}
				// Symbolleiste
				else if (tempElement.getName().equalsIgnoreCase(
						XmlBezeichnerMenuSymbol.elementSymbolLeiste))
				{
					String returnString = "Symbolleiste";
					return returnString;
				}
				// Submen
				else if (tempElement.getName().equalsIgnoreCase(_submenuElement))
				{
					// String returnString = "SUBMEN (ID: "
					// + tempElement.getAttributeValue(_submenuElementAttributId)
					// + " Name: "
					// + tempElement.getAttributeValue(_submentElementAttributName)
					// + " )";
					String returnString = "SUBMEN \""
							+ tempElement.getAttributeValue(_submentElementAttributName)
							+ "\"";
					return returnString;
				}
				// Item
				else if (tempElement.getName().equalsIgnoreCase(_itemElement))
				{
					// String returnString = "ITEM (ID: "
					// + tempElement.getAttributeValue(_itemElementAttributId)
					// + " ActionId: "
					// + tempElement.getAttributeValue(XmlBezeichnerMenu.attActionId)
					// + " Name: "
					// + tempElement.getAttributeValue(_itemElementAttributName) + " )";

					String returnString = "\""
							+ tempElement.getAttributeValue(_itemElementAttributName) + "\"";
					return returnString;
				}
				// Separator
				else if (tempElement.getName().equals(_elementSeparator))
				{
					String returnString = "Separator";
					return returnString;
				} else
				{
					String returnString = "Element \"" + tempElement.getName()
							+ "\" ist unbekannt und wird nicht verarbeitet!";
					// System.out.println("1. " + returnString);
					return returnString;
					// throw unknownElement(element);
				}
			} else
			{
				String returnString = "Das Objekt ist nicht vom Typ org.jdom.Element!";
				// System.out.println("2. " + returnString);
				return returnString;
				// throw unknownElement(element);
			}

		}

		/**
		 * Bestimmt das Symbol eines Elements
		 * 
		 * @param element
		 *          Element, zu dem das Symbol bestimmt werden soll
		 * @return Symbol des Elements als Image
		 */
		public Image getImage(Object element)
		{
			String iconPfad = "icons/menubar/";
			String iconPfadKomplett = null;

			Image returnImage = null;
			if (element instanceof Element)
			{
				Element tempElement = (Element) element;
				if (tempElement.getName().equalsIgnoreCase(_menuElement))
				{
					iconPfadKomplett = iconPfad + "menu.gif";
				} else if (tempElement.getName().equalsIgnoreCase(_submenuElement))
				{
					iconPfadKomplett = iconPfad + "menusubmenu.png";
				} else if (tempElement.getName().equalsIgnoreCase(_itemElement))
				{
					iconPfadKomplett = iconPfad + "menuitem.gif";
				} else if (tempElement.getName().equals(_elementSeparator))
				{
					iconPfadKomplett = iconPfad + "separator_16x16.png";
				} else
				{
					iconPfadKomplett = iconPfad + "unbekannt_16x16.png";
				}
			}

			if (_ir.get(iconPfadKomplett) == null)
			{
				try
				{
					_ir.put(iconPfadKomplett, Activator
							.getImageDescriptor(iconPfadKomplett));

					System.out.println("=> IMAGE " + iconPfadKomplett + " hinzugefgt.");
				} catch (IllegalArgumentException iae)
				{
					iae.printStackTrace();
				}
			}

			returnImage = _ir.get(iconPfadKomplett);

			return returnImage;
		}

		/**
		 * Aufrumen - Images im ImageCache "disposen" sowie den ImageCache leeren.
		 */
		public void dispose()
		{
			Iterator<Image> it = imageCache.values().iterator();
			while (it.hasNext())
			{
				Image image = it.next();
				image.dispose();
			}
			imageCache.clear();
		}

		/**
		 * Liefert eine RuntimeException, falls das Element nicht bekannt ist
		 * 
		 * @param element
		 *          Element, zu dem die RuntimeException generiert wird
		 * @return RuntimeException
		 */
		@SuppressWarnings("unused")
		protected RuntimeException unknownElement(Object element)
		{
			if (element instanceof Element)
			{
				Element tempElement = (Element) element;
				return new RuntimeException("Unbekanntes XML-Element im Baum "
						+ tempElement.getName());
			} else
			{
				return new RuntimeException(
						"Es wurde eine unbekannte Klasse fr die Darstellung eines Elementes verwendet: "
								+ element.getClass().getName());
			}
		}

	}

	@Override
	public void doSave(IProgressMonitor monitor)
	{
		_debug.fine("Speichern...");
		aktionSpeichernGlobal();
	}

	@Override
	public void doSaveAs()
	{
		_debug.fine("Speichern unter...");
		aktionSpeichernGlobal();
	}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException
	{
		setSite(site);
		setInput(input);
		setPartName(input.getName());
	}

	@Override
	public void feuereSchmutzigGeandert()
	{
		firePropertyChange(IEditorPart.PROP_DIRTY);
	}

	@Override
	public boolean isDirty()
	{
		if (_msld != null)
		{
			return _msld.istSchmutzig();
		} else
		{
			return false;
		}
	}

	@Override
	public boolean isSaveAsAllowed()
	{
		return true;
	}

}
