/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.editoren;

import org.eclipse.jface.action.Action;

public class EditorOeffnenAktionsSpeicher
{
	/**
	 * Aktion zum ffnen eines leeren Meneditors
	 */
	private Action _actionMenuEditorLeer = null;

	/**
	 * Aktion zum ffnen eines Meneditors mit ffnen-Dialog
	 */
	private Action _actionMenuEditorOeffnenDialog = null;

	/**
	 * Aktion zum ffnen eines leeren Symbolleisteneditors
	 */
	private Action _actionSymbolleisteEditorLeer = null;

	/**
	 * Aktion zum ffnen eines Symbolleisteneditors mit ffnen-Dialog
	 */
	private Action _actionSymbolleisteEditorOeffnenDialog = null;

	/**
	 * Privater Konstruktor
	 */
	private EditorOeffnenAktionsSpeicher()
	{
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse
	 * {@link EditorOeffnenAktionsSpeicher}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen
	{
		private static EditorOeffnenAktionsSpeicher INSTANZ = new EditorOeffnenAktionsSpeicher();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse
	 * {@link EditorOeffnenAktionsSpeicher}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link EditorOeffnenAktionsSpeicher}
	 */
	public static EditorOeffnenAktionsSpeicher getInstanz()
	{
		return Innen.INSTANZ;
	}

	/**
	 * Liefert die Aktion zum ffnen eines leeren Meneditors.
	 * 
	 * @return Aktion zum ffnen eines leeren Meneditors
	 */
	public Action getActionMenuEditorLeer()
	{
		return _actionMenuEditorLeer;
	}

	/**
	 * Setzt die Aktion zum ffnen eines leeren Meneditors.
	 * 
	 * @param menuEditorLeer
	 *          Aktion zum ffnen eines leeren Meneditors
	 */
	public void setActionMenuEditorLeer(Action menuEditorLeer)
	{
		_actionMenuEditorLeer = menuEditorLeer;
	}

	/**
	 * Liefert die Aktion zum ffnen eines Meneditors mit ffnen-Dialog.
	 * 
	 * @return Aktion zum ffnen eines Meneditors mit ffnen-Dialog
	 */
	public Action getActionMenuEditorOeffnenDialog()
	{
		return _actionMenuEditorOeffnenDialog;
	}

	/**
	 * Setzt die Aktion zum ffnen eines Meneditors mit ffnen-Dialog.
	 * 
	 * @param menuEditorOeffnenDialog
	 *          Aktion zum ffnen eines Meneditors mit ffnen-Dialog
	 */
	public void setActionMenuEditorOeffnenDialog(Action menuEditorOeffnenDialog)
	{
		_actionMenuEditorOeffnenDialog = menuEditorOeffnenDialog;
	}

	/**
	 * Liefert die Aktion zum ffnen eines leeren Symbolleistenediors.
	 * 
	 * @return Aktion zum ffnen eines leeren Symbolleistenediors
	 */
	public Action getActionSymbolleisteEditorLeer()
	{
		return _actionSymbolleisteEditorLeer;
	}

	/**
	 * Setzt die Aktion zum ffnen eines leeren Symbolleistenediors.
	 * 
	 * @param symbolleisteEditorLeer
	 *          Aktion zum ffnen eines leeren Symbolleistenediors
	 */
	public void setActionSymbolleisteEditorLeer(Action symbolleisteEditorLeer)
	{
		_actionSymbolleisteEditorLeer = symbolleisteEditorLeer;
	}

	/**
	 * Liefert die Aktion zum ffnen eines Symbolleistenediors mit ffnen-Dialog.
	 * 
	 * @return Aktion zum ffnen eines Symbolleistenediors mit ffnen-Dialog
	 */
	public Action getActionSymbolleisteEditorOeffnenDialog()
	{
		return _actionSymbolleisteEditorOeffnenDialog;
	}

	/**
	 * Setzt die Aktion zum ffnen eines Symbolleistenediors mit ffnen-Dialog.
	 * 
	 * @param symbolleisteEditorOeffnenDialog
	 *          Aktion zum ffnen eines Symbolleistenediors mit ffnen-Dialog.
	 */
	public void setActionSymbolleisteEditorOeffnenDialog(
			Action symbolleisteEditorOeffnenDialog)
	{
		_actionSymbolleisteEditorOeffnenDialog = symbolleisteEditorOeffnenDialog;
	}

}
