/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.editoren.dialoge;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert einen Dialog, mit dessen Hilfe sich Speicherort und das
 * Subjekt, fr welches gespeichert werden soll, bestimmen lassen.<br>
 * <br>
 * Mgliche Speicherorte sind:
 * <ul>
 * <li>Benutzereinstellungen Netzweit</li>
 * <li>Benutzereinstellungen Lokal</li>
 * <li>Allgemeine Einstellungen Netzweit</li>
 * <li>Allgemeine Einstellungen Lokal</li>
 * </ul>
 * <br>
 * Mgliche Subjekte sind:
 * <ul>
 * <li>Benutzer angmeldet</li>
 * <li>Benutzer auswlbar</li>
 * <li>Allgemein</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class LadeSpeicherDialogMenu extends Dialog
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private Shell _shell = null; // @jve:decl-index=0:visual-constraint="10,10"

	private Composite _bereichOben = null;

	private Group _gruppeAuswahlLeiste = null;

	private Group _gruppeAuswahlOrt = null;

	private Group _gruppeAuswahlSubjekt = null;

	private Composite _bereichUnten = null;

	private Composite _bereichUntenRechts = null;

	private Button _schaltflaecheOk = null;

	private Button _schaltflaecheAbbrechen = null;

	/**
	 * Hilfe Schaltflche
	 */
	private Button _schaltflaecheHilfe = null;

	private Button _radioButtonBenutzereinstellungenNetzweit = null;

	private Button _radioButtonBenutzereinstellungenLokal = null;

	private Button _radioButtonAllgemeineEinstellungenNetzweit = null;

	private Button _radioButtonAllgemeineEinstellungenLokal = null;

	/**
	 * Subjektauswahl Benutzer angemeldet
	 */
	private Button _radioButtonSubjektBenutzerAngemeldet = null;

	/**
	 * Subjektauswahl Benutzer auswhlen
	 */
	private Button _radioButtonSubjektBenutzerAuswahl = null;

	/**
	 * Subjektauswahl Allgemeine Einstellungen
	 */
	private Button _radioButtonSubjektAllgemein = null;

	/**
	 * Speichert den Ort, der zurckgegeben wird.
	 */
	private EnumSpeicherOrt _rueckgabeOrt = null;

	/**
	 * Zurckzugebendes Subjekt
	 */
	private EnumSubjekt _rueckgabeSubjekt = null;

	/**
	 * Enumerator zur Bestimmung, welche Leisten Radio Buttons dargestellt werden.
	 */
	private EnumLeistenArt _leistenArt = null;

	private Button _radioButtonLeisteMenuLeiste;

	private Button _radioButtonLeisteSymbolLeiste;

	private Button _radioButtonLeisteStatusLeiste;

	public LadeSpeicherDialogMenu(Shell parent)
	{
		super(parent, 0);
	}

	/**
	 * ffnet den Dialog zur Auswahl des Speicherortes (als Instanz des
	 * Enumerators {@link EnumSpeicherOrt}) und des Subjektes (als Instanz des
	 * Enumerators {@link EnumSubjekt}), welche verpackt in einer Instanz der
	 * Klasse {@link SpeicherInfo} zurckgeliefert werden.
	 * 
	 * @param artLadenSpeichern
	 *          Bestimmt an Hand des Enumerators {@link EnumDialogSpeichernLaden}
	 *          ob der Dialog zum Laden oder Speichern verwendet wird.
	 * @param leistenArt
	 *          Bestimmt an Hand des Enumerators {@link EnumLeistenArt}, welche
	 *          Leistenauswahl Radio Buttons dargestellt werden.
	 * 
	 * @return Liefert den selektierten {@link EnumSpeicherOrt} oder
	 *         <code>null</code> zurck, wenn keiner selektiert wurde.
	 */
	public SpeicherInfo oeffne(EnumDialogSpeichernLaden artLadenSpeichern,
			EnumLeistenArt leistenArt)
	{
		SpeicherInfo speicherInfo = null;

		this._leistenArt = leistenArt;

		erzeugeShell();

		switch (artLadenSpeichern)
		{
		case laden:
			_shell.setText("Laden");
			break;
		case speichern:
			_shell.setText("Speichern");
			break;
		default:
			_shell.setText("Laden");
			break;
		}

		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		_shell.open();
		Display display = getParent().getDisplay();
		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		if ((_rueckgabeOrt != null) && (_rueckgabeSubjekt != null))
		{
			speicherInfo = new SpeicherInfo(_leistenArt, _rueckgabeOrt,
					_rueckgabeSubjekt);
		}

		return speicherInfo;
	}

	/**
	 * Initialisiert die Shell des Dialogs
	 */
	private void erzeugeShell()
	{
		_shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		_shell.setText("Laden und Speichern");
		erzeugeBereichOben();
		erzeugeGruppeAuswahlLeiste();
		erzeugeGruppeAuswahlOrt();
		erzeugeGruppeAuswahlSubjekt();
		erzeugeBereichUnten();

		/*
		 * Prfen in Abhngigkeit vom DaV Onlinestatus, was ein/auszublenden ist.
		 */
		einausblenden();

		/*
		 * Vorbelegung der Radio Buttons in der Gruppe Ort
		 */
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			_radioButtonBenutzereinstellungenNetzweit.setSelection(true);
			_rueckgabeOrt = EnumSpeicherOrt.benutzerNetzweit;
			selektiereOrt(_rueckgabeOrt);
		} else
		{
			_radioButtonBenutzereinstellungenLokal.setSelection(true);
			_rueckgabeOrt = EnumSpeicherOrt.benutzerLokal;
			selektiereOrt(_rueckgabeOrt);
		}

		/*
		 * Leisten Art auswerten und die anderen Arten deaktivieren!
		 */
		switch (_leistenArt)
		{
		case menuLeiste:
			_radioButtonLeisteMenuLeiste.setSelection(true);
			_radioButtonLeisteSymbolLeiste.setEnabled(false);
			_radioButtonLeisteStatusLeiste.setEnabled(false);
			break;
		case symbolLeiste:
			_radioButtonLeisteSymbolLeiste.setSelection(true);
			_radioButtonLeisteMenuLeiste.setEnabled(false);
			_radioButtonLeisteStatusLeiste.setEnabled(false);
			break;
		case statusLeiste:
			_radioButtonLeisteStatusLeiste.setSelection(true);
			_radioButtonLeisteMenuLeiste.setEnabled(false);
			_radioButtonLeisteSymbolLeiste.setEnabled(false);
			break;

		default:
			break;
		}

		_shell.setSize(new Point(300, 190));
		_shell.setLayout(new GridLayout());
		_shell.pack(true);
	}

	/**
	 * Erzeugt den Bereich Oben.
	 */
	private void erzeugeBereichOben()
	{
		GridData gridDataOben = new GridData();
		gridDataOben.horizontalAlignment = GridData.FILL;
		gridDataOben.grabExcessHorizontalSpace = true;
		gridDataOben.verticalAlignment = GridData.FILL;
		gridDataOben.grabExcessVerticalSpace = true;
		_bereichOben = new Composite(_shell, SWT.NONE);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;

		_bereichOben.setLayout(gridLayout);
		_bereichOben.setLayoutData(gridDataOben);

	}

	private void erzeugeGruppeAuswahlLeiste()
	{
		GridData gridDataAuswahlLeiste = new GridData();
		gridDataAuswahlLeiste.horizontalAlignment = GridData.FILL;
		gridDataAuswahlLeiste.grabExcessHorizontalSpace = true;
		gridDataAuswahlLeiste.grabExcessVerticalSpace = true;
		gridDataAuswahlLeiste.verticalAlignment = GridData.FILL;

		_gruppeAuswahlLeiste = new Group(_bereichOben, SWT.NONE);
		_gruppeAuswahlLeiste.setLayout(new GridLayout());
		_gruppeAuswahlLeiste.setLayoutData(gridDataAuswahlLeiste);
		_gruppeAuswahlLeiste.setText("Auswahl Leiste:");

		/*
		 * Radio Button MenLeiste
		 */
		_radioButtonLeisteMenuLeiste = new Button(_gruppeAuswahlLeiste, SWT.RADIO);
		_radioButtonLeisteMenuLeiste.setText("Menleiste");
		_radioButtonLeisteMenuLeiste.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (_radioButtonLeisteMenuLeiste.getSelection())
				{
					_leistenArt = EnumLeistenArt.menuLeiste;
				}
			}
		});

		/*
		 * Radio Button SymbolLeiste
		 */
		_radioButtonLeisteSymbolLeiste = new Button(_gruppeAuswahlLeiste, SWT.RADIO);
		_radioButtonLeisteSymbolLeiste.setText("Symbolleiste");
		_radioButtonLeisteSymbolLeiste.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (_radioButtonLeisteSymbolLeiste.getSelection())
				{
					_leistenArt = EnumLeistenArt.symbolLeiste;
				}
			}
		});

		/*
		 * Radio Button StatusLeiste
		 */
		_radioButtonLeisteStatusLeiste = new Button(_gruppeAuswahlLeiste, SWT.RADIO);
		_radioButtonLeisteStatusLeiste.setText("Statusleiste");
		_radioButtonLeisteStatusLeiste.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (_radioButtonLeisteStatusLeiste.getSelection())
				{
					_leistenArt = EnumLeistenArt.statusLeiste;
				}
			}
		});

		if ((_leistenArt.equals(EnumLeistenArt.menuLeiste))
				|| (_leistenArt.equals(EnumLeistenArt.symbolLeiste)))
		{
			_radioButtonLeisteMenuLeiste.setEnabled(true);
			_radioButtonLeisteSymbolLeiste.setEnabled(true);
			_radioButtonLeisteStatusLeiste.setEnabled(false);

			if (_leistenArt.equals(EnumLeistenArt.menuLeiste))
			{
				_radioButtonLeisteMenuLeiste.setSelection(true);
			} else if (_leistenArt.equals(EnumLeistenArt.symbolLeiste))
			{
				_radioButtonLeisteSymbolLeiste.setSelection(true);
			}
		} else if (_leistenArt.equals(EnumLeistenArt.statusLeiste))
		{
			_radioButtonLeisteMenuLeiste.setEnabled(false);
			_radioButtonLeisteSymbolLeiste.setEnabled(false);
			_radioButtonLeisteStatusLeiste.setEnabled(true);
			_radioButtonLeisteStatusLeiste.setSelection(true);
		}

	}

	/**
	 * Erzeugt die Gruppe zur Auswahl von Speicherorten.
	 * 
	 */
	private void erzeugeGruppeAuswahlOrt()
	{
		GridData gridDataAuswahl = new GridData();
		gridDataAuswahl.horizontalAlignment = GridData.FILL;
		gridDataAuswahl.grabExcessHorizontalSpace = true;
		gridDataAuswahl.grabExcessVerticalSpace = true;
		gridDataAuswahl.verticalAlignment = GridData.FILL;
		// gruppeAuswahlOrt = new Group(shell, SWT.NONE);
		_gruppeAuswahlOrt = new Group(_bereichOben, SWT.NONE);
		_gruppeAuswahlOrt.setLayout(new GridLayout());
		_gruppeAuswahlOrt.setLayoutData(gridDataAuswahl);
		_gruppeAuswahlOrt.setText("Auswahl Speicherort:");

		/*
		 * Benutzer Einstellungen Netzweit
		 */
		_radioButtonBenutzereinstellungenNetzweit = new Button(_gruppeAuswahlOrt,
				SWT.RADIO);
		_radioButtonBenutzereinstellungenNetzweit
				.setText("Benutzereinstellungen Netzweit");
		_radioButtonBenutzereinstellungenNetzweit
				.addSelectionListener(new SelectionAdapter()
				{
					public void widgetSelected(SelectionEvent e)
					{
						if (_radioButtonBenutzereinstellungenNetzweit.getSelection())
						{
							_rueckgabeOrt = EnumSpeicherOrt.benutzerNetzweit;
							selektiereOrt(_rueckgabeOrt);
						}
					}
				});

		/*
		 * Benutzer Einstellungen Lokal
		 */
		_radioButtonBenutzereinstellungenLokal = new Button(_gruppeAuswahlOrt,
				SWT.RADIO);
		_radioButtonBenutzereinstellungenLokal
				.setText("Benutzereinstellungen lokal");
		_radioButtonBenutzereinstellungenLokal
				.addSelectionListener(new SelectionAdapter()
				{
					public void widgetSelected(SelectionEvent e)
					{
						if (_radioButtonBenutzereinstellungenLokal.getSelection())
						{
							_rueckgabeOrt = EnumSpeicherOrt.benutzerLokal;
							selektiereOrt(_rueckgabeOrt);
						}
					}
				});

		/*
		 * Allgemeine Einstellungen Netzweit
		 */
		_radioButtonAllgemeineEinstellungenNetzweit = new Button(_gruppeAuswahlOrt,
				SWT.RADIO);
		_radioButtonAllgemeineEinstellungenNetzweit
				.setText("Allgemeine Einstellungen Netzweit");
		_radioButtonAllgemeineEinstellungenNetzweit
				.addSelectionListener(new SelectionAdapter()
				{
					public void widgetSelected(SelectionEvent e)
					{
						if (_radioButtonAllgemeineEinstellungenNetzweit.getSelection())
						{
							_rueckgabeOrt = EnumSpeicherOrt.allgemeinNetzweit;
							selektiereOrt(_rueckgabeOrt);
						}
					}
				});

		/*
		 * Allgemeine Einstellungen Lokal
		 */
		_radioButtonAllgemeineEinstellungenLokal = new Button(_gruppeAuswahlOrt,
				SWT.RADIO);
		_radioButtonAllgemeineEinstellungenLokal
				.setText("Allgemeine Einstellungen Lokal");
		_radioButtonAllgemeineEinstellungenLokal
				.addSelectionListener(new SelectionAdapter()
				{
					public void widgetSelected(SelectionEvent e)
					{
						if (_radioButtonAllgemeineEinstellungenLokal.getSelection())
						{
							_rueckgabeOrt = EnumSpeicherOrt.allgemeinLokal;
							selektiereOrt(_rueckgabeOrt);
						}
					}
				});

	}

	/**
	 * Hilfsfunktion zur Darstellung der korrekten Subjekte zum jeweiligen
	 * {@link EnumSpeicherOrt}.
	 * 
	 * @param ort
	 *          {@link EnumSpeicherOrt}, an Hand dessen die darzustellenden
	 *          Subjekte bestimmt werden.
	 */
	private void selektiereOrt(EnumSpeicherOrt ort)
	{
		if ((ort.equals(EnumSpeicherOrt.benutzerLokal))
				|| (ort.equals(EnumSpeicherOrt.benutzerNetzweit)))
		{
			_radioButtonSubjektAllgemein.setSelection(false);
			_radioButtonSubjektAllgemein.setEnabled(false);
			_radioButtonSubjektBenutzerAngemeldet.setEnabled(true);
			_radioButtonSubjektBenutzerAuswahl.setEnabled(true);

			if ((!_radioButtonSubjektBenutzerAngemeldet.getSelection())
					&& (!_radioButtonSubjektBenutzerAuswahl.getSelection()))
			{
				_radioButtonSubjektBenutzerAngemeldet.setSelection(true);
				_rueckgabeSubjekt = EnumSubjekt.benutzerAngemeldet;
			}
		} else if ((ort.equals(EnumSpeicherOrt.allgemeinLokal))
				|| (ort.equals(EnumSpeicherOrt.allgemeinNetzweit)))
		{
			_debug.fine("=>ALLGEMEIN");
			_radioButtonSubjektBenutzerAngemeldet.setSelection(false);
			_radioButtonSubjektBenutzerAuswahl.setSelection(false);
			_radioButtonSubjektBenutzerAngemeldet.setEnabled(false);
			_radioButtonSubjektBenutzerAuswahl.setEnabled(false);
			_radioButtonSubjektAllgemein.setEnabled(true);
			_radioButtonSubjektAllgemein.setSelection(true);
			_rueckgabeSubjekt = EnumSubjekt.allgemein;

		}

	}

	/**
	 * Erzeugt die Gruppe zur Auswahl von Subjekten
	 */
	private void erzeugeGruppeAuswahlSubjekt()
	{
		GridData gridDataAuswahlSubjekt = new GridData();
		gridDataAuswahlSubjekt.horizontalAlignment = GridData.FILL;
		gridDataAuswahlSubjekt.grabExcessHorizontalSpace = true;
		gridDataAuswahlSubjekt.grabExcessVerticalSpace = true;
		gridDataAuswahlSubjekt.verticalAlignment = GridData.FILL;
		_gruppeAuswahlSubjekt = new Group(_bereichOben, SWT.NONE);
		_gruppeAuswahlSubjekt.setLayout(new GridLayout());
		_gruppeAuswahlSubjekt.setLayoutData(gridDataAuswahlSubjekt);
		_gruppeAuswahlSubjekt.setText("Auswahl Subjekt:");

		/*
		 * Subjekt Benutzer angemeldet
		 */
		_radioButtonSubjektBenutzerAngemeldet = new Button(_gruppeAuswahlSubjekt,
				SWT.RADIO);
		_radioButtonSubjektBenutzerAngemeldet.setText("Benutzer angemeldet");
		_radioButtonSubjektBenutzerAngemeldet
				.addSelectionListener(new SelectionAdapter()
				{
					public void widgetSelected(SelectionEvent e)
					{
						if (_radioButtonSubjektBenutzerAngemeldet.getSelection())
						{
							System.out
									.println("radioButtonSubjektBenutzerAngemeldet selektiert");
							_rueckgabeSubjekt = EnumSubjekt.benutzerAngemeldet;
						}
					}
				});

		/*
		 * Subjekt Benutzer Auswahl
		 */
		_radioButtonSubjektBenutzerAuswahl = new Button(_gruppeAuswahlSubjekt,
				SWT.RADIO);
		_radioButtonSubjektBenutzerAuswahl.setText("Benutzer auswhlen");
		_radioButtonSubjektBenutzerAuswahl
				.addSelectionListener(new SelectionAdapter()
				{
					public void widgetSelected(SelectionEvent e)
					{
						if (_radioButtonSubjektBenutzerAuswahl.getSelection())
						{
							System.out
									.println("radioButtonSubjektBenutzerAuswahl selektiert");
							_rueckgabeSubjekt = EnumSubjekt.benutzerAuswahl;
						}
					}
				});

		/*
		 * Subjekt Allgemein
		 */
		_radioButtonSubjektAllgemein = new Button(_gruppeAuswahlSubjekt, SWT.RADIO);
		_radioButtonSubjektAllgemein.setText("Allgemein");
		_radioButtonSubjektAllgemein.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (_radioButtonSubjektAllgemein.getSelection())
				{
					_debug.fine("radioButtonSubjektAllgemein selektiert");
					_rueckgabeSubjekt = EnumSubjekt.allgemein;
				}
			}
		});

	}

	/**
	 * Erzeugt den Bereich Unten.
	 * 
	 */
	private void erzeugeBereichUnten()
	{
		GridData gridDataUnten = new GridData();
		gridDataUnten.horizontalAlignment = GridData.FILL;
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.verticalAlignment = GridData.CENTER;
		_bereichUnten = new Composite(_shell, SWT.NONE);
		_bereichUnten.setLayout(new GridLayout());
		erzeugeBereichUntenRechts();
		_bereichUnten.setLayoutData(gridDataUnten);
	}

	/**
	 * Erzeugt den Bereich Unten Rechts.
	 * 
	 */
	private void erzeugeBereichUntenRechts()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		GridData gridDataUntenRechts = new GridData();
		gridDataUntenRechts.grabExcessHorizontalSpace = true;
		gridDataUntenRechts.verticalAlignment = GridData.CENTER;
		gridDataUntenRechts.horizontalAlignment = GridData.END;
		_bereichUntenRechts = new Composite(_bereichUnten, SWT.NONE);
		_bereichUntenRechts.setLayoutData(gridDataUntenRechts);
		_bereichUntenRechts.setLayout(rowLayout);

		_schaltflaecheOk = new Button(_bereichUntenRechts, SWT.NONE);
		_schaltflaecheOk.setText("OK");
		_schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}
		});

		_schaltflaecheAbbrechen = new Button(_bereichUntenRechts, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}
		});

		_schaltflaecheHilfe = new Button(_bereichUntenRechts, SWT.PUSH);
		_schaltflaecheHilfe.setText("Hilfe");
		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}
		});
	}

	/**
	 * Ein-/Ausblenden von Optionen in Abhngigkeit vom Onlinestatus.
	 */
	private void einausblenden()
	{
		// DaV-Verbindung aktiv
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			_radioButtonAllgemeineEinstellungenNetzweit.setEnabled(true);
			_radioButtonBenutzereinstellungenNetzweit.setEnabled(true);
		} else
		{
			_radioButtonAllgemeineEinstellungenNetzweit.setEnabled(false);
			_radioButtonBenutzereinstellungenNetzweit.setEnabled(false);
		}
	}

	/**
	 * Aktion hinter dem OK-Button, schliet den Dialog, erfasste Daten werden
	 * durch die ffnen-Methode bergeben.
	 */
	private void aktionOk()
	{
		_shell.dispose();
	}

	/**
	 * Aktion hinter dem Abbrechen-Button, schliet den Dialog und es werden keine
	 * Daten durch die ffnen-Methode bergeben.
	 */
	private void aktionAbbrechen()
	{
		_rueckgabeOrt = null;
		_rueckgabeSubjekt = null;
		_shell.dispose();
	}

	/**
	 * Aktion hinter dem Hilfe-Button, ffnet die Hile zu diesem Dialog.
	 */
	private void aktionHilfe()
	{
		String helpContextId = Activator.PLUGIN_ID + ".menuleisteneditor";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * Zur Bestimmung, ob der Dialog zum Laden oder Speichern verwendet wird.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum EnumDialogSpeichernLaden
	{
		/**
		 * Dialog zum Speichern
		 */
		speichern,

		/**
		 * Dialog zum Laden
		 */
		laden
	}

}
