/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.editoren.dialoge;

/**
 * Diese Klase liefert die Rckgabe der Klasse {@link LadeSpeicherDialogMenu}. Damit lsst sich bestimmen, wo
 * gespeichert wird ({@link EnumSpeicherOrt}) und fr "wen" ({@link EnumSubjekt}) gespeichert wird.
 * 
 * @author HofmannJ
 * 
 */
public class SpeicherInfo
{
  /**
   * Art der Leiste (Symbolleiste oder Statusleiste)
   */
  private EnumLeistenArt _leistenArt = null;

  /**
   * SpeicherOrt
   */
  private EnumSpeicherOrt _speicherOrt = null;

  /**
   * Subjekt
   */
  private EnumSubjekt _subjekt = null;

  /**
   * Konstruktor
   * 
   * @param leistenArt
   *          {@link EnumLeistenArt} - Bestimmt die Art der Leiste (Symbol oder Menleiste)
   * @param speicherOrt
   *          {@link EnumSpeicherOrt} - Ort, wo zu Speichern ist
   * @param subjekt
   *          {@link EnumSubjekt} - Subjekt, fr das zu Speichern ist
   */
  public SpeicherInfo(EnumLeistenArt leistenArt, EnumSpeicherOrt speicherOrt, EnumSubjekt subjekt)
  {
    this._leistenArt = leistenArt;
    this._speicherOrt = speicherOrt;
    this._subjekt = subjekt;
  }

  /**
   * Liefert die Art der Leiste.
   * 
   * @return Art der Leiste.
   */
  public EnumLeistenArt getLeistenArt()
  {
    return _leistenArt;
  }

  /**
   * Liefert den SpeicherOrt.
   * 
   * @return {@link EnumSpeicherOrt}
   */
  public EnumSpeicherOrt getSpeicherOrt()
  {
    return _speicherOrt;
  }

  /**
   * Liefert das Subjekt.
   * 
   * @return {@link EnumSubjekt}
   */
  public EnumSubjekt getSubjekt()
  {
    return _subjekt;
  }

  public String toString()
  {
    String re = "Leisten Art: " + _leistenArt.toString() + " Speicher Ort: " + _speicherOrt.toString() + " Subjekt: "
        + _subjekt.toString();
    return re;
  }

}
