/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.generatoren;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.jdom.Element;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.EreignisMenuSymbolLeisteDatenGeandert;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.IZuhoererMenuSymbolLeisteDatenAenderung;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlBezeichnerMenuSymbol;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse generiert das Men zur Laufzeit des Rahmenwerks.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 07:00:44 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class MenuGenerator
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
 
  /**
   * Menmanager, in dem das Men aufgebaut wird.
   */
  private IMenuManager _menuManager = null;

  /**
   * MenBarDaten-Klasse, die den Aufbau des Mens innerhalb einer XML-Struktur beinhaltet.
   */
  private MenuSymbolLeisteDaten _msld = null;

  /**
   * Speichert Submens fr interne Zwecke.
   */
  private List<MenuManager> _listeSubmenus = new LinkedList<MenuManager>();

  /**
   * Konstruktor. Eine Instanz eines {@link IMenuManager} sowie eine Instanz von {@link MenuSymbolLeisteDaten}, enthlt
   * eine XML-Stuktur die den Aufbau des Mens beschreibt, werden bergeben und das Men baut sich auf.
   * 
   * @param menuManager
   *          Instanz eines {@link IMenuManager}, in dem das Men generiert werden soll.
   * @param mbd
   *          Instanz von {@link MenuSymbolLeisteDaten}, enthlt die XML-Struktur des Mens, an Hand der im
   *          {@link IMenuManager} das Men aufgebaut wird.
   */
  public MenuGenerator(IMenuManager menuManager, MenuSymbolLeisteDaten mbd)
  {
    this._menuManager = menuManager;
    this._msld = mbd;
    init();
  }

  private void init()
  {
    // registrieren der Zuhrer bei MenuBarDaten
    IZuhoererMenuSymbolLeisteDatenAenderung zuhoerer = new IZuhoererMenuSymbolLeisteDatenAenderung() {

      public void ausfuehren(EreignisMenuSymbolLeisteDatenGeandert e)
      {
        generiereMenu();
      }

    };
    _msld.hinzufuegenZuhoerer(zuhoerer);

  }

  public void generiereMenu()
  {
    // Element element = _menuBarData.getWurzel().getChild(_elementMenuName);
    Element basisMenuKnoten = _msld.getWurzel().getChild(XmlBezeichnerMenuSymbol.elementMenu);
    
    if (basisMenuKnoten != null)
    {
      _debug.fine(getClass().getName() + "::generiereMenu() => Begin der Mengenerierung...");

      _menuManager.removeAll();
      generiereMenuRekursiv(_menuManager, basisMenuKnoten);
      // _menuManager.add(generiereEditAction());
      _menuManager.update(true);
      
			/*
			 * Listener darber informieren, dass die Menleiste aktualisiert wurde.
			 */
			MenueLadeInformation.getInstance()
					.benachrichtigeListenerUeberAktualisierungMenueleiste();

      _debug.fine(getClass().getName() + "::generiereMenu() => Ende der Mengenerierung...");

    }
    else
    {
      System.err
          .println(getClass().getName()
              + "::generiereMenu => In der von Ihnen bergebenen MenuBarDaten-Instanz ist kein Knoten \"menu\" vorhanden. Kann das Men nicht generieren.");
    }

  }
 
	private void generiereMenuRekursiv(IMenuManager manager, Element element)
  {
    // Item?
    if (element.getName().equalsIgnoreCase(XmlBezeichnerMenuSymbol.elementItem))
    {
      String tempId = element.getAttributeValue(XmlBezeichnerMenuSymbol.attId);

      IAction tempAction = null;

      // if(_mapActions.containsKey(tempId));
      // {
      // manager.add(_mapActions.get(tempId));
      // }

      ElementRegistrierung elementRegistrierung = ElementRegistrierung.getInstanz();

      // DescriptorAction
      DeskriptorAktion descriptorAction = elementRegistrierung.getDeskriptorAktion(tempId);
      if (descriptorAction != null)
      {
        tempAction = descriptorAction.getAktion();
      }

      // GlobaleDescriptorAction
      DeskriptorAktion globaleDescriptorAction = elementRegistrierung.getDeskriptorAktionGlobal(tempId);
      if (globaleDescriptorAction != null)
      {
        tempAction = globaleDescriptorAction.getAktion();
      }

      if (tempAction != null)
      {
        manager.add(tempAction);
      }
      else
      {
        _debug.fine(getClass().getName()
            + "::generiereMenuRekursiv => Es konnte in der Elementregistrierung keine passende Action mit der ID "
            + tempId + " gefunden werden.");
      }

    }

    // Separator
    if (element.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator))
    {
      Separator separator = new Separator();
      manager.add(separator);
    }

    // Submen?
    if (element.getName().equalsIgnoreCase(XmlBezeichnerMenuSymbol.elementSubMenu))
    {
      // Neues Submen anlegen
      String tempSubmenuName = element.getAttributeValue(XmlBezeichnerMenuSymbol.attName);
      MenuManager tempSubmenu = new MenuManager(tempSubmenuName);
      _listeSubmenus.add(tempSubmenu); // Speichern fr interne Zwecke
      manager.add(tempSubmenu); // Zum Manager hinzufgen

      // Men rausziehen und Rekursion starten
      Element kindMenu = element.getChild(XmlBezeichnerMenuSymbol.elementMenu);
      generiereMenuRekursiv(tempSubmenu, kindMenu);
    }

    // Men?
    if (element.getName().equalsIgnoreCase(XmlBezeichnerMenuSymbol.elementMenu))
    {
    	 @SuppressWarnings("unchecked")
      Iterator it = element.getChildren().iterator();
      while (it.hasNext())
      {
        Element tempElement = (Element)it.next();

        // Item?
        if (tempElement.getName().equalsIgnoreCase(XmlBezeichnerMenuSymbol.elementItem))
        {
          generiereMenuRekursiv(manager, tempElement);
        }

        // Separator?
        if (tempElement.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator))
        {
          generiereMenuRekursiv(manager, tempElement);
        }

        // Submen?
        if (tempElement.getName().equalsIgnoreCase(XmlBezeichnerMenuSymbol.elementSubMenu))
        {
          generiereMenuRekursiv(manager, tempElement);
        }

      }
    }
  }

//  /**
//   * Generiert eine Action, mit der sich die Symbolleisteneditor-View ffnet.
//   * 
//   * @return Eine Action.
//   */
//	private Action generiereEditAction()
//  {
//    Action actionEdit = new Action() {
//      public void run()
//      {
//        if (_msld != null)
//        {
//          try
//          {
//            /**
//             * Perspektive ffnen
//             */
//            IPerspectiveDescriptor perspectiveDescriptor = PlatformUI.getWorkbench().getPerspectiveRegistry()
//                .findPerspectiveWithId(_perspectiveId);
//
//            PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().setPerspective(perspectiveDescriptor);
//
//            /**
//             * EditView initialisieren
//             */
//            PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(_idMenuEditor);
//            SichtMenuSymbolLeisteEditor t = (SichtMenuSymbolLeisteEditor)PlatformUI.getWorkbench()
//                .getActiveWorkbenchWindow().getActivePage().findView(_idMenuEditor);
//            t.externeInitialisierung(_msld);
//
//          }
//          catch (PartInitException e)
//          {
//            e.printStackTrace();
//          }
//        }
//      }
//    };
//
//    actionEdit.setText("Meneditor");
//    actionEdit.setToolTipText("Bearbeitet das Men");
//    ImageDescriptor descriptor = PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
//        ISharedImages.IMG_OBJ_FILE);
//    actionEdit.setImageDescriptor(descriptor);
//
//    return actionEdit;
//  }

}
