/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Speichert selektierte SystemObjekte. <br>
 * Hinweis: Diese Klasse ist als Singleton ausgelegt, damit auch zu einem spteren Zeitpunkt, nach dem schlieen des
 * Dialogs {@link MeldungsanzeigeDialog}, alle bereits empfangenen Betriebsmeldungen wieder angezeigt werden knnen.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:55:46 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class ObjektSpeicher
{
  /**
   * Speichert die einzigste Instanz der Klasse {@link ObjektSpeicher}.
   */
  private static ObjektSpeicher _instanz = null;

  /**
   * Gibt die Maximale Anzahl von Betriebsmeldungen an.
   */
  private int _maximaleAnzahlObjekte = 1000;

  /**
   * Speichert die empfangenen Betriebsmeldungen und hlt diese vorrtig.
   */
  private List<SystemObject> _lSystemObjekte = null;

  /**
   * Liefert die einzigste Instanz der Klasse {@link MeldungenS
   * peicher}.
   * 
   * @return Einzigste Instanz der Klasse {@link ObjektSpeicher}.
   */
  public static ObjektSpeicher getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new ObjektSpeicher();
    }
    return _instanz;
  }

  /**
   * Liefert die einzigste Instanz der Klasse {@link ObjektSpeicher}.
   * 
   * @param maxAnzahlObjekte
   *          Setzt die maximale Anzahl der Objekte, die gespeichert werden. Wird nur bei der ersten
   *          Initialisierung der Singletonklasse gesetzt und ist im Nachhinein (z.B. beim zweiten Aufruf dieser
   *          Methode) nicht vernderbar. Defaultwert == 1000.
   * 
   * @return Einzigste Instanz der Klasse {@link ObjektSpeicher}.
   */
  public static ObjektSpeicher getInstanz(int maxAnzahlBetriebsmeldungen)
  {
    if (_instanz == null)
    {
      _instanz = new ObjektSpeicher(maxAnzahlBetriebsmeldungen);
    }
    return _instanz;
  }

  /**
   * Privater, nicht ffentlicher Konstruktor dieser Singletonklasse.
   */
  private ObjektSpeicher()
  {
    init();
  }

  /**
   * Privater, nicht ffentlicher Konstruktor dieser Singletonklasse.
   * 
   * @param maxAnzahlBetriebsmeldungen
   *          Setzt die maximale Anzahl von Betriebsmeldungen, die gespeichert werden.
   */
  private ObjektSpeicher(int maxAnzahlBetriebsmeldungen)
  {
    _maximaleAnzahlObjekte = maxAnzahlBetriebsmeldungen;
    init();
  }

  /**
   * Fhrt die Initialisierung durch.
   * 
   */
  private void init()
  {
//    _daVKommunikation = DaVKommunikation.getInstanz();

    _lSystemObjekte = new LinkedList<SystemObject>();

//    IBetriebsmeldungListener betriebsmeldungListener = new IBetriebsmeldungListener() {
//
//      public void betriebsmeldungEingang(BetriebsmeldungEreignis e)
//      {
//        /*
//         * Abarbeiten der eingehenden Meldung.
//         */
//        List<Meldung> meldungen = e.getBetriebsmeldungen();
//        for (Meldung meldung : meldungen)
//        {
//          hinzufuegenBetriebsmeldung(meldung);
//
//        }
//
//      }
//
//    };

//    _daVKommunikation.hinzufuegenBetriebsmeldungsListener(betriebsmeldungListener);

  }

  /**
   * Fgt der Liste der Betriebsmeldungen eine neue Betriebsmeldung am Anfang (Index 0) hinzu. Nachfolgende, "ltere"
   * Elemente werden um eine Position nach hinten verschoben. <br>
   * bersteigt beim Hinzufgen einer Betriebsmeldung die Anzahl der Betriebsmeldungen die maximale Anzahl von
   * Betriebsmeldungen, so wird die jeweils lteste (am frhsten hinzugefgte) Betriebsmeldung gelscht.
   * 
   * @param betriebsmeldung
   *          Betriebsmeldung (Instanz der Klasse {@link Meldung}), die hinzugefgt werden soll.
   */
  public void hinzufuegenSystemObjekt(SystemObject objekt)
  {
    _lSystemObjekte.add(0, objekt);

    /*
     * Prfen, ob die Anzahl der Betriebsmeldungen die Anzahl der maximal zulssigen Betriebsmeldungen bersteigt.
     */
    if (_lSystemObjekte.size() > _maximaleAnzahlObjekte)
    {
      removeAeltesteBetriebsmeldung();
    }
  }

  /**
   * Entfernt die lteste Betriebsmeldung in der Liste der Betriebstmeldungen.
   * 
   */
  private void removeAeltesteBetriebsmeldung()
  {
    int letzteElementPosition = _lSystemObjekte.size() - 1;
    _lSystemObjekte.remove(letzteElementPosition);

  }

  /**
   * Liefert die Liste mit allen bisher empfangenen Betriebsmeldungen. Fr eine erneute Darstellung muss lediglich die
   * Tabelle der Betriebsmeldungen in {@link MeldungsanzeigeDialog} mit dieser Liste neu aufgebaut werden.
   * 
   * @return Liste mit allen empfangenen Betriebsmeldungen.
   */
  public List<SystemObject> getSystemObjekte()
  {
    return _lSystemObjekte;
  }

  public void setSystemObjekte(List<SystemObject> objekte)
  {
    _lSystemObjekte.clear();
    
    for (SystemObject so : objekte)
    {
      hinzufuegenSystemObjekt(so);
    }
    
//    _lSystemObjekte = objekte;
    
  }

}
