/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.EventObject;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert den Event fr die Objektzusammenstellung.
 * 
 * @author HofmannJ
 * 
 */
public class ObjektzusammenstellungEreignis extends EventObject
{

  /**
   * svUID
   */
  private static final long serialVersionUID = -3373524711596220320L;

  /**
   * Als Source wird eine Liste von {@link SystemObject}s bergeben.
   * 
   * @param source
   *          Liste von {@link SystemObject}s.
   */
  public ObjektzusammenstellungEreignis(Object source)
  {
    super(source);

  }

  /**
   * Liefert die SystemObjekte des Events in einer Liste oder <code>null</code>, wenn die Liste keine Elemente
   * enthlt.
   * 
   * @return Liste von SystemObjekten oder <code>null</code>.
   */  
	@SuppressWarnings("unchecked")
	public List<SystemObject> getSystemObjekte()
  {
    List<SystemObject> returnListe = new LinkedList<SystemObject>();

    if (super.source instanceof List)
    {
      List tempListe = (List)super.source;

      Iterator it = tempListe.iterator();
      while (it.hasNext())
      {
        Object object = it.next();
        if (object instanceof SystemObject)
        {
          returnListe.add((SystemObject)object);
        }
      }

    }

    if (returnListe.size() > 0)
    {
      return returnListe;
    }
    else
    {
      return null;
    }

  }

}
