/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class ObjektzusammenstellungFallAdapter extends ViewerDropAdapter
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Liste der SystemObjekte
   */
  private List<SystemObject> _listeObjekte = null;

  /**
   * Verweis auf den TableViewer
   */
  @SuppressWarnings("unused")
	private TableViewer _tableViewer = null;

  /**
   * Ziel
   */
  private Object _ziel = null;

  /**
   * Syso Flag
   */
  private boolean _zielIstSystemObjekt = true;

  private SpeicherTransfer _speicherTransfer;

  /**
   * Initialisierung des FallAdapters
   * 
   * @param viewer
   *          Viewer, auf den sich der Falladapter bezieht.
   * @param listeObjekte
   *          Liste der Systemobjekte, mit denen der Viewer gefttert wird.
   */
  protected ObjektzusammenstellungFallAdapter(Viewer viewer, List<SystemObject> listeObjekte)
  {
    super(viewer);
    this._listeObjekte = listeObjekte;

    if (viewer instanceof TableViewer)
    {
      this._tableViewer = (TableViewer)viewer;
    }

  }

  @Override
  public boolean performDrop(Object data)
  {
    _debug.fine(getClass().getName() + "::performDrop \n\t=>TYP: " + data.getClass() + "\n");

    if (data instanceof String)
    {
      String tempString = (String)data;

      // Prfen auf [PID] am Anfang des Strings:
      if (tempString.length() > 5)
      {
        String vergleich = tempString.substring(0, 5);

        if (vergleich.equals("[PID]"))
        {

          String tempPid = tempString.substring(5);
          _debug.fine("tempPid: " + tempPid);

          _speicherTransfer = SpeicherTransfer.getInstanz();
          if (_speicherTransfer.getPid() != null)
          {
            if (_speicherTransfer.getPid().equals(tempPid))
            {
              _debug.fine(getClass().getName() + "::performDrop \n\t=>Target: " + _ziel.getClass().getName() + "\n");

              int neuePosition = positionObjektInListe(_ziel, _listeObjekte);
              _speicherTransfer.setNeuePosition(neuePosition);
              int altePosition = _speicherTransfer.getAltePosition();

              _debug.fine(getClass().getName() + "::performDrop \n\t=> POS - ALT: " + altePosition + " NEU: "
                  + neuePosition + "\n");

              if ((neuePosition > -1) && (altePosition > -1))
              {
                _listeObjekte.remove(altePosition);

                if (neuePosition < altePosition)
                {
                  _debug.fine(getClass().getName() + "::performDrop \n\t=> neuePosition < altePosition");

                  _listeObjekte.add(neuePosition, _speicherTransfer.getSystemObjekt());
                }
                else if (neuePosition > altePosition)
                {
                  _debug.fine(getClass().getName() + "::performDrop \n\t=> neuePosition > altePosition");

                  _listeObjekte.add(neuePosition--, _speicherTransfer.getSystemObjekt());
                }
              }
              else if ((altePosition == -1) && (neuePosition > -1))
              {
                _debug.fine(getClass().getName() + "::PerformDrop \n\t=> (altePosition == -1 ) && (neuePosition > -1)");

                _listeObjekte.add(neuePosition, _speicherTransfer.getSystemObjekt());
              }
              else if (neuePosition < 0)
              {
                _debug.fine(getClass().getName() + "::PerformDrop \n\t=> neuePosition < 0");

                _listeObjekte.add(_speicherTransfer.getSystemObjekt());
              }
              else if (!_zielIstSystemObjekt)
              {
                _debug.fine(getClass().getName() + "::target \n\t=> Ziel ist kein SystemObjekt. Anhngen an Tabelle.");

                if (altePosition > -1)
                {
                  _listeObjekte.remove(altePosition);
                }
                _listeObjekte.add(_speicherTransfer.getSystemObjekt());
              }
              else
              {
                _debug.fine(getClass().getName() + "::target \n\t=> Sonst...");

                _listeObjekte.add(_speicherTransfer.getSystemObjekt());
              }

              // _listeObjekte.add(speicher.getSystemObject());
              _speicherTransfer.zuruecksetzenSpeicher();

              getViewer().setInput(_listeObjekte);
            }
            else
            {
              _debug.info(getClass().getName() + "::performDrop \n\t=> Die Pid des in "
                  + _speicherTransfer.getClass().getName() + "hinterlegten SystemObjekt(PID="
                  + _speicherTransfer.getPid() + ") entspricht nicht der Pid des zu droppenden SystemObjekts(PID="
                  + tempPid + "). Drop Abgebrochen.\n");
            }

          }
          else
          {
            _debug.info(getClass().getName() + "::performDrop \n\t=> In " + _speicherTransfer.getClass().getName()
                + " ist KEIN SystemObjekt hinterlegt. Drop Abgebrochen.\n");
          }

        }
        else
        {
          _debug.info(getClass().getName() + "::performDrop \n\t=> KEIN SYSTEMOBJEKT\n");
        }
      }

    }

    return false;
  }

  @Override
  public boolean validateDrop(Object target, int operation, TransferData transferType)
  {

    this._ziel = target;

    if (target instanceof SystemObject)
    {
      // if(TextTransfer.getInstance().isSupportedType(transferType))
      // {
      // return true;
      // }
      // else
      // {
      // return false;
      // }
      _debug.fine("SYSO");
      _zielIstSystemObjekt = true;
      return true;
    }
    else
    {
      _debug.fine("KEINSYSO");
      _zielIstSystemObjekt = false;
      return true;
    }

  }

  @SuppressWarnings("unchecked")
	private int positionObjektInListe(Object object, List liste)
  {
    int i = -1;

    Iterator it = liste.iterator();
    while (it.hasNext())
    {
      i++;
      if (it.next().equals(object))
      {
        break;
      }
    }  

    return i;
  }

}
