/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class ZugZuhoererSystemObjekt extends DragSourceAdapter
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Viewer, auf dem gearbeitet wird.
   */
  private Viewer _tableViewer;

  /**
   * Liste von Systemobjekten
   */
  private List<SystemObject> _listeDaten;

  /**
   * Hlt die Singleton-Instanz der Klasse SpeicherTransfer vor.
   */
  private SpeicherTransfer _speicherTransfer;

  /**
   * Konsturktor
   * 
   * @param listViewer
   * @param listeDaten
   */
  public ZugZuhoererSystemObjekt(Viewer tabelle, List<SystemObject> listeDaten)
  {
    this._tableViewer = tabelle;
    this._listeDaten = listeDaten;
  }

  public void dragStart(DragSourceEvent event)
  {
    _debug.fine(getClass().getName() + "::dragStart => Start");

    event.doit = !_tableViewer.getSelection().isEmpty();
  }

  public void dragFinished(DragSourceEvent event)
  {
    if (!event.doit)
      return;
    // if the gadget was moved, remove it from the source viewer
    // if (event.detail == DND.DROP_MOVE) {
    // IStructuredSelection selection =
    // (IStructuredSelection)_viewer.getSelection();
    // for (Iterator it = selection.iterator(); it.hasNext();) {
    // //Entfernen der Elemente
    // }
    // _viewer.refresh();
    // }
    if (event.detail == DND.DROP_MOVE)
    {
      System.out.println(getClass().getName() + " => Lschen.");
      // IStructuredSelection selection
      // =(IStructuredSelection)_viewer.getSelection();
      // Iterator und aktualisieren der Elemente
      _tableViewer.refresh();
    }

    _debug.fine(getClass().getName() + " => dragFinished");

  }

  public void dragSetData(DragSourceEvent event)
  {
    _debug.fine(getClass().getName() + "::dragSetData => begin.");

    IStructuredSelection selection = (IStructuredSelection)_tableViewer.getSelection();

    // SpeicherDNDTransfer.getInstanz().setAltePosition(_tableViewer.getList().getSelectionIndex());

    Object selectedObject = selection.getFirstElement();
    _speicherTransfer = SpeicherTransfer.getInstanz();
    _speicherTransfer.setAltePosition(positionObjektInListe(selectedObject, _listeDaten));
    
    // SystemObject?
    if (selectedObject instanceof SystemObject)
    {
      SystemObject systemObject = (SystemObject)selectedObject;
      _speicherTransfer.setSystemObjekt(systemObject);
      String daten = "[PID]" + systemObject.getPidOrNameOrId();
      event.data = daten;
    }
   

  }

  @SuppressWarnings("unchecked")
	private int positionObjektInListe(Object object, List liste)
  {
    int i = -1;
    Iterator it = liste.iterator();
    while (it.hasNext())
    {
      i++;
      if (it.next().equals(object))
      {
        break;
      }
    }

    return i;
  }

}
