/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.LinePrint;
import net.sf.paperclips.PageDecoration;
import net.sf.paperclips.PageNumber;
import net.sf.paperclips.PageNumberFormat;
import net.sf.paperclips.PageNumberPageDecoration;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.PaperClips;
import net.sf.paperclips.Print;
import net.sf.paperclips.PrintJob;
import net.sf.paperclips.SimplePageDecoration;
import net.sf.paperclips.TextPrint;
import net.sf.paperclips.ui.PrintPreview;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.printing.PrintDialog;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.DefsSeiteneinstellungen;

public class DruckvorschauDialog extends Dialog
{
	private final Display display;

	private PrintJob _printJob;

	private Shell shell;

	private Button _schaltflaecheVorhergehendeSeite;

	private Label _labelSeitenNummer;

	private Button _schaltflaecheNachfolgendeSeite;

	private ScrolledComposite _verschiebbaresComposite;

	private PrintPreview _druckVorschau;

	private double[] _verschiebePosition;

	/**
	 * Flag, zeigt an ob gedruckt wurde und wird von der ffnen-Methode als
	 * Ergebnis zurckgeliefert.
	 */
	private boolean _gedruckt = false;

	/**
	 * Konstruktor. bernimmt eine Eltern-{@link Shell} als Parameter.
	 * 
	 * @param parent
	 *          Eltern-{@link Shell}
	 */
	public DruckvorschauDialog(Shell parent)
	{
		super(parent, 0);

		this.display = Display.getDefault();
	}

	public boolean oeffne(PrintJob printJob)
	{
		if (printJob == null)
		{
			_printJob = new PrintJob("Druckvorschau Test", createPrint2())
					.setMargins(72);
		}
		else
		{
			_printJob = printJob;
		}

		erzeugeShell();

		shell.open();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _gedruckt;
	}

	/**
	 * ffnet den Druckvorschaudialog.
	 * 
	 * @param page
	 *          Zu druckende Seite.
	 * @param Druckauftrag
	 *          Titel des Druckauftrags.
	 * @return Liefert <code>true</code>, wenn die Seite gedruckt wurde, sonst
	 *         <code>false</code>.
	 */
	public boolean oeffne(PagePrint page, String Druckauftrag)
	{
		if (page != null)
		{
			PagePrint erweitertePage = erweiterePage(page, Druckauftrag);

			_printJob = new PrintJob(Druckauftrag, erweitertePage).setMargins(72);

		}

		erzeugeShell();

		shell.open();

		Display display = getParent().getDisplay();
		while (!shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _gedruckt;
	}

	public Shell erzeugeShell()
	{

		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		shell.setText("Druckvorschau");
		shell.setBounds(100, 100, 800, 600);
		shell.setLayout(new GridLayout(1, false));

		erzeugeSchaltflaechenBereich(shell).setLayoutData(
				new GridData(SWT.FILL, SWT.FILL, true, false));
		erzeugeVerschiebbareVorschau(shell).setLayoutData(
				new GridData(SWT.FILL, SWT.FILL, true, true));

		_druckVorschau.setLazyPageLayout(true);
		display.timerExec(10, new Runnable()
		{
			public void run()
			{
				_druckVorschau.setPrintJob(_printJob);
				aktualisiereVorschauGroesse();
				aktualisiereSeitenNummer();
			}
		});

		shell.setVisible(true);

		return shell;
	}

	private Control erzeugeSchaltflaechenBereich(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout(16, false);
		layout.marginWidth = layout.marginHeight = 0;
		composite.setLayout(layout);

		_schaltflaecheVorhergehendeSeite = erzeugeTextSchaltflaeche(composite,
				"<-", "Schaltet zur vorhergehenden Seite um", new Listener()
				{

					@Override
					public void handleEvent(Event event)
					{
						setVorschauSeitenIndex(_druckVorschau.getPageIndex()
								- _druckVorschau.getHorizontalPageCount()
								* _druckVorschau.getVerticalPageCount());

					}
				});

		_labelSeitenNummer = new Label(composite, SWT.NONE);

		_schaltflaecheNachfolgendeSeite = erzeugeTextSchaltflaeche(composite, "->",
				"Schaltet auf die nachfolgende Seite um", new Listener()
				{
					public void handleEvent(Event event)
					{
						aktionSeiteVor();
					}
				});

		erzeugeTextSchaltflaeche(composite, "Horiz. fllen",
				"Fllt die Darstellung horizontal", new Listener()
				{
					public void handleEvent(Event event)
					{
						_druckVorschau.setFitHorizontal(true);
						_druckVorschau.setFitVertical(false);
						sichereVerschiebePosition();
						aktualisiereVorschauGroesse();
						wiederherstellenVerschiebePosition();
					}
				});

		erzeugeTextSchaltflaeche(composite, "Vert. fllen",
				"Fllt die Darstellung vertikal", new Listener()
				{
					public void handleEvent(Event event)
					{
						_druckVorschau.setFitVertical(true);
						_druckVorschau.setFitHorizontal(false);
						sichereVerschiebePosition();
						aktualisiereVorschauGroesse();
						wiederherstellenVerschiebePosition();
					}
				});

		erzeugeTextSchaltflaeche(composite, "Opt. fllen",
				"Fllt die Darstellung optimal", new Listener()
				{
					public void handleEvent(Event event)
					{
						_druckVorschau.setFitVertical(true);
						_druckVorschau.setFitHorizontal(true);
						sichereVerschiebePosition();
						aktualisiereVorschauGroesse();
						wiederherstellenVerschiebePosition();
					}
				});

		erzeugeTextSchaltflaeche(composite, "+", "Hinein zoomen", new Listener()
		{
			public void handleEvent(Event event)
			{
				setVorschauSkala(_druckVorschau.getAbsoluteScale() * 1.1f);
			}
		});

		erzeugeTextSchaltflaeche(composite, "-", "Heraus zoomen", new Listener()
		{
			public void handleEvent(Event event)
			{
				setVorschauSkala(_druckVorschau.getAbsoluteScale() / 1.1f);
			}
		});

		erzeugeTextSchaltflaeche(composite, "1:1", "An Skalierung heranzoomen",
				new Listener()
				{
					public void handleEvent(Event event)
					{
						setVorschauSkala(1);
					}
				});

		erzeugeTextSchaltflaeche(composite, "Hoch", "Hochformat", new Listener()
		{
			public void handleEvent(Event event)
			{
				_printJob.setOrientation(PaperClips.ORIENTATION_PORTRAIT);
				_druckVorschau.setPrintJob(_printJob);

				loescheVerschiebePosition();
				aktualisiereVorschauGroesse();
				aktualisiereSeitenNummer();
			}
		});

		erzeugeTextSchaltflaeche(composite, "Quer", "Querformat", new Listener()
		{
			public void handleEvent(Event event)
			{
				_printJob.setOrientation(PaperClips.ORIENTATION_LANDSCAPE);
				_druckVorschau.setPrintJob(_printJob);

				loescheVerschiebePosition();
				aktualisiereVorschauGroesse();
				aktualisiereSeitenNummer();
			}
		});

		erzeugeGrafischeSchaltflaeche(composite, "print.gif", "Drucken",
				new Listener()
				{
					public void handleEvent(Event event)
					{
						PrintDialog dialog = new PrintDialog(shell, SWT.NONE);
						PrinterData printerData = dialog.open();
						if (printerData != null)
						{
							PaperClips.print(_printJob, printerData);
							_druckVorschau.setPrinterData(printerData);
						}
					}
				});

		erzeugeLabel(composite, "Horz. Seiten:");
		erzeugeSeitenzaehlerSpinner(composite, new Listener()
		{
			public void handleEvent(Event event)
			{
				_druckVorschau.setHorizontalPageCount(((Spinner) event.widget)
						.getSelection());
				loescheVerschiebePosition();
				aktualisiereVorschauGroesse();
				aktualisiereSeitenNummer();
			}
		});

		erzeugeLabel(composite, "Vert. Seiten:");
		erzeugeSeitenzaehlerSpinner(composite, new Listener()
		{
			public void handleEvent(Event event)
			{
				_druckVorschau.setVerticalPageCount(((Spinner) event.widget)
						.getSelection());
				loescheVerschiebePosition();
				aktualisiereVorschauGroesse();
				aktualisiereSeitenNummer();
			}
		});

		return composite;
	}

	private void aktionSeiteVor()
	{
		setVorschauSeitenIndex(_druckVorschau.getPageIndex()
				+ _druckVorschau.getHorizontalPageCount()
				* _druckVorschau.getVerticalPageCount());
	}

	private Control erzeugeVerschiebbareVorschau(Composite parent)
	{
		_verschiebbaresComposite = new ScrolledComposite(parent, SWT.BORDER
				| SWT.H_SCROLL | SWT.V_SCROLL);
		_verschiebbaresComposite.setExpandHorizontal(true);
		_verschiebbaresComposite.setExpandVertical(true);

		_druckVorschau = new PrintPreview(_verschiebbaresComposite, SWT.NONE);
		_verschiebbaresComposite.setContent(_druckVorschau);

		_verschiebbaresComposite.addListener(SWT.Resize, new Listener()
		{
			public void handleEvent(Event event)
			{
				Rectangle bounds = _verschiebbaresComposite.getClientArea();

				_verschiebbaresComposite.getHorizontalBar().setPageIncrement(
						bounds.width * 2 / 3);
				_verschiebbaresComposite.getVerticalBar().setPageIncrement(
						bounds.height * 2 / 3);

				if (_druckVorschau.isFitHorizontal() ^ _druckVorschau.isFitVertical())
				{
					sichereVerschiebePosition();
					aktualisiereVorschauGroesse();
					wiederherstellenVerschiebePosition();
				}
			}
		});

		_druckVorschau.setFitVertical(true);
		_druckVorschau.setFitHorizontal(true);

		Listener dragListener = new Listener()
		{
			private final Point dpi = display.getDPI();

			private boolean scrollable = false;

			private boolean dragging = false;

			private Point dragStartScrollOrigin = null;

			private Point dragStartMouseAnchor = null;

			public void handleEvent(Event event)
			{
				switch (event.type)
				{
				case SWT.Resize:
					loescheVerschiebePosition();
					Rectangle bounds = _verschiebbaresComposite.getClientArea();
					Point size = _druckVorschau.getSize();
					scrollable = size.x > bounds.width || size.y > bounds.height;
					if (!scrollable && dragging)
						endDragging();
					break;
				case SWT.MouseDown:
					loescheVerschiebePosition();
					if (scrollable && event.button == 1)
						beginDragging(event);
					break;
				case SWT.MouseMove:
					if (dragging)
					{
						loescheVerschiebePosition();
						Point point = _druckVorschau.toDisplay(event.x, event.y);
						_verschiebbaresComposite.setOrigin(dragStartScrollOrigin.x
								+ dragStartMouseAnchor.x - point.x, dragStartScrollOrigin.y
								+ dragStartMouseAnchor.y - point.y);
					}
					break;
				case SWT.MouseUp:
					loescheVerschiebePosition();
					if (dragging)
						endDragging();
					break;
				case SWT.MouseEnter:
					display.addFilter(SWT.MouseWheel, this);
					break;
				case SWT.MouseWheel:
					if (event.count != 0)
					{
						if (scrollable
								&& !dragging
								&& (event.stateMask == SWT.NONE || event.stateMask == SWT.SHIFT))
						{
							loescheVerschiebePosition();
							bounds = _verschiebbaresComposite.getClientArea();
							size = _druckVorschau.getSize();
							Point origin = _verschiebbaresComposite.getOrigin();
							int direction = event.count > 0 ? -1 : 1;
							// Prefer vertical scrolling unless user is pressing Shift
							if (size.y > bounds.height && event.stateMask == SWT.NONE)
								origin.y += direction * Math.min(dpi.y, bounds.height / 4);
							else if (size.x > bounds.width)
								origin.x += direction * Math.min(dpi.x, bounds.width / 4);
							_verschiebbaresComposite.setOrigin(origin);
							event.doit = false;
						}
						else if (event.stateMask == SWT.CTRL)
						{ // Ctrl+MouseWheel -> zoom
							float scale = _druckVorschau.getAbsoluteScale();
							setVorschauSkala(event.count > 0 ? scale / 1.1f : scale * 1.1f);
						}
					}
					break;
				case SWT.MouseExit:
					display.removeFilter(SWT.MouseWheel, this);
					break;
				}
			}

			private void beginDragging(Event event)
			{
				dragStartScrollOrigin = _verschiebbaresComposite.getOrigin();
				dragStartMouseAnchor = _druckVorschau.toDisplay(event.x, event.y);
				dragging = true;
			}

			private void endDragging()
			{
				dragging = false;
				dragStartMouseAnchor = null;
				dragStartScrollOrigin = null;
			}
		};

		_verschiebbaresComposite.addListener(SWT.Resize, dragListener);
		_druckVorschau.addListener(SWT.MouseDown, dragListener);
		_druckVorschau.addListener(SWT.MouseMove, dragListener);
		_druckVorschau.addListener(SWT.MouseUp, dragListener);

		// These are for mouse wheel handling
		_druckVorschau.addListener(SWT.MouseEnter, dragListener);
		_druckVorschau.addListener(SWT.MouseExit, dragListener);

		return _verschiebbaresComposite;
	}

	private Button erzeugeGrafischeSchaltflaeche(Composite parent,
			String imageFilename, String toolTipText, Listener selectionListener)
	{
		Button button = erzeugeSchaltflaeche(parent, toolTipText, selectionListener);
		button.setImage(erzeugeImage(imageFilename));
		return button;
	}

	private Button erzeugeTextSchaltflaeche(Composite parent, String text,
			String toolTipText, Listener selectionListener)
	{
		Button button = erzeugeSchaltflaeche(parent, toolTipText, selectionListener);
		button.setText(text);
		return button;
	}

	private Button erzeugeSchaltflaeche(Composite parent, String toolTipText,
			Listener selectionListener)
	{
		Button button = new Button(parent, SWT.PUSH);
		button.setToolTipText(toolTipText);
		button.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		button.addListener(SWT.Selection, selectionListener);
		return button;
	}

	private Spinner erzeugeSeitenzaehlerSpinner(Composite parent,
			Listener selectionListener)
	{
		Spinner spinner = new Spinner(parent, SWT.BORDER);
		spinner.setMinimum(1);
		spinner.setMaximum(99);
		spinner.addListener(SWT.Selection, selectionListener);
		return spinner;
	}

	private void erzeugeLabel(Composite parent, String text)
	{
		new Label(parent, SWT.NONE).setText(text);
	}

	private Image erzeugeImage(String filename)
	{
		final Image image = new Image(display, getClass().getResourceAsStream(
				filename));

		shell.addListener(SWT.Dispose, new Listener()
		{
			public void handleEvent(Event event)
			{
				image.dispose();
			}
		});

		return image;
	}

	private void aktualisiereSeitenNummer()
	{
		int pageIndex = _druckVorschau.getPageIndex();
		int pageCount = _druckVorschau.getPageCount();
		int visiblePages = _druckVorschau.getHorizontalPageCount()
				* _druckVorschau.getVerticalPageCount();
		boolean layoutComplete = _druckVorschau.isPageLayoutComplete();
		String text = (visiblePages > 1 ? "Seiten " + (pageIndex + 1) + "-"
				+ Math.min(pageCount, pageIndex + visiblePages) : "Seite "
				+ (pageIndex + 1));
		if (layoutComplete)
			text += " von " + pageCount;
		_labelSeitenNummer.setText(text);
		_schaltflaecheVorhergehendeSeite.setEnabled(pageIndex > 0);
		_schaltflaecheNachfolgendeSeite.setEnabled(pageIndex < pageCount
				- visiblePages
				|| !layoutComplete);
		shell.layout(new Control[] { _labelSeitenNummer });
	}

	private void sichereVerschiebePosition()
	{
		Point size = _druckVorschau.getSize();
		if (size.x == 0 || size.y == 0)
		{
			loescheVerschiebePosition();
		}
		else if (_verschiebePosition == null)
		{
			Point origin = _verschiebbaresComposite.getOrigin();
			_verschiebePosition = new double[] { (double) origin.x / (double) size.x,
					(double) origin.y / (double) size.y };
		}
	}

	private void loescheVerschiebePosition()
	{
		_verschiebePosition = null;
	}

	private void wiederherstellenVerschiebePosition()
	{
		if (_verschiebePosition != null)
		{
			Point size = _druckVorschau.getSize();
			_verschiebbaresComposite.setOrigin((int) Math
					.round(_verschiebePosition[0] * size.x), (int) Math
					.round(_verschiebePosition[1] * size.y));
		}
	}

	private void aktualisiereVorschauGroesse()
	{
		Point minSize;
		Rectangle bounds = _verschiebbaresComposite.getClientArea();
		if (_druckVorschau.isFitHorizontal())
		{
			if (_druckVorschau.isFitVertical())
				minSize = new Point(0, 0); // Best fit
			else
				minSize = new Point(0, _druckVorschau.computeSize(bounds.width,
						SWT.DEFAULT).y); // Fit
			// to
			// width
		}
		else
		{
			if (_druckVorschau.isFitVertical())
				minSize = new Point(_druckVorschau.computeSize(SWT.DEFAULT,
						bounds.height).x, 0); // Fit to height
			else
				minSize = _druckVorschau.computeSize(SWT.DEFAULT, SWT.DEFAULT); // Custom
			// scale
		}
		_verschiebbaresComposite.setMinSize(minSize);
	}

	private void setVorschauSkala(float scale)
	{
		_druckVorschau.setFitVertical(false);
		_druckVorschau.setFitHorizontal(false);
		_druckVorschau.setScale(scale);
		sichereVerschiebePosition();
		aktualisiereVorschauGroesse();
		wiederherstellenVerschiebePosition();
	}

	private void setVorschauSeitenIndex(int pageIndex)
	{
		if (_druckVorschau.isPageLayoutComplete())
			pageIndex = Math.min(pageIndex, _druckVorschau.getPageCount()
					- _druckVorschau.getHorizontalPageCount()
					* _druckVorschau.getVerticalPageCount());
		pageIndex = Math.max(pageIndex, 0);
		_druckVorschau.setPageIndex(pageIndex);
		aktualisiereSeitenNummer();
	}

	public static Print createPrint()
	{
		DefaultGridLook look = new DefaultGridLook();
		look.setCellSpacing(5, 2);

		GridPrint grid = new GridPrint("p:g, p:g, p:g, p:g, p:g", look);

		String text = "The quick brown fox jumps over the lazy dog.";
		for (int i = 0; i < 100; i++)
			grid.add(new TextPrint(text));

		PagePrint page = new PagePrint(grid);
		page.setHeader(new SimplePageDecoration(new TextPrint("Vorschautest",
				SWT.CENTER)));
		page.setHeaderGap(5);
		page.setFooterGap(5);
		PageNumberPageDecoration footer = new PageNumberPageDecoration(SWT.CENTER);
		footer.setFormat(new PageNumberFormat()
		{
			public String format(PageNumber pageNumber)
			{
				return "Seite " + (pageNumber.getPageNumber() + 1);
			}
		});
		page.setFooter(footer);

		return page;
	}

	public Print createPrint2()
	{
		DefaultGridLook look = new DefaultGridLook();
		look.setCellSpacing(5, 2);

		GridPrint grid = new GridPrint("p:g, p:g, p:g, p:g, p:g", look);

		String text = "The quick brown fox jumps over the lazy dog.";
		for (int i = 0; i < 100; i++)
			grid.add(new TextPrint(text));

		PagePrint page = new PagePrint(grid);

		IPreferenceStore preferenceStore = Activator.getDefault()
				.getPreferenceStore();
		// preferenceStore.getString(DefsSeiteneinstellungen.fusszeileLinks);

		// page.setHeader(new SeitenDekoration("A [Datum]", "B [Uhrzeit]", "C",
		// true));
		page.setHeader(new SeitenDekoration(preferenceStore
				.getString(DefsSeiteneinstellungen.kopfzeileLinks), preferenceStore
				.getString(DefsSeiteneinstellungen.kopfzeileMitte), preferenceStore
				.getString(DefsSeiteneinstellungen.kopfzeileRechts), true));

		page.setHeaderGap(5);
		// page.setFooter(new SeitenDekoration("D", "E", "F [Seite]/[Seitenanzahl]",
		// false));
		page.setFooter(new SeitenDekoration(preferenceStore
				.getString(DefsSeiteneinstellungen.fusszeileLinks), preferenceStore
				.getString(DefsSeiteneinstellungen.fusszeileMitte), preferenceStore
				.getString(DefsSeiteneinstellungen.fusszeileRechts), false));
		page.setFooterGap(5);

		return page;
	}

	private PagePrint erweiterePage(PagePrint page, String titel)
	{
		IPreferenceStore preferenceStore = Activator.getDefault()
				.getPreferenceStore();
		// preferenceStore.getString(DefsSeiteneinstellungen.fusszeileLinks);

		// page.setHeader(new SeitenDekoration("A [Datum]", "B [Uhrzeit]", "C",
		// true));
		page.setHeader(new SeitenDekoration(preferenceStore
				.getString(DefsSeiteneinstellungen.kopfzeileLinks), preferenceStore
				.getString(DefsSeiteneinstellungen.kopfzeileMitte), preferenceStore
				.getString(DefsSeiteneinstellungen.kopfzeileRechts), true, titel));

		page.setHeaderGap(5);
		// page.setFooter(new SeitenDekoration("D", "E", "F [Seite]/[Seitenanzahl]",
		// false));
		page.setFooter(new SeitenDekoration(preferenceStore
				.getString(DefsSeiteneinstellungen.fusszeileLinks), preferenceStore
				.getString(DefsSeiteneinstellungen.fusszeileMitte), preferenceStore
				.getString(DefsSeiteneinstellungen.fusszeileRechts), false, titel));
		page.setFooterGap(5);

		return page;
	}

	/**
	 * Klasse zur Festlegung der Dekoration (Kopf- bzw. Fuzeile) einer Seite.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class SeitenDekoration implements PageDecoration
	{
		/**
		 * Text im linken Bereich
		 */
		private String _links = null;

		/**
		 * Text im mittleren Bereich
		 */
		private String _mitte = null;

		/**
		 * Text im rechten Bereich
		 */
		private String _rechts = null;

		/**
		 * Kopf oder Fuzeile?
		 */
		private boolean _istKopfzeile = false;

		private LinkedList<String> _sts;

		private String _titel;

		/**
		 * Konstruktor.
		 * 
		 * @param links
		 *          Text im linken Bereich.
		 * @param mitte
		 *          Text im mittleren Bereich.
		 * @param rechts
		 *          Text im rechten Bereich.
		 * @param istKopfzeile
		 *          Wenn <code>true</code>, dann handelt es sich um die Kopfzeile
		 *          und der Trennstrich wird nach allen anderen Elementen
		 *          gezeichnet. Bei <code>false</code> wird zuerst der Trennstrich
		 *          und dann alle anderen Elemente gezeichnet.
		 */
		public SeitenDekoration(String links, String mitte, String rechts,
				boolean istKopfzeile)
		{
			this(links, mitte, rechts, istKopfzeile, "");
		}

		/**
		 * Konstruktor.
		 * 
		 * @param links
		 *          Text im linken Bereich.
		 * @param mitte
		 *          Text im mittleren Bereich.
		 * @param rechts
		 *          Text im rechten Bereich.
		 * @param istKopfzeile
		 *          Wenn <code>true</code>, dann handelt es sich um die Kopfzeile
		 *          und der Trennstrich wird nach allen anderen Elementen
		 *          gezeichnet. Bei <code>false</code> wird zuerst der Trennstrich
		 *          und dann alle anderen Elemente gezeichnet.
		 * @param titel
		 *          Text, mit dem das Feld <code>[Titel]</code> ersetzt wird.
		 */
		public SeitenDekoration(String links, String mitte, String rechts,
				boolean istKopfzeile, String titel)
		{
			this._links = links;
			this._mitte = mitte;
			this._rechts = rechts;
			this._istKopfzeile = istKopfzeile;
			this._titel = titel;
		}

		@Override
		public Print createPrint(PageNumber pageNumber)
		{
			DefaultGridLook look = new DefaultGridLook();
			look.setCellSpacing(2, 2);

			GridPrint grid = new GridPrint("l:pref:none, c:pref:grow, r:pref:none",
					look);
//			GridPrint grid = new GridPrint("p:g, p:g, p:g",
//					look);

			if (!_istKopfzeile)
			{
				grid.add(new LinePrint(SWT.HORIZONTAL), GridPrint.REMAINDER);
			}

			_sts = new LinkedList<String>();
			_sts.add(_links);
			_sts.add(_mitte);
			_sts.add(_rechts);
			pruefeStrings();

			/*
			 * Seitenzahlen setzen.
			 */
			for (String text : _sts)
			{
				if ((text.contains(Defs.FELD_SEITE))
						|| text.contains(Defs.FELD_SEITEN_ANZAHL))
				{
					PageNumberPageDecoration pnpd = new PageNumberPageDecoration();
					pnpd.setFormat(new PageNumberFormatErweitert(text));
					grid.add(pnpd.createPrint(pageNumber));

				}
				else
				{
					grid.add(new TextPrint(text));
				}
			}

			if (_istKopfzeile)
			{
				grid.add(new LinePrint(SWT.HORIZONTAL), GridPrint.REMAINDER);
			}

			return grid;
		}

		/**
		 * Prft Strings und fhrt evtl. Ersetzungen durch Felddaten durch. *
		 */
		private void pruefeStrings()
		{
			Date date = new Date();

			DateFormat datumFormat = new SimpleDateFormat("dd.MM.yyyy");
			String datum = datumFormat.format(date);

			DateFormat zeitFormat = new SimpleDateFormat("HH:mm:ss");
			String zeit = zeitFormat.format(date);

			LinkedList<String> neueStrings = new LinkedList<String>();

			Iterator<String> it = _sts.iterator();
			while (it.hasNext())
			{
				String pruefling = it.next();

				if (pruefling.contains(Defs.FELD_DATUM))
				{
					pruefling = pruefling.replaceAll(Defs.REGEX_DATUM, datum);
				}

				if (pruefling.contains(Defs.FELD_UHRZEIT))
				{
					pruefling = pruefling.replaceAll(Defs.REGEX_UHRZEIT, zeit);
				}

				if (pruefling.contains(Defs.FELD_TITEL))
				{
					pruefling = pruefling.replaceAll(Defs.REGEX_TITEL, _titel);
				}

				neueStrings.add(pruefling);
			}

			_sts = neueStrings;

		}

	}

}