/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.status;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.basislib.login.LoginDialog;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert ein Statuscomposite fr Debug-Zwecke. Kann zur Laufzeit: 1. Die
 * DaV-Verbindungsdaten des aktuellen Benutzers anzeigen
 * 
 * @author HofmannJ
 * 
 */
public class StatusComposite extends Composite
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private BenutzerAngemeldet ba; // @jve:decl-index=0:

	private Group groupStatusUebersicht = null;

	private Label labelThreadCounter = null;

	private Text textThreadCounter = null;

	private Button buttonStop = null;

	private Button buttonStarten = null;

	private Thread th = null;

	private Thread ct = null;

	private Label labelBenutzername = null;

	private Text textBenutzername = null;

	private Button buttonLoginDialog = null;

	private Label labelPasswort = null;

	private Text textPasswort = null;

	private Label labelIp = null;

	private Text textIpAdresse = null;

	private Label labelPort = null;

	private Text textPort = null;

	public StatusComposite(Composite parent, int style)
	{
		super(parent, style);
		initialize();

	}

	private void initialize()
	{
		ba = BenutzerAngemeldet.getInstanz();

		createGroupStatusUebersicht();
		setSize(new Point(300, 200));
		setLayout(new FillLayout());
		// sl = new StatusLeser(textThreadCounter);
		th = new Thread(new Timer());
		th.start();
	}

	/**
	 * This method initializes groupStatusUebersicht
	 * 
	 */
	private void createGroupStatusUebersicht()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		groupStatusUebersicht = new Group(this, SWT.NONE);
		groupStatusUebersicht.setText("Status berischt");
		groupStatusUebersicht.setLayout(gridLayout);
		labelThreadCounter = new Label(groupStatusUebersicht, SWT.NONE);
		labelThreadCounter.setText("ThreadCounter:");
		textThreadCounter = new Text(groupStatusUebersicht, SWT.BORDER);
		textThreadCounter.setLayoutData(gridData);
		buttonStop = new Button(groupStatusUebersicht, SWT.NONE);
		buttonStop.setText("STOP");
		buttonStarten = new Button(groupStatusUebersicht, SWT.NONE);
		buttonStarten.setText("Start");
		labelBenutzername = new Label(groupStatusUebersicht, SWT.NONE);
		labelBenutzername.setText("Benutzername:");
		textBenutzername = new Text(groupStatusUebersicht, SWT.BORDER);
		textBenutzername.setEditable(false);
		labelPasswort = new Label(groupStatusUebersicht, SWT.NONE);
		labelPasswort.setText("Passwort:");
		textPasswort = new Text(groupStatusUebersicht, SWT.BORDER);
		textPasswort.setEditable(false);
		labelIp = new Label(groupStatusUebersicht, SWT.NONE);
		labelIp.setText("IP-Adresse:");
		textIpAdresse = new Text(groupStatusUebersicht, SWT.BORDER);
		textIpAdresse.setEditable(false);
		labelPort = new Label(groupStatusUebersicht, SWT.NONE);
		labelPort.setText("Port:");
		textPort = new Text(groupStatusUebersicht, SWT.BORDER);
		textPort.setEditable(false);
		buttonLoginDialog = new Button(groupStatusUebersicht, SWT.NONE);
		buttonLoginDialog.setText("Login");
		buttonLoginDialog.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				Shell shell = new Shell();
				LoginDialog login = new LoginDialog(shell);
				// login.OpenShell();
				if (login.anzeigen())
				{
					BenutzerAngemeldet ba = BenutzerAngemeldet.getInstanz();
					_debug.fine("Name: " + ba.getName());
					_debug.fine("Passwort: " + ba.getPasswort());
					_debug.fine("IP: " + ba.getIp());
					_debug.fine("Port: " + ba.getTcpPort());
				}
			}
		});
		
		buttonStarten.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (ct != null && ct.isAlive())
				{
					ct.interrupt();
				} else
				{
					ct = new Thread(new KonsolenTimer());
					ct.start();
				}
			}

		});
		
		buttonStop.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{

				if (th != null && th.isAlive())
				{
					_debug.fine("Stoppe StatusThread!");
					th.interrupt();
				} else
				{
					_debug.fine("Starte StatusThread!");
					th = new Thread(new Timer());
					th.start();
				}
			}			
		});
	}

	private class Timer implements Runnable
	{
		int i = 0;

		public void run()
		{
			boolean go = true;
			while (go)
			{
				Display.getDefault().syncExec(new Runnable()
				{
					public void run()
					{
						// textThreadCounter.setText(d.toString());
						i++;
						textThreadCounter.setText(String.valueOf(i));
						if (ba.getName() != null)
						{
							textBenutzername.setText(ba.getName());
							textPasswort.setText(ba.getPasswort());
							textPort.setText(String.valueOf(ba.getTcpPort()));
							textIpAdresse.setText(ba.getIp());
						}
					}
				});
				try
				{
					Thread.sleep(500);
				} catch (InterruptedException ie)
				{
					// ie.printStackTrace();
					_debug.fine("Timerthread gestoppt => " + ie.toString());
					go = false;
				}

			}
		}
	}

	private class KonsolenTimer implements Runnable
	{
		int i = 0;

		public void run()
		{
			boolean go = true;
			while (go)
			{
				Display.getDefault().syncExec(new Runnable()
				{
					public void run()
					{
						// textThreadCounter.setText(d.toString());
						i++;
						_debug.fine("I: " + i);
					}
				});
				try
				{
					Thread.sleep(500);
				} catch (InterruptedException ie)
				{
					// ie.printStackTrace();
					_debug.fine("KonsolenTimerThread gestoppt => " + ie.toString());
					go = false;
				}

			}
		}
	}

}
