/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.statusleiste.editor;

import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;
import de.bsvrz.sys.funclib.debug.Debug;

public class SichtVerfuegbareStatusleistenElemente extends ViewPart
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  public static final String ID = "de.bsvrz.buv.rw.basislib.viewVerfuegbareStatusleistenElemente";

  private TableViewer _tableViewer = null;

  private Composite _top = null;

  private ZugZuhoererVerfuegbaresStatusLeistenElement _zugZuhoererVerfuegbaresStatusLeistenElement;

  @Override
  public void createPartControl(Composite parent)
  {
    parent.setLayout(new FillLayout());

    _top = new Composite(parent, SWT.NONE);
    _top.setLayout(new FillLayout());

    erzeugeTabelle();
    initialisiereTabelle();
    erzeugeKontrolle();

    PlatformUI.getWorkbench().getHelpSystem().setHelp(_top, "de.bsvrz.buv.rw.basislib.statusleisteneditor");
  }

  private void erzeugeKontrolle()
  {
    // Aktualisieren
    IAction actionAktualisieren = new Action() {
      public void run()
      {
        initialisiereTabelle();
      }
    };
    actionAktualisieren.setText("Aktualisieren");
    actionAktualisieren.setToolTipText("Aktualisieren");
    actionAktualisieren.setImageDescriptor(Activator.getImageDescriptor("icons/allgemein/refresh.gif"));

    // Testelemente erzeugen
    IAction actionTestElementeErzeugen = new Action() {
      public void run()
      {
        StatusLeisteElement testelem1 = new StatusLeisteElement("plugin.1.statusleistenelement.1",
            "1.stes StatusLeistenElement", null);
        StatusLeisteElement testelem2 = new StatusLeisteElement("plugin.1.statusleistenelement.2",
            "2.tes StatusLeistenElement", null);
        StatusLeisteElement testelem3 = new StatusLeisteElement("plugin.2.statusleistenelement.1",
            "3.tes StatusLeistenElement", null);
        StatusLeisteElement testelem4 = new StatusLeisteElement("plugin.2.statusleistenelement.2",
            "4.tes StatusLeistenElement", null);
        StatusLeisteElement testelem5 = new StatusLeisteElement("www.eclipse.org", "5.tes StatusLeistenElement", null);
        try
        {
          StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(testelem1);
          StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(testelem2);
          StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(testelem3);
          StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(testelem4);
          StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(testelem5);
          initialisiereTabelle();
        }
        catch (IllegalArgumentException iae)
        {
          iae.printStackTrace();
        }
      };
    };
    actionTestElementeErzeugen.setText("Testelemente erzeugen");
    actionTestElementeErzeugen.setToolTipText("Testelemente erzeugen ist nur whrend des Debuggens verfgbar.");
    actionTestElementeErzeugen.setImageDescriptor(Activator.getImageDescriptor("icons/berechtigung/add_16x16.png"));

    IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();

    toolBarManager.add(actionTestElementeErzeugen);

    toolBarManager.add(actionAktualisieren);

  }

  /**
   * Initialisiert die Tabelle
   * 
   */
  private void initialisiereTabelle()
  {
    _debug.fine("init Tabelle");
    _tableViewer.setInput(StatusLeisteVerwaltung.getInstanz());
    _debug.fine("init Ende");
  }

  /**
   * Erzeugt die Tabelle
   * 
   */
  private void erzeugeTabelle()
  {
    _tableViewer = new TableViewer(_top, SWT.FULL_SELECTION | SWT.MULTI);

    TableColumn tc1 = new TableColumn(_tableViewer.getTable(), SWT.CENTER);
    tc1.setText("ID:");
    tc1.setWidth(200);

    TableColumn tc2 = new TableColumn(_tableViewer.getTable(), SWT.LEFT);
    tc2.setText("Bezeichnung:");
    tc2.setWidth(300);

    _tableViewer.getTable().setHeaderVisible(true);
    _tableViewer.getTable().setLinesVisible(true);

    _tableViewer.setContentProvider(new StatusLeistenElementeInhaltsVersorger());
    _tableViewer.setLabelProvider(new StatusleistenElementeDarstellungsVersorger());

    /*
     * int ops = DND.DROP_COPY; Transfer[] dropTransfers = new Transfer[] { TransferDescriptorElement.getInstanz(),
     * TransferDescriptorAction.getInstanz(), TextTransfer.getInstance()}; // _treeViewer.addDropSupport(ops, transfers,
     * new // TreeDropAdapterDescriptorElement(_treeViewer)); _treeViewer.addDropSupport(ops, dropTransfers, new
     * TreeDropAdapterElementViewMenuBarEditor(_treeViewer, _menuBarData));
     * 
     * 
     * Transfer[] dragTransfers = new Transfer[]{TextTransfer.getInstance()};
     * 
     * _treeViewer.addDragSupport(ops, dragTransfers, new DragZuhoererXmlElement(_treeViewer));
     */

    int ops = DND.DROP_MOVE;
    Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
    _zugZuhoererVerfuegbaresStatusLeistenElement = new ZugZuhoererVerfuegbaresStatusLeistenElement(_tableViewer);
    _tableViewer.addDragSupport(ops, dragTransfers, _zugZuhoererVerfuegbaresStatusLeistenElement);

  }

  @Override
  public void setFocus()
  {
    if (_top != null)
      _top.setFocus();

    initialisiereTabelle();

  }

  private class StatusLeistenElementeInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      Object[] returnElemente = null;
      if (inputElement instanceof StatusLeisteVerwaltung)
      {

        StatusLeisteVerwaltung sv = (StatusLeisteVerwaltung)inputElement;
        TreeMap<String, StatusLeisteElement> elemente = sv.getVerfuegbareStatusLeistenElemente();
        if (elemente.size() > 0)
        {
          returnElemente = new Object[elemente.size()];
          int i = -1;
          Iterator<Entry<String, StatusLeisteElement>> it = elemente.entrySet().iterator();
          while (it.hasNext())
          {
            i++;
            Entry<String, StatusLeisteElement> entry = it.next();
            returnElemente[i] = entry.getValue();
          }
        }
        else
        {
          returnElemente = new Object[0];
        }
      }

      return returnElemente;
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class StatusleistenElementeDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object element, int columnIndex)
    {

      String returnString;
      if (element instanceof StatusLeisteElement)
      {
        StatusLeisteElement statusLeisteElement = (StatusLeisteElement)element;
        // ID
        if (columnIndex == 0)
        {
          returnString = statusLeisteElement.getId();
        }
        // Bezeichnung
        else if (columnIndex == 1)
        {
          returnString = statusLeisteElement.getBezeichnung();
        }
        // Sonst
        else
        {
          returnString = "Fr diese Spalte findet keine Versorgung statt.";
        }

      }
      else
      {
        returnString = "Nicht untersttztes Element vom Typ " + element.getClass().getName() + ".";
      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

}
