/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.statusleiste.editor;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.jdom.Element;

import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.XmlBezeichnerStatus;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:19:34 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class ZugZuhoererStatusLeistenElementXML extends DragSourceAdapter
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Viewer, auf dem gearbeitet wird.
   */
  private StructuredViewer _viewer;

  private ZugElementSpeicher _zugElementSpeicher;

  public ZugZuhoererStatusLeistenElementXML(StructuredViewer viewer)
  {
    this._viewer = viewer;
  }

  public void dragFinished(DragSourceEvent event)
  {

    if (!event.doit)
      return;
    // if the gadget was moved, remove it from the source viewer
    // if (event.detail == DND.DROP_MOVE) {
    // IStructuredSelection selection =
    // (IStructuredSelection)_viewer.getSelection();
    // for (Iterator it = selection.iterator(); it.hasNext();) {
    // //Entfernen der Elemente
    // }
    // _viewer.refresh();
    // }
    if (event.detail == DND.DROP_MOVE)
    {
      _debug.fine(getClass().getName() + " => Lschen.");
      // IStructuredSelection selection
      // =(IStructuredSelection)_viewer.getSelection();
      // Iterator und aktualisieren der Elemente
      _viewer.refresh();
    }

    _debug.fine(getClass().getName() + " => dragFinished");

  }

  /**
   * Liefert die per dragging selektierten Elemente des TableViewers.
   */
  public void dragSetData(DragSourceEvent event)
  {

    _debug.fine(getClass().getName() + "::dragSetData => begin.");

    IStructuredSelection selection = (IStructuredSelection)_viewer.getSelection();

    Object selectedObject = selection.getFirstElement();

    if (selectedObject instanceof Element)
    {
      Element xmlStatusLeistenElement = (Element)selectedObject;
      String id = "[XML]" + xmlStatusLeistenElement.getAttributeValue(XmlBezeichnerStatus.attributId);
      _zugElementSpeicher = ZugElementSpeicher.getInstanz();
      _zugElementSpeicher.setZugElement(xmlStatusLeistenElement);

      if (TextTransfer.getInstance().isSupportedType(event.dataType))
      {
        event.data = id;
      }
    }

    _debug.fine(getClass().getName() + "::dragSetData => Ende.");

  }

  /**
   * Wird whrend des Starts der Drag-Vorgangs aufgerufen. Bestimmt den Datentyp und stellt sicher, dass keine leere
   * Selektion "gedraged" werden kann.
   */
  public void dragStart(DragSourceEvent event)
  {
    _debug.fine(getClass().getName() + "::dragStart => Start");

    // TransferData dataType = event.dataType;
    event.doit = !_viewer.getSelection().isEmpty();
  }

}
