/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.statusleiste.generator;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.action.IContributionItem;
import org.jdom.Element;

import de.bsvrz.buv.rw.rw.statusleiste.StatusLeisteZugriff;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.EreignisStatusLeisteGeaendert;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.IZuhoererStatusLeisteAenderung;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.XmlBezeichnerStatus;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Generiert die Statusleiste zur Laufzeit.
 * 
 * @author HofmannJ
 * 
 */
public class StatusLeisteGenerator
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Verweis auf die {@link StatusLeisteVerwaltung}.
   */
  private StatusLeisteVerwaltung _slv = null;

  /**
   * Verweis auf {@link StatusLeisteZugriff}.
   */
  private StatusLeisteZugriff _slz = null;

  /**
   * Privater Konstruktor
   */
  private StatusLeisteGenerator()
  {
    init();
  }

  /**
   * Speichert die Singleton-Instanz der Klasee {@link StatusLeisteGenerator}.
   * 
   * @author HofmannJ
   * 
   */
  private static class Innen
  {
    private static StatusLeisteGenerator INSTANZ = new StatusLeisteGenerator();
  }

  /**
   * Liefert die Singleton-Instanz der Klasse {@link StatusLeisteGenerator}.
   * 
   * @return Singleton-Instanz der Klasse {@link StatusLeisteGenerator}
   */
  public static StatusLeisteGenerator getInstanz()
  {
    return Innen.INSTANZ;
  }

  /**
   * Fhrt die Initialisierung durch.
   * 
   */
  private void init()
  {
    // Holen von Statusleiste Verwaltung + Zugriff
    this._slv = StatusLeisteVerwaltung.getInstanz();
    this._slz = StatusLeisteZugriff.getInstanz();

    // Registrieren eines Zuhrers auf nderungen der Struktur der Statusleiste
    IZuhoererStatusLeisteAenderung zuhoerer = new IZuhoererStatusLeisteAenderung() {

      public void ausfuehren(EreignisStatusLeisteGeaendert e)
      {
        _debug.fine(getClass().getName() + "::init.zuhoerer \n\t=> Erzeuge Statusleiste.\n");

        generiereStatusLeiste();
      }

    };

    _slv.hinzufuegenZuhoerer(zuhoerer);
  }

  /**
   * Generiert die Statusleiste, holt die Sortierung der Elemente sowie die {@link StatusLeisteElement}e aus
   * {@link StatusLeisteVerwaltung} und baut ber {@link StatusLeisteZugriff} die StatusLeiste auf.
   * 
   */
  public void generiereStatusLeiste()
  {  	
    List<Element> statusleisteElemente = _slv.getListeXmlStatusleisteElemente();

    if ((statusleisteElemente != null) && (statusleisteElemente.size() > 0))
    {
      // Leeren der Statuszeile
      _slz.entferneAlleStatusZeilenElemente();

      // Lesen der Sortierung der Statuszeilenelemente
      Iterator<Element> it = statusleisteElemente.iterator();
      while (it.hasNext())
      {
        Element element = it.next();
        String id = element.getAttributeValue(XmlBezeichnerStatus.attributId);

        // Versuch, die Elemente aus der Verwaltung zu holen und, sofern
        // vorhanden, i.d. Statusleiste zu schieben.
        try
        {
          StatusLeisteElement statusLeisteElement = _slv.getStatusLeistenElement(id);
          IContributionItem slElementContributionItem = statusLeisteElement.getElement();
          _slz.hinzufuegenStatusZeilenElement(slElementContributionItem);

        }
        catch (IllegalArgumentException iae)
        {
//          _debug.info(getClass().getName() + "\n\t=>Es existiert kein StatusLeistenElement mit der ID " + id + " \n");
        }

      }
    }
    else
    {
//      _debug
//          .info(getClass().getName()
//              + "::generiereStatusLeiste \n\t=> Es sind keine Statusleistenelemente zum Generieren der Statusleiste vorhanden.\n");
    }

  }

}
