/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;

public class DialogSpaltenSortierungsReihenfolge extends Dialog
{

	private Shell shell = null;
	private Composite compositeOben = null;
	private Composite compositeUnten = null;
	private Group gruppeSpalten = null;
	private Group gruppeSortierung = null;
	private Composite compositeKontrolle = null;
	private Button _schaltflaecheOk = null;
	private Button _schaltflaecheAbbrechen = null;
	private Button _schaltflaecheHilfe = null;
	private Table tabelleSpalten = null;
	private Table tabelleSortierung = null;
	private TableViewer tableViewerSpalten = null;
	private TableViewer tableViewerSortierung = null;

	/**
	 * Flag zeigt an, ob die Spaltenreihenfolge gesetzt wurde.
	 */
	private boolean _spaltenreihenfolgeGesetzt = false;

	/**
	 * Struktur der {@link TabellenDatenStruktur}
	 */
	private TabellenDatenStruktur _tds = null;

	/**
	 * Reihenfolge der Spalten
	 */
	private LinkedList<SpaltenElement> _sortierung = new LinkedList<SpaltenElement>();
	private Button _schaltflaecheLoeschen;

	public DialogSpaltenSortierungsReihenfolge(Shell parent)
	{
		super(parent, 0);
	}

	/**
	 * This method initializes compositeOben
	 * 
	 */
	private void createCompositeOben()
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeOben = new Composite(shell, SWT.NONE);
		createGruppeSpalten();
		compositeOben.setLayout(gridLayout);
		createGruppeSortierung();
		compositeOben.setLayoutData(gridData);
	}

	public boolean open(TabellenDatenStruktur tds)
	{
		Shell parent = getParent();
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);

		/*
		 * TDS setzen
		 */
		_tds = tds;

		erzeugeDialog();

		/*
		 * bersicht aller Spalten
		 */
		tableViewerSpalten.setInput(_tds.getAlleSpaltenTitel());

		// /*
		// * bersicht der aktuellen Sortierung
		// */
		// Integer[] sortierung = _tds.getSortierung();
		// for (Integer wert : sortierung)
		// {
		// SpaltenElement spaltenTitel = _tds.getSpaltenTitel(wert);
		// System.out.println("===>" + spaltenTitel);
		// _sortierung.add(spaltenTitel);
		// }
		//
		// SpaltenElement[] ar =
		// SpaltenElement.spaltenElementListe2Array(_sortierung);
		//
		// for (SpaltenElement spaltenElement : ar)
		// {
		// System.out.println("=>" + spaltenElement);
		// }
		//
		// tableViewerSortierung.setInput(ar);
		
		sortierungAusTabelleDatenStrukturLaden();
		aktuelleSortierungInTabelleDarstellen();

		shell.setSize(500, 300);
		shell.open();
		Display display = parent.getDisplay();
		while (!shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _spaltenreihenfolgeGesetzt;
	}

	protected void sortierungAusTabelleDatenStrukturLaden()
	{
		_sortierung.clear();

		/*
		 * bersicht der aktuellen Sortierung
		 */
		Sortierung[] sortierung = _tds.getSortierung();
		for (Sortierung wert : sortierung)
		{
			SpaltenElement spaltenTitel = _tds.getSpaltenTitel(wert.getSpalte());
			_sortierung.add(spaltenTitel);
		}
	}

	/**
	 * Aktualisiert die Darstellung der aktuellen Sortierung.
	 */
	protected void aktuelleSortierungInTabelleDarstellen()
	{

		SpaltenElement[] ar = SpaltenElement.spaltenElementListe2Array(_sortierung);		

		tableViewerSortierung.setInput(ar);
	}

	/**
	 * This method initializes compositeUnten
	 * 
	 */
	private void createCompositeUnten()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		compositeUnten = new Composite(shell, SWT.NONE);
		compositeUnten.setLayout(new GridLayout());
		createCompositeKontrolle();
		compositeUnten.setLayoutData(gridData1);
	}

	/**
	 * Liefert die Bearbeitete Sortierung von {@link SpaltenElement}en als Liste.
	 * 
	 * @return Sortierung von {@link SpaltenElement}en als Liste
	 */
	protected LinkedList<SpaltenElement> getSortierung()
	{
		return _sortierung;
	}

	protected TabellenDatenStruktur getTabellenDatenStruktur()
	{
		return _tds;
	}

	/**
	 * This method initializes gruppeSpalten
	 * 
	 */
	private void createGruppeSpalten()
	{
		GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.grabExcessVerticalSpace = true;
		gridData5.verticalAlignment = GridData.FILL;
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		gruppeSpalten = new Group(compositeOben, SWT.NONE);
		gruppeSpalten.setLayout(new GridLayout());
		gruppeSpalten.setLayoutData(gridData2);
		gruppeSpalten.setText("Verfgbare Spalten:");
		tabelleSpalten = new Table(gruppeSpalten, SWT.NONE);
		tabelleSpalten.setHeaderVisible(true);
		tabelleSpalten.setLayoutData(gridData5);
		tabelleSpalten.setLinesVisible(true);
		tableViewerSpalten = new TableViewer(tabelleSpalten);

		TableColumn tc = new TableColumn(tabelleSpalten, SWT.CENTER);
		tc.setText("Titel der Spalte:");
		tc.setWidth(200);

		UebersichtContentProvider cp = new UebersichtContentProvider();
		tableViewerSpalten.setContentProvider(cp);
		UebersichtLabelProvider lp = new UebersichtLabelProvider();
		tableViewerSpalten.setLabelProvider(lp);

		int ops = DND.DROP_MOVE;
		Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
		DragSourceAdapterTabellenSpalten _dragSourceAdapter = new DragSourceAdapterTabellenSpalten(
				tableViewerSpalten);

		tableViewerSpalten.addDragSupport(ops, dragTransfers, _dragSourceAdapter);
	}

	/**
	 * This method initializes gruppeSortierung
	 * 
	 */
	private void createGruppeSortierung()
	{
		GridData gridData6 = new GridData();
		gridData6.horizontalAlignment = GridData.FILL;
		gridData6.grabExcessHorizontalSpace = true;
		gridData6.grabExcessVerticalSpace = true;
		gridData6.verticalAlignment = GridData.FILL;
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		gruppeSortierung = new Group(compositeOben, SWT.NONE);
		gruppeSortierung.setLayout(new GridLayout());
		gruppeSortierung.setLayoutData(gridData3);
		gruppeSortierung.setText("Aktuelle Sortierung:");
		tabelleSortierung = new Table(gruppeSortierung, SWT.FULL_SELECTION);
		tabelleSortierung.setHeaderVisible(true);
		tabelleSortierung.setLayoutData(gridData6);
		tabelleSortierung.setLinesVisible(true);
		tableViewerSortierung = new TableViewer(tabelleSortierung);
		
		
		@SuppressWarnings("unused")
		MenuManager mm = getKontextMenu(tableViewerSortierung.getControl()); // Muss bleiben !		

		

		TableColumn tcSpaltenTitel = new TableColumn(tabelleSortierung, SWT.CENTER);
		tcSpaltenTitel.setText("Spaltentitel:");
		tcSpaltenTitel.setWidth(100);
		
		TableColumn tcSortierung = new TableColumn(tabelleSortierung, SWT.CENTER);
		tcSortierung.setText("Sortierung");
		tcSortierung.setWidth(100);

		UebersichtContentProvider cp = new UebersichtContentProvider();
		tableViewerSortierung.setContentProvider(cp);
		UebersichtLabelProvider lp = new UebersichtLabelProvider();
		tableViewerSortierung.setLabelProvider(lp);

		int ops = DND.DROP_MOVE;
		ViewerDropAdapterSpaltenReihenfolge dropAdapter = new ViewerDropAdapterSpaltenReihenfolge(
				tableViewerSortierung, this);
		Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
		tableViewerSortierung.addDropSupport(ops, dragTransfers, dropAdapter);

	}

	private void aktionSchliessen()
	{
		shell.setVisible(false);
		shell.dispose();
	}

	private void aktionOk()
	{
		// Rckgabewert des Dialogs erzeugen
		_tds.setSortierung(_sortierung);
		_spaltenreihenfolgeGesetzt = true;
		
		aktionSchliessen();
	}

	private void aktionLoescheAktuelleSortierung()
	{
		_sortierung.clear();
		aktuelleSortierungInTabelleDarstellen();
	}
	
	private void aktionHilfe()
	{
		String helpContextId = Activator.PLUGIN_ID + ".sortierDialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * This method initializes compositeKontrolle
	 * 
	 */
	private void createCompositeKontrolle()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.pack = false;
		GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.END;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.CENTER;
		compositeKontrolle = new Composite(compositeUnten, SWT.NONE);
		compositeKontrolle.setLayoutData(gridData4);
		compositeKontrolle.setLayout(rowLayout);
		_schaltflaecheOk = new Button(compositeKontrolle, SWT.NONE);
		_schaltflaecheOk.setText("OK");
		_schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}			
		});
		
		_schaltflaecheAbbrechen = new Button(compositeKontrolle, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");		
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});

		_schaltflaecheLoeschen = new Button(compositeKontrolle, SWT.NONE);
		_schaltflaecheLoeschen.setText("Sortierung Loeschen");
		_schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionLoescheAktuelleSortierung();
			}			
		});
		
		
		_schaltflaecheHilfe = new Button(compositeKontrolle, SWT.NONE);
		_schaltflaecheHilfe.setText("Hilfe");		
		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}			
		});

	}

	/**
	 * Erzeugt die Darstellung des Dialogs
	 */
	private void erzeugeDialog()
	{
		shell.setText("Spaltensortierung");
		createCompositeOben();
		createCompositeUnten();

		shell.setLayout(new GridLayout());
		shell.setSize(new Point(400, 250));
	}
	
  public MenuManager getKontextMenu(Control control)
  {
    MenuManager menuMgr = new MenuManager("#PopupMenu");
    menuMgr.setRemoveAllWhenShown(true);
    menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager)
      {
        fuelleKontextMenu(manager);
      }
    });

    Menu menu = menuMgr.createContextMenu(control);
    control.setMenu(menu);

    return menuMgr;
  }
  
  public void fuelleKontextMenu(IMenuManager manager)
  {
  	Action aufsteigend = new Action(){
  		public void run()
  		{  			
  			setSortierungAufAktuellerSelektion(true);
  			aktuelleSortierungInTabelleDarstellen();
  		}
  	};
  	aufsteigend.setText("Aufsteigend");

  	
  	
  	Action absteigend = new Action()
  	{
  		public void run()
  		{
  			setSortierungAufAktuellerSelektion(false);
  			aktuelleSortierungInTabelleDarstellen();
  		}
  	};
  	absteigend.setText("Absteigend");
  	
  	
  	
  	
  	manager.add(aufsteigend);
  	manager.add(absteigend);
  	
  	
  	ISelection selection = tableViewerSortierung.getSelection();
  	if(selection instanceof IStructuredSelection)
  	{
  		IStructuredSelection iss = (IStructuredSelection) selection;
  		Object firstElement = iss.getFirstElement();
  		if(firstElement instanceof SpaltenElement)
  		{
  			SpaltenElement spaltenElement = (SpaltenElement) firstElement;
  			if(spaltenElement.istAufsteigend())
  			{
  				aufsteigend.setChecked(true);
  			}
  			else
  			{
  				absteigend.setChecked(true);
  			}
  		}
  	}

  }
	
  
  private void setSortierungAufAktuellerSelektion(boolean aufsteigend)
  {
  	ISelection selection = tableViewerSortierung.getSelection();
  	if(selection instanceof IStructuredSelection)
  	{
  		IStructuredSelection iss = (IStructuredSelection) selection;
  		Object firstElement = iss.getFirstElement();
  		if(firstElement instanceof SpaltenElement)
  		{
  			SpaltenElement spaltenElement =(SpaltenElement) firstElement;
  			spaltenElement.setAufsteigend(aufsteigend);
  		}
  	}
  }
	

	/**
	 * ContentProvider fr die bersicht der vorhandenen {@link SpaltenElement}e.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class UebersichtContentProvider implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object re[] = null;

			if (inputElement instanceof SpaltenElement[])
			{
				SpaltenElement[] spaltenElemente = (SpaltenElement[]) inputElement;
				re = spaltenElemente;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * LabelProvider fr die bersicht der vorhandenen {@link SpaltenElement}e.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class UebersichtLabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;
			if (element instanceof SpaltenElement)
			{
				if(columnIndex == 0)
				{
				SpaltenElement se = (SpaltenElement) element;
				re = se.toString();
				}
				else if(columnIndex == 1)
				{
					SpaltenElement se = (SpaltenElement) element;
					if(se.istAufsteigend())
					{
						re = "aufsteigend";
					}
					else
					{
						re = "absteigend";
					}
				}
			}
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

}
