/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

/**
 * Interface zur Reprsentation von Tabellenzeilen. Liefert die Anzahl der
 * Attribute der Zeile, die Attribute sowie das Objekt, welches hinter der Zeile
 * steht. <br>
 * <b>Achtung:</b> Alle Objekte, die Attribute reprsentieren, mssen das
 * Interface {@link Comparable} erfllen, da sonnst keine Sortierung und keine
 * nderungsdarstellung mglich ist.
 * 
 * @author HofmannJ
 * 
 */
public interface ITabellenZeile extends Comparable<ITabellenZeile>
{
	/**
	 * Liefert die Anzahl der Attribute einer Zeile zurck, welche als Inhalte in
	 * den Spalten dargestellt werden.
	 * 
	 * @return Anzahl der Attribute einer Zeile
	 */
	public int getAnzahlAttribute();

	/**
	 * Liefert ein Attribut an einer bestimmten Position zurck. Dieses Attribut
	 * reprsentiert den Inhalt einer Zelle der jeweiligen Spalte dieser Zeile.
	 * 
	 * @param idx
	 *          Position des Attributes
	 * @return Attribut
	 */
	public Object getAttribut(int idx);

	/**
	 * In diesem Array mssen die Indizes der Attribute angegeben werden, welche
	 * bei der nderungsdarstellung bercksichtigt werden sollen.
	 * 
	 * @return Array bestehend aus den Indizes der jeweiligen Attribute
	 */
	public int[] getZuBeruecksichtigendeAttribute();

	/**
	 * Liefert das Objekt, welches hinter einer Zeile steht. Fr die
	 * nderungsdarstellung ist dieses Objekt zwingend notwendig!
	 * 
	 * @return Objekt der Tabellenzeile
	 */
	public Object getZeilenObject();
}
