/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.ui.internal.ZyklischesSpeichern;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse sendet ein Signal fr das zyklische Speichern der Arbeitsumgebung aus.<br>
 * Erfllt BuV-7
 * 
 * @author HofmannJ
 * 
 */
public class ZyklischesSpeichern implements Runnable
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger(); 
  
  
  /**
   * Thread-Flag, zeigt an ob der Thread luft oder nicht.
   */
  private boolean _threadLaufe = false;

  /**
   * Zeit die der Thread warten soll.
   */
  private long _threadWarteZeit = 15 * 60 * 1000; // 15 Minuten
//  private long _threadWarteZeit = 2 * 1000; // 15 Minuten

  /**
   * Privater Konstruktor
   */
  private ZyklischesSpeichern()
  {
    startThread();
  }

  /**
   * Speichert die Singleton-Instanz dieser Klasse
   * 
   * @author HofmannJ
   * 
   */
  private static class Innen
  {
    private static ZyklischesSpeichern INSTANZ = new ZyklischesSpeichern();
  }

  /**
   * Liefert die Singleton-Instanz der Klasse {@link ZyklischesSpeichern}.
   * 
   * @return Singleton-Instanz der Klasse {@link ZyklischesSpeichern}
   */
  public static ZyklischesSpeichern getInstanz()
  {
    return Innen.INSTANZ;
  }

  /**
   * Startet den Thread
   */
  private void startThread()
  {
    if (_threadLaufe == false)
    {
      Thread th = new Thread(this);
      th.setName("Zyklisches Speichern");
      _threadLaufe = true;
      th.start();
    }
  }

  /**
   * Stoppt den Thread
   */
  public void stopThread()
  {
    if (_threadLaufe)
    {
      _threadLaufe = false;
    }
  }

  /**
   * Sendet das Signal zum Speichern des Arbeitsbereiches. Wird zyklisch vom Thread aufgerufen.
   */
  private void zyklischesSpeichersignalSenden()
  {
    _debug.fine("SENDE SIGNAL!");
    try
    {
      ResourcesPlugin.getWorkspace().save(true, null);     
    }
    catch (CoreException e)
    {
      System.err.println(getClass().getName()
          + "::zyklischesSpeichersignalSenden Fehler beim zyklischen Speichern\n\t=> " + e.toString());
      e.printStackTrace();
    }
  }

  @Override
  public void run()
  {
    while (_threadLaufe)
    {
      try
      {
        Thread.sleep(_threadWarteZeit);
      }
      catch (InterruptedException e)
      {
        System.err
            .println(getClass().getName() + "::run Fehler des Threads zum zyklischen Speichen => " + e.toString());
      }
      zyklischesSpeichersignalSenden();
    }
  }

}
