/*
 * This file is part of Rahmenwerk.
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * 
 * Copyright (c) 2012 Swarco Traffic Systems GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * Swarco Traffic Systems GmbH - initial API and implementation
 * 
 * 
 * 
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau 
 * T. +49 (0)7225 / 64-03
 * F. +49 (0)7225 / 64-300 
 * E. office.sts@swarco.de
 */
package de.bsvrz.buv.rw.rw.ui.internal.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.rw.basislib.login.LoginDialog;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.ui.internal.Activator;

/**
 * Realisiert eine {@link Action} zum Anmelden an den DatenVerteiler
 * 
 * @author HofmannJ
 * 
 */
public class AktionAnmelden extends Action implements IDaVVerbindungZuhoerer
{
	/**
	 * Flag fr dynamische Darstellung
	 */
	private boolean _dynamisch = true;

	/**
	 * Konstruktor. Login-Dialog wird mit statischem Layout aufgerufen.
	 */
	public AktionAnmelden()
	{
		initialisiere();
	}

	/**
	 * Konstruktor.
	 * 
	 * @param dynamisch
	 *          Wenn <code>true</code>, dann wird der Login-Dialog mit einem
	 *          dynamischen Layout aufgerufen, bei <code>false</code> mit einem
	 *          statischen Layout.
	 */
	public AktionAnmelden(boolean dynamisch)
	{
		this._dynamisch = dynamisch;
		initialisiere();
	}

	/**
	 * Fhrt die Initialisierung der Action durch.
	 */
	private void initialisiere()
	{
		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(this);
		setText("Anmelden");
		setEnabled(true);
		this.setId(Activator.PLUGIN_ID + ".aktionAnmelden");
		this.setText("Anmelden");
		this.setToolTipText("ffnet den Dialog zum Anmelden.");

		validiereVerbindung(DaVVerbindung.getInstanz().istDaVVerbindungAktiv());
	}

	@Override
	public void run()
	{

		// if(BetriebsArtAuswerten.getInstanz().getBetriebsArt().equals(BetriebsArt.online))
		// {
		@SuppressWarnings("unused")
		Display display = Display.getDefault();
		// Display display = Display.getCurrent();

		LoginDialog login = new LoginDialog(new Shell());

		boolean erfolgreichGeoffnet = false;
		if (!_dynamisch)
		{
			erfolgreichGeoffnet = login.anzeigen();
		} else
		{
			erfolgreichGeoffnet = login.anzeigenDynamisch();
		}

		// if (login.anzeigen())
		if (erfolgreichGeoffnet)
		{

			BenutzerAngemeldet ba = BenutzerAngemeldet.getInstanz();

			/*
			 * Hier ist anmelden an den DaV mglich.
			 */
			DaVVerbindung dav = DaVVerbindung.getInstanz();
			dav.setDaVBenutzer(ba.getName());
			dav.setDaVPasswort(ba.getPasswort());
			dav.setDaVIp(ba.getIp());
			dav.setDaVPort(ba.getTcpPort());
			if (dav.baueVerbindungMitDaVAuf())
			{
				// String sObj = "fs.mq.a10.0000.1fs";
				// String sAtg = "atg.fahrStreifen";
				// SystemObject obj =
				// dav.getDaVVerbindung().getDataModel().getObject(sObj);
				// AttributeGroup atg =
				// dav.getDaVVerbindung().getDataModel().getAttributeGroup(sAtg);
				// Data data = obj.getConfigurationData(atg);
				// String printString = "\n Obj: " + sObj + " ATG: " + sAtg + " = ";
				// printString = printString + "Plugin Lage " +
				// data.getScaledValue("Lage").getValueText();
				//
				// TitleAreaDialog titleAreaDialog = new TitleAreaDialog(new
				// Shell(display));
				// titleAreaDialog.create();
				// titleAreaDialog.setTitle("DaV-Verbindung hergestellt");
				// titleAreaDialog.setMessage("Verbindung hergestellt mit " +
				// dav.getDaVVerbindung() + " " + printString);
				//
				// if (titleAreaDialog.open() == Dialog.OK)
				// {
				//
				// }

				setEnabled(false);
			}

		}
		// }
		// else
		// {
		// MessageBox mb = new MessageBox(new Shell(Display.getCurrent()),
		// SWT.ICON_INFORMATION | SWT.OK);
		// mb.setText("Anmelden am DaV nicht mglich!");
		// mb.setMessage("Das Rahmenwerk befindet sich im Offline-Modus.\n" +
		// "Es ist somit kein Zugriff auf den DaV mglich.");
		// mb.open();
		// }

	}

	@Override
	public void verbindungAbgebaut(DaVVerbindungEreignis e)
	{
		validiereVerbindung(false);
	}

	@Override
	public void verbindungAufgebaut(DaVVerbindungEreignis e)
	{
		validiereVerbindung(true);
	}

	/**
	 * Prft ob die Verbindung zum DaV aktiv. Abhngig hiervon wird diese Aktion
	 * aktiviert bzw. deaktiviert.
	 * 
	 * @param davVerbindungAktiv
	 *          Flag das anzeigt, ob die Verbindung zum DaV aktiv ist (
	 *          <code>true</code>) oder nicht (<code>false</code>).
	 */
	private void validiereVerbindung(boolean davVerbindungAktiv)
	{
		this.setEnabled(!davVerbindungAktiv);
	}
	

}
