/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.fail;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.ToolBar;
import org.jdom.Attribute;
import org.jdom.Element;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.buv.rw.rw.Activator;

/**
 * @author HofmannJ
 * 
 */
public class StatusLeisteVerwaltungTest
{
	/**
	 * Pfad zur Testdatei. Vorsicht: Existiert diese Datei, so wird sie beim Ende
	 * des JUnit-Testes gelscht!
	 */
	private static final String _pfadTestDatei = "/C:/junittest.xml";

	/**
	 * Instanz der Klasse {@link StatusLeisteVerwaltung} fr den Singleton-Test
	 */
	private static StatusLeisteVerwaltung _singleton = null;

	/**
	 * Speichert eine Instanz der Klasse {@link StatusLeisteVerwaltung}
	 */
	private static StatusLeisteVerwaltung _verwaltung = null;

	private static String _xmlStruktur = null;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		_verwaltung = StatusLeisteVerwaltung.getInstanz();

		_xmlStruktur = "";
		_xmlStruktur = _xmlStruktur
				+ "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
		_xmlStruktur = _xmlStruktur + "<statusleiste>\n";
		_xmlStruktur = _xmlStruktur
				+ "\t<element id=\"sl.zeitelement\" bezeichnung=\"Zeitelement\" />\n";
		_xmlStruktur = _xmlStruktur + "</statusleiste>";

	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
		leereVerwaltung();
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
		/*
		 * Lschen der Testdatei
		 */
		File file = new File(_pfadTestDatei);
		if (file.exists())
		{
			if (file.isFile())
			{
				file.delete();
			}
		}

		/*
		 * Leeren der Verwaltung
		 */
		leereVerwaltung();
	}

	/**
	 * Entfernt die Elemente der XML-Struktur aus der gespeicherten
	 * StatusLeisteVerwaltung-Instanz
	 */
	private void leereVerwaltung()
	{
		/*
		 * Vor jedem Test muss die Liste der Statusleistenelemente geleert werden.
		 */
		Iterator<Entry<String, StatusLeisteElement>> it = _verwaltung
				.getVerfuegbareStatusLeistenElemente().entrySet().iterator();

		while (it.hasNext())
		{
			_verwaltung.entferneElement(it.next().getValue());
		}

		/*
		 * Die Elemente mssen aus der JDOM-Struktur entfernt werden
		 */
		LinkedList<Element> loeschen = new LinkedList<Element>();
		if (_verwaltung.getListeXmlStatusleisteElemente() != null)
		{
			Iterator<Element> itEle = _verwaltung.getListeXmlStatusleisteElemente()
					.iterator();
			while (itEle.hasNext())
			{
				loeschen.add(itEle.next());
			}

			Iterator<Element> itLoeschen = loeschen.iterator();
			while (itLoeschen.hasNext())
			{
				_verwaltung.loescheXmlElement(itLoeschen.next());
			}
		}

	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#getInstanz()}.
	 * <br>
	 * Prfung auf Singleton
	 */
	@Test
	public void testGetInstanz()
	{
		Thread thread1 = new Thread(new SingletonRun());
		Thread thread2 = new Thread(new SingletonRun());
		thread1.start();
		thread2.start();

		try
		{
			thread1.join();
			thread2.join();
		} catch (InterruptedException e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#hinzufuegenElement(de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement)}.
	 * <br>
	 * Prfen, ob ein Element auch hinzugefgt wurde.
	 */
	@Test
	public void testHinzufuegenElement()
	{
		String id = Activator.PLUGIN_ID + ".testContributionItemHinzufuegen";
		ContributionItem item = new ContributionItem(id);
		StatusLeisteElement element = new StatusLeisteElement(id, item);

		_verwaltung.hinzufuegenElement(element);
		StatusLeisteElement aktuell = _verwaltung.getStatusLeistenElement(id);

		assertSame(element, aktuell);
	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#entferneElement(de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement)}.
	 * <br>
	 * Prfen ob sich ein {@link StatusLeisteElement} auch wieder entfernen lsst.
	 */
	@Test
	public void testEntferneElement()
	{
		String id = Activator.PLUGIN_ID + ".testContributionItemEntfernen";
		ContributionItem item = new ContributionItem(id);
		StatusLeisteElement element = new StatusLeisteElement(id, item);

		_verwaltung.hinzufuegenElement(element);
		_verwaltung.entferneElement(element);

		boolean nichtMehrVorhanden = false;

		try
		{
			StatusLeisteElement statusLeistenElement = _verwaltung
					.getStatusLeistenElement(id);
			System.out.println("ID: " + statusLeistenElement.getId());
		} catch (IllegalArgumentException iae)
		{
			nichtMehrVorhanden = true;
		}

		assertEquals(true, nichtMehrVorhanden);
	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#leseSortierungAusXmlString(java.lang.String)}.
	 */
	@Test
	public void testLeseSortierungAusXmlString()
	{
		_verwaltung.leseSortierungAusXmlString(_xmlStruktur);
		List<Element> listeXmlStatusleisteElemente = _verwaltung
				.getListeXmlStatusleisteElemente();
		Iterator<Element> it = listeXmlStatusleisteElemente.iterator();
		String wert = null;
		while (it.hasNext())
		{
			Element element = it.next();
			if (element.getName().equals("element"))
			{
				wert = element.getAttribute("id").getValue();
			}
		}

		assertEquals("sl.zeitelement", wert);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#leseSortierungAusXmlDatei(java.lang.String)}.
	 */
	@Test
	public void testLeseSortierungAusXmlDatei()
	{
		try
		{
			FileWriter fw = new FileWriter(_pfadTestDatei);
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(_xmlStruktur);
			bw.close();
			fw.close();

			_verwaltung.leseSortierungAusXmlDatei(_pfadTestDatei);
			List<Element> listeXmlStatusleisteElemente = _verwaltung
					.getListeXmlStatusleisteElemente();
			Iterator<Element> it = listeXmlStatusleisteElemente.iterator();
			String wert = null;
			while (it.hasNext())
			{
				Element element = it.next();
				if (element.getName().equals("element"))
				{
					wert = element.getAttribute("id").getValue();
					break;
				}
			}

			assertEquals("sl.zeitelement", wert);
		} catch (IOException e)
		{
			e.printStackTrace();
			fail("Die Testdatei " + _pfadTestDatei
					+ " konnte nicht erstellt werden: " + e.toString());
		}
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#schreibeSortierungInXmlString()}.
	 */
	@Test
	public void testSchreibeSortierungInXmlString()
	{
		_verwaltung.leseSortierungAusXmlString(_xmlStruktur);

		/*
		 * XML-String aus der Verwaltung holen und merken
		 */
		String xmlString = _verwaltung.schreibeSortierungInXmlString();

		/*
		 * Verwaltung wieder leeren
		 */
		leereVerwaltung();

		/*
		 * Gemerkten XML-String in die Verwaltung einlesn
		 */
		_verwaltung.leseSortierungAusXmlString(xmlString);

		/*
		 * Prfen, ob das Element vorhanden ist, auslesen und den Wert der ID
		 * vergleichen
		 */
		List<Element> listeXmlStatusleisteElemente = _verwaltung
				.getListeXmlStatusleisteElemente();
		Iterator<Element> it = listeXmlStatusleisteElemente.iterator();
		String wert = null;
		while (it.hasNext())
		{
			Element element = it.next();
			if (element.getName().equals("element"))
			{
				wert = element.getAttribute("id").getValue();
				break;
			}
		}

		assertEquals("sl.zeitelement", wert);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#schreibeSortierungInXmlDatei(java.lang.String, boolean)}.
	 */
	@Test
	public void testSchreibeSortierungInXmlDatei()
	{
		/*
		 * Sortierung einlesen
		 */
		_verwaltung.leseSortierungAusXmlString(_xmlStruktur);

		/*
		 * In Datei schreiben
		 */
		_verwaltung.schreibeSortierungInXmlDatei(_pfadTestDatei, true);

		/*
		 * Verwaltung leeren
		 */
		leereVerwaltung();

		/*
		 * Aus Datei einlesen
		 */
		_verwaltung.leseSortierungAusXmlDatei(_pfadTestDatei);

		/*
		 * Prfen, ob das Element vorhanden ist, auslesen und den Wert der ID
		 * vergleichen
		 */
		List<Element> listeXmlStatusleisteElemente = _verwaltung
				.getListeXmlStatusleisteElemente();
		Iterator<Element> it = listeXmlStatusleisteElemente.iterator();
		String wert = null;
		while (it.hasNext())
		{
			Element element = it.next();
			if (element.getName().equals("element"))
			{
				wert = element.getAttribute("id").getValue();
				break;
			}
		}

		assertEquals("sl.zeitelement", wert);
	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#aktualisieren()}.
	 */
	@Test
	public void testAktualisieren()
	{
		ZuhoererStatusLeisteAenderung zuhoererStatusLeisteAenderung = new ZuhoererStatusLeisteAenderung();
		_verwaltung.hinzufuegenZuhoerer(zuhoererStatusLeisteAenderung);

		_verwaltung.aktualisieren();

		EreignisStatusLeisteGeaendert ereignisStatusLeisteGeaendert = zuhoererStatusLeisteAenderung
				.getEreignisStatusLeisteGeaendert();

		_verwaltung.entferneZuhoerer(zuhoererStatusLeisteAenderung);

		assertEquals(true, ereignisStatusLeisteGeaendert != null);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#hinzufuegenXmlElement(org.jdom.Element)}.
	 */
	@Test
	public void testHinzufuegenXmlElement()
	{
		String elementName = "element";
		String attIdName = "id";
		String attValue = "test123";

		Element element = new Element(elementName);
		element.setAttribute(new Attribute(attIdName, attValue));

		_verwaltung.hinzufuegenXmlElement(element);

		Element pruefElement = null;
		Iterator<Element> it = _verwaltung.getListeXmlStatusleisteElemente()
				.iterator();
		while (it.hasNext())
		{
			pruefElement = it.next();
			if (pruefElement.getAttributeValue(attIdName).equals(attValue))
			{
				break;
			}
		}

		assertEquals(element, pruefElement);
	}

	/**
	 * Testethode fr
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#verschiebeXmlElement(org.jdom.Element, org.jdom.Element)}.
	 */
	@Test
	public void testVerschiebeXmlElementElementElement()
	{
		/*
		 * Testelemente erzeugen
		 */
		String elementName = "element";
		String attIdName = "id";

		String attValueElement1 = "test1";
		String attValueElement2 = "test2";
		String attValueElement3 = "test3";

		Element element1 = new Element(elementName);
		element1.setAttribute(new Attribute(attIdName, attValueElement1));

		Element element2 = new Element(elementName);
		element2.setAttribute(new Attribute(attIdName, attValueElement2));

		Element element3 = new Element(elementName);
		element3.setAttribute(new Attribute(attIdName, attValueElement3));

		/*
		 * Hinzufgen der Elemente zur Verwaltung
		 */
		_verwaltung.hinzufuegenXmlElement(element1);
		_verwaltung.hinzufuegenXmlElement(element2);
		_verwaltung.hinzufuegenXmlElement(element3);

		/*
		 * Aktuelle Reihenfolge: test1, test2, test3 Verschieben
		 */
		Element a = _verwaltung.getListeXmlStatusleisteElemente().get(0);
		Element b = _verwaltung.getListeXmlStatusleisteElemente().get(2);
		_verwaltung.verschiebeXmlElement(a, b);

		/*
		 * Aktuelle Reihenfolge: test2, test3, test1 Verschieben
		 */
		a = _verwaltung.getListeXmlStatusleisteElemente().get(0);
		b = _verwaltung.getListeXmlStatusleisteElemente().get(1);
		_verwaltung.verschiebeXmlElement(a, b);

		/*
		 * Reihenfolge jetzt: test3, test2, test1
		 */

		List<Element> liste = _verwaltung.getListeXmlStatusleisteElemente();
		System.out.println("SIZE: " + liste.size());

		Iterator<Element> it = liste.iterator();
		while (it.hasNext())
		{
			System.out.println("==> " + it.next().getAttributeValue("id"));
		}

		boolean richtigeReihenfolge = false;
		if ((liste.get(0).getAttributeValue(attIdName).equals(element3
				.getAttributeValue(attIdName)))
				&& (liste.get(1).getAttributeValue(attIdName).equals(element2
						.getAttributeValue(attIdName)))
				&& (liste.get(2).getAttributeValue(attIdName).equals(element1
						.getAttributeValue(attIdName))))
		{
			richtigeReihenfolge = true;
		}

		assertEquals(true, richtigeReihenfolge);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#istXmlElementKindVon(org.jdom.Element, org.jdom.Element)}.
	 */
	@Test
	public void testIstXmlElementKindVon()
	{
		/*
		 * Prfung ist Kindelement
		 */
		boolean istKindElement = false;
		Element elternElement = new Element("Elternelement");
		Element kindElement = new Element("Kindelement");
		elternElement.addContent(kindElement);
		istKindElement = _verwaltung.istXmlElementKindVon(kindElement,
				elternElement);

		/*
		 * Prfung ist NICHT Kindelement
		 */
		boolean istNichtKindElement = false;
		Element elternElement2 = new Element("Elternelement");
		if (!_verwaltung.istXmlElementKindVon(kindElement, elternElement2))
		{
			istNichtKindElement = true;
		}

		assertEquals(true, istKindElement && istNichtKindElement);
	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#loescheXmlElement(org.jdom.Element)}.
	 */
	@Test
	public void testLoescheXmlElement()
	{
		/*
		 * Testelemente erzeugen
		 */
		String elementName = "element";
		String attIdName = "id";

		String attValueElement1 = "test1";
		String attValueElement2 = "test2";
		String attValueElement3 = "test3";

		Element element1 = new Element(elementName);
		element1.setAttribute(new Attribute(attIdName, attValueElement1));

		Element element2 = new Element(elementName);
		element2.setAttribute(new Attribute(attIdName, attValueElement2));

		Element element3 = new Element(elementName);
		element3.setAttribute(new Attribute(attIdName, attValueElement3));

		/*
		 * Hinzufgen der Elemente zur Verwaltung
		 */
		_verwaltung.hinzufuegenXmlElement(element1);
		_verwaltung.hinzufuegenXmlElement(element2);
		_verwaltung.hinzufuegenXmlElement(element3);

		/*
		 * Lschen des 2 Elementes
		 */
		_verwaltung.loescheXmlElement(element2);

		/*
		 * Prfen, ob das element2 nicht mehr vorhanden ist
		 */
		boolean nichtMehrVorhanden = true;

		Iterator<Element> it = _verwaltung.getListeXmlStatusleisteElemente()
				.iterator();
		while (it.hasNext())
		{
			Element pruefElement = it.next();
			if (pruefElement.getName().equals(elementName))
			{
				String attributWert = pruefElement.getAttributeValue(attIdName);
				if (attributWert.equals(attValueElement2))
				{
					nichtMehrVorhanden = false;
					break;
				}
			}
		}

		assertEquals(true, nichtMehrVorhanden);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#aktionUndo()}.
	 */
	@Test
	public void testAktionUndo()
	{
		/*
		 * Testelemente erzeugen
		 */
		String elementName = "element";
		String attIdName = "id";

		String attValueElement1 = "test1";
		String attValueElement2 = "test2";
		String attValueElement3 = "test3";

		Element element1 = new Element(elementName);
		element1.setAttribute(new Attribute(attIdName, attValueElement1));

		Element element2 = new Element(elementName);
		element2.setAttribute(new Attribute(attIdName, attValueElement2));

		Element element3 = new Element(elementName);
		element3.setAttribute(new Attribute(attIdName, attValueElement3));

		/*
		 * Hinzufgen der Elemente zur Verwaltung
		 */
		_verwaltung.hinzufuegenXmlElement(element1);
		_verwaltung.hinzufuegenXmlElement(element2);
		_verwaltung.hinzufuegenXmlElement(element3);

		/*
		 * Lschen des 2 Elementes
		 */
		_verwaltung.loescheXmlElement(element2);

		/*
		 * Rckgngig machen des Lschens
		 */
		_verwaltung.aktionUndo();

		boolean elementWiederVorhanden = false;
		Iterator<Element> it = _verwaltung.getListeXmlStatusleisteElemente()
				.iterator();
		while (it.hasNext())
		{
			Element pruefElement = it.next();
			if (pruefElement.getName().equals(elementName))
			{
				String attributWert = pruefElement.getAttributeValue(attIdName);
				if (attributWert.equals(attValueElement2))
				{
					elementWiederVorhanden = true;
					break;
				}
			}
		}

		/*
		 * Prfen ob nach dem UNDO element2 wieder vorhanden ist
		 */
		assertEquals(true, elementWiederVorhanden);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung#aktionRedo()}.
	 */
	@Test
	public void testAktionRedo()
	{
		/*
		 * Testelemente erzeugen
		 */
		String elementName = "element";
		String attIdName = "id";

		String attValueElement1 = "test1";
		String attValueElement2 = "test2";
		String attValueElement3 = "test3";

		Element element1 = new Element(elementName);
		element1.setAttribute(new Attribute(attIdName, attValueElement1));

		Element element2 = new Element(elementName);
		element2.setAttribute(new Attribute(attIdName, attValueElement2));

		Element element3 = new Element(elementName);
		element3.setAttribute(new Attribute(attIdName, attValueElement3));

		/*
		 * Hinzufgen der Elemente zur Verwaltung
		 */
		_verwaltung.hinzufuegenXmlElement(element1);
		_verwaltung.hinzufuegenXmlElement(element2);
		_verwaltung.hinzufuegenXmlElement(element3);

		/*
		 * Lschen des 2 Elementes
		 */
		_verwaltung.loescheXmlElement(element2);

		/*
		 * Rckgngig machen des Lschens
		 */
		_verwaltung.aktionUndo();

		/*
		 * Erneutes lschen mittel REDO-Operation
		 */
		_verwaltung.aktionRedo();

		/*
		 * Prfen, ob das element2 nach Lschen / Redo / Undo wieder nicht mehr
		 * vorhanden ist
		 */
		boolean nachRedoWiederNichtVorhanden = true;

		Iterator<Element> it = _verwaltung.getListeXmlStatusleisteElemente()
				.iterator();
		while (it.hasNext())
		{
			Element pruefElement = it.next();
			if (pruefElement.getName().equals(elementName))
			{
				String attributWert = pruefElement.getAttributeValue(attIdName);
				if (attributWert.equals(attValueElement2))
				{
					nachRedoWiederNichtVorhanden = false;
					break;
				}
			}
		}

		assertEquals(true, nachRedoWiederNichtVorhanden);
	}

	/**
	 * Testklasse zum Prfen von Singletons
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SingletonRun implements Runnable
	{

		@Override
		public void run()
		{
			StatusLeisteVerwaltung s = StatusLeisteVerwaltung.getInstanz();
			synchronized (StatusLeisteVerwaltung.class)
			{
				if (_singleton == null)
				{
					_singleton = s;
				}
				assertSame(s, _singleton);
			}
			{

			}

		}

	}

	/**
	 * Implementier ein {@link IContributionItem} fr den Test.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ContributionItem implements IContributionItem
	{
		/**
		 * ID
		 */
		private String _id = null;

		/**
		 * Konstruktor
		 * 
		 * @param id
		 *          ID dieses {@link ContributionItem}
		 */
		private ContributionItem(String id)
		{
			this._id = id;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void fill(Composite parent)
		{
		}

		@Override
		public void fill(Menu parent, int index)
		{
		}

		@Override
		public void fill(ToolBar parent, int index)
		{
		}

		@Override
		public void fill(CoolBar parent, int index)
		{
		}

		@Override
		public String getId()
		{
			return _id;
		}

		@Override
		public boolean isDirty()
		{
			return false;
		}

		@Override
		public boolean isDynamic()
		{
			return false;
		}

		@Override
		public boolean isEnabled()
		{
			return false;
		}

		@Override
		public boolean isGroupMarker()
		{
			return false;
		}

		@Override
		public boolean isSeparator()
		{
			return false;
		}

		@Override
		public boolean isVisible()
		{
			return false;
		}

		@Override
		public void saveWidgetState()
		{
		}

		@Override
		public void setParent(IContributionManager parent)
		{
		}

		@Override
		public void setVisible(boolean visible)
		{
		}

		@Override
		public void update()
		{
		}

		@Override
		public void update(String id)
		{
		}

	}

	/**
	 * Listener-Implementierung, mit dessen Hilfe sich das auftretende Ereignis
	 * "einfangen" lsst.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ZuhoererStatusLeisteAenderung implements
			IZuhoererStatusLeisteAenderung
	{
		private EreignisStatusLeisteGeaendert _ereignis = null;

		@Override
		public void ausfuehren(EreignisStatusLeisteGeaendert e)
		{
			_ereignis = e;
		}

		public EreignisStatusLeisteGeaendert getEreignisStatusLeisteGeaendert()
		{
			EreignisStatusLeisteGeaendert re = null;

			re = _ereignis;

			return re;
		}

	}

}
