/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu;

import static org.junit.Assert.assertEquals;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.bsvrz.buv.rw.rw.Activator;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;

/**
 * Test der {@link ElementRegistrierung}. <br>
 * <b>Wichtig:</b> Dieser Test kann nur als PDE JUnit-Test erfolgen!
 * 
 * @author HofmannJ
 * 
 */
public class ElementRegistrierungTest
{
	private ElementRegistrierung _elementRegistrierung = null;

	@Before
	public void setUp() throws Exception
	{
		_elementRegistrierung = ElementRegistrierung.getInstanz();
	}

	@After
	public void tearDown() throws Exception
	{
	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.menu.ElementRegistrierung#registriereDeskriptorElement(de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement)}.
	 */
	@Test
	public void testRegistriereDeskriptorElement()
	{

		String pluginId = Activator.PLUGIN_ID;
		String actionId = Activator.PLUGIN_ID
				+ ".JUnit.ElementRegistrierungTest.registriereDeskriptorElement";
		String labelText = "JUnit AktionElement fr den Registrierungs Test";
		String toolTippText = "Ein Tooltip";
		String actionDescription = "Beschreibung der Action";
		String iconPfad = "Ein beliebiger String fr den IconPfad";
		String hoverIconPfad = "Ein beliebiger String fr den HoverIconPfad";
		String disabledIconPfad = "Ein beliebiger String fr den DisabledIconPfad";
		boolean ausklappbar = false;

		DeskriptorElement deskriptorElement = new DeskriptorElement(pluginId,
				actionId, labelText, toolTippText, actionDescription, iconPfad,
				hoverIconPfad, disabledIconPfad, ausklappbar);

		_elementRegistrierung.registriereDeskriptorElement(deskriptorElement);

		List<DeskriptorElement> deskriptoren = _elementRegistrierung
				.getPluginElementDeskriptoren(pluginId);

		DeskriptorElement aktuell = null;

		Iterator<DeskriptorElement> it = deskriptoren.iterator();
		while (it.hasNext())
		{
			DeskriptorElement element = it.next();
			if (element.getAktionID().equals(actionId))
			{
				aktuell = element;
				break;
			}
		}

		assertEquals(deskriptorElement, aktuell);
	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.menu.ElementRegistrierung#registriereDeskriptorAktion(de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion)}.
	 */
	@Test
	public void testRegistriereDeskriptorAktion()
	{
		String pluginId = Activator.PLUGIN_ID;
		String actionID = Activator.PLUGIN_ID
				+ ".JUnit.ElementRegistrierungTest.registriereDeskriptorAktion";
		Action action = new Action()
		{
			public void run()
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Test ElementRegistrierung");
				mb
						.setMessage("Es wird die Methode registriereDeskriptorAktion getestet");
			}
		};
		action.setId(actionID);
		action.setText("Test ElementRegistrierung - registriereDeskriptorAktion");

		DeskriptorAktion deskriptorAktion = new DeskriptorAktion(pluginId, action,
				false);
		_elementRegistrierung.registriereDeskriptorAktion(deskriptorAktion);

		List<DeskriptorAktion> deskriptoren = _elementRegistrierung
				.getPluginAktionDeskriptoren(pluginId);
		DeskriptorAktion aktuell = null;
		Iterator<DeskriptorAktion> it = deskriptoren.iterator();
		while (it.hasNext())
		{
			DeskriptorAktion aktion = it.next();
			if (aktion.getAktion().getId().equals(actionID))
			{
				aktuell = aktion;
				break;
			}
		}

		assertEquals(deskriptorAktion, aktuell);

	}

	/**
	 * Testmethode fr
	 * {@link de.bsvrz.buv.rw.rw.menu.ElementRegistrierung#registriereDeskriptorAktionGlobal(de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion)}.
	 */
	@Test
	public void testRegistriereDeskriptorAktionGlobal()
	{
		String pluginId = Activator.PLUGIN_ID;
		String actionID = Activator.PLUGIN_ID
				+ ".JUnit.ElementRegistrierungTest.registriereDeskriptorAktionGlobal";

		Action action = new Action()
		{
			public void run()
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Test ElementRegistrierung");
				mb
						.setMessage("Es wird die Methode registriereDeskriptorAktionGlobal getestet");
			}
		};
		action.setId(actionID);
		action
				.setText("Test ElementRegistrierung - registriereDeskriptorAktionGlobal");

		DeskriptorAktion deskriptorAktionGlobal = new DeskriptorAktion(pluginId,
				action, true);
		_elementRegistrierung
				.registriereDeskriptorAktionGlobal(deskriptorAktionGlobal);

		List<DeskriptorAktion> deskriptoren = _elementRegistrierung
				.getPluginAktionDeskriptorenGlobal(pluginId);
		DeskriptorAktion aktuell = null;
		Iterator<DeskriptorAktion> it = deskriptoren.iterator();
		while (it.hasNext())
		{
			DeskriptorAktion aktion = it.next();
			if (aktion.getAktion().getId().equals(actionID))
			{
				aktuell = aktion;
			}

		}

		assertEquals(deskriptorAktionGlobal, aktuell);
	}

}
