/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw;

import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.ArbeitsbereichAktivStatus;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.TrayHilfsklasse;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.buv.rw.rw.offlinekonfiguration.BetriebsArtAuswerten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Erweitert den Erweiterungspunkt org.eclipse.ui.startup. <br>
 * 
 * <u>Aufgaben:</u> <br>
 * Diese Klasse wird erst aktiviert, wenn die Basisinitialisierung der
 * Eclipse-RCP vollzogen wurde.
 * <ul>
 * <li>Dadurch ist z.B. auch der Zugriff auf die Balloon-ToolTips von TrayItems
 * mglicht.<br>
 * Diese Klasse setzt ein entsprechendes Flag in der Klasse
 * {@link ArbeitsbereichAktivStatus}, so dass die entsprechenden Methoden der
 * Klasse {@link DaVVerbindung} einen BalloonToolTip zum TrayItem setzen knnen.
 * </li>
 * <li>Des weiteren wird die Auswertung der Betriebsart ber
 * {@link BetriebsArtAuswerten} gestartet.</li>
 * </ul>
 * 
 * 
 * 
 * 
 * @version $Revision: 1.3 $ / $Date: 2012/07/12 09:55:44 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class Start implements IStartup
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Instanz der Klasse {@link ArbeitsbereichAktivStatus}.
	 */
	private ArbeitsbereichAktivStatus aktivStatus = null;

	public void earlyStartup()
	{

		_debug
				.fine(getClass().getName()
						+ "::earlyStartup \n\t=>setzeArbeitsbereichAktiv in DaVVerbindung wurde mit dem Parameter true aufgerufen.\n");

		aktivStatus = ArbeitsbereichAktivStatus.getInstanz();

		aktivStatus.setArbeitsbereichAktiv(true);
		if (aktivStatus.getArbeitsbereichAktiv())
		{

			_debug.fine(getClass().getName()
					+ "::earlyStartup \n\t=> Der Arbeitsbereich wurde in der Klasse "
					+ aktivStatus.getClass().getName() + " als aktiv markiert.\n");

		} else
		{
			_debug.fine(getClass().getName()
					+ "::earlyStartup \n\t=>Der Arbeitsbereich konnte in der Klasse "
					+ aktivStatus.getClass().getName()
					+ " nicht ordnungsgem als aktiv markiert werden!\n");
		}

		@SuppressWarnings("unused")
		BetriebsArtAuswerten instanz = BetriebsArtAuswerten.getInstanz(); // Muss
		// bleiben!

		/*
		 * Tray bei Bedarf nachinitialisieren
		 */	
			final Display display = PlatformUI.getWorkbench().getDisplay();

			if (display != null)
			{
				Runnable run = new Runnable()
				{

					@Override
					public void run()
					{
						TrayHilfsklasse.getInstance().init(display);
					}
				};

				display.asyncExec(run);
			} else
			{
				System.out
						.println("Start.earlyStartup() -> Tray nicht initialisierbar, Display ist null!");
			}


		/*
		 * Prelader fr Informationen aus dem DaV starten
		 */
		DaVPreLader.getInstanz();

		_debug.fine(getClass().getName() + "+=> StartUP beendet");
	}
}
