/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav;

import java.io.File;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.Activator;
import de.bsvrz.buv.rw.rw.dav.TrayHilfsklasse.Farbe;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.ApplicationCloseActionHandler;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.DavConnectionListener;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;

/**
 * Diese Singleton-Klasse verwaltet die Verbindung zum DaV. <br>
 * Es knnen sich Listener vom Typ {@link IDaVVerbindungZuhoerer} registrieren,
 * um ber nderungen an der Verbindung zum DaV informiert zu werden.
 * 
 * @version $Revision: 1.9 $ / $Date: 2012/05/30 11:02:55 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class DaVVerbindung implements Runnable
{
	/**
	 * Der Debugger
	 */
	private static Debug _debug = Debug.getLogger();

	/**
	 * Flag, zeigt ann dass zum 1.sten Mal gestartet wurde.
	 */
	private boolean _ersterStart = true;

	/**
	 * IP-Adresse des DaV
	 */
	private String _ip = null;

	/**
	 * Port des DaV
	 */
	private int _port = -1;

	/**
	 * DaV Benutzername
	 */
	private String _benutzer = null;

	/**
	 * DaV Passwort
	 */
	private String _passwort = null;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _benutzerVerbindung = null;

	/**
	 * Urlasser DaV Benutzername
	 */
	private String _urlasserBenutzer = null;

	/**
	 * Urlasser DaV Passwort
	 */
	private String _urlasserPasswort = null;

	/**
	 * Urlaserverbindung zum Dav.
	 */
	private ClientDavInterface _urlasserVerbindung = null;

	/**
	 * Gibt die aktuelle Verbindugn zum DaV an.
	 */
	private ClientDavInterface _ausgangsVerbindung = null;

	// /**
	// * Flag, zeigt an ob die Verbindung zum DaV aktiv ist oder nicht.
	// */
	// private boolean _verbunden = false;

	/**
	 * Singleton-Instanz dieser Klasse.
	 */
	private static DaVVerbindung _instanz = null;

	/**
	 * Liste der Zuhrer, die ber den Zustand der Verbindung zum DaV informiert
	 * werden wollen.
	 */
	private List<IDaVVerbindungZuhoerer> _lZuhoerer = null;

	/**
	 * 
	 */
	private boolean _arbeitsBereichAktiv = false;

	/**
	 * Flag dafr, dass der Thread laufen soll.
	 */
	private boolean _laufeThread = false;

	/**
	 * Zeigt an, ob der Infobereich aktiv ist.
	 */
	private boolean _infoBereichAktiv = false;

	/**
	 * Liste derjenigen Klassen die das Interface
	 * {@link IDaVVerbindungsAbbauWarten} implementieren und erst geprft werden
	 * mssen, bevor die Verbindung abgebaut wird.
	 */
	private LinkedList<IDaVVerbindungsAbbauWarten> _verbindungsabbauWarten = new LinkedList<IDaVVerbindungsAbbauWarten>();

	/**
	 * Flag das anzeigt ob es ok ist, die DaV-Verbindung zu trennen oder nicht.
	 */
	public boolean _davVerbindungTrennenOk = true;

	private Display _display = null;

	private boolean _workbenchListenerAdded = false;

	private int counter = 0;

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link DaVVerbindung}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link DaVVerbindung}.
	 */
	public static DaVVerbindung getInstanz()
	{
		_debug.fine("DaVVerbindung => Liefere Instanz");
		if (_instanz == null)
		{
			synchronized (DaVVerbindung.class)
			{
				if (_instanz == null)
				{
					_instanz = new DaVVerbindung();
					_debug.fine("DaVVerbindung => Instanz erzeugt");
				}
			}
		}

		if (ArbeitsbereichAktivStatus.getInstanz().getArbeitsbereichAktiv())
		{
			_instanz.setzeArbeitsbereich(true);
		}

		return _instanz;
	}

	/**
	 * Privater Konstruktor.
	 * 
	 */
	private DaVVerbindung()
	{
		init();
		TrayHilfsklasse.getInstance();
	}

	/**
	 * Fhrt die Initialisierung durch.
	 * 
	 */
	private void init()
	{
		if (ArbeitsbereichAktivStatus.getInstanz().getArbeitsbereichAktiv())
		{
			setzeArbeitsbereich(true);
		}

		_lZuhoerer = new LinkedList<IDaVVerbindungZuhoerer>();

		// Tray-Item erzeugen
		infoBereichDeaktivieren();
	}

	/**
	 * Verbindungsaufbau zum DaV.
	 * 
	 * @return Liefert <code>true</code>, wenn die Verbindung zum DaV aufgebaut
	 *         werden konnte, sonst <code>false</code>.
	 */
	public boolean baueVerbindungMitDaVAuf()
	{
		_debug.fine(getClass().getName() + " => baueVerbindungMitDaVAuf()");

		_benutzerVerbindung = verbindeMitDav();

		if (_benutzerVerbindung != null)
		{
			/*
			 * DaV-Verbindung wurd aufgebaut
			 */

			infoBereichAktivieren();
			if (_urlasserVerbindung == null)
			{
				_ausgangsVerbindung = _benutzerVerbindung;
			}
			benachrichtigeZuhoererAufbau("Verbindungsaufbau erfolgreich!");

			// Message Sender fr Betriebsmeldungen initialisieren,
			// sobald Verbindung aufgebaut ist!
			ClientApplication app = _benutzerVerbindung.getLocalApplicationObject();
			MessageSender instance = MessageSender.getInstance();
			instance.init(_benutzerVerbindung, "", app.getName());

			return true;
		} else
		{
			// Kein DaV-Verbindungsaufbau

			infoBereichDeaktivieren();

			return false;
		}

	}

	public boolean baueUrlasserVerbindungMitDaVAuf()
	{
		_debug.fine(getClass().getName() + " => baueUrlasserVerbindungMitDaVAuf()");
		_urlasserVerbindung = verbindeUrlasserMitDav();

		if (_urlasserVerbindung != null)
		{
			_debug.fine(getClass().getName()
					+ " => UrlasserVerbindung zum DaV wurde erfolgreich aufgebaut.");
			infoBereichUrlasserAktivieren();

			_ausgangsVerbindung = _urlasserVerbindung;
			benachrichtigeZuhoererAufbau("Verbindungsaufbau als Urlasser erfolgreich!");
			return true;
		} else
		{
			_debug
					.fine(getClass().getName()
							+ " => Es konnte keine Urlasser-Verbindung zum DaV aufgebaut werden.");

			if (_benutzerVerbindung != null)
			{
				infoBereichAktivieren();
			} else
			{
				infoBereichDeaktivieren();
			}

			return false;
		}
	}

	@SuppressWarnings("unused")
	private void DaVAbbruchsMeldung()
	{
		_debug.fine("==>VOR MELDUNG");

		_display = Display.getDefault();

		_display.syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				MessageBox mb = new MessageBox(_display.getActiveShell(), SWT.OK
						| SWT.ICON_ERROR | SWT.APPLICATION_MODAL);

				mb.setText("Fehler!");
				mb.setMessage("Die DaV-Verbindung wurde getrennt!");
				int open = mb.open();

				if (open > 0)
				{
					_debug.fine("OK");
				}
				_debug.fine("RUN2");
			}
		});
		_debug.fine("NACH MELDUNG <==");
	}

	/**
	 * Baut die Verbindung zum DaV auf.
	 * 
	 * @return Verbindung zum DaV.
	 */
	private ClientDavInterface verbindeMitDav()
	{
		try
		{

			/*
			 * Initialisierung der ClientDaVParameter der Verbindung zum DaV erfolgt
			 * mit Hilfe der Kommandozeilenparameter.
			 */
			String[] commandLineArgs = Platform.getCommandLineArgs();
			LinkedList<String> lArgs = new LinkedList<String>();

			for (String arg : commandLineArgs)
			{
				lArgs.add(arg);
			}

			String[] aArgs = new String[lArgs.size()];
			int i = 0;
			for (String arg : lArgs)
			{
				aArgs[i] = arg;
				i++;
			}

			ClientDavParameters cdp = new ClientDavParameters(aArgs);

			_benutzerVerbindung = new ClientDavConnection(cdp);
		} catch (MissingParameterException e)
		{
			e.printStackTrace();
		}

		// IP setzen
		_benutzerVerbindung.getClientDavParameters()
				.setDavCommunicationAddress(_ip);

		// Port setzen
		_benutzerVerbindung.getClientDavParameters().setDavCommunicationSubAddress(
				_port);

		/*
		 * Applikationsname setzen
		 */
		String aktuelleAdresse = "";

		try
		{
			aktuelleAdresse = "_auf_" + InetAddress.getLocalHost().getHostAddress();
		} catch (UnknownHostException e1)
		{
		}

		if ((BenutzerAngemeldet.getInstanz().getName() != null)
				&& (BenutzerAngemeldet.getInstanz().getName().length() > 0))
		{
			aktuelleAdresse = aktuelleAdresse + "_Benutzer_"
					+ BenutzerAngemeldet.getInstanz().getName();
		}

		_benutzerVerbindung.getClientDavParameters().setApplicationName(
				"Rahmenwerk" + aktuelleAdresse);

		/*
		 * Kommandozeilenparameter holen. Suchen des bentigten Parameters
		 * -ConfigurationPathBenutzer
		 */
		KmdParameter parameter = Kommandozeilenparameter.getInstanz()
				.getKommandoZeilenParameter("-ConfigurationPathBenutzer");

		if (parameter != null)
		{
			String pfad = parameter.getWert();

			File testFile = new File(pfad);
			if ((testFile.exists()) && (testFile.isDirectory()))
			{
				_benutzerVerbindung.getClientDavParameters().setConfigurationPath(pfad);
			} else
			{
				String msg = "Der beim Parameter -ConfigurationPathBenutzer angegeben Pfad \""
						+ pfad + "\"existiert nicht.";

				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
			}
		}

		/*
		 * ApplikationsTyp bernehmen
		 */
		KmdParameter parameterApplikationsTyp = Kommandozeilenparameter
				.getInstanz().getKommandoZeilenParameter("-applicationTypePid");

		if (parameterApplikationsTyp != null)
		{
			String pid = parameterApplikationsTyp.getWert();
			_benutzerVerbindung.getClientDavParameters().setApplicationTypePid(pid);
		}

		// Verbinden
		try
		{
			_benutzerVerbindung.connect();
			_davVerbindungTrennenOk = false;

			_benutzerVerbindung.setCloseHandler(new ApplicationCloseActionHandler()
			{

				public void close(String error)
				{
					_benutzerVerbindung = null;
					_ausgangsVerbindung = null;
					thStart();
					benachrichtigeZuhoererAbbau("DaV weggebrochen: " + error);

					counter++;

					error = counter + " " + error;

					FehlerMeldung fe = new FehlerMeldung();
					if (fe.meldungDaVGetrennt(error))
					{
					}

					// DaVAbbruchsMeldung();

					infoBereichDeaktivieren();
				}

			});

			DavConnectionListener dcl = new DavConnectionListener()
			{

				public void connectionClosed(ClientDavInterface arg0)
				{
					_debug
							.fine(getClass().getName()
									+ "::verbindeMitDaV \n\t=>Die Verbindung zum DaV wurde geschlossen.\n");
				}

			};
			_benutzerVerbindung.addConnectionListener(dcl);

		} catch (CommunicationError e)
		{
			String msg = "Kommunikationsfehler beim Aufbau der DaV-Verbindung. Es wird im Offlinemodus gearbeitet.";

			if (ArbeitsbereichAktivStatus.getInstanz().getArbeitsbereichAktiv())
			{
				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID, msg, e));
			}

			FehlerMeldung fe = new FehlerMeldung();
			if (fe.meldungDaVGetrennt(e.getMessage()))
			{
			}
			_benutzerVerbindung = null;
		} catch (ConnectionException e)
		{
			FehlerMeldung fe = new FehlerMeldung();
			if (fe.meldungKeinVerbindungsaufbau(e))
			{
			}
			_benutzerVerbindung = null;
		}

		// Benutzer und Passwort setzen
		if (_benutzerVerbindung != null)
		{
			try
			{
				_benutzerVerbindung.login(_benutzer, _passwort);
			} catch (InconsistentLoginException e)
			{
				FehlerMeldung fe = new FehlerMeldung();
				if (fe.meldungKeinLoginBenutzer(e))
				{
				}
				_benutzerVerbindung = null;
			} catch (CommunicationError e)
			{
				FehlerMeldung fe = new FehlerMeldung();
				if (fe.meldungKeinLoginBenutzer(e))
				{
				}

				_benutzerVerbindung = null;
			}
		}

		return _benutzerVerbindung;
	}

	/**
	 * Baut die Urlasser Verbindung zum DaV auf.
	 * 
	 * @return Verbindung zum DaV.
	 */
	private ClientDavInterface verbindeUrlasserMitDav()
	{
		try
		{
			_urlasserVerbindung = new ClientDavConnection();
			_urlasserVerbindung.setCloseHandler(new ApplicationCloseActionHandler()
			{

				public void close(String error)
				{
					if (_benutzerVerbindung != null)
					{
						_urlasserVerbindung = null;
						_ausgangsVerbindung = _benutzerVerbindung;
						benachrichtigeZuhoererAufbau("Urlasserverbindung Abbgebrochen!");

						infoBereichAktivieren();
					} else
					{
						_urlasserVerbindung = null;
						_ausgangsVerbindung = null;
						infoBereichDeaktivieren();
						benachrichtigeZuhoererAbbau("Urlasser + Benutzerverbindung weggebrochen");
					}

					FehlerMeldung fe = new FehlerMeldung();
					if (fe.meldungDaVGetrennt(error))
					{
					}
				}

			});

		} catch (MissingParameterException e)
		{
			e.printStackTrace();
		}

		// IP setzen
		_urlasserVerbindung.getClientDavParameters()
				.setDavCommunicationAddress(_ip);

		// Port setzen
		_urlasserVerbindung.getClientDavParameters().setDavCommunicationSubAddress(
				_port);

		/*
		 * Kommandozeilenparameter holen.
		 */
		String[] kommandozeilenArgumente = Platform.getCommandLineArgs();

		/*
		 * Suchen des bentigten Parameters -ConfigurationPathBenutzer
		 */
		for (int i = 0; i < kommandozeilenArgumente.length; i++)
		{
			if (kommandozeilenArgumente[i]
					.equalsIgnoreCase("-ConfigurationPathUrlasser"))
			{
				i++;
				String pfad = kommandozeilenArgumente[i];

				File testFile = new File(pfad);
				if ((testFile.exists()) && (testFile.isDirectory()))
				{
					_urlasserVerbindung.getClientDavParameters().setConfigurationPath(
							pfad);
					_debug
							.fine("Benutze zum Zwischenspeichern von Konfigurationsdaten fr den Urlasser den Pfad: "
									+ pfad);
				} else
				{
					_debug
							.error("Der beim Parameter -ConfigurationPathUrlasser angegeben Pfad existiert nicht.");
				}

				break;
			}
		}

		// Verbinden
		try
		{
			_urlasserVerbindung.connect();
			DavConnectionListener dclUrlasser = new DavConnectionListener()
			{

				public void connectionClosed(ClientDavInterface connection)
				{
					_debug
							.fine(getClass().getName()
									+ "::verbindeUrlasserMitDaV \n\t=>Urlasser-Verbindung zum DaV wurde geschlossen.\n");
				}
			};
			_urlasserVerbindung.addConnectionListener(dclUrlasser);

		} catch (CommunicationError e)
		{
			_debug.info(getClass().getName()
					+ "::verbindeUrlasserMitDav \n\t=> Kommunikationsfehler: "
					+ e.toString());

			FehlerMeldung fe = new FehlerMeldung();
			if (fe.meldungDaVGetrennt(e.getMessage()))
			{
			}

			_urlasserVerbindung = null;
		} catch (ConnectionException e)
		{
			_debug.info(getClass().getName()
					+ "::verbindeUrlasserMitDav \n\t => Verbindungsausnahmefehler: "
					+ e.toString() + "\n");

			FehlerMeldung fe = new FehlerMeldung();
			if (fe.meldungKeinVerbindungsaufbau(e))
			{
			}
			_urlasserVerbindung = null;
		}

		// Benutzer und Passwort setzen
		if (_urlasserVerbindung != null)
		{
			try
			{
				_urlasserVerbindung.login(_urlasserBenutzer, _urlasserPasswort);
			} catch (InconsistentLoginException e)
			{
				_debug
						.error(getClass().getName()
								+ "::verbindeUrlasserMitDav \n\t=> Falsche Urlasser Zugangsdaten zum DaV: "
								+ e.toString() + "\n");

				_urlasserVerbindung = null;
			} catch (CommunicationError e)
			{
				_debug
						.error(getClass().getName()
								+ "::verbindeUrlasserMitDav \n\t=> Kommunikationsfehler bei bergabe der Zugangsdaten: "
								+ e.toString() + "\n");
				_urlasserVerbindung = null;
			}
		}

		return _urlasserVerbindung;
	}

	/**
	 * Trennt die Verbindung zum DaV. Dies ist nur mglich, wenn keine
	 * Urlasser-Verbindung zum DaV aktiv ist.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn keine Benutzerverbindung zum
	 *         DaV mehr existiert bzw. wenn alle Verbindungen zum DaV geschlossen
	 *         werden konnten.
	 */
	public boolean trenneDaVVerbindung()
	{
		if (!istTrennenOk())
		{
			// System.out.println("1");
			_debug.fine("DaVVerbindung.trenneDaVVerbindung() => Nicht ok");

			DaVVerbindungTrennThread th = DaVVerbindungTrennThread.getInstanz();
			th.starteThread();

			synchronized (th)
			{
				try
				{
					th.wait();
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}

			// thStartSynchron();
			return true;
		} else
		{
			// System.out.println("2");
			_debug.fine("DaVVerbindung.trenneDaVVerbindung() => TRENNE");
			if (_urlasserVerbindung == null)
			{
				if (_benutzerVerbindung != null)
				{
					System.out.println("Vor Trennen");
					_benutzerVerbindung.disconnect(false, null);
					_benutzerVerbindung = null;
					_ausgangsVerbindung = null;
					_debug.fine(getClass().getName()
							+ " => Verbindung zum DaV wurde getrennt.");

					/*
					 * Variablen zurcksetzen
					 */
					_ip = null;
					_port = -1;
					_benutzer = null;
					_passwort = null;

					benachrichtigeZuhoererAbbau("Verbindung wurde getrennt!");

					infoBereichDeaktivieren();

					return true;
				} else
				{
					_debug
							.fine(getClass().getName()
									+ "::trenneDaVVerbindung \n\t=>Es existiert keine Benutzerverbindung zum DaV, die getrennt werden kann.\n");
					return true;
				}
			} else
			{
				_debug
						.error(getClass().getName()
								+ "::trenneDaVVerbindung\n\t=> Verbindung kann nicht getrennt werden, Urlasserverbindung zum DaV ist noch aktiv.  \n");
				return false;
			}

		}

	}

	/**
	 * Trennt die Verbindung zum DaV. Dies ist nur mglich, wenn keine
	 * Urlasser-Verbindung zum DaV aktiv ist.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn keine Benutzerverbindung zum
	 *         DaV mehr existiert bzw. wenn alle Verbindungen zum DaV geschlossen
	 *         werden konnten.
	 */
	// public boolean trenneDaVVerbindung()
	// {
	// if (!_davVerbindungTrennenOk)
	// // if(!istTrennenOk())
	// {
	// System.out.println("1");
	// _debug.fine("DaVVerbindung.trenneDaVVerbindung() => Nicht ok");
	// // DaVVerbindungTrennThread th = new DaVVerbindungTrennThread();
	// DaVVerbindungTrennThread th = DaVVerbindungTrennThread.getInstanz();
	// th.starteThread();
	//			
	//			
	// // thStartSynchron();
	// return false;
	// }
	// else
	// {
	// System.out.println("2");
	// _debug.fine("DaVVerbindung.trenneDaVVerbindung() => TRENNE");
	// if (_urlasserVerbindung == null)
	// {
	// if (_benutzerVerbindung != null)
	// {
	// System.out.println("Vor Trennen");
	// _benutzerVerbindung.disconnect(false, null);
	// _benutzerVerbindung = null;
	// _ausgangsVerbindung = null;
	// _debug.fine(getClass().getName()
	// + " => Verbindung zum DaV wurde getrennt.");
	// System.out.println("Nach Trennen");
	//
	// /*
	// * Variablen zurcksetzen
	// */
	// _ip = null;
	// _port = -1;
	// _benutzer = null;
	// _passwort = null;
	//
	// benachrichtigeZuhoererAbbau("Verbindung wurde getrennt!");
	//
	// infoBereichDeaktivieren();
	//
	// return true;
	// }
	// else
	// {
	// _debug
	// .fine(getClass().getName()
	// + "::trenneDaVVerbindung \n\t=>Es existiert keine Benutzerverbindung zum
	// DaV, die getrennt werden kann.\n");
	// return true;
	// }
	// }
	// else
	// {
	// _debug
	// .error(getClass().getName()
	// + "::trenneDaVVerbindung\n\t=> Verbindung kann nicht getrennt werden,
	// Urlasserverbindung zum DaV ist noch aktiv. \n");
	// return false;
	// }
	//
	// }
	//
	// }
	/**
	 * Trennt die Verbindung zum DaV. Dies ist nur mglich, wenn keine
	 * Urlasser-Verbindung zum DaV aktiv ist.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn keine Benutzerverbindung zum
	 *         DaV mehr existiert bzw. wenn alle Verbindungen zum DaV geschlossen
	 *         werden konnten.
	 */
	public boolean trenneDaVVerbindungSWTThreadsicher()
	{
		if (!_davVerbindungTrennenOk)
		{
			_debug.fine("DaVVerbindung.trenneDaVVerbindung() => Nicht ok");
			// DaVVerbindungTrennThread th = new DaVVerbindungTrennThread();
			DaVVerbindungTrennThread th = DaVVerbindungTrennThread.getInstanz();
			th.starteThread();
			// thStartSynchron();
			return false;
		} else
		{
			_debug.fine("DaVVerbindung.trenneDaVVerbindung() => TRENNE");
			if (_urlasserVerbindung == null)
			{
				if (_benutzerVerbindung != null)
				{
					_benutzerVerbindung.disconnect(false, null);
					_benutzerVerbindung = null;
					_ausgangsVerbindung = null;
					_debug.fine(getClass().getName()
							+ " => Verbindung zum DaV wurde getrennt.");

					/*
					 * Variablen zurcksetzen
					 */
					_ip = null;
					_port = -1;
					_benutzer = null;
					_passwort = null;

					benachrichtigeZuhoererAbbau("Verbindung wurde getrennt!");

					infoBereichDeaktivieren();

					return true;
				} else
				{
					_debug
							.fine(getClass().getName()
									+ "::trenneDaVVerbindung \n\t=>Es existiert keine Benutzerverbindung zum DaV, die getrennt werden kann.\n");
					return true;
				}
			} else
			{
				_debug
						.error(getClass().getName()
								+ "::trenneDaVVerbindung\n\t=> Verbindung kann nicht getrennt werden, Urlasserverbindung zum DaV ist noch aktiv.  \n");
				return false;
			}

		}

	}

	/**
	 * Trennt die Urlasserverbindung zum DaV.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Urlasser-Verbindung zum
	 *         DaV beendet wurde.
	 */
	public boolean trenneUrlasserDaVVerbindung()
	{
		if (_urlasserVerbindung != null)
		{
			_urlasserVerbindung.disconnect(false, null);
			_urlasserVerbindung = null;
			_debug
					.fine(getClass().getName()
							+ "::trenneUrlasserDaVVerbindung \n\t=> UrlasserVerbindung zum DaV wurde getrennt.\n");

			/*
			 * Variablen zurcksetzen
			 */
			_urlasserBenutzer = null;
			_urlasserPasswort = null;

			if (_benutzerVerbindung != null)
			{
				_ausgangsVerbindung = _benutzerVerbindung;
				benachrichtigeZuhoererAufbau("Urlasser getrennt, alte Benutzerverbindung wieder aktiv");
				infoBereichAktivieren();
			} else
			{
				_ausgangsVerbindung = null;
				benachrichtigeZuhoererAbbau("Urlasser + Benutzerverbindung wurden getrennt.");
				infoBereichDeaktivieren();
			}
			return true;
		}
		return true;
	}

	/**
	 * Trennt die Urlasserverbindung zum DaV.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Urlasser-Verbindung zum
	 *         DaV beendet wurde.
	 */
	public boolean trenneUrlasserDaVVerbindungSWTThreadsicher()
	{
		if (_urlasserVerbindung != null)
		{
			_urlasserVerbindung.disconnect(false, null);
			_urlasserVerbindung = null;
			_debug
					.fine(getClass().getName()
							+ "::trenneUrlasserDaVVerbindung \n\t=> UrlasserVerbindung zum DaV wurde getrennt.\n");

			/*
			 * Variablen zurcksetzen
			 */
			_urlasserBenutzer = null;
			_urlasserPasswort = null;

			if (_benutzerVerbindung != null)
			{
				_ausgangsVerbindung = _benutzerVerbindung;
				benachrichtigeZuhoererAufbau("Urlasser getrennt, alte Benutzerverbindung wieder aktiv");
				infoBereichAktivieren();
			} else
			{
				_ausgangsVerbindung = null;
				benachrichtigeZuhoererAbbau("Urlasser + Benutzerverbindung wurden getrennt.");
				infoBereichDeaktivieren();
			}
			return true;
		}
		return true;
	}

	/**
	 * Trennt alle Verbindungen zum DaV.
	 * 
	 * @return Liefert <code>true</code>, wenn Benutzer- und Urlasserverbindung
	 *         zum DaV getrennt werden konnten, sonst <code>false</code>.
	 */
	public boolean trenneAlleVerbindungen()
	{
		boolean urlasserVerbindungGeschlossen = false;
		boolean benutzerVerbindungGeschlossen = false;

		if (_urlasserVerbindung != null)
		{
			urlasserVerbindungGeschlossen = trenneUrlasserDaVVerbindung();
		}
		if (_benutzerVerbindung != null)
		{
			benutzerVerbindungGeschlossen = trenneDaVVerbindung();
		}

		if (urlasserVerbindungGeschlossen && benutzerVerbindungGeschlossen)
		{
			benachrichtigeZuhoererAbbau("Alle Verbindungen wurden getrennt.");
			return true;
		} else
		{
			return false;
		}
	}

	/**
	 * Trennt alle Verbindungen zum DaV.
	 * 
	 * @return Liefert <code>true</code>, wenn Benutzer- und Urlasserverbindung
	 *         zum DaV getrennt werden konnten, sonst <code>false</code>.
	 */
	public boolean trenneAlleVerbindungenSWTThreadsicher()
	{
		boolean urlasserVerbindungGeschlossen = false;
		boolean benutzerVerbindungGeschlossen = false;

		if (_urlasserVerbindung != null)
		{
			urlasserVerbindungGeschlossen = trenneUrlasserDaVVerbindungSWTThreadsicher();
		}
		if (_benutzerVerbindung != null)
		{
			benutzerVerbindungGeschlossen = trenneDaVVerbindungSWTThreadsicher();
		}

		if (urlasserVerbindungGeschlossen && benutzerVerbindungGeschlossen)
		{
			benachrichtigeZuhoererAbbau("Alle Verbindungen wurden getrennt.");
			return true;
		} else
		{
			return false;
		}
	}

	/**
	 * Prft, ob die Verbindung zum DaV aktiv ist.
	 * 
	 * @return Liefert <code>true</code>, wenn die Verbindung zum DaV aktiv ist,
	 *         sonst <code>false</code>.
	 */
	public boolean istDaVVerbindungAktiv()
	{
		// if (_benutzerConnection != null)
		if (_ausgangsVerbindung != null)
		{
			return true;
		} else
		{
			return false;
		}
	}

	/**
	 * Liefert die Verbindung zum DaV oder <code>null</code>, wenn keine
	 * Verbindung zum DaV exitiert.
	 * 
	 * @return DaV-Verbindung.
	 */
	public ClientDavInterface getDaVVerbindung()
	{
		// return this._benutzerConnection;
		return this._ausgangsVerbindung;
	}

	/**
	 * Fgt einen Zuhrer, der ber den Status der Verbindung zum DaV informiert
	 * werden mchte, zur Liste der Zuhrer hinzu.
	 * 
	 * @param zuhoerer
	 *          Zuhrer, der zur Liste der Zuhrer hinzugefgt werden soll.
	 */
	public void hinzufuegenZuhoerer(IDaVVerbindungZuhoerer zuhoerer)
	{
		synchronized (_lZuhoerer)
		{
			_lZuhoerer.add(zuhoerer);
		}
	}

	/**
	 * Entfernt einen Zuhrer von der Liste der Zuhrer.
	 * 
	 * @param zuhoerer
	 *          Zu entfernender Zuhrer.
	 */
	public void entferneZuhoerer(IDaVVerbindungZuhoerer zuhoerer)
	{
		synchronized (_lZuhoerer)
		{
			_lZuhoerer.remove(zuhoerer);
		}
	}

	/**
	 * Benachrichtigt alle Zuhrer darber, dass sich am Status der Verbindung zum
	 * DaV etwas gendert hat.
	 * 
	 * @param grundVerbindungsAufbau
	 *          Grund fr den Verbindungsaufbau
	 * 
	 */
	private void benachrichtigeZuhoererAufbau(String grundVerbindungsAufbau)
	{
		Iterator<IDaVVerbindungZuhoerer> it = _lZuhoerer.iterator();

		// Zuhrer darber informieren, dass eine Verbindung zum DaV besteht.

		DaVVerbindungEreignis event = new DaVVerbindungEreignis(this,
				this._benutzerVerbindung, this._ip, this._port, this._benutzer,
				this._urlasserBenutzer, grundVerbindungsAufbau);
		while (it.hasNext())
		{
			IDaVVerbindungZuhoerer zuhoerer = it.next();
			zuhoerer.verbindungAufgebaut(event);
		}
	}

	/**
	 * Benachrichtigt alle Zuhrer darber, dass sich am Status der Verbindung zum
	 * DaV etwas gendert hat.
	 * 
	 */
	private void benachrichtigeZuhoererAbbau(String grund)
	{
		synchronized (_lZuhoerer)
		{
			Iterator<IDaVVerbindungZuhoerer> it = _lZuhoerer.iterator();

			// Zuhrer darber informieren, dass keine Verbindung zum DaV
			// besteht.
			DaVVerbindungEreignis event = new DaVVerbindungEreignis(this, grund);
			while (it.hasNext())
			{
				IDaVVerbindungZuhoerer zuhoerer = it.next();
				zuhoerer.verbindungAbgebaut(event);
			}
		}

	}

	/**
	 * Fgt einen Klasse zur Liste der Klassen hinzu die geprft werden mssen,
	 * bevor die Verbindung abgebaut werden kann.
	 * 
	 * @param warten
	 *          Instanz einer Klasse, die das Interface
	 *          {@link IDaVVerbindungsAbbauWarten} implementiert.
	 */
	public void hinzufuegenVerbindungsabbauWarten(
			IDaVVerbindungsAbbauWarten warten)
	{
		_verbindungsabbauWarten.add(warten);

		if (_verbindungsabbauWarten.size() > 0)
		{
			_davVerbindungTrennenOk = false;
		}
	}

	/**
	 * Entfernt eine Klasse von der Liste der Klassen die geprft werdne mssen,
	 * bevor die Verbindung abgebaut werden kann.
	 * 
	 * @param warten
	 *          Instanz einer Klasse, die das Interface
	 *          {@link IDaVVerbindungsAbbauWarten} implementiert.
	 */
	public void entfernenVerbindungsabbauWarten(IDaVVerbindungsAbbauWarten warten)
	{
		_verbindungsabbauWarten.remove(warten);

		if (_verbindungsabbauWarten.size() == 0)
		{
			_davVerbindungTrennenOk = true;
		}
	}

	/**
	 * Holt fr die DaV-Verbindung automatisch die Verbindungsdaten aus
	 * {@link BenutzerAngemeldet}.
	 * 
	 * @return Liefert <code>false</code>, falls aus {@link BenutzerAngemeldet}
	 *         die notwendigen Daten nicht erfolgreich bezogen werden konnten oder
	 *         bereits eine Benutzerverbindung zum DaV besteht, sonst
	 *         <code>true</code>.
	 */
	public boolean holeBenutzerDaVVerbindung()
	{
		boolean returnWert = false;
		if (_benutzerVerbindung == null)
		{
			// Benutzerinstanz holen
			BenutzerAngemeldet benutzer = BenutzerAngemeldet.getInstanz();

			// Daten auslesen und auf Komplettheit prfen
			String name = benutzer.getName();
			String passwort = benutzer.getPasswort();
			String ip = benutzer.getIp();
			int tcpPort = benutzer.getTcpPort();

			if ((name != null) && (!name.equals("")))
			{
				if ((passwort != null) && (!passwort.equals("")))
				{
					if ((ip != null) && (!ip.equals("")))
					{
						if (tcpPort >= 0)
						{
							this._benutzer = name;
							this._passwort = passwort;
							this._ip = ip;
							this._port = tcpPort;
							returnWert = true;
						} else
						{
							_debug.info(getClass().getName()
									+ "::holenBenutzerDaVVerbindung \n\t=>In "
									+ benutzer.getClass().getName()
									+ " wurde kein TCP-Port gesetzt. Abbruch!\n");
						}
					} else
					{
						_debug.info(getClass().getName()
								+ "::holenBenutzerDaVVerbindung \n\t=>In "
								+ benutzer.getClass().getName()
								+ " wurde keine IP gesetzt. Abbruch!\n");
					}
				} else
				{
					_debug.info(getClass().getName()
							+ "::holenBenutzerDaVVerbindung \n\t=>In "
							+ benutzer.getClass().getName()
							+ " wurde kein Passwort gesetzt. Abbruch!\n");
				}
			} else
			{
				_debug.info(getClass().getName()
						+ "::holenBenutzerDaVVerbindung \n\t=>In "
						+ benutzer.getClass().getName()
						+ " wurde kein Benutzer gesetzt. Abbruch!\n");
			}

		} else
		{
			_debug
					.info(getClass().getName()
							+ "::holenBenutzerDaVVerbindung \n\t=> Es ist eine Benutzerverbindung zum DaV aktiv. Setze keine neuen Benutzerdaten. Abbruch!\n");
		}

		return returnWert;
	}

	/**
	 * Holt fr die Urlasser-DaV-Verbindung automatisch die Verbindungsdaten aus
	 * {@link UrlasserAngemeldet}.
	 * 
	 * @return Liefert <code>flase</code>, falls aus {@link UrlasserAngemeldet}
	 *         die notwendigen Daten nicht erfolgreich bezogen werden konnten oder
	 *         bereits eine Urlasserverbindung zum DaV besteht, sonst
	 *         <code>true</code>.
	 */
	public boolean holeUrlasserDaVVerbindung()
	{
		boolean returnWert = false;

		if (_urlasserVerbindung == null)
		{
			UrlasserAngemeldet urlasserAngemeldet = UrlasserAngemeldet.getInstanz();
			String urlasserName = urlasserAngemeldet.getName();
			String urlasserPasswort = urlasserAngemeldet.getPasswort();

			if ((urlasserName != null) && (!urlasserName.equals("")))
			{
				if ((urlasserPasswort != null) && (!urlasserPasswort.equals("")))
				{
					this._urlasserBenutzer = urlasserName;
					this._urlasserPasswort = urlasserPasswort;

				} else
				{
					_debug.info(getClass().getName()
							+ "::holenUrlasserDaVVerbindung \n\t=>In "
							+ urlasserAngemeldet.getClass().getName()
							+ " wurde kein UrlasserPasswort gesetzt. Abbruch! \n");
				}
			} else
			{
				_debug.info(getClass().getName()
						+ "::holenUrlasserDaVVerbindung \n\t=>In "
						+ urlasserAngemeldet.getClass().getName()
						+ " wurde kein UrlasserBenutzer gesetzt. Abbruch! \n");
			}

		} else
		{
			_debug
					.error(getClass().getName()
							+ "::holenUrlasserDaVVerbindung \n\t=>Es ist eine Urlasserverbindung zum DaV aktiv. Setze keine neuen Urlasserdaten. Abbruch! \n");
		}

		return returnWert;
	}

	/**
	 * Liefert die IP-Adresse vom DaV.
	 * 
	 * @return DaV IP-Adresse.
	 */
	public String getDaVIp()
	{
		return _ip;
	}

	/**
	 * Setzt die IP-Adresse vom DaV, solange noch keine Verbindung existiert.
	 * 
	 * @param daVIp
	 *          DaV IP-Adresse.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 */
	public void setDaVIp(String daVIp) throws IllegalArgumentException
	{
		if (_benutzerVerbindung == null)
		{
			_debug.fine(getClass().getName() + " => Ip gesetzt.");

			_ip = daVIp;
		} else
		{
			throw new IllegalArgumentException(
					"Die IP-Adresse des DaV kann nur gesetzt werden, wenn keine Verbindung zum DaV besteht.");
		}
	}

	/**
	 * Liefert den Port vom DaV.
	 * 
	 * @return DaV-Port.
	 */
	public int getDaVPort()
	{
		return _port;
	}

	/**
	 * Setzt den DaV-Port, solange noch keine Verbindung exisitert.
	 * 
	 * @param daVPort
	 *          DaV-Port.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 */
	public void setDaVPort(int daVPort) throws IllegalArgumentException
	{
		if (_benutzerVerbindung == null)
		{
			_port = daVPort;
		} else
		{
			throw new IllegalArgumentException(
					"Der Port des DaV kann nur gesetzt werden, wenn keine Verbindung zum DaV besteht.");
		}
	}

	/**
	 * Liefert den DaV Benutzernamen.
	 * 
	 * @return DaV Benutzernamen
	 */
	public String getDaVBenutzer()
	{
		return _benutzer;
	}

	/**
	 * Setzt den DaV Benutzernamen, solange noch keine Verbindung zum DaV
	 * existiert.
	 * 
	 * @param daVBenutzer
	 *          DaV Benutzernamen.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 */
	public void setDaVBenutzer(String daVBenutzer)
			throws IllegalArgumentException
	{
		if (_benutzerVerbindung == null)
		{
			_benutzer = daVBenutzer;
		} else
		{
			throw new IllegalArgumentException(
					"Der Benutzername kann nur gesetzt werden, wenn keine Verbindung zum DaV besteht.");
		}
	}

	/**
	 * Liefert das Passwort vom DaV.
	 * 
	 * @return DaV Passwort.
	 */
	public String getDaVPasswort()
	{
		return _passwort;
	}

	/**
	 * Setzt das Passwort zum DaV, solange noch keine Verbindung zum DaV
	 * existiert.
	 * 
	 * @param daVPasswort
	 *          DaV Passwort.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 */
	public void setDaVPasswort(String daVPasswort)
			throws IllegalArgumentException
	{
		if (_benutzerVerbindung == null)
		{
			_passwort = daVPasswort;
		} else
		{
			throw new IllegalArgumentException(
					"Das Passwort kann nur gesetzt werden, solange keine Verbindung zum DaV besteht");
		}
	}

	public String getUrlasserBenutzer()
	{
		return _urlasserBenutzer;
	}

	public void setUrlasserBenutzer(String benutzer)
	{
		if (_urlasserVerbindung == null)
		{
			_urlasserBenutzer = benutzer;
		} else
		{
			_debug
					.error(getClass().getName()
							+ "::setUrlasserBenutzer + \n\t=>Der UrlasserBenutzer kann nur gesetzt werden, wenn keine Urlasserverbindung zum DaV besteht. \n");
		}

	}

	public String getUrlasserPasswort()
	{
		return _urlasserPasswort;
	}

	public void setUrlasserPasswort(String passwort)
	{
		if (_urlasserVerbindung == null)
		{
			_urlasserPasswort = passwort;
		} else
		{
			_debug
					.error(getClass().getName()
							+ "::setUrlasserPasswoer + \n\t=>Das UrlasserPasswort kann nur gesetzt werden, wenn keine Urlasserverbindung zum DaV besteht. \n");
		}
	}

	/**
	 * Stellt das TrayItem in Farbe dar und zeigt damit eine aktive Verbindung zum
	 * DaV an. Liefert als ToolTip einen Text zu IP, Port und DaV-Benutzer.
	 * 
	 */
	private void infoBereichAktivieren()
	{
		TrayHilfsklasse.getInstance().setMeldung(
				Farbe.gruen,
				"DaV-Verbindung aufgebaut",
				"Verbunden mit DaV " + _ip + " auf \nPort " + _port + " als Benutzer "
						+ _benutzer + ".");
	}

	/**
	 * Stellt das TrayItem in Schwarzweis dar und zeigt damit keine aktive
	 * Verbindung zum DaV an. <br>
	 * Als ToolTip kommt der Hinweis, dass man mit keinem DaV verbunden ist.
	 * 
	 */
	private void infoBereichDeaktivieren()
	{
		TrayHilfsklasse
				.getInstance()
				.setMeldung(
						Farbe.sw,
						"Keine DaV Verbinung aktiv",
						"Es ist keine Verbindung zum DaV aktiv.\nDiese wurde entweder getrennt oder\nwar noch nicht aktiv.");

		if (_ersterStart)
		{
			_ersterStart = false;
		}

	}

	/**
	 * Schaltet den Infobereich ab.
	 */
	public void infoBereichAbschalten()
	{
		TrayHilfsklasse.getInstance().dispose();
	}

	private void infoBereichUrlasserAktivieren()
	{
		String messageUrlasser = "Verbunden mit DaV " + _ip + " auf Port " + _port
				+ " als\nUrlasser " + _urlasserBenutzer + ".\n";

		String messageBenutzer = "Verbunden mit DaV " + _ip + " auf Port " + _port
				+ " als\nBenutzer " + _benutzer + ".";

		String msg = "Zwei DaV-Verbindungen sind aktiv:\n" + messageUrlasser
				+ messageBenutzer;

		TrayHilfsklasse.getInstance().setMeldung(Farbe.rot,
				"Urlasserverbindung mit DaV aufgebaut.", msg);
	}

	public void setzeArbeitsbereich(boolean wert)
	{
		_arbeitsBereichAktiv = wert;

		if (_arbeitsBereichAktiv)
		{
			if (!_workbenchListenerAdded)
			{
				PlatformUI.getWorkbench().addWorkbenchListener(new IWorkbenchListener()
				{

					@Override
					public boolean preShutdown(IWorkbench workbench, boolean forced)
					{
						return true;
					}

					@Override
					public void postShutdown(IWorkbench workbench)
					{

						TrayHilfsklasse.getInstance().setMeldung(Farbe.rot,
								"Rahmenwerk fhrt herunter",
								"Das Rahmenwerk wird gerade beendet.");

						TrayHilfsklasse.getInstance().dispose();

					}
				});

				_workbenchListenerAdded = true;
			}
		}
	}

	public boolean istArbeitsbereichAktiv()
	{
		return _arbeitsBereichAktiv;
	}

	private void thStart()
	{
		if (!_laufeThread)
		{
			_laufeThread = true;
			Thread th = new Thread(this);
			th.setName("Aufrum-Thread");

			Display.getDefault().asyncExec(th);

			// th.start();
		} else
		{
			_debug.fine("Es ist bereits ein Thread aktiv!");
		}
	}

	public void run()
	{
		while (_laufeThread)
		{
			_debug.fine("Thread aktiv!");

			if (_urlasserVerbindung == null)
			{
				_debug.fine("URLASSER NULL");
				if (_benutzerVerbindung != null)
				{
					// infoBereichAktivieren();
				}
			}
			if (_benutzerVerbindung == null)
			{
				_debug.fine("BENUTER NULL");
				if (_infoBereichAktiv)
				{
					infoBereichDeaktivieren();
					_infoBereichAktiv = false;
				}
			}

			try
			{
				Thread.sleep(2000);
				_laufeThread = false;

			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}

		}

	}

	/**
	 * Diese Methode durchluft die Liste der {@link IDaVVerbindungsAbbauWarten}
	 * und prft, ob die Verbindung schon getrennt werden kann.
	 * 
	 * @return <code>True</code> wenn die Verbindung getrennt werden kann, sonst
	 *         <code>false</code>.
	 */
	protected boolean istTrennenOk()
	{
		boolean returnWert = true;

		if (_verbindungsabbauWarten.size() == 0)
		{
			_debug
					.fine("DaVVerbindung.istTrennenOk() => LISTE ENTHLT KEINE KINDER!");

			returnWert = true;
		} else
		{
			_debug.fine("DaVVerbindung.istTrennenOk() => Anzahl Kinder in Liste: "
					+ _verbindungsabbauWarten.size());
			_debug.fine("DaVVerbindung.istTrennenOk() => DURCHLAUFE LISTE");
			Iterator<IDaVVerbindungsAbbauWarten> it = _verbindungsabbauWarten
					.iterator();
			while (it.hasNext())
			{
				IDaVVerbindungsAbbauWarten warten = it.next();
				if (!warten.DaVVerbindungsAbbauOk())
				{
					returnWert = false;
					break;
				}
			}
		}

		return returnWert;
	}

	/**
	 * Produziert die Fehlermeldungen bei Fehlerhaften Verbindungsaufbau und
	 * Verbindungsabbruch.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class FehlerMeldung
	{

		/**
		 * Rckgabewert einer Fehlermeldung.
		 */
		private boolean _re;

		private static final int MODALITY = SWT.SYSTEM_MODAL;

		public FehlerMeldung()
		{
		}

		public boolean meldungDaVGetrennt(final String meldung)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{

				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();

					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					mb.setText("DaV-Verbindung getrennt");
					mb
							.setMessage("Die DaV-Verbindung wurde getrennt. Es wird im Offline-Modus gearbeitet\nGrund: "
									+ meldung);
					int open = mb.open();

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		@SuppressWarnings("unused")
		public boolean meldungDaVGetrenntUrlasser(final String meldung)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();

					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					mb.setText("DaV-Verbindung des Urlassers getrennt");
					mb
							.setMessage("Die DaV-Verbindung des Urlassers wurde getrennt.\nGrund: "
									+ meldung);
					int open = mb.open();

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		@SuppressWarnings("unused")
		public boolean meldungKommunikationsFehler(final String meldung)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{

				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();

					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					mb.setText("Kommunikationsfehler");
					mb
							.setMessage("Es ist ein Kommunikationsfehler mit dem DaV aufgetreten.\nGrund: "
									+ meldung);
					int open = mb.open();

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		@SuppressWarnings("unused")
		public boolean meldungKommunikationsFehlerUrlasser(final String meldung)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{

				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();

					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					mb.setText("Kommunikationsfehler");
					mb
							.setMessage("Es ist ein Kommunikationsfehler mit dem DaV bei der Urlasserverbindung aufgetreten.\nGrund: "
									+ meldung);
					int open = mb.open();

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		public boolean meldungKeinVerbindungsaufbau(final Exception e)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{

				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();

					String meldungsText = "Es konnte keine Verbindung mit dem DaV aufgebaut werden. Es wird im Offlinemodus gearbeitet.";

					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					mb.setText("Kein Verbindungsaufbau mglich");
					mb.setMessage(meldungsText + "\nGrund: " + e.getMessage());
					int open = mb.open();

					Activator.getDefault().getLog().log(
							new Status(Status.WARNING, Activator.PLUGIN_ID, Status.OK,
									meldungsText, e));

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		@SuppressWarnings("unused")
		public boolean meldungKeinVerbindungsaufbauUrlasser(final String meldung)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();

					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					mb.setText("Kein Verbindungsaufbau mglich");
					mb
							.setMessage("Es konnte keine Urlasser-Verbindung mit dem DaV aufgebaut werden.\nGrund: "
									+ meldung);
					int open = mb.open();

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		public boolean meldungKeinLoginBenutzer(final Exception e)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{

				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();

					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					String meldungsText = "Der Benutzer konnte nicht eingeloggt werden.";
					mb.setText("Kein Login des Benutzers mglich");
					mb.setMessage(meldungsText + "\nGrund: " + e.getMessage());
					int open = mb.open();

					Activator.getDefault().getLog().log(
							new Status(Status.WARNING, Activator.PLUGIN_ID, Status.OK,
									meldungsText, e));

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		@SuppressWarnings("unused")
		public boolean meldungKeinLoginUrlasser(final String meldung)
		{
			_re = false;

			final Display display = Display.getDefault();

			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					Shell shell = erzeugeShell(display);
					shell.open();
					MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR
							| MODALITY);

					mb.setText("Kein Login des Urlassers mglich");
					mb.setMessage("Der Urlasser konnte nicht eingeloggt werden.\nGrund: "
							+ meldung);
					int open = mb.open();

					if (open > 0)
					{
						_re = true;
						shell.dispose();
					}

					while (!shell.isDisposed())
					{
						if (!display.readAndDispatch())
							display.sleep();
					}
				}
			};

			display.syncExec(run);

			return _re;
		}

		/**
		 * Erzeugt Eine {@link Shell}
		 * 
		 * @return Liefert die neu erzeugte {@link Shell}.
		 */
		private Shell erzeugeShell(Display display)
		{
			Shell shell = new Shell(display, SWT.APPLICATION_MODAL);
			shell.setSize(0, 0);
			shell.setVisible(false);
			return shell;
		}

	}

}
