/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav;

import java.util.EventObject;

import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Liefert den Event der ans die Zuhrer der DaV-Verbindung durch deren Listener {@link IDaVVerbindungZuhoerer}
 * bertragen wird. <br>
 * Dadruch kann z.B. die Verbindung oder der Grund fr den Verbindugnsabbau abgefragt werden.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:02 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class DaVVerbindungEreignis extends EventObject
{
  /**
   * DaV-Adresse
   */
  private String _ip = null;

  /**
   * DaV-Port
   */
  private int _port = -1;

  /**
   * Benutzer-Name
   */
  private String _benutzer = null;

  /**
   * Urlasser-Name
   */
  private String _urlasser = null;

  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection = null;

  /**
   * Gibt den Grund fr den Verbindungsabbau an
   */
  private String _grundVerbindungsabbau = null;

  /**
   * Gibt den Grund fr den Verbindungsaufbau an
   */
  private String _grundVerbindungsaufbau = null;

  /**
   * Generierte serielle Id
   */
  private static final long serialVersionUID = -3901539737900085638L;

  /**
   * Konstruktor, wenn eine Verbindung aufgebaut wurde.
   * 
   * @param source
   *          Eventauslsendes Objekt.
   * @param ipAdresse
   *          IP-Adresse der DaV-Verbindung.
   * @param port
   *          Port der DaV-Verbindung.
   * @param benutzer
   *          Benutzer der DaV-Verbindung.
   * @param urlasser
   *          Urlasser-Benutzer der DaV-Verbindung.
   * @param grundVerbindungsAufbau
   *          Grund fr den Verbindungsaufbau.
   */
  public DaVVerbindungEreignis(Object source, ClientDavInterface connection, String ipAdresse, int port,
      String benutzer, String urlasser, String grundVerbindungsAufbau)
  {
    super(source);
    this._connection = connection;
    this._ip = ipAdresse;
    this._port = port;
    this._benutzer = benutzer;
    this._urlasser = urlasser;
    this._grundVerbindungsaufbau = grundVerbindungsAufbau;
  }

  /**
   * Konstruktor, wenn die Verbindung abgebaut wurde.
   * 
   * @param source
   *          Eventauslsendes Objekt.
   * @param grundVerbindungsAbbau
   *          Grund fr den Verbindungsabbau.
   */
  public DaVVerbindungEreignis(Object source, String grundVerbindungsAbbau)
  {
    super(source);
    this._grundVerbindungsabbau = grundVerbindungsAbbau;
  }

  /**
   * Liefert den DaV-Benutzer.
   * 
   * @return DaV-Benutzer.
   */
  public String getDaVBenutzer()
  {
    return _benutzer;
  }

  /**
   * Liefert den DaV-Urlasser-Benutzer
   * 
   * @return DaV-Urlasser-Benutzer.
   */
  public String getDaVUrlasser()
  {
    return _urlasser;
  }

  /**
   * Liefert die DaV-Verbindung.
   * 
   * @return DaV-Verbindung.
   */
  public ClientDavInterface getDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Liefert die DaV-IP Adresse.
   * 
   * @return DaV-IP Adresse.
   */
  public String getDaVIpAdresse()
  {
    return _ip;
  }

  /**
   * Liefert den DaV-Port.
   * 
   * @return DaV-Port.
   */
  public int getDaVPort()
  {
    return _port;
  }

  /**
   * Liefert den Grund fr den Verbindungsabbau.
   * 
   * @return Grund fr den Verbindungsabbau.
   */
  public String getGrundDaVVerbindunsAbbau()
  {
    return _grundVerbindungsabbau;
  }

  public String getGrundDaVVerbindungsAufbau()
  {
    return _grundVerbindungsaufbau;
  }

}
