/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.deskriptoren;

import org.eclipse.jface.action.IAction;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;

/**
 * Diese Klasse beschreibt den Elemente, die in Men- und Symbolleisten
 * eingebettet werden knnen, bevor die eigentliche {@link IAction} in der
 * Klasse {@link ElementRegistrierung} registriert wurde.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 08:25:32 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class DeskriptorElement implements IDeskriptorElementUndAktion
{
	/**
	 * ID des Plug-Ins, dem dieses Element zugeordnet wird.
	 */
	private String _pluginID = null;

	/**
	 * ID der Action, die von diesem {@link DeskriptorElement} stellvertretend
	 * beschrieben wird.
	 */
	private String _aktionID = null;

	/**
	 * Text der Action
	 */
	private String _text = null;

	/**
	 * ToolTip der Action
	 */
	private String _toolTipText = null;

	/**
	 * Beschreibung der Action.
	 */
	private String _aktionBeschreibung = null;

	/**
	 * Pfad zum Icon der Action
	 */
	private String _iconPfad = null;

	/**
	 * Pfad zum HoverIcon der Action
	 */
	private String _hoverIconPfad = null;

	/**
	 * Pfad zum DisabledIcon der Action
	 */
	private String _deaktiviertesIconPfad = null;

	/**
	 * Ausklappaktion? SWT.DROP_DOWN?
	 */
	private boolean _ausklappAktion = false;

	/**
	 * Konstruktor, initialisiert ein Objekt vom Typ {@link DeskriptorElement}.
	 * Dieses dient als Stellvertreterobjekt fr noch nicht vorhandene Actions, so
	 * dass trotzdem schon eine Menstruktur bzw. Symbolleistenstruktur aufgebaut
	 * werden kann.
	 * 
	 * @param pluginId
	 *          ID des Plug-Ins.
	 * @param actionId
	 *          ID der Action.
	 * @param labelText
	 *          Label der Action.
	 * @param toolTipText
	 *          ToolTipText der Action.
	 * @param actionDescription
	 *          ActionDescription
	 * @param iconPfad
	 *          Pfad zum Icon der Action.
	 * @param hoverIconPfad
	 *          Pfad zum HoverIcon der Action.
	 * @param disabledIconPfad
	 *          Pfad zum DisabledIcon der Action.
	 * @param ausklappAktion
	 *          Ausklappaktion?
	 */
	public DeskriptorElement(String pluginId, String actionId, String labelText,
			String toolTipText, String actionDescription, String iconPfad,
			String hoverIconPfad, String disabledIconPfad, boolean ausklappAktion)
	{
		this._pluginID = pluginId;
		this._aktionID = actionId;
		this._text = labelText;
		this._toolTipText = toolTipText;
		this._aktionBeschreibung = actionDescription;
		this._iconPfad = iconPfad;
		this._hoverIconPfad = hoverIconPfad;
		this._deaktiviertesIconPfad = disabledIconPfad;
		this._ausklappAktion = ausklappAktion;

	}

	/*
	 * Getter
	 */

	/**
	 * @return PluginID.
	 */
	public String getPluginID()
	{
		return _pluginID;
	}

	/**
	 * Liefert die ID der reprsentierten Action.
	 */
	public String getAktionID()
	{
		return _aktionID;
	}

	/**
	 * Liefert den Text der Action.
	 * 
	 * @return Text der Action.
	 */
	public String getText()
	{
		return _text;
	}

	/**
	 * Liefert den ToolTipText der Action.
	 * 
	 * @return ToolTipText de Action.
	 */
	public String getToolTipText()
	{
		return _toolTipText;
	}

	/**
	 * Liefert die Beschreibung der Action.
	 * 
	 * @return Beschreibung der Action.
	 */
	public String getAktionBeschreibung()
	{
		return _aktionBeschreibung;
	}

	/**
	 * Liefert den Pfad zum Icon der Action.
	 * 
	 * @return Iconpfad.
	 */
	public String getIconPfad()
	{
		return _iconPfad;
	}

	/**
	 * Liefert den Pfad zum HoverIcon der Action.
	 * 
	 * @return Hovericonpfad.
	 */
	public String getHoverIconPfad()
	{
		return _hoverIconPfad;
	}

	/**
	 * Liefert den Pfad zum DisabledIcon der Action.
	 * 
	 * @return Disablediconpfad.
	 */
	public String getDeaktiviertesIconPfad()
	{
		return _deaktiviertesIconPfad;
	}

	public IAction getAktion()
	{
		return null;
	}

	public Image getHoverIcon()
	{
		return null;
	}

	public Image getIcon()
	{
		return null;
	}

	public boolean istGlobaleAktion()
	{
		return false;
	}

	public boolean istAusklappAktion()
	{
		return _ausklappAktion;
	}

	@Override
	public String toString()
	{
		return getText();
	}

	
}
