/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.deskriptoren;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Beschreibt die Wurzel der auswhlbaren Elemente fr den den Meneditor.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 08:25:32 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class DeskriptorWurzel
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Name der Wurzel
   */
  private String _name = null;

  /**
   * Descriptor Typ
   */
  private EnumDeskriptorTyp _enumDescriptorTyp = null;

  /**
   * Liste der Plug-Ins.
   */
  private List<DeskriptorPlugin> _listePlugins = null;

  /**
   * {@link ElementRegistrierung}.
   */
  private ElementRegistrierung _registrierung = null;

  /**
   * Konstruktor.
   * 
   * @param name
   *          Name des Wurzelknotens.
   * @param enumDescriptorTyp
   *          Deskriptortyp.
   */
  public DeskriptorWurzel(String name, EnumDeskriptorTyp enumDescriptorTyp)
  {
    _debug.fine(getClass().getName() + "::DescriptorWurzel => Starte Initialisierung...\n");

    this._name = name;
    this._enumDescriptorTyp = enumDescriptorTyp;
    init();

    _debug.fine(getClass().getName() + "::DescriptorWurzel => Bereit.\n");

  }

  /**
   * Initialisierung
   * 
   */
  private void init()
  {
    _listePlugins = new LinkedList<DeskriptorPlugin>();
    _registrierung = ElementRegistrierung.getInstanz();

    List<String> pluginIDs = _registrierung.getPluginIDs();
    Iterator<String> it = pluginIDs.iterator();
    while (it.hasNext())
    {
      String pluginID = it.next();
      _debug.fine(getClass().getName() + "::init => Generiere Plug-In mit Plug-In ID: " + pluginID);

      DeskriptorPlugin descriptorPlugin = new DeskriptorPlugin(pluginID, this._enumDescriptorTyp);
      this._listePlugins.add(descriptorPlugin);

    }

  }

  /**
   * Liefert den Namen der Wurzel.
   * 
   * @return Wurzelname.
   */
  public String getName()
  {
    return this._name;
  }

  /**
   * Liefert den Descriptor Typ.
   * 
   * @return Descriptor Typ.
   */
  public EnumDeskriptorTyp getDeskriptorTyp()
  {
    return this._enumDescriptorTyp;
  }

  /**
   * Liefert eine Liste mit Instanzen der Klasse {@link DeskriptorPlugin}. Es sin in dieser alle bekannten Plug-Ins
   * vorhanden.
   * 
   * @return Liste aller Plugins.
   */
  public List<DeskriptorPlugin> getPlugins()
  {
    return this._listePlugins;
  }

	@Override
	public String toString()
	{
		return "Wurzelknoten";
	}
  
  

}
