/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.xml.datenstruktur;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Definiert die Hintergrunddatenstruktur, auf der ein Men aufbaut. Untersttzt
 * werden Mens, Untermens sowie Menpunkte.
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/02/05 07:38:54 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class MenuSymbolLeisteDaten implements Cloneable
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Flag zum setzen des DocTypes
	 */
	private boolean _setzeDocType = false;

	/**
	 * JDOM Document, in dem die Verarbeitung erfolgt
	 */
	private Document _doc = null;

	/**
	 * Name des Wurzelelementes der MenuBar
	 */
	private String _menuBarRootElementName = XmlBezeichnerMenuSymbol.elementMenuLeiste;

	/**
	 * Name der Itemelemente der Toolbar
	 */
	private String _toolBarItemElementName = XmlBezeichnerMenuSymbol.elementItem;

	/**
	 * Name des Id-Attributs des Elementes item
	 */
	private String _toolBarItemAttributId = XmlBezeichnerMenuSymbol.attId;

	/**
	 * Wurzelelement des JDOM-Baums
	 */
	private Element _elementWurzel = null;

	/**
	 * Name vom Element item
	 */
	private String _elementItemName = XmlBezeichnerMenuSymbol.elementItem;

	private String _elementItemAttrId = XmlBezeichnerMenuSymbol.attId;

	/**
	 * Name vom Element separator
	 */
	private String _elementSeparatorName = XmlBezeichnerMenuSymbol.elementSeparator;

	@SuppressWarnings("unused")
	private String _elementSeparatorId = XmlBezeichnerMenuSymbol.attId;

	// /**
	// * Name vom Element groupmarker
	// */
	// private String _elementGroupmarkerName =
	// XmlBezeichnerMenu.elementGroupMarker;
	//
	// private String _elementGroupmarkerId = XmlBezeichnerMenu.attId;

	/**
	 * Name vom Element menu
	 */
	private String _elementMenuName = XmlBezeichnerMenuSymbol.elementMenu;

	private String _elementMenuAttrId = XmlBezeichnerMenuSymbol.attId;

	/**
	 * Name vom Element submenu
	 */
	private String _elementSubMenuName = XmlBezeichnerMenuSymbol.elementSubMenu;

	/**
	 * Name vom Attribut Id
	 */
	private String _elementSubMenuAttrId = XmlBezeichnerMenuSymbol.attId;

	/**
	 * Speichert Listener, die sich auf den Event
	 * {@link EreignisMenuSymbolLeisteDatenGeandert} anmelden.
	 */
	private List<IZuhoererMenuSymbolLeisteDatenAenderung> _zuhoerer = new LinkedList<IZuhoererMenuSymbolLeisteDatenAenderung>();

	/**
	 * Dirty-Flag
	 */
	private boolean _schmutzig = false;

	/**
	 * Zuhrer, der ber eine nderung des Dirty-Flags informiert werden mchte.
	 */
	private IFeuereSchmutzigGeandert _feuereSchmutzigGeandert = null;

	/**
	 * Verweis auf den UNDO-Stack
	 */
	private UndoStackMenuSymbol _undoStack = null;

	/**
	 * Dieser Enumerator gibt an, um welche Leistenart es sich handelt.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum enumLeistenArt
	{
		/**
		 * Leistenart Menleiste
		 */
		menue,
		/**
		 * Leistenart Symbolleiste
		 */
		symbol
	}

	/**
	 * Art der Leiste, Default = menue
	 */
	private enumLeistenArt _enumArtDerLeiste = enumLeistenArt.menue;

	/**
	 * Konstruktor.
	 * 
	 */
	public MenuSymbolLeisteDaten()
	{
		_undoStack = new UndoStackMenuSymbol();
		erzeugeBasisDoc();
	}

	/**
	 * Setzt den UNDO-Stack
	 * 
	 * @param undoStack
	 */
	public void setUndoStack(UndoStackMenuSymbol undoStack)
	{
		this._undoStack = undoStack;
	}

	/**
	 * Liefert den {@link UndoStackMenuSymbol} zurck.
	 * 
	 * @return {@link UndoStackMenuSymbol}.
	 */
	public UndoStackMenuSymbol getUndoStack()
	{
		return _undoStack;
	}

	public void pushAktuellerZustandUndoStack()
	{
		if (_undoStack != null)
		{
			_undoStack.push(this.clone());
		}
	}

	/**
	 * Setzt die Art der Leiste. <br>
	 * <b>Vorsicht:</b> Die Struktur des Dokumentes wird hierbei
	 * <u><i><b>NEU</b></i></u> initialisiert!
	 * 
	 * @param leistenArt
	 *          Art der Leiste.
	 */
	public void setzeLeistenArt(enumLeistenArt leistenArt)
	{
		this._enumArtDerLeiste = leistenArt;
	}

	/**
	 * Liefert die Art der Leiste zurck.
	 * 
	 * @return
	 */
	public enumLeistenArt getLeistenArt()
	{
		return _enumArtDerLeiste;
	}

	/**
	 * Setzt die Struktur des Mens zurck.
	 * 
	 * @param sicher
	 *          Muss <code>true</code> sein, damit das Men zurckgesetzt wird.
	 * @throws IllegalArgumentException
	 *           Wurd geworfen, wenn der <code>false</code> bergeben wurde.
	 */
	public void zuruecksetzenMenuStruktur(boolean sicher)
			throws IllegalArgumentException
	{
		if (sicher)
		{
			erzeugeBasisDoc();
			setSchmutzig(true);
		} else
		{
			throw new IllegalArgumentException(
					"Die Struktur des Mens konnte nicht zurckgesetzt werden, da der Parameter \"sicher\" mit false gesetzt wurde.");
		}
	}

	/**
	 * Erzeugt das Basisdokument, lehnt sich an toolbar.dtd an
	 * 
	 */
	private void erzeugeBasisDoc()
	{
		if (_enumArtDerLeiste == enumLeistenArt.menue)
		{
			_elementWurzel = new Element(_menuBarRootElementName);
			XmlElementMenu hauptmenue = new XmlElementMenu("hauptmenue.1");
			_elementWurzel.addContent(hauptmenue.getElementMenu());
		} else
		{
			_elementWurzel = new Element(XmlBezeichnerMenuSymbol.elementSymbolLeiste);
			XmlElementMenu hauptsymbolleiste = new XmlElementMenu(
					"hauptsymbolleiste.1");
			_elementWurzel.addContent(hauptsymbolleiste.getElementMenu());
		}

	}

	/**
	 * Liest eine XML-Datei ein
	 * 
	 * @param dateiPfad
	 *          Kompletter Pfad zur Datei
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn die Datei nicht existiert
	 */
	public void einlesenXmlAusDatei(String dateiPfad)
			throws IllegalArgumentException
	{
		SAXBuilder builder = new SAXBuilder();

		File file = new File(dateiPfad);
		if (file.exists())
		{
			try
			{
				builder.setValidation(false);
				_doc = builder.build(file);
				_elementWurzel = _doc.getRootElement();

				setSchmutzig(false);

				feuereEreignis();

			} catch (JDOMException e)
			{
				e.printStackTrace();
			} catch (IOException e)
			{
				e.printStackTrace();
			}

		} else
		{
			throw new IllegalArgumentException("Die Datei " + dateiPfad
					+ " existiert nicht!");
		}
	}

	/*
	 * public void einlesenXml(String xmlString) { SAXBuilder builder = new
	 * SAXBuilder(); builder.setValidation(false); try { StringReader reader = new
	 * StringReader(xmlString); _doc = builder.build(reader); _wurzelElement =
	 * _doc.getRootElement(); } catch (JDOMException e) {
	 * System.out.println(getClass().getName() + " => JDOMException");
	 * e.printStackTrace(); } catch (IOException e) {
	 * System.out.println(getClass().getName() + " => IOException");
	 * e.printStackTrace(); } }
	 */

	/**
	 * Liest die XML-Struktur aus einem String ein.
	 * 
	 * @param xmlString
	 *          String der XML-Struktur, die eingelesen werden soll.
	 */
	public void einlesenXmlAusString(String xmlString)
	{
		SAXBuilder builder = new SAXBuilder();
		builder.setValidation(false);
		try
		{
			StringReader reader = new StringReader(xmlString);
			_doc = builder.build(reader);
			_elementWurzel = _doc.getRootElement();

			setSchmutzig(false);

			feuereEreignis();

		} catch (JDOMException e)
		{
			_debug.warning(getClass().getName() + " => JDOMException", e);

		} catch (IOException e)
		{
			_debug.warning(getClass().getName() + " => IOException", e);
		}
	}

	/**
	 * Liefert eine formatierte Ausgabe der erzeugten XML-Struktur. Setzt das
	 * Dirty-Flag auf <code>false</code>.
	 * 
	 * @return Formatierte XML-Struktur
	 */
	public String ausgabeXmlFormatiert()
	{
		String returnString = null;
		if (_doc == null)
		{
			_doc = new Document(_elementWurzel);
		}

		// Doctype setzten

		if (_setzeDocType)
		{
			if (_enumArtDerLeiste == enumLeistenArt.menue)
			{
				DocType doctype = new DocType(
						XmlBezeichnerMenuSymbol.xmlDocTypeElementNameMenuLeiste,
						XmlBezeichnerMenuSymbol.xmlDocTypePublicId,
						XmlBezeichnerMenuSymbol.xmlDocTypeSystemIdMenuLeiste);
				_doc.setDocType(doctype);
			} else
			{
				DocType doctype = new DocType(
						XmlBezeichnerMenuSymbol.xmlDocTypeElementNameMenuLeiste,
						XmlBezeichnerMenuSymbol.xmlDocTypePublicId,
						XmlBezeichnerMenuSymbol.xmlDocTypeSystemIdSymbolLeiste);
				_doc.setDocType(doctype);
			}

		}

		Format prettyFormat = Format.getPrettyFormat();
		XMLOutputter out = new XMLOutputter(prettyFormat);
		Format f = out.getFormat();
		f.setEncoding(XmlBezeichnerMenuSymbol.xmlCodierung);
		out.setFormat(f);

		returnString = out.outputString(_doc);

		setSchmutzig(false);

		feuereEreignis();

		return returnString;
	}

	/**
	 * Liefert eine formatierte Ausgabe der erzeugten XML-Struktur. Setzt das
	 * Dirty-Flag <i><b>NICHT</b></i> auf <code>false</code>.
	 * 
	 * @return Formatierte XML-Struktur
	 */
	public String ausgabeXmlFormatiertNoDirty()
	{
		String returnString = null;
		if (_doc == null)
		{
			_doc = new Document(_elementWurzel);
		}

		// Doctype setzten

		if (_setzeDocType)
		{
			if (_enumArtDerLeiste == enumLeistenArt.menue)
			{
				DocType doctype = new DocType(
						XmlBezeichnerMenuSymbol.xmlDocTypeElementNameMenuLeiste,
						XmlBezeichnerMenuSymbol.xmlDocTypePublicId,
						XmlBezeichnerMenuSymbol.xmlDocTypeSystemIdMenuLeiste);
				_doc.setDocType(doctype);
			} else
			{
				DocType doctype = new DocType(
						XmlBezeichnerMenuSymbol.xmlDocTypeElementNameMenuLeiste,
						XmlBezeichnerMenuSymbol.xmlDocTypePublicId,
						XmlBezeichnerMenuSymbol.xmlDocTypeSystemIdSymbolLeiste);
				_doc.setDocType(doctype);
			}

		}

		Format prettyFormat = Format.getPrettyFormat();
		XMLOutputter out = new XMLOutputter(prettyFormat);
		Format f = out.getFormat();
		f.setEncoding(XmlBezeichnerMenuSymbol.xmlCodierung);
		out.setFormat(f);

		returnString = out.outputString(_doc);

		feuereEreignis();

		return returnString;
	}

	/**
	 * Schreibt einen String in eine Datei
	 * 
	 * @param dateiPfad
	 *          Kompletter Pfad zur Datei, in die geschrieben werden soll
	 * @param schreibString
	 *          String, der in die Datei geschrieben werden soll
	 * @param ueberschreiben
	 *          true == berschreiben der Datei, falls schon vorhanden
	 * @throws IllegalArgumentException
	 *           Wir geworfen, wenn die Datei existiert und nicht berschrieben
	 *           werden kann
	 */
	public void schreibeString(String dateiPfad, String schreibString,
			boolean ueberschreiben) throws IllegalArgumentException
	{
		File file = new File(dateiPfad);

		// File existiert, auf berschreiben prfen
		if (file.exists() && ueberschreiben)
		{
			try
			{
				PrintWriter pw = new PrintWriter(file);
				pw.print(schreibString);
				pw.close();
			} catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}
		// File existiert nicht und kann einfach angelegt werden
		else if (!file.exists())
		{
			try
			{
				PrintWriter pw = new PrintWriter(file);
				pw.print(schreibString);
				pw.close();
			} catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}
		// File existiert und ueberschreiben == flase
		else
		{
			throw new IllegalArgumentException("Die Datei " + dateiPfad
					+ " existiert und ueberschreiben == flase");
		}
	}

	/*
	 * public void einlesenXml(String xmlString) { SAXBuilder builder = new
	 * SAXBuilder(); builder.setValidation(false); try { StringReader reader = new
	 * StringReader(xmlString); _doc = builder.build(reader); _wurzelElement =
	 * _doc.getRootElement(); } catch (JDOMException e) {
	 * System.out.println(getClass().getName() + " => JDOMException");
	 * e.printStackTrace(); } catch (IOException e) {
	 * System.out.println(getClass().getName() + " => IOException");
	 * e.printStackTrace(); } }
	 */

	/**
	 * Fgt einen {@link IZuhoererMenuSymbolLeisteDatenAenderung} zur Liste der
	 * Zuhrer hinzu.
	 * 
	 * @param zuhoerer
	 *          {@link IZuhoererMenuSymbolLeisteDatenAenderung}.
	 */
	public void hinzufuegenZuhoerer(
			IZuhoererMenuSymbolLeisteDatenAenderung zuhoerer)
	{
		_zuhoerer.add(zuhoerer);
	}

	/**
	 * Entfernt einen {@link IZuhoererMenuSymbolLeisteDatenAenderung} von der
	 * Liste der Zuhrer.
	 * 
	 * @param zuhoerer
	 *          {@link IZuhoererMenuSymbolLeisteDatenAenderung}.
	 */
	public void entferneZuhoerer(IZuhoererMenuSymbolLeisteDatenAenderung zuhoerer)
	{
		_zuhoerer.remove(zuhoerer);
	}

	/**
	 * Abfeuern des Ereignis, benachrichtigen aller Zuhrer.
	 * 
	 */
	private void feuereEreignis()
	{

		if (_zuhoerer.size() > 0)
		{
			EreignisMenuSymbolLeisteDatenGeandert e = new EreignisMenuSymbolLeisteDatenGeandert(
					"Menbardaten gendert.");
			Iterator<IZuhoererMenuSymbolLeisteDatenAenderung> it = _zuhoerer
					.iterator();
			while (it.hasNext())
			{
				IZuhoererMenuSymbolLeisteDatenAenderung aenderung = it.next();
				aenderung.ausfuehren(e);
			}
		}
	}

	/**
	 * Bestimmt die Position eines {@link Element} in der Liste der Kind-Elemente
	 * seines Eltern-Elements.
	 * 
	 * @param element
	 *          Element, dessen Position bestimmt werden soll.
	 * @return Position des Elements i.d. Liste der Kind-Elemente des
	 *         Eltern-Elements.
	 */
	public int bestimmeElementPositionInElternElement(Element element)
	{
		int returnWert = -1;
		Element elternElement = element.getParentElement();
		if (element != null)
		{
			@SuppressWarnings("unchecked")
			Iterator it = elternElement.getChildren().iterator();
			while (it.hasNext())
			{
				returnWert++;
				Element item = (Element) it.next();
				if (item.equals(element))
				{
					break;
				}
			}
		}
		return returnWert;
	}

	/**
	 * Bestimmt die Anzahl der Kindelemente von einem Element
	 * 
	 * @param element
	 *          Element, von dem die Anzahl der Kindelemente bestimmt werden soll
	 * @return Anzahl der Kindelemente des Elements
	 */
	public int bestimmeAnzahlKinderImElement(Element element)
	{
		int returnWert = 0;
		returnWert = element.getChildren().size();
		return returnWert;
	}

	/**
	 * Liefert das Elemen mit der jeweiligen id
	 * 
	 * @param id
	 *          Id des Elements
	 * @return Gibt das gefundene Element zurck
	 */
	@SuppressWarnings("unused")
	private Element getElement(String id)
	{
		Element returnElement = null;
		@SuppressWarnings("unchecked")
		Iterator it = _elementWurzel.getChildren(_toolBarItemElementName)
				.iterator();
		while (it.hasNext())
		{
			Element tempElement = (Element) it.next();
			if (tempElement.getAttribute(_toolBarItemAttributId).getValue()
					.equalsIgnoreCase(id))
			{
				returnElement = tempElement;
				break;
			}
		}

		return returnElement;
	}

	/**
	 * Sorgt fr die Ausgabe des kompletten Baumes auf die Shell
	 * 
	 */
	public void ausgabe()
	{
		ausgabeParser(0, _elementWurzel);
	}

	/**
	 * Ausgabe des kompletten Baumes auf dem Schirm, bentigt fr spteres
	 * Anzeige/Editiertool als "Basis".
	 * 
	 * @param tiefe
	 *          Aktuelle Rekursionstiefe
	 * @param em
	 *          Element, dass nach seinen Kindern geparst werden soll
	 */
	public void ausgabeParser(int tiefe, Element em)
	{
		String ausgabeString = "";

		// Item
		if (em.getName().equalsIgnoreCase(_elementItemName))
		{
			XmlElementItem item = new XmlElementItem();
			item.generiereElement(em);
			ausgabeString = ausgabeString + item.toString();
		}

		// Menu
		if (em.getName().equalsIgnoreCase(_elementMenuName))
		{
			XmlElementMenu menu = new XmlElementMenu();
			menu.generiereElement(em);
			ausgabeString = ausgabeString + menu.toString();
		}

		// SubMenu
		if (em.getName().equalsIgnoreCase(_elementSubMenuName))
		{
			XmlElementSubmenu submenu = new XmlElementSubmenu();
			submenu.generiereElement(em);
			ausgabeString = ausgabeString + submenu.toString();
		}

		// Separator
		if (em.getName().equalsIgnoreCase(_elementSeparatorName))
		{
			XmlElementSeparator separator = new XmlElementSeparator();
			separator.generiereElement(em);
			ausgabeString = ausgabeString + separator.toString();
		}

		String ausgabeVorlauf = "";
		for (int i = 0; i < tiefe; i++)
		{
			ausgabeVorlauf = ausgabeVorlauf + "-";
		}
		ausgabeVorlauf = ausgabeVorlauf + "> ";

		_debug.fine(ausgabeVorlauf + ausgabeString);

		@SuppressWarnings("unchecked")
		Iterator it = em.getChildren().iterator();
		while (it.hasNext())
		{
			Element tempEm = (Element) it.next();
			tiefe++;
			ausgabeParser(tiefe, tempEm);
			tiefe--;
		}

	}

	@SuppressWarnings("unchecked")
	public static void elementAusgeben(Element element)
	{
		System.out.println("=>Element: \"" + element.getName() + "\"");
		List attributes = element.getAttributes();

		for (Object obj : attributes)
		{
			if (obj instanceof Attribute)
			{
				Attribute att = (Attribute) obj;
				System.out.println("\t-> Attribut " + att.getName() + " = \""
						+ att.getValue() + "\"");
			}
		}
	}

	public static ArrayList<Element> getParentElements(Element element)
	{
		ArrayList<Element> re = new ArrayList<Element>();

		re.add(element);

		if (element.getParentElement() != null)
		{
			re.addAll(getParentElements(element.getParentElement()));
		}

		return re;
	}

	/**
	 * Prft, ob das Lschen eines {@link Element}es erlaubt ist oder nicht. Das
	 * Wurzelelement von Men-/Symbolleiste darf nicht gelscht werden!
	 * 
	 * @param loeschElement
	 *          {@link Element}, welches gelscht werden soll.
	 * @return Liefert <code>true</code>, wenn das {@link Element} gelscht werden
	 *         darf, sonst <code>false</code>.
	 */
	public static boolean pruefeObElementLoeschenErlaubt(Element loeschElement)
	{
		boolean loeschen_erlaubt = true;

		if (loeschElement != null)
		{
			/*
			 * Prfen, ob Lschen erlaubt ist. Das Wurzelelement von Men/Symbolleiste
			 * darf nicht gelscht werden!
			 */
			if (loeschElement.getParentElement() != null)
			{
				Element parentElement = loeschElement.getParentElement();				
				if ((parentElement.getName()
						.equals(XmlBezeichnerMenuSymbol.elementMenuLeiste))
						|| (parentElement.getName()
								.equals(XmlBezeichnerMenuSymbol.elementSymbolLeiste)))
				{
					loeschen_erlaubt = false;
				}
			}
		}

		return loeschen_erlaubt;
	}

	/**
	 * Lscht ein Element aus der XML-Struktur
	 * 
	 * @param loeschElement
	 *          Zu lschendes Element.
	 */
	public void loescheElement(Element loeschElement)
	{
		if (loeschElement != null)
		{
			if (pruefeObElementLoeschenErlaubt(loeschElement))
			{
				/*
				 * Sichern des aktuellen Zustands
				 */
				pushAktuellerZustandUndoStack();

				loeschElement.detach();
				setSchmutzig(true);
			}
		}
	}

	/**
	 * Verschiebt eine Element vom Typ Item an eine bestimmte Position in einem
	 * Men.
	 * 
	 * @param quelle
	 *          Quellelement
	 * @param ziel
	 *          Zielelement
	 * @param position
	 *          Position im Zielelement
	 */
	public void verschiebeItem(Element quelle, Element ziel, int position)
	{
		if (quelle.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
		{
			if ((ziel.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
					|| (ziel.getName()
							.equals(XmlBezeichnerMenuSymbol.elementSymbolLeiste)))
			{
				/*
				 * Sichern des aktuellen Zustands
				 */
				pushAktuellerZustandUndoStack();

				Element kopie = (Element) quelle.clone();
				quelle.detach();
				setzeElementBei(ziel, kopie, position);
				setSchmutzig(true);
			} else
			{
				_debug
						.warning(getClass().getName()
								+ "::verschiebeItem \n\t=> Es handelt sich beim Ziel nicht um ein Men!\n");
			}
		} else
		{
			_debug.warning(getClass().getName()
					+ "::verschiebeItem \n\t=> Es handelt sich nicht um ein Item!");
		}
	}

	/**
	 * Einfgen eines Item ans Ende eines Mens.
	 * 
	 * @param quelle
	 *          Item, dass ans Ende eines anderen Mens eingefgt werden soll.
	 * @param ziel
	 *          Zielmen, an dass das Item angehngt werden soll.
	 */
	public void verschiebeItem(Element quelle, Element ziel)
	{
		if (quelle.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
		{
			if ((ziel.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
					|| (ziel.getName()
							.equals(XmlBezeichnerMenuSymbol.elementSymbolLeiste)))
			{
				/*
				 * Sichern des aktuellen Zustands
				 */
				pushAktuellerZustandUndoStack();

				Element kopie = (Element) quelle.clone();
				quelle.detach();
				ziel.addContent(kopie);
				setSchmutzig(true);
			} else
			{
				_debug
						.warning(getClass().getName()
								+ "::verschiebeItem \n\t=> Es handelt sich nicht um ein Men, in dass das Item verschoben werden soll.");
			}
		} else
		{
			_debug.warning(getClass().getName()
					+ "::verschiebeItem \n\t=> Es handelt sich nicht um ein Item!");
		}
	}

	// /**
	// * Verschiebt ein Men in ein anderes Submen.
	// *
	// * @deprecated
	// * @param idMenu
	// * Quelle bzw. Id des Mens, dass verschoben werden soll.
	// * @param idZielSubMenu
	// * Ziel bzw. Id des SubMens, in dass das Men verschoben werden
	// * soll.
	// * @throws IllegalArgumentException
	// * Wenn Quelle || Ziel nicht existent sind.
	// */
	// public void verschiebeMenu(String idMenu, String idZielSubMenu)
	// throws IllegalArgumentException
	// {
	// // Men suchen
	// Element quellElement = sucheMenu(idMenu);
	// if (quellElement != null)
	// {
	// Element zielElement = sucheSubMenu(idZielSubMenu);
	// if (zielElement != null)
	// {
	// /*
	// * Sichern des aktuellen Zustands
	// */
	// pushAktuellerZustandUndoStack();
	//
	// Element clonedStructure = kopiereElementStruktur(quellElement);
	// loescheMenu(idMenu);
	// zielElement.addContent(clonedStructure);
	// setSchmutzig(true);
	// } else
	// {
	// throw new IllegalArgumentException("Das Ziel-Submen mit der Id "
	// + idZielSubMenu + " existiert nicht!");
	// }
	// } else
	// {
	// throw new IllegalArgumentException("Das Quell-Men mit der Id " + idMenu
	// + " existiert nicht!");
	// }
	// }

	/**
	 * Verschiebt ein Men in ein leeres Submen
	 * 
	 * @param quelle
	 *          Men, dass verschoben werden soll.
	 * @param ziel
	 *          Submen, in dass das Men verschoben werden soll.
	 */
	public void verschiebeMenu(Element quelle, Element ziel)
	{
		if (quelle.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
		{
			if ((ziel.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
					|| ((_enumArtDerLeiste == enumLeistenArt.symbol) && (ziel.getName()
							.equals(XmlBezeichnerMenuSymbol.elementItem))))
			{
				if (istXmlElementKindVon(ziel, quelle))
				{
					_debug
							.warning(getClass().getName()
									+ "::verschiebeMenu\n\t=>Das Zielelement ist Kindelement vom Quellelement. Abbruch. \n");
				} else
				{
					if (ziel.getChildren().size() > 0)
					{
						if (_enumArtDerLeiste == enumLeistenArt.menue)
						{
							_debug
									.warning(getClass().getName()
											+ "::verschiebeMenu\n\t=>Im Zielsubmen sind bereits Elemente vorhanden. Abbruch. \n");
						} else
						{
							_debug
									.warning(getClass().getName()
											+ "::verschiebeMenu\n\t=>Im ZielItem sind bereits Elemente vorhanden. Abbruch. \n");
						}
					} else
					{
						/*
						 * Sichern des aktuellen Zustands
						 */
						pushAktuellerZustandUndoStack();

						Element kopie = (Element) quelle.clone();
						quelle.detach();
						ziel.addContent(kopie);
						setSchmutzig(true);
					}
				}

			} else
			{
				_debug
						.warning(getClass().getName()
								+ "::verschiebeMenu\n\t=>Bei dem Ziel handelt es sich nicht um ein SubMen. \n");
			}

		} else
		{
			_debug
					.warning(getClass().getName()
							+ "::verschiebeMenu\n\t=>Bei der Quelle handelt es sich nicht um ein Men. \n");
		}
	}

	// /**
	// * Verschiebt ein Men in ein anderes Submen an eine bestimmte Position.
	// *
	// * @param idMenu
	// * Quelle bzw. Id des Mens, dass verschoben werden soll.
	// * @param idZielSubMenu
	// * Ziel bzw. Id des SubMens, in dass das Men verschoben werden
	// * soll.
	// * @param zielPosition
	// * Position, an die Men verschoben werden soll.
	// * @throws IllegalArgumentException
	// * Wenn Quelle || Ziel nicht existent sind.
	// */
	// public void verschiebeMenu(String idMenu, String idZielSubMenu,
	// int zielPosition) throws IllegalArgumentException
	// {
	// verschiebeMenu(idMenu, idZielSubMenu);
	// /*
	// * Ein Men wird fest an ein SubMen gebunden, von daher ist es unntig, ein
	// * Verschieben mit Positionierung zu ermglichen.
	// */
	// }

	// /**
	// * Verschiebt ein SubMen in ein anderes Men.
	// *
	// * @deprecated
	// *
	// * @param idSubMenu
	// * Id des Submens, dass verschoben werden soll.
	// * @param idZielMenu
	// * Id des ZielMens, in dass das Submen verschoben werden soll.
	// * @throws IllegalArgumentException
	// */
	// public void verschiebeSubMenu(String idSubMenu, String idZielMenu)
	// throws IllegalArgumentException
	// {
	// // SubMen suchen
	// Element quellElement = sucheSubMenu(idSubMenu);
	// if (quellElement != null)
	// {
	// Element zielMenu = sucheMenu(idZielMenu);
	// if (zielMenu != null)
	// {
	// /*
	// * Sichern des aktuellen Zustands
	// */
	// pushAktuellerZustandUndoStack();
	//
	// Element clonedStructure = kopiereElementStruktur(quellElement);
	// loescheSubMenu(idSubMenu);
	// zielMenu.addContent(clonedStructure);
	// setSchmutzig(true);
	// } else
	// {
	// throw new IllegalArgumentException("Das Ziel-Men mit der Id "
	// + idZielMenu + "existiert nicht!");
	// }
	// } else
	// {
	// throw new IllegalArgumentException("Das Quell-SubMen mot der Id "
	// + idSubMenu + " existiert nicht!");
	// }
	// }

	// /**
	// * Verschiebt ein SubMen in ein anderes Men an eine bestimmte Position.
	// *
	// * @deprecated
	// * @param idSubMenu
	// * Id des Submens, dass verschoben werden soll.
	// * @param idZielMenu
	// * Id des ZielMens, in dass das Submen verschoben werden soll.
	// * @param zielPosition
	// * Position innerhalb des Zielmens.
	// * @throws IllegalArgumentException *
	// */
	// public void verschiebeSubMenu(String idSubMenu, String idZielMenu,
	// int zielPosition) throws IllegalArgumentException
	// {
	// // SubMen suchen
	// Element quellElement = sucheSubMenu(idSubMenu);
	// if (quellElement != null)
	// {
	// Element zielMenu = sucheMenu(idZielMenu);
	// if (zielMenu != null)
	// {
	// /*
	// * Sichern des aktuellen Zustands
	// */
	// pushAktuellerZustandUndoStack();
	//
	// Element clonedStructure = kopiereElementStruktur(quellElement);
	// loescheSubMenu(idSubMenu);
	//
	// if (pruefePositionInElement(zielMenu, zielPosition))
	// {
	// setzeElementBei(zielMenu, clonedStructure, zielPosition);
	// setSchmutzig(true);
	// } else
	// {
	// zielMenu.addContent(clonedStructure);
	// setSchmutzig(true);
	// }
	//
	// } else
	// {
	// throw new IllegalArgumentException("Das Ziel-Men mit der Id "
	// + idZielMenu + "existiert nicht!");
	// }
	// } else
	// {
	// throw new IllegalArgumentException("Das Quell-SubMen mot der Id "
	// + idSubMenu + " existiert nicht!");
	// }
	// }

	/**
	 * Verschiebt ein Submen in ein anderes Men an eine bestimmte Position.
	 * 
	 * @param quelle
	 *          Quellsubmen, welches verschoben werdne soll.
	 * @param ziel
	 *          Zielmen, in das das Submen verschoben werden soll.
	 * @param position
	 *          Position im Zielsubmen, an die das Submen verschoben werden
	 *          soll.
	 */
	public void verschiebeSubMenu(Element quelle, Element ziel, int position)
	{
		if (quelle.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
		{
			if (ziel.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
			{
				if (!istXmlElementKindVon(ziel, quelle))
				{
					/*
					 * Sichern des aktuellen Zustands
					 */
					pushAktuellerZustandUndoStack();

					Element kopie = (Element) quelle.clone();
					quelle.detach();
					setzeElementBei(ziel, kopie, position);
					setSchmutzig(true);
				} else
				{
					_debug
							.warning(getClass().getName()
									+ "::verschiebeSubMenu \n\t=> Das Ziel ist Kindelement des eigenen Elementes. Vorgang abgebrochen!\n");
				}

			} else
			{
				_debug
						.warning(getClass().getName()
								+ "::verschiebeSubMenu \n\t=> Es handelt sich beim Ziel nicht um ein Men!\n");
			}
		} else
		{
			_debug
					.warning(getClass().getName()
							+ "::verschiebeSubMenu \n\t=> Es handelt sich nicht um ein Submen.\n");
		}
	}

	/**
	 * Hngt ein Submen an ein Men an.
	 * 
	 * @param quelle
	 *          Quellsubmen.
	 * @param ziel
	 *          Zielsubmen.
	 */
	public void verschiebeSubMenu(Element quelle, Element ziel)
	{
		if (quelle.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
		{
			// Ziel Men
			if (ziel.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
			{
				if (!istXmlElementKindVon(ziel, quelle))
				{
					/*
					 * Sichern des aktuellen Zustands
					 */
					pushAktuellerZustandUndoStack();

					Element kopie = (Element) quelle.clone();
					quelle.detach();
					ziel.addContent(kopie);
					setSchmutzig(true);
				} else
				{
					_debug
							.warning(getClass().getName()
									+ "::verschiebeSubMenu \n\t=> Das Ziel ist Kindelement des eigenen Elementes. Vorgang abgebrochen!\n");
				}
			}
			// Ziel Item, denn an eine DropDown-Action wird ein Men gebungen.
			else if (ziel.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
			{
				_debug.fine("MEN");
				String ausklappbar = ziel
						.getAttributeValue(XmlBezeichnerMenuSymbol.attItemAusklappbar);
				if ((ausklappbar != null) && (!ausklappbar.equals("")))
				{
					_debug.fine("Mache was zu machen ist.");
					Element kopie = (Element) quelle.clone();
					quelle.detach();
					ziel.addContent(kopie);
					setSchmutzig(true);
				} else
				{
					_debug
							.warning(getClass().getName()
									+ "::verschiebeSubMenu \n\t=> Das Attribut ausklappbar im Zielelement ist nicht auswertbar. Vorgang abgebrochen!\n");
				}
			} else
			{
				_debug
						.warning(getClass().getName()
								+ "::verschiebeSubMenu \n\t=> Es handelt sich beim Ziel nicht um ein Men!\n");
			}
		}
		// Sonst
		else
		{
			_debug
					.warning(getClass().getName()
							+ "::verschiebeSubMenu \n\t=> Es handelt sich nicht um ein Submen.\n");
		}
	}

	/**
	 * Prft rekursiv, ob ein Element Kindelment von einem Elternelement ist.
	 * 
	 * @param KindElement
	 *          Element von dem geprft werden soll, ob es Kindelement ist.
	 * @param potElternElement
	 *          Element gegen das geprft werden soll, ob es das Kindelement
	 *          enthlt.
	 * @return
	 */
	public boolean istXmlElementKindVon(Element KindElement,
			Element potElternElement)
	{
		boolean istKind = false;
		if (KindElement == potElternElement)
		{
			istKind = true;
		} else if (potElternElement.getChildren().size() > 0)
		{
			@SuppressWarnings("unchecked")
			Iterator it = potElternElement.getChildren().iterator();
			while (it.hasNext())
			{
				Element neuesPotElternElement = (Element) it.next();
				istKind = istXmlElementKindVon(KindElement, neuesPotElternElement);
				if (istKind)
				{
					break;
				}
			}
		}

		return istKind;
	}

	// /**
	// * Hngt ein Item an ein Men an
	// *
	// * @param item
	// * Item, dass angehngt werden soll
	// * @param zielMenuId
	// * Id des ZielMens, an dass das Item angehngt werden soll
	// * @throws IllegalArgumentException
	// * Zielmen existiert nicht || Item mit gleicher Id existiert
	// * bereits
	// */
	// public void hinzufuegenItem(XmlElementItem item, String zielMenuId)
	// throws IllegalArgumentException
	// {
	// String newId = item.getElementItem().getAttributeValue(_elementItemAttrId);
	// if (sucheItem(newId) == null)
	// {
	// Element zielMenu = sucheMenu(zielMenuId);
	// if (zielMenu != null)
	// {
	// zielMenu.addContent(item.getElementItem());
	// setSchmutzig(true);
	// } else
	// {
	// throw new IllegalArgumentException("Das Men mit der Id " + zielMenuId
	// + " existiert nicht!");
	// }
	// } else
	// {
	// throw new IllegalArgumentException(
	// "Es existiert bereits ein Item mit der Id " + newId + "!");
	// }
	// }
	//
	// /**
	// * Hngt ein Item an ein Men an
	// *
	// * @param item
	// * Item, dass angehngt werden soll
	// * @param zielMenuId
	// * Id des ZielMens, an dass das Item angehngt werden soll
	// * @param zielPosition
	// * innerhalb des ZielMens, Elemente an gleicher bzw. nachfolgender
	// * Stelle werden nach hinten verschoben
	// * @throws IllegalArgumentException
	// * Zielmen existiert nicht || Item mit gleicher Id existiert
	// * bereits
	// */
	// public void hinzufuegenItem(XmlElementItem item, String zielMenuId,
	// int zielPosition) throws IllegalArgumentException
	// {
	// String newId = item.getElementItem().getAttributeValue(_elementItemAttrId);
	// if (sucheItem(newId) == null)
	// {
	// Element zielMenu = sucheMenu(zielMenuId);
	// if (zielMenu != null)
	// {
	// if (pruefePositionInElement(zielMenu, zielPosition))
	// {
	// setzeElementBei(zielMenu, item.getElementItem(), zielPosition);
	// } else
	// {
	// _debug.fine("Einfgen am Ende");
	// zielMenu.addContent(item.getElementItem());
	// ausgabeParser(0, zielMenu);
	// setSchmutzig(true);
	// }
	//
	// } else
	// {
	// throw new IllegalArgumentException("Das Men mit der Id " + zielMenuId
	// + " existiert nicht!");
	// }
	// } else
	// {
	// throw new IllegalArgumentException(
	// "Es existiert bereits ein Item mit der Id " + newId + "!");
	// }
	// }

	/**
	 * Hngt einen Separator an ein Zielmen an.
	 * 
	 * @param separator
	 *          Separator.
	 * @param zielMenu
	 *          Zielmen.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn das Zielelement nicht vom Typ menu ist.
	 */
	public void hinzufuegenSeparatorInMenuDep(XmlElementSeparator separator,
			Element zielMenu) throws IllegalArgumentException
	{
		try
		{
			/*
			 * Sichern des aktuellen Zustands
			 */
			pushAktuellerZustandUndoStack();
			hinzufuegenSeparatorAnMenu(separator.getElementSeparator(), zielMenu);
			setSchmutzig(true);
		} catch (IllegalArgumentException iae)
		{
			throw iae;
		}
	}

	/**
	 * Hngt einen Separator an ein Zielmen an.
	 * 
	 * @param separator
	 *          Separator als Element, dass an ein ZielMen angehngt werden soll.
	 * @param zielMenu
	 *          Zielmen, an dass der Separator angehngt werden soll.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn das angegebene Zielmen nicht vom Typ menu
	 *           ist.
	 * 
	 * 
	 */
	public void hinzufuegenSeparatorAnMenu(Element separator, Element zielMenu)
			throws IllegalArgumentException
	{
		if (zielMenu.getName().equals(_elementMenuName))
		{
			/*
			 * Sichern des aktuellen Zustands
			 */
			pushAktuellerZustandUndoStack();
			separator.detach();
			zielMenu.addContent(separator);
			setSchmutzig(true);
		} else
		{
			throw new IllegalArgumentException(
					"Das angegebene Zielelement ist nicht vom Typ menu.");
		}
	}

	/**
	 * Fgt einen Separator an die Stelle des Zielelementes ein. Das Zielelement
	 * wird um eine Postition nach hinten verschoben.
	 * 
	 * @param separator
	 *          Separator, der eingefgt werden soll.
	 * @param zielElement
	 *          Zielelement, an dessen Position der Separator eingefgt wird.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn das Zielelement nicht vom Typ item oder
	 *           submenu ist.
	 */
	public void hinzufuegenSeparatorAnPositionInMenuDep(
			XmlElementSeparator separator, Element zielElement)
			throws IllegalArgumentException
	{
		/*
		 * Sichern des aktuellen Zustands
		 */
		pushAktuellerZustandUndoStack();

		hinzufuegenSeparatorInMenu(separator.getElementSeparator(), zielElement);
		setSchmutzig(true);
	}

	/**
	 * Fgt einen Separator an die Stelle des Zielelementes ein. Das Zielelement
	 * wird um eine Postition nach hinten verschoben.
	 * 
	 * @param separator
	 *          Separator, der eingefgt werden soll.
	 * @param zielElement
	 *          Zielelement, an dessen Position der Separator eingefgt wird.
	 */
	public void hinzufuegenSeparatorInMenu(Element separator, Element zielElement)
	{
		if (zielElement.getName().equals(_elementItemName)
				|| zielElement.getName().equals(_elementSubMenuName))
		{
			int position = bestimmeElementPositionInElternElement(zielElement);

			/*
			 * Sichern des aktuellen Zustands
			 */
			pushAktuellerZustandUndoStack();

			Element clone = (Element) separator.clone();

			separator.detach();

			setzeElementBei(zielElement.getParentElement(), clone, position);
			setSchmutzig(true);
		} else
		{
			throw new IllegalArgumentException(
					"Ein Separator kann nur in die Position eines Elements vom Typ submenu oder item eingefgt werden.");
		}
	}

	/**
	 * Fgt ein Element vom Typ Separator in eine bestimmte Stelle im Men ein.
	 * 
	 * @param quellSeparator
	 *          Separator, der eingefgt werden soll.
	 * @param zielMenu
	 *          Zielmen, in dass der Separator eingefgt werden soll.
	 * @param position
	 *          Position im Zielmen.
	 */
	public void hinzufuegenSeparatorInMenu(Element quellSeparator,
			Element zielMenu, int position)
	{
		if (zielMenu.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
		{
			/*
			 * Sichern des aktuellen Zustands
			 */
			pushAktuellerZustandUndoStack();

			Element clone = (Element) quellSeparator.clone();
			quellSeparator.detach();
			setzeElementBei(zielMenu, clone, position);
			setSchmutzig(true);
		}

	}

	/**
	 * Hngt einen GroupMarker an ein Men an.
	 * 
	 * @param groupmarker
	 *          GroupMarker, der angehngt werden soll.
	 * @param zielMenuId
	 *          Id des ZielMens, an dass der GroupMarker angehngt werden soll.
	 * @throws IllegalArgumentException
	 *           Zielmen existiert nicht || Item mit gleicher Id existiert
	 *           bereits
	 */
	// public void addGroupMarker(XmlElementGroupMarker groupmarker,
	// String zielMenuId) throws IllegalArgumentException
	// {
	// }
	/**
	 * Hngt einen GroupMarker an ein Men an.
	 * 
	 * @param groupmarker
	 *          GroupMarker, der angehngt werden soll.
	 * @param zielMenuId
	 *          Id des ZielMens, an dass der GroupMarker angehngt werden soll.
	 * @param zielPosition
	 *          innerhalb des ZielMens, Elemente an gleicher bzw. nachfolgender
	 *          Stelle werden nach hinten verschoben
	 * @throws IllegalArgumentException
	 *           Zielmen existiert nicht || Item mit gleicher Id existiert
	 *           bereits
	 */
	// public void addGroupMarker(XmlElementGroupMarker groupmarker,
	// String zielMenuId, int zielPosition) throws IllegalArgumentException
	// {
	// }
	// /**
	// * Hngt ein Men an ein SubMen an
	// *
	// * @param menu
	// * Men, dass angehngt werden soll
	// * @param zielSubMenuId
	// * Id des ZielSubMens, an dass das Menu angehngt werden soll
	// * @throws IllegalArgumentException
	// * ZielSubMen existiert nicht || Menu mit gleicher Id existiert
	// * bereits
	// *
	// * @deprecated
	// */
	// public void hinzufuegenMenu(XmlElementMenu menu, String zielSubMenuId)
	// throws IllegalArgumentException
	// {
	// String newId = menu.getElementMenu().getAttributeValue(_elementMenuAttrId);
	// if (sucheMenu(newId) == null)
	// {
	// Element zielSubMenu = sucheSubMenu(zielSubMenuId);
	// if (zielSubMenu != null)
	// {
	// /*
	// * Sichern des aktuellen Zustands
	// */
	// pushAktuellerZustandUndoStack();
	//
	// zielSubMenu.addContent(menu.getElementMenu());
	// setSchmutzig(true);
	// } else
	// {
	// throw new IllegalArgumentException("Das SubMen mit der Id "
	// + zielSubMenuId + " existiert nicht!");
	// }
	// } else
	// {
	// throw new IllegalArgumentException(
	// "Es existiert bereits ein Men mit der Id " + newId + "!");
	// }
	// }
	// /**
	// * Hngt ein Men an ein SubMen an
	// *
	// * @param menu
	// * Men, dass angehngt werden soll
	// * @param zielSubMenuId
	// * Id des ZielSubMens, an dass das Menu angehngt werden soll
	// * @param zielPosition
	// * Zielposition innerhalb des Zielsubmens. Elemente an gleicher bzw.
	// * nachfolgender Stelle werden nach hinten verschoben
	// * @throws IllegalArgumentException
	// * ZielSubMen existiert nicht || Menu mit gleicher Id existiert
	// * bereits
	// *
	// * @deprecated
	// */
	// public void hinzufuegenMenu(XmlElementMenu menu, String zielSubMenuId,
	// int zielPosition) throws IllegalArgumentException
	// {
	// /*
	// * Ein Men kann immer nur Bestandteil eines SubMens oder einer MenBar
	// * sein. Deshalb gibt es hier KEINE Funktion, um ein Men an einer
	// * bestimmten Position einzufgen! Es erfolgt hier die Weiterleitung an die
	// * "NORMALE" addMenu-Methode.
	// */
	// hinzufuegenMenu(menu, zielSubMenuId);
	// setSchmutzig(true);
	// }
	/**
	 * Hngt ein SubMen an ein Men an
	 * 
	 * @param submenu
	 *          Submen, dass angehngt werden soll
	 * @param zielMenuId
	 *          Id des ZielMens, an dass das Submen angehngt werden soll
	 * @throws IllegalArgumentException
	 *           ZielMen existiert nicht || SubMen mit gleicher Id existiert
	 *           bereists
	 * 
	 */
	public void hinzufuegenSubMenu(XmlElementSubmenu submenu, String zielMenuId)
			throws IllegalArgumentException
	{
		String newId = submenu.getElementSubmenu().getAttributeValue(
				_elementSubMenuAttrId);
		if (sucheSubMenu(newId) == null)
		{
			Element zielMenu = sucheMenu(zielMenuId);
			if (zielMenu != null)
			{
				/*
				 * Sichern des aktuellen Zustands
				 */
				pushAktuellerZustandUndoStack();

				zielMenu.addContent(submenu.getElementSubmenu());
				setSchmutzig(true);
			} else
			{
				throw new IllegalArgumentException("Das Men mit der Id " + zielMenuId
						+ "existiert nicht!");
			}
		} else
		{
			throw new IllegalArgumentException(
					"Es existiert bereits eine SubMen mit der Id " + newId + "!");
		}
	}

	/**
	 * Hngt ein SubMen an ein Men an
	 * 
	 * @param submenu
	 *          Submen, dass angehngt werden soll
	 * @param zielMenuId
	 *          Id des ZielMens, an dass das Submen angehngt werden soll
	 * @param zielPosition
	 *          Zielpostion innerhalb des ZielMens. Elemente an gleicher bzw.
	 *          nachfolgender Stelle werden nach hinten verschoben.
	 * @throws IllegalArgumentException
	 *           ZielMen existiert nicht || SubMen mit gleicher Id existiert
	 *           bereists
	 */
	public void hinzufuegenSubMenu(XmlElementSubmenu submenu, String zielMenuId,
			int zielPosition) throws IllegalArgumentException
	{
		String newId = submenu.getElementSubmenu().getAttributeValue(
				_elementSubMenuAttrId);
		if (sucheSubMenu(newId) == null)
		{
			Element zielMenu = sucheMenu(zielMenuId);
			if (zielMenu != null)
			{
				if (pruefePositionInElement(zielMenu, zielPosition))
				{
					/*
					 * Sichern des aktuellen Zustands
					 */
					pushAktuellerZustandUndoStack();

					setzeElementBei(zielMenu, submenu.getElementSubmenu(), zielPosition);
					setSchmutzig(true);
				} else
				{
					/*
					 * Sichern des aktuellen Zustands
					 */
					pushAktuellerZustandUndoStack();

					zielMenu.addContent(submenu.getElementSubmenu());
					setSchmutzig(true);
				}

			} else
			{
				throw new IllegalArgumentException("Das Men mit der Id " + zielMenuId
						+ "existiert nicht!");
			}
		} else
		{
			throw new IllegalArgumentException(
					"Es existiert bereits eine SubMen mit der Id " + newId + "!");
		}
	}

	/**
	 * Fgt an einer bestimmten Stelle ein Element als Kind ein
	 * 
	 * @param emZiel
	 *          Ziel-Element, in dass ein Kindelement eingefgt werden soll
	 * @param emEinfuegen
	 *          Element, dass als Kind eingefgt werden soll
	 * @param position
	 *          Position, an der das Element eingefgt wird
	 */
	@SuppressWarnings("unchecked")
	public void setzeElementBei(Element emZiel, Element emEinfuegen, int position)
	{
		/*
		 * Sichern des aktuellen Zustands
		 */
		pushAktuellerZustandUndoStack();

		LinkedList<Element> neu = new LinkedList<Element>();

		Iterator it = emZiel.getContent().iterator();
		while (it.hasNext())
		{
			Object object = it.next();
			if (object instanceof Element)
			{
				Element element = (Element) object;
				neu.add(element);
			}
		}

		Iterator<Element> itNeu = neu.iterator();
		while (itNeu.hasNext())
		{
			itNeu.next().detach();
		}

		neu.add(position, emEinfuegen);

		emZiel.setContent(neu);

	}

	/**
	 * Fgt an einer bestimmten Stelle ein Element als Kind ein
	 * 
	 * @param emZiel
	 *          Ziel-Element, in dass ein Kindelement eingefgt werden soll
	 * @param emEinfuegen
	 *          Element, dass als Kind eingefgt werden soll
	 * @param position
	 *          Position, an der das Element eingefgt wird
	 */
	// @SuppressWarnings("unchecked")
	// public void einfuegenElementAt(Element emZiel, Element emEinfuegen,
	// int position)
	// {
	// // Eine Position < 0 wurde angegeben. In diesem Fall wird das
	// // einzufgende Element als erstes Kind in der Kinder-Liste des
	// // Zielelementes gefhrt.
	// if (position < 0)
	// {
	// position = 0;
	// }
	//
	// System.out.println("Pos: " + position);
	//
	// if (_debug)
	// {
	// System.out.println("\nVorher: ");
	// ausgabeParser(0, emZiel);
	// }
	//
	// // Listen
	// List<Element> childsEmZiel = emZiel.getChildren();
	// List<Element> childsNeu = new LinkedList<Element>();
	//
	// int i = -1;
	// Iterator<Element> it = childsEmZiel.iterator();
	// while (it.hasNext())
	// {
	// i++;
	//
	// if (i == position)
	// {
	// childsNeu.add(emEinfuegen);
	// System.out.println("EINFGEN");
	// }
	//
	// Element tempElementZiel = it.next();
	// Element tempElement = cloneElementStruktur(tempElementZiel);
	// tempElement.detach();
	// System.out.println("TEMPELEMENT: " + tempElement.getAttributeValue("id"));
	// childsNeu.add(tempElement);
	// }
	//
	// // Korrektur fr das Einfgen am Ende
	// if (position == childsEmZiel.size())
	// {
	// childsNeu.add(emEinfuegen);
	// }
	//
	// emZiel.setContent(childsNeu);
	//
	// if (_debug)
	// {
	// System.out.println(getClass().getName() + " => Anzahl Kinder: "
	// + childsNeu.size());
	//
	// System.out.println("\nNachher: ");
	// ausgabeParser(0, emZiel);
	// }
	//
	// }
	/**
	 * Prft, ob eine Position innerhalb der Kindelemente eines Elements vorhanden
	 * ist.
	 * 
	 * @param element
	 *          Element zu dem geprft werden soll, ob die angegebene Position
	 *          innehalb der Kindelemente existiert
	 * @param position
	 *          Position, die geprft werden soll
	 * @return <code>true</code>, wenn die Position innerhalb der Kindelemente
	 *         existiert, sonst <code>false</code>
	 */
	public boolean pruefePositionInElement(Element element, int position)
	{
		boolean returnWert = false;
		int anzahlKinder = element.getChildren().size();

		if (position < anzahlKinder)
		{
			returnWert = true;
		}
		return returnWert;
	}

	// /**
	// * Liefert die letzte Position der Kindelemente eines Elements
	// *
	// * @param element
	// * Element, zu dem die letzte Position innerhalb der Kindelemente
	// * bestimmt werden soll
	// * @return Bestimmte Position
	// *
	// * @deprecated
	// */
	// public int getLetztePositionInElement(Element element)
	// {
	// int returnWert = -1;
	// int anzahlKinder = element.getChildren().size();
	// returnWert = anzahlKinder;
	// return returnWert;
	// }

	// ////
	//
	/*
	 * Suchen und Finden
	 */

	/*
	 * Direkt
	 */

	// /**
	// * Sucht ein Item an Hand seiner Id
	// *
	// * @param id
	// * Id des Items, dass gesucht werden soll
	// * @return Das gefundene Item oder null
	// */
	// public Element sucheItem(String id)
	// {
	// Element returnElement = null;
	//
	// Element hauptMenu = _elementWurzel.getChild(_elementMenuName);
	// returnElement = sucheItemRekursiv(id, hauptMenu);
	//
	// return returnElement;
	// }
	/**
	 * Rekursive Hilfssuche fr Item
	 * 
	 * @param id
	 *          Id des zu suchenden Items
	 * @param element
	 *          Element, dass durchsucht werden soll
	 * @return Das gefundene Item oder null
	 */
	@SuppressWarnings("unused")
	private Element sucheItemRekursiv(String id, Element element)
	{
		Element returnElement = null;

		// Men?
		if (element.getName().equalsIgnoreCase(_elementMenuName))
		{
			@SuppressWarnings("unchecked")
			Iterator it = element.getChildren().iterator();
			while (it.hasNext() && returnElement == null)
			{
				Element tempElement = (Element) it.next();

				// Item?
				if (tempElement.getName().equalsIgnoreCase(_elementItemName))
				{
					String tempId = tempElement.getAttributeValue(_elementItemAttrId);
					if (tempId.equalsIgnoreCase(id))
					{
						returnElement = tempElement;
						break;
					}
				}

				// Submen?
				if (tempElement.getName().equalsIgnoreCase(_elementSubMenuName))
				{
					returnElement = sucheItemRekursiv(id, tempElement);
				}

			}
		}

		// Submen
		if (element.getName().equalsIgnoreCase(_elementSubMenuName))
		{
			if (element.getChildren().size() > 0)
			{
				Element menu = element.getChild(_elementMenuName);
				returnElement = sucheItemRekursiv(id, menu);
			}
		}

		return returnElement;
	}

	/**
	 * Sucht einen Separator an Hand seiner Id.
	 * 
	 * @param id
	 *          Id des Separators, der gesucht werden soll.
	 * @return Der gefundene Separator oder null.
	 */
	// public Element sucheSeparator(String id)
	// {
	// Element returnElement = null;
	// return returnElement;
	// }
	/**
	 * Rekursive Hilfssuche fr Separatoren.
	 * 
	 * @param id
	 *          Id des zu suchenden Separators.
	 * @param element
	 *          Element, dass durchsucht werden soll.
	 * @return Der gefundene Separator oder null.
	 */
	// private Element sucheSeparatorRekursiv(String id, Element element)
	// {
	// Element returnElement = null;
	// return returnElement;
	// }
	/**
	 * Sucht einen GroupMarker an Hand seiner Id.
	 * 
	 * @param id
	 *          Id des GroupMarkers, der gesucht werden soll.
	 * @return Der gefundene GroupMarker oder null.
	 */
	// public Element sucheGroupMarker(String id)
	// {
	// Element returnElement = null;
	// return returnElement;
	// }
	/**
	 * Rekursive Hilfssuche fr GroupMarker.
	 * 
	 * @param id
	 *          Id des zu suchenden GroupMarkers.
	 * @param element
	 *          Element, dass durchsucht werden soll.
	 * @return Der gefundene GroupMarker oder null.
	 */
	// private Element sucheGroupMarkerRekursiv(String id, Element element)
	// {
	// Element returnElement = null;
	// return returnElement;
	// }
	/**
	 * Sucht das zu einer Id zugehrige Men
	 * 
	 * @param id
	 *          Id des Mens, das gesucht werden soll
	 * @return Element, dass das gesuchte Men enthlt oder null
	 */
	public Element sucheMenu(String id)
	{
		Element returnElement = null;
		Element hauptMenu = _elementWurzel.getChild(_elementMenuName);

		// Ist das Hauptmen das gesuchte Men?
		if (hauptMenu.getAttributeValue(_elementMenuAttrId).equalsIgnoreCase(id))
		{
			returnElement = hauptMenu;
		} else
		{
			returnElement = sucheMenuRekursiv(id, hauptMenu);
		}
		return returnElement;
	}

	/**
	 * Rekursive Hilfssuche fr Mens
	 * 
	 * @param id
	 *          Id des Mens, dass gesucht werden soll
	 * @param element
	 *          Element, dass durchsucht werden soll
	 * @return Gefundenes Men oder null
	 */
	private Element sucheMenuRekursiv(String id, Element element)
	{
		Element returnElement = null;

		// Men?
		if (element.getName().equalsIgnoreCase(_elementMenuName))
		{
			if (element.getAttributeValue(_elementMenuAttrId).equalsIgnoreCase(id))
			{
				returnElement = element;
			} else
			{
				// Submens
				@SuppressWarnings("unchecked")
				List subMenus = element.getChildren(_elementSubMenuName);
				if (subMenus.size() > 0)
				{
					@SuppressWarnings("unchecked")
					Iterator it = subMenus.iterator();
					while (it.hasNext() && returnElement == null)
					{
						Element subMenu = (Element) it.next();
						returnElement = sucheMenuRekursiv(id, subMenu);
					}
				}
			}
		}

		// Submen?
		if (element.getName().equalsIgnoreCase(_elementSubMenuName))
		{
			@SuppressWarnings("unchecked")
			Iterator it = element.getChildren(_elementMenuName).iterator();
			while (it.hasNext())
			{
				Element elementMenu = (Element) it.next();
				_debug.fine("-Men: " + elementMenu.getAttributeValue("id"));
				if (elementMenu.getAttributeValue(_elementItemAttrId).equalsIgnoreCase(
						id))
				{
					returnElement = elementMenu;
					break;
				} else
				{
					returnElement = sucheMenuRekursiv(id, elementMenu);
				}

			}
		}

		return returnElement;
	}

	/**
	 * Sucht ein Submen mit einer bestimmten Id und liefert es als
	 * <code>Element</code>
	 * 
	 * @param id
	 *          Id des Submens, dass gesucht werden soll
	 * @return Gefundenes Submen als <code>Element</code>, sonst
	 *         <code>false</code>
	 */
	public Element sucheSubMenu(String id)
	{
		Element returnElement = null;
		Element hauptMenu = _elementWurzel.getChild(_elementMenuName);
		returnElement = sucheSubMenuRekursiv(id, hauptMenu);
		return returnElement;
	}

	/**
	 * Rekursive Suchfunktion zu <code>sucheSubMen</code>
	 * 
	 * @param id
	 *          Id des zu suchenden Submens
	 * @param element
	 *          <code>Element</code>, in dem nach dem Submen gesucht werden soll
	 * @return <code>Element</code> des gefundenen Submens oder <code>null</code>
	 *         .
	 */
	private Element sucheSubMenuRekursiv(String id, Element element)
	{
		Element returnElement = null;

		// Submenu?
		if (element.getName().equalsIgnoreCase(_elementSubMenuName))
		{
			if (element.getAttributeValue(_elementSubMenuAttrId).equalsIgnoreCase(id))
			{
				returnElement = element;
			} else
			{
				@SuppressWarnings("unchecked")
				List children = element.getChildren(_elementMenuName);
				@SuppressWarnings("unchecked")
				Iterator it = children.iterator();
				while (it.hasNext() && returnElement == null)
				{
					Element tempElement = (Element) it.next();
					returnElement = sucheSubMenuRekursiv(id, tempElement);
				}
			}

		}

		// Menu?
		if (element.getName().equalsIgnoreCase(_elementMenuName))
		{
			@SuppressWarnings("unchecked")
			Iterator it = element.getChildren(_elementSubMenuName).iterator();

			while (it.hasNext())
			{
				Element tempElement = (Element) it.next();
				Element returnCheckElement = sucheSubMenuRekursiv(id, tempElement);
				if (returnCheckElement != null)
				{
					returnElement = returnCheckElement;
				}

			}
		}

		return returnElement;
	}

	//
	// ////

	/**
	 * Hilfsmethode zu cloneElementStruktur, fhrt das rekursive Clonen aller
	 * Kindelemente im XML-Baum durch
	 * 
	 * @param em
	 *          <code>Element</code>, dass mit seinen Kindelementen geklont werden
	 *          soll
	 * @return Geclonte Elementstruktur
	 */
	@SuppressWarnings("unused")
	private Element kopiereElementStrukturRekursiv(Element em)
	{
		Element returnElement = null;

		// Item
		if (em.getName().equalsIgnoreCase(_elementItemName))
		{
			XmlElementItem tempItem = new XmlElementItem();
			tempItem.generiereAusElement(em);
			returnElement = tempItem.getElementItem();
		}

		// Menu
		if (em.getName().equalsIgnoreCase(_elementMenuName))
		{
			XmlElementMenu tempMenu = new XmlElementMenu();
			tempMenu.generiereAusElement(em);
			returnElement = tempMenu.getElementMenu();
		}

		// Submenu
		if (em.getName().equalsIgnoreCase(_elementSubMenuName))
		{
			XmlElementSubmenu submenu = new XmlElementSubmenu();
			submenu.generiereAusElement(em);
			returnElement = submenu.getElementSubmenu();
		}

		// Separator
		if (em.getName().equalsIgnoreCase(_elementSeparatorName))
		{
			XmlElementSeparator separator = new XmlElementSeparator();
			separator.generiereAusElement(em);
			returnElement = separator.getElementSeparator();
		}

		@SuppressWarnings("unchecked")
		Iterator it = em.getChildren().iterator();
		while (it.hasNext())
		{
			Element tempEm = (Element) it.next();
			Element element = kopiereElementStrukturRekursiv(tempEm);
			returnElement.addContent(element);
		}
		return returnElement;
	}

	/**
	 * Liefert die Wurzel des JDOM-Baumes als Element.
	 * 
	 * @return Wurzel des JDOM-Baumes als Element.
	 */
	public Element getWurzel()
	{
		return _elementWurzel;
	}

	/**
	 * Setzt das Dirty-Flag.
	 * 
	 * @param wert
	 *          <code>true</code> oder <code>false</code>.
	 */
	public void setSchmutzig(boolean wert)
	{
		this._schmutzig = wert;

		if (_feuereSchmutzigGeandert != null)
		{
			_feuereSchmutzigGeandert.feuereSchmutzigGeandert();
		}
	}

	/**
	 * Setzt den Listener, der ber eine nderung des Dirty-Flags informiert
	 * werden mchte.<br>
	 * <b>Achtung:</b> Es kann nur einen Listener geben, da die Struktur genau
	 * einem Editor zugeordnet wird.
	 * 
	 * @param feuereSchmutzigGeandert
	 */
	public void setFeuereSchmutzigGeaendert(
			IFeuereSchmutzigGeandert feuereSchmutzigGeandert)
	{
		this._feuereSchmutzigGeandert = feuereSchmutzigGeandert;
	}

	/**
	 * Liefert <code>true</code>, wenn sich etwas an der DatenStruktur gendert
	 * hat, sonst <code>false</code>.
	 * 
	 * @return
	 */
	public boolean istSchmutzig()
	{
		return _schmutzig;
	}

	/**
	 * Liefert eine String-Darstellung eines beliebigen Elements vom Typ
	 * org.jdom.Element. Kindelemente werden bei der Darstellung NICHT
	 * bercksichtigt
	 * 
	 * @param element
	 *          Element
	 * @return String-Darstellung des Elements
	 */
	public String ElementNachString(Element element)
	{
		String returnString = "";
		returnString = returnString + element.getName() + "[";
		@SuppressWarnings("unchecked")
		Iterator it = element.getAttributes().iterator();
		while (it.hasNext())
		{
			Attribute attr = (Attribute) it.next();
			returnString = returnString + "A(" + attr.getName() + ","
					+ attr.getValue() + ");";
		}
		returnString = returnString + "]";

		return returnString;
	}

	/**
	 * Erzeugt eine Kopie dieser Klasse. Dabei wird folgendes kopiert:<br>
	 * <ul>
	 * <li>Dokument, tiefe Kopie,</li>
	 * <li>ElementWurzel, tiefe Kopie,</li>
	 * <li>Dirty-Flag, tiefe Kopie,</li>
	 * <li>Listener, der auf nderungen des Dirty-Flags hrt, Kopie der Referenz!</li>
	 * <li>Undo-Stack</li>
	 * </ul>
	 */
	public MenuSymbolLeisteDaten clone()
	{
		MenuSymbolLeisteDaten msld = new MenuSymbolLeisteDaten();

		String ausgabeXmlFormatiertNoDirty = this.ausgabeXmlFormatiertNoDirty();
		msld.einlesenXmlAusString(ausgabeXmlFormatiertNoDirty);

		msld._schmutzig = this._schmutzig;

		if (this._feuereSchmutzigGeandert != null)
		{
			msld._feuereSchmutzigGeandert = this._feuereSchmutzigGeandert;
			msld._feuereSchmutzigGeandert.feuereSchmutzigGeandert();
		}

		msld._undoStack = this._undoStack;

		return msld;
	}

	/**
	 * Bestimmt die nchste, freie ID.
	 * 
	 * @param wurzelElement
	 *          WurzelElement des JDOM-Baumes, von dem an die nchste freie ID
	 *          bestimmt werden soll.
	 * @param namePruefElement
	 *          Name des Elementes, fr welches die nchste freie ID zu bestimmen
	 *          ist.
	 * @param nameAttributID
	 *          Name des Attributes, welches fr das Element die ID beinhaltet.
	 * @return Liefert die nchste freie ID >= 0.
	 */
	public int bestimmeFreieId(Element wurzelElement, String namePruefElement,
			String nameAttributID)
	{
		int re = 0;

		LinkedList<Integer> ids = browseIds(wurzelElement, namePruefElement,
				nameAttributID);

		if (ids.size() > 0)
		{
			Collections.sort(ids);

			// Iterator<Integer> it = ids.iterator();
			// while (it.hasNext())
			// {
			// Integer wert = it.next();
			// System.out.println("WERT: " + wert);
			// }

			for (int i = 0; i < ids.size(); i++)
			{
				if (re == ids.get(i))
				{
					re = re + 1;
				} else
				{
					break;
				}
			}

		}

		return re;
	}

	/**
	 * Durchluft rekursiv die Baumstruktur und bestimmt die IDs der jeweiligen
	 * {@link Element}e.
	 * 
	 * @param element
	 *          {@link Element}, von dem an die Baumstruktur rekursiv durchlaufen
	 *          werden soll.
	 * @param namePruefElement
	 *          Name der zu bercksichtigenden {@link Element}e.
	 * @param nameAttributId
	 *          Name des {@link Attribute}s, welches die ID eines {@link Element}
	 *          es bestimmt.
	 * @return Eine Liste mit den IDs der jeweiligen {@link Element}e als
	 *         {@link Integer}.
	 */
	private LinkedList<Integer> browseIds(Element element,
			String namePruefElement, String nameAttributId)
	{
		LinkedList<Integer> re = new LinkedList<Integer>();

		/*
		 * Eigene ID bestimmen und zur Liste der IDs hinzufgen
		 */
		if (element.getName().equals(namePruefElement))
		{
			String value = element.getAttributeValue(nameAttributId);
			if (value.length() > 0)
			{
				try
				{
					Integer add = Integer.valueOf(value);
					re.add(add);

				} catch (NumberFormatException nfe)
				{

				}
			}
		}

		/*
		 * IDs der Kindelemente bestimmen
		 */
		if (element.getChildren().size() > 0)
		{
			@SuppressWarnings("unchecked")
			Iterator it = element.getChildren().iterator();
			while (it.hasNext())
			{
				Object child = it.next();
				if (child instanceof Element)
				{
					Element childElement = (Element) child;
					LinkedList<Integer> ids = browseIds(childElement, namePruefElement,
							nameAttributId);
					re.addAll(ids);
				}
			}
		}

		return re;
	}

}
