/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.xml.datenstruktur;

import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Klasse zur Reprsentation eines "Separators". Die Definition eines Separators richtet sich nach
 * <code>menubar.dtd</code> bzw. <code>toolbar.dtd</code>.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:05 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class XmlElementSeparator
{
  private String _id = null;

  private String _name = null;

  private Element _elSeparator = null;

  private String _elSeparatorName = XmlBezeichnerMenuSymbol.elementSeparator;

  private String _elSeparatorAttributId = XmlBezeichnerMenuSymbol.attId;

  private String _elSeparatorAttributName = XmlBezeichnerMenuSymbol.attName;

  /**
   * Parameterloser Konstruktor
   * 
   */
  public XmlElementSeparator()
  {
  }

  /**
   * Konstruktor, erzeugt direkt einen Separator mit id und name
   * 
   * @param id
   *          Id des zu erzeugenden Separators.
   * @param name
   *          Name des zu erzeugenden Separator.
   */
  public XmlElementSeparator(String id, String name)
  {
    this._id = id;
    this._name = name;
    generiereElement();
  }

  /**
   * Generiert das <code>Element</code> fr den <code>JDOM</code>-Baum
   * 
   */
  private void generiereElement()
  {
    _elSeparator = new Element(_elSeparatorName);
    Attribute elItemId = new Attribute(_elSeparatorAttributId, _id);
    Attribute elItemName = new Attribute(_elSeparatorAttributName, _name);
    _elSeparator.setAttribute(elItemId);
    _elSeparator.setAttribute(elItemName);
  }

  /**
   * Generiert die Objektreprsentation eines Separators aus einem <code>Element</code>. Es wird hierzu kein Clone
   * erstellt, folglich bleiben Eltern und Kinder des Elements erhalten.
   * 
   * @param el
   *          <code>Element</code>, zu dem die Objektreprsentation erstellt wird.
   */
  public void generiereElement(Element el)
  {
    if (el.getName().equalsIgnoreCase(_elSeparatorName) && el.getAttributes().size() == 2)
    {
      _elSeparator = el;
      _id = el.getAttributeValue(_elSeparatorAttributId);
      _name = el.getAttributeValue(_elSeparatorAttributName);
    }
  }

  /**
   * Generiert die Objektreprsentation eines Separators aus einem <code>Element</code>. Es handelt sich hierbei um
   * eine flache Kopie, Eltern- und Kindelemente gehen verloren.
   * 
   * @param el
   *          <code>Element</code>, von dem eine flache Kopie erstellt werden soll.
   */
  public void generiereAusElement(Element el)
  {
    if (el.getName().equalsIgnoreCase(_elSeparatorName) && el.getAttributes().size() == 2)
    {
      _id = el.getAttributeValue(_elSeparatorAttributId);
      _name = el.getAttributeValue(_elSeparatorAttributName);
      generiereElement();
    }
  }

  /**
   * Liefert das <code>Element</code> des Separators.
   * 
   * @return Das <code>Element</code> des Separators.
   */
  public Element getElementSeparator()
  {
    return _elSeparator;
  }

  /**
   * Liefert eine String-Reprsentation des Separators.
   * 
   * @return Stringreprsentation des Separators.
   */
  public String toString()
  {
    String returnString = "Element: " + _elSeparatorName + " Id: " + _id + " Name: " + _name;
    return returnString;
  }

}