/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur;

import org.eclipse.jface.action.IContributionItem;
import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Diese Klasse kapselt die Elemente der Statuszeile, so wie sie von der Klasse {@link StatusLeisteVerwaltung} bentigt
 * werden.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:02 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class StatusLeisteElement
{
  /**
   * ID des Statuszeilenelements.
   */
  private String _id = null;

  /**
   * Optionale Bezeichnung des Statuszeilenelements.
   */
  private String _bezeichnung = null;

  /**
   * Das {@link IContributionItem}, dass das eigentliche Statuszeilenelement liefert.
   */
  private IContributionItem _contributionItem = null;

  /**
   * Konstruktor, setzt ID, Bezeichnung und {@link IContributionItem}.
   * 
   * @param id
   *          ID des Elements.
   * @param bezeichnung
   *          Bezeichnung des Elements.
   * @param contributionItem
   *          {@link IContributionItem} des Elements.
   */
  public StatusLeisteElement(String id, String bezeichnung, IContributionItem contributionItem)
  {
    this._id = id;
    this._bezeichnung = bezeichnung;
    this._contributionItem = contributionItem;
  }

  /**
   * Konstruktor, setzt ID un {@link IContributionItem}.
   * 
   * @param id
   *          ID des Elements.
   * @param contributionItem
   *          {@link IContributionItem} des Elements.
   */
  public StatusLeisteElement(String id, IContributionItem contributionItem)
  {
    this._id = id;
    this._contributionItem = contributionItem;
  }

  /**
   * Liefert die Id des Elements.
   * 
   * @return Id
   */
  public String getId()
  {
    return _id;
  }

  /**
   * Liefert die Bezeichnung des Elements.
   * 
   * @return Bezeichnung
   */
  public String getBezeichnung()
  {
    return _bezeichnung;
  }

  /**
   * Liefert das {@link IContributionItem} hinter diesem Element
   * 
   * @return {@link IContributionItem}.
   */
  public IContributionItem getElement()
  {
    return _contributionItem;
  }

  /**
   * Liefert die zugehrige XML-Darstellung.
   * 
   * @return JDOM-Element der XML-Darstellung.
   */
  public Element konvertiereInJDOMElement()
  {
    Element element = new Element(XmlBezeichnerStatus.elementElement);
    Attribute attId = new Attribute(XmlBezeichnerStatus.attributId, this._id);
    element.setAttribute(attId);
    Attribute attBezeichnung = new Attribute(XmlBezeichnerStatus.attributBezeichnung, this._bezeichnung);
    element.setAttribute(attBezeichnung);
    return element;
  }

}
