/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert die Verwaltung der Elemente der Statuszeile. <br>
 * Hierzu gehrt auch, die Struktur aus einer XML-Datei zu lesen bzw. diese in
 * eine XML-Datei zu schreiben. <br>
 * Diese Klasse ist als Singleton ausgelegt. <br>
 * Die Methode <code>aktualisieren</code> muss manuell aufgerufen werden.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:02 $ / ($Author:
 *          HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class StatusLeisteVerwaltung
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * HashMap der StatusLeistenElemente. Bauen sich auf aus Id und
	 * StatusLeisteElement
	 */
	private TreeMap<String, StatusLeisteElement> _verfuegbareElemente = new TreeMap<String, StatusLeisteElement>();

	/**
	 * JDOM Document, in dem die Verarbeitung erfolgt
	 */
	private Document _doc = null;

	/**
	 * Wurzel-Element
	 */
	private Element _elementWurzel = null;

	/**
	 * XML Encoding
	 */
	private String _xmlEncoding = "ISO-8859-1";

	/**
	 * Liste aller registrierten Zuhrer.
	 */
	private List<IZuhoererStatusLeisteAenderung> _zuhoerer = new LinkedList<IZuhoererStatusLeisteAenderung>();

	/**
	 * UNDO-Stack
	 */
	private UndoStackStatusLeiste _undoStack = new UndoStackStatusLeiste();

	/**
	 * Private Konstruktor
	 * 
	 */
	private StatusLeisteVerwaltung()
	{
		init();
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link StatusLeisteVerwaltung}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen
	{
		private static StatusLeisteVerwaltung INSTANZ = new StatusLeisteVerwaltung();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link StatusLeisteVerwaltung}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link StatusLeisteVerwaltung}
	 */
	public static StatusLeisteVerwaltung getInstanz()
	{
		return Innen.INSTANZ;
	}

	/**
	 * Fhrt die Initialisierung durch
	 * 
	 */
	private void init()
	{
		// XML-Document initialisieren
		erzeugeXmlBasisDoc();
	}

	/**
	 * Fgt eine Element zur Statuszeile hinzu.
	 * 
	 * @param element
	 *          Hinzuzufgendes Element.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn das Element bereits existiert.
	 */
	public void hinzufuegenElement(StatusLeisteElement element)
			throws IllegalArgumentException
	{
		/*
		 * Einfgen i.d. HashMap falls nicht vorhanden Sortierung via XML-File
		 * Einfgen via Klasse StatuLeisteZugriff
		 */
		synchronized (_verfuegbareElemente)
		{
			if (!_verfuegbareElemente.containsKey(element.getId()))
			{
				_verfuegbareElemente.put(element.getId(), element);
			}
			else
			{
				throw new IllegalArgumentException(
						"Es existiert bereits eine Statusleistenelement mit der Id "
								+ element.getId()
								+ ". Es kann nicht nochmal i.d. Statusleiste eingefgt werden.");
			}
		}

	}

	// /**
	// * Fgt ein Element an eine bestimmte Stelle in der Statuszeile ein.
	// Existiert
	// * die angegebene Position nicht, wird das Element am Ende der Statusleiste
	// * eingefgt.
	// *
	// * @param element
	// * Hinzuzufgendes Element.
	// * @param position
	// * Position des Elements i.d. Statuszeile.
	// * @throws IllegalArgumentException
	// * Wird geworfen, wenn das Element bereits existiert.
	// */
	// public void hinzufuegenElementAnPosition(StatusLeisteElement element,
	// int position) throws IllegalArgumentException
	// {
	// /*
	// * Suchen ob bereits vorhanden. Einfgen via Klasse StatusLeisteZugriff an
	// * passender Stelle. XML-File anpassen, einfgen an letzter Stelle etc...
	// */
	// if (!_elemente.containsKey(element.getId()))
	// {
	// _elemente.put(element.getId(), element);
	// } else
	// {
	// throw new IllegalArgumentException(
	// "Es existiert bereits eine Statusleistenelement mit der Id "
	// + element.getId()
	// + ". Es kann nicht nochmal i.d. Statusleiste eingefgt werden.");
	// }
	// }

	// /**
	// * Verschiebt ein vorhandenes Element an eine neue Position.
	// *
	// * @param statusLeisteElement
	// * Element, welches verschoben werden soll.
	// * @param neuePosition
	// * Position, an die das Element verschoben werden soll.
	// * @throws IllegalArgumentException
	// * Wird geworfen, wenn das zu verschiebende Element nicht bekannt
	// * ist.
	// */
	// public void verschiebeElementAnPosition(
	// StatusLeisteElement statusLeisteElement, int neuePosition)
	// throws IllegalArgumentException
	// {
	// if (_elemente.containsKey(statusLeisteElement.getId()))
	// {
	// // Verschieben im XML-File, Statusleiste neue aufbauen.
	// } else
	// {
	// throw new IllegalArgumentException("Das Element mit der Id "
	// + statusLeisteElement.getId() + " existiert nicht.");
	// }
	// }

	/**
	 * Entfernt ein Element aus der Statuszeile.
	 * 
	 * @param element
	 *          Zu entfernendes Element.
	 */
	public void entferneElement(StatusLeisteElement element)
	{
		/*
		 * Suchen ob vorhanden. Aus _elemente entfernen. Aus StatusLeiseteZugriff
		 * entfernen.
		 */
		synchronized (_verfuegbareElemente)
		{
			if (_verfuegbareElemente.containsKey(element.getId()))
			{
				_verfuegbareElemente.remove(element.getId());
			}
			else
			{
				throw new IllegalArgumentException(
						"Es existiert kein Element mit der Id " + element.getId()
								+ ", dass aus der Statusleiste entfernt werden kann.");
			}
		}
	}

	/**
	 * Liefert ein {@link StatusLeisteElement} an Hand seiner ID.
	 * 
	 * @param id
	 *          Id des StatusLeisteElements.
	 * @return Das StatusLeisteElement.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn das StatusLeisteElement nicht existiert.
	 */
	public StatusLeisteElement getStatusLeistenElement(String id)
			throws IllegalArgumentException
	{
		synchronized (_verfuegbareElemente)
		{
			if (_verfuegbareElemente.containsKey(id))
			{
				return _verfuegbareElemente.get(id);
			}
			else
			{
				throw new IllegalArgumentException(
						"Es existiert kein StatusLeistenElement mit der ID " + id);
			}
		}
	}

	/**
	 * Liest die Sortierung der Elemente der Statuszeile aus einer XML-Struktur in
	 * einem String und baut die Struktur der Statusleiste demenstprechend auf.
	 * <br>
	 * Ist ein Element nicht in der HashMap der Elemente vorhanden, wird dieses
	 * nicht dargestellt. Es wird kein Fehler geworfen.
	 * 
	 * @param xmlStruktur
	 *          XML-Struktur.
	 */
	public void leseSortierungAusXmlString(String xmlStruktur)
	{
		einlesenXmlAusString(xmlStruktur);
		// feuereEreignis();
	}

	/**
	 * Liest die Sortierung der Elemente der Statuszeile aus einer XML-Struktur in
	 * einer Datei und baut die Struktur der Statusleiste dem entsprechend auf.
	 * <br>
	 * Ist ein Element nicht in der {@link HashMap} der Elemente vorhanden, wird
	 * dieses nicht dargestellt. Es wird kein Fehler geworfen.
	 * 
	 * @param dateiPfad
	 *          Pfad zur XML-Datei.
	 */
	public void leseSortierungAusXmlDatei(String dateiPfad)
	{
		einlesenXmlAusDatei(dateiPfad);
		// feuereEreignis();
	}

	/**
	 * Schreibt die Sortierung der Elemente der Statuszeile in eine XML-Struktur.
	 * 
	 * @return XML-Struktur.
	 */
	public String schreibeSortierungInXmlString()
	{
		String returnString = ausgabeXmlFormatiert();
		// feuereEreignis();
		return returnString;
	}

	/**
	 * Schreibt die Sortierung der Elemente der Statuszeile als XML-Struktur in
	 * eine Datei.
	 * 
	 * @param dateiPfad
	 *          Pfad zur Datei, in die die XML-Struktur geschrieben werden soll.
	 * @param ueberschreiben
	 *          Eine bereits vorhandene Datei wird ber schrieben, wenn
	 *          <code>true</code>.
	 */
	public void schreibeSortierungInXmlDatei(String dateiPfad,
			boolean ueberschreiben)
	{
		schreibeString(dateiPfad, ausgabeXmlFormatiert(), ueberschreiben);
	}

	/**
	 * Erzeugt das Basisdokument, lehnt sich an toolbar.dtd an
	 * 
	 */
	private void erzeugeXmlBasisDoc()
	{
		_doc = new Document();
		_elementWurzel = new Element(XmlBezeichnerStatus.elementStatusleiste);
		_doc.setRootElement(_elementWurzel);
	}

	/**
	 * Liefert eine formatierte Ausgabe der erzeugten XML-Struktur
	 * 
	 * @return Formatierte XML-Struktur
	 */
	private String ausgabeXmlFormatiert()
	{
		String returnString = null;
		if (_doc == null)
		{
			_doc = new Document(_elementWurzel);
		}

		// Doctype setzten
		/*
		 * if (_setDocType) { DocType doctype = new DocType(_xmlDocTypeElementName,
		 * _xmlDocTypePublicId, _xmlDocTypeSystemId); _doc.setDocType(doctype); }
		 */

		Format prettyFormat = Format.getPrettyFormat();
		XMLOutputter out = new XMLOutputter(prettyFormat);
		Format f = out.getFormat();
		f.setEncoding(_xmlEncoding);
		out.setFormat(f);

		returnString = out.outputString(_doc);

		_debug.fine(getClass().getName()
				+ "::ausgabeXmlFormatiert => Ausgabestring:\n" + returnString);

		// feuereEreignis();

		return returnString;
	}

	/**
	 * Schreibt einen String in eine Datei
	 * 
	 * @param dateiPfad
	 *          Kompletter Pfad zur Datei, in die geschrieben werden soll
	 * @param schreibString
	 *          String, der in die Datei geschrieben werden soll
	 * @param ueberschreiben
	 *          true == berschreiben der Datei, falls schon vorhanden
	 * @throws IllegalArgumentException
	 *           Wir geworfen, wenn die Datei existiert und nicht berschrieben
	 *           werden kann
	 */
	private void schreibeString(String dateiPfad, String schreibString,
			boolean ueberschreiben) throws IllegalArgumentException
	{
		File file = new File(dateiPfad);

		// File existiert, auf berschreiben prfen
		if (file.exists() && ueberschreiben)
		{
			try
			{
				PrintWriter pw = new PrintWriter(file);
				pw.print(schreibString);
				pw.close();
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}
		// File existiert nicht und kann einfach angelegt werden
		else if (!file.exists())
		{
			try
			{
				PrintWriter pw = new PrintWriter(file);
				pw.print(schreibString);
				pw.close();
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}
		// File existiert und ueberschreiben == flase
		else
		{
			throw new IllegalArgumentException("Die Datei " + dateiPfad
					+ " existiert und ueberschreiben == flase");
		}
	}

	/**
	 * Liest eine XML-Datei ein
	 * 
	 * @param dateiPfad
	 *          Kompletter Pfad zur Datei
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn die Datei nicht existiert
	 */
	private void einlesenXmlAusDatei(String dateiPfad)
			throws IllegalArgumentException
	{
		// SAXBuilder builder = new SAXBuilder();
		String dateiInhalt = "";

		File file = new File(dateiPfad);
		if (file.exists())
		{

			FileReader fr;
			try
			{
				String line;
				fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				while ((line = br.readLine()) != null)
				{
					dateiInhalt = dateiInhalt + line + "\n";
				}
				einlesenXmlAusString(dateiInhalt);

			}
			catch (FileNotFoundException e)
			{
				_debug.fine(getClass().getName()
						+ "::einlesenXmlausDatei => Die Datei existiert nicht: \n"
						+ e.toString());
			}
			catch (IOException ioe)
			{
				_debug.fine(getClass().getName()
						+ "::einlesenXmlausDatei => IO-Fehler: \n" + ioe.toString());
			}
		}
		else
		{
			throw new IllegalArgumentException("Die Datei " + dateiPfad
					+ " existiert nicht!");
		}
	}

	/**
	 * Liest eine XML-Struktur aus einer XML-Struktur ein.
	 * 
	 * @param xmlString
	 *          String mit XML-Struktur.
	 */
	private void einlesenXmlAusString(String xmlString)
	{
		SAXBuilder builder = new SAXBuilder();
		builder.setValidation(false);
		try
		{
			StringReader reader = new StringReader(xmlString);

			Document tempDoc = builder.build(reader);
			if (tempDoc.getRootElement().getName().equals(
					XmlBezeichnerStatus.elementStatusleiste))
			{
				// alles ok
				_doc = tempDoc;
				_elementWurzel = _doc.getRootElement();
				// feuereEreignis();
			}
			else
			{
				_debug
						.fine(getClass().getName()
								+ "::einlesenXmlAusString => Fehler: Ungltiges Format der bergebenen XML-Struktur.");
			}

			// _doc = builder.build(reader);
			// _elementWurzel = _doc.getRootElement();
		}
		catch (JDOMException e)
		{
			_debug.fine(getClass().getName()
					+ "::einlesenXmlAusString => JDOMException: \n" + e.toString());
		}
		catch (IOException e)
		{
			_debug.fine(getClass().getName()
					+ "::einlesenXmlAusString => IOException: \n" + e.toString());
		}
		catch (NullPointerException e)
		{
			if (xmlString == null)
			{
				_debug.fine(getClass().getName()
						+ "::einlesenXmlAusString => Der bergebene Xml-String ist null:\n"
						+ e.toString());
			}
		}
	}

	/**
	 * Fgt einen {@link IZuhoererStatusLeisteAenderung} zur Liste der Zuhrer
	 * hinzu.
	 * 
	 * @param zuhoerer
	 *          {@link IZuhoererStatusLeisteAenderung}.
	 */
	public void hinzufuegenZuhoerer(IZuhoererStatusLeisteAenderung zuhoerer)
	{
		_debug
				.fine(getClass().getName()
						+ "::hinzufuegenZuhoerer \n\t=> Ein Zuhrer wurde zur Liste der Zuhrer hinzugefgt. \n");

		synchronized (_zuhoerer)
		{
			_zuhoerer.add(zuhoerer);
		}
	}

	/**
	 * Entfernt einen {@link IZuhoererStatusLeisteAenderung} von der Liste der
	 * Zuhrer.
	 * 
	 * @param zuhoerer
	 *          {@link IZuhoererStatusLeisteAenderung}.
	 */
	public void entferneZuhoerer(IZuhoererStatusLeisteAenderung zuhoerer)
	{
		_debug
				.fine(getClass().getName()
						+ "::hinzufuegenZuhoerer \n\t=> Ein Zuhrer wurde von der Liste der Zuhrer entfernt. \n");

		synchronized (_zuhoerer)
		{
			_zuhoerer.remove(zuhoerer);
		}
	}

	/**
	 * Abfeuern des Ereignis, benachrichtigen aller Zuhrer.
	 * 
	 */
	private void feuereEreignis()
	{
		_debug.fine(this.getClass().getName()
				+ "::feuereEreignis \n\t=> Event abgefeuert!\n");
		_debug.fine(this.getClass().getName()
				+ "::feuereEreignis \n\t=> Anzahl Listener: " + _zuhoerer.size()
				+ ".\n");

		if (_zuhoerer.size() > 0)
		{
			EreignisStatusLeisteGeaendert e = new EreignisStatusLeisteGeaendert(
					"Statusleiste gendert.");
			Iterator<IZuhoererStatusLeisteAenderung> it = _zuhoerer.iterator();
			while (it.hasNext())
			{
				IZuhoererStatusLeisteAenderung aenderung = it.next();
				aenderung.ausfuehren(e);
			}
		}
	}

	public void aktualisieren()
	{
		feuereEreignis();
	}

	/**
	 * Fgt ein XmlElement zur Wurzel hinzu, wenn es vom Typ element ist.
	 * 
	 * @param quellElement
	 *          Hinzuzufgendes XmlElement.
	 */
	public void hinzufuegenXmlElement(Element quellElement)
	{
		if (quellElement.getName().equals(XmlBezeichnerStatus.elementElement))
		{
			sichereAktuellenZustandAufUndoStack();

			quellElement.detach();
			synchronized (_elementWurzel)
			{
				_elementWurzel.addContent(quellElement);
			}
		}
	}

	/**
	 * Verschiebt ein QuellElment an die Position eines ZielElements innerhalb
	 * dessen ElternElements.
	 * 
	 * @param quellElement
	 *          QuellElement
	 * @param zielElement
	 *          ZielElement
	 */
	public void verschiebeXmlElement(Element quellElement, Element zielElement)
	{
		int position = bestimmeElementPositionInElternElement(zielElement);
		verschiebeXmlElement(quellElement, zielElement.getParentElement(), position);
	}

	/**
	 * Verschiebt eine XML-Element in ein anderes XML-Element an eine bestimmte
	 * Position.
	 * 
	 * @param quellElement
	 *          QuellElement, dass in ein Zielelement verschoben werden soll.
	 * @param zielElement
	 *          ZielElement, in dass das QuellElement verschoben wird.
	 * @param position
	 *          Position im Zielelement, in die das QuellElement eingefgt wird.
	 */
	private void verschiebeXmlElement(Element quellElement, Element zielElement,
			int position)
	{
		_debug.fine(getClass().getName()
				+ "::verschiebeXmlElement => Anzahl Kinder Nachher: "
				+ zielElement.getChildren().size());

		/*
		 * Prfen ob ZielElement Kindelement vom Quellelement ist.
		 */
		if (!istXmlElementKindVon(zielElement, quellElement))
		{
			sichereAktuellenZustandAufUndoStack();

			quellElement.detach();

			// Listen
			@SuppressWarnings("unchecked")
			List<Element> childsEmZiel = zielElement.getChildren();
			List<Element> childsNeu = new LinkedList<Element>();

			int i = -1;
			Iterator<Element> it = childsEmZiel.iterator();
			while (it.hasNext())
			{
				i++;

				if (i == position)
				{
					childsNeu.add(quellElement);
					_debug.fine(getClass().getName() + " => EINFGEN in Position " + i);

				}

				Element tempElementZiel = it.next();
				Element tempElement = (Element) tempElementZiel.clone();

				_debug.fine(getClass().getName() + "::TEMPELEMENT: => TEMPELEMENT: "
						+ tempElement.getName());

				childsNeu.add(tempElement);
			}

			// Korrektur fr das Einfgen am Ende
			if (position >= childsEmZiel.size())
			{
				childsNeu.add(quellElement);
			}

			zielElement.setContent(childsNeu);

			_debug.fine(getClass().getName()
					+ "::TEMPELEMENT: => Anzahl Kinder Nachher: "
					+ zielElement.getChildren().size());

			// System.out.println("\nNachher: ");
			// ausgabeParser(0, zielElement);

		}
		else
		{

		}
	}

	/**
	 * Prft rekursiv, ob ein Element Kindelment von einem Elternelement ist.
	 * 
	 * @param KindElement
	 *          Element von dem geprft werden soll, ob es Kindelement ist.
	 * @param potElternElement
	 *          Element gegen das geprft werden soll, ob es das Kindelement
	 *          enthlt.
	 * @return
	 */
	public boolean istXmlElementKindVon(Element KindElement,
			Element potElternElement)
	{
		boolean istKind = false;
		if (KindElement == potElternElement)
		{
			istKind = true;
		}
		else if (potElternElement.getChildren().size() > 0)
		{
			@SuppressWarnings("unchecked")
			Iterator it = potElternElement.getChildren().iterator();
			while (it.hasNext())
			{
				Element neuesPotElternElement = (Element) it.next();
				istKind = istXmlElementKindVon(KindElement, neuesPotElternElement);
				if (istKind)
				{
					break;
				}
			}
		}

		return istKind;
	}

	/**
	 * Lscht das angegebene Element.
	 * 
	 * @param loeschElement
	 *          Zu lschendes Element
	 */
	public void loescheXmlElement(Element loeschElement)
	{
		if (loeschElement != null)
		{
			sichereAktuellenZustandAufUndoStack();
			loeschElement.detach();
		}
	}

	/**
	 * Bestimmt die Position eines {@link Element} in der Liste der Kind-Elemente
	 * seines Eltern-Elements.
	 * 
	 * @param element
	 *          Element, dessen Position bestimmt werden soll.
	 * @return Position des Elements i.d. Liste der Kind-Elemente des
	 *         Eltern-Elements.
	 */
	private int bestimmeElementPositionInElternElement(Element element)
	{
		int returnWert = -1;
		Element elternElement = element.getParentElement();
		if (element != null)
		{
			@SuppressWarnings("unchecked")
			Iterator it = elternElement.getChildren().iterator();
			while (it.hasNext())
			{
				returnWert++;
				Element item = (Element) it.next();
				if (item.equals(element))
				{
					break;
				}
			}
		}
		return returnWert;
	}

	/**
	 * Liefert die {@link TreeMap} der verfgbaren {@link StatusLeisteElement}e.
	 * 
	 * @return TreeMap der verfgbaren {@link StatusLeisteElement}e.
	 */
	public TreeMap<String, StatusLeisteElement> getVerfuegbareStatusLeistenElemente()
	{
		return this._verfuegbareElemente;
	}

	/**
	 * Liefert die Liste der Elemente der StatusleistenElemente aus der XML-Datei.
	 * Diese gibt auch die Sortierung der Elemente innerhalb der Statusleiste vor.
	 * Hiernach wird vom {@link StatusLeisteGenerator} die Statusleiste aufgebaut.
	 * 
	 * @return
	 */
	public List<Element> getListeXmlStatusleisteElemente()
	{
		List<Element> returnListe = new LinkedList<Element>();

		synchronized (_elementWurzel)
		{
			if (_elementWurzel.getChildren().size() > 0)
			{
				@SuppressWarnings("unchecked")
				Iterator it = _elementWurzel.getChildren().iterator();
				while (it.hasNext())
				{
					Element hinzufuegenElement = (Element) it.next();
					_debug.fine(getClass().getName()
							+ "::getListeXmlStatusleisteElemente => Element: "
							+ hinzufuegenElement.getName() + " ID: "
							+ hinzufuegenElement.getAttributeValue("id"));
					returnListe.add(hinzufuegenElement);
				}
				return returnListe;
			}
			else
			{
				return null;
			}
		}
	}

	// /**
	// * Prft ob ein Element mit einer bestimmten ID im Editor vorhanden ist.
	// * @return
	// */
	// public boolean pruefeObStatusLeistenElementInXmlStruktur()
	// {
	// return false;
	// }

	/**
	 * Liefert das Wurzelelement zurck. ManuelleSynchronisation nicht vergessen!
	 */
	public Element getWurzel()
	{
		return this._elementWurzel;
	}

	/**
	 * Liefert den UNDO-Stack zurck.
	 * 
	 * @return {@link UndoStackStatusLeiste}
	 */
	public UndoStackStatusLeiste getUndoStack()
	{
		return _undoStack;
	}

	/**
	 * Fhrt die UNDO-Operation auf der Datenstruktur aus. Das {@link Document}
	 * wird mit der aktuellsten Version vom UNDO-Stack ersetzt.
	 * 
	 * @return Liefert <code>true</code>, wenn die UNDO-Operation erfolgreich
	 *         ausgefhrt werden konnte, sonst <code>false</code>.
	 */
	public boolean aktionUndo()
	{
		boolean erfolgreich = false;

		if (_undoStack != null)
		{
			Object object = _doc.clone();
			if (object instanceof Document)
			{
				Document clone = (Document) object;

				Document pop = _undoStack.pop(clone);
				if (pop != null)
				{
					_doc = pop;
					_elementWurzel = _doc.getRootElement();
					erfolgreich = true;
				}
			}
		}

		return erfolgreich;
	}

	/**
	 * Fhrt die REDO-Operation auf der Datenstruktur aus. Das {@link Document}
	 * wird auf dem STACK gesichert und mit der REDO-Variante ersetzt.
	 * 
	 * @return Liefert <code>true</code>, wenn die Aktion erfolgreich
	 *         durchgefhrt wurde und die Struktur mit der REDO-Variante vom Stack
	 *         ersetzt wurde, sonst <code>false</code>.
	 */
	public boolean aktionRedo()
	{
		boolean erfolgreich = false;

		Object object = _doc.clone();
		if (object instanceof Document)
		{
			Document clone = (Document) object;

			if (_undoStack != null)
			{
				Document redo = _undoStack.getRedo();
				if (redo != null)
				{
					_undoStack.push(clone);
					_doc = redo;
					_elementWurzel = _doc.getRootElement();
					erfolgreich = true;
				}
			}
		}

		return erfolgreich;
	}

	/**
	 * Sichert den aktuellen Zustand auf den UNDO-Stack
	 */
	public void sichereAktuellenZustandAufUndoStack()
	{
		Object object = _doc.clone();
		if (object instanceof Document)
		{
			Document clone = (Document) object;
			_undoStack.push(clone);
		}
	}

	// /**
	// * Liefert eien Liste der IDs der Statusleistenelemente entsprechend ihrer
	// * Sortierung in der XML-Struktur.
	// *
	// * @return IDs der Statusleistenelemente als Liste.
	// */
	// public List<String> getListeStatusLeistenElementeIDs()
	// {
	// List<String> listeElementIds = new LinkedList<String>();
	//
	// if (_elementWurzel != null)
	// {
	// @SuppressWarnings("unchecked")
	// Iterator<Element> it = _elementWurzel.getChildren().iterator();
	// while (it.hasNext())
	// {
	// Element element = it.next();
	// String id = element.getAttributeValue(XmlBezeichnerStatus.attributId);
	// listeElementIds.add(id);
	// }
	// return listeElementIds;
	// } else
	// {
	// return null;
	// }
	// }

}
