/*
 * This file is part of Rahmenwerk.
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * 
 * Copyright (c) 2012 Swarco Traffic Systems GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * Swarco Traffic Systems GmbH - initial API and implementation
 * 
 * 
 * 
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau 
 * T. +49 (0)7225 / 64-03
 * F. +49 (0)7225 / 64-300 
 * E. office.sts@swarco.de
 */
package de.sts.esptool.dlgs;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;

import de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen.EinstellungenBenutzerSpezifischNetzweit;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class DialogBenutzerWaehler extends Dialog
{

	private Debug _debug = Debug.getLogger();

	private Shell _shell = null; // @jve:decl-index=0:visual-constraint="10,10"

	private Composite _bereichOben = null;

	private Composite _bereichUnten = null;

	private Group _gruppeOben = null;

	private Composite _bereichUntenRechts = null;

	private Button _schaltflaecheOk = null;

	private Button _schaltflaecheAbbrechen = null;

	/**
	 * {@link TableViewer} zur Darstellung der Benutzer
	 */
	private TableViewer _tvBenutzer = null;

	/**
	 * Benutzerspezifische, netzweite Einstellungen. Initialisierung erfolgt i.d.
	 * Methode initialisiere.
	 */
	private EinstellungenBenutzerSpezifischNetzweit _ebsn = null;

	/**
	 * Liefert die Verbindung zum DaV
	 */
	private DaVVerbindung _dav = DaVVerbindung.getInstanz();

	/**
	 * Selektierter Benutzer
	 */
	private String _selektierterBenutzer = null;

	public DialogBenutzerWaehler(Shell parent)
	{
		super(parent, 0);
	}

	/**
	 * ffnet den Dialog und liefert den selektierten Parameter zurck.
	 * 
	 * @return Liefert den selektierten Benutzer als String zurck.
	 */
	public String oeffne()
	{
		erzeugeShell();
		initialisiereBenutzerliste();

		_shell.open();
		_shell.setSize(_shell.getBounds().width + 1, _shell.getBounds().height);

		Display display = getParent().getDisplay();

		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _selektierterBenutzer;
	}

	/**
	 * This method initializes sShell
	 */
	private void erzeugeShell()
	{
		_shell = new Shell();
		_shell.setText("Benutzerwhler");
		erzeugeCompositeOben();
		erzeugeCompositeUnten();
		_shell.setSize(new Point(400, 280));
		_shell.setLayout(new GridLayout());
	}

	/**
	 * Initialisiert die Benutzerliste.
	 */
	private void initialisiereBenutzerliste()
	{
		if (_dav != null)
		{
			ClientDavInterface daVVerbindung = _dav.getDaVVerbindung();
			if (daVVerbindung != null)
			{
				if (_ebsn == null)
				{
					_ebsn = new EinstellungenBenutzerSpezifischNetzweit(
							(ClientDavConnection) daVVerbindung);
				} else
				{
					_ebsn.setDaVVerbindung((ClientDavConnection) daVVerbindung);
				}

				List<SystemObject> bestimmeAlleBenutzerSO = _ebsn
						.bestimmeAlleBenutzerSO();

				if ((bestimmeAlleBenutzerSO != null)
						&& (bestimmeAlleBenutzerSO.size() > 0))
				{
					LinkedList<String> benutzer = new LinkedList<String>();
					Iterator<SystemObject> it = bestimmeAlleBenutzerSO.iterator();
					while (it.hasNext())
					{
						SystemObject systemObject = it.next();
						benutzer.add(systemObject.getName());
					}

					_tvBenutzer.setInput(benutzer);
				}
			}
		}
	}

	/**
	 * Erzeugt den Bereich Oben.
	 * 
	 */
	private void erzeugeCompositeOben()
	{
		GridData gridDataOben = new GridData();
		gridDataOben.horizontalAlignment = GridData.FILL;
		gridDataOben.grabExcessHorizontalSpace = true;
		gridDataOben.grabExcessVerticalSpace = true;
		gridDataOben.verticalAlignment = GridData.FILL;
		_bereichOben = new Composite(_shell, SWT.NONE);
		_bereichOben.setLayout(new GridLayout());
		erzeugeGruppeOben();
		_bereichOben.setLayoutData(gridDataOben);
	}

	/**
	 * Erzeugt den Bereich Unten.
	 * 
	 */
	private void erzeugeCompositeUnten()
	{
		GridData gridDataUntenRechts = new GridData();
		gridDataUntenRechts.horizontalAlignment = GridData.FILL;
		gridDataUntenRechts.grabExcessHorizontalSpace = true;
		gridDataUntenRechts.verticalAlignment = GridData.CENTER;
		_bereichUnten = new Composite(_shell, SWT.NONE);
		_bereichUnten.setLayout(new GridLayout());
		erzeugeCompositeUntenRechts();
		_bereichUnten.setLayoutData(gridDataUntenRechts);
	}

	/**
	 * Setzt den selektierten Benutzer.
	 * 
	 * @param benutzer
	 *          Benutzer
	 */
	private void aktionBenutzerSelektiert(String benutzer)
	{
		this._selektierterBenutzer = benutzer;
	}

	/**
	 * Erzeugt die Gruppe Oben.
	 * 
	 */
	private void erzeugeGruppeOben()
	{
		GridData gridDataTabelleBenutzer = new GridData();
		gridDataTabelleBenutzer.horizontalAlignment = GridData.FILL;
		gridDataTabelleBenutzer.grabExcessHorizontalSpace = true;
		gridDataTabelleBenutzer.grabExcessVerticalSpace = true;
		gridDataTabelleBenutzer.verticalAlignment = GridData.FILL;
		GridData gridDataGruppeOben = new GridData();
		gridDataGruppeOben.horizontalAlignment = GridData.FILL;
		gridDataGruppeOben.grabExcessHorizontalSpace = true;
		gridDataGruppeOben.grabExcessVerticalSpace = true;
		gridDataGruppeOben.verticalAlignment = GridData.FILL;
		_gruppeOben = new Group(_bereichOben, SWT.NONE);
		_gruppeOben.setLayout(new GridLayout());
		_gruppeOben.setLayoutData(gridDataGruppeOben);
		_gruppeOben.setText("Benutzer:");

		_tvBenutzer = new TableViewer(_gruppeOben, SWT.FULL_SELECTION | SWT.MULTI);

		TableColumn tcBenutzer = new TableColumn(_tvBenutzer.getTable(), SWT.LEFT);
		tcBenutzer.setText("Benutzer:");
		tcBenutzer.setWidth(355);

		_tvBenutzer.getTable().setHeaderVisible(true);
		_tvBenutzer.getTable().setLayoutData(gridDataTabelleBenutzer);
		_tvBenutzer.getTable().setLinesVisible(true);

		_tvBenutzer.setContentProvider(new BenutzerInhaltsVersorger());
		_tvBenutzer.setLabelProvider(new BenutzerDarstellungsVersorger());

		_tvBenutzer.addSelectionChangedListener(new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				Object firstElement = selection.getFirstElement();

				if (firstElement instanceof String)
				{
					aktionBenutzerSelektiert((String) firstElement);
				}
			}
		});
	}

	/**
	 * Schliet den Dialog, es erfolgt die Rckgabe des zuvor selektierten
	 * Benutzers durch die Methode {@link DialogBenutzerWaehler#oeffne()}.
	 */
	private void aktionOk()
	{
		_shell.dispose();
	}

	/**
	 * Schliet den Dialog, es wird <code>null</code> von der Methode
	 * {@link DialogBenutzerWaehler#oeffne()} zurckgeliefert.
	 */
	private void aktionAbbrechen()
	{
		_selektierterBenutzer = null;
		_shell.dispose();
	}

	/**
	 * Erzeugt den Bereich Unten Rechts.
	 * 
	 */
	private void erzeugeCompositeUntenRechts()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		GridData gridDataUntenRechts = new GridData();
		gridDataUntenRechts.horizontalAlignment = GridData.END;
		gridDataUntenRechts.grabExcessHorizontalSpace = true;
		gridDataUntenRechts.verticalAlignment = GridData.CENTER;
		_bereichUntenRechts = new Composite(_bereichUnten, SWT.NONE);
		_bereichUntenRechts.setLayoutData(gridDataUntenRechts);
		_bereichUntenRechts.setLayout(rowLayout);
		_schaltflaecheOk = new Button(_bereichUntenRechts, SWT.NONE);
		_schaltflaecheOk.setText("OK");
		_schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}
		});

		_schaltflaecheAbbrechen = new Button(_bereichUntenRechts, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}
		});
	}

	/**
	 * ContentProvider zur Darstellung der Benutzer, bernimmt als Parameter eine
	 * Liste von Strings.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class BenutzerInhaltsVersorger implements IStructuredContentProvider
	{

		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{

			Object[] re = null;
			if (inputElement instanceof LinkedList)
			{
				LinkedList liste = (LinkedList) inputElement;
				if (liste.size() > 0)
				{

					re = new Object[liste.size()];

					int i = 0;
					Iterator it = liste.iterator();
					while (it.hasNext())
					{
						Object object = it.next();
						String einfuegen = "leer";

						if (object instanceof String)
						{
							einfuegen = (String) object;
						}

						re[i] = einfuegen;

						i++;
					}
				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}
	}

	/**
	 * {@link LabelProvider} fr die Tabelle.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class BenutzerDarstellungsVersorger implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "Nicht bekannter Datentyp";

			if (element instanceof String)
			{
				re = (String) element;
				_debug.fine("HIER");
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

}
